(* ::Package:: *)

(************************************************************************)
(* This file was generated automatically by the Mathematica front end.  *)
(* It contains Initialization cells from a Notebook file, which         *)
(* typically will have the same name as this file except ending in      *)
(* ".nb" instead of ".m".                                               *)
(*                                                                      *)
(* This file is intended to be loaded into the Mathematica kernel using *)
(* the package loading commands Get or Needs.  Doing so is equivalent   *)
(* to using the Evaluate Initialization Cells menu command in the front *)
(* end.                                                                 *)
(*                                                                      *)
(* DO NOT EDIT THIS FILE.  This entire file is regenerated              *)
(* automatically each time the parent Notebook file is saved in the     *)
(* Mathematica front end.  Any changes you make to this file will be    *)
(* overwritten.                                                         *)
(************************************************************************)



BeginPackage["Lautrup`NDSolveBV`"];
Unprotect[NDSolveBV];

(* messages *)
NDSolveBV::"usage"="Numerical differential equation boundary value solver.\n\nNDSolveBV[eqs,vars,intv,init,opts] where\n\n\teqs\tdifferential equations and boundary values: eqs=(odes,bva,bvb)\n\tvars\tthe dependent variables \n\tintv\tthe integration variable and its solution interval with an intermediary joining point\n\tinit\tthe initial values for unknown parameters\n\topts\tvarious options\n\t";
NDSolveBV::"lengtherror"="The number of equations, boundary conditions left and rigth, variables, and initial conditions must be the same (`1`)";
NDSolveBV::"structureerror"="The boundary values bva and bvb must refer to a and b, respectively";
NDSolveBV::"intervalerror"="Interval must be of the form {x,a,c,b} with a < c < b";
NDSolveBV::"equationerror" ="Equations must be of the form eqs={odes,bva,bvb}";

Begin["`Private`"];
(* query for list of numeric values *)
NumericListQ[x_]:=And@@NumericQ/@x;

(* the function itself *)
NDSolveBV[eqs_,vars_,intv_,init_,opts___]:=Module[{x,a,c,b,n,ivars,aBV,bBV,odes,cvals,solac,solbc,sol,roots,args,varsac,varsbc,fun,sub,int,optsNDSolveBV,optsNDSolve,optsFindRoot,optsFunctionInterpolation},

(* check interval structure *)
If[Length[intv]!=4,Message[NDSolveBV::"intervalerror"];Return[]];
x=intv[[1]];a=intv[[2]];c=intv[[3]];b=intv[[4]];If[!(a<=c<=b),Message[NDSolveBV::"intervalerror"];Return[]];

(* check the number of equations and their structure *)
If[Length[eqs]!=3,Message[NDSolveBV::"equationerror",n];Return[]];
odes=eqs[[1]];
bva=eqs[[2]];
bvb=eqs[[3]];

(* check the numbers of equations, boundary values left and right, and initial values *)
n=Length[vars];
If[!(Length[odes]==Length[bva]==Length[bvb]==Length[init]==n),Message[NDSolveBV::"lengtherror",n];Return[]];
If[! (Length[Cases[bva,_[a]==_]]==Length[Cases[bvb,_[b]==_]]==n),Message[NDSolveBV::"structureerror"];Return[]];

(* get options *)
optsNDSolve=Sequence@@FilterRules[Flatten[{opts}],Options[NDSolve]];optsFindRoot=Sequence@@FilterRules[Flatten[{opts}],Options[FindRoot]];optsNDSolveBV=Sequence@@FilterRules[Flatten[{opts,Options[NDSolveBV]}],Options[NDSolveBV]];optsFunctionInterpolation=Sequence@@FilterRules[Flatten[{opts}],Options[FunctionInterpolation]];

(* the unknown boundary values *)
ivars=(First[#1]&)/@init;

(* integrate from a to c and from c to b*)
solac[args_?NumericListQ]:=First[NDSolve@@({odes\[Union]bva,vars,{x,a,c},optsNDSolve}/. MapThread[#1->#2&,{ivars,args}])];solbc[args_?NumericListQ]:=First[NDSolve@@({odes\[Union]bvb,vars,{x,c,b},optsNDSolve}/. MapThread[#1->#2&,{ivars,args}])];

(* construct solution *)
cvals=(#1[c]&)/@vars;
sol[args_?NumericListQ]:=(cvals/. solac[args])-(cvals/. solbc[args]);

(* determine the roots for the unknown boundary values *)
roots=FindRoot@@{sol[ivars],init,optsFindRoot};
args=ivars/. roots;
varsac=solac[args];
varsbc=solbc[args];

(* output the two-part solution *)
fun[f_]:=Evaluate[If[#1<c,Evaluate[f[#1]/. varsac],Evaluate[f[#1]/. varsbc]]]&;
sub=(#1->fun[#1]&)/@vars;
{sub,roots}];

End[];

SetAttributes[NDSolveBV,{Protected,ReadProtected}];
EndPackage[];



