(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    544625,      10408]
NotebookOptionsPosition[    531110,       9949]
NotebookOutlinePosition[    531604,       9968]
CellTagsIndexPosition[    531561,       9965]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Hydrostatic shapes", "Title",
 InitializationCell->True,
 CellChangeTimes->{3.4309104887455997`*^9}],

Cell["\<\
SetDirectory[NotebookDirectory[]]; 
<<Lautrup`\
\>", "Input",
 PageWidth->Infinity,
 InitializationCell->True,
 CellChangeTimes->{{3.3961770879009137`*^9, 3.3961770884489136`*^9}, 
   3.3961786858489137`*^9, 3.479209087289*^9, 3.588569539871722*^9},
 ShowSpecialCharacters->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Section 4.2 The centrifugal force", "Title",
 CellChangeTimes->{{3.501738196579872*^9, 3.5017382121697645`*^9}, 
   3.575537837813204*^9}],

Cell[CellGroupData[{

Cell["Case: Newton's bucket", "Section",
 CellChangeTimes->{{3.5017382894891863`*^9, 3.5017382909372697`*^9}, 
   3.575537837813204*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{3.575537837814204*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(9\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755382106819444`*^9, 3.586581208313941*^9, 
  3.5869672123416877`*^9, 3.5869692199035683`*^9, 3.5885695500880513`*^9, 
  3.588579709217583*^9, 3.5890766224874973`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{3.575537837825209*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[CapitalOmega]", "=", 
   RowBox[{"2", 
    RowBox[{"\[Pi]", "/", "Second"}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"angular", " ", "frequency"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"a", "=", 
    RowBox[{"10.", " ", "Centimeter"}]}], " ", 
   RowBox[{"(*", " ", "radius", " ", "*)"}], ";"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}]}], "Input",
 CellChangeTimes->{
  3.4032385264445767`*^9, 3.4664841508755655`*^9, 3.575537837826208*^9, 
   3.5869668442525215`*^9, {3.586966906716017*^9, 3.586966954907799*^9}, {
   3.5869670085782385`*^9, 3.5869670230999813`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Height at rim", "Subsection",
 CellChangeTimes->{3.5755378378302083`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"h", "=", 
  RowBox[{
   RowBox[{
    FractionBox[
     SuperscriptBox["\[CapitalOmega]", "2"], 
     RowBox[{"2", 
      SubscriptBox["g", "0"]}]], 
    SuperscriptBox["a", "2"]}], "//", 
   RowBox[{"ToUnit", "[", "Centimeter", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5755361761038227`*^9, 3.575536176795905*^9}, {
   3.575536637340165*^9, 3.5755366430208454`*^9}, 3.5755378378312063`*^9}],

Cell[BoxData[
 TemplateBox[{"2.012839124693827`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"cm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: centimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Centimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575538210963978*^9, 3.5865812084509363`*^9, 
  3.5869672126177206`*^9, 3.586969220156596*^9, 3.588569550223091*^9, 
  3.588579709347578*^9, 3.5890766226225405`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4.1 Ultracentrifuge", "Section",
 CellChangeTimes->{{3.501738330192515*^9, 3.5017383389770174`*^9}, 
   3.5755378378412066`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{3.5755378378412066`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(4\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{{3.575538240268492*^9, 3.575538257656579*^9}, 
   3.5865812086089144`*^9, 3.5869672128737507`*^9, 3.586969220411625*^9, 
   3.5885695503770676`*^9, 3.5885797095016384`*^9, 3.5890766227735376`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{{3.5869681331461706`*^9, 3.5869681343443146`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"10.", " ", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"radius", " ", "of", " ", "centrifuge"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{"1000.", " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"water", " ", "density"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[CapitalOmega]", "=", 
  RowBox[{
   RowBox[{"60000.", " ", "RPM"}], " ", "//", "ToSI", 
   RowBox[{"(*", " ", 
    RowBox[{"rotations", " ", "per", " ", "minute"}], " ", 
    "*)"}]}]}]}], "Input",
 CellChangeTimes->{{3.4664841795792074`*^9, 3.466484180316249*^9}, {
   3.466484238285565*^9, 3.4664842775648117`*^9}, {3.466484314761939*^9, 
   3.4664843219263487`*^9}, 3.5755378378462105`*^9, {3.586966963147787*^9, 
   3.586966996158747*^9}, {3.586967034111302*^9, 3.586967067534313*^9}, {
   3.5869676745911493`*^9, 3.5869676768134174`*^9}, 3.58696771399988*^9, {
   3.586968147171853*^9, 3.5869681737740445`*^9}, {3.587091414891181*^9, 
   3.5870914171114473`*^9}}],

Cell[BoxData[
 TemplateBox[{"6283.185307179587`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"per \"", "\[InvisibleSpace]", "\"second\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: reciprocal seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["1", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5869681741380873`*^9, 3.586969220524638*^9, 
  3.58856955045508*^9, 3.58857970957265*^9, 3.5890766228455057`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.5869681860735235`*^9, 3.586968192727319*^9}}],

Cell["Acceleration at the rim", "Text",
 CellChangeTimes->{{3.586967133959282*^9, 3.5869671415761957`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g", "=", 
  RowBox[{
   RowBox[{"a", " ", 
    SuperscriptBox["\[CapitalOmega]", "2"]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{3.575537837852209*^9}],

Cell[BoxData[
 TemplateBox[{"3.9478417604357433`*^6"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"s\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters per second squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", 
        SuperscriptBox["\"Seconds\"", "2"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.575538211524044*^9, 3.5755382579876204`*^9}, 
   3.5865812087449484`*^9, 3.58696721320479*^9, 3.586969220625651*^9, 
   3.588569550507175*^9, 3.588579709627614*^9, 3.589076622904558*^9}]
}, Open  ]],

Cell["g-factor", "Text",
 CellChangeTimes->{{3.586967158072176*^9, 3.586967160023408*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox["g", 
  SubscriptBox["g", "0"]]], "Input",
 CellChangeTimes->{3.5755378378572106`*^9}],

Cell[BoxData["402567.8249387654`"], "Output",
 CellChangeTimes->{{3.575538211636057*^9, 3.5755382581006336`*^9}, 
   3.586581208801979*^9, 3.5869672133158035`*^9, 3.5869692207926702`*^9, 
   3.588569550568119*^9, 3.5885797096866674`*^9, 3.5890766229665165`*^9}]
}, Open  ]],

Cell["Pressure at the rim", "Text",
 CellChangeTimes->{{3.5869671812239532`*^9, 3.586967184055292*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"p", "=", 
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], 
    SubscriptBox["\[Rho]", "0"], 
    SuperscriptBox["\[CapitalOmega]", "2"], 
    SuperscriptBox["a", "2"]}], "//", 
   RowBox[{"ToUnit", "[", "Atmosphere", "]"}]}]}]], "Input",
 CellChangeTimes->{3.5755378378612103`*^9}],

Cell[BoxData[
 TemplateBox[{"1948.1084433435692`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"atm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: atmospheres"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Atmospheres\""}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.575538211909093*^9, 3.5755382583556643`*^9}, 
   3.5865812089409795`*^9, 3.586967213577834*^9, 3.5869692210507016`*^9, 
   3.588569550702097*^9, 3.5885797098156776`*^9, 3.589076623096573*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Section 4.3 The figure of Earth", "Title",
 CellChangeTimes->{{3.501738767266514*^9, 3.5017387732648573`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CleanGlobal", "[", "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(5\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.5755383498826456`*^9, 3.5755388908595543`*^9, 3.575538961405019*^9, {
   3.575538997046294*^9, 3.575539021731258*^9}, 3.5865812091029997`*^9, 
   3.586967213841866*^9, {3.586969209844359*^9, 3.5869692213187337`*^9}, 
   3.588569550872118*^9, 3.5885797099706573`*^9, 3.589076623252594*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", 
   RowBox[{"AccelerationDueToGravity", "//", "N"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"EarthRadius", "//", "N"}]}], ";"}], "\n", 
 RowBox[{"\[CapitalOmega]", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"2", "\[Pi]"}], "SiderealDay"], "//", "ToSI", " ", 
   RowBox[{"(*", " ", 
    RowBox[{"absolute", " ", "angular", " ", "rotation", " ", "rate"}], " ", 
    "*)"}]}]}]}], "Input",
 CellChangeTimes->{{3.394535773620536*^9, 3.394535845186536*^9}, 
   3.3946033082629385`*^9, {3.430281032777295*^9, 3.4302810398802958`*^9}, {
   3.5869674393909297`*^9, 3.586967472170863*^9}, {3.5869680848953824`*^9, 
   3.5869680866715937`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.00007292115855306578`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"per \"", "\[InvisibleSpace]", "\"second\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: reciprocal seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["1", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755383500336623`*^9, 3.5755388909465656`*^9, 3.575538961592041*^9, {
   3.575538997231316*^9, 3.5755390219112797`*^9}, 3.586581209127983*^9, 
   3.5869672138878727`*^9, {3.586969209979375*^9, 3.586969221364739*^9}, 
   3.58856955089913*^9, 3.588579709995672*^9, 3.589076623279577*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.586968276536376*^9, 3.5869683011593285`*^9}}],

Cell["Ratio of centrifugal acceleration to standard gravity", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"q", "=", 
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["\[CapitalOmega]", "2"], "a"}], 
    SubscriptBox["g", "0"]], "100", "Percent"}]}]], "Input",
 CellChangeTimes->{{3.57553894981863*^9, 3.5755390167046537`*^9}}],

Cell[BoxData[
 TemplateBox[{"0.34584403372260447`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"%\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: percent"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Percent\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755383501986837`*^9, 3.5755388911275854`*^9, {3.5755389546282053`*^9, 
   3.575539022105302*^9}, 3.5865812092630396`*^9, 3.586967214160904*^9, {
   3.5869692101743975`*^9, 3.586969221552762*^9}, 3.5885695510031595`*^9, 
   3.588579710091713*^9, 3.589076623378595*^9}]
}, Open  ]],

Cell[TextData[{
 "Earth\[CloseCurlyQuote]s levitation angular velocity (",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"q", "=", "1"}], ")"}], TraditionalForm]]]
}], "Text",
 CellChangeTimes->{{3.501739110137125*^9, 3.501739138920771*^9}, {
  3.5017392342412233`*^9, 3.5017392410336123`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[CapitalOmega]", "0"], "=", 
  RowBox[{
   SqrtBox[
    FractionBox[
     SubscriptBox["g", "0"], "a"]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.430105373865345*^9, 3.430105403235345*^9}}],

Cell[BoxData[
 TemplateBox[{"0.0012399763901912288`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"per \"", "\[InvisibleSpace]", "\"second\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: reciprocal seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["1", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575538350352702*^9, 3.575538891279606*^9, 3.5755389619210815`*^9, {
   3.575538997557358*^9, 3.575539022251321*^9}, 3.5865812093470483`*^9, 
   3.586967214321925*^9, {3.586969210339418*^9, 3.5869692216977797`*^9}, 
   3.588569551121155*^9, 3.5885797101996927`*^9, 3.5890766234896317`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"2", "\[Pi]"}], 
   SubscriptBox["\[CapitalOmega]", "0"]], "//", 
  RowBox[{"ToUnit", "[", "Minute", "]"}]}]], "Input",
 CellChangeTimes->{{3.430105407158345*^9, 3.4301054410973454`*^9}}],

Cell[BoxData[
 TemplateBox[{"84.45302341886519`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"min\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: minutes"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Minutes\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575538350408709*^9, 3.575538891337613*^9, 3.575538961967087*^9, {
   3.575538997604362*^9, 3.5755390222983265`*^9}, 3.586581209376051*^9, 
   3.5869672144109335`*^9, {3.586969210397425*^9, 3.5869692217977934`*^9}, 
   3.588569551149212*^9, 3.58857971022873*^9, 3.589076623516626*^9}]
}, Open  ]],

Cell["Intuitive depth of polar \"valley\"", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"q", " ", "a"}], "//", 
  RowBox[{"ToUnit", "[", "Kilometer", "]"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"22.058406277153914`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755383504747167`*^9, 3.5755388913796167`*^9, 3.575538962009091*^9, {
   3.575538997641368*^9, 3.5755390223913364`*^9}, 3.586581209422058*^9, 
   3.5869672145229473`*^9, {3.5869692104974365`*^9, 3.5869692220498233`*^9}, 
   3.5885695512001925`*^9, 3.5885797102787323`*^9, 3.5890766235686336`*^9}]
}, Open  ]],

Cell["Actual centrifugal deformation", "Text",
 CellChangeTimes->{{3.5017401383049326`*^9, 3.501740155968943*^9}, 
   3.5017402686173863`*^9, 3.50174042778449*^9, {3.5017404629845037`*^9, 
   3.5017404657286606`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["a", "equator"], "=", 
   RowBox[{"6378.137", "Kilometer"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["a", "pole"], "=", 
   RowBox[{"6356.752", "Kilometer"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.430114312016*^9, 3.430114354083*^9}, {3.430114567535*^9,
    3.4301145861099997`*^9}, 3.430115192308*^9, 3.5017399832190623`*^9, {
   3.501740223066781*^9, 3.501740235076468*^9}, 3.5017402686183863`*^9, 
   3.50174042778549*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["a", "equator"], "-", 
  SubscriptBox["a", "pole"]}]], "Input",
 CellChangeTimes->{{3.43011458992*^9, 3.430114591909*^9}, 
   3.5017399832210627`*^9, {3.5017402382836514`*^9, 3.501740268620386*^9}, 
   3.5017404277874904`*^9}],

Cell[BoxData[
 TemplateBox[{"21.38499999999931`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755383505207205`*^9, 3.5755388914246225`*^9, 3.575538962058096*^9, {
   3.575538997691375*^9, 3.5755390224403434`*^9}, 3.586581209446062*^9, 
   3.586967214650963*^9, {3.586969210553442*^9, 3.586969222113829*^9}, 
   3.5885695512282066`*^9, 3.5885797103057384`*^9, 3.5890766235965953`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4.2 Rotating neutron star (pulsar)", "Section",
 CellChangeTimes->{{3.4031691133729696`*^9, 3.40316911620597*^9}, {
  3.5017393526489964`*^9, 3.501739369720972*^9}, {3.501739793153191*^9, 
  3.5017397932741985`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.40316913198197*^9, 3.4031691348789697`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(6\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.5755383507737527`*^9, 3.5755388916856537`*^9, 3.57553896232213*^9, {
   3.5755389979434032`*^9, 3.575539022691373*^9}, 3.5755392210951786`*^9, 
   3.5865812096020813`*^9, 3.586967214970001*^9, 3.586968499649146*^9, {
   3.586969210826476*^9, 3.5869692224808736`*^9}, 3.5885695513842263`*^9, 
   3.5885797104587193`*^9, 3.5890766237586155`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{{3.40316914090897*^9, 3.4031691425099697`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"G", "=", "GravitationalConstant"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[CapitalOmega]", "=", 
   FractionBox[
    RowBox[{"2", "\[Pi]"}], 
    RowBox[{"1.4", " ", "Milli", " ", "Second"}]]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"angular", " ", "frequency"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"10", " ", "Kilometer"}]}], " ", 
  RowBox[{"(*", " ", "radius", " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"M", "=", 
  RowBox[{"2", "SunMass", " ", 
   RowBox[{"(*", " ", "mass", " ", "*)"}]}]}]}], "Input",
 CellChangeTimes->{{3.4031691446059694`*^9, 3.4031691924309697`*^9}, {
  3.40316935745597*^9, 3.4031693709429693`*^9}, {3.40316943886397*^9, 
  3.4031694698559694`*^9}, {3.40316988870397*^9, 3.40316989215897*^9}, {
  3.40317002433597*^9, 3.40317002793497*^9}, {3.4031701795509696`*^9, 
  3.40317018430297*^9}, {3.5869676534256105`*^9, 3.5869676574530935`*^9}, {
  3.5869678269424295`*^9, 3.5869678296947594`*^9}, {3.586967884750367*^9, 
  3.586967911883622*^9}, {3.5869679493441167`*^9, 3.586967968111367*^9}}],

Cell[BoxData[
 TemplateBox[{"3.9782`*^30"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"kg\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilograms"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilograms\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5869679178683386`*^9, {3.586967949734163*^9, 3.5869679686164274`*^9}, 
   3.5869684997581577`*^9, {3.586969210881483*^9, 3.58696922253288*^9}, 
   3.588569551409231*^9, 3.5885797104838357`*^9, 3.5890766238056216`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalOmega]", "//", "ToSI"}]], "Input",
 CellChangeTimes->{{3.5017396515900946`*^9, 3.5017396607066154`*^9}}],

Cell[BoxData[
 TemplateBox[{"4487.9895051282765`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"per \"", "\[InvisibleSpace]", "\"second\""}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: reciprocal seconds"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["1", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575538350875763*^9, 3.5755388919146805`*^9, 3.5755389624181395`*^9, {
   3.5755389980344133`*^9, 3.5755390229184003`*^9}, 3.575539221330206*^9, 
   3.586581209652049*^9, 3.5869672160231276`*^9, 3.5869684998121643`*^9, {
   3.5869692109924965`*^9, 3.5869692225968876`*^9}, 3.5885695514262385`*^9, 
   3.5885797105067725`*^9, 3.5890766238476343`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.586968397367872*^9, 3.5869683983749924`*^9}}],

Cell["Surface gravity", "Text",
 CellChangeTimes->{{3.4031694116789694`*^9, 3.4031694141739693`*^9}, {
   3.4031695522869697`*^9, 3.4031695999979696`*^9}, 3.586968411111521*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["g", "0"], "=", 
  RowBox[{
   FractionBox[
    RowBox[{"G", " ", "M"}], 
    SuperscriptBox["a", "2"]], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.40316941865397*^9, 3.4031694734549694`*^9}}],

Cell[BoxData[
 TemplateBox[{"2.6549870288000005`*^12"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"s\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters per second squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", 
        SuperscriptBox["\"Seconds\"", "2"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755383511137934`*^9, 3.5755388921777124`*^9, 3.5755389627781844`*^9, {
   3.5755389982644424`*^9, 3.5755390231864324`*^9}, 3.575539221558234*^9, 
   3.586581209774125*^9, 3.5869672163631687`*^9, {3.586968492995347*^9, 
   3.586968500049192*^9}, {3.586969211230523*^9, 3.58696922284892*^9}, 
   3.5885695515512247`*^9, 3.588579710629754*^9, 3.589076624053653*^9}]
}, Open  ]],

Cell["Centrifugal acceleration", "Text",
 CellChangeTimes->{{3.586968414234897*^9, 3.5869684182643814`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["g", "c"], "=", 
  RowBox[{
   RowBox[{
    SuperscriptBox["\[CapitalOmega]", "2"], "a"}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4031696064319696`*^9, 3.40316961727897*^9}}],

Cell[BoxData[
 TemplateBox[{"2.0142049798141553`*^11"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            SuperscriptBox["\"s\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: meters per second squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", 
        SuperscriptBox["\"Seconds\"", "2"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575538351366824*^9, 3.5755388922857246`*^9, 3.575538962883197*^9, {
   3.575538998531473*^9, 3.5755390232954454`*^9}, 3.5755392216612463`*^9, 
   3.586581209826067*^9, 3.586967216502186*^9, {3.586968493170368*^9, 
   3.5869685001572065`*^9}, {3.5869692113435364`*^9, 3.5869692242670875`*^9}, 
   3.588569551606211*^9, 3.5885797106807475`*^9, 3.5890766241196613`*^9}]
}, Open  ]],

Cell["\<\
Centrifugal acceleration in units of surface gravity\
\>", "Text",
 CellChangeTimes->{{3.5869684588072443`*^9, 3.586968481143924*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"q", "=", 
  RowBox[{
   FractionBox[
    SubscriptBox["g", "c"], 
    SubscriptBox["g", "0"]], "100", "Percent"}]}]], "Input",
 CellChangeTimes->{{3.40316967220597*^9, 3.40316968164697*^9}, {
  3.586968485199413*^9, 3.5869684882067723`*^9}}],

Cell[BoxData[
 TemplateBox[{"7.586496498721256`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          RowBox[{"\"%\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: percent"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Percent\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.5755383515038404`*^9, 3.5755388924127407`*^9, 3.575538963002211*^9, {
   3.5755389986644897`*^9, 3.5755390234364624`*^9}, 3.5755392217792606`*^9, 
   3.586581209896077*^9, 3.5869672166572037`*^9, {3.586968488693831*^9, 
   3.5869685002792196`*^9}, {3.5869692114735537`*^9, 3.586969224506118*^9}, 
   3.5885695516702194`*^9, 3.588579710747755*^9, 3.5890766241897135`*^9}]
}, Open  ]],

Cell["\<\
Ratio of q for star and Earth (see above)\
\>", "Text",
 CellChangeTimes->{{3.5869685301218014`*^9, 3.5869685381197615`*^9}, {
  3.5869685743601093`*^9, 3.5869685770634327`*^9}, {3.5869686162341347`*^9, 
  3.5869686165191665`*^9}, {3.586968738423794*^9, 3.5869687408230815`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"q", "/", 
  TemplateBox[{"0.3458"},
   "QuantityUnit",
   DisplayFunction->(TooltipBox[
     StyleBox[
      RowBox[{#, "\[InvisibleSpace]", 
        StyleBox[
         RowBox[{" ", 
           RowBox[{"\"%\""}]}], "QuantityUnitTraditionalLabel"]}], 
      ShowStringCharacters -> False], "Unit: percent"]& ),
   InterpretationFunction->(RowBox[{"Quantity", "[", 
      RowBox[{#, ",", "\"Percent\""}], "]"}]& )]}]], "Input",
 CellChangeTimes->{{3.4031702339829693`*^9, 3.4031702411029696`*^9}, {
  3.5017398568838367`*^9, 3.501739860459041*^9}, {3.5869685814589615`*^9, 
  3.586968591883211*^9}}],

Cell[BoxData["21.938971945405598`"], "Output",
 CellChangeTimes->{
  3.5755383515538464`*^9, 3.5755388924597464`*^9, 3.5755389630382156`*^9, {
   3.5755389987094955`*^9, 3.5755390234754667`*^9}, 3.575539221816264*^9, 
   3.586581209920082*^9, 3.5869672167812214`*^9, {3.586968493329387*^9, 
   3.586968500318226*^9}, 3.5869685925972977`*^9, {3.5869692115705643`*^9, 
   3.5869692246591353`*^9}, 3.5885695517162256`*^9, 3.588579710796773*^9, 
   3.5890766242366905`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Centrifugal deformation of a spherical planet", "Section",
 CellChangeTimes->{{3.3946814792479353`*^9, 3.394681482992935*^9}, 
   3.394681995008935*^9, 3.5017404277954907`*^9, {3.5017404844577312`*^9, 
   3.5017404907850933`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{3.5017404277954907`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(7\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.5755383518178787`*^9, 3.5755388927227783`*^9, 3.5755389633122478`*^9, {
   3.575538998980527*^9, 3.575539023751501*^9}, 3.5865812107532277`*^9, 
   3.5869672174122934`*^9, 3.586969031423951*^9, {3.5869692118435974`*^9, 
   3.5869692252031994`*^9}, 3.588569551870249*^9, 3.588579710954824*^9, 
   3.589076624492708*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{{3.58696868304915*^9, 3.5869687124076824`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["g", "0"], "=", "AccelerationDueToGravity"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"a", "=", "EarthRadius"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", "5.5"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"average", " ", "density", " ", "of", " ", "Earth"}], " ", "*)"}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "1"], "=", "4.5"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"average", " ", "mantle", " ", "density"}], " ", "*)"}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"q", "=", 
   RowBox[{"0.35", " ", "Percent"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "centrifugal", " ", "acceleration", " ", "in", " ", "units", " ", "of", 
    " ", "surface", " ", "gravity"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.394603687079938*^9, 3.394603687461938*^9}, {
   3.4302887217052956`*^9, 3.4302888048892956`*^9}, {3.4302894577532954`*^9, 
   3.4302894770002956`*^9}, {3.4302897445872955`*^9, 
   3.4302897721682954`*^9}, {3.4304572702646084`*^9, 3.4304572964486084`*^9}, 
   3.5017404277964907`*^9, {3.5869687009012938`*^9, 3.586968702078434*^9}, {
   3.5869687880117455`*^9, 3.5869688661901236`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.5869686936724243`*^9, 3.586968694647542*^9}}],

Cell["Centrifugal deformation range", "Text",
 CellChangeTimes->{{3.5869689362275286`*^9, 3.586968951847402*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "0"], "=", 
  RowBox[{
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", " "}]], "q", " ", "a"}], "//", 
   RowBox[{"ToUnit", "[", "Kilometer", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.501740551188548*^9, 3.501740578686121*^9}}],

Cell[BoxData[
 TemplateBox[{"11.161739749999999`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575538351933892*^9, 3.5755388929738073`*^9, 3.5755389634222617`*^9, {
   3.5755389990865393`*^9, 3.575539023992528*^9}, 3.586581211370262*^9, 
   3.586967217579314*^9, 3.586969031616974*^9, {3.58696921195461*^9, 
   3.5869692253382177`*^9}, 3.5885695519242926`*^9, 3.5885797110158324`*^9, 
   3.5890766246777315`*^9}]
}, Open  ]],

Cell["Correction factor", "Text",
 CellChangeTimes->{{3.586968979400709*^9, 3.586968982056028*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 SuperscriptBox[
  RowBox[{"(", 
   RowBox[{"1", "-", 
    RowBox[{
     FractionBox["3", "5"], 
     FractionBox[
      SubscriptBox["\[Rho]", "1"], 
      SubscriptBox["\[Rho]", "0"]]}]}], ")"}], 
  RowBox[{"-", "1"}]]], "Input",
 CellChangeTimes->{
  3.5017404277964907`*^9, {3.5869690200325837`*^9, 3.5869690244431133`*^9}}],

Cell[BoxData["1.9642857142857144`"], "Output",
 CellChangeTimes->{
  3.5755383519718966`*^9, 3.5755388930128126`*^9, 3.5755389636302867`*^9, {
   3.575538999124544*^9, 3.575539024029532*^9}, 3.5865812114402905`*^9, 
   3.5869672176203194`*^9, {3.5869690260453043`*^9, 3.586969031659979*^9}, {
   3.5869692120656257`*^9, 3.586969225805273*^9}, 3.588569551944296*^9, 
   3.5885797110477934`*^9, 3.5890766247037754`*^9}]
}, Open  ]],

Cell["\<\
Corrected centrifugal deformation range\
\>", "Text",
 CellChangeTimes->{{3.5869689956716595`*^9, 3.586969005287814*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["h", "0"], "=", 
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox["1", 
      RowBox[{"2", " "}]], "q", " ", "a"}], 
    RowBox[{"1", "-", 
     RowBox[{
      FractionBox["3", "5"], 
      FractionBox[
       SubscriptBox["\[Rho]", "1"], 
       SubscriptBox["\[Rho]", "0"]]}]}]], "//", 
   RowBox[{"ToUnit", "[", "Kilometer", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.4302887501392956`*^9, 3.4302887548262954`*^9}, {
   3.430457282367608*^9, 3.4304572872816086`*^9}, 3.501740427797491*^9}],

Cell[BoxData[
 TemplateBox[{"21.9248459375`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575538352071907*^9, 3.5755388931218266`*^9, 3.575538963735297*^9, {
   3.575538999219556*^9, 3.575539024125543*^9}, 3.5865812115442867`*^9, 
   3.586967217734334*^9, 3.586969031755989*^9, {3.5869692121686363`*^9, 
   3.5869692259182854`*^9}, 3.588569551994302*^9, 3.588579711128804*^9, 
   3.5890766247737436`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Proof of eq. (4.11)", "Section",
 CellChangeTimes->{{3.5017418966655054`*^9, 3.5017418973685455`*^9}, {
  3.586969085431429*^9, 3.5869690859754953`*^9}, {3.586969162727705*^9, 
  3.586969169639536*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.430104899551345*^9, 3.430104902689345*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(5\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{
  3.575538352322938*^9, 3.575538893381858*^9, 3.575538964023332*^9, {
   3.5755389994775867`*^9, 3.5755390243855762`*^9}, 3.5865812126324596`*^9, 
   3.5869672180173674`*^9, 3.586969032022022*^9, {3.5869692124346685`*^9, 
   3.5869692268413982`*^9}, 3.588569552150323*^9, 3.5885797115358686`*^9, 
   3.5890766249988337`*^9}]
}, Open  ]],

Cell[TextData[{
 "Put ",
 Cell[BoxData[
  FormBox[
   RowBox[{"c", "=", 
    RowBox[{"cos", " ", "\[Theta]"}]}], TraditionalForm]]],
 " and calculate integral"
}], "Text",
 CellChangeTimes->{{3.5017422430573177`*^9, 3.5017422529068813`*^9}, {
  3.5869692574010673`*^9, 3.5869692609354877`*^9}, {3.587016137453381*^9, 
  3.5870161378664303`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Integral]", 
   RowBox[{"-", "1"}], "1"], 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox["c", "2"], 
      SuperscriptBox["u", "2"]}], "+", 
     RowBox[{
      FractionBox["1", "2"], 
      RowBox[{"(", 
       RowBox[{"1", "-", 
        SuperscriptBox["c", "2"]}], ")"}], 
      RowBox[{"(", 
       RowBox[{"1", "-", 
        SuperscriptBox["u", "2"]}], ")"}]}], "-", 
     FractionBox["1", "3"]}], 
    SqrtBox[
     RowBox[{"2", 
      RowBox[{"(", 
       RowBox[{"1", "-", "u"}], ")"}]}]]], 
   RowBox[{"\[DifferentialD]", "u"}]}]}]], "Input",
 CellChangeTimes->{{3.43062421213161*^9, 3.4306243841966095`*^9}, {
  3.4306254788136096`*^9, 3.4306254805846095`*^9}}],

Cell[BoxData[
 RowBox[{
  FractionBox["2", "15"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{"3", " ", 
     SuperscriptBox["c", "2"]}]}], ")"}]}]], "Output",
 CellChangeTimes->{
  3.5755383562184067`*^9, 3.5755388965982456`*^9, 3.575538964734419*^9, {
   3.5755390002886834`*^9, 3.575539025049656*^9}, 3.5865812156548166`*^9, 
   3.586967221125741*^9, 3.5869690327141037`*^9, {3.586969213120752*^9, 
   3.5869692276354914`*^9}, 3.588569556293867*^9, 3.588579713222084*^9, 
   3.589076627546097*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Section 4.4 The Earth, the Moon, and the tides", "Title",
 CellChangeTimes->{{3.5017388907655773`*^9, 3.501738892175658*^9}, {
  3.501740872360918*^9, 3.5017408823214884`*^9}}],

Cell[CellGroupData[{

Cell["The tidal field", "Section",
 CellChangeTimes->{{3.5017411395061984`*^9, 3.5017411432404118`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.501741146460596*^9, 3.5017411498677907`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(2\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755393703820896`*^9, 3.5755394639573174`*^9, 
  3.586581216105897*^9, 3.586967221488783*^9, 3.5869692288466377`*^9, 
  3.587092651067505*^9, 3.587124817620534*^9, 3.5885695566949034`*^9, 
  3.588579713472103*^9, 3.5890766277501607`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{{3.5870162771641455`*^9, 3.587016278874349*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"G", "=", 
   RowBox[{"GravitationalConstant", "//", "ToSI"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"EarthRadius", "//", "N"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DD", "=", 
   RowBox[{"MoonOrbitRadius", "//", "N"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"m", " ", "=", " ", 
   RowBox[{"MoonMass", "//", "N"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", " ", "=", " ", 
   RowBox[{"EarthMass", "//", "N"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.5017411769083376`*^9, 3.5017411851398087`*^9}, {
  3.5017413636040163`*^9, 3.501741364619074*^9}, {3.5755394581536217`*^9, 
  3.5755394594837804`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.5870162873073626`*^9, 3.58701628881054*^9}}],

Cell["Distance of CM from center of Earth", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"d", "=", 
  RowBox[{
   RowBox[{"DD", " ", 
    FractionBox["m", 
     RowBox[{"m", "+", "M"}]]}], "//", 
   RowBox[{"ToUnit", "[", "Kilometer", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.575539529084132*^9, 3.5755395318584657`*^9}}],

Cell[BoxData[
 TemplateBox[{"4672.250467672661`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575539370717129*^9, 3.575539464258355*^9, 
  3.5755395365700293`*^9, 3.586581216222912*^9, 3.586967221676806*^9, 
  3.5869692301857996`*^9, 3.587092651275529*^9, 3.5871248177205887`*^9, 
  3.5885695568149157`*^9, 3.5885797135981183`*^9, 3.589076627843173*^9}]
}, Open  ]],

Cell["\<\
Depth of center-of-mass of Earth-Moon system\
\>", "Text",
 CellChangeTimes->{{3.5870163055165467`*^9, 3.587016311196229*^9}, {
  3.58701639938781*^9, 3.587016409883068*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a", "-", "d"}], "//", 
  RowBox[{"ToUnit", "[", "Kilometer", "]"}]}]], "Input",
 CellChangeTimes->{{3.5755395612900143`*^9, 3.57553956189707*^9}}],

Cell[BoxData[
 TemplateBox[{"1705.8865323273385`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755393708081408`*^9, 3.5755394643533664`*^9, 
  3.5755395626441603`*^9, 3.586581216294947*^9, 3.586967221794821*^9, 
  3.5869692303848248`*^9, 3.5870926513655405`*^9, 3.587124817766595*^9, 
  3.5885695568788815`*^9, 3.5885797137981443`*^9, 3.589076627891179*^9}]
}, Open  ]],

Cell["\<\
Uniform (non-tidal) Moon acceleration field acting on Earth\
\>", "Text",
 CellChangeTimes->{{3.501741405180394*^9, 3.5017414268166313`*^9}, {
  3.58701642865132*^9, 3.5870164313706474`*^9}, {3.5870164616752834`*^9, 
  3.587016463530506*^9}, {3.5870165630994525`*^9, 3.5870165919459133`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"G", " ", "m"}], 
   SuperscriptBox["DD", "2"]], "//", 
  RowBox[{"ToUnit", "[", "Micrometer", "]"}]}]], "Input",
 CellChangeTimes->{{3.5017411948403635`*^9, 3.5017412325405197`*^9}}],

Cell[BoxData[
 TemplateBox[{"33.18896058870014`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"\[Micro]m\"", "\[InvisibleSpace]", "\"/\"", 
            "\[InvisibleSpace]", 
            SuperscriptBox["\"s\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: micrometers per second squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Micrometers\"", 
        SuperscriptBox["\"Seconds\"", "2"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755393711031785`*^9, 3.5755394646223993`*^9, 
  3.586581216718953*^9, 3.5869672220828557`*^9, 3.5869692307328634`*^9, 
  3.587092651644574*^9, 3.5871248178985853`*^9, 3.588569557302978*^9, 
  3.5885797139841676`*^9, 3.5890766280311985`*^9}]
}, Open  ]],

Cell["Scale of tidal Moon field", "Text",
 CellChangeTimes->{{3.5017414333840075`*^9, 3.5017414370412164`*^9}, {
  3.587016744690242*^9, 3.5870167470025177`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"3", "a", " ", "G", " ", "m"}], 
   RowBox[{
    SuperscriptBox["DD", "3"], " "}]], "//", 
  RowBox[{"ToUnit", "[", "Micrometer", "]"}]}]], "Input"],

Cell[BoxData[
 TemplateBox[{"1.6520540075779993`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"\[Micro]m\"", "\[InvisibleSpace]", "\"/\"", 
            "\[InvisibleSpace]", 
            SuperscriptBox["\"s\"", "2"]}]}], 
        "QuantityUnitTraditionalLabel"]}], ShowStringCharacters -> False], 
    "Unit: micrometers per second squared"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Micrometers\"", 
        SuperscriptBox["\"Seconds\"", "2"]]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755393714732203`*^9, 3.5755394650294456`*^9, 
  3.5865812170309715`*^9, 3.5869672225739145`*^9, 3.586969232292054*^9, 
  3.587092652063622*^9, 3.5871248181026373`*^9, 3.5885695578090124`*^9, 
  3.5885797142081985`*^9, 3.5890766282412214`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tidal range", "Section"],

Cell["Tidal constant", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["H", "0"], "=", 
  RowBox[{
   RowBox[{
    FractionBox["3", "2"], "a", 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["a", "DD"], ")"}], "3"], 
    FractionBox["m", "M"]}], "//", 
   RowBox[{"ToUnit", "[", "Centimeter", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.5017410922724967`*^9, 3.5017410974957952`*^9}}],

Cell[BoxData[
 TemplateBox[{"53.77318290302951`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"cm\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: centimeters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Centimeters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575539597824379*^9, 3.5865812171590443`*^9, 
  3.586967223011969*^9, 3.5869692328211155`*^9, 3.5870926522976503`*^9, 
  3.5871248181866474`*^9, 3.5885695591192055`*^9, 3.588579714304242*^9, 
  3.5890766283232484`*^9}]
}, Open  ]],

Cell["\<\
Influence of tidal range self-potential\
\>", "Text",
 CellChangeTimes->{{3.3946814792479353`*^9, 3.394681482992935*^9}, {
  3.394681990400935*^9, 3.3946819921609354`*^9}, {3.394682148720935*^9, 
  3.394682150272935*^9}, {3.5017416569627953`*^9, 3.501741659095917*^9}, {
  3.5870172666028624`*^9, 3.5870172699942694`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    SubscriptBox["\[Rho]", "1"], "=", 
    RowBox[{"1.0", " ", 
     RowBox[{"Kilogram", "/", 
      SuperscriptBox["Meter", "3"]}]}]}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"water", " ", "density"}], " ", "*)"}], ";"}], " "}], "\n", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Rho]", "0"], "=", 
   RowBox[{"5.5", " ", 
    RowBox[{"Kilogram", "/", 
     SuperscriptBox["Meter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"average", " ", "density", " ", "of", " ", "Earth"}], " ", "*)"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.394603687079938*^9, 3.394603687461938*^9}, {
  3.3946821585639353`*^9, 3.394682159809935*^9}, {3.430711263271407*^9, 
  3.430711269012407*^9}, {3.5017417144200816`*^9, 3.501741760077693*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 FractionBox["1", 
  RowBox[{"1", "-", 
   RowBox[{
    FractionBox["3", "5"], 
    FractionBox[
     SubscriptBox["\[Rho]", "1"], 
     SubscriptBox["\[Rho]", "0"]]}]}]]], "Input",
 CellChangeTimes->{{3.3946821862419353`*^9, 3.394682206034935*^9}, {
  3.430711284254407*^9, 3.4307113203504066`*^9}}],

Cell[BoxData["1.1224489795918369`"], "Output",
 CellChangeTimes->{3.5755395982854347`*^9, 3.586581217535077*^9, 
  3.5869672233390074`*^9, 3.586969233817235*^9, 3.587092652596686*^9, 
  3.587124818331674*^9, 3.5885695593272495`*^9, 3.5885797145682726`*^9, 
  3.5890766284702144`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tidal cycles", "Section",
 CellChangeTimes->{{3.501744602041244*^9, 3.5017446039123507`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.4303796650230246`*^9, 3.4303796680930247`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(8\\)\[NoBreak] symbols\"\>"}]], "Message", \
"MSG",
 CellChangeTimes->{3.5755396439039087`*^9, 3.5865812177841063`*^9, 
  3.5869672236110387`*^9, 3.5869692342932916`*^9, 3.587092652860718*^9, 
  3.5871248184806457`*^9, 3.5885695596792784`*^9, 3.5885797152943583`*^9, 
  3.589076628628276*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coefficients", "Subsection",
 CellChangeTimes->{{3.4304523039746084`*^9, 3.4304523055606084`*^9}}],

Cell["Use \[Psi] = \[CapitalOmega] t+ \[Phi] to write", "Text",
 CellChangeTimes->{{3.5870175236767073`*^9, 3.5870175524581614`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"cos\[Theta]", "=", 
   RowBox[{
    RowBox[{
     RowBox[{"Sin", "[", "\[Delta]", "]"}], 
     RowBox[{"Sin", "[", "\[Delta]0", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Cos", "[", "\[Delta]", "]"}], 
     RowBox[{"Cos", "[", "\[Delta]0", "]"}], 
     RowBox[{"Cos", "[", "\[Psi]", "]"}]}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4303797052110248`*^9, 3.430379755295025*^9}}],

Cell[TextData[{
 "Take ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["h", "0"], "=", 
    RowBox[{"1", " ", "to", " ", "get"}]}], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Text",
 CellChangeTimes->{{3.5870175656267414`*^9, 3.5870175809715824`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"h", "=", 
  RowBox[{
   RowBox[{
    SuperscriptBox["cos\[Theta]", "2"], "-", 
    FractionBox["1", "3"]}], "//", "Expand"}]}]], "Input",
 CellChangeTimes->{{3.430379674864025*^9, 3.4303797021740246`*^9}, {
  3.430379767183025*^9, 3.4303798870230246`*^9}, {3.430379919953025*^9, 
  3.430379946848025*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", "3"]}], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Delta]", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Delta]0", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Psi]", "]"}], "2"]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Cos", "[", "\[Delta]", "]"}], " ", 
   RowBox[{"Cos", "[", "\[Delta]0", "]"}], " ", 
   RowBox[{"Cos", "[", "\[Psi]", "]"}], " ", 
   RowBox[{"Sin", "[", "\[Delta]", "]"}], " ", 
   RowBox[{"Sin", "[", "\[Delta]0", "]"}]}], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Sin", "[", "\[Delta]", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Sin", "[", "\[Delta]0", "]"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.5755396441159334`*^9, 3.586581219480283*^9, 
  3.586967223733054*^9, 3.5869692343402977`*^9, 3.587092652900723*^9, 
  3.5871248185306935`*^9, 3.58856955983632*^9, 3.5885797157284307`*^9, 
  3.589076628688238*^9}]
}, Open  ]],

Cell["Simplify the squared cosine", "Text",
 CellChangeTimes->{{3.587017587404356*^9, 3.587017599994866*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"h", "//.", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"Cos", "[", "\[Psi]", "]"}], "2"], "\[Rule]", 
    FractionBox[
     RowBox[{
      RowBox[{"Cos", "[", 
       RowBox[{"2", "\[Psi]"}], "]"}], "+", "1"}], "2"]}]}], "//", 
  "Expand"}]], "Input",
 CellChangeTimes->{{3.430379948703025*^9, 3.430379956734025*^9}, {
  3.430380254208025*^9, 3.430380259503025*^9}, {3.430380524319025*^9, 
  3.430380524815025*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   FractionBox["1", "3"]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Delta]", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Delta]0", "]"}], "2"]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Delta]", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Delta]0", "]"}], "2"], " ", 
   RowBox[{"Cos", "[", 
    RowBox[{"2", " ", "\[Psi]"}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Cos", "[", "\[Delta]", "]"}], " ", 
   RowBox[{"Cos", "[", "\[Delta]0", "]"}], " ", 
   RowBox[{"Cos", "[", "\[Psi]", "]"}], " ", 
   RowBox[{"Sin", "[", "\[Delta]", "]"}], " ", 
   RowBox[{"Sin", "[", "\[Delta]0", "]"}]}], "+", 
  RowBox[{
   SuperscriptBox[
    RowBox[{"Sin", "[", "\[Delta]", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Sin", "[", "\[Delta]0", "]"}], "2"]}]}]], "Output",
 CellChangeTimes->{3.575539644155939*^9, 3.5865812195503445`*^9, 
  3.58696722378006*^9, 3.5869692343923025`*^9, 3.587092652933728*^9, 
  3.587124818546692*^9, 3.588569559891292*^9, 3.588579715759433*^9, 
  3.589076628706281*^9}]
}, Open  ]],

Cell[TextData[{
 "Check the expression for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "0"], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Text",
 CellChangeTimes->{{3.5870176159637804`*^9, 3.5870176443831935`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["1", "3"]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "\[Delta]", "]"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"Cos", "[", "\[Delta]0", "]"}], "2"]}], "+", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Sin", "[", "\[Delta]", "]"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"Sin", "[", "\[Delta]0", "]"}], "2"]}]}], "\[Equal]", 
   RowBox[{
    FractionBox["3", "2"], 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Sin", "[", "\[Delta]", "]"}], "2"], "-", 
      FractionBox["1", "3"]}], ")"}], 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"Sin", "[", "\[Delta]0", "]"}], "2"], "-", 
      FractionBox["1", "3"]}], ")"}]}]}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{{3.430380274656025*^9, 3.4303804024310246`*^9}, {
  3.430380528833025*^9, 3.430380529184025*^9}}],

Cell[BoxData["True"], "Output",
 CellChangeTimes->{3.5755396442689514`*^9, 3.5865812196483436`*^9, 
  3.586967223862071*^9, 3.586969234484313*^9, 3.5870926530367393`*^9, 
  3.587124818632663*^9, 3.588569559967314*^9, 3.588579716949544*^9, 
  3.589076628750246*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example 4.3 The Stanford Experiment and Figure 4.1", "Section",
 CellChangeTimes->{{3.430379659727025*^9, 3.4303796628760247`*^9}, {
  3.501744566921235*^9, 3.501744580168993*^9}, {3.5017454798804536`*^9, 
  3.5017454846727276`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.4304524101856084`*^9, 3.4304524132636085`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(34\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5755396519038696`*^9, 3.5865812206974325`*^9, 
  3.586967224144102*^9, 3.5869692347593465`*^9, 3.5870926532957706`*^9, 
  3.5870932842924843`*^9, 3.587124818782695*^9, 3.5885695607513943`*^9, 
  3.588579717240697*^9, 3.588579787402547*^9, 3.589076628909308*^9, 
  3.5890766640447083`*^9, 3.5890767818276224`*^9, 3.589076845820666*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parameters", "Subsection",
 CellChangeTimes->{{3.5054081372385683`*^9, 3.50540814530903*^9}, {
  3.5870176749588604`*^9, 3.5870176761860056`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"\[Delta]", "=", 
   RowBox[{"37.4", "Degree"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"Stanford", " ", "latitude"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Delta]0", "=", 
   RowBox[{"20.", " ", "Degree"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"lunar", " ", "latitude", " ", 
    RowBox[{"(", "guess", ")"}]}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"h0", "=", 
   RowBox[{"90", "Centimeter"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"tidal", " ", "range", " ", 
    RowBox[{"(", "guess", ")"}]}], " ", "*)"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"a", "=", "18"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"start", " ", "time"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"b", "=", 
   RowBox[{"a", "+", "56"}]}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{"finish", " ", "time"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"LunarDay", "=", 
   RowBox[{"24.8379", "Hour"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.430452318077608*^9, 3.4304523195346084`*^9}, {
   3.4304524753366084`*^9, 3.430452522615608*^9}, {3.430452586398608*^9, 
   3.4304525879396086`*^9}, {3.430452996405608*^9, 3.430452999084608*^9}, {
   3.4304530611186085`*^9, 3.4304531397086086`*^9}, {3.4304532304076085`*^9, 
   3.4304532306876087`*^9}, 3.4304533721546087`*^9, {3.4304536425506086`*^9, 
   3.4304536918826084`*^9}, {3.4304538470946083`*^9, 
   3.4304538473706083`*^9}, {3.4304538813076086`*^9, 
   3.4304538820086083`*^9}, {3.430453913537608*^9, 3.4304539156306086`*^9}, {
   3.4304539605486083`*^9, 3.4304539644426084`*^9}, {3.4304547761256084`*^9, 
   3.4304548112786083`*^9}, {3.4304550163626084`*^9, 
   3.4304550600466084`*^9}, {3.4304551266796083`*^9, 3.430455127294608*^9}, {
   3.4304555854936085`*^9, 3.430455586457608*^9}, {3.4304556338406086`*^9, 
   3.430455660188608*^9}, {3.4304557228146086`*^9, 3.4304557546196084`*^9}, {
   3.4304559328516083`*^9, 3.4304559334366083`*^9}, 3.430456406740608*^9, 
   3.4304575508356085`*^9, {3.4304576207236085`*^9, 3.430457634016608*^9}, {
   3.4304593573776083`*^9, 3.4304593576266084`*^9}, {3.4304593929286084`*^9, 
   3.4304593932596083`*^9}, {3.4304594245426083`*^9, 3.430459444023608*^9}, {
   3.4304781990100126`*^9, 3.4304781994200125`*^9}, {3.430489812931012*^9, 
   3.4304899205970125`*^9}, {3.4304899561180124`*^9, 
   3.4304899562880125`*^9}, {3.4304901500410123`*^9, 
   3.4304901843940125`*^9}, {3.430490700577012*^9, 3.43049081056*^9}, {
   3.430490864977*^9, 3.430490898441*^9}, {3.430493949023*^9, 
   3.4304939492980003`*^9}, {3.430493994766*^9, 3.4304940525810003`*^9}, {
   3.430494437539*^9, 3.4304944643900003`*^9}, {3.430494868374*^9, 
   3.4304948744700003`*^9}, {3.430494931828*^9, 3.430494936646*^9}, {
   3.4304949952869997`*^9, 3.430495043322*^9}, {3.43049561626*^9, 
   3.430495655627*^9}, {3.430497991759*^9, 3.430498017657*^9}, {
   3.430498058972*^9, 3.4304980592650003`*^9}, {3.430498089535*^9, 
   3.4304982171029997`*^9}, {3.4304982638190002`*^9, 3.430498286837*^9}, {
   3.4304983562019997`*^9, 3.430498376324*^9}, {3.501744084044616*^9, 
   3.501744113099278*^9}, {3.501744158364867*^9, 3.501744184347353*^9}, {
   3.50174501597992*^9, 3.501745081226652*^9}, {3.501745191643967*^9, 
   3.5017452199155846`*^9}, {3.5017455673204546`*^9, 
   3.5017456851011915`*^9}, {3.587017736974304*^9, 3.5870177446732244`*^9}, {
   3.5870177860151854`*^9, 3.5870178137305107`*^9}, {3.587017852175124*^9, 
   3.5870178691541605`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Subsection",
 CellChangeTimes->{{3.5870914943437157`*^9, 3.587091495333832*^9}}],

Cell[TextData[{
 "Radial gravity in units of standard gravity  ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Gamma]", "=", 
    RowBox[{
     SubscriptBox["g", "r"], "/", 
     SubscriptBox["g", "0"]}]}], TraditionalForm]],
  FormatType->"TraditionalForm"],
 " "
}], "Text",
 CellChangeTimes->{{3.5870919616237803`*^9, 3.5870920232931848`*^9}, {
  3.587092125064392*^9, 3.5870921762205286`*^9}, {3.5870922091254773`*^9, 
  3.5870922376048937`*^9}, {3.587092470696863*^9, 3.5870924788048344`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Gamma]", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", "h0"}], 
    RowBox[{"EarthRadius", " "}]], " ", "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.4304533786386085`*^9, 3.430453395063608*^9}, {
   3.4304556815696087`*^9, 3.4304556818126087`*^9}, {3.430456946516608*^9, 
   3.430456966740608*^9}, 3.4304939609440002`*^9, {3.4304949657869997`*^9, 
   3.430494968113*^9}}],

Cell[BoxData["2.8221406971973163950539162140936`8.*^-7"], "Output",
 CellChangeTimes->{3.5755396521949043`*^9, 3.586581221000511*^9, 
  3.586967224271117*^9, 3.5869692349703712`*^9, 3.5870926534607906`*^9, 
  3.5870932845395117`*^9, 3.587124818840691*^9, 3.5885695613024487`*^9, 
  3.588579717368617*^9, 3.5885797875005064`*^9, 3.589076628968313*^9, 
  3.589076664143773*^9, 3.5890767819206038`*^9, 3.5890768459227147`*^9}]
}, Open  ]],

Cell["Lunar angular frequency", "Text",
 CellChangeTimes->{{3.587092251542568*^9, 3.5870922654142327`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalOmega]", "=", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{"2", "\[Pi]"}], "LunarDay"], "//", 
    RowBox[{"ToUnit", "[", 
     RowBox[{"1", "/", "Hour"}], "]"}]}], "//", "NoUnit"}]}]], "Input",
 CellChangeTimes->{{3.430452318077608*^9, 3.4304523195346084`*^9}, {
   3.4304524753366084`*^9, 3.430452522615608*^9}, {3.430452586398608*^9, 
   3.4304525879396086`*^9}, {3.430452996405608*^9, 3.430452999084608*^9}, {
   3.4304530611186085`*^9, 3.4304531397086086`*^9}, {3.4304532304076085`*^9, 
   3.4304532306876087`*^9}, 3.4304533721546087`*^9, {3.430454460681608*^9, 
   3.4304544752316084`*^9}, {3.5017448955040293`*^9, 3.501744899580262*^9}, {
   3.501744943935799*^9, 3.501744953901369*^9}, {3.5017450919492655`*^9, 
   3.501745096501526*^9}, {3.5052909355298204`*^9, 3.5052909397540617`*^9}}],

Cell[BoxData["0.2529676545593462`"], "Output",
 CellChangeTimes->{3.575539652264911*^9, 3.586581221072481*^9, 
  3.5869672243421254`*^9, 3.586969235053383*^9, 3.587092653521797*^9, 
  3.5870932846035194`*^9, 3.5871248188756933`*^9, 3.5885695613834534`*^9, 
  3.588579717440633*^9, 3.5885797875385427`*^9, 3.589076629004343*^9, 
  3.589076664183751*^9, 3.589076781955599*^9, 3.5890768459567275`*^9}]
}, Open  ]],

Cell["\<\
Coefficients of null potential surface (4.21)\
\>", "Text",
 CellChangeTimes->{{3.587092281030106*^9, 3.587092282902331*^9}, {
  3.587092399030265*^9, 3.587092417878525*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "=", 
  RowBox[{
   FractionBox["3", "2"], 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Sin", "[", "\[Delta]", "]"}], "2"], "-", 
     FractionBox["1", "3"]}], ")"}], 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"Sin", "[", "\[Delta]0", "]"}], "2"], "-", 
     FractionBox["1", "3"]}], ")"}]}]}]], "Input",
 CellChangeTimes->{{3.430452554424608*^9, 3.4304525819156084`*^9}}],

Cell[BoxData[
 RowBox[{"-", "0.011544324799402376`"}]], "Output",
 CellChangeTimes->{3.5755396523149176`*^9, 3.5865812214795723`*^9, 
  3.586967224383131*^9, 3.586969235096389*^9, 3.5870926535538025`*^9, 
  3.5870932846415234`*^9, 3.58712481889271*^9, 3.5885695614225035`*^9, 
  3.5885797174716415`*^9, 3.588579787565541*^9, 3.589076629028346*^9, 
  3.5890766642097588`*^9, 3.5890767819726443`*^9, 3.589076845974682*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A1", "=", 
  RowBox[{
   FractionBox["1", "2"], 
   RowBox[{"Sin", "[", 
    RowBox[{"2", "\[Delta]"}], "]"}], 
   RowBox[{"Sin", "[", 
    RowBox[{"2", "\[Delta]0"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.4304526234896083`*^9, 3.430452657275608*^9}}],

Cell[BoxData["0.31015032289497024`"], "Output",
 CellChangeTimes->{3.575539652356922*^9, 3.586581222063647*^9, 
  3.586967224426136*^9, 3.5869692351503935`*^9, 3.5870926535858054`*^9, 
  3.5870932846755285`*^9, 3.5871248189096985`*^9, 3.5885695614675236`*^9, 
  3.5885797175036163`*^9, 3.5885797875895176`*^9, 3.589076629049304*^9, 
  3.5890766642357316`*^9, 3.5890767819926057`*^9, 3.589076845993684*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A2", "=", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Delta]", "]"}], "2"], " ", 
   SuperscriptBox[
    RowBox[{"Cos", "[", "\[Delta]0", "]"}], "2"]}]}]], "Input",
 CellChangeTimes->{{3.4304526610786085`*^9, 3.4304526774956083`*^9}}],

Cell[BoxData["0.2786352731379508`"], "Output",
 CellChangeTimes->{3.5755396523939266`*^9, 3.5865812221856194`*^9, 
  3.586967224469141*^9, 3.5869692352024*^9, 3.587092653618811*^9, 
  3.5870932847085304`*^9, 3.5871248189247417`*^9, 3.5885695615125184`*^9, 
  3.588579717644662*^9, 3.588579787609557*^9, 3.589076629076289*^9, 
  3.5890766642527437`*^9, 3.5890767820086026`*^9, 3.5890768460116863`*^9}]
}, Open  ]],

Cell[TextData[{
 "Change in strength of gravity relative to standard gravity ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"here", " ", "-", 
     SubscriptBox["g", "0"]}], ")"}], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Text",
 CellChangeTimes->{{3.4304577796776085`*^9, 3.430457792018608*^9}, {
  3.430494903083*^9, 3.430494903784*^9}, {3.5870924299899793`*^9, 
  3.5870924473980684`*^9}, {3.587092496021901*^9, 3.587092528194762*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"g", "[", "t_", "]"}], "=", 
    RowBox[{
     RowBox[{"-", "\[Gamma]"}], 
     RowBox[{"(", 
      RowBox[{"A0", "+", 
       RowBox[{"A1", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"\[CapitalOmega]", " ", "t"}], "]"}]}], "+", 
       RowBox[{"A2", " ", 
        RowBox[{"Cos", "[", 
         RowBox[{"2", 
          RowBox[{"(", 
           RowBox[{"\[CapitalOmega]", " ", "t"}], ")"}]}], "]"}]}]}], 
      ")"}]}]}], ";"}], " "}]], "Input",
 CellChangeTimes->{{3.4304526862716084`*^9, 3.4304526909106083`*^9}, {
   3.4304529581646085`*^9, 3.4304529880976086`*^9}, 3.4304531183316083`*^9, {
   3.4304532084586086`*^9, 3.4304532172186084`*^9}, {3.430453307966608*^9, 
   3.4304533172536087`*^9}, {3.430453404478608*^9, 3.4304534052126083`*^9}, 
   3.4304534361586084`*^9, {3.4304544843296084`*^9, 3.4304544957696085`*^9}, {
   3.430454815049608*^9, 3.4304548285156083`*^9}, {3.4304577626376085`*^9, 
   3.4304577877986083`*^9}, {3.4304944741809998`*^9, 3.430494477516*^9}}],

Cell["Plot of calculated g[t] ", "Text",
 CellChangeTimes->{{3.58709254552684*^9, 3.587092598230166*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figa", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"g", "[", "t", "]"}], 
     SuperscriptBox["10", "9"]}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "a", ",", "b"}], "}"}], ",", 
    RowBox[{"Frame", "\[Rule]", "True"}], ",", 
    RowBox[{"Axes", "\[Rule]", "False"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"a", ",", "b"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "250"}], ",", "150"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Dashed", ",", "Black"}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4304531954816084`*^9, 3.4304532070986085`*^9}, {
   3.4304532384366083`*^9, 3.430453248952608*^9}, {3.4304532831726084`*^9, 
   3.4304532882846084`*^9}, {3.4304533226096087`*^9, 3.4304533458946085`*^9}, 
   3.4304535008026085`*^9, 3.430453652400608*^9, {3.4304540931186085`*^9, 
   3.430454189464608*^9}, {3.4304544984256086`*^9, 3.4304545453556085`*^9}, {
   3.4304546383176084`*^9, 3.430454666853608*^9}, {3.4304548422086086`*^9, 
   3.4304548553046083`*^9}, {3.4304548925406084`*^9, 3.4304549850226088`*^9}, 
   3.4304550715336084`*^9, 3.4304551498426085`*^9, {3.4304558815486083`*^9, 
   3.4304559057926083`*^9}, {3.430457563016608*^9, 3.4304575673006086`*^9}, {
   3.4304578325646086`*^9, 3.430457899655608*^9}, {3.4304579500726085`*^9, 
   3.4304581638826084`*^9}, 3.4304582320986085`*^9, {3.4304592901806087`*^9, 
   3.4304592911216087`*^9}, {3.4304593351876087`*^9, 
   3.4304593395606084`*^9}, {3.4304779852810125`*^9, 
   3.4304779924680123`*^9}, {3.430478108412012*^9, 3.4304781220460124`*^9}, {
   3.430489356610012*^9, 3.4304893814910126`*^9}, {3.4304894124820127`*^9, 
   3.4304894619340124`*^9}, {3.4304895248020124`*^9, 3.430489533318012*^9}, {
   3.4304896377480125`*^9, 3.4304896411180124`*^9}, {3.430489676721012*^9, 
   3.4304896829230127`*^9}, {3.4304897260990124`*^9, 3.4304897564630127`*^9}, 
   3.430490203575012*^9, {3.430493474602*^9, 3.4304934811470003`*^9}, {
   3.501744206094597*^9, 3.5017442380954275`*^9}, {3.501744269983251*^9, 
   3.501744274948535*^9}, {3.5017443299046783`*^9, 3.5017443399102507`*^9}}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJwUmXk8VG8bxlXWkpJKKFmKsWRfxn4ZhhnGLJYZCVmmslUoJZVCi1CWtFBJ
WiVtpBI5JaJEC5Il6kclpZBI4j3vXz7355znuZfrvp/zfI1q0Gb3dTNFRESo
oiIi//97L1lXTkTECImj09k3R/io2t9zZZ2oEcSH2G/V+/moTcy2fS5phN7t
bEez93w075oMPS5rhNrKtXLUp3z8jGwgdNWNUPT6rvK1bD7GNu7lZ2kaYYfj
S9qig3xMhZl8G9MxwikXqaKlO/iQXndavtrECBPfN4+J+fJBWR2xabWTEaQ6
TpRiGR/6fBXRKhcj+NN9VXOl+TDzaM5ZwTGCXnJaYOZfLzi6Wdf8EBhhwRKK
8+o2LwTYSy89EGqEp5U3d7EPe+Gk1rX622lG2LtjLPBNryfOagSsXZJphG2H
SjbNafTEJfWFo7uPGUF2qXzzhzJPlC7bpcLMM0J16ec5YQc98VKWtb37hhEi
Ig6xB9Q9ITkxsEL6jRFito7gpYsH5o2dLY96S+YrGej9b5UHFv/y4L7tMEJb
3jG95/M9sGLwwc6CXiMwy/nDzBZ34GPqa+pvI0Qe4Vq3r3bH9mc6e9cpGKPD
nSa3h8dD79D2ai1lY+xK+Tb7sw4P7gpPxAfVjJHF/SvfK8aDbojv4W26xkh7
76Vy9h4XPWKHcw/aGaOUE2y+UoELF/vB0sJ1xmCWRo/cPewG5Xs3+7/fMgb1
79ktqbeckdo9qXu7zBgNo7USZ9ycMS7OjNz2wBgp88UaJ7844Y3Xh99TT4xh
vdr9wrVlTkgekRWd32aMF8xs7u94RwzrbVE2mjLGEelhjQNq9qi5aOqxzcUE
va3nJJSGjSHDUL3fzzaB++ezsQ/kjMH/Kr3cz8MEytsCYhNJnT/r9X518DVB
7A2tfRlbDCB1LytRdpMJmhyH/kq+1ILbsx+3rmWawPSRxJ3g0sVoGSyc/1+b
CX4kWr5PSFAj+syVGzkbTOE4Y96IsZIjkZbOa78Wboq/7uLixcccCZPP+z5J
RZpinkhXoep8OpF4fGCqOtYU5dOfQn6LOhEqo/f0qSmmwBH6AqVBZ8L3tkem
SrEpFj05WFpe4Uq81j3kOTRsii1N/H/sszwibl9FAHvMFJLNAV8T23mEaueP
iKK/pnjYybh0YpE7EZnK379O1AzqoYzRtanuhMxX9bJ3i8zw4KjZd/cYD4J5
+aH8Y6oZCg9d3f7X3osgVH+1Z8WbQbQ2z8HynjfReL4qm51khgKZHX6+w95E
x8pU9uxkM+j63ZW00F1N/NZSe5yQaYaXPza9X3Z2NbHKiFsYecEME8lUr9gE
HyKXdm07+5kZmsLefU8x9yWuPNluOLvJDCJm9I3tEb5EmZPDQM0bM8zwu2rZ
fM6XeOXS7m/TZQapBYveP57tR0h6SDrp/jRDSczm6Kg2PyImWLhw9mJzaM09
xaIFrSUS+/QbaxTN8UghIK4mcy2RseHvwYTl5hh9PqrZS6wlroVn/R2nmOPB
5IjjuEwA8XEL8fGzlTnCwl8Sx0MDCO6+pbdqAs1hFlJ0tGphIOEv/iUsYb05
Eh8LZ+poBRLhySUrbMLNsVvmwjU/m0DiQJrLydKt5lhTsdhUfl0gUZkdu+f8
QXNkLXn27uWtQEL7YrNbQrE58i0jZecjiJArsNP+dssckTJrlAs5QcRk3lVx
QZk5SkpDRuesDSIaT+6t0q0yx9JM61l6u4OI6DRdw7aX5sjNmHjbWRZErDl0
Yq5DizmY2ZphlU+CCMcDM78WvzPHZtrf7nWvg4jFe9sKkj6aozDJNXvu9yDi
/pZ9C/V/mcONulnt9vJgoiDyx8+ccXM8exDS+FwnmEjZ6PNC9J85frw3vVVk
Hkz4bjA40C5Gxb+/kfxKdjAhsqZzbL88FWLx84bV4oKJfoFz808lKqy76lVC
9wUTrz1v31yjQoV+sPairUeCiQvs5FBDLSoeB/tfeXsumGDQTDo6LamYjlRR
fVUTTBjanb3rbEeFmdMXZYvGYELRenb2bQcq4uz3XRe2BhPfTHtYySwqEq3e
/RX7FEy0GLlqjXCpkKr3jt35PZh4qF8m5u9FRXeTpWjpr2AiXSvtobE/FXNi
j73eOFNIxGqM5eYFUfHs0aOOQUkhEaAetF1qAxUNffcSjOYJCaNlVIPuzVTY
ioVaz1ASEkqK56VdtpL+nrxZnqYiJETlZfpLY6nob9po826lkGid33suJYGK
lh/nmu7rC4mquZz40f1UbDkwoehmIiSuzC73CUihwtdQhF1AFRKZEivNnx+h
opeXw79vLSTiRDPkzI5SUfT4hnoWhETwjL8/8k9QUU8xvKjrKCRYU+sa5pym
QgXzGpKdhYTp35dXtuVTMaSGlEsuQkJ53Gr/hwtU7E97+Hq/m5CQGL0UyCqk
Qlw+JUeDKyR+Dsna3i2mIm3r5TfJ7kLi3eAuRbXbVDidV9l+zVNIPB74/Dut
jMx/qdjOTL6QKPri/masnIoskbC35t5CIruv8kZQFRU5FmH7z64WErs+UtJe
VFMhf1ZjT52PkFjXfTSEWkdFzPzaqttrhESp+InhGS+o0Hj+rDbUl6yXXu6u
56+oaP25W+ktaXt4nRE/1kpFfmftgwnSLtiVn+HfQcXEjd+Xmkl76Px5RUoP
Fcn8ui4haeP5pQtDvWR/JQYEFZH7pw8X6j3op4Kb50a9Qvp/r1B8b98gGW/l
QeEaMr5V9jdp7BEq9u2K/P5EQMYbUtIgP076f/+6vddLSDSkl/E/TFJxTf0/
0wcepL537/dcnWEBdpatFIMnJMLeV4RtFbdAj+1AQCpbSJSLEb9s5ljA86nA
Zb+rkJBaVR0vMd8COyTXNJsyhIS3Z63kq4UWUHtuPO8MqdflnfVZuQoWSA9N
ln5A6vm7oGGpUNkC/RHru9NJvbOHXhuMaVpgvu5vb39jIfHfktZyQtcCYyLV
ut56ZP/hnWOKoQXWRbENpLSExKsj3d7LrCzQ3KxvkLlMSKiUffz4yc4Cn5Y+
FYbIC4nIrr6Im44WeBCZZzE2X0jI6H7b68C2wD3HRgfrWULCpX78SmigBSht
xRc39AYTOT//Ghmvt0Cxwey86Y5g4ov8dMVkmAXa9/ev5r4JJg6sF3uVvtUC
i07WP136KJionrXgz52DFri+9mfs+pxgYoHOoqQ9aRaQ6Lm/t56c5yD3JXOZ
mRbQyLOr/ZREngfnlFU7cy0QckjkA3djMGFtq8Oced0CwXuLdvVbBxOp6/Te
PL9tgYQL/IwVBsFEe5qh37G7FhBNfX9TST2Y2NFhHkV5ZIHVDx8vsZMMJspi
HXPYzRY43j/vRWBTEKFf4tefO2EBRy0LIy9eEPE+/YoCe9oCaYw6tUfk+ZkW
McKcIWoJZa2svz/0g4gvKw9d3TDXEtkHRM6fmBtE5J8sDTdRsUThWPJt+dpA
Yn78nMEXdEtcFt2Xv1I3kHjow1dOcLEEEa+y47N8IBFhfo5twrHE0kHlq1Gz
Aom6n2Y3cr0tsWja9FjPuwAiITgockO4Jf51z3Ey3hdADDPuD4tkWiLOdPCR
VN5aIn+lqHrpMXJ9Skzk701rCfZMjseGXEtQf6y8utduLXH1QW/Ji/OWoFOj
TaK6/YkgvfkxuWWWULyfQ+1Q9CfeyG0YM+60RO7m3QOn9/oSpe8XTq7XtIJT
2pxfW2VWE7Zm0+x6HSsINbtcH7Z5E08P95/TMbCCaPmOHQ0F3kS79UPnn1Qr
iHyNLbU08yamczcc3eFihU3xNmHbfASEi6BcO22jFZa5ZDno5HgR3U1rV98q
sQLn+5UzN0Z5xOxHV8v+2lnD+tJa1W0PHIkJ2qq7FY7WWLhE7IqkmSPR/+TG
3d1Maxw27zlx4qYDUVdXem/a3RqaGnKSpRdpxMGXD8tnrbfGw7qrnpQMEKI9
rx/OSbPGIXtr49kvqcS/qYlapTZrCO37Apc0qhDf9u5+2tlpjQPV+95WZCwj
OmeK1J35YI2cwIGPmw8qEBXiYvXLB6wRsn6qJ32nLBEnM++5+pQ18lrlvodK
tVX9XqbepLPCBo1K9Wv2jyzDT2vXt1aRNjiq+mW45CR5j7HYOFG01QbnKOHX
uqLNccAsfdnSHTZYUJowsJz8riw1eBM8mWCDsFOzrD7+tYCz+pqfFUdt4P7S
U3ou1wZ5UuFStnfJ9/PPCWm99mC1plhhygbz2mrPlI84Q/7NNf+bM21x/ejy
raUcBj42NSaoSNii/WiluGwRA7H1C+pE5tvCur1p+vtaJs5X5Ho+UrXFFKPa
S+GxCyYKrm6k0W2RMLDQl7fBDZc3PzvrmGaLuXvsgzfJ8/BS1SQ0L8MWhh8j
Lo9Z8DDx5ozReLYt8p0339P05YFNjaopOmOLmeeKJXae5eG3yJKBBTdssaAq
U+eTqjscs9aZfXhli4cnjjVmLfLAJsemactWW/TQ9tZvMPLAyd/U+ux2W/hM
PdnezfbAgI+0L+M/WwwYSmmUHfBAlnpJwo1ftkjbEVnyd8gDPaUiL3bL2yHo
kkPNE5I3Zm8IO/5WyQ4t+iVGt154wkShea2hih2k52ybaUvyysHdl4d7KXYI
vl2xhTrfC3pO7CUsSzvIr64qvh7sBe/xux8u2tpBd0mO4entXki8qlo0TbPD
4KTpaa1UL7TIjNqUuNphwEx39opbXtj19lSwop8d7LU7vh0f98KlQ2Krtgba
Ib9phm2SFB8vrTb/frHODkrvkzbOU+RDPZ92KGGTHR7URu6XsOTjWUj/9S97
7FA6q/DF9hg+fil6xNL22eEzu8tCmMSH8osK+9PJdhiLEQ7+zOAj2jCjmZNp
hyvP5A8OXOVDYcJsoqzADhuNA6gh7XxcbaVGbb5shzDtQT+HT3xYllh+0bxm
h/e1mu3Xh/jwibBtPXnHDjpShkfMJQT4yoAbr9wOXo/qNb1kBYhbSXsiVWUH
asfpjllKAuS+p9+Oq7MDgerNi1YJoP3AWcv4hR1ev/r2JNZUgPITzPyBV3bY
uvPPsk02ArhsdZW/0GqHZeEWD0YdBWjnuh3x7bBD+nPP7nksAcJWccQW9djB
8xqzqcJdgAkp3q4XvaQ+naGvJ70FUKz2DLMdtENCj6SVuVCAq2f5H34P22Hn
rHJLo1ABLHd5e98Ys4Ne+O+ERxsFeObt07Rh0g7K5zOC+6IEKFbUU88QAdRm
njPJixFgVPKG/7eZgIh2EutbrAA2Y/q5DDHANe6N6MudAuzvu9lyQQI4ti3i
PjNegBdvDGVnzAYkVIR1q/cKsOjxbZafNLCeJpUlkiiA303j5PsywNXSnljb
JAEu5pVWL5IFFh0u+CC3T4DvaabTUXKAGNPcOIG0TXeWWTYuAiK+lLYeIO3d
oebbtJcAi/+tMV9J2jWCe7cOKAJPi7Mf+5L7zXWy+P5xKeBUPPB5FenPy6Sc
Yrec9L9ynkIOGc8ZNSvhKVVA8fjz+jwy3r75FWfH1IHiK5KRdrsEWDVt3eGh
AZiN1STt3CFAzPfKxTcpZL518ns8tglQ2WHrLq0DyFepv38aLYDYs6rDIauA
d3uWLX+3SQC3e6h/og8MMhweHwwT4NilR6KqRsC2ve3M5nUCdGXTsNsEyDbT
tawKEGBlUvXOd2ZA4ukEBeYaATZGOd41tQBiI1Uio70EuLO2ZjjTCojxd66z
4gjwz81Jb9AGmMfXvn6RIcBhbcbFSzSyXlpX/PwsBWhZUt8zkw5cOefBvGYk
wDIJl6VrnYEND5vF87RJff9zPSrPAvz2unEjFEh9XzU0bmEDzX82zHWZT+pL
uM1+yQWMGo5NNYiTep7mJCR7Aa/vnhh5MszHotSXFb0CYOEFVXOrL3z47eCN
wwe44aKmt7qLj+9eHpv/+APDpuvqj9TyMXeewC8sBEhdnaowks3H43JN7bww
YOO2vB15yXzErh///SoCsIkozJixi4/eypwMiyigSrRv5VAgH+URndWScYD6
YSWRA9p8hDwL1L6cCrh1uryffcOL5HijsfbDgOY1ye7teV54ozrriUwGsP/r
vZWXDnvBdscFv23ZgHDnZRlmuBcWUT5n0M8AGSvZLuXqXni8P2Lsv+uA7kWF
wK5Dnog1tHkifwv4t0C/51qMJ1Z1zc10LQHemn+hMgI9cdLkhnbJXSCI4mPZ
Z+6Jzf8N+SUSgLTb1n/PP3pgqf32JyqvgU2fa7u+6ntg22RCpt8o8PyolEfC
NR7WVirfIcaA9C2dMXMzeWDEP2hTnwDaAg8T22N4UBAZXf51CiTPnyj8asND
xayQ69sk7bHPsuWj8AUXM+ewn6cr2ePzPUNnqSYOjigoij22t4dugn1PqjIL
29vLKCsd7cG7Kyc18tEVAac8WMlO9hj2SHnmedkVhssOH2W72qOuo+vlPANX
NKuIqHV42mPplvdNOfYuUKR8sv21wR6ZosVHC9czcNnsdqzGEXsEqJ7e0Frj
SDZC9Ob4DHvkPbObtj/iiLaNxutbs+yR4qpS8IHvCMmLpR4HTthjyp99ofqL
A0Lk7up9yrfHulznAMxzgPbPB70XS+xRX3XpfFKIPR6n7ur4d8ceI54pqUMm
9vDRtHntdc8eHcr2n67NsEeq/8MqsUrSn2z/uOxp4FsDkbuu1h6H1Pqs4wvt
cL2whrvynT1+LHpr+6rYGk70A867O+xxPjxP/j7PGu97nGxbuuwxkHw9MvS3
FeYtqdPZ/9EeVQGhFno0K0QeeCbeN2AP1YBZbzP+s4BRcFPFhSl7vO5l633l
kBw9lV4yKULD9n3C2SkzzBGUy73qOYsG8fyxCoVSMxx99eqEqCQNj/zSkmWX
muGXXXO0UJaGN5IxXmfHTFC29B1lxQoa7A6J6ot0G6Eipv+mhwYNSsnJNU8u
GOFx4x9qEoWGzRpz3sSEG6FxryLzoy4NKta90wV/DdHXuyY035SGhwusJLia
hvhqEzHUZE5DidvnvrxfBvh5fNeOaQsaLOY+/d372ACTjDOH/G1paKk4fGJN
kAEWFr8vXOpMg8Dy1TlNH30oiv8wZDFpePff8ZdxovpQWTt9f6cr+dyjq7Lx
ph50ZVWetXNoOBxiNy9hnh6MwgzcZ7vTEJb3JvK/R6tArUa7hScNFxRqUtnb
VsEhJvDrSW8aZjoa+jj16YLRGBVd50ODlAv3fOtZXbA1EyfGfGmgVcx8tc1X
Fz7vCmYLAmnY9cE15nuHDqJterUXhdFwOeKXdfeINmKP/7rtGEHDyYQPgYpV
2oj/IWq1dRMNnLEOZlSaNlLOrXR9E03Dlg9RXttXaSNjwvTNzBgabl589pIy
QxvHPZzWGG0nz1jO2YcTrVo4J7YhPHMnDcMVnnNFD2nhkv/2EWI3DbZdTB+b
dVq4dvfgzp97aAi9/m4o10ELt+efnKWSSMO8QlxZtkIL90KvpHL20XCjcZVj
rbgWHj6+J7fnAA0/Lj2MPzZAwROl+lPXk8n8knbOSntNwbOt79Tfp9AwJukZ
efUBBS9f9BfNPUyD2ozTXkOXKGjVmDC2SafBebOkWWA2BZ17ZldEZNIwa/Da
gYkkCj62KTqePkqDg/uf65UxFHw21Gl4fox8fuyuZFEoBd9SrDz/nqBh26zZ
PY/XUjD8n2undi4NV+6EbhHzpmDc2lfoc5qGKdeLI9HuFEwdi/h2KI+G5uHu
pDkcCkR/7Np6P58G6+IMiyY3CmYzDk9+KSD1+5Sw9CGbgvnnzuxbcpGGNc6v
NNp4FCyeKJZmXKbB790Px2UCCpZ6PMzeXkiD3oYCr8P+FKhda1x6uYjUR3um
NiWEAopY94XWYhp0Knfv/76FglX+P3TFb9Lw3aN9WVcCBesbns9Zd5vUt+1w
acRRCvIsr3ytLqVhdO5y/4OXKXh7ZV+92l0aNumu26/7kIxHPvBKwn0aeIRP
4pq3FDD32xzseUBD6SXVXLkRChJHFNbbPaRh4W6JaQ9ZLfxqeq3+7zEN+T/9
lcK8tLDK9sZM3xoasu5tfWezUwvrr6V+KH9Kw9qaf0PHzmuhLdnxbGwD2f/Z
56Ma/2pBdkwl/m0jOX8G15jXdLThsu6fr9krsh948/yl/LVRYV+m+KuFBs+l
41yrZ9oYvZH1x72NhtRlV+fMFdWBnvLmtlvtNBRtUHsggA7yJzSPb+6moZ+T
2ccldPAuRDSm8QMNx+Q1Cv/O0sWCtz0eq3ppUJdrHpZ30cW+khzZgS/k+bBv
c8/pLl1Uqm77yRwg6xPUajlDZxV+p7s3XflOw7h3cHJj3CqERMw5vH6YBn/j
5wW5qnpgrdwt+fEvDU+Xx0aWpOpj/9HVnzFFQ3z02XjFb/p4OMOs9qyIAwoM
vf6FaRrA4P1gkp+YA7o9jnuePUyeFycCpttkHNBnwhJ9tMEQbmI2781lHWC5
fNGipecMcWCLQuVxOQcozBz9nN1hiHHO6zjPJQ5Q3Zzzu83DCJ2Sjr+bVMnn
c5tfx/KMsWi7SoveCge4pse6Zhw1BrtvsuSwhgO04ucNl7Uag3h0J8pVxwFO
RPBHrQATXIjT/F5r4gCpT19tNu4xRcS32b2Vzg4gfNdEHPxujgVLzD1/MR3Q
rNeUuMiSivsOwU+0WQ54dWmbT9RBKsRPPbhwgusApZwY95QVFjjHjFgXtdoB
xWdy+ZIhlmi99OLTinAHiP3cUemzgLxIBWb0px5xQN7DLeRhZA+51kU/Vrc4
gBOWl1wx7oxvi2XKqt46wLxFsWzImIFagfjule0OkPPe/9FuMwNx737PHnrv
AHHFoAvLPzHQ0/lWI5n87q3cuHLr3WYmrv+X41/21wEO0vQtOSWucBla1rhA
zRE7eLKj8WEcrDBafCx2hSN0dKP/yF3kYCpaxve9hiN476hHkro5uPVrqr9Q
h/zOyv+xHdXiYsl4t5i9qSM8DvRLcs9w8WnqnM1mhiOsNDb8GYvnIVFao/j5
JkeIdJ8xstD0wBwKzVkjyhFsndrxV3YeyHbw/7B3iyP0KTerLL09cDnu+CKz
WEcUhryoiU32QMMX8fj8BEeMlJQqHPnkgUU1n9kx2Y4osxn8vOakJ/J6ZvU3
HXdEgdoOdlqxJyiTy5O0cxxRVW8/Ev/YE5bG3ne7zzhCqBJVeHnAE/7n6pa7
XHaE546LvT8svXBld+FP5XJH9EhXBV5p8IJhTk3KjgpHSI39GznV5YXy0g8r
mh86Qjv4lDx90AsvBhRWH6p2RODIrkV3ZPgYXp3yaKTBERWmFPaYKx87Yy6t
YTc5Ijjpb3WiDx+imY9Hr7xyxApNg9+VIXzI101o+bc6otLlv0fYx4e1WXhW
Xbcj3PdIXxfe5aOGd1BX/aMjXJa/KvGp5oO98Xzt7l5yP+/Ps/428hF4oWPC
qN8RTXxOv3kfH1+rxrIPDziC7itl9uYHH1s75PS/fHfE7o/frRZP8HFgASv4
zLAjFAa1RjPmCjBPL+Tf2C9HhGb0b3y6WICTzH0n3MccEStxTfv8cpLP9lY8
l5gkn//plmEYCGB8um1d0JQjtsy1mj2fKkDF3V/TFSJ01A5vGYqzE6BpUNck
WoyOvZotduZuAtQ+Pbn2ggQd2UN7ZJU9yPfzRVNbpei43mvw5xbJgyU7Issk
pekwkO8/MOxH+nPv/GApQ8eWcjVWd5AA53QYczfOp0Ps4puO7RvI+ERLqWcX
0BHfsvnZg3AB0ruWC18tpMNncKD46mYBDpSlps+SpyPoatUyxy0kj6WPlZsq
0PGd+eX2QZKPtoYEf9qgRMeF8a8SMSQ/hdk3yeYuo+O3ztt7c0i+ClS0smlY
Tsf8CUawK8lf3iOXQqZU6eCFhl8yJvmM3bAg22AFHdR7E2pPEgSgX4yvCtKg
gzjjtHOS5Dnr+K9fsyl0zEw08egkec9YwF/8VJsOpxv23GCSB7UNHtv/0aXD
eed+hUzSVpXS26ijT8edkBLBOtJe8jHnpJ8hHQNYdqebXD/vgdiTdGM66lbe
6Zom9xfPjvrxyJSO86fm7akh/U9GdCn+MieBKnUdw4CMb4TOdNKwpOOn1fGv
dmT8X5XvRHlb0xFH11ryi+ThnjGVMym2dKjFcfezyPzfvkyrqwAdkXX9nxlk
fRoLx0cGaXRMLkwY+0LyY02icLkqnY7/5rhv1CXrW7HmpYuHMx26X5micmT9
r869cu6uKx2Hz/cdqgsk9fok96LfjQ6/jQO/j/uSelXtGVfi0nHCsztBQkDy
eJSAs9eTjqhZjBmNLgLscqmOu82nY+rsYIK6owDR6vqXer3pWKLwlapoI0BA
i/g/hh+pV2lczNdVAgiuR1N2rqWjspd2rHolqcfB9x7FgWS/dNUtt1lG1t+i
7KrsejrMMr5WzJ8jgPyZdd7tm+jw3hMUbfqRD5ltr5Kko+jQuHDMbO5bch45
Njdst9Bx8tJD9X3PyfkVWSR+fjsd6+bQJ9glfDQKn5SE76WjXNLyoFc8OY82
Bt1nEunoy0mLUY/ko2Lx6dkv99Fx+0bXw3SSBwvrtgSaHKKjuo6t7ODAxz5d
9Xn/MulYH34wV24mH9cV12T6Z9ORfjwt1ueHF9okj8oRx+mI+cPR0Ov0gm7f
zCVJp+joLVD6vLPUC81nPqpIXaSjYMKOcWutF6ZSFc+HXaZju15Zpz/TC5Q4
95UNhXSIaM6MzDHywi7+Y63063SE3L7bcWeWF9TnFRgtukeuf1fteSTfE6x/
70q3ldPxeN+4z6UDntg2IGveVkFH94kkE1aEJ549TbA69YiOMRfJf/Zmnoje
G+io+pyOT/f3lrY/8cDjIRX+qvd0fJj14f6PBnd86/ZuO9JDxynXT0mTRe5Y
3Jjh8/MjHW47ft7PTXFH2FWRtSWf6TgzMVKY5uSOBcE9GyyGyHynv33i3Och
qPnsDrqoE8Q+yiddSuFixl3lM37aTnALrn8uqHLFyaBLN4/rOiEz6qzH3lBX
GMjoPWnSc8LVZ2NXZsiR/LjOZsDe2AkHlAcrFwhd8FDO10LD2glYdsg/dYr8
/kXmtAy6OaHwef7cukVO+KW1UGZvlBNctOW1uUV2+HRaak/BPSdomeV3rgqX
QaHegNZSR2e8XKywNMiaRax4FrRhh5MzGv+FMu5XsYi8de0XWhnOmOP/uvIT
zY3IOl2/PNPNGeEhR50eMNhE3Owri8QFzuScr1UQ0+ISrE/CGT9DnfH4keKM
PF13YvDM+7bqdGeIXlEK+beQT4RY8BerZjlDuKTYr5XHJz42v/CIz3ZGof4G
k21H+ETrnMpGao4zqly4GuISAqJqx6knxQXOuHhdcIcxKiAyvLxvnrhDvl+o
Hh9Ss5ownvv6YHiHM/RDfBRe6PoRiutz+PldzrC7nKom4edHzKwKWNnS7Yye
XUldw2l+xOvIH49te53xZWiDT+eAHxHdLD0l+90ZMzbeYX+86E/cPOW89d6U
M8QFF47PFgsgTv6ScfguwsDi65o5yYoBxF63Vlm1WQyE5h3Kf6cfQHCmhTdS
JRi4KacQMLI6gBgMSvzqP58BifPOFpNFAYSudmWAmBoDSJmKUHQOJBYm7dO3
XMFASXG4zbnVgcRkh+vUJg0GvMxVfo6GBxLPD7efatNmQOm1f6NERiARNjTW
WkTec7rrR71rWgKJwntGbu50Br4/3nM1VBBEZMhOKB10ZsD0fXf/tfVBRGzY
o68PmOS9Z+Lo7fsxQQRjKTd5JZuBvxTvz3pHg4gvezZWj/MZcB5Rckh4HkRo
OF+xOBvCQJHDv9ajhsGE/R2rtbPCGVg6gM9utsGEr3rTvg0bGdjCy/lY7RJM
ZE2PNulHM/Dm5N4rH4OCiX93HdYTOxlYdKtIqS89mFii8TZ1RTwDBUuUNGJz
gwmj7LBbyXsZ2JEQu7rmQjAREpk5yd3PgPT0YPThe8FEM6U768MRBmbGazk0
dQYTP45H36NnMvD09Qp/vb5gYraY+PvCo6S/9hoTp+/BBD7oakWfJPNxmth+
djKYWMMh2K25DAQab8vpmSUktlV6bLU8Q/rfH9bzbraQuJYTVzWzgIFVCqcE
v+SFxFMJmb71FxhoX1vmvEhZSHyMOTf7+SUGjHZJdX1XFxJL3Ou8jhYxsDuK
dqBGT0gYE2t2jhUzkLF04bonxkKCrfcjf81NBuRCf8fHUYVE0uzF39TvMBA8
PkdK2l5I5MUWyibfZeAHz4Xz2VFI3Ptkbf7tPpmf8OauHQwh8cbzpS+3goFq
50d7HrsKicHHwYmlDxlYbbGe8ZgtJKQMxy4vecRA1NDg01iekFhxNuXFrmoG
FMdkh/o8hITdXOWRnhoG7vZ/uyvFFxI+O28todeR+leLLugVCImYfkfbwmcM
+C5w+r11tZDIELQFz33BQMgqMb8KHyFRVBN+KKqJgVLfKbvyNUKi1nj6essr
Bs7/88rd5CskPpzLarZoZqDwzMmQd6Q9OU9j4kwrA8lzH1z5/+/L8vH3l898
x8C1CEWf16Rt9I1FX9/BQKXurt0BpO3m0xP2rIsBl7kyiy6S+4fUbcnQ62Hg
+rz1K/JJ/0lmEmVZHxlYEpx4k0fGl3cht+N3LwP3GPVV98n47y/Qm7HmMwPq
PyV92ryERPPeRxpV/Qy0uG5PLibzHxz0ZKl/Y2Cg65aXBVkfKb8vUQcHGRDb
XfliJ1m/Fc93nhj4yYBAVnxqK1lfW4t5lZwRBn4y/UdXkPVffbngY8koAzou
09WppD5bF5lJLhkn68VJ3lkIIZGeVL9q1wQ5n+wpvXhrIXF1yNejZ5IB19Tn
w1Kk3j0vkvKuzGDi3GlpUWeyP64utjoxQ5SJmRde/5LVIuu/djjdR5yJY3VL
Ro+T/TR7KCBBeg4TFsOdaU1k/zVbLIlbN5eJwppva5JkyfwTm6IfzmPiHeVs
4RDZr8YLbYWRC5nwc1e3nSL7e8p31Ld+MRPx9UTt6dFgou7iNS81BSbGXx7n
/Sbnwddcybl5GWlHx023dwUTB3zGtaiaTOTpXvq3qTyY4J6/oZapxcT6jVde
9t8MJpS+rVf6qsPEn3O3+1UuBxM3d7dInzZg4tu/PZefZAUT7/Jv/5i2YIJx
Y/vVz+uDiQv9oV+8rZmI+xL3j7kmmNhkpPrhli0TWvxyrTBOMDHzSfrrYAcm
CtiFg71mwYTu54g7T1lMzOs5lPVsZjDxW3/FdRUO6b/RaJ7aaBDxKLbj0g4e
E5p3s+kWn4MI/myXkzp8Jo446n66/CyI2LNKc2f6WiYMJpevP3wkiHi95QME
0Ux0xq3IvTc7iDhdcdLi5lYmSr5Eten+CSQ2iHGNpLYzMdjXtmndZ/I8Pf5Q
/cFOJnw6NYZXVAcSK8tPiS/fz4QIsenWp+2BRKwIv+HTCSZqyn9vrG4NIG6K
+/Hlc5mo2nWzWa46gPgiLexxPs1Ef+k+mumNAMJbIfrXlXwmKl/e3vbtQABB
NTqyNLyQSZ6vIXamJgHEeHBtxI9yJi7nuC4YlV5LxD41mzvexSS5yGFmxhJf
4uYLmxOUHiY8GW0yNZ/WEF/eOKqu/shEtbtrx+3SNYR3j7tp+ScmXowEtD3g
rCGofzb57frBxOurmadyEn2Ice3LxSIzXPAmK0DXotOb2HFEni21wgUSW78r
dQm9iN7Cr3qhGi64Xd3i/E3Di+DWVM6rp7hg6xdZh6dfPAmNyeBXyatcoBz7
Tasv3JN4HXrTU9LcBUU6l6J6NnsQOnTmGgkX8h4xtqOqeTOP6PobFyIW6YI7
mt8yTt5wIexD3yfOqHABX+/nO72VJoQibcYo9aELdHOuBVD0jYlhxRUbIgkX
sHQFPhstjIgLDaGu3U9ccHfLb8MvbANC0mBUrvKFC+6brAmdO0khGn/PuRjb
7YKZ2/r9VAg5wmefxdMfM13hszRNQ+SQOoz9fC00xUguj3t0O9lzJaTN9hT5
S7jinDH/4BllTVR+fpL+Yo4rckauhrpf14aKK9e7aKErpt1DWk51GeCT7Ib+
9RquyKUGVD96bI7os9lz3jNdUbRsvcvtCBom3Du4s1muMPz5Za9sEw2J4mrH
zdiuWPixvuqrgQOObryhku5O7u9/NUV8xAGlVs9MscYVNkxP/8eRJDe9nV5b
EOGK5ZfNSlg8BuJTnS42bnKFdJh+1OOrDIjbHf46EemKMacyr/xZTMhfUozx
iHGFWpVqYkYJE9Stpimi8a4w+ht6XkPaFTvmh5euT3cF93WpLv+sG2Y+ufUn
K9MVPefUetsH3ZCyfdy26qgrLinKDd6zYSP3/f56+ZOuCNvqdmf3OzbKr+W/
r8sn/ZXd/n1IlItJRqukzm1XmFiWR31S52Hf5FK2oNQV7/b8ekt34kH6ZvDR
pDJXiHdfU2WF8LBMfmhZZ7krfss2a7GKeLDtm2N8uNqV3PftqOwqd+xNsPf7
0eyKzi1n9uxY7gFrF88/Lm9dwUuSeJdl64GxBRuOXXrniqPsP7ts/Tyw6ULa
C7/3ruA4vSp0zfHAmqdvbRo+u+L+BpazubQnFmd8faf5lcx7qvhop6YnXnv/
i0n65grkFcYscPAE46vadcshVySfUeDox3rCbO4m5cIJV9hXvzDW6vLEUMve
ctF/rhDy08bYo564lneUHzBN9kdNv6XoXPL+b3D/iLwoC4vTGnuXWnlhvofo
9P65LDhXlDWKZnmhQUn+1Id5LGTLbTl44ZIXDvZqmdssYIEls5D5ppzkjRjO
5l+LWRD78ba964MXBk7mdgepsDCD3b7xsjYflwKLdz5UYyGw+HlmlRUfQdqE
vOJKFi7kZn9az+Kj7UEf55UWC8TPuHXBEXxk7xsf0NVlQaJM5e69nXxw3OYk
J+uxEHXxWvKZFD5q3xsQdsYsyPSHLlp8mY+Eyw6+p0xZWNR8dc7VUj6sI/nj
v81ZkEsbHHvxiI+SGbsMrluz0M5jOtS285G/4mn+UicW5ok3qe6WEMD3+zvr
WAYLDQuvGh+TJXmw7FvbGxdyvfXJU7ZKAhxxXiCbymGhbmi2qxPJl8z5K4s/
8VgYiVopcdFUANF35kyaJwsVyjtnHid5dGeYX8IfbxaO2zXf12cJYGYcucxr
DQvekVr/2twFGP6beP+mHwtzZFYfWbRagOInx7ykA1go/ttS1ucvQMjhK0Mb
glhw+rn2MVMowAr+g8PVQha+6s6Ytg8VoFu5UWv5BhZOrpSubNwoQO7nnpq4
UBauL1Pg/IwSgH9zJLA1nAXru7NmX44RYI8ZP11qEws3WLazH8YKcKXiXoV1
JAvn9qudiSV5/xVN6evmaBaMfPuk3+4WYKJut/z5rWR8gvCL7/YIoM7pcWzd
xkL9BafSvQkCsFpo0VI7WFDWv5PckChAzJqLZ613sjBwMW9LZZIAeR8kXmze
zcKyxKynHvsEeLohbKJgD9k/IrVt6aT983uDZmsCC18WLvgXRdoKW/W9pPax
YOs5lDpBrqdNZCZaH2DBoG33ey3SDt/768bmZBYeKmzm/P/392xxQVdBCgub
n8hbxe8VoDLt/uzWNBayEoKnz8cL8GnBUqpUOguvHjT/jdolwLyc+HXWmWT9
fBIzv+wQgLr8Q9bmoyx0b1+kLrldgMCLDkTBMRb8pZ4ufLVFgBSdS99bTrBw
NdmvwSFSgJJbkkpSuSz8OqJ9KyBCgE7zcIb1abJe4ZK6lBAB9BwNzhfks+Bb
OXnkHqmf4FnWy5YCFlos/biJpL57uaP/JC+S/XvMz2fcQ4DXvuXemwtZODvL
cs6AswB/Py49UFDEQs5x+xMh9qTeoXtKWopZmKi2NMu2JOsb4yhjfZuFcE32
wTEdsr5/L1ltLiXny05V22CFALUJUqEFZSyUWbo/WrJUgCVHGqslH7AwblhZ
+Xe2ABWXvHe0VLPQOWPv68V9fPTpPrgkWUv2u67mcCc5DzIly5qt6sjnKplz
BC/5CKj6qFfQwEJyB+EcWs6HaFtE76YWFham6w0vTOVjlX+TbMFbFnTuO/93
KJ4Pfq+hXcs7FiIz1mvejeLjys/fOVbvWZj82f/NWsAHSyqBI/mZzD9R9rDY
cnK+LY+Vnxsn369r+DSvwAs/W6ptRf+ysO3QPMO+dC+wooYfr//HgmtbdVbU
bi+SgznPdWa64Y3QtP22wAsxi2Z33pnjhlCTbVsHJb3A/xn/r17ZDbnTBh/E
1nqS348QDDu6gVJR8fdVtzu22p944unsBq/yqf+Mat3xsrOGcZfpBlF3tRkb
rrkjWU6dt4vthpyY+7msWHeM7+0KFPd2w7Lr1iWaMu5oW+OepBjuhvWZK44M
G/GQK2tdS8t0wxfWHUOjJjaEYeUdu4664Z/tg9s+J9nQq6YOlR1zQ8yL5Dth
gWw82mq6VCfXDQeH751wHiHXvdWLljvvBt8fezRUF7vBLE9VubfUDVmef4zv
rHbFa22Jbfvb3DAQffGG53/OOJ20P7Wq3Q2N0ubT2685Y33nrHN/Ot2gtKv7
RGmMM/4cFmnY+MENUr0BF/ZIOkNl6I8q/6sbHK07/zug74RNd7+90Jh0Q0vc
m7Xn9zlCyvH1yrrlbBTkHbsn42WPxO1bZcZU2bh7fipr6Up7TF5dPLZyBRvH
ZGTmbBoFBmXX1CVR2KgmUm70HAded/8XZmfIxiqliFuscjuc3jl6o4zGxjbr
dfFRP62x6MaJk32ObJxcUXZPKdcaRz5aJCx0ZiPd69jyaEdrJDDj3aNd2bD6
W7lA95QV1stLjK7yZONyCe/DPJ4l9EuWWF5cx8bjYoXswB5zXPlUrta8gY2f
P2JNnh4zh5qi35xZYWz8vbnV4Y+rORYmnOsM2MQGM91L6lm5GcbZ2nuWbmdD
4k3HIW6BKdl3DRtcd7AxvDhUgeVviq93NnHjdrIxoF5rr6Zkiq6lJapte9hk
HXdLrjthgsdfrZ4cTSb96dzrjso1hpXy+2uPU9hgGzY0bvYzRilv77GhNDZ0
pW8v8VI1xuV7T9ZzMtnQOuN/o6PICGkH3KTm5LJRnKhsPaPBEGLlP4YsTrOx
7tDV+KDjhoj/ntkekseGlHvgurJAQ0R6thbVFrDxMGCAa/XXAF5qa9mJRWzc
Y60u6LQwwMzwF4d7itm4XXbxgLKkAa6XWL2wvcmG4tO8fwtu6EOcvoT1t5SN
Wc/XZxAjerh9+EDq6rtszH0uvHAmRw9rW389u3ufjcARmeq39nq4t+EVc+tD
NmqW12NjzioIb9odek2wYf03LukFYxXm/ymuM6hmo23G9InsCV2EpqQ4f3/K
RnD4dmrAOl0oFjvQ179iQ8XuQSTFWwe1o7f2PXlD9sOTue8mFHUQbavyRK2V
Dd6FEjvHHm08b5qkdbezcUrCo5oWrY3YJeGJNl3kfh3D8/7aamNF4LtHp7rZ
KBEsLzeQ0cau4Tvw7mXjufnSBwqlWqBYrdhb9omNszkpxU0pWmhJyqpa2M9G
XuMtE0mhFhIbRKajB9gwn9O1+YmdFvQWbbZ99Z2NqdIb4WLKWmj369qt/5ON
06LL1RqmKDhwybXy8DAb3YoRcQofKTD6cX9y4BcbofuVmYNPKeg2p1i7jLHh
YDbNcb1JQdre4zuv/GHjwTc5W8NTFFDrRR+IT7JJPj91Iz+Zgl7ZLRPCKTbU
JyT0cmMpyPD5YFEtwkHv/Wl95XAKrM9zdqjO4uCmHV9hVQAF/QOV9/aIccCa
KW31VEDBcRPd8S4JDvoWsN5841FA251rbj2bg6ALYXPz2BQM1khuz5XmYKNo
74wONwpyZbaXjctw8Ffct+w8hwInQd8oX5YDl94zy8Y9KBg562F6R46D2uTT
8m0+FOR/ebRVbjEHMYItUQ5CCliGBqVRSzg46L+hzyKSgvEdeSNNihwETH1R
fBBPwYXH0sZ6yzhYP/P4pdp0CrhzdkanLedAqGs92/c8BZMe/be+qnKwz7Cz
NO4+BVdOC4YYKzigahfPVHtNgVdfjcFlDQ5eRqvY8L5TMFPPJFJMi4OOqb5c
iTlauL6t4EawDgeKykN2jjpaEJfco6diwIFn+6Is1hYt3OZ+3xhvxIG/2Emt
xae0sDZnTXGnCQf/kmy6Amu0cE+bqptjwYHHxrr8OFVtCLdcCh+z4qC6R1Oc
7qGN+RULi7xsyfx7rxulHdRGqNuQ1gIHDr5/TyJSxrSR6HFjwpnOgXuteL++
vg5Or974fLczB9yfpw0nQnTwcl1/RL8rB9lrdtuNf9QBNf7DDcKTXP8na+6m
n7pw33d2728+Bwv/dQvmW6xCRIofT3c1ud81kexnSauQf/zd8Ak/DtZwPtH3
LNeD5I1XppvXc3BFs8Hr1mZ9qN1JF78YwoG1QM6zo04f1g/c3raHcTD/cxpf
fIEBIp/Wxzpt5uBT34UFtpEGaOt+9GBZLAetlWJqF6mGuDz/tn3DITKe1B0Z
uuNGIBZHLpiZxoH0n9R9wSbGaF+q95/5EQ4atUrOJkcZQ0br6r7zWRxcexwx
58SgMbbZFzyNO8WBaXHfuRMjJnCKznLTKuYg4lWHS5GmOQJiucprb3CwyVqu
aDLKHHHxMj+yb3FwL2p5nEqlOa6npGSIlJH5hqgVvfWiYvH5xDdvH3Kgltwl
8yLDAn1volcfeMnB0GrfGgkda3K+JvcOvOYgRGa9xetka+ydffAKt4WDD1fd
wiM+W0PR/tSYYjsHk8P3k0Qv24B9vfrYjf84eLataFOUvh3Kkhe+bvvNQT51
9UqWoT2qlddcPCbDxXtq2pu2M47IfjO57IsJFz0yebRqknuyGj2UFplxoebQ
+noryQUZ9VeX0My58Fjpk6pM3nsPV3nLnbYg36e0zfHoY+FAUakk15aLKlcD
V+kmN8QmRYzcdeZiAbUoSTmPA1/jzrqDPlzMP3Zn7rsjPPjoGdeWruGSHHs2
5VMhD95aKdUffLkQrbjv/foJD17LqQ+t1nIxc4GEtf0ED+w52SU/gkl/6c17
g4Ldgf9c8wQbufhtIFXzeZUHVhx9sEVzLxd7ah9n1JzxxLuIcgskcJGoKny+
/ZYnjjjdn/ZO5MI/v4b264knxv+Upabs4+JPSrYVc8ATDQG3z39P5mLkbXYT
19QLCZa3QsVTuDB/+UhIc/aC2cKb+stTuXAebPo17u2F/KfFD7iHubix+GXM
7Z3kfUqv8E1JJhefXMQc6A+9oC15Jachi4u6Uxt/shq90PPh0tq+o+R+Vy9k
LXzvBZdjFwYWH+fiaUI5/cOkF5Qnz86Ky+VCw0oye605H29a8uqzTnHBcTGW
4dP5SL5xJr3oNBfCyy33JTz4GA46pdSVx4XqZvnVuRtJnqs/bozzXFwacPsY
cZaPneeP/fG+wIXT7BlzVhXxYbA7uyrqIhcGY3oD58v4yDXIcj1/mYuUrhXX
ShpIPpydKVtxhYuMKcpu1///Xteb/ra5kIsiaHw68YGPTScOB4tfI+sXfXET
a5QP9ag0reXFXCQr3k8umSL50yX1h/l1sv6aP/RfkDxIm0reGXqTC4UzB7UW
Kwgw9vagfeItLqaj9RIcVQW4duuAxKnbZHyaByLVtUheXLfvaEMpF4NqP/SH
zAVosE1a3XeHi8xdvPndtgIkLklcPlVG1sNBunAnXYBvz/cU6d/not242riK
J8C5i/FRjHIu7t7ZYr9OQPLbnt3mgQ/I/k3+anvPVwDCaGd1ViWp3/BpbvB6
kgek4w4VPeRCzs8hrTxMAO1PsZwnVVw8vp8iXrmJ5MOq7Yu6CC6yk/r4YdEk
L+Vs6xh9xMW8CObixyQPMrfEnJOp5sKyyrC7muTBKdbWDZpPuIgjzrltJnmw
RGPLKtRwISGiOV1N8mCISPSIdy05D4UdWY9IHizrbRUeesrFXuUvVgdJHhSr
t2q9X8dF5O3JjD6SzzyL852/1nMxdPvV0v//vl2QKXZf8Tlpx3MjwkjeG4oJ
03Zt4GJD1XGZDNKGT9OpnS+4+Hf0TzaLtI/Ymsy91kjmVyJ59Cy5vkstJ76z
iZy/3Zl5yeT+uhJTP6RfcTE8s4AjQfqPGwgKtHnNhaRIGpTI+Oqbnr7e+Ias
R+3nuU9I/ltSquuY10zGq3KCPyNOgPUnM+80tpD9qFL34PU2Ae7s+q0x3Urq
92Wsx5zkP9HANSf128j+Eo6GG2wWwJ1OSAW8I/v953+iFWS987VW7sxo58Jq
0obfvk6AH3NTvhEdXCw68VY6NUAA2+FBv6FOLti/e96+8BEgrdWjSfU9Fyb9
X0IKPAXoKL8H924udtl3b5Fkk/qdXXY7sYfcf5X8jWmn//Py5+z/PnLhJn3i
VD7ZT4tZLPGFvVwcy02pYeoLIDS4td2xj5xPppJskoYAM//E+Vz8zEW+nKLC
RTkBuF3dz1u+cLF83qXAw1ICnH3kaCP+lQub1zK9otN8WKfIqGz4Rp5Xu+7V
53/hI2XTlswT37nwaX/a8KSTj3fubTPrBsnzINiTuZ3kwW1KBX2UIS5u72qR
OkjO361rpkVfR7n4VSd7y203HyKZuUuVxrgIXkbb/oGcZ3bM9GHXcS78/KaK
Z/rz8c2mPvLaBBeBrOQl36350GjyNds0zQVFuEIlYNQLW0seXc4T4UFezuGE
30cvVJ/QUGiawcN/rcPyX8jzJyDg5199UR7ehs8z/v//n04NJT0akuRB66mx
632uF+YvLGLFLOBBn2LiGpzuicVv7x7IlONBfX1D8+9tnlDKfUIUL+SRHGWp
punvCQ2V9yafFvPg2eqQ6q/jCWtd2WUCJR7O7rM5rvvIAxsct3+nruDBTvnW
eHi7OyIk9ml6reQhdXqHYvcDd0Q/ywiM0uBhT+XEt6HT7tjFvdpSSOHhmgvO
j/m5I8u3q1JxFQ9i3ihP6eDh4VaHI5Om5H6i12fIknP1xJxbt8ScBx3VGbPP
5XNRP+E705TKw/uaO6dbdnLRvHfbto2WPHybznu3ypCLr2mF/u9tefhzdP5T
F30OFl+cp08480DVPJq95AoLG1s6mpJW83BFENXwWckZz/bHbcn14SGzaYpS
VuUEipmC/K01PEjV7uscCnLCx+N8//d+PFTFmd2+c5kOvverb9QgHiZq7p47
ou0IdNRKDYbx0ORs5pu62B4LPtxy8N7Fw5ZlOmY9Kqa49+3gXd2zPJi7bJHU
1l9GbLhaFkt85YGxUW2Wqqk7sdAxOKN9gAeDmvLq05HuxOOueVd+fePBqWKd
3+8id0JZNvQt5QcPMVojG+3UPIjm7UpmmSNkPTz0vefN9SRA3zsSOMnDl+Ur
NrW3exFLepibZpEcvn1E9mgHdzVRu+P3/mXz3CF+61mJx4HVxBa582fM57vD
2JHPPPxgNdHoNNkQvsAdQ90l2+xX+hD7iq/rNC92xw0mJ/n4qA8xFCfXf2G5
Ox6cUFtmfciXeLaoK9jJ0B3OMUZzh7atJb5utXRJMHKH1rf0rSuvrCVmN58w
qDB2h+abDS+/ta0lXLLc/xmauSNd5Yb9yaUBxHOZuuPLrNxRb66bTo0LIBok
Sup/ObpDMUWSvU87kPi2fv5NfSd3nJaJ1Ii3CySkazceD3Mm493/0sTyfwFj
AZz+VGHATIQoY+BJUkCBV/1K93xhwM0AnvzLTlJARGE8BnrLYcDO+Ygvo1hS
QECKxyBLXGLA7ljmFd5ZUkCJ4ZkvOW1iwA64Q/wYW1JA7zKsZON9YsBOdv7I
jl1SQE6QYOtqnmLAz/JzYnpiUkDWEfxbPdxiwPBR0Ui1Y1JA2z1hhAPrYsAQ
sS4v8GRSQH7KR0qD+WLAUG/p+2VnUkAVrKGErhVjwNDrXpVRbFJAoA4x3atK
Y8DwSrx7jG1SQFWufhE3V2PAEKoZYsduUkC1m2+0eWNjwFBo1C49cVJA/SQz
SiR7Y8DR5EnIKHZSQOV2JSMIp2PA8kOnrmN3UkD+HUswSLFjwBKjBJWeeFJA
GLBM2j27Y8BSYb9hFHtSQPxWPThJzmPAcsAcSE98UkDHh5uJXtdjwJIfei6K
fVJAKCC2sijgY8CyftcUxX5SQC/0C4an6GPA0t00+/9/UkA1rKDX2vBjwMtd
Sts=
     "]]}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{20., 0},
  Frame->True,
  FrameLabel->{None, None},
  Method->{},
  PlotRange->{{18, 74}, {-250, 150}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.5755396529559946`*^9, 3.586581222266629*^9, 
  3.586967224758176*^9, 3.58696923528941*^9, 3.5870926539288464`*^9, 
  3.587093284783541*^9, 3.5871248192857847`*^9, 3.5885695621375885`*^9, 
  3.588579717736709*^9, 3.588579787649525*^9, 3.5890766291163125`*^9, 
  3.589076664295779*^9, 3.5890767820446076`*^9, 3.589076846050733*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Measurement data", "Subsection",
 CellChangeTimes->{{3.505408151582389*^9, 3.505408153261485*^9}, {
  3.5054086348680315`*^9, 3.505408635741081*^9}}],

Cell["Pick up Stanford data", "Text",
 CellChangeTimes->{{3.5870926097335443`*^9, 3.5870926180055366`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"rawfig", "=", 
  RowBox[{
  "Import", "[", "\"\<data/shapes.stanforddata.jpg\>\"", "]"}]}]], "Input",
 CellChangeTimes->{{3.4304757059470124`*^9, 3.430475714457012*^9}, 
   3.4304780045760126`*^9, 3.4309104985826*^9, {3.5017425484207835`*^9, 
   3.501742556260232*^9}, {3.5017426131394854`*^9, 3.5017426319955635`*^9}, {
   3.5017427208606462`*^9, 3.501742728348075*^9}, {3.503374324976756*^9, 
   3.5033743255737906`*^9}, {3.5053737830461864`*^9, 
   3.5053737902055964`*^9}, {3.505407977961458*^9, 3.505407978903512*^9}, {
   3.505408376230238*^9, 3.5054083768372726`*^9}, {3.588579812077627*^9, 
   3.5885798153530455`*^9}, 3.5885800690332756`*^9}],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJzsvQnYTmXX/188KJlJCRmTISFlKJEhQxkiFIlQknmeZxGRyJCMmZKxjIWQ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    "], {{0, 319}, {508, 0}}, {0, 255},
    ColorFunction->RGBColor],
   BoxForm`ImageTag["Byte", ColorSpace -> "RGB", Interleaving -> True],
   Selectable->False],
  BaseStyle->"ImageGraphics",
  ImageSizeRaw->{508, 319},
  PlotRange->{{0, 508}, {0, 319}}]], "Output",
 CellChangeTimes->{3.5885800694753284`*^9, 3.589076629775412*^9, 
  3.5890766643417845`*^9, 3.5890767820976543`*^9, 3.5890768461046987`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Construct figure", "Subsection",
 CellChangeTimes->{{3.587093405143982*^9, 3.5870934078143034`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"height", ",", "width", ",", "depth"}], "}"}], "=", 
  RowBox[{"Dimensions", "[", 
   RowBox[{"ImageData", "[", "rawfig", "]"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.589076839663928*^9, 3.5890768420631905`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"319", ",", "508", ",", "3"}], "}"}]], "Output",
 CellChangeTimes->{
  3.589076782170656*^9, {3.589076842505249*^9, 3.589076846129735*^9}}]
}, Open  ]],

Cell["\<\
Determine actual dimensions from image\
\>", "Text",
 CellChangeTimes->{{3.587092889000054*^9, 3.5870929138620358`*^9}, {
  3.587093045556836*^9, 3.5870930842454786`*^9}, {3.587093238089939*^9, 
  3.587093239926158*^9}, {3.5885803459844522`*^9, 3.58858034823977*^9}, {
  3.5885810468645396`*^9, 3.5885810486237392`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xmin", ",", "ymin"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"xmax", ",", "ymax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"497", ",", "318"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"xsize", "=", 
   RowBox[{"xmax", "-", "xmin"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ysize", "=", 
   RowBox[{"ymax", "-", "ymin"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.4304760025250125`*^9, 3.4304760037820125`*^9}, {
  3.4304760556780124`*^9, 3.4304761160080123`*^9}, {3.4304761968750124`*^9, 
  3.4304762710170126`*^9}, {3.4304771975580125`*^9, 3.4304772006710124`*^9}, {
  3.4304772722110124`*^9, 3.4304772753350124`*^9}, {3.501743724663061*^9, 
  3.501743761221152*^9}, {3.5017440121255026`*^9, 3.5017440484835825`*^9}, {
  3.50174532055134*^9, 3.501745373588374*^9}, {3.5017454140196867`*^9, 
  3.501745425772359*^9}, {3.5885799226256733`*^9, 3.588579957681128*^9}, {
  3.5885802610176945`*^9, 3.58858026512118*^9}, {3.588580359337148*^9, 
  3.5885803641367946`*^9}}],

Cell["Construct desired image dimensions", "Text",
 CellChangeTimes->{{3.5870929265495586`*^9, 3.5870929330293355`*^9}, {
  3.5870930358616734`*^9, 3.587093040566238*^9}, {3.587093070743861*^9, 
  3.5870930754774265`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Xmin", ",", "Ymin"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"a", ",", 
     RowBox[{"-", "250"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"Xmax", ",", "Ymax"}], "}"}], "=", 
   RowBox[{"{", 
    RowBox[{"b", ",", "150"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Xsize", "=", 
   RowBox[{"Xmax", "-", "Xmin"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Ysize", "=", 
   RowBox[{"Ymax", "-", "Ymin"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.4304774225170126`*^9, 3.4304776108890123`*^9}, {
  3.4304894791530123`*^9, 3.4304894846170125`*^9}, {3.4304895837380123`*^9, 
  3.4304895863710127`*^9}, {3.4304896476450124`*^9, 3.4304896507570124`*^9}, {
  3.4304896930990124`*^9, 3.4304897028630123`*^9}, {3.4304897625830126`*^9, 
  3.4304897651970124`*^9}}],

Cell["\<\
Convert raster to desired image\
\>", "Text",
 CellChangeTimes->{{3.587093362742894*^9, 3.587093370804861*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"data", "=", 
   RowBox[{"Reverse", "[", 
    RowBox[{"ImageData", "[", "rawfig", "]"}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.5885808637782288`*^9, 3.5885808686328907`*^9}, {
  3.5885809172410717`*^9, 3.588580921080508*^9}, {3.5885810129772644`*^9, 
  3.588581017968853*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"raster", "=", 
   RowBox[{"Raster", "[", 
    RowBox[{"data", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Xmin", "-", 
          RowBox[{"Xsize", 
           FractionBox["xmin", "xsize"]}]}], ",", 
         RowBox[{"Ymin", "-", 
          RowBox[{"Ysize", " ", 
           FractionBox["ymin", "ysize"]}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Xmax", "+", 
          RowBox[{"Xsize", " ", 
           FractionBox[
            RowBox[{"width", "-", "xmax"}], "xsize"]}]}], ",", 
         RowBox[{"Ymax", "+", 
          RowBox[{"Ysize", " ", 
           FractionBox[
            RowBox[{"height", "-", "ymax"}], "ysize"]}]}]}], "}"}]}], "}"}]}],
     "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4304763742890124`*^9, 3.4304763923410125`*^9}, 
   3.4304764263220124`*^9, {3.4304764851660123`*^9, 3.4304765210320125`*^9}, {
   3.4304767848190126`*^9, 3.4304768289610124`*^9}, {3.4304768622040124`*^9, 
   3.4304768910840125`*^9}, {3.4304776203660126`*^9, 
   3.4304776701440125`*^9}, {3.4304778593970127`*^9, 3.430477861615012*^9}, {
   3.4304779120620127`*^9, 3.4304779278880124`*^9}, {3.588580226067219*^9, 
   3.5885802511224017`*^9}, {3.588580479983474*^9, 3.588580482346795*^9}, {
   3.588580601658983*^9, 3.5885806023380547`*^9}, 3.588580654943699*^9, {
   3.5885807163865466`*^9, 3.588580727386945*^9}, {3.5885809603545227`*^9, 
   3.588580960902567*^9}, {3.588580992922635*^9, 3.588580993712762*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figb", "=", 
  RowBox[{"Graphics", "[", 
   RowBox[{"raster", ",", 
    RowBox[{"AspectRatio", "\[Rule]", 
     RowBox[{"1", "/", "GoldenRatio"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4304769009820127`*^9, 3.4304769049890127`*^9}, {
   3.4304769417930126`*^9, 3.4304770043880124`*^9}, {3.5017431884373903`*^9, 
   3.501743225587515*^9}, {3.501743260269499*^9, 3.5017433127475004`*^9}, {
   3.5017443778924236`*^9, 3.501744395108408*^9}, 3.5017444374598303`*^9, {
   3.5885807325845947`*^9, 3.5885807327535844`*^9}, {3.588580983145419*^9, 
   3.5885810258698215`*^9}}],

Cell[BoxData[
 GraphicsBox[RasterBox[CompressedData["
1:eJzs3WevZleV6Pt9db8A2cRunW/QfZqmoQn7K5AzfaQr3dcXOgDdhMNLXjqU
7bJdBoQQQqiFnGOVAeMmGduVc67aee9qwAHnvnfOqdY52uP+d9dwzVVrPVX/
IaEfUj1eez1zzRWeMcea87/9X//PR//v/3Nubm7+/5ibe/n/+1/5/4/sKrEx
v9k/r/Hvwbkavw+ePlXi34P0+e99t8S/B+nztD//s0b0f9SItrh4afu0/yRt
h74v7U+2ncmf/bTEvwezn6f9zLY/HV/aDrXzf7xa4kLwz2pEs/2TpPb56SMl
LgSzxyt73Gl/yF79irZD/Z+Oe/Z8z34v6icfqhGlv/t3NS4EaX9o+7Q/2fbM
Hq/s9TNrdvt03Kl9st83e92j/c/uD20na/a6mj0u2fbpdZ/qdf357ndKXAjS
9nv186zZ55Ds8xhJ+zP0dno9Lw19XKidT50scSFI35e2Q/2QzruTJ0psBIc+
j75Z40Iw+/yT/bt0/lJ/6HXd63WeUvv0Oi69pP2k70XbyT6/0X2TpO30Ou6v
vlJiI5h9nvxCjY0gtRs9B9Lfzfbb1/p8cbEOfb5Qf+h1naG/26s9e7VPdn+y
12faDvXz7PWNtp89Xtn2yZo9vtnnmV7t1qv/D3286Hkp+xxLxyt7Xc1+X3Lo
/pbN22TzqL3Ou2w/yT5v9LLX8cr2w6zZ3wXZ7dDxGqudaTt03aDfBbQden7o
lY/Nni/0+Wz7ZNs/e32j7dN1u9f9qFd+I3teDH19GFrq59njTu2ffU7O5kl6
HUf6XtQ+tP/0d7N5KrLXcaftZ58f6Ptmr5PZ6wmZvQ9m2y27P72uw+RYeSFq
z+x1INvPs+1J23f8vZnth/R5Ou60Hcfftz7ujr9v/b0cf9+6n2evq/T5rHR9
yLZPr+te9vyl/Rn6vpNtz+zfzeZ/svUJveorsteN7PNSL7PHpdfvU9qfobfj
+HvT8fetdfz98thr/P2/up9fqtnzlO4jdP5SP6fvS88DtJ3P11gPZo9X9rj0
ktot+1x9/FiJteA3amwEP1gjmt3P7P00Wwcy9HWG+hu9N0H9kPbnXTU2gvR7
mdoh+7um1/lL28m2J9Xh0Pei3330+e/cXmLjoqV+vmtnifUgHXfaHzq+ZLYd
su2TrYPKjtNlnwOzzyF0vE4cL7EepM/TeU39hNqNPk/9hD4/9PUza/a403ao
n9N9h9qNns/puGfPl+z9ItuetJ3sfY3OU+rPJLXDKy+XWA9Sf6Dt0HHpdf/q
ZbZOldqBrg90vlB/pr9Ln9/5cIm1IN1/qf1n/fqTbTdqB3pO/nqN9SCdv1Qn
TNc3+r50naT7CF1vqT/Q36V2oPak7QzdT6jfZq+ft+8osR6k6xv1Qzq+dFzo
upH93dHrPtXreNH+kNQO9Ps3+zxA/YE+T/2Z/u7Q7dPrOGbH4+h3K/XD7O9o
+rt03aP2H7o9s8cx+zso+zuCrifZ+wJd56mfZOeRGPq86CVd5+k+Ttcr6v8k
9efsfYG2n837UX+j/aF+Qr8jaDvZ31+9zP7+Iun7Zn9v0vUw257Z63Cv85fM
3o+y0nlE7ZD9fUrHkdo/a6/+n+3P9PlsXX32uYKen7P5WPq7dB+k/af2zPbn
XucLPX/S/tP9ndqZ+jPdF+g40v7TfTObP5wVs++t0PnYa3yNPk/9Nvs8n31e
oustPZ9kx01m/fmBjuPXaqwFaX/oPkjtnP29QI513mXztCQ9F2Xva73yCfS7
e6z8ba/nCtpPx9+37re0nWx7Ov6+tdl2yLaP4+/THH/PXq+y96le12c6r+k4
ZvshbT87PkJS+2fbbazjRX+X2pnuO3R831ljPUj7Sccre79z/N3x94vJPzj+
7vj7xVwn6fnhaht/zz6PZe9rQz8n9Hreo/sR7c/Q8x1lpeNO14e77ypxNviO
Go8Hr7+uxJ7gm2v8W/CW7SV+Hdx2Q4lfBOnzb60R3X5ziV9dtLSd7PZvvqnE
L4MfrrE72K4Tvwu2/vJU8PU19gRbu94bpP38SI09Qfq719T4VTDbzrQdktqf
2vlNNR4Ltuvlk8F233si2O4PTwXpeNH26bx4e40fB+nv0vaztutTlP4ufZ6k
v9u+32+DdHzpupHtb+SO20o8Hnxbjd8E31jj7uD+fTWC1D7U36hf3XZrid8G
2/3niSB9vtd1Mnv9zF4n31Ij+roau4N0vrfr4s+D2fPi1ltK/CZI10naz173
l17Hhe6n1P+pv1F/pvZs23swSM8VtB36PB3fXu059P2OzouWR/ldkPobXd/o
8/R3W/v9JJi9L2T7efZ5o9f26fpDz2/U/+n6QP2Wrtv0vNfOv13BG64v8WiQ
9p/6CfU3Or7Ur3o9R/V6/hz6dwFdV1t73BHM3t+pX2Wvz9T+dLzoe2XNtmcv
s/frdt7fF6TngfY8uC9I+0Pn49D3r+x1Mnv9pPsItTNdD+nz9HfpekjtSdc9
Ol/Gen7odV3KXk/oOk/3BWpnOi5Te34b+vpD11Xqh/T57HWM+jNJz+e0P7Sd
7HlN/TN73aD+M/Tx7SXdN7PPe9RPSLqv0XHJ9odsOwz9Oyt7/6L+tnWe9s4g
Hd/s9Tb7vEe/63s919H2STrudFwoX03Xhz27awTpeLV23RHc+vPxuGSfn3v1
/6zUz2+6scS/BbN5Hvp8tt/Sdanljx8K0vEa+3ltqONFn6fzjraTPR/p82+o
cVeQ+n/2OSebP6HfudTfrtTnB/q+7br1r8Hrri2xO5jNJ2TzPHQ9Gfq5vdfv
XMofZp8f6O/SOC/lE+g+RddVan/H3//35zrH3x1/f206/t50/P21XSd75XXp
fKf7F/XPdj79PEj9h54r6Lhn85N0XMZ6fugltT+dR9TOlJeg8yKbf6P9HOt3
U6/8j+PvWz9vDH39cfzd8fdL+f3b2mN2x9+z17Gx2nnr9v9ZcOv8XnxePXa0
xGqQ6t6z8xWMVZ+fXcf8zjtKnA5Se36yxmLwT8+ViH66xnLwn2usBF94vsRy
8MjhEhfvgw+UuHhb/4rS/rf+G/1KjaVguw8vBv+mxlKQ2vPokRrBv69xNtjO
y6Ug7Se1Q7b9aTvU/tnj+N4a0UMHSywG31Mj2q5zS0HaDh2v554tsRBs58fB
4P33lVi8aGl/qJ/Q9yJ7bYfOi/Y7bCVIx/czNVaCtB26bjz/pxJLwX17S5wN
Uh6Azl867vT5L9e4eD9VYylI52/2+pk9H3tdB+i6SveX9rx8JvjFGqeDdL7T
9ZA+T+1MxyV73xna7PGi6w99X7oOtOfQ48F2Pp0LZq8/dD0cuj9nzyPq/yRd
B6gftt+1i0Fqz7+qcS7Yfr8eC9L+0HHp9TzW6zjSdr5aI0r9is4Lup7QdSz7
XNHe1z8RbO9/nwzScaf9p+NL+0Nmvy/156H7T/Y8pX5CtueUI8Ebt5U4H6Tn
B2ofOo50PtLn6TgO3Z7ZdqbnBLLVGS0E/6HGuSA979Hxov386xoLQTouvdp/
avcvuh7S8zA9V7TxnzPBlr88Hsxet+m4ZH+3jnW+kNSvsr/jqN3o/tLykSeC
99xd4lyQjvvQz2lDS+1J/Z/Ol2z7Z/9uy4MeDdJ9nLZPn3/g/hJLQXreoPM3
mzcb63myl/Rc1+a/ORX8pxoLQWp/+r1Av3+z1+2h70dDX1epHai/0fNem2/m
XDD7fJ4933s9x5JDP9dR+5N0PaHfuVQvl+3/dFx6jV+MlX+jfkjX+ez1n9qT
xi/aOPaxIN0vxr7/X+7jSPlwOr70+Ww/obwr5dXp/KL7SPY5ga6H2XGTWX9+
aPVxC8E2vn00SO1AZtsze//Ktiddb3uNg2TvX9l+S9e9z9U4FqTnkOz4FJ13
jr83HX/f+jg6/u74u+Pvw42/93re6PX83+pDTwa/VONMMJv/ofM6m78d+vkq
a/b5IXu+0/3lszWOBg/sL3E+SP08Ww8w9DhF9jxy/P21XWeG/r1D54Xj746/
O/4+XPvQ9um5hZ4DWz14lH4XtHUYTgbb8+n2ILUnrZtAdfhTq8/P1u3TeHdr
j/i+Rvb70joL9L4A7Scdl6z0d7PvX9C6D9n2p8/TfrZ5Ss4G6XzJ9hNqB9r/
7Psg2fVKSOpX9L2y/TZ7vGi9mPa9TwXpe2X/7tSkdVLo89n+ll2vhNZzod+P
7T3Ck8Gh263XcR/6+pndPh1fOn/pPD18qMRSkJ5zsv2Q+km2/al/zoq9+vPH
a5wP0jrLvf5ur37e67kiex/sdX2jz7ff/WtBem+C9jPb/tn2yV5/ej2H9Dov
el1P7r2nxPngs8+UiGbvI9njS2avt2T2/tLrukftkF0P7r/XOBts79nHddlo
PcFs+2Tbmex1nvb6/ZVdv4+2c/xYibUg5UVpPURqt7GeH8b6XZy9bmT7LeV5
qN6713mR/b5DX3+y/b/XdYD6CX2e7lPvr7EWzPaTqT2/Df383Gv71K8oj9Tr
93v289n+lm23XtfVoc/rT9RYCN53b4korZ+bbf+stP1ZOU/J7H3kj38ocSY4
9O/cbP/vdX8Zuv/3uo/T71yqr+t1P+31+6VX/o22n/2+vZ5DqD/QfE0fq3Eu
ONZzwtBm258+nx3voONCv2voOby957cazP7OHfr5Kvt3Z+X5gfIM9L5Yr+eu
Xk4tf9vruYh86MESK8E2b/S5YHb72f10/L1J++n4e3Po+5Hj769Nx99fm2ON
vw+d76XjS8eFtkN5Caozp/bJPm9k+1uv+3iv57rs+FGv/vz0H0ucDe7aWWI9
2OvvTu35Ldsfel3f6POOv/dt/17HhXT8ve/9K9sfrrbx9+zfzeb3hj7vSHo/
t82v/8UgvVeSfc6h/el1fcj6Xz13bJbq82l9hOz+UD+h32vZ/c/2t17PsXRe
0/eieg/6/DdrXAjSPHVUv0rt395/vRCkz2fv19SetP3s9Tzbr7LtT+Oe1B9e
ebnEerC9f3Y4SO1Px53yUXSfou9F/Zak7Wd/39H1IdsPaX+y9xd6X4zG73pd
b+n7Uv+kz1N/6JUX7fXcRceR+g/1f6pPoPmC2nVxI0jfl57fqJ1p/3vdp3o9
32avw9nrAHnXnSXOBNt7/KtBaufsfvZqz7Han+47tJ1XXymxEaTjQvOf0Hxo
LT96IZj9vr3slbeh9qfrLfU32s4Ha2wEs/35ozXOBin/RucRbZ/6G/Uf2g5d
b+k6Se3Qa5yaPp89r7P5AZpvoc37uBZs/74RpP78oRoXgtlxTPpeQ+fxxho3
b79r14Ot3mdfkK57tJ/UH7L3tV7XyaHvU9nnHxo/pe3Q/Fd/WeNMkNqZniez
16VsO/e6jtF+0vmevW7T5+l6TtuhvCjVn1A7jJUHHvq8oOs27Sf1W3o+oesV
SfP/0P2I+lv2dxa1Q/Y8pf3pdZ72uh9l+wnNw0nrv1DeifIVdB3O5iWy1/9e
Dv07jtqB+gPlz6lei/pDNs9G2xmrPiQ7btsrD0/9nNZHbnWp+4LUz+l+R/vT
676Tbf9se2b7IV2f6f5Cx4XyFTTP6p7dJU4H6XlvrPxPL+k40v2O2pk+n623
pM/T8x69V5sdT6H7VPb3bHZ8odd9Z6znh5dfKrEWvO7aEqeDdN3L5ntpf+h4
Uf8c63m+13HP5mFofiF6Ps9en+l6mx237dVujr83HX93/N3x9ytv/H3o+1R2
+9RPsvcpel+M5gemfvKNGhtB+rt0XLLXsV7t3Ks+v9c4Ox3Hv6hxOrjz4RJr
Qdofytdl221os8eX+o/j71vr+Lvj7xdzHtHns+c17f+VOv4+1vgXHUeS9p9+
j9Dz3utr3Blsz30rQXoOpOd2+jx9L/p8L7P9oe1XXDe5jTP8LkjnBeXTsvuf
/R06dHtm95M+36s/0DoRtA4FbSd7fo3VbmNJz130eTovaJ0p2g5dP6fW/3tJ
9zX6PLUDbYfuU7ReTFtf5XCQ9ofOIzqOY513Y9lrHgZaN4rey8u+F0l5g7Hu
41Mze7xo/Uqqk5yV+0LWbN1Cdrw728/puffDNU4Hh34ve2rvI/faPv1ezm6H
5j2jdSqz26ffm9n2yfbDrFPrP3Q9bL97TgRPHC+xHqT2p7+bfS+PnkOy32vo
32W96sro+9L4XcsH7AlSnpCe37LtmW2Hoftzr/cpaPvZfk7zMt1wfYmzQdpO
9vfF1Ox1XmT/braug+a5/VyNteDY7Xqx7dzrvpOd7zG7n5SfpP5Pz3vZPGf2
vKb+mW3PWfm9nD1/af1reg89e32j+1p2/4c+X8Yy+7xE9fn/UmM1SNsZ6/fv
1PLt2fpD2s4HaqwH31RjfzCbx8t+r6mdF73+bvZ5mK5X9B4fzVc8dPtMzexz
3dC/O+6+q8TZIB3HbB41W5fVqx2mZvb5gfJO9F7z0O0z9PZn5TmE/i7VdVMe
Kfv+TvZ3x9SeB7L7SZ93/P21tdtYOv5+eXT8fdzx97HO9+zvmhu3lTgfpDwt
fS96PsnWGQ59XIZu/+z9KHu82jjGmSC1P+1nr7zlWDr+/traYazj0mv7jr87
/n4ljb9PrZ3p/kXtQ/NvzEFk1yOj9yyGbp9e7UntRuML7b70ZDD7PE/Pw73y
jUPnz7P5imx9S7bd7ryjxOngNTWWgtnv2+v3aa/rCbVn9rmFrid0X8v2qwcf
KLEc/M/3XTZL59HQ46Fj2eu4Z68bNK5H83h/ssbBYPY94ux44ljr/87Kc9en
aiwF2/jq+WB2/8caR+hV/9bL7LwQtJ0//qHEmSDVLdD3zY7/Zh36eZi2k70f
0f2CfofS36X3WGm9A7ruDX1cxjKb16XtZPNadHxpHXm67mWfz7P3nez2yaHv
X72un9n7O82DQc+H9HdpPRf6PO1Ptt9ObfyOrpP0falfUZ1wy6vtD9J1L3tc
Zv06OfT5Re3TxuWWg1+qcSaY/T2V/V0/tefkXseFxiNo+/SeC60DS/N59mr/
ods5a698LH2erofUPvQcQvP/0LzT2XxXdpyL2i27nVnpJ9Ru765xPviPNaLU
blSHP/Tv4uz5O/R7RrR9yh/S+UXzXLV2PRmk99BpXi/af6pT7dUPhz5fet1/
qZ9/vcZ6kOYLau+1HAjS7+he4+NjOXTeic7T7HMv1Tdef12JM8Gx23Uos88P
vcaVstunvASNw96+o8R6MNs+dH0mex2XWcnfUh0C/W7K1htk25l+p8/Kc1rW
7HE/eKDEQpDGeWk72fvm2O00lJQHyF736HcQrWN16GCJxWA2D0zHy/H3rfut
4+9bH9+xrgPZcai31zgapP3Pfi+6DmTXNejVntnr9tDPFfReM82T/+Ua0ezz
HrUDHa+pjVMMnd/IXm+p3puOb6/33WY9f0v7mZ3Xgtz6/eV4nxq7PS7VbN1y
9jmN8sPZcW2aX/3ZZ0qcD36ixkKQ8lH0vei8ztbtZ/OcvY7v1Mbfad2Qocff
s+s7DN0+2fVWsnXL2TwMnS9tXPfbQar3o/NoVu472fOR1j1s893uDWb7VfY5
dug6yezzUq/xrOz3on5I70VSf6a/S+dpr+vG0L8rs/df+rt0Pcn2W3rufe7Z
EgvBXu02NbPnXa9+RZ+n/k/jcbRub3Y/s3WM2faZ2u/T7H5mx/UeuL/EUpDq
gnqN/9L36nXfH+u5grbf6z5I+epsfUL299qstH/2d2v2dwr9XaqXozzbrM9f
kZWeB7L9MPt8Qtvfv6/EuWCv+vzs7wX6fK/ngaldV7P3C5rn9vixEmtB+rvZ
ukSq4xrrOa2X1M407knfl9onOz9ndr6FofMzQ9sr/5a9372vxkqQ5s+n62ev
68zU8j+9jkt2njrq/9tvLrEUzM5jM7V2zu5ndjvZ54pe+VI6Xrt2llgPZr9X
r3m3hp4XKNvOvY47Xffoee+Zp0tEqf/0av9s/+x1Xe11f6Tt9Ho+p3kFaTw3
WxeUzUtkHSvP0KsOnH7/0nWPnvey76n1ap+hzbZ/9vN0XmT7+ZHDJZaDNN4x
68eF7FUnk+0P2f2k47Xz4RJrwV7vGVF/I3vNl5Vtz7Guq3Q9fEuNxWB2vDL7
XJF9fph1s/dryrt+pMaZIG0nO8/hlXr97DWeS+MUNF/0ww+VWA1ObZw0uz/U
bo6/T1P6XlN7z5qeH7L1+XR/yeZJhv69n3Xo/aHt03voNJ8hmc0rZvvh1PLn
vc4L2n72OZ/e+6Pjkn0OHPp6Ndbxos9nx9nJF18osRJsf/9UkPrP1Po/mb0O
Z7efnc+QtvONGhtBmjeAjmO2rp7ua73q6un+OPR52mv72Xaj8XeaL4X+bq9x
MdpP6v+Uh6T8Sfa9gKGvw3S+33xTicVgy+fdGfxqjeVgdn1VOi5jjSvRdWnr
/nAy2MYTngxm+wPN5/COGmtBqmOh+XCo/2fzFUOPB2W3T9fVvXtKnAnSOonZ
vHqvfjhWfiDbzr3GL2g/s+9hjf37fqjn7Wy+Zeh+Rf5tjdUgfT77/JltB/r8
rPw+ov2n+xRJ86n+dY2FYPY6nD0us/77qFe+jtqN6nyy7Zb9nTXrZo9X9j5O
z7eU3x67PS63dN3OtnO239L2aZ46quPK/q7P7idtf+j5TrPboevV0Otl0Lxb
2XxU9vdjr/UcpzY+SPuTrf+h7d92a4nVYPa4DJ2fH8te4yPZ/D/NM/PGGueC
9Bwy9DodUxu/y17fsu+b0N/9Wo21YPa5emrtPPT+9MrDZ+8XtB5or3miev3e
H/p+nf27Q98fs8979FyRfd6g75Wtxxt6fKfXcez1/gI972W3M+vtnN1+9jzq
dd7R8eo1/j41e7V/9ndxdpzllu0lloNUnz92u07FXvmx7HMyjSPTfb/X+yC0
/Wy+/WrL32bnB6D27LWe0VjP80Obfe79mxpLwey8Adnfv1dq+9P9KHtcqP9T
HcJY9R7k0OO2jr/PlkPn67LHi+rzb72lxJEgHV96/qHrwNDjibPyuyybN6D6
QMoXZZ/Txvo92MvseddL2h+axzt7XLLny9Se93rtTzYvStun9/hovaRe9Spj
tfPQ+0/boes/7Sf18+z8/Nnn7aHHB7P1n7M+/k7nV7a/ZfOxNP6e7W/Z75uV
+i2ti03zedL7DlTvvfX8+Q8E6b2/odtnaOk+S5//XI1jwdZOPwrS+ssfrxGl
91lovSQal6f1kqif0/nY67qXvf7Tduh6SOO2lLeh+tXs9xr690V2+3S8sutT
Z/ef7DWenv09Oytmj2O23ei8zt5/af+z87Jm31fKPufP+vra2fekyK/UWAr+
Y43zwbH659TMvj+S3X6vcW06XrSfs/57n6Tvmx0Hzz6fUP+fWh5gaOn7Zs+X
7O8v+vy2G0qcC1Id11j1+b36ydD5gV7bp+1k5wnsNY5Av6/p873yJNnrSa/2
z+blaDvZ34+95uuYFXudR9n7O61HSb9/ez3P9Gqfsdo/+7zUq66g1/yoU3Po
38XZ69XQ26HrXq/7AvXP7PVk6OfSsX5f0Dz5JLVDr/bM/r4bq91Ieg7J1hdl
80XZ/aT5u+jzvc73qd2/6DkqW7dA/bnX+xpTqw/p1f60naGv2++tsRz8Uo0z
wbHbdSr2eu7tdT0h6bjT+Z79fTr0782x7JW/pfbPXiez9fm9nk9mxew4wjU1
loLZdZx7zbczdvtdqnQ9yebZhs4TZp8zs3/X8ffXdlzGHj+/VHvV9w5dR0fv
Y1J9fvb7Zvttdv+HHgfsZba+KPt9s/tDv7uz6zJP7fmh13mavX5mf7fS/Y6c
9fqTXudpdnyHtv/+GmvBv6xxJji19umVB872/+zvO9oOnS+9nluG/p1L288e
l175nF7HPbt92k729ym1Q/a6R/Md0efp9wjVB9K6NrTOF9VL0/zDhw6WWAzS
/Lf/VCP6p+dKLAY/XON0cA6C3lun40vtQ+sgDJ3vyvY3+vzrapwOXndtid1B
uj7Q9ml+NpqvlebdHWt+oV51ONnjRf2Q8tWfqrEUzB6voa+f2e2Q2ftgr/sv
9cPs9TxbBz4rDv27Pnsc6e9m542k+2Z2nDdbn5/9XZB1ankGOq9pHdiP1TgX
7DVuOLV8S9Zev4No+zSfDH2+13VvrPYcej/puZGk40XXmez1k+pzppYf62W2
3oO2k617pOs81efTfKrZ+0V2P6kf9rp+9vp8r/MxW8+TPS/od0S2v2Xru3qN
D2afA3tdN7J1d9nfa3T9pO871u/9oc3m2XrlvSk/0/IQp4NDPydPrf2p/1M7
ZMcxaX96zas2tfdhxxpfm1r7DP2eZvZ99my7Te25NHu/oOc6Mvt7Idue2efD
qbU/7U82T5htz+zzYfb5rVc+c+jf19nrZzbPn13vmLaT/f0ytftXr/bPtk92
f+i5+oH7SywFr7b6/Gw793p+6FVHmv1dll2PrNf+j32cL9bsc2b2vtBrviA6
XtnPj93elyqdv3Rcbr6pxGLwj38ocSZI289eN2bl/pU1+/tl6Hotes4cq36p
VztTe2afq7N/1/H3re3VH4au0zt8qMRS8DM1jgSH7re9tj+1543s8xhth37/
Zr9v9vfa0MdraLPnb/Y6SXWJ9HmqV6S/Oyvt3yt/S9vPtgN9/qUXS6wGaVxj
6DzP1H430fWh1/ga/d7Jjh/RcaH9zI7L0N/tle/N9pPs58caf8+ep3TfyZ53
2Tqo7PaH/l1A+095aWo3ur8892yJhSDNn//2GqvB7PedmtnrzJ7dJU4H23Xr
ySCtm0PvcVCd+dD5Ger/veYZ6JV3oudkeu66684SZ4I0D0N2nCKbjx06z5Dd
Tzou2Xob+rt0/c+2czZvMytS+0wt70T3Efp8tv9k22fodiOHfr7KStdnWm/l
H2qcC2af0+h8nPU8dnb8Lvucn72/Z8+7WRnHyd4He9W50f5Qf87WP/RaH2HW
zV4Hsvl/el6ldakO7C8RzV73suOz2efqoc/fofNL1J693s/Ntj/Zax7sXser
V/02ma1Pzt6nsvfBXtf/bDsM7dB5XTpe228usRR8fY0zwV75yalJ7Z99fsve
F3rVT9J1Kbs/Y7Vztp9n829kr/bJ7ufQ9fm9+mf2+15q/+jdzjQ/wJtqnL9o
Kd9FzyfZ+85Y4whkr/Ej6m9kth3oeSM73jRUf32t7dlr+9RPstcBOr/od2v2
+jwr+Yde7U/byfZ/6uc0Dvj8n0osBalO9Up93su2c7b9s9vJPn/S+TX0fMvZ
6/nlPq6v1ezvx6Hze9nn4VmffzVrtv/TvIV/X+NskK572Xbudd7NutQOZPZ5
stfzs+PvTWpnx9+3brde7+vR57PjGrReEtXn03ay3yv7nDDWew297FWfT+dv
9r5D7UzPLb3qtcb6HTf0eU3j6XR9o89Prd16tXP2eTX7e4f2h86LY0dLrAZp
nuRe83dl22es497rOpzdfjYvnR3XGGvcc+jn/Fkff8/WaWS3Q5+n4/Wd20ts
BKlOnvLMtH3q57T/2XwpHa+31FgMtnGMHwRpvnGqi6Z2o3xjr3qJrHQ+0nFp
x/NkkObPn9r4F32v7PW217hPr/OapHWu31ZjJUjPCUPXU41Vn0PHN3v9zOb/
6fND/76YmvS9hn5uyZrNy2V/h2Y/n33e6NVuY7V/9vrw4AMlloO0zk72OTb7
HNLrPjjW72LafrbfDr1+xNDPUUO3M133hr4v0+ez15mrbXyNzN7He9Vd0/Xt
kzUWg9nrUnYdmWz/HCsvNPRzbPZ+kf2dkt3PqdV7j3Vd7VW3TGa/L+3P0Hmz
oY9jr/f+6PM0fwKtJzJ0nnAss3WMJF2vaPvUP7Prh9K4+dTGfXr1c2r/XuPd
2XbLno+92iH7nmmvcbSxfjeRdB7R8+HBAyUWgvfdWyKarYPNPrcMXdeR3c7Q
v7vpvMjuD/3Oyu5/9nj1aoeht59th+zvVjqO2Tzw1J6rx2r/7P2CPk/PA5+t
sRqk95Jm/bhknZXn2F7PA2M9h2S/79SeQ7J1bnR/yT43Zn+njN1PhzLbnrds
L7EczF73snnyK/X6OfTvCOrP9JzfK2/fqx16bd/x92lK32vofGP2Ok91qu+s
cTxIx5d+R2efE7L1APS9pvZcQd+31zhv9jylfpL9fTF0uw1tr/2h5w36nZVt
52zeeGrtmb0O9Pq9TNeNdj1bD/5ljTNBur7NynP1rPwuyD7XDV2nkX1uof3M
Pnf1ui4N/RxL16VsHr7X/T2bf5haXU32upe9HtJ76G+ocVfwyOESy8Gp3cez
0v2X+mHLc58MfrjG7iDd97PrV2a/V7b/Z7ef7Z+97uPZ8Zcv11gMfq3GWvBy
97//dOj8QPb+nr0O96qXzh7fsX/fX+78QPY4Zu+/2X6Vvb7Rdug9L9r/L9TY
CPbqz2Pd17K/x2k7lM+hdRLp+9LvnWx79uq3U6sry+a7su2Q/Xx2/b5e7d+r
PamfZ/Mnva6f2fkTel1XZ92x6vOpDv8rNZaC2e1n17PoVWc41rj50OOStP/U
H7L7kx3/pc9nrxtTs9d1iZ7rsvMS03U7m88Zu1032yvvnb0e3r6jxHqQ5mGg
7WR/f03N7HMXHS/K09J5RO2WvV/Myvo7vfo5bb9XPxx6XImk7dB1stf3petq
r/HcsaT2pN+ztJ4IbZ/q87PjAr3q9Hq1z9D3zV7rSdH1Nts+vX7/kkOfL72u
S/Q8kP279LyXHUebWn3C0O2fPd/pvKD9eeXlEuvBm28qsRgcu10vt9l8KZmt
qxz6ekv9gfYn+9yb/bvZz08tf5t9zqf9pOf27Htns/I81svsefovNVaDzz1b
YiFI25lavcdYUv/vtd49Hd+p5dMcf286/t4c+vqQfU6g3zVfrHE6SNsZet2x
qb2fnj3u2fpG2g6d79n97HX9zObBZv05JNsPs3lX2j7N3zW1+12v/O3QeVTK
y72xxrngrPTb7O+4Xvfl7Phddlw428971X9m682yv+N67X/2eA2dT6Djla1r
ou1nf9/R56l/Zu+z2fw8PYdnt0PtSfeLm24ssRBs64TcEWzrJq0Es/126PGp
rNn5bNs6ACeCO24r8XiQ/m6v95SHbh9VVVVVVVVVVVVVVVVVVVVVVVVVVVVV
bVId+P33lVgMtnr8a4Pvq7ES7PX+wljtY32+qqqqqqqqqqqqqqqqqqqqqqqq
qqqqql6M1udvrfX5qqqqqqqqqqqqqqqqqqqqqqqqqqqqqnoxWp+/tdbnq6qq
qqqqqqqqqqqqqqqqqqqqqqqqqurFaH3+1lqfr6qqqqqqqqqqqqqqqqqqqqqq
qqqqqqoXo/X5W2t9vqqqqqqqqqqqqqqqqqqqqqqqqqqqqqpejNbnb631+aqq
qqqqqqqqqqqqqqqqqqqqqqqqqqp6MVqfv7XW56uqqqqqqqqqqqqqqqqqqqqq
qqqqqqrqxWh9/tZS3fv/qBE9eaJG8MM1dgdpO7Q/2c9npePSIvrnNaK0n7T9
sY6vqqqqqqqqqqqqqqqqqqqqqqqqqqqqDifVY2frirN1zlTP3GtedNof2v5N
N5ZYCL6zxo7g4UMlloLf/U6JC0Gq2x/7+G+2zX+/EfxmjQvBP6txMrhnd40g
1bdT+/z0kRIXgrQd+nyv9qd+Rf2Z/q7z/6uqqqqqqqqqqqqqqqqqqqqqqqqq
ql55Zuvzx6orpnnmqf6Z6qXJt9RYBO8LvvhCiZUg7T+159Tm1c++l9Hq4k8F
t99c4lfB224tsRr8lxrRhx4ssRKk9qft7NpZYj1I749k2y27bsLQx1FVVVVV
VVVVVVVVVVVVVVVVVVVVVVUvv9n6/Ox84CTNM//5GuvBz9ZYDb6vxkrwyOES
y8G311gNfrTG2eDranwv+M81VoLUzlRvn3Xo/vDqKyU2gn9X40Kwtevh4Ftr
/Cj4TzUWLtp77ylxPnjjthLRz9RYCe64rcRakL5Xtj/T8XKefFVVVVVVVVVV
VVVVVVVVVVVVVVVV1avHbH0+1SHT52m++m/U2Ag+/FCJ1WCr714OfrnGYvC5
Z0ssBD9dYzl45x0lTgfnIG7ZXmJ5frPZevup1W9n9+e6a0ucDrb3GHYHqR6e
+sk3a1wInjxRYiP4rhrRP6txIUh/N9tu2fNlasddVVVVVVVVVVVVVVVVVVVV
VVVVVVVVL91sfX62rpg+3yL6oRoXgh+ssRG8fUeJ9SDNw/+d20tsBGk+9ja/
/o+DNE/72MfzUqX586lOvrXr8eA7ajwepONL+0PrNVD/6dUO2fOil2Mff1VV
VVVVVVVVVVVVVVVVVVVVVVVVVX3tZuuQe21navOH03z7VJ//4AMlloNUTz7W
9+olHa82b/2JYHvv4Ykg1flTP6E6fJqXPrv/dLxoXn0yuz+qqqqqqqqqqqqq
qqqqqqqqqqqqqqp65dmrrp4+36v+maTtk//xaokLwQP7S5wPvqnGtuDhQyWW
gqdOlrgQnJX6baqf/+53SkSpPv/660rsCWb3h4770O+PZPvtlfpehqqqqqqq
qqqqqqqqqqqqqqqqqqqqql68Y817n/272XrsrO+psRh8fY07g2+rsRKkOvxe
9eRD9wd6v4D2v82LfyK447YSjwfp7/aaD3/o9lFVVVVVVVVVVVVVVVVVVVVV
VVVVVVXVJtWB339ficXg62pcG3xfjZUgzaNO9e1Tm1ff+nxVVVVVVVVVVVVV
VVVVVVVVVVVVVVVVvRitz99a6/NVVVVVVVVVVVVVVVVVVVVVVVVVVVVV9WK0
Pn9rrc9XVVVVVVVVVVVVVVVVVVVVVVVVVVVV1YvR+vyttT5fVVVVVVVVVVVV
VVVVVVVVVVVVVVVVVS9G6/O31vp8VVVVVVVVVVVVVVVVVVVVVVVVVVVVVb0Y
rc/fWuvzVVVVVVVVVVVVVVVVVVVVVVVVVVVVVfVitD5/a63PV1VVVVVVVVVV
VVVVVVVVVVVVVVVVVdWL0fr8rW3x70H6Xn9W42Twu98p8SR4IUjt8+c1orSf
Q7cztUOv9qf3DmbFsfqtqqqqqqqqqqqqquqV5NB5eNo+jbPQuMz/qBGlv3v7
jhLrwf94tcSFIO3Pq6+U2AiOfdxUVVVVVVVVVXVaUv6K8lHZvBzlr7L15LMu
tc/BAyUWgm+t8aPgp2ssB+l40fHttf9D98OtP38q2Orfnwp+s8aF4NB5Zurn
VP+f3R867tn1CMaur7c+X1VVVVVVVVVVVVV1fKdWn58d18iOc/Ual5navFiq
qqqqqqqqqnp5pHxRdjuU18rWgVO+i/JXY9f/XqpUj/2eGovBt9d4KHjbrSVW
g2P3r8vtjttKHAteU+P+4FtqLAYP7C9xPrh/X4lzwY/XOB/8pxoLwZtuLHHx
frLGYvDLNaKfqLEQfG+N5WD2PYVZcez+qKqqqqqqqqqqqqp6JTi1+nyaj+jv
akR/+kiJKI0/0vrLNO5pHb6qqqqqqqqqqv7v0jze2e1QnT/lu6ge+B011oI3
31RiMZitc56aVC995x0lTgfnIGj+fMpPUl6RjiO9RzB0/8zme99V40TwhutL
PBo8crjEcpCOy1dqLAXbOgbLwcOHSiwFX3i+xHKQjmN2+/R+B/3dbPvPikP3
T1VVVVVVVVVVVVXVq8Gp1efTdmj8kT6/8+ESa8Hs/tP42ljrU6uqqqqqqqqq
6rj2qrvuVef/+Rrrwe03l1gKPnB/idmV6qU/UuNM8HU1vhV8W42V4KzXM2fz
me17nAq29tsT7NVvyWw70/5k512hv5vNS49dX299vqqqqqqqqqqqqqrq+M5K
ff6J4yXWg4/sKrERfGeN48Gv1VgL0jgOzaufHZdRVVVVVVVVVdUrQ6p/zubZ
6POUjyKzea2x2+9Spfnt77u3xELwTTXuCR49UmIlmD1eY7fHZmn9BXqv5OSJ
GsE/q/FkkPKi1N8+VONCsG0vSvufPV9aRLPtSduh+v+x6+utz1dVVVVVVVVV
VVVVHd9Zqc+ncR8aD23rFR8O3rK9xHKQxonGPj6qqqqqqqqqqjobUr0uSfkx
qjemuvRsHXK2/n9q0vc6eKDEQrDV4/8g2PKFy0HKQ2br3ilvOXQ/pP2k/fnO
7SVOBNv3eCrYq+59aOn8ovOI2i27XsDY9fXW56uqqqqqqqqqqqqqju/U6vNp
vDI7X/3Oh0usBenzvcZPxz6eqqqqqqqqqqo6rJQXovrnseq0ybHrf4eqH35L
jcVgm5f9J8Ejh0ssBykfONbxGtpWn34ySPPn/12NC8Fsvje7n/QeRK/6+V7t
Ofb5YX2+qqqqqqqqqqqqqur4Tq0+/9ZbSqwEn/5jibPBj9aIZufXonElcuzj
pqqqqqqqqqqq49hr3gbaDtUhZ+dvp7pl+ruzIuUbn3u2xEKQ6vNpnU2aX52k
9ieH7p+0n9SeNH/+jttKPB6kv5utw7c+XFWnNj5F2/nud0pcCH6jxkaQ/m52
HWe6rtJ7f9n3lcjsuja0rkr2fjRUP1NVVVVVVVVV1WmYzQf2yu+RlH+jfGAb
X7p4v1BjI0h/N1v3/uorJTaClK/76xoLweuuLbE7OAfxh9+XOD2/2a3Xcd4I
Uj6z17z6tC7AyRMlNoLZfkjjtpd6nqiqqqqqqqqq6tVhNt9F+cPsdrJ18l+t
sRx8V41dwZtvKrEYzOY/Ke861vGyPl9VZ8Wp1edTvX22rp6k7fcaD6Lr8CO7
SmwEaXyK7oPZ+93U7o+qqqqqqqqqqjqus1Kfn627pvwebZ/GcSiflp2X4+s1
1oP0+UMHSywG31zjQPCtNb4d/HSN5SD93V7z6lN7ZvsDHcdsP+w1j4qqqqqq
qqqqql4Z9qqrz64LkN3PV14usR78ZI3F4OtqfCt4260lVoM0L8qs5NOsz1fV
WXFW6vOzf/fll0qsBenv9ppnnq7DdP/Kvo9Gdf40H9fxYyXWgmOtL6Oqqqqq
qqqqquPaKx/YazvZfB2Z3Z/P11gP7tpZIpr9XrSdY0dLrAY/U+NIsNXFnwzu
21sj2MaXngh+tsbRII3H0fd6e43VYGu/48E31TgfpONO70f0eo9DVVVVVVVV
VVWvTqm+rtd8JpS/ovUuaTs0f8Xf1FgKzkG8s8b6/GZpnmRqn2w7DH0crc9X
1VlxavX52Xnyr6mxFKTravY9rw/W2AjSfTM7nvX+GmtBmleftjPr77Wpqqqq
qqqqqurlcVbq82l+iWw9dq953T9XYy14/XUlzgSz2399jTPBt9X4TbDNd3U0
OAdxw/UlHp3fLNXDU7vRvPrXXVvidPA9NRaD1A4tj7sRpM9nxw1VVVVVVVVV
VfXqNFtfTfWB2e1QXR/V4dN2nnu2xEKwzYexLXjrLSVWgtn5T6Y2D7D1+ao6
K06tPp/G1+h+9+4a54OHD9UI0ngT7T+9L0Dfi6Tvdcv2EoeD7b5xMrj95hJL
wS/VOBP8QI314Nj9TlVVVVVVVVVVx3Ho+vxe801l152kcaKdD5dYC9K8HFQf
/lc1zgXfWOMXwTfXWAjS/tP61G1eq38NtvG03wTfWmN78A01vh/8hxrngtn+
sGd3idPBNq//2SCNP2bHJbPzvaiqqqqqqqqq6tUp1VdTfpLmxc3WaWfXhaR1
P1u+8GCw5Qt3BFtecDmYne83O1/x0Fqfr6qz4tTq8+l6TtfVE8dLrAfbPFIr
wey4Hpmdl4n2n9aVfl2N08H77i2xEPx4jf3B5/9UYilI81xd7v6nqqqqqqqq
qqqXV8rLZbfTqz6fpPGOb9TYCGbHibLjULQOJq0TTds5eKDEQrDNw3Eo2Or9
7w628bRvB9v8H78Otvzi94LtvYF7g7QuQBsPPB58R42Hgwf21wi28a61YLaf
7NpZYj2Y3Y6qqqqqqqqqql7ZUt1gtp6Q8qs0/zBt55s1LgTbvBpngy2fd21w
DuLFF0qszG82Ox9+ti59aK3PV9VZcWr1+TS/fa95kOg+SPcd2s9HdpXYCNJ+
0n38lZdLrAez3+u9NZaDdB8fqj+pqqqqqqqqquq0Hbo+f+jtv6vGRjC7LjPl
6555usS54A3Xl3g0+Bc1Tgdp3KfV0T8QpHny5yDaOtU/mN8szVd/5HCJ5eDH
auwL0t+lccC31LgvSPPqUx44+94HzaOSff9CVVVVVVVVVVWvDLP1kNm6eqrT
zuaj3lNjMfjRGnuDbT6PXwVbfu6LwePHSqwFaf1Qajf6vr3ywFmtz1fVWXFq
9fm0Haozp3WoaVyG7i+9vi/d19q8+KtBmsee7vs0H1R2Pn/Hp1RVVVVVVVVV
r05nvT6f1n2m7Tz8UInVIH3+8KESS8E2n8dTwTYOcyo4B/H6GnfOb5a23+r0
dwZbXf+vg/R3W/3++fnN0nrZLa95LNjm67orSH+3rQ9wYH6zR4+UWAlSHjg7
r1evdVRVVVVVVVVVVXW2zNbb95r3I7t9Wk/z5ptK/DJ4z90lzgWvqXF/cOfD
JdaCX6+xHqT9p3zdWPWHVCdJ7dzmdzkBPhHsNR9ytp6T8uq0PzS/NH2ett+r
n6teDWbHv7LjaNnt0+ez4yl0/cxe56k+n/aH2oe2Q+8LvK3Gb4Lt/rkYpPmj
2rxTi8Hs96XP03te2Xmrhn5vzuu/qqqqqqqqquq49pr3o9d60OSrr5TYCNLn
31xjIdjq6H8epPGyt9f4cbDVxd8T/Jcaq8FnnylxPjgH0fKIP5nfbKvfPxyk
djhxvMR6sM27fzhI+9PyrD+d32wbJ/xVsOVNHwzesr3EcnDo/qmqqqqqqqqq
qls7K/MDnzxRYiNIdW5Ur0jbp/05drRGsM1z/73g7TtKrAdbnmwl2P67bwXp
89n557Pzxk/NP/y+xOng3j01glTfTvWKlMemeZupvjRbT5vVfKnqpdur/rlX
fX72/Zpsvf13v1PiQjA7fkfz27fr7pkgvd9E+7PthhLngrQ//1jjfHAO4o01
7p7fbBt/OxnMtk+v6zO1W7beftbv+6qqqqqqqqqqs+5Y9flbzze1FKRxjS/U
2Ai2uvszYJz3vs2fcV+w1eP/NtjGx64NtvcADgRv3FbisWCr6/9BsOUdfxE8
dLDEYpDmCaHj9dUay0Gab/8dNR4OzkHQ+ws33VhiIUj55LHW11ZVVVVVVVVV
vdqcWn1+tj6N1uuk+rT311gLPvRgiZVgy289Fmx5wd1B2j7lx9p6mzuCO24r
sRbsVf85Vr13tm6zzfNxOEjt9lc1zgXvvqvE2eC+vSWirX7zXJDWb33pxRKr
Qarzp+NI+VLnQ1adzv0uu55Fdvt0/6LrCb2/Rtv/dI3lYLuunAq2dZuPB2ke
J6rbp/en6PN0/W/vqx0J0rjkC8+XWA7SdZ6eK+j+lV1vJSvdF4aeh19VVVVV
VVVVVV+bY9Xn79pZYj1I26d8XRtvWQfjuEzLq/0iSPNT3XZrid8Gqa6+1fUf
DH6ixoHgDdeX2Bucg3hPjcX5zVL+7eiREivBtj7niWCbx+t3Qdqf9n7DnvnN
0nsWtE5or/WmVVVVVVVVVVX1tTm1+nyaj6LXep1t/om1YKt/Pxacg2j/3X/O
b/G//JsaS8E2T8nJIG3/4YdKrM5vlvJplCec2jy62TpGmj+f8pOUf6Z1DXY+
XGItSP2k1aWuByn/SfPPZOffpuNIdfvW86sO59D1+bT97PWErhu0DjXNJ9/m
kdoXnIOg99So7p3GrWj7tF72/feVWAz+bY3V4DdqbARpvezs/YueWz5QYz1I
9ymS1tGe2n1fVVVVVVVVVfVqc6z6fMq/0fbfVWMj+LEa54Ktvv6O4Ftr/Cj4
hhqPBucgqJ6f5reneZ/a318O0nz1NE9Xe29gNUjtf/BAjSB9X4q2X4/Pb5bm
56fjS+tBmz9UVVVVVVVVVb08Tq0+n7bzyK4SG0H6POXlXnm5xHqw1SE+Ejx2
tEaQ5kt/+aUSa8Gn/1jibJDmG2nrea4G6XtRe9Lnx5o/n/LD9N4BzdtM86tQ
veXQ30tVrx6HXpck+z5OdjtUr077v/V43BeDtB4N1ZnTPPltez8LUj0/jaO1
cb+9wTbP1mKwtdNTwXvvKXE+SHXytK4BPYdk+4n3O1VVVVVVVVXVaTpWfT7N
7/GpGkvBZ58pcT64f1+NINXhP/N0iXNBqm9veb7TQfq7bR3jQ0HKy1H7HNhf
4nyQ5u/6UI0LQTpeLR+5P9jynb8O0vxUbXzyJ8H2/sSu4IsvlFgJvvpKiY3g
2OeHqqqqqqqqqurV4qzU51P9dpuHPkrbaXmv5eAcBM0DTNunesiv1FgKtvzc
PUHaz2y7ZedLGVraHzqO7X2Ck0GaPz87vzHlV+k9C6q3/LsaF4KUD6c6W8ob
U/s4z4nqpd+PprYduo7R9Z/eX6P67bae89kgjcft3VPiTLC9N3Y0+E81FoJ0
vXp3jfPBNg72eDA7D1Vr11Pzm6Xr/5dqnAnSutXZ4073kf9Z49+DdN+0Pl9V
VVVVVVVVdZqOVZ9P80Jcf12Jnwdp3UxaD7qNw9wZpPnqad6PVl+/FLymRnQO
4qYbSyzMb5bah/Jy2eNy5HCJ5SCt+0nzg22/ucRSsI0jPRWkdqDxx179WVVV
VVVVVVVVX5tTq89veakLQdr+F2psBOnzbR3IY8GW3/ppkN4LoHk5qE7y/vtK
LAZbnvLbwU/WWAxS/R5936nV51O9Os3z39YTOB6k+fOz6yxkpfbvdR5R3T7V
/1O7ZfuJ6tVg9v2moaXzNPveGV3nWz37WrCN8/wq2Orrfxykz/9zjZUgfa/2
/tn54HXXltgdnIOg99ro85+rcWx+sx+pcSZ4/FiNIM1n9eUai8Fe8+RTe/p+
lqqqqqqqqqrqNB2rPp/GC9q40wPBfXtrBGn+dpo/v61X+f0g5RvpvQBa9/Oz
NY4Gv1ZjLUjtSe1D0nwdlK+jefip3dp41++C7T2DfwtSXvSPfyhxJkj5w6nl
yVVVVVVVVVVVr1SnVp+fnf/8gzU2gjT/RssLPhqk/FX2+1K+i+b9aPmzbcH3
1lgOUt6P5v2gOs+p1e1TP2n1+6eC37m9xBNBel+D8tVUD0/vEWSPe6/6SWqf
oc9f1SvJqdUz0/WH3guj+2P2e9H88zS+8+Ya/xakeZnaeN2TwbZO9bVBWg+F
1rOm7cxBtH/fPb9Z+jx9X2ofel+s7e+5IN2vaT787Lxe1K/G6ueqqqqqqqqq
qlebY9XnU16R5kGied23nu/9UJDmoaJ8WsvPnQ7S+Fd2XVFqH/p8dlwyOw5I
7yO07/3F4Btr3B2cg3hXjSfmN0t5RevzVVVVVVVVVVUvj1Orz6ftZPNI7d9P
BGmekK/XWA/S/tA8//R9aZ1Nyr+9+EKJlSDl/eg9hbH712bpfQE6jq3u/kSw
zTvyeJD+brZ+1bp31SvP7HsuY90fh77Ov7vG+WCrKz8abPNaPRSkcSK6PlO9
OtWT0/zzbZ77PcE5iLZOzsPzm6XP37itxGPzm23fb1/wvntLLATb31kLtvHP
nUFqH3o+offRevXzqdnrvFNVVVVVVVXV6Ti18Sn6/PtrrAXp71K+q9VNbwRp
O208ZCNIn6d54OnzVIff6srvD7b6+seCbX6LA0FaVzp7vGheC9pOtp6/13Gk
v0vzZbV1RE8H9+6pEWzjh98KvqXGD4MPP1RiNUjvQWT7Z69+m10POvt5VVVV
VVVVVdVezvq8B5Rno/krts43xmj5vFPzm6X9oTp8+jzVVVIekuYBfl+NlWB2
/pCpzRdtfb6qTs1Zqc+n6+dHa5wNfrrG4SBtp60nszc4B9HGeQ7Ob3bnwyXW
gtQO/1jjfLDV3Z8J0ngfrbtN8+E37w22evkdwZtvKvHL4CdqLATp+9K6AzT+
dehgicUgrX9N/Wfs+nrr81VVVVVVVVV1s1Orz89uh+Zp/1CNKG0/uz+0/u/f
1bgQfPCBEsvBOQiaV4rydTRPBa2PTPPMU94yu14k5cey6ybTeFx2nVP6XrSd
L9Y4HaR5+Ok9i+PHagRpPpDs+CNJ35famY5vr/NXVVVVVVVVVVWb2XnpP1lj
Mdjm7fhBkD5PebNsXbf1+Vtrfb6qajNbn0/baeuxHAjOQbz0YonV+c229WR+
HaTt0N/9QI314O07SkTfVuPB4PXXlfh5sM1nfz7Y5pE6E2zvK+wN0v2Xvi+N
Zz3/pxJLwb+qcS5Ix5HGB3ftLLEepONI83GNXV9vfb6qqqqqqqqqbnZq9fk0
HxTVDx87WiO4/eYSh4K96vnJlq9aC362xtFgm5/jriCtQ015tjb+dh6M8+1T
XpHyYHQc6bhQO1Pelerks3XglJejefspX9rmBTkYpPcmWrtuC76+xp1Bmref
vhfNh0/fN5vfpuPoPPmqqqqqqqqqerU5dJ0V5c0oP0Pz6Lb6vt8Ev1BjI0h5
pGw7WJ+/tdbnq6o26bpH1/mHHiyxEqR51LeeTz7OGz8H0eZh+sX8Ztu43HKQ
vi/df1t9/WPBNh//L4N0H/lmjejhQzWCNP5I63e38dgngwcP1AjSPPz795U4
F/zbGqtB+l7Z+b7Grq+3Pl9VVVVVVVVVNzu1+nwan6Ltt3nijwWPHqkRpO3Q
+Bflweg9Apo3g/KEJM2b8fYaPw7SPBVtvOt0sI3XHQl+sMZGkI4jjetRvT0d
31dfKbERzOZvs/k6+jz9XcqjUp5zDqKt2/nE/GZpP+m4UDtnz8de86epqqqq
qqqqqk7NWambonk/KO/U1u08GcyuQ5ptN+vzt9b6fFXVZnZ8p123TgXpfbTb
bi1xNDgH8eEau+c32+ZtOh6k/T95osRGkMYH2zz3e4LtfnhH8OM1zgdpHLB9
798Gb7i+xN5ge2/gRJDajd53oPms2nPK94M0LxY9/9C64dTfxq6vn/pzpqqq
qqqqqqpefqdWn0/jCFQ/3+qaN4JtPOpCkP4ujZvQ/tN8DrR+9BwEza9On6d5
2l98ocRKkOrPb72lxEqQ2oek40L9gdqN6vNpOzQfPs0zT+sj0Hwm1B9oPhOS
jm+bZ+VHwa/VWAtSf86ej9l6fjru5g9VVVVVVVVVdWpm843ZdRuH9p9qLASp
zu3I4RLLwTb/x4Ug/d3svPrW52+t9fmqqs3suNI/1jgf/HSNw0Fab7rNa/Xb
YPvvloP/UmM1SN+L9p/Gue67t8RCsI3LHQl+qsZS8Ks1loPt/nAs2J4Xzgbp
e7XnhUeCcxA0/tXGRQ8G23sJZ4LZcUb6/Nj19dbnq6qqqqqqqupmp1afT/XD
NH94dn9OHC+xHsyOQ9Hn2/jJE8FWR78/+KfnSiwGaV2APbtrBNt8GKvB7Pgj
HS8a18vOV0/rDmTrxrPjjFTfTt/3SzXOBGm+kVZffzj4kRp7gm2dz38N3nlH
idPBbP/PrgeRPX/NH6qqqqqqqqrqWPaaD2To/Ab9XcqDUX0dzYtLeZ427+96
kPaT5gHOfi/r85vW56uqNrPzcdF29u8rcS5I9eHt7zwVbOMw60H6u9nxxOw6
xZ+vsR7MtnMbbzwf/ESNA8E2//1CsNXv7w1SO7f7+beCNA/Vrp0l1oPHjpZY
DbZ1ElaC1A5j19dbn6+qqqqqqqqqm51afX523ioa78jWIZOUJ2z118vBOYj3
1Fic3yz9Xapj/1yNtSBth+rnKR9I61zTvPS0fRpHy9bzU3+g/aR59bN5zqNH
SqwEX3i+xHKwzV9yKNjylHuC1E9oHn7KZ9JxzB53Ol5k9jxSVVVVVVVVVR3a
bB6y19/N5me+UWMj2OafeDLY1ns8EaTvS/P30uedP7+v1uerqjaz9fl0Xfpi
jdPBOYg2rvLD+c22caHjQXpPjcah6D07+nz2PThqH3p+oPW1r7+uxJ7gNTXu
D950Y4mF4NdrrAep/Q/srzG/WXoeaPfJk8Fsvxq7vt76fFVVVVVVVVXd7NTq
87PzhPdan5rmbaDPt3UyzwZpXvTP1lgN0vqV99xd4lwwWy+dbU86Xtm/S3nI
7HsZr7xcYj34zRoXgvR3aT/buqarwez3pXU5Dx8qsRRs71s8GaT5Ro4fK7EW
vNTz8786XtTO5g9VVVVVVVVVVZtUp0fzb9D8vdddW+JnwXfXOB/M1tVn82ak
9flbm81L/0WN00FaR5XWCc3uZ3b+apL6Ia1/mq3LNQ+pV5LZ8TI6X+j9r+z5
Qvev7HrHNN9Rtn1oPOvGbSUeC9L7TW3e90eDNM9Sr+ObnTeMPk/tQOM11H/+
tsZqsF2PfxJs8+7/ILjz4RJrwTbv/r8F5yBu2V5ieX6zb6xxd/DLNRaD1P97
Pedk5xnr1R9UVVVVVVVVVbNSXqhXvTfVb9M8Udnt07zxLV/1SLCtL300SHmz
ltc5FaT1Jd9VYyNI34vyP73eayDHWh88u2449UPKe9M8J9lxwzvvKHE62MYn
7wjS+p4fqnEhmG3P7P5nz19VVVVVVVVVnb5j1cdSniFbf05SfozydfR5yhdl
8yStrnB/MFuPna0fo+1Q/oeOO7VDmx/3fLCtO/rt4AP3l1gK9qqLm1pdN+Ut
23sbJ4MtD/1EkOZPpvdB6PM0H8jLL5WI9novgKR+lV2PYKzjq3o5zV6fad1h
ut7S9YG2Q/uZrU/+WI19wVav/Ysgjd/RuOFnaqwEqT1PHC+xHqR2ozr2oZ9z
jhwusRykeezbffiBYKvHfyzYxql+Fmz/vj84B0H1+e+tEX3pxRKrQWpPOi7Z
+fbpPMqOq9J2Lvf1QlVVVVVVVVWvXLP1vVuPU8TxBfq7VG9P+Rmaz6fV168G
23/30+Df1zgbpO91370lFoLZ/E82zzP0eGt2nnz6u/R96fhm85m0bkJ2O9T+
9H1b/nJ3sOVBtwe33VDiXJD2h+r2e32vS70uqKqqqqqqqur0HKs+P5v/6VU3
lc1HUV6Ftn/7jhLrwe03l/hVkOb1zeZnaD8/V2MtSPOcfL3GepDyTpTnpDrA
F18osRKk70vfK1uPN/R5RP2B5g9p63keDbZ84feCN1xf4mzwSzXOXLRtPumz
wb+psRSkOkmq2+81j3ev+ZBVL+d9c6z9oesh1bFnx8vofKfxO7qe0/609WR2
BN9S44fBZ58pcT5I+5O9nmTfL8jer2mcNDsvE72PQOs+0/2a3sto/eKp4Dtq
PB6k9YDo+ecTNRaC1D+z7Un3l+xzI+1Pdr0JVVVVVVVVVdVe9pp/m9ZzpM/T
uBLlFenzNE7R8kwPB6men8bjWh7xYJDWbaT526kdsuNQs16fn/2+2fYkaf1W
mm+qHf/fBVu++V+DH66xO0jtSXlyynvT98qep6qqqqqqqqo6u1J+JpvP6fV3
KS9BdXfZOq7s/mTnD7mmxlKw1V8/FGzzBJ8LUjtQnuerNZaDf/xDiTPBNn//
+SDVdd9zd4lzwVbHdyw4B0HzGA/Vv8eW6jZbfvpksNUz7g5m6z+p31K++gs1
NoL0fgT1z+z89tl6yLHev1C9mPtF9v6YHR+h847qyal+ODufD9WBv77GmSBt
h+YjavfDHwfb+0nfCrbxk53Bf6hxLkj7k10fJPs8kK3f/nyN9WD2fQFaP4XW
D6Lt0Pjs4UMlloJ/+H2J08E5iLZO+K75zdK8+rSf2fbPjn9R+2fPI+9fqqqq
qqqqqtrLXvX5VI+dXU+TpPGslgd6ItjyhfuCtJ8030Vbp/p0kMbR6P0C+l40
npI9Xtn2HGtetXZ8NoK9+smDD5RYDrb5xo4F6Ti2+c+WgjTfyJtr3BukecZo
/2k8i/LJvdazUFVVVVVVVdXpm63P75VHyu4P1ZVRXo62Q5+nvGU2n3nnHSVO
B2ke4GNHS6wGs+1G9dvvr7EWpONI9WZU53bwQImF4Btr3B385xorQWpn+rv0
fbN17L2k/Bt9r7Zfp4I0XzHVx1L+eeh2oPxhth41+57O1OYt16vTqdXnU505
XQcoz0/3o7vuLHEm+LYaR4J0HWh19L8L3nRjiQPBe+8pcT747hpRev+I2v8v
a5wJtvGZtSBth+6bND9Yr/5Gx5Hu71Rn/rUaa8Hsfl5/XYmfB+l5oM2L9v0g
rSuUHZ/qtS453d+z6x1knydVVVVVVVVVVcle9fk0ftTyaseDX6mxFKS8Tcv/
rQfbPBx3BGn9RxrXo7xodl4vmu+CPk/fNzvemj3uY9Xn0/gsvX+R3U/qh23+
qtNBGmekeT9ovheaH+bmm0ocDNK8/fR96b2Psda1V1VVVVVVVdXpOLX8AOV/
snX1tK5idl5Tqj9sdWf7grfeUuI3Qdo+5euy899SXjGbp6X9+USNheCbamwL
Uv6W6two7zr0eyJZKQ9M86i0es9jQVpPM7s/VBfa5hdZC2br/6keNbv+adax
jq/qxdwfx9ofqrvOvr9D663Qe0O0XgzNr07jbk//scTZIH3f7PxCtA5ye5/g
N8F2vzofzM5jT2bn4xra7P30hedLLAd33Fbi8eBba/wouPX6CHG96ef/VGIp
mL1f9Jr3Pnscva+pqqqqqqqqai971efTPOH799UI0rrMNA713LMlFoKUJ/xA
jfUg5TNp/nbKw1A9OY2PULv1mqeo13HProdO7UPjO9n93LWzxHow2z9pPhbK
AzdPBucgWt73nvnNtvzlt4Nt3rXFILVzdt3bbDurqqqqqqqqqvYyW5dO+SWq
z8/uD9X7tfkbfhb8fI3jQdo+1XtTnpDyP/R9s/XYlO+i+Uxaneb9wb+tsRqk
dsjmmcfKa9HfpXlamqeCH6mxJ0j1hL3eU8jOJ9NLOk9pf6b2Xobq5TQ7/1L2
vTYaL2t1xyeDtJ+fqXEkSPXb2fOdrofZedFpnqXsPO3Z+84Ha2wEqd47u316
P4vGs6jdaD0FOo5tvvwzwbZ++O4gjcO2dct/EWzrSy8Es8+H1D7ZcdJe6yOo
qqqqqqqqqmbNjptk5w04fqzEWjC7n23c7Eiw5aV+F6TxrF717ZSXo89TfXU2
HzXr9fn0XgNt58TxEutBqtvfut7+4vO0tN739ptL/CrY6ut/GaT5Rtq8aytB
amfKu2bXec/2E1VVVVVVVVXVrNk8THYdxuzfpfl1qS6d5pul/B7l06hOL5u3
6VWPff99JRaDbd2Au4MPPlBiOUjHJbs+6dTmp6X9aXm4U8G9e2oEs/WZQ7+/
QPlDqpvN5p+ndhxVL8VsXT3l7bPX7V73QXpvi64z76qxK9jW6zgWpL/70osl
VoO0P3Q/zc6XTtL26Xhl67ez10lqh+z3au8LrgdpvZV77ylxPkjvF9BxpHmo
aP2d1h6ngjQ/G51fNA6bfb+Djq/vkamqqqqqqqrq0PaaP5/GX2g7lI+iemxa
L/jOO0qcDmbrnKlunL5vtn1efqnEWpA+T+MvvfJFQ6/rOvQ6lb3GrejztB5E
y08/EZyDeH2NO+c3S+cLnRfZfP5Y82ipqqqqqqqqqlJdHH2e8jNUb095J5pv
ts2LfzrY8jQ/D9J+Ut4y2z40Hz7luyifSfk9qsc+sL/E+SDV17X6u+Vgdn+o
Hcaq686+Z9HyxCeCNC9xr+87dP5W9Wp26HGW7DzzNC6QXaeY7qf0nhrVXdP1
n75vmzd9X/ALNU4E6X6XHXebWn11r/satfPba6wGP1pjb/ANNc4GqZ3fVuNI
cA6C5s9v+3U0SO+b0HNFdtyQzqPs+xG91gdXVVVVVVVV1cvv0Pkiytv0mveD
8jY0XkZ5SPL5P5VYCr6zxs7gkcMlloPO26MXc95lx7noPZE5iNZPd8xvlvKc
rd5/Izh2+6mqqqqqqqqqXqq95sXN2uaFfST4jhrHgjTPKuUzs/OW0H5SPVg2
z0mfp/nzX1fj2uD7aqwE6XtR/m1qeVrr81Vn16nVY9P1kOqQs+uqPLKrxEaQ
1o+mdYfb9eZUkK5jtP8fq3Eu2P7+iSC9N0ftSfO6U7vRe3PZ+uqpzYuVXYfo
4YdKrAbpe1E/+cPvS5wO7t9XIzgH0daX/uH8Zmn8l/o/zZ/f3vOI0nNm9rkl
e7xUVVVVVVVVdTr2ys/02k623j5bz59d37mtx3g8SPNZtXzhWtC8yjTtVSc/
ln9Z40zw8KEawTae+cVgWy/9V0Fax4HOL6rnn9q4p6qqqqqqqqpqdt3D7Dyr
Dz1YYiXY5v19INjmlz0bzM57TPX5tJ/Z9snmOa3P31rr81Vn16mNr9F1g+rD
6e/S9YGuV3Sf+mqN5eAN15fYG2z17MeDbVxuPUjztN91Z4kzQbqv0f09+15b
L6n9h74O0/f9Ro2NIM0D/9KLJVaD2ecQ8rlnSywE6flhDuLZZ0qcn99s9rhk
3/ek4zjWe6OqqqqqqqqqOh0pv9er/pzyP9n1/rLja/R3W17w0SDNz5D9u3pl
mF0PgvJ41A8pb0l5Y5rX5Y019gXbfGwPBx98oMRykPLwUxvfVFVVVVVVVVXt
VWdO+RbKC72hxtlgm89gV7CtZ7gSzH5fmuekV740u16q9flba32+6tUjnUfZ
622v/aF5v2k+/Ox2PlHjQHAOgtanpnag8QsaT6H7YLZu/Os11oNUX51tz7Hu
U9QPad0Euk/RcwiNZ1H7vL3GarDdJ08G31TjnmB7jvhW8LZbSxwNfrbGapDe
Q8m2J/XD7PPV0P1BVVVVVVVVVS+/2TrkXtuncQr6PK37THm2l18qsRak+cb3
7S1xNjj28dFxzJ4X2fHKbN6P1nE4eKBGsM3Tth08HKT5amh/suu3qqqqqqqq
qqr2kurqe80DT5//TI0jwXfW+F2Q5ifJ/l3Kw1CdGG3H+vzLo/X5qjr0+Fr2
/Sy6Ln2wxkaw1d0vBNt7Zg8G2/q99wez3zdb975rZ4n1IF0Pqc6cttPrePXq
D9l5+LPzumfXHaB+SP2NvlebX2ot2OrojwbnIFo9/m/nN0vrPmSPS6/nMe+/
qqqqqqqqqjr0e/3ZOvzseNa2G0qcC7Z65G8HH36oxGqw1/xUenmc2ngcSf2f
1itv3+NUkPKNbZzze8GW13w8OCvzvaiqqqqqqqqqZuvzs3kMqlvbu6dG8NZb
ShwJ0vYp30if7zUPM80X0eu9Buvzm9bnq86u2TrboaV6YJr/nLbTrisbwba+
9Nlgu68+GaTreZsH/WiQrocvvVhiNXjieIn1IG0nW09O7yNkj0t2fLDX9bbX
8wDNe9bW/9kIZtcjoP5Gn6e/e/xYjSC9J9LWNXo02P67tSDtDz0v9XqenNpz
i6qqqqqqqqpevEPXz/ea54Gk+uTsdm6+qcTBII1rZOd5MH9yZUvHN9v/KV9K
eWPazr33lDgf/FyNY8E31vhFkPLq2fn8VVVVVVVVVVXHkvIYlM+keT9o+5+q
sRRs+cVfBp95usS5IG2f5iehz2fXcySpztP6/L5an686uw49/pWVxq3ovkbX
ebo+HNhf4nyQ5vlp9c7fD9J9M/t9s9f/XtvPbofqrunzva63vdqB+kn2eYnG
c79WYy2YbU8aF2vPfU8FaX6299RYDH6hxkaQ3gPNvh/hOtSqqqqqqqqqV55D
1+dnt0P5omxegvI8NM8GjZfddWeJM8HsONfYx1mHlfJsNE8I5bcpr0h5P+qH
lBd9U43zwTY/zUPBlo98IPjlGotBeo8gm6dVVVVVVVVVVe1ltg6Z5nGl7bR1
BU8EW33WI8H311gL0vaz85PQdrJ1jL3WA7U+f2utz1e9eqTzOlvH22t/aP55
uv5/tsZqkOYFavOa7wrS36XxEbpO0vei9ty1s8R6MDsfEa2v3auef2rjjzS+
k31OeOXlEuvB7P7QOC/tz9bvg8Ro/evk/GaPHimxEqT9pP5A6yJN7X0fVVVV
VVVVVb10s/MmZc3uD+XBsutvvvB8ieXg038scTb4+hp3Binf8o0aG8Hsuo06
rr3yXUPnzSj/nJ135e67SpwNtrr77cE5iJYvPz6/2V7fV1VVVVVVVVW1l9l8
Xauv3wjS5/fvqxGk+RBoO73q5On7Ut41Wy9Kf5fyw9bn97W166ngddeW2B2k
OsDsccyeR7QdOl7Z4zj2cdBp2Wt922y/zV73SLo+Z9dxzs6TQ9+Lzvd2/Xgy
SOML119XYk+Q6vw9368Ms+O5JPWH7HrTH69xPtj648+DcxC3bC9xeH6z9Hdd
h11VVVVVVVVVh573fuj8J823cNutJVaDb6lxX7CN83wv+PUa60Ha/17zSuls
OXTemPo/zdNC5xHNS9PyfE8F2/nxwyDV57+xxrkgzdtP+0ntSeOJtB36PI13
0PsO2fFoVVVVVVVV1avBoefxIOl3enYeA5qXnvJ79HmaH/gjNfYEs99r6LxT
to601/avqbEUpDztP9dYCWb7W6/3EYaW6mNv31HieLDl2x4Lvq3GSvDBB0os
B7ffXGIpeOhgicUgzW/zgRrrwew6udn3VqxvnC173S961eeT2ftO9ntl54fP
Sudde29sNfj5GseDc8mgebEOHyqxFKS67uz9XS+PvcaFaRwn+14Y1e23evz9
wTmINp782/nNHjxQI/jXNRaC2fcIpvYcoqqqqqqqqqqX7li/9yk/SXWzNO9H
dj4Qmq/jwP4awex4Qa95IXS2HHocgfKTtN76jdtKnA/SOGb79/3BN9T4frDV
3d8dbP++N0j5SRp3oO9L+Uxqt17r3o7dv1RVVVVVVVWn6Fj1+ZQHoDwe1YlR
HpLyCe+vsRZs8wT/KNjq908Gh25Pym+8+kqJjeDOh0usBSmP9MD9JZaCf3qu
xGLwvTWWg39f42xwDuLWW0qszG82Wx84K/Mz03H8Qo0TwZtvKvHL4EMPllgJ
fqZG9Ks1loOtX0epPp/mvcnWz1P/p/Odzuuhr0v62hx6fdtex32svDHVJ9P1
nPLn1A40/88naxwMtvd7fhNs4wCPBucgPlVjaX6z1uHPlr3us73ec6Ht/OH3
JU4H5yDovct31tgZbONXJ4N0nmavV7Py3KKqqqqqqqqq0anVo2bnp6LtUJ6/
jRv8OtjymqeC2fV/s/up45od9xx6HIfGj7LvfbT5qw4G2/ZOBVt9/b7gvr01
grTO+JdqnAnSeFx23qHsPEjZfL75f1VVVVVVVdWLN1tH1Cuvkl1/k+obs3X7
bT6P88E5CKpjz84nkG03+jzlPdr86+vBz9ZYDVL99ldqLAWPHC6xHPyrGueC
LV/77eDf1lgN0vel407551mp3255uNNBWq+B6nWpHbLSeZe9DmSvJ/R3h55H
Xfs66/X5f1bjQpDeH6HrP913eq0HkZ3H+8M1TgdvurHEvwXnIGg9lDbv+EKw
vW+0Ecx+Xx3Xoftz9jmK+v8nahwItvVntgVb//1i8JmnS5wL0nUg+37Z1Mbx
VVVVVVVVVfXyO3TeO7sdWp+3jedsD957T4nzQcoHzsp4jb42xxpfpnGukydK
bAQpL0fjp/R3ab1pmseGzqOXXiyxGsyev73yjdn3a2g+E893VVVVVVVV1Wg2
fzJ0fWy2Tjv7e7/N134k+JYa9wVp3UOa34P2h/KuQ6/nSHkqqh/Lev99JRaD
lHd6X42VYLY+PFu3f7nPq/+U9pPm6aX6/F7fq9f5S9/LusSr0179auj3v6jf
ZrefXa85u94ubYfOI6pzbutOnwm2euRvBdt7CT8JfrnGYpD2h+6Pveq6ta9j
rU+R7Q90/t6yvcRysK0v/ViwnQc/D36txlow2z40Djj2cVZVVVVVVVXVy2d2
nGjocbdXXi6xHmx//1Tw9TXuDFJd7odqXAj2ms9/7OOpOak/9zq+veaBz45L
/k2NpeDxYzWCNI9Zmy9nIbhrZ4n1II279Ro3zM6j4vz5qqqqqqqqqpfuWOsS
Dr2ftC7nx2vsD9J2qD4zm3ca2qHzujffVGIxOAfx/J9KLM1vNlvvOitSnurE
8RrBVie7O0j5MTq+1J5UD0yf71VXnz0vpnad0a2d2roG2XVVsu830XgT7Q9t
h8bFqE6Y8vZHj5RYCf5FjdPBN9T4frBdb34W/GCNjSB9X1qPYKz+oFs79LhY
djya7jt0Pz12tMRqsL1n+cNg6++PBul70Tgyfd/seJaqqqqqqqqqTt+p5T8p
r0J5iReeL7EcbPMc/DpI81nR/lDeler5aTtTm39JX5vZfGCvv5vN82fn2bim
xlLwbTUeDM5BtHHpB+Y3++wzJc4HKd9OedTseUTjg3Qc6e86rqeqqqqqqqp6
5Um/919+qcRakPIe/1DjXLDX+oy9zNY5U54kO/885WeoPv9NNbYFW15qJdir
DnxqUj685flOBGn+/Gz7ZPtVNv+WbYfsfObm92bLqR0Xuh5SP+/1fhDV7Wff
i2njXYvBN9a4O/jOGr8LzkG0f985v9lP1zgc/FyNtSB932/WiI7VH/S1Sc8D
2fM9uw5O9n5H67DPJYPei6F1lOh9gaHX71ZVVVVVVVXVy+/U6vOz8/bQ/B5t
fo47gu+qsStI9cyU58nmXafWztocepxo6O3TOF12Xpo2z9Z6kN5zofNrDqLl
HU/Mb/YLNTaCtJ/Z/Dy9X5Odh8T3a1RVVVVVVVVn1+x89TduK3E+SPX5Dz9U
YjWYna+e9n9W1iMg6ft+qsZSkOatfejBEivB7Pz/VL83tfVPKR/V8lgng20e
371B2g59X8qHDz1PcvZ9EPq89Y16KVK/ytbTUr/NzvtE/Zbm5aa6d1o/95mn
S5wLtnz/F4N/+H2J08FedfXZ9T7G7i/6/2/2+py9/tPfpfsUjf9SXf3BAzWC
cxB795Q4M79ZGp+i/ff9FFVVVVVVVVXNzjs9dN6M5t9u42aPBa+/rsSeYLZO
uNf8DDquQ4+79ZoPjbaTrRun+Xbo87feUuJIkMZJaX2Kt9f4bbDNr7MepHbI
nqfUPtl28LxWVVVVVVVVnV2pLot+77d5C44HW13VT4NUD5mdh5n2P1tHTdun
ukeqH6P2ydY503beU2MxOAex/eYSS/ObpTxPr7zc0P0zW49K82DsuK3E48Fs
v6LPW9+uU3DofpjNz9N26LpK12eq26ft0H5SPvyB+0ssBT9R40Cw1eP/LEjj
bkcOl1gOUvv0qsfWcZ2V+0L2eYbq5PfsrhFs50eUng+z782N3X6qqqqqqqqq
+r/M5g+zecXsOp7ZulbKvz33bImF4Idr7A6+uca9wVZfvBL8Ro2NYHZ+g2y+
dOz+ojmnNn5HeTyaP4fOL8oTbruhxL4grYdLdfttPrkfBVu+fyFI143s9Yq+
F33eOnxVVVVVVVXVi7dXPqRXHTjlDV55ucR6kPIkLc/xiyDlSWh/aF6Rq63+
ir7v/feVWAy2urZrg++rsRLM5pGm1v7W56tevFOrz6frT3b9VrofkbSfNE7X
3ls6GaT1pt9UY1uQ8v+tTvl0cNfOEutBuv7QuMbUrtva1+z9ms47WueC3tOk
7dP59foae4I0P1VzNZid59/7r6qqqqqqqup0zOYPs/XnQ+cBKH/y4AMlloO0
zuY1NX4VpHwg5XMo/0nt5vqDein2ykP2Wrfi0MESi8H/XuNssOX1nwy2vOOP
g/fcXeJcMJsXpf3PzkeXXY9AVVVVVVVV9Wp26HUzh94fmnf9DTUeDR4/ViNI
26c856zUjffS+vyttT5f9eKdWn0+nb+0Dmx2nK7Vy28EP1BjPUjjU20d6TPB
uWR8vsbx+c3S+2jUzlO7Duu4Dv2+Jz2P0flL3nRjiQPB/1xPabMH9tcItufH
teDYx0FVVVVVVVVV/2t7zZ/fKz/Wa75xyivSfB2fqXEkmM2L0v5Qfiabh9Rx
ndp8+JQ/zL5HQ9J88tn1cOk82runRnAOov37mfnNZq9j9B5N9jpGedqhj7uq
qqqqqqrq1Ww2L0d5A5p3l7bzthorQZpnoM2jfy449PwJs671+Vtrfb7qxTu1
+nyaH6bXvFi98t4njpdYD76zxo7gHATV+dP7bu36tBGk8QVqz+x6BHp5nJXn
HNpPem78lxqrQZonfw6Cnie/VmMtSPvv/VdVVVVVVVV1Ombzh7Sd7DzSlCfs
lZ+huuI2f9X3gy3f+VTwD78vcTpI8/Nn85/Z+uGx+4tOS+pX2fVtafu0fsQH
a2wEaTu0zmZb5/ax4BzEddeWOD2/Wfq79H3pvZjs+xfZcXxVVVVVVVVVvXiz
8wNk5xuh/CHtz9/XOBts+YxtQarnn1q999S0Pn9rrc9XvXinVp/fa14dmjeG
9ofy8888XeJc8OM1zgfbfew3wU/VOBR8R421oOtK6xTOa7qv0flF74PQ8157
f2VncA6irXfxxPxmaZyLxqfoOWfs46aqqqqqqqp6NZrNH9L4Vzav2Gtcg8bR
7ru3xEJwDqKNm/1gfrM3biuxP/in50osBrPrG2aPy+XuH3ppTm2+fcrD0+cp
35jN79H8IS0f+a/BNp/c3cF2XTkVpL9Lecvs+wW0HefhUVVVVVVVVY1S3q9X
PqRX3RHlP+m9/s/WOBps8388GuxVf0j5h6nVjffS+vyttT5f9eKdlfr87Htk
NP88+YkaC0Gqt7/t1hK/Dc5BtO0dmN9sdr4gGn98+aUSa0HK87vu7ZUtnS/Z
852eB+g8yr7/8pEaZ4LtufLJIJ13bR791WB23jxVVVVVVVVVvfxm84eUh8+O
i/Wa54rGy9r6lieCtD7gNTV+Fbz3nhLng9nxNVqX03kMrmynVp//jRobwez+
0+cp733zTSUWg2+t8e1gGwd4MEjj4NnrRrYdrrZxcFVVVVVVVdVLsdfvZcpL
9MqnUX6P8oGUP/xYjX3B7Dqh9HmaP2Hs4zyU1udvrfX5qpd+Hxl6+3Q+0vWH
6vB7zQ/z6RrLwXYenwresr3Er4NvqfHD4E03ljgQbONyG8Fs3X52vh2dptm6
+rHG1+g87fU8sGd3idPBOQg6v2j966GOn6qqqqqqqqrmzeYPaTvZz1P+jer2
aRyKPt/yggeDrX7/J8FP1jgYpPUBs+1M2+m1nqlO07HGoeh8zM4bQ/2WxgUo
T/6VGkvBNo59f7DNo78jeP11JfYE6f2X7Pei4/XqKyU2gpe7P6mqqqqqqqpe
Tfaql6Z5UL9WYy247YYS+4I0z8B115Y4HczOk0z50mzedda1Pn9rrc9XvXin
Vp9P1/nse2c0bxV9nrb/yssl1oNUh//mGgeClP+n/XlHjbUgfZ7ep8vW+eu4
9jofh94OPb/R+NqJ4yXWg/R3n/5jibPBuWS8u8b5+c06L5yqqqqqqqrqdMzm
D3vV85OU539kV4mNIM0rcvuOEr8LtjzifcGWLzkepP2k8TuqE87Of3W1zYs1
64413kR5tuz6FLR9Or+y35fmyWnn91PBOYgbt5V4bH6zdD7S/tD5Rdefd9XY
CNL2pzZuq6qqqqqqqjoFx8qfUH1+W5dvNdjq7n8dbHXf3wu+t8ZykL4v5Q0o
X5HN0459nC9V6/P7ttuHa5wO7t1TI0jrTmbrYHvVzVI/71XnTF5t78XMumPN
s539u1T3S9K8MVSfn51v//j/y96dNUlSXenej49wEI1oRHM+w6vTrQkN8RXQ
hBj1GdDQakmgNi65rHmi0JFhMhmGYTKgCmpGIGbEUPOYWUPOQ2RZS0wFFG2v
7b2tzfrE0/8Ui9qe7pH5rJvfRUV5enh4uIevvfba53KItA41XVdp+9Qnh76/
tdbv9vd3ddi19amj/a/o9+eXc1wSy+/NbeJ3cxwRo+t00Peu1nGm6xiNM0Y/
96Y/X2uttdZaa621tgvSc310XInyhFRXTM/1J0/kEMu//0nsQZRxmyf7w27d
kmJGpLxKtM92rfVJ7dq06fUvonlv+v5SX/2bcsyLZZzwsFj68TwjfiHHhEj7
SfNo6DpGectov7K2zxdrrbXWWmuttaNhrefKpvts1FqHjvIPtfpXUF1QyStM
i1Tv/Q85fi9G+xtQ/oHqGLtWH077Q3VrlD+J5p2+mmNGpD7Pd+SYE6Pvd1Tq
9qm/dOmLf04s61K+IH74QYqZz2xZD3NW/OD9FNPir3LMi9/KMRCj689G63td
Bzha1uo31db+R8/naD0wXQdoflNZt/YJseTXHxVPnUwxI9J1nvYn+jk6326b
MHp/p9fTef7euykmxTIOtk/sQdC60jROTdcZ+t7R/kfH0Ujaz7Y/f2uttdZa
a621tk3pOZ3Gm2i8LJo/pOd3Wh+wjI8dFnsQZRzmdH/Yu3OcEx/ekWJBjM5T
OHQwxUCM5nnaPi9st6w1HhEdF4iOa9DrKb9H43c7tqd4XaRxRhoXiF6Xon1F
/P211lprrbXWWnstjnp9frRvdjSPQdun/CTlPaiuuPQr3Stu2ZziZZHW3YvW
I1Hfg6b7CdPxpDzwwQMpFsV9e1PMiVQXWuro1e3bUsyJP8lxSexB/DLHXH/Y
6Peia3X4JJ3nJd82Lpbz/VWRPkc6njQPYu+eFLPinTnmxXtyDMToeU6fL32/
otu3q8Nov+hof5tovp22Q+tZ0Hbouk3X4TLPbJNYxtnuE0t/rGmR+vlH59nR
9TY6HuH8fDftWj/86PeX5p/S7z36PVPq8V8XqX/+9Tl2iXQ/jd7X6Hcgfa+j
249eD/39tdZaa6211lq7FqTnX3oep9dHn7sp//ZAjoFI9bo9iNtzzPSHff+9
FFPi93NMitHjSfn/tj9nO9rWGmePfk9r9a/7fOtjviOW/nYnxFp9Oc6Pp1gS
6fXOH1prrbXWWmutvRabrp+vVQcVrVuOzn+n7VCeLbpO5TO7U0yJtH5fySOe
FGndQMqj0v5QvVN0PkKtulM6ntSH5OOPUsyLp0+lmBVpfsQ3csyLP80xIZa6
0d+JVLcWPf5dy+tG+9bel+OieOxoDpHOq+jfpfM2Og+l1vkcnQfUtb7rtq5N
3wej+XPKY9N26Hp15cMUs+K/5pgWaXyt3K/eFmmeDo0vRI9z9L5G24nWD9u1
aa37V/TvHj2S4pL4wxwnxB4EvX7/vhTzIvXJp/2MrgNVaz5pdJ6vtdZaa621
1lrbZaPP19H8A+XlaHyH/i7Vx27bmuJVsfRbekX8Wo5ZcdPGFJMi1f1Sn6hP
r6YYiDQed2+OJZHWs277fLHdslZ9fjQ/RuMF0XXMaT+pD0npF/KaSN/T6PGM
rt8R/VzaPl+stdZaa6211o6GtZ4ra9WdRvvYU54hup5dtM6WtkN5ttLH9LTY
g7gux6X+sJT/jPYNoLpEOs5krf7Dtc4HOj4knT/Ub/9LOR4XqQ980+d59HhG
pfOEzreSbx4Xb8txWKzVb6fWugPR84o+F+frVoej0heFzn/qhx+ta6U+9tRn
m66T1D+f1iuJ5smjRuuHSX/fV4ddmz8Vve/TePrYuRSLIt3XTp3MIZb5iE+K
NC+S1teg/af9ofdVa10ta6211lprrbV2NRld15i2Q8/pNP4VHZ+idaLL3/mT
WP7OeZHynNTn6szpFHNidHxhVNZBtqNltD4/Oh8n+nqqw4/O66F5NNS/rlw/
3hZ/lWNejOb5o/3urLXWWmuttdbaa7HpOurodqLrZtLfrdW3M/r6Mr9/Tly/
LsXzYqlvelDc82yKGZH2J5oniR5PqhMja/U7bbr+n/zg/RTTYvl81ok/yjEr
rtY6MarbvzvHObHU574hUh6Mtk/fa3o9nf+15jtEvy+jUu9t6xrNn9N26Lyi
Oli6fp47m2JBvD/Hokj7Q/3ty3jauEj3wU8+TrEgRr8v0eNcqz7frg67Vp9P
49q1+q3RfZPq82nezYH9KRbE6Dgdva/o7yh6X9H5gNZaa6211lpr7Shaq0/+
c4dSLInRfBr1zabxr9LP4/ci9VOK5vkpP0B5mG/lGIg0HkfrAkTX47Zr02h9
fnQcNppvr7Wd8j2eEnsQZVz9of6wWzanOCHelWNBjB7n6HXSWmuttdZaa639
LI5KfX40f0V1PtH9jPYlLvXaF8WSP9gm7tie4nWx1DUvirSftF5ArT4Atern
6fOl7f+9/v3DUv6Ezh/y2WdSTIvlc3tMpDrV6LoMo9535bePpBgXS577HTGa
P48eH8q3N32c6TxcrfM17PLSeR6dx0HXeeoHRa+n83PnwykWxc2bUkyJ5fv9
lljq7g+LJV/+snjoYIqBGD2e0fw5Xf+j827aPr9sHduqw4+OZ0XHwWk8/fs5
JsUN61O8INLvH/rd++scCyLdB2v9LnJ9vrXWWmuttdbatSzlr6LPxZQ3i+a3
33s3xaR4Y45TYg9i65YUM/1ho+NN0TwM9fmn17u+116LtcZVKT9Waz+j+b07
csyJ/5TjoFjyi/eJ/7WexrDU/yf6vpw/tNZaa6211lrbhLX6xtfqC011OFRn
SNuhfhfRfgKUZ6C+H9R3lPIJpW/xWbHUOS6JtD+0/7Sd6OfbdF1i03Vx0XUb
Kd9b+jrsFr+RY16k/Ynmyduq26c8Hp2HpR/MuHhbjsNidP4C1SVGj090nd9a
60G4f/5oWevzavpzp/sdnc/Ub5+2T/ntst7sy2LpX/WsWOr6XxLp79L3i64D
0fw5jZfR95eOc1t13fbz2dbnRedVdNyWfg9THT7dx2mc/ac5JsQyTv6q2IM4
eiTFpf6wNL+V3i/tJ83DjR7PUZ8Xaa211lprrbXW/nebHnegPCf13/hujiNi
GR97UuxB7N6VYrI/LO1PtP9VNK9Idfvuk2+vxej4O+XBaPya/m503k00z0lu
25rilFj6zL0tlny8Gt3PqG2fF9Zaa6211lprR9tRqc+nujjKm1EejPIA1Gc+
mpco9YknxB7EX/8jxcX+sJQ/ieYJqT6f8gxUx9X0+gjRfqfR8432n/zg/RTT
YunfsFMseaNZkc63aF169DjXkr5HtJ/UP7/04f6LSH83Oi7g+ljbhftjW9J9
6tOrKQZirXkxZ8/kEGm9mOXXsW3/OFobuQ5Erw/0/Yr+rrs3x5IYfV/jYykG
Yrl+XBCvy/GUSOPs9H6prj66/7X67XvczVprrbXWWmvtStpWn6Jo/23aPo0X
0Haojr3M+98jljr9dWLJQxwRDx5IsShSvpTGy6KfS7TvR9vnnbVdvC49vCPF
gkjf97JO7/NitM9PtH+RtdZaa6211lq7knYtf0jboTwYPadTfRQ9p1M9+Q9y
TIk9iLtyLPSHpT6oterVqb6Ljtuo1KlGpfNhz7MpZkTKD1P/fDqeterKmtb1
+dZeu9F6+Og8JppfRvtD9xEat/pmjkWR6ni/kONR8aMrKebEtj8fuzqtdV9o
uj6fxnOj60TQ6+n6UOYXDsTye3BcpPmJ5e+/I9L+RI8zbadWPzqPm1trrbXW
WmutXUm7Nr7W9HjExx+lmBdvzvEXsfT5OC3u25tiToz2AYjOU6B8LG2fxhO7
Nu5j7X83un50dH1qyuP9c45JseQv/yiW8djfidE+e9TXiPKu/v5aa6211lpr
rV1J26qDjdZH0fNyrToc2s7GDSkmxNIneLdI62z+6bkUl0X6u1TfRft/fjzF
krjW6pRcn7+8rs+3a9murWdB3wu6X9D39+onKRbFHdtTzItfyvG4WNaHeUbs
QWzamGKyP2x0foG1K/n9rVWfT6+Pjh+RNH4U/Z18a44F8dTJHOKWzSleFrdu
STEjRuvqqa9ddDtd+31lrbXWWmuttXZtOir1+dG6dBpv+l6OCbHkEV8WH9mZ
Ykyk7VPegNanpnwCbZ/6aNFxcF9uu5LS+GOtvGWt/aT8J/XPp3k6PYjrc0z2
h6X9oTxqdFy+qc/VWmuttdZaa+3atladUi3puZjyirX6bVLf0dOncohl/b0X
xEMHUwzE6PGP9glxnqHo+vzldX2+XcvSedi1eWo0XkZ5Zhq3ovWgexDUJ3/z
phQviV/NMSOu9Odq14ZN1+fX2s9oXyn6vkfXl6frw7mzKRbE8v0+KvYgyvzU
o/1h9+5JMStGj0+t+Qi1PkdrrbXWWmuttfaz2LX6fKonj/aDovGy0qfquNiD
uDHHqf6w0ef6aJ988v/+NsVlkf5udJzO2muR8mZd6zsUnUfw/RyTYrmuHBTL
uvRnxbY/H2uttdZaa6219vPYtfr8aH1Orfr8L+aYEUs/4BfFW3K8JdbqpxHt
40F5G9fnuz7/v+v6fGuvXfq+RKW6XOo/T+NBD+QYiKUv/oJI42Ll+nefWMbZ
fi+WPjdzYrQfl7XX8r2Lbqet+vzf5FgSo9un7xf1hSt19xNiD4LWmy7f74fE
9etSXBSjv4dpfJyeC2g7a+13r7XWWmuttdbadu1afX40/09/t/S5H4jHj+UQ
exAlf/Baf9iv5JgWaX9o/6mvPm2H8gbU/6rWOonWduG6FF2nnvKQlPej9TVK
vvB5sdTj7xTvzTEuUp8iWl+Yxlm8/q+11lprrbXW2pW0a3Ww0bq+aH5s+b7E
F0SqSyzr8p0Vo31ISMoz0Ospr7LW8oeuz19e1+dbe+3WOj+j42jR6znlmcv/
uyCW8bFt4o9ynBJpvoDrY20TNl2fX2ud6Gifq+j7ivaLo/72tK50WX/+hFh+
Hz0olvv5BTE6H+Hfc1wWa/XTW+nz1VprrbXWWmvt2rBr9fnR7dC8+PtzLIrl
ufwdsQdR+mC90h+WxteobxXV4X8rx0Ck40B512idv7VdNpr/jOY5aT4LjcPS
+rwlH/mYuHVLipNidNzcWmuttdZaa61dC0bn6Uf750frFe/JMRDL33lH7EFs
3JBioj8s7Wd03gH1H6jV96Dt86IpXZ+/vK7Pt2vZWudVW+dnWcd1QYzW81O9
femr/ahY8uVTIu1nrXlq1q7k9ze6fbpv0vlP90Fa/4LGm6iOPfp3x8dSDMST
J3KI1Fe/rCs1KdL+RPtxtX3eWWuttdZaa621y9m1+nzKA0TzIWdOp5gTKX9Y
xlV+J/4sx6RIeU7af3q/0fFBWn8wOm6y1vpi2ZVx1PtOlD73S2K5fpwWqT6/
B1H65031h6V86adXUwxEWsez7eNnrbXWWmuttXY0rFV3VOvvUh0O1eHT66nO
h/JmtP0v5VgQSx7xaZHqt/8tx6xIxyfaR5Tye9H6/LVW5+z6/OV1fb61zV1n
ap230XlVlE+mPjble39epOvhnTnOiKM+XmDXpk3X59Pvz+h+1hrnpb5zdN04
cjiHSOtNfzXHjEj7Q8eH3i9dD2ke60qfT9Zaa6211lpr17Zdq8+n7VB/ePKO
HHPi9Tl2iUeP5BApj0rjfbQeKEnvl/KiJZ8xELv2udu1aa3zKjrPJbr9aP+N
7+a4JG7fluK0WOruXxL/OcekSPvTtfFZa6211lprrbWrw67V50fr6ik/QK8n
6Xn81hwL4vJ9P54Xqe8oHR/qj1rrODvPUHR9/vJG+1qXuryLYqnje1ukdW+p
TwXl/6nPBvW3ofz8oYMpBiJtn44P9TGOXgdqrSvalm2dt7XsWv/t6H2TXk/n
/4b1KS6Jpb/1i2IP4us55vrDUn2v+0dZq0bHv6Lfo+j9nX6f/EuOKbGMg20S
y/+7INLfpd/t0Xp7er2vP9Zaa6211lprm7Br4270XE/rR9N2KL9X8txjYg/i
azlm+8NS34C2P09r7d+/zlCejfpvRK+HNN63dUuKV8T161JcFCkvSvN02j7e
1lprrbXWWmvXll3r7xHtG0z5Pdo++fCOFG+IPYjbc8z0h6X8J+0/vV+qB47W
e9N2aD8pH0LHmbZP0naif5fWIzh4IMWi+P57KabEm3LsE2keBx3/H+e4LHZt
ncTo9670jxkTb8jxjPiDHFMirWP70xwT4nU5Lok0X+Dpp1JcEkufHvV7OSZE
6uNRPs/zIq2nSXWDbdfXuz6/zvtqqz7/3NkUC+LdOc6JO7aneF0s3+MnxJJH
vyDS9Xy1nifWdkHqN0Wvp/t49Hd16VM1J34xx7Niua6cEWmcjn6/RecX0Py+
tj83a6211lprrbW2ptH6/GgegPLY1N/jlhxviXflWBA9j97a1SeNM1IfIcoT
luvHklj6DB0R/zHHaZGuM77+WGuttdZaa63tgl2rz6c6Z+p3Hd2fUkc8LZbn
+IdEqt+memPqp035UspLROu6/z3HZZHWIaU6aqpzpj7MP8mh0utp+yRt/8s5
1BPHU0yJVF/dg/hljrn+sNE+El3L/0Tr327LcVEs5+874rdzLInROkMy2n84
ut5BdH+in2/b9fWuz6/zvtqqz/9Vjnlx44YUfxZ7wYjeZ+l7vVrPH2tXUuoz
H91O9P5L8yIf2ZniTZHG5ak/FV3fovMLor9nrLXWWmuttdbaUbTpPlfUp6iM
jz0mLj/O0v7xsta2d12i8QUa76Z1e2/M8apY+gy9KdL+0Ph728fPWmuttdZa
a+3asmv1+fR66uNBf/eBHAOx1KEfF7+Q41GR+mlf/STFokj9AaLHjeqOSJrX
QH2PR2Vdv8/X730gbt6UYkosfdmfEkv/13kx+nlRXVyt+vCo0e/7/8pxUSx5
rHdEyvNTHiw6jhB9PUnnf/T40Pui60Db9fWuz6/zvtqqz6e+06Uf1VmxrOew
Wyz57D0i7SftD53ntb6n1q4ma91fon+31nbKukbjIl1PSv3+QKTt0++l6O9/
a6211lprrbV2NUl9n6hvDOXlDh1MMRC3bkkxI9L6uWfP5BBp/50ntLb7Rtfp
oOsP1eHTdv45x6TYgyjjk8/3h6V1NkdlHNxaa6211lpr7eq2a/X5tbZPdUEn
T+QQ169L8bx46mSKGbHW8fl7/UGvNW8Z/Xzpc4nOp4juJ22f6kKjfWW/lmNW
pDzzR1dSzImUd6L9HJX++XS+lfkc4+LOh1P8RaT6fMrL0boPtD/R+Rp0/KP1
1bXmWbRdX+/6/Drvq636fNrOvr0p5sQyD+klsdTpHxfp+0h/t2vXN2u7bPR7
XWud+ujvOpJ+F1F9Po2v0fxWel8eR7PWWmuttdZau5aN1sdSvpHm7/9Ljimx
5At/L5b1mU+LNF5D4wKrNf9s7Ura9HhQND8Z/V5/8H6KabGs17FNpH5Ee/ek
mBXp79L7avvztNZaa6211lq7Ou1afT7Nc6c+vZTfozrhY0dziD0Iqi+iOivK
c9LryVp9VqnvKNU7RecLUB4jWvdF26e6aPrcab0A6p9PeZ7inFirHrup73Nt
y3E9L1L//GiffPocaX/ofKO/S9b6HKP9ftuur3d9fp331bX6/Id3pFgQr8+x
S7wvx0Ux2qc6ep1v+/O3tk1r9bGvNS+G7ss0nk733zIO9pJYrkNviO+/l2JK
pP2k66TH96211lprrbXWrgWj4ym0nVtyLIk35zgnlr7UT4pXPkwxK9LfjY6X
tX28re2ioz6OQ993Gs+9J8eYWNZLf0KkcUzaHxrXa/s4WWuttdZaa61dnXat
Pj+aVyQPHkixKJb84gGx5BcfFKPz7n+c47IYrcuK1iPR/IXocaPPpVYdfvR8
oDo0yp/QcS7rJEyLpe7+MZH6UUePZ3S+w7V+3/6edHzoOJf81phI/fOj+Xaa
jxOtw48eB3q/0f7hUduur3d9fp331VZ9Pl03qM9M+b7uFEv97JQYrfuNzlNr
+/O3tk2jv6NoO9HfabQdup7Q95rq9mn9KZr/uHVLihmR9pN+99Za18Zaa621
1lprre2y0b7W1JfpmzkWRVp/k8bR9jybYkak5/da+RBr17LRcdW2xnco3xgd
d6DrFfXP37QxxTGRxinaGp+11lprrbXWWrs27Vp9Pm2HnqNLveFApP6cN+bY
I5a+wsdEqrePrh9KdU1UX1Trc6H9rFXP37TR/DNt54s5ZsQv5HhUPH0qxaxI
x3NU+khHz6t7c4yL5fun/fOj+1OrHp7y/zQeEd1+dB3h6PVhVGzqvFwpV2t9
/sYNKSbEcp17Vnz2mRTTIu0nfU+7dn2zdhSt1S+Org+1fp/QOh203j31s6L1
rei+efWTFIsi7afXp7bWWmuttdZa24Rt5VGj4yOHDqYYiDQ+Rf2LaN79TTn2
iadOppgRo3W51trPbq3+adFx/Fr996h/Pr2+9NV7XjxyOIdIffza/tystdZa
a6211q4tu1afT3U4/yfHhHhbjsNiqbt/VSz1iX8Qx87lEKl+nvIMTfcDoXpj
yq/Wqsut1Ve/6b9Lx+32HDPil3I8Lv4yx5wY3f/oedL0953OEzrOJR82LlL/
fPq70fe7WuvDbbHWeBadt9H+7dH62Oi8EhqHonklVG9fxsFOi1SHX+Zd/VHc
vi3FnEj7T/cdj69Zu/LXyaZ/Pyz/u+uC2IOg+Y+lD99xkfYnOo7m3w/WWmut
tdZaa9eC0b4f58dziLT+5nU5nhIf2ZliTLwlx5JI+0njbm0fV2tXg033f/tO
jssi9YUr15slkf4ujb+UviAviaVO/z5x754Us2Kt40njO9Ht03Zq9XWx1lpr
rbXWWttNaz2/Ux1R9Ln17JkU8+JXckyLd+Y4I/YgSt3QRH9YyhO6P2c3pc+F
1l0teZp14jdyzIvR/rRdO09cn2+7cF9ouj6f8pa11muIrqdA+1PWo1gSb8jx
jEjzy8q8ov1imZd2SqQ8Oe1n9H4dnR9h7Vqwa+tN0Pedrg/U9778Xv4zqPX5
ZXvPidE6/LtzLIpd+91lrbXWWmuttdZ+FqP9f+i5nvIP9+QYE3sQZT79S/1h
f5ZjUnS9vbXNXQdqbafW+r/Rvj3nzqZYEKPjDtQnv9Tp7xa3bkkxI46PpRiI
0XGN6HoBJF3PPU5qrbXWWmuttd10VJ7Xon3Fqa7ymzkWxR/kOC7S+ptUj932
cbIxXZ+/vK7Pt9dirfrSruWH6fXR7VAe9Tc5lkTazvdzTIob1qd4QexBlL77
R/vDRsfLonly99W3trnrZ1S6TtLvAarDp+vbvr0p5kQaFyvzh94QaZyOfj/4
d4K11lprrbXW2tVkrfp82n7pd3VGLOtzPiSeOZ1DfHhHigWx7eNn7Wqw1nhi
W+ty0nWJxvfp9d/OsSTSdawHUfKRx/vD0nXsxzkui9HjEM0Dd62vi7XWWmut
tdba5W36Oa7p7Zf1Mwci/d1v5RiIpU/wHrHMj39FpD7GlDdwP5Bu6vr85XV9
vu3C9b/pzzdaHx49P2n9VtpO9LjRetC35Tgs0v3urhxnRcp70/WB3m9bn6+1
a1m6ntRaf4R+39L2y7jWlFj65T8tlr5T74jUtz/ah8rrd1hrrbXWWmutXQvS
czqNZ1E/q+tyvCDenmNGpOf0aD+uto+ftV206fGXtsZ3qO49Wg9P+cPS3+iY
WPqCPC5Sff6fnktxWaT9oXGWaD/86HXS9fzWWmuttdZa265N18U1vb7e+fEU
SyJtZ/++FPMi9f34UY5TYlvH09bV9fnL6/p8ey2Oen0+1Z1Gz08a56LjQ/Wi
hw6mGIh0XyvXpyfFcn/bJpb3cUGk90XHIZqXttZeu9HxmlrX1ej8pjKfdVzs
QZRx/qf6w5b9vSDS9bPpcUZrrbXWWmuttbbL0nPxR1dSzIk35HhZXL8uxWGR
6mBpf/7z0xRLYnTevbW2uetDdDv0/aXxEcrjRccXqJ/erqdTTIinTuYQyzjJ
XpG2H80r0vyC6PomlI/1uqLWWmuttdZau7qt9XxHz7NUJ0zboef9p55McVEs
/Ts3iV/OcUmM7o/tpq7PX17X59trsen6/OjnTq+nfGa07jSaN6bzPJqP3bY1
xazYgyjrzDzRH/bokRSXRHpf0Xyvx9es/ezS96Wt+x193//vb1NcFj+9mmIg
PpBD3bA+xRGx/M56ULwzxxmRtt/252mttdZaa6211tY0mn+jfOPGDSkmROrv
UcbNjouUByap71bXxkesXQvWyjfSdYbm70THU+j127elmBNLHvGwuHVLipNi
mR/wJ/GRnSkGYq11AaLjJl3LG1trrbXWWmutrWutOkmS8m9Ub0/bObA/xYJY
8oiviV/M8QfxKzmmxVLPOBCp33K0ztOujK7PX17X59sm7he1thPdPp23lDeu
VZ9P0v2CXv+rHPMi9Xsp96Wd4Jj4nRyXP7MHD6RYFOk40+cVnY9g7Vqwa/c1
+l7TdYzG3eh3xZUPU8yKNH+21OPPi88dSrEkUl8+6tPV9udvrbXWWmuttdYu
Z636/PJcfl7sQVyf41h/2HtyDMRoPSrlGdo+3tauZpse349+r+m6cWuOBZHG
cWg94r/9NcUlkdbxLOMtMyKNm9D7outwtM+V1wO11lprrbXW2rVp03WY0fqZ
+3Msil/I8bS4eVOKl8Rv5lgUqb6oa/VUdnldn7+8rs+3XbDp+vzo/YVeH90f
qhf9WY5JsfSlflukPjDUd7r0oxoXo58L5cnp+xtd59pae+3Xybb6KVGdPF03
bsoxL5Z5tA+Jt+QYF11vb6211lprrbXWqpQHoPp8Ggcp620eEfftTTEnUn6A
9pPypW0fP2u7aK3xRLo+NF0HfnOORTH6vr6dY0mkev5yPXte7EHckGO6P+y3
cgzEaN6V3m90XZLo51vrc7TWWmuttdZau7xNP5c1/fxO62BSXX15zr5P3LY1
xavivTmWRNqfUi95WWzq87PXpuvzlze6vmS5flwQv5TjDfGTj1MsiOfOplDL
/1Mpz0b9vaN9LaLnD/XBoNc7P7Yy0nGOfu9+k2NJjPbbp/Ow3H9mxe/nmBTv
yHFapH4vVAdL99Po8XT/K2u7Y3R+E0nXt+j6VvQ7+Rc5ZsRSd/+WuGF9iiPi
/n0p5sW2PwdrrbXWWmuttTHbypdG8721/m40Dx/17hyL4pHDOcSy3uYTIq1D
vWljikmR+v/Q/Hp6fdOfu7WryVr9jpren1rrztN1m8YZ169L8bzYgyh1AeP9
Yem6Te+X8qLRvlKfXk0xED3OaK211lprrbXdlOromv670fn4tJ/Uh3P3rhST
YskjHhR7y4buPz1HN11/2PTzda3PJfp3Kc/wQI6BSPXYH3+UYl6kOnCq9964
IcWESPWuv86xIFJ+aVTyzNG+DSWvf078hxy7xZMnUkyL77+XYko8fizFpPje
uynUvXtSzIp35VgQqc9G9Pve1nXVLm+0Pj8636rp8TuaF0D982/M8apY7nsD
MbqfdH2j70XT45jWriabXpeq1n7SdYD+Ls0DKutWTYllnY3nxB7El3Nc6g9L
+x+dX2CttdZaa621tl3b6jcVzfdSHozGQUj6u1QXSvtP4yDRPtIP70hxtj9s
tP8JjevV6hdt7Vq2a/X59P2l6wBdD2ncgdbp2LE9xbxY5g+9KFJ/vy2bU5wQ
aT/p/dL+0+dC26f8quvzrbXWWmuttXZ1S8+bTdcXfXQlxZxI89/p+brU+R4T
aX/oObrpOp+u1eeT0T6rVC+6fVuKOfGHOabFUvc9Jf48h7p1S4oZsdThnhd7
ELSflCen+vyu5U+i803K9+miWOrp3hapHp7WsaU8W9neZZFeT3mtWt8XGjeh
cRnXLa+Mtfrn0/lD9fPUJ5/WZ3nqyRQXxX/M8ZpY1n09IN6e46RY6/pD14da
8xraPl+sbdPoeittjbtF73fR93tfjovi9TleFMv17LxI431tf87WWmuttdZa
a2N2rf8zPf9S/o3GTWrlx2g71Pem5NH/KFKfoqNHUlwSadynrIO3JP72kRQq
va9afbattSt/naTrEo2nROfj0OvpOlPGb0+LNF+J6vnpOk/vt9Y4VHReQNvn
hbXWWmuttdbamPT82HS/aMpz7nw4xaJI9Yq0/iblJ6PPv02v09e1+nz63Gvl
Tyi/QfXY0X4LVB97Q45psfSf3iOWPjELIr1fOm50ntSqJ48ara8r82AuiiVP
f0SMbr/W+VZL2p/ofnZtXsZqNZofpnGl6HlL32uah/LX/0hxUaT5ZT0I6iMd
7d8SfV/ReWo+/61t7ntR675D10mah0i/0+j19HfpeljmX24SS9/9l8RSvz8p
jsp6RtZaa6211lpri13rC035MXrerDVuEt2fI4dTXBRLH/3HxTJOMi6eOJ5i
SozOi6e8KNn052itvXZrrT9C46E0LkPjGqUuYFEs9fXHRKorKH2S9oqP7Ewx
EGvVS9QaP237vLDWWmuttdbatW6tPpy18p+UP6T+w7Sd06dyiKUeZ5dIffgp
rxjNE9aqH+5afX5b+fCmt//hBylmRMrD7N+XYl7s2vuqtT9Up1f634yLN+V4
XTx0MMVApL731E+D6gApj0fjI1Q3SNL23Q+8m0bP52gfeLpO0vlM9zvKox47
mmJCLHX6D4rl+jQn0n5G77MkHbeuXd+sXQvW+p1G16ta10nqZ0XrifQgvpDj
z/1hy++OebHtz8daa6211lprbcyu1eeTlH+O9uGh8anz4ymWRHp+L8/HE2IP
gp7Hy/87KlK/I3pfVHdaKz9prV15o/X50XU0SFqnm15P/YWe2Z1iSizjKtvE
bVtTzIrROvnoOvW0nWg9v7XWWmuttdbalbFrdeO0P5S3pLqgDetTHBGpf/6n
V1MMRNpPyn9G+4S0dZyj26fPJbqeAn2OVP9Mx5PqselzofpYOn9+mGNapH6t
0TzMqIwjRM+H8vmcF0u98GExml+KStunPFX086K/G/1euJ5/ZYzW59N2ovPI
aF4J5YEP7E+xIJb71UmR5g3RdY/eb7R/Phm9X/v8t7b70vWTvu8P5BiI9Lvu
n3IMxC2bU0yL5Tr3mFh+778gRvtutX28rbXWWmuttdb+z7aVP6+Vz6f8G42P
RPN1lGejflY0XvatHAPx7hyLYvR4Uh7VeUJrR9da4y/Ljz9qXpH6gNF4cVkn
ZEGkcWHqB0LridP7peND403R40b3i66NO1trrbXWWmut/XzSc2LT+bRf5ZgX
qU9+ec5+Q6y17h7VQ9Z6v6NSn19rf5qW3u+pkylmxFIv9oT4yxxzYnR/KH/S
dB17LUu97XmR6vOpjxBJfTBofg3l+SmPVyt/Fa3Pd35sZYzmh8fHUgxEGi+j
vxu9j3zwfoppsQdR8r7P94ettS5Ares8fR9rzRO0djU5KvcFqm+P7j+9ntar
OnE8h1j64u8Tr3yYYlaMXp+ttdZaa6211rZr08/L0XGiaN6e8t61+mb8OseC
WPoM7Rap7wc9R9PfpXzj1U9SLIqUJxyVvkbWdtmufY/oulprXCA6fkHX84MH
UiyKpT/c2+KmjSmOidH7CF3/o+va07iVx1+stdZaa621drSsNf89un3qe/n0
UykuiaX+93fi9m0pTouUF6U+7bT/VKdUK7/atfr8Wn+X8tiU16X8Lc2PoLpu
Oq9uzzEjlr4Ij4rRfDVJ+9O1vkC0n/fkGBMf2ZniTTFa5xw9D6PzSppeB8Tj
CO0avX9Fx9duybEkUh9p+rt0PSnjZi+JZZ7QQZHuO9H1Xul7Gu07TesFtH1e
WNtFa9W3Nz3ORb+faTvl/j8Qo7+333s3xaRIdQU35Dghtv05W2uttdZaa639
n20rj0rPv9E8GOUVaR46bYfqOf8tx6z4kxyXxB4ErU93R47TYrTfTvT40/p6
TX/u1q4mu1afHx2XiUrjF9H1ten6vHlTiuNiWW/kFTE6rkrj4LQdui94/WJr
rbXWWmut7abR5/Smn9/p+ZHqecpz5DtiD6L0QT/ZH5bWv6PnZbKtz6vp7VN9
V3Ref9PHJyrlJZbvn79T/FqOWTG6P7XmcdSS8ur0+f72kRTj4s6HU/xFjH4u
9HrXvXfTWteBputOo+uK0nairy/zVwZimU92USz1qs+J9+YYF6P3qeg4I61z
3dZ5Yu1qMjrPq+nf7XR9iNbV0/2dxr/o9bQ+Nc2jLOtoTYrR+0t0vhUdh1p1
C9Zaa6211lpr6xp9Tow+30X71ZBlfGxMLOtF7xdLfnGdWPpXvSb+PMeUGB2/
iPbrqHV8rF3Ljvr8pmi//eh61r/JsSR+/FGKebFcLx8Ut25J8YpI4yZ351gU
o/cdGlei4+k8pLXWWmuttXa1OirPv9F6QrJW3izaP4TqFcs89T+IN+V4HZwX
o8cn2pfYtit9X/Y8m2JGpDz2N3LMi5T/ieaR2jo+rs+31+Ko1+dH87q0fgfl
e8+eSTEvlu/NW2K53twnUn3+j3NcFqN9Wui4jY+lGIjR4+zvr7Wf/fsS3U7T
18nouj8kXa/oOkz1AFRXsGF9iiNiqVNYEGutu02/o+g+Et1OdHzTWmuttdZa
a+3yRvOQ0ec7qlen7dM6wuW58LxI89apvrS8vwvizTkWxWh+oOQ1l8S2P2dr
bX2j6xTTdY/GKcjodZjyaVQ/X9bhfEbsQWzckGKiPyztD40vR4+/6/CttdZa
a621tth0HVo0T0jPy9G+nVSfT3+Xnn9pO1T3uHw/kOfFbVtTzIp0HOjvkm19
7vbz6fr85XV9vm3ifhTdTtN13dH7IEn3NZLuI2U+2bNimX/2R/FnOSZFqnel
+yy9no5brfW43T/f2pW/Djf9vaNxebr+UL09jd+RpV/UmNiDKM8Bp/rDfvJx
igUxej+6JceSGD3+tE6Bf/9Ya6211lprbV1r9QmJrkdM26F83eZNKaZE6ltV
1pHbJZZ57XMiHQfKc9L7iuYbo59L2+eLtfbvX/ei9fNRo/OGKM9Gebzjx3KI
PYiy7vqb/WHHzqVYFKN19QcPpFgUaf+bPv7WWmuttdZa2zXbyiNF+wNH+8ZH
31f0efDXORbEf8zxmEj1jdH3Ra9/IMdApO10re7aFl2fv7yuz7fXYq3PpVZ9
Pr2ervM0rlSrHzJZ8rU7Qc3rfnQlxZxI3yMymgem/Da9nsbvXJ9v7cpfP5v+
3tH1JNo3LzrP96c5JkTqE1j6CD4tfi3HrEjzkmg/z4+nWBKj88Kov+JKn5fW
WmuttdZau9qNPkdH843R/CE9/+7YnmJeLP2rHhdpvGz/vhTzIu1/rX74lHdt
q77X2rVg0+NoteoQouNB0flQ0XGN7+WYEMv4zBOgju88/VSKSyL9XVov4NDB
FAOx7fPLWmuttdZaa7ti032Ao9bqqxl9X9Hn3927UkyKPYiSd3yjPyzVt5Tn
44FI+0N9PyhP2LW6a1t0ff7yuj7fXotN15dGt0PnbfR+RHndaP05jWf9Q45N
YrkvjYm/ybEkRo9PdP4afU+j/bWu9TyzdjXa9O+BaF1BdHwt2k+P9pN+R9H6
I2We7mmxXEd3i6Ue/6j4zRyLYq06+ejzUfT6bK211lprrbX28xmt/4zWnUbH
j+g5murqexDl+fih/rBXPkwxK9L6dzQPnY4nHTeqOx2VcRNrR9Gu1SHQ/tB1
kq5LTa+zTPUJ5Tq6TaT1SmheFfXzd59Aa6211lprrf18dq0+Pyo9R1O9OtWx
ROtbyjpv58QeRDl+F/rD0vbHx1IMxOjxieYTVvrzs/+vrs9fXtfn22txVOrz
o33jo9dzev2NOWbFG3I8I37wfoppkf5udN4B7Wf0OhbtG3at55m1q9Gmfw9E
n0foOkB1AtH9p9fT88XDO1IsiOV32fNiD2LX0ykm+sNG9zN6va01f9nXVWut
tdZaa62ta7Q+n56Xo+um0etp+7/IMSOW5+D7RJq3TuMadHyiz8W0Dp37eFjb
nevbqPzdaB4sej2n69K2rSlmxbIOyUmR6vOpT2C0XwfVM1DdBc2Havpzt9Za
a6211tq27Nrzb7T/LdUxRusbSdqfe3OMi9T3Y/26FBdFygfS/tD7iuYPXT/c
TV2fv7yuz7fXYvRzieZLo/tD5xX1XYlK239kZ4qBePpUDrFcV54Ub82xINJ1
JtrPiqT8cK363rbPU2u7aNfmN9H1Lfr7n7ZD40e0fboenjmdQ6R1Scr19aIY
3X/qcxV9zoquL9C1343WWmuttdZaO+q2VZ8f3c8v57gklufRP4pHj+QQP72a
YiDS343O36fnfToO0edia+21X9/aGl+j73t0fd5oni06TkH9Q04cTzEl9iDu
yHG6P2y0n2H0uu3+HtZaa6211tq1ZtPPudE8Ib0+uu5kdD+pjoXqnH+U45RI
9dL0+o+upJgTo3lRzzdfHbo+f3ldn2+vxVGpz49ez6P5Ybo+UB1+D+KeHIP+
sHR8yjy2JTF63KL9nMno9TC6fWvXsm39roheH76dY0mMjkPRdr6T47JY+lG9
KZb9fUe8JceSSNd/mrcVHe+j+5Gvk9Zaa6211lq7Mkbzk1T3SHlCkrZPz900
/kX1+Tu2pzgjRvuZ1OoHEu1z4udia5u7vo3K+Fq0b150XRK6nn8rx0CM9s+n
eVK0/2PnUiyKdF+4+kmKRbHtz81aa6211lprm7Kt51l63qy1XmS0fobmld+Y
Y1Z8eEeKs+J1OZ4SqX8+zUMvecd58YEcA5HeV3Se/qjkN9aars//fFLdPtXn
l7zUK+K/5ZgVT51MMSPSur3P7E4xJf4yx5x48ECKRTHan2fUP8empePT1vWQ
vo/R+tLnDqVYEmk7N+R4Riz1oTvFHkT5f9P9YZs+btba7tv0dTVan0Cvj86H
ousq/V0ah7otx0Vxy+YUL4s0v4nuI1S3QO+LtkPzAqLbWenzz1prrbXWWmtX
yq49/9JzWbT+k8abaD3oUrf+ptiD+H6OY/1h78+xKEbnodN4QXQ8Mfp8ba1d
fdaajxO9X9D1h+rkab1OqltYvy7FYfGbORZF2s/oep1tf57WWmuttdZa2xWj
fezpOSuaH6PtR/tG0t+N9oEsdbvPil/I8ahYnjvfEek403GIfl7R7azW+plo
HoCOw29yLIlnz6SYF7duSTEj/muOabGcRzPiV3OoRw6nuCj2IG7NsdAfNpqH
H5X8CdXLFS+IpU7/LfGTj1MsiKW//qJIdfVlHs+CeOhgioEYHRcgo/X5XatX
X2tGx9fofKPPl65jd+Y4I9J9rfRTOSZSnxbPB7HWdq0+ITrviZ4XovOLqc/e
yRMppsUyPvaY+KUcCyL9XZrfR79D6Hegr+fWWmuttdZau7xde/6l59ZoX3r6
u5QnL/2rXhB7EH/9jxQX+8PWqoePrjdH26HXu3++tatPGj+tlR+LXjeWXzd8
Sdy4IcWEeFOO10Wq2z9xPIcYXR+Z1l+uVRdhrbXWWmuttW05KnUU0bwi9b2n
7dPzHR2fT6+mGIjU94P6DFPdY3Q/6fWUH4j22Xb97fJSHpvqXSkvfXMOlfrA
nDubYkF8790UkyL1Q7grx4JI5w85Kusv0OdV6ssuiN/NcUQs6+teFqPX1abn
v0T7I0XnW43KfaQt2zr/6TpDr6f7y7t/SzEh9iBongv93Vrr41hrR9eu1SdE
t0/jXFTHTvX/tJ/03EHzpOi6HX0++naOJZHeF22n6d851lprrbXWWjsqdu35
N7o/tC4nvf7KhylmRRqnKM+1T4u0njX9Xco3Rp9Po31ynOe0du0Y7XMV3X6t
fBrlIakevvRbelGk63MZlz4rRq+fXRtHttZaa6211tpa1nreqVWXG+1jSa//
To7LIv3daP3M5k0ppsRo3w+qo6b9pLp6Os70esoTuo/HyhjNk5P0+d6eY0b8
Xzl+J+7fl2JerHWedO28ouNW6uvPi7flOCxSv9ymzwfKy9HnRdcxuu517fOy
xVr1pdHXb1if4pJI/VJ+kuOS6HVLrbVk1+oTaDvRPvl0/6U6eZp3eexoigmx
1C/sFmndn9K/akqMHs/o80j0d8tKn3/WWmuttdZau1KOyvMvSfPT6fXLj39p
lHWGT/aHpfVnKd94fjzFkhjth0/9eaLPv7U+X2vt2rHW9YTyeNu2ppgVexCU
h9zzbIoZkfJ+dF2l/fd4pbXWWmuttdYWo32V6fVUL0p5M3p9dF429Q2mupof
5pgWqU9+D+LA/hQL/WGjx4fygfS+ouuZtn1+rbR0PLtWX0qf4y9yzIilHn+d
+NGVFHMife+i/dK7Vn9L3+tbcoyLJT/0tkjXGTo+9P0i6fhHr1eUx4ten7v2
Oa5W6TjT9T+6HgqdJ3Qd2LE9xRnxSzkeF2l+ULTfsrV27di1+gSqw4+OB0Xn
F+/elWJSLONjr4rld8ohscyX2ibS/Kno7//oel7R+acrff5Za6211lpr7UrZ
teff6PMd5avpubKMj42JX8zxB7E8v86J0XrOaN6etkPz6ynfTnmAlT7PrLXt
2XRfxOjr6Xp4a44FceOGFEfFMp78oPjM7hRTIu0n3UfoOu95T9Zaa6211tpR
N5qPasto/Xl0+9G+H3TcbsjxjFjWf9sl0v5H62lpP6P1NtF+mG2fFytttJ85
1aPS66PbIak+ltaN/UaOeZHyJNF8SNfqr2j/y/d+XCzfs3fEaL1xdH4NfX+b
rnOOzrdq63NcrUbr8+k+QvWllOcs/ZiPimW96FdFmo92d45F0fWZ1lpy1OsT
6P4YvV/TvN1NG1NMikeP5BBpHeqbcsyLtD/UbzD6edV6XrPWWmuttdbaUbdr
z7/RPlfkIztTDMTt21K8JtJ8c8rbR8dlao1zRfsX1Vpv11o7ulI+sK3xtei6
ITfnWBTLfKnXxPXrUlwUafvR/oQr/flZa6211lprbW1r1cU1nVek57JonTBt
h/paUD3J1U9SLIpUr1jyi2+JtJ/Rvta16mmpHni11uXWOm+jx79W3wPyww9S
zIilHv/34v59KebFaD/2Ucmr0H6WurNx8bYch8Xo51hrfYqm+9s3fT7bz2d0
HIryrlSfSX2raF5P8SXxuUMplsS/Nw/ps97HrbWrz67VJ0R/99LvJbLW78Ay
PnZOLOuZvCHS+Br93WidBh3P6DigtdZaa6211q5Wu/b8WysvR32YabzsuhxH
RNo+PadH15mNjndE55vXmu9grR1daTy0Vj4wuj90faNxHLpfnDieQ9y8KYVK
64+Mj6UYiLT/tcZVrbXWWmuttXbUjda3kPScRX0nonXL0XUwf51jQdy2NcUp
sQdR+oKc7g8bnR9Br6fjVmv7q7W/cVvrDDa9/T3PppgRKR9+Z455kfLV0fOh
rfr86HUp2j+f8lq0fcpH1ZpHQ32BotuPXleb/hxHxVE5bmU90gmxrEf6PHif
uPPhFOdEOg+7Nk/HWtsdu1afQPdr2j79Hohuh/rn/zzHlFh+r50RS/+qh8Rd
T6eYEKnPIe0n/a64N8eS2LX7oLXWWmuttda2Zdeef2k7NC5A+b1f5ZgXKa9Y
+leNi9E6zFrvK1rnT9unPICfi61dOzZdV19rnIXq5Om6uvz6y3tE+rv/+WmK
JZFev1rrE6y11lprrbVrx1Hpn091IJQfo9eXPN+SSH/3yocpZsUbcjwjlnrn
g2LJ550XqY4xehwof1ir3n615g+jfcjp+FM+pNZxpvOE8jMnT6SYFnsQd+SY
6w9bq99pW+dP9HtB9fml7vgvIv3dWvXwTR8f+/msNe8pOr4TtZy/S2JZX/R5
scxXe06kdTdOnUwxI9L+RPOu1tq1Y636AZJ+L0X72tF8YfqdFr2/79ieYl78
WY5JsdTRj4k0H7P8+5si7U90XnCteZ3WWmuttdZaO+p2bR3SWv2cKd9O419l
vvg28cs5Lom0n+6fbK0ddaP5z1rrd38nx2WRrufUx6PkF58Qy3V9r1j+fSDS
+6W8a3S81VprrbXWWmu7Zq311Oj5qFY9RjSfSX/3NzmWRPq7VN9Y6hBPil/K
8bhY6venxQdyDMRaz5tUj0RGt9PUeWk/m3SeUP/80p9mnfiNHPMife7RvvRt
HR/X59smjPaPqvV3o+sm0P1l+X7Lj31mad0N2n/3z7fWRq11f4zOb601f4rm
JUV/R9HzS+lHNSuWdb1eFb+b44jY1ufY9Dw1a6211lprrW3L6PNsdH405dko
T0jPlbXq3o8fyyFSnyt6nm37c7PW2pW26fwn9WGj+wiNK12X44jYg6D6ivtz
LIr0vpw/tNZaa6211o66tZ77as3vptfX6ptB+Umqz6HtlLrm58UexK05FvrD
lv7El8Va8+hr1VG7Pr+buj5/eV2fb5v4frW1P7XWaz57JsW8WOaT7Re3bE7x
skh1p3RfpvPc57+1q8+m1+dqetwq+nrq73R+PMWSGN1Pen75eo45keYLl3Gz
p8QD+1MsiLT/tdYLsNZaa6211tpRl55/u9angvaHnvvuyrEg0vZvyvG6uGlj
ihfFkl9fFH+c47LYteNprbW18mBNbyda307X2y/mmBFvyXFI/EqOaZGu/21/
ntZaa6211lrblF17fqR8ZrRfMdXl0utpnbV7cqjl33eKpb7+bZHmiUePG9XD
NF1v37U6VVt0ff7yuj7fXou1+kfRdbvWvKfofo6PpRiIlEct34O3RPoeUT+W
6PWk7c/fWvv5rfV7IDr/tGv7T88vdJ2kfob0fr+VYyCW33dPirQeyscfpZgX
6X3R9ZzeV3QdtFqfo7XWWmuttda2ZdPrSkf77dPz5rt/SzEhbt6U4rj40ZUU
c2Kp03xWLPnFt8Rof4+2P09rrR22a/UVtB263tL9haT5U0eP5BCj46rUz7DW
OJ211lprrbXWtmXX8l303Ef1jbQdyjfS66nvJc0Hp7qX7dtSnBaj89Ppc6G6
l+jnWGt/2jpPbNH1+cvr+nx7LUbH0ej70vT3gu53dN98/70UU2IPovRdfqM/
7L05lsTo/LiV/lyttc076vX59FwQ3Q5d92qtO7Zvb4o5kfrkl3r8h8QbckyL
9Hdp/+m4de33obXWWmuttdY27ajU5/88x5RI41wnT6SYFsu42R/EO3KcFun9
1upnYq21o2I0/0mvp/tFtB4gup9le++IJR/5gkjbp/qHtj8fa6211lprrb1W
u1YXGq3Pp3wd9auP7k/piz8m9iBoHU+qY4zOT4/OUyDpOdd9jEdL1+cvr+vz
7bUYvR7W6uNB522t91XyoJdEWhfmzOkc4iM7UwxE+ruuz7d27Vjr9yTdH2vd
N+l+Tb9/6Dr2nRyXxaave7/IMSNuWJ/iBZF+B96c45z47zkui/T5uo7CWmut
tdZau9ak56Ou5cGi+0P1/NT34x9ybBJ/mmNCpL9L6376edNau1qNjrdGXx8d
t6Lr7adXUwzEHgTV569fl+KiSP3z2/58rLXWWmuttfZa7dr6a/RcSfO76e/S
cyLVmdydY1Es9SkHxFLP8jux5B2Pi7R9el+18rq0nWi9Tdvnqf2fdX3+8ro+
316L0XGf6PWTzs/oPKz7cyyKd+aYF0te87xY7j9/Er+eY06M9pem4+nz31pL
Nl2HH13vg67z0e3Q9ZPm89L72r8vxbx4e46TYumX/5pY+h2eEKPjd3Sc6Tg0
PR/NWmuttdZaa9sy+jwb7YdPz1mUV6R6+2ge+wc5jos9iCsfppjtD0vbr9Xn
2Vprm7bp+opo//xoX33qc3hLDvXhHSkWxFIH8XuxB3F9jhf7w9I4V62+WNZa
a6211lrblk3X1Ue3T6+P1qVTH0uqAynPi1NiD4Lq9ql/yPnxFEti9H1F86VU
D+l14laHrs9fXtfn25WU8oTR/Cpt/8c5LoslPzouHjuaQzx6JId4Q45nxGhe
l/R5bq0la10nSfo9Q/NVo+M+9Ppa40dUt3/6VA6x9J16Xiz1+A+JN+bYI5bf
O4siHTeqA/H131prrbXWWrvW7Fp+LFrPT+NHlD8s9ZW7xKufpFgUo/nzWutK
W2ttLZvud9e1+wXNn/rrf6S4KH4hx9PiXTnOil0bX7bWWmuttdbaWtaqn4+u
sxbdT9o+9f2g7VC9blknc0yk58cexOZNKab6w9b6vOh4Rvvtuw/J6tD1+ctL
eXuS6vPL9/7P4i9yzIgfvJ9iWiz9aNWv5FC3b0sxJ46dS7EoRr/Xo/I51jJ6
/6o1f4rq6qP3R+q3TPe1n+S4JJbz6DWR6jx370oxKT6yM8VA9Dos1lpbpPtO
dD0RqougOgf6HVjWkT4i/lOOJ8Qzp1PMibT/dN+k+2Dbn4+11lprrbXWXqu1
8mDRvGWt/ae/S8+DZZ73Y2KZ5/2q+NyhFEtidN3S6OuttXatSfej6PXzPz9N
sSTS67dsTjEtfinH4+K5szlE2n60H8io2Pb5Yq211lprrW3frj1HROtVonnR
m3LMiz2IUs94uj/swQMpFsW1tv5atA9qW/Wc0f7qJM0T2b8vxbxIdd20/sKJ
4ynUrVtSzIjleF8QexB0/lP+n87nUa/rLvMVLoql7vhN8Zs5FkWqn6c6urtz
qGWe0ECkOu1ax6Hp63zX7i/Lr6eg63tG6/bp9dR/mPaHXk/n25HDKS6KZX2Y
TSKtC0N9+Gk9UzoOlAce9euGtXbt2LX5R/87x2XxnhwD8aknU1wUaf7mt3Ms
iXQ/it7H19rzkbXWWmuttXb1WSuf2fQ619TXhfKWX80xI9K6bD/KcUqkPLb7
R1lrbV1rzfOicfPz4ymWxHJdPy/2IMq/v90flvYnuh7NqNj2+WKttdZaa61t
3649R0TnfUfriku94YRY+kfuFEv9y3mR6lXa/jxX2mh9ftfmfUS3Q+ch9Yf5
Vo6BuPPhFIvi/TnUcj4OxPfeTTEplvP0j+LZMynmxZU+b1bq812+b/wFsdSp
HRa/k+OyGD2vovXeUaN98qPX/+j3umv3F/q7VJ8f7R9CUt+PaP0nzVMr149z
Yg9i44YUE/1haX0NOn8of1vr/VprbVt2re6C6vPp+rxhfYojYg/ipzkm+sPS
/tC8AM/DstZaa6211tpi0/2LaJyC1jWj57WyfuyE2IP4QY6p/rC0P/Q82/bn
Y62112rXxr+i9xe6btN42W05Loo35HhZLH3094tU/1/rOHfNlT4vrbXWWmut
tfa/rDW/m/pvUJ/q63McE0ve8VGx/Puk2Pbx67ptPZ9G64pr2db+nzyRYlqk
+nxar5bqxqPzYpquPydpP5cfpzgvluvHO2K0j320Tj66HXpfNC+JXt+1Ovmm
t0/Hn+rJ6XjS+RA9/6ne/tYcC+LxYykmxdLP/g2xB0HjdNHjTPOSan2O1lo7
KtJ9h+4L0ft7dL7YLTneEm/MsUcs42mHRdo+1VfQ/rf9+VhrrbXWWmvttRrN
ZzY9PkJ5Tso30nbOnc0h9iCovxC9L1qvzfO7rbWjbtfqtKP7E+2zRP3f/iXH
lFjWc94t7tieYl6s9b66ZtvnqbXWWmuttXbtGq1LodfTPGvqK35djqfEHsS2
rSlm+8NG+/a3fbxX2q7V4VN9FM3viNbr1qrbjx63LZtTTIs03+TKhylmxej6
s/Q9bas+//ONj1wQ169LcViMfq/pOETrxqN975s+zl0zejzpc4l+vtQ3PtoP
itbFKOs2XBTLOtJ7xR5E6Zf8Qn9Y2p/xsRQDke6z7pNvrV2ttvW7Lro/tB7T
T3JcEqk+v9xnHhRpvRW6z9LvRt8XrLXWWmuttaNu08+JtbYfHXc7cjiHSONo
lBel58Tf5FgSPb/bWjvqdq1OO9o/pFYfJ+q3T3nIn+eYEmutA941V/q8tNZa
a6211tr/MponJA8eSLEobt2S4qTYCwbVT1IfEtfnr4zRutxR2U86z2k7H7yf
YlqM9s+P/t22P/9ho/MCyvG+IFL/2Oh4AdWnLb8/ej40Xc8WnWcx6t8vel9U
Z05/l+azRD8vqqukfiPbt6U4LfYgSn3l4f6wtD/RdUPo/InO62nr/LHWWrJr
9fkkPQf9KMesWNZXOSv2IMp8rPH+sPQcFO3zb6211lprrbWjYtPPiU33aXlk
Z4qBWNZbOyTSOrOUR6W8X608qrXWds2u1WlHr6vRcTG6T9H9hfrn0/hs23X0
rs+31lprrbXWrjZr1VWePpViViz9wv8sln7D20R6TqT6k2j9ra1rtF60a/kQ
Mlqn/cUcM2I5358W78wxL9bq395WXj2a5y/9G8bBt0S6/tD2/z3HZZH2P3rd
i9bJR/tmNF1f3fT3MXp+Ut8nOg7Rzzd6nnwnh/pvOWbFch97TLw9x0mR1qGm
zzd6XY2uY1Lrc7fW2lrW+j0Tva7SdqL3/aufpFgUy7yuOXHTxhQvit/PMSnS
+B09N7nuwlprrbXWWjvq1spn1qrro9fT8+ZNOebFMg62SaT8Oc3Ljo5r1Moz
W2vtWrNW/7roevGU96O+cLQOSxnHPilG3++o2Pb5Yq211lprrW3frj1H0HMf
PSf+NMeEWPoEPyhSX3Gqy33uUIol0XUm7Rqt/6R8L70+up1ofXtU+rv/mmNa
pHrdX+aYE6nOir530brups+H6LoV9+YYF+k6EP2+07qQtB263tLxpM+rrf72
0Xko0ePQ9P5TXjE6zkWfSzm/lkTaH/rcy/f4NPiQ+OEHKWZE+rs0TyFaVx+d
v2CttV2z1v00Oh8zun1aP/o3OVQa/yrPSevE63O8KN6aY0Fs+nhaa6211lpr
7ajYdH8byk9SXu7UyRQzYg/ivXdTTPaHLf31l8RoXjfah7/tz9Naa7tidN4T
vZ76REXzeHv3pJgV/ynHE+LNOf4i0jhU2/X1rs+31lprrbXWXqtde46gv0t1
g/fkGBN7EKXe5Mn+sD/McUK8O8ei6L4f7VqrP3N0O2StvHq0npnq88t5/Tvx
6znmxOhx65rR/afrRum7/7YYvR7WmpfRdF+m6PkW3X6tzzd6n6Lzgerwa51X
dJ+i8aYHcgzELZtTnBBLXeQukfKcv8oxL9L5RvNcovX2tJ5FrfPBWmtrOSr9
D+n6TK+neYJlPaWjIvVLpPlfZT2WGZHmBdQ6bl2z7fPXWmuttdZa+/mN5kvp
OSv6vED5Q3p9dDyC8o035nhVpD5XX8sxK0bXXaV++9HPpe3zxVprh226D1V0
XRLaDt13KN8YHeei63+puz8n9iDKuNagP2z0uNE4IO1/dBzQ41/WWmuttdZa
slYdJtl0fS/1sX/2mRTTIvXJ70Ec2J9ioT8s7Y/7A6+MXaufic7LoPwG5QGi
/as/eD/FtPilHI+L27elmBOjxyHah6Hp84TGR2i84NzZHOIjO1O8KdL7pfOt
Vt6slk1/X2q9r1r7SZ9X9PtVPv+BSN/f6P5/I8e8SONlpc/xH8SyHvXr4ti5
FIti9HOJrovR1joa1lobddTr8+n3BvW/enhHirNiD2LzphQv9Yf95OMUCyK9
r/PjKZbEtuvru/J8Ya211lprrV15o/X59JwVzYtG+4dEn0do3vTWLSleEUve
cY+4f1+KeTG6n9H5CF632lpri9F+X7Qdun/R6+m6Tdd5Go8r47TnxR5EyTtO
9YeleozoevT0fqPj3TQe3fb5Yq211lprre2uTc/vJum56bePpFgSaT/pea0H
UeZbH+wP++nVFAOR9rPp+Qi2WKt+Jjo/Jfq9aMuv5pgRS3/Tx8Rf5pgTo59L
W9cNkvJFtJ/lez8m1uqf3/T5H83LNT2u0XR9fq39idaNUz1hrf2n9VlK3+KX
xNK/6j5xx/YUZ0TKW9aqF6X7YDQ/bK21bTnq9fnRupH330sxJfYgbsjxTH/Y
7+WYEOm+Vuu4dc2VPl+ttdZaa6219YyOI9Tq10F1fZRnI2n79+dYFEv/nv1i
6f/xinj1kxSLIh3P6HhB0+vJWmvtqFur72J0nCs6j4yu52XdzgmR1n+nfvt0
36R8aXQcMDrOFR2Pbvs8stZaa6211rbvqNTn03b+9tcUl8QexIb1KY70h6X1
yKL9PVyXWNeu1ed3rS695MtnRMpvXPkwxaw46nmD6HyZW3KMi+X7+o4YPT61
6uRpO5R3ivbXrXX8a22/6TrJ6HkS7bNxb44l8Z4cA5Hm19D6Fz2Id/+WYqI/
LO0njSdGj3Otdb2ttbYtR70+n+6/dJ2n8am7cpwVy+/KZ8Xyu/J58Y4cp8Xo
OkSjYtvnr7XWWmuttfbzGx1HiD4v0PMa5dOifaLKetGXxdL/flYsdZGPitfn
OCb+56cplkQ6ntG8Kz23jvo4hbXWds3o9ZzykPR6um7/IMeUSPejH+Y4IUb7
ldH7pdfXWg86Op5orbXWWmutXX1G843R7Uf7eES3f/BAikXxzhxnxNIPeyeo
fbMpn1lrnnjbn/+o2lb9TPT7UqufefT7ePJEimmx9NveJG7bmmJWHPW8Ac2v
oeNP14EyH+gtkfJRdNyir4++3+h52PQ6EbXuI02Pv9Dxpz7zD+QYiPT5Ro/b
nmdTzIhUn095S5p3Q8eB3i+9nvY/ui6qtdZ2zVGvz6fX0+8i+ruHDqYYiNu3
pXhNLPefp8Xy9y+I9DzVdn296/OttdZaa61du9bKc9K4GOUha9UBlnz2QCz9
qC6JNF5QxsHOi9HjEM17R/P5bZ8v1lr7We1afomut9H9ic4jo3U8S5+PdWK5
H74jRsehovPgat2XnT+01lprrbXWNv08WKs+n17/vRwTYskjviTSep27nk4x
IZb+2Utirf1v+/MfVV0/s7zUf5vqeE+fSjErRvuadm0dAcov0X4+sjPFmEj5
n1Hv2xPt40TnQ/Q8ie5n0+cVvV+qG6T1XKLjTc8dSrEklvvVlFjGwZ4TS/+q
F8XoetO0Lkx0XelaffittbYtR70+n+6bdF+j9VxofzZvSvGSuGN7itfF6P2x
7fp6P19Ya6211lq7dq1Vnx9d7yxaf055b+rDQ/3wexC7d6WY7A9Lz3d0fKLP
g9F++9ZaOyrWur9EpeswXW+j4zu0fdrO7TlmxDJv7AWxzDM7KJa847xY67jR
fdz3KWuttdZaa20t26o3oOe4e3MsiTflOCP2IEp9/aH+sL/IMSNSHSbtv/t4
rIy16me6Vk9eSzqfS97iCbH0PZ0To8ct2heo6eMQXW+xXA/GxTK+8bZI/RMo
70R1cfR66mfedP4nOk7U9Pelrfr8Wuct1cnfluOiSHWP1M/q+LEcYq16ezrf
aq2zUOs8sdbaWo56fX50HI3qOkgaLyvPYa+L1M+K7lNt19e7Pt9aa6211tq1
a636yWj+Nvp3Kd+4cUOKCfG6HE+J1K/46znmRMqHUx41uq6rn7OstWvNWvX5
0T5UtB3qMx+dn0V5y08+TrEg/n85Loq0rvRXckyLtY4b3cej8+/aPr+stdZa
a6213XVU6vPvy3FR7EGUfsN/7A9b6kfmxVrzxJs+bmvNtuptKM8QXf+O6p3K
+ahSn23Kk/xrjmmR8hhnTqeYE+n4R+vz28pL0PeX9v/uHOfEMl/hNXF8LMVA
pHWEy/YWxftzqKV//0A8P55iSYxef+g8b6sOP3q+1dofOm6UZ6PvI82noO2/
926KSbGsa/FnUNe/KH1Bzoj0fqPjg3QcoudbdL6Mtda25ajX51Pde3S8b/m+
i7puyw05Xha3bE4xLdY6bl2z7fPXWmuttdZa+/mNjgvQ62lcILo/lH+j+san
nkxxUexB3JzjQH9YynvXOs7ROv9RX8fWWmvJWvklyhPSuDNth15P/TdqvS+a
d1b6VO0Wf5ZjUqS/S/MU6P3SOFqt3wltn3fWWmuttdba1WO07y5th15/5cMU
s2KpB3lG7EHclWOhPyztD9VXu/5wZWyrfibaf4DWX7gnx0D8Zo5F8dc5FkTa
DtX9lv43vxP37U0xJ0aP/6j0B6C6tYd3pDgr0ryG0i/2kkh9HtavS6GW43dB
pP4P+/elmBepPpyOQ3SeRVufV1v5K5oXFu2PQfMpqA/V3/6a4pLYg/jwgxQz
/WHp+hD9fOn7Umuc0flJa+2o2/R1LDruE33OovsX/a44dTLFjFjmh70unjie
Ykqk/fl2jiUxepyj6xNFf8d6Xra1tmvWqltr+r4Wvd7Sc1n079LzUXQ79HxE
9xeS8kjR+7v7N1prR92m50fX2k60To+ea+jv0vML/d2yfuakWPL9z4P3iaX/
x1ExOq5H90d6v02vZ2qttU3btecvun5Gr8+HDqYYiLSd6HWb6uGpj+I/5nhM
pHWl6e/SfTZa1/Gfn6ZYEq/1fLLWWmuttdaytfofNt2vmLZDfYCj248+v0Sf
B8nST3pM7EGUOvq3+8NS/TP9XTpuVK/o+sNi1+qXousjkL/JsSRS39FdT6eY
EKm/Dfm9HBNi6XNzTuxBnD2TYr4/LL3fURn/jc7XoHr70o/huEh115RforwN
5btuybEk1rp+RuvzyabrtNu6ftL7pT4h9Pn+Kse8uHlTipfEHgStQ/2NHPMi
XZeieeNoP6tR+XyttbaWXft9S79PaH4x/V26r5X7wAWxB0HzFul3OO0/1TFG
f5+Myu9Ya61ty+i6gSRdb+l+VGsecTQ/GV0Xr9ZxjvaxpOND24l+Xit9nllr
7X856vX5lB+jdW/p70bzvVT/T+tslnXPdollvOJVcefDKRbF6P5H6yHpvub+
+dbartm1+nz6nU/X52i9SvR5jcbRaH/K/WpeLHX6+8Qv5viD+NGVFHMi5Tmj
74uOW3Se3Uqfr9Zaa6211o6ytcY1ms4fRsenov3Ao/Ue5Ni5FIvinTnOiNQP
vAfxkxyX+sPS+6XPN7qOmy12rX4p+v2l7VP9T7R/Pp3/Bw+kUB/IMRCpfz71
gae8Bx2HaD+3tsZ/6fhT3XKp/xoXjx7JIdaa39F0X6BovQTtT1vf37bqt6Pn
LZ1Xt+W4KFK9PY2XlfkZh0T6u7XmTdSqw2/rvLLW2qbt2u9buv5TfQjVw9N4
2S9yzIilbvEJ8djRHGL0ONDvllq/x6K/l3z/stZ2za7Vh5BUnxCtS6yVl7v6
SYpFkV5P6x6W9fcWRLov0zx9+ru17oOuD7HWds1Rr8+PjhfQ/Zqe16L3RxpH
6wWj9KeZ6g9L+T0ymp/0uJu1dlTs2vMX5bXoeSFa1xEdL4vWpZw8kWJapP5p
lIf8ao4ZkfY/er+mz6tWnYy11lprrbVW7VrfIcqDUf05bSdah0B/l9Zxjr4v
qjcu9fgPijfm2CN+PcecGP186XhSvrRWneSo27X6JdoOvZ7O865dB0qfgBmR
8hX79qaYE6N1uV2TPhfKk5R653Gx1PW/BWr9P10H6LpB4w50/YyOp0fr96J5
s6hdq8Onv/vp1RQDMToPrtTdXxK3bknxikh1+Js2pnhRrPU9rdVfMTqO2fTn
a621TTsqv2/pPkX3O9oOzQ8t42MviaXf4XNiWfdsINLfbXo9rGj/K/dvtNZ2
za7lJchon8Za9e2UB6A84b05lsQf5Dgu/mOO0yLdN6P3EarbpP7JdP+qdd+0
1tpajnp9Pm0nWqdH9zuaT0156dLn40WxB1HylC/0h/3nHJMi/d1oXjG6ruu1
nmfWWlvbpuvzo/e16DhgdFyJ8op0v6PrP+UD6Tmr7O87ItWHbNyQYkKMHs/o
OCCNCzt/aK211lpr7bU76vOjo/03ah2H6Ljhr3LMi1/I8ahYnrf+LJ47m2JB
pOdQWu8sWn/r569i1+qXon1von+X8h6Ur6B6bHo9jc9+Jce0WOqh/ih+I8e8
SMdnVOr26TpD+18+twtirb6v0XUhm66viNY/0H5G+0h0rT6f9p+u/6VeXo3+
3XL/OSuWOo7HRFrfoem8YvT77vE1a+1as2u/b+m5r9bvt21bU8yKpR5/t1jG
x9aJu55OMSHSvEX6fUK/n+n4ROePj8rvXmutrWV0Xbzo73/Km1H9Ie0n1aXT
62kdw3I/uSDueTbFjHjkcIqLYqkfmRKjx5/uU9F6D9+/rLWj4mqtz6e8Pb1+
+Xy1Suu/0HrTZR3dN8QTx1NMiXfnWBQpX0rHh3zuUIolcaXPP2ut/bw2fd+p
Ne4THeel5wt6HinjzCrtJ60jf3uOk2IPgvrR3Zfjohidp0zPv7SeNW3H9SHW
WmuttdZ2x2h9Y3T79BwR7ZMcnX9Nz330vEZ5uQ/eTzEt9iBoPnX0uFG/LHpf
bdWXjopdq1+i87bW83K0/jlaH0uvp/X7aF2Js2dSzIvR60Y0n9P0+UbXGar7
Gh/LIZZxjMMiHYem6+Wi87Bq1UXT9yu6/23V50frFqLfO+rfSPNfHt6R4qxY
zq8nxfXrUlwUaT/pPInW50fPn1rnibXWjoqj8vs22q+Y3PlwikWx1Om/KvYg
zpzO0R+W+jHW6m9P26fjPyr9qK21tmv1IdH7VPT6THXsdN2mfOPxYykmxftz
LIq0P/S+qI6R5iNQPSTlMWh/6O9G88DWWtu0o16fH73f0d+l6za9/pc55sSS
R7xPpL7E0fsC1WdG84fR+nw/l1lru2at+07T9SE0LkPzyJpev5LuOyUvOCf2
IGi9TuqLRfUedHzo/UbXcbPWWmuttdZ2x6afv2j7Tfe5pfGm6HMc9fEofT4e
F0udyF9EyvtFjw/tZ7TOc6XPs7btWv0SbadWXX1b8zVoPgutN1H6c8+J0Xqn
pt9XLWn/qT6/1AW8JUbnd0THZej8oXEQGq+nz3GtzSeKzjeJ1j/8nxwTYvme
PS32gkHnJ50P0fG1pn8PeBzNWrtaHZXftyQ9H0V/t1yf45hYflduE8vz3HGR
1o2ivxu930XrP7v2O99aa8lR+b0dvX7S8yzNx6+V/6E+/3TfpL4H/5JjSowe
H+rTGM0btP35W2vtsKNen0/3i+j4AtXn0/789T9SXBR7ENflONIflu6z9BxE
41zRdcqi88Q9vmatXa221b+R7mt0/Y+OE9HzGl3P6X76vRxHRZqPtnVLildE
6jcSfY6OruM5Ks/p1lprrbXWrgWbXr8sWrdQqw8GSc8vN+WYF48dzSGW+o7H
xFKnv1/cuyfFrHhLjiWR9j/6POv6jeKo1C/R6yk/QHVE9L2Lnj/ROvn330sx
JZb8xIPij3LMiqM+nhsdTynrH46J5XrwlhjNq0TXo4++r+jxof2n/RyVOv9o
/3YaF6PXU3+MTRtTHBPpPkXzZW7OcUCk6wy932hdJUnHoVb/zLbOE2utrWXX
ft/Wml9JfbTuzDEvbtyQ4s/iDTmeEct9Sdc7o/lx9Lsluo52tD6/1v3UWmtH
xWj/iujzVzTPEK3Ho+t2dH2xaJ7w+zkmxUd2pnhTpHXWyjqHs+I/5JgSy/YG
ovOT1tpRcdTr86PrL9P4QnQ/y/3hlFjy/etAXR+W+mtF+/lTvjR6H691fKy1
dlRsq38jvf7TqykGYnQ8NHo9p/sFrWtWxt9eFL+Y4xWR3lc0j0rj/rXWF7DW
WmuttdaqbY1rRPOEVNcXfS6j7USfO6geeMf2FGdEqm+kfozr16U4LFK9Bx3P
6LrYHuda3q7VL0XnxUTHqWv93ehx+EWOGZHqo8r3bF6k7Uf7ANSa1xM1mi8q
dWfnxfL/3hGj4wu1jgMdf9qfWuunRN9v9HOJnv+0n9E+t7Q/dL/45OMUC+Jd
Oc6KtM5LuV89JN6W47AYPR+u9ftjrbX2/7Xp32+19ofuj9/OsSTSdqhOIzq+
dvxYDrHUFW4SjxzOIdL26fk0+jtw7FyKRbFrn7u1du0YzUvUet6k5ynq307b
ob4c0f7A0XVVaH5ZWX9sIN6eY0YsfRbVL+e4JJZ5AH8St2xOcUL8YY5pkfo9
lrrLPSLdr+n4RJ/rqd4yOn8tuj5drfyDtbb71spr1VofObouKj0f1bovUz0h
3ReoPp/y/9F1Oem+E70v0PHxc5a11n4+a/Wvi/5uj+5PtJ6Enr+obqTc934n
fi3HrFgr7xed322ttW3ZVv1SVHoeoeev6H2QnoOi77dWfycanyr1Jgti9O9G
87rRccDo+nG1zivnD61dO9bKHzZ9X4vWJ0THy2g70X6/27amOCX2gvHrHAv9
Yek40P291roDtT7HrtW3Ux616fXaor8Pa1mrj0F0+/S7gvzwgxQzItVB0boV
0Tx5rXGQtiz9U8fF0kf/TZHqDSi/RPN6onVlo3I8o/Xw0e3X2g5dtx/IMRBp
Oz/NMSGW82WnSHnCn+WYFKkvYtufs7XWrhW7lp+M/h6gcaLo/Zp+51DfSPod
RfOsz57JIVI+kMYNo7/TonU1o/J7zFo7utaqr6Dn0Fp13bSdu3Msitu3pZgT
aR7Zt3IMRMr77Xw4xTmx3Gf2itfleEE8czqHSOvrbVif4ojYgyj3wyf6w5b6
yldFyvPQ50X3/TJf/KxY7rfzIs13iJ6HtfLM1trV56iMr9Ua96fnF8r70Xyx
kr/fJ9K60r/KMS/WqruIfo407hbtC1Tr87XW2lG3rXHbaB0FPZ/S68tz25xY
6vsuiKUv3cvi0SM5RHpupXrC6PrjbZ8X1lp7rUbHj0h6nqLnEbpfUH4p2t+D
jK5bHa0zjOZjo9J+Uv1/9DhEx0npc6fjQ/dfkrZTa768tbY71sr71eq/Ha2j
oP2JPl9En7/OnU2xIJbxonGxjI89JZZ1yZ4VDx5IsSjS/kTrn2k7TT9/da0+
P7qd6O+xqHSfrXX/bev4R/t7l+/BjEj9c+7MMS+Oet47el0t/fPGRFqPPjov
NTo/lJ4j6HobXb+Arld0/Y/+7o0+l9X6fKPbifYD+fijFPNime9yXKRxMeqf
f0eOOfF/57gsjvr31Fpr7fLWmu8WHf+KWuofF0X6HdWDKO/vQn/Y6Lxv+n1C
dYb0vuj1vv9aa5u2Vn0FPffR80Wt8azSb2NWpHX9aF7zxg0pjoq0/mapT3xd
7AXj+hwv9oct+ZWTIs3Xpnr+8rz4kkj1JJTnvDXHgnhfjotiyQM9KZZ5ALMi
fb6Ul4vmpaP5Os+Ps3b1Wet3ddN5e7r+UD0JPY9E9/Pdv6WYEHsQdD8t95Np
kdaXiR7naB44mif385e11i5vrfF9+p1fq86ffv/fn2NR/G6OI+LmTSmOi+V5
8c8iPQdR/T+tI0b9QOj9um7fWts1m65vjG6f7jvRecQkXbcp//nJxykWxAP7
U6j0dynvSvPCaDvfzLEolj4ic2JT581/WWu+djSPHa3b9POjtavPUa/Pp+tY
tD6f8mkl/zcQS/7vhHhzjr+IZRxsl1ienw6L0fVZmu770bXzLbr96HlYqw98
0/1zovPvoud/dN5ftJ8nbYfGVcu48R9E6pMWzatTXiX6e6mW0fkg9+QYE0t9
2V9EOv60P20dh+j3sa3PJXp9i163o+No9HdLPm9CpDqQ9etSPC/2ID66kmKu
P2z08236c7TWWrsyRu/j0T4V9HejzzX0/BVdH638bj3VH7bWfFv6vR2dL+l+
v9bapo3mW5pez5HWF4vW9VHd/pbNKV4W6Xmc5k3Tdf7I4RxiD4LqG+n1ZZzt
j/1hv5DjaZHqRmj7ZX8v9ofduyfFrFjqVi6JtL4A5W/pc6x1vtFzfbTeNfp3
rbXdcdT751P/w2jemCz1GufEHgTN86J+Pp9eTTEQ6fkr2qcl+rvFdRTWWvv5
rDW/u+l1q+k+QnlFWv+rPFc9KlKfuvJ89pwYzZdGx/HbGoe11lqyVp1GW/72
kRRLIvWHp/tdrec1qpOnfN3xYykmRZpP/c85JsVf5pgTaT5CqVuZF2mdU3q/
dB+n+370ed/Pg9ZaclTyh3Qdo/VfovWK9Pp/yzEr0rgb9ROm8SzKT0bHiej5
KFqXHq2T6dp5En1+p98tXbtvjsrvyej+03Gm+nwaX/5GjnmRfl/Red61+vzo
+kqln+uYSPOG6PctHQfqvxr9HkXnHUTHTWodZ3o9HZ/o/tB1qVb9HvUZ/kWO
GbF8nufFch/bJtL6L/RcWes50Vpr7WjZ9Dw7uo9Tfw/aDo1PUf6Q1kfbsD7F
CyI9b5LRPHN0PR1rrW3aWv0TaJ5R9PmCrvP35lgS6Tm05P+mxdtznBTL9sZF
Goc6eSLFtFiet14R6XmtB1HyKE/0hy3973eLZXuPiVQ38vCOFG+I5X54RCz5
iXNi+ftTItVh0udOvzeiv0OieVo6b7uWb7TWNne/i26nVv/GaP+r6Papnp/u
j+X+8LxYrvcHxF1Pp5gQ6XknOp5Va35Wrd8n1lpri7WeH2n7dH2u9VwZ/bs0
7/j7OY6JpQ5xn0j7Q/MFaH/o/t72eWGttcO21X+YtlNrvS3aPj0HlfUtl8Sr
n6RYFEv/xYtiyTueEG/LcVH88IMUM+LpUylmRaq3/FmOSbHk/46L5fi9I9L4
XfRzjPa5jf5uoTxzrboja233HfX6fKoDjPYjojpVWufl/fdSTIm0vjPVMVIf
Y8orRvt+R6//bdXPN7396HGo9Xejn2O0finaH6DW/kT3k85P+p3zwfsppsUe
BK3fN+rr7kWvz1RfTb9Xo7/rmp6n0/T2o+c/bb/peRy1jgM9L9yZY148cTyH
2IOgOhPa/6bnWVhrre2m0Xma0Xlqtfow0/4cPJBiUaQ8IdXtU16U5hFQvQft
v+vzrbVdM5qfiV7/6bpHecLo/tM6fSVvd1TsBYPqz8nlt6N19XfkOC2WfIla
6iV/J5btbRKpPj96HGgdt9Jn6xmR1h2IrttInzs9t9K4J22nVn7AWrt2rLU+
NV1/aj0fUb+sv/01xSWx3B8eFGm+GPXtiR5POg5UfxjNw3udFGutrWv0+ZG2
Q9d/6j/W9Pui+hO6jzyzO8WUSPfNW3MsiNF152k83eN31tqu2XR9frRuil4f
rYugvBNdnykvWvJnJ8TyHPak+PMcU2I0T1vmk02KNC/g0MEUA5HWP6U6qLLO
wDmx9DEdiLXOk2jej14fzVtaa0fXtuaXRes0ov096DoWzY/R/tyTYyAu389K
+01RH+NofSNJx5+eB0flfItuv+l17uh+WqsP9qgbzZ98LcesSOPCH11JMSdG
j3/X1imI7j/9Li31CG+J0T4/lL+Kfo+izzXRz6vp9YWj97Wm+1+Nj6UYiLSu
xPL9rHS8jOo9Sj/IGTE6L6ZW3yprrbXL2/T8uKh0f6R69ehzXPT5NPq7i+oe
qT6f5ndHf89E979Wn0xrrY1aq76Crm/U9z56Pf91jgWxPB+fF8t96W2R+sNT
HXv5f8+J1BexB0Gvvz7HLrGM770llvWrT4u0/2U87Xmx/P0zYllH9M8ivS/a
fqnTPCXu35diXoyeD5QPia7L4/pMa23UWs9r0f6NdF+O9tGl/odUB9KDoL49
VK8SHS+j9xW9bkfH49o+v6y1tutGnx/pvlarzo3qKKL3C9oOvd9SR39GpPnU
X8kxLVJekZ6PouvUtH2+WGvXrk3PG2p63KSsG7Yo0vWZruelPnFMLPe9CyLV
vdPz13dzXBKpfz6t/1Lmk70pbtuaYlak40b1/O/+LcWESOuy0fw1ul/T+Fo0
T0hG85bW2tF11OvzaX/o+Shal077SX3vS93i78Vyn9kpUv0k7Q89V9J1m/a/
rXXKulafXytfSudt9P3Wmk/RdJ1wdP/p+Z3e155nU8yIZb7LH0TqnxZdR+D/
Z+/Nduy6jnTd9Qp1zq5jVNV5CDdyb5/1Cpb7pvZ+BlnubRXgS1/aEiVRlrxh
GEbBMAxBIimJFEn1fcskmcxkssu+54Ylq5e8cTBGQCjv9deXlb85JtdcmRE3
3wUXZ852NBF/RLjfV6v3lkh/l64rxpWLwgO3FXtS+O1qm0Kqr370oWJrwrsO
FlvfNam/FfVBCF2AMnRwSjrO3b8utimkvINW40Or94rOM/QOG8LQX5wRUv5y
nNejwoh3nRSSzpDOf9L7WSSTyeSks2/6fPq7rfSW7vqT4lB0npT/SDpGiq/R
+dP6wd0XUD/rrp9vMplMun4JGodbxd2oP+ah+4stCO+5u9gFIfkrflBtRRg6
9EPCARjVJaB+nd+pdkFI10X74sgnWBCSbj/+zpbw13cVe14Y8+FNwtDXPyuk
+0P3c+ZcsVUhvW9Ur9LVn9D6hOpLp24zmdx7nJS8V3c/RfsyGj9DJ3JZ+M/V
/igkP//77xXbFFKdGXedQHFD9zg0X7TKT0wmk8n9xlb6fDo+zV/uedLfdfu4
UXyT9CQDMMrXprp2rp4w6+cnk8m+sdW6uut1u7tPjP3QqpD8ftRvhY7ztWrT
wvDPPQbUOBfFxXbuj6Z+v9gXviAcgN1+oNjZ4ShpHxp1sx4XUj3/n1TbEH6h
2rbwZ9W2hK4eppV/MplM9p/jml9c0vr/kZPFrgrpODv7D5Wkh5w+W01I9ali
3ntQSPFBOn/a97n68Fb9vvv2nozL/9mqzyzVwSb9Oe3fP1Pt2knHd3XR77xd
bE1IumvKZxyAkZ6c7nPXeQpdk+IppB+g9Sc9d+rrEf2vVoSUV0uM9b6S9hFU
7/3BB4opb6i2LPxRtTWh65dz6/+7eRb0dykO5frTYj+0LaT5K77jg8I77yi2
KiQdQu4jkslkcm/T1f+76/ZWeV6kx3D7XIc/cl5IfsjI535CSDpJ+rvu/RlX
37RkMnn92XUelltnw/XDUF3cv35QbFtIf5fqUZw9U2xJGPune4WRX/6ckPQS
VM+Qjk915kMX//iuSXWf6D6T/5P2xfRcQne/IaT+ofH/LgnDPzElpDxxqsNM
Ok+qD0DrAbcfn7s+od/T90Ln2aoeSKtx4FrHr2Ryktn1POsen35P4wnVb6Tj
h25iUxj+2mkh7YMGYFQfJsefZDKZTPaBrfyfn6u2KaR6awMw2p+SPpPidLRO
cPMgkslksmu6cRzKO3b3TR+8X2xLSH+X/Eik9wi/46yQ9BikP/lUtVUh+RUH
YFQfg/x7pM8h3TvtK0m3T/Mj5YOHv/DnQjpO1O+fEca8ekIY8+o5oVvPNvO4
k8mky3H5xyhuQvMmHYd0CzROfqnaKWGM9w8LP15tUejWLXHzEYh0Xa3qwI/r
Pelan0/HcXW29N5SfRjqE0Q650nhN6qtCeN7mhcOwKh+uPscW72H42LsPy4L
w1/yqpB03TQeku6a4ss0zhDJz+OOG3Q+7n6NztM9n6516bSPo35ecV+1rxnN
XxGXe1Ho7kNdnUNX30kymUwmu2Gr+BQd351PW633aP9FesKoZ3xESHrFyJO+
JHTzuN39bzKZTPaNtN+k/QLtL6iuUdRnmhEOwCiOQ3EfOk7oE58ejpL6Pkc8
5xkh6f8jb/4BIfkhab9P8a/w08wIyW9DOkyK31E+fsTHloQRD/ydMOpdPSaM
/3dG6L6f1I+b+g7Q+0zHd+t7pD4/mewPJ12f754njf80b4af8hXhhblqQtLP
jPs5J5PJZDJZ2Mr/SfoQmmd/c0+xl4QRrzsu/Hq1VWHG45LJ5KST6kuQPoTG
21Z1zql/Mek3SM9z/33F5oU0blM9k09UWxJSnUbK84q6JmeEVFf2lmrbwtAf
vSI8c7rYopDuf/j/FoUUd6M+nnQfSLcf/suXhceOFlsXnjxRbFtI10V+v1b1
ypLJ5OSy63pcLklv6ereiZTPRXpFqlMd/VWWha7OxI3vuHryruuK902f3yo+
5a7r6PfkJ6f1m6t/7htpHUt10Unv5NaLo/ffZav31iXdT9J3UT5R3+Kqbjza
PT59d5QX0Gp8oPMnXQTpTGg8oX4otF+gfmSkc6B87Vb9o8f1viWTyWSyLbvW
57t06wP/f9WUpPMM/eO0kPyZpFck/aHrf6N60dSfK+tfJZNJl66/hfYLtC8j
fxf5/WicjzjPZWHEZ2aFND5H3v8xYfghfi+kuklRV/68MPqlvCCkuv3ULy/O
63kh/V3qYxh+yqeEAzDSw5M+hPylse98AnhaSHE39/yp/tgXq10VRt39GSH1
c6TviN5b97vrmuMaZ5LJSeSk6/Np30G/Pz1VbEE4AItx+uhwlB+rtiDcq3V1
kslkMrk32PV6e+ZcsVUh1WGjfG3qk+7qQHK/kEwm+0byN1Lch45D+gr3OKTH
Dr/dlpDqYLh9kKkOauy3DgrJ/0l+zvj3RSHV/zxxvNiWkPTwlI9Gujjyi5Lu
JfyO9wvp9wOwmFefHo6S6mtRnK7rOsnJZHLvsW/r81b6cyLVd6I4HcWVaB/k
6hvdeZl0oeN6jn3T53edd0Zx5/Qz70zquxT6pj8JKT9xr95/iufSupTqvJHO
gfRatN6m49B45erk6e8SW9Xhp+NQfMo9vpun474n36l2QTgAo+/r8KFii0L3
fFKfn0wmk3ubXevEaN50z5OO4+4LflRtTTgAi3XEo8NRUtyN7gP5jek83f5B
yWRyctnK3+L6zdy+vbTPonpTdJ6UP0W6cao/T7pBih9RfIfiWVRPmOpcUb5Y
1Pt/Vhh5AP8u/HS1NSHdZ3p/Iu60IqTzpL4zN1WbFw7A4v4dGI6S9PnBXwip
Txzp/995u9iakO4b1T37abVNYdffu+tndv2orc4/mdxLnHR9PuXz0jxOdXVi
/nlQSP7Gm6stCkmPkbqFZDKZTPaBrfycpN+L+XMVqPPsAGz6bLHl4SjpurqO
VyaTyWTXJD8V+SHJf+XqqF/7c7F5IdWRiL6LU0Dtz0j+tPBTPimkOsPhJ3tO
SHp1mneiLsV54TerzQqpnhXVGwl/50PCAdjO/UZVVx96okeE9Hfp/sT5nhfS
+0nzfqs6pclkcnLZNx1+K10l6S5onqX+zlRnmPpQx/y2LnTvs1tnqW99T/qm
z6f95rjeczof0tWQv3pSSPrt71dbFtL6h+Kne9VvQO8J1c+P+3RK6Orc3PXh
uNaTrb5fmkdovnCv14130/GjDuKGkPq2xD7jXiHpTKIu4JrQ1U/m/iKZTCb3
NruuH9V1ny+XVC+X1qukz4zzvSKk9QbpUek8+9YHKplMdkdab7fS9bU6Tzo+
6dWp3vsdtxebFn6t2oqQdPjUp28AFvWiTg9H+d1qC8Loi3JG+Jt7im0LSSc5
darYvPCBI8WWheQPpHglzRcXLxTbElIck55v3NdlIcUTqR8c3X96XhR/pHpi
8b5MC0knE3URXhRSPYG4jqtCd752/Zzu9+geJ5ncz5x0fb57njEvXRTSPih0
DTPCW6ptC/eqPz+ZTCaTe4Ot/J/UP472vwfvLPascAAW+8uLw1G6etRx3+9k
MpkcJY1X5IchfZ2rW/h8tS0h1e8l3cW56WrC0PW8LIx91X1C6l9J+nyKT5FO
nvqzUF9p8puFv/ZpYfz9m4TkvyV9C10v+esob8Lty0l6UdLVuHVNs25GMrn3
2LWOohXdelykB6bjk/6Zxn/K84q6XKeE1N+EzofGYTc+0rd6iX3T57vHp/vv
6njdv7tX98X0XGj9FuuaQ0LSR7Wq+zcpccl4764Ib6w2JWwV/3XPk3QC7n12
5wW6Xjofd7yi86fj07hB13X3r4ttCsM/NyukvmCkQ6C+ZtSPzO1T0yqvIZlM
JpP9ZKt9Zav1Bs0vpPdw+6L+uNq6MPadl4UUR6M+R3RddP6k93PzZ8f9HiWT
yfbjsHucVvlQdBy3T3TkH58XDsDuOlhsfTjK0IOfFVKci+JidD4RT1sRUt8T
0mO08uP992pXhRQniuueF1I+ePy/GeFbbxZbEdK+8ifVNoQPHyumpPkr5tUZ
IeUvfLzaopDiqjdUWxbSc6Q8C/o9+Qfo+dK6xdW10nfdt3ovyWSfuVf1+V+s
dlUY+WGnhKTfiLyrJaF7f1KfkEwmk8k+sGtdDelIaf9C/cJCB/q4kPbLbn3I
ZDKZHBdpX9NKz0NxpR9WWxWS/pz6jv3l9WKLwuhzclYY/qFXhXT8ARjp4cOf
dEVI/rGd88X0flI+AvURID/kN6rNCL9c7bRw576cqpOh+0N56HSfSS968kSx
bWHqZJLJ5Lj0+a7e0tVL09996MFiq8IBGPVNpr4tbp9u2ge5elrKg271Pozr
OO7xu6636epOu9btTwrpumbOFVsVUh8liufS+EDr9r7VZXWfe/RBuiSMf39V
SLoIGj9JJ+D6bVp9F63Wq/SeuOfTyn9F9/8z1daF9HxJf0J9pamehjsv0/dC
Oor0+yWTyeTeYKv4VCtdojt/ucd31w+hi7wkJL+fm9/t6lX2av5vMpm8dtI4
6a7/aV9DfioaxygPnern035nAEZxGerXTPss6jdNOu0Lc9WEFO9znyPlbUUe
9qIw9JJnhFF3aktI9fyjztaG0PXTkq6e+n1TXaw77yi2KiR/CO2vyd977Gix
deHOfcb1frp5c259ADqOu3/vm38smewDJ12fT/uvz1XbFJIeI+qIvCCkuBvV
t9x5PbD7eSSZTCaTyS7YtT6ffk/916iO8QCM9pvUZ23c9zuZTCZH2ao+J/mF
yJ/2/WrLwtj3zAnJbxb5UlPCAdinqq0OR0njNv3+7JliS0LSn4S/8ylh9NW8
IKT+AnQcqv8fdU0uCcOf94yQ8tTofkZe2nPDUdJxqG5/+I//IKQ6MOG3uyJ0
44DJZDLZiq3q/7jHcetxRZ7TY8NR3lxtUejqQKi/ibuPozon7nPZq/p8eh+6
Pk96Hyi+1kpH1DfS9f6g2oqQ1jlUv7RVnfZx0a33HvuDS0KKL7vvlfv7rvOX
SddNf5fup5vHROfv+q/oOVL868BtxZaEFBejfJYBGNUDpPGQ7psb77vW7ySZ
TCaT/WDX8Smiu08k0jqB5n0i7eNIDxm6kkeF5Bcl/zBdV86/yeT+Yat9Vtf5
O7QP+mq1ZSHlN1HfPerXHDr9E8LYR14WRpzqjDD8ELPCf612UUj6bdL531Jt
W0j3La5jW0j3P/L4rwrd/TvpJMn/9r1qS0KqA0n1mek5xv74SeGZ09WEdF1z
54ttCKleFuVlfLLaipD64rn+BHperfLfaf2Q+vxkUjnp+nyaF95/r9imkMZD
ipeRnpDOv9X+LplMJpPJLtjK/0n5sFRfl84ndJ0Lwtgn3SN04+njvt/JZDI5
Sjc+QuMb7YOiLvuakMbbj1RbE4a/8Kgw/Jf/Loz6Fs8L6fx/Wm1T+MZfii0J
D91fbEE4AKO6KFR3gnSS4e/8vZDuA+nwyT8cdf1PC0N3PyukvgPk36P8AtKt
7dxfQC38lIvDUea+OJlMtiLtR0j/SXEft2455cFRnlrUYXpcSP1TPlttQ+je
n1Z5yn/9oNi20H0urfqH9k2fT8eh39P7Nq6+EuOuf3+tpOuidVHkKf5RSPFu
GgdavQ+tnq9L8tt8p9oFYeRxviJ09fat9PBdx60ozk7X654Pjc90f1z9Px3/
9deKLQhJfzIAi33JoeEoqV8Y3U/6Luj3rfq5JJPJZHJvs2913cn/+b//Wuyq
kH4f/rl1IelbqK6ye/5Ul3Jc9zOZTHbHL1TbFtLvaR/k6nvddT7ty6huRox/
m0Lad1BcZgB2eqracJQUD6L4EY3bJ44X2xLS/XF11+68SftN1+9Kx6c4KdVV
Jj0n1fui94Tq7X+i2pKQ9OdR92tLGHqWi0LKs4vrPik8fKjYojDq/W8Jo+7+
kpDu/8+qbQnd/J3U4SeTu+e4+vZ2fRzqGxI6kWNCymtz87n6th9MJpPJ5N6g
qydp9Xfp+HQ+lNdG9UBIrxJ+x18IP+xHN0ryZ9I87uoxcn+RTO49dl2XyT1O
6LWvCt3jhP/vnJD8VHScG6tNCUnvRPss0vvReD4AI//ba38uNi/8SbUNIc1f
3662KQz/1gVh/P1TQuo3HfWuTgvpfKhfAOnqY759SRh9aR4Q0n0OPemHOtP/
INV7oeNQXf34+ytCt18t9Xul99nV8dJxksmksm96bIojkE6VxmGKZ8W8Nye8
/UCxp4Q0frp6+ORksW/5Ba2+03HlI9DfPTddbEVI/YOof3qr+kJuHLNr0jon
1okXhVQ/n+L+tL6i+DWNq3Qc8vNQ/lQrnQA93679crQfpN/TvEb192JeOiuk
/lmhH3lGSDpDOk9a57v3LdftyWQymbwWuvN71/tTWt+++UaxZSHlzdF+k/6u
q6dN3Usy2X+28g9QHN/9u7RPcf35szPF1oTUR++9d4ttCEkHSPEs6itN9aCC
Twvjul8R/qjamrCVfp5+T8/dnRdofqHnS+8V+Q2mz1YTUl9p6otNf3dnf++m
0F0PhF9qWjgAi7ov9w9HSfXzqU5FrAueEJLfzB1nqP+Cu75y/24yuR/Y9b6D
/G/0e8rbJf8ezXeUf0R9T8jvR+uKrnWSyWQymdwb7JsOnOKerl7O/btfr7Yq
jHjgY8LYf88I3fh4qz4CqdtPJvszfrrHGZduinQX7v7orTeLrQipP+aXqp0S
xrh7Tkj186leU9SfeF5I5091g8Ov85JwAEZ1S6gOzP/8TbGrQnqONA9+pdoZ
IeWdfazagpCOT/6uyA94UUg6K+orR/06qa4L5QWQ/i3ei1NCyhegvgmt6neR
jovez1bjQDK5H9hqXqZ50PWzkf+f9KX/Vu2qkI4ffUVmhBTX+3K100JaD6T+
cG+wb/p8Og59L/Qekt+Avi+aZ+k4FD+l35MOmdYzEb9dF9L5960evku6/9F3
65KQ+oO47/O44iNd15ty8xTcdSwd350XqA4e1fGLdfpvhbR/dO8nxdGI7v3p
6n1KJpPJ5N7iuPT5tJ6kvqjhp7wkHIDF+uKR4Sgpn47+burrksnJJX2nrt/b
/X2rPFyqX0F1qEKnPCek/ewALPoqHx+OMvx8a0KqB/XxaovCON9VId0HGreJ
rfylbj0T2p+Sn9OdR6hu1UerzQspjubmlRMpj4D6dEfd+ieFpMOnPncUZyTd
PuV9uHkcFEdz+zskk8ndk8bzVutwN45A48Dc+WIbwhh3Twip7mKMi2eE5Pej
9QatWybdn59MJpPJthxXv3sizbO0n6L4OB2f/H60r6E44EeqPSvsWseSev5k
sj/sej/S6jxpX+D6zWicJL8f9fGk+h4DsNgPHR6OMvSGzwvd/onUb4X8lqS/
+l61JSHpUuh8aJ6i49B9oDobbr9I8puRrmbmXDVhvC+XheEvfFFIeRaUF0D1
TCiPg963hx4stjocpdv31q0L1HWd4WRyP3BS4vVuXIniDgduK/akkMbJGNev
CCn+lfuFvcFJ0eePqx5+q/tG+f7/rdoR4beqbQjpeidlfKP7RnGN8OtcElJ+
Ja1X6e9SPTo6jutfakX3ubvvwy3VtoXuedL9pONHHtiikPYvA7CLF6oNR0nn
Set29z3JPLVkMplM7oZ92zfROoHibnT+VI+F6oQMwFy/aN/uZzKZ3P13SvEm
Wle7x6HxzfXb0/6U9NhU74jqh5MOkPqFRf2hg8Kd61NtCGlf4+YXx7i+JPxM
tXUhHYfmnVZxSfq9Gw+l39M+94vVrgrpfY46zVeF9Lyo3hf5zX5abVNI71X4
pQ4IB2D0Hn6z2rrQHTeirtu20B2X6Dnmvj6Z3D0npW7M4UPFFoXRr+N3QtKZ
RH+Vi0IaN0gvkfH9ZDKZTHbBVvF39/ju/p2OT/sg+j3Vf6Z6zrQv6/r+u8+l
b30TksnktZP8D24/UKqv+P1qy8LvVLsgJL9ZjLuXhAMwqqsffqNloXvfSA/v
1pl05xfa57r+xqg3vyik/PHQ6c8J3boToffcEoZ/bh6o9TCp/zXp/6MOxwvC
224t9oSQ3ivStVI9E5o3qQ8C+Z/pflJ8IffvyaSy6/zTVsch3TvlhdFxot72
qnAAFvukR4ejfPONYsvCcelRk9eHfdPnd623d3XXreq8kX6J/P9U7869n137
Z1qRngvFQWLd/YqQxlXyF9E6n9Z7O/dH3v3fJdL6v+v5i95/V6dHf5d0LLHP
+KNwAEb6gRurLQjpfEhH0SrOmH6zZDKZTP4t+zYvuPp88kNSHemos7IiHIBF
f9GF4ShJl0jXRevJcd3nZHI/s5U+n9bhdBxaz7vxL6pzTvvQc9PVhLTviDr3
zwpjf/Oc8OZqi0KKm7z7TrF1IY3/pCcknX/095sVUhyK6ivS/aF4Cj13tw81
zWu0r3f9Qu53Qb8nf2/UOZsT0jxI13vrr4o9LhyARbzuwzySUWpeSfizXxJ+
qtqqsNX4Q+OA659pdT7J5F5iq/owdJxW4+0/VpsWxnx+j3AARvUYyY/aN397
MplMJvc23X33uOpC0+9pHU75xaTPpz711G+O5vFW+oTU5yeTSfJLkP+NvneK
v5A/MPw6U0KqYx+69ZeF1K+E6nW88ZdiSro/ri6L9Pl0n0n/456P+9xpHiT9
fPR7mRGSPt/VmVN9D6orS+8hvT+R7/6EMN7HXwhj//0n4QCM9HXuc6E4Iz13
eq/cv5tM7ge28hO2qqtD4yTtLyiv7f33im0KqU6XO77RPsXVM4/7+Sc9Too+
n95DN+5GcXk3H5Pmcfreo2/PspDyE6nOWNf67XG9b/Tcqb9S1Bc8JWwVx2lF
em/p/XHPn74jes/d96RVff633yq2Kozv6REh1bMiPT/tQ2nfQc+FrsvV82R8
P5lMJpPXQne91GodSOuKr1ZbFtI+dHammjDWuT8Xkk71/GyxdaEbfxz380wm
k/9Bd7/v+gHc8yG/1vGHi20Ko6/vceEA7B+qTQ1HSb+P/c3Dw1HSOExxPbpe
+r3bxzP0+OtC6tfs7h9b1Rlw/ULuPLKzH0P18FS/kfxON1RbFsZ3oP1Pd+5z
rfkmkX9wWRh5dU8L4/69KiT/DL3n8f6eHY7ygSPFloU/qLYi/OD9YltC11+d
64dksrtxmNi1zj/yyZ4XRn/OY8Ko7/h7Ia0HctxIJpPJZBfsW5yXzof26VTf
jOZNWldQfnrsv18UDsCobrCrx2u1bmlVf29c70My2We2qmPZ6rujcYPqQpCe
4WvVVoR33lHsnJD8M5Gv/ICQ9kGkw6c4TviHrgipHib5wciP10p3EXVAt4X0
e/q7VM+E6pPQ8d0+L9SHmuZZV99C9WN/WG1VSH1Uo27NivDj1RaFA7CYt08N
Rxl1SjaFNL9TPwXKf8n9fjK5e9K+oNVx3OOTDvPb1TaFNJ+Gv3BDSHlwpHuM
4z0jpH2Tq2Mc9/NPeuybPp+OQ+ufSckTeejBYqtCqp9P3zt9j7RO6LqfiEvX
D0P6fKrDRusoWt/S72m/QPumcY2H5C+i86fjuPm2RFrnv/dusQ1h1FM8LYz9
3xFh1NOdEsb8dlkYfbQ2he51kc6B7n+u25PJZDJ5LRyXPp/WmbcfKLYspD5H
lH9KdXqpzhX93axLmUxOLlv5B1rpqz9XbVNIdTMirvWYkOIFA9NI/x+6/vuF
Fy9UE47Lf+LWIXSfL9VVcONNbp4X7Zfdv+uej+uXpuOQv5fef8p/j/7SZ4Tf
qnZeSO9tfB83AX8pJB0OXZd7/1v5Q5LJvUTXf+sev9X+hcaB8CM+JaR5nH5P
eUDuddG8nLq7ZDKZTP4t++bvon2cq2938+tJj0r9vGLePiCkeCLdZ9LztNId
uc891wnJ5O7p6mHcfhatzsftF3zkcLElIeURU79C8jeGLumM8NjRYutC0m+c
PVNsSUjXS/sj2ofSfOTqYYhUr55+T/Mg1esgnQn9vtV3QXkQn6+2JZw5V01I
/a/pedH9obwG8qtTnJHy76ZOFVN+ttqGkPqxut9pMrmfSeNz3/Y1FFeivCca
n6PPyUPC224t9oQwxo/LQjrPrueF5HjZN32+27/ePX/aR9P8S3mUlGdH60Cq
90XfL9Wdo+t1dcLj0pO7ujLqV0j5R5QHQf0LaF9DejDKDyUduBvndevVu30o
6D2hcd79u1SXj/JhyW82Wlf3Q0Y+94rQ7VNG10v5F11/F8lkMpnc2+xbXjP5
x2gdS+s08u9R3d2okzEtpPXA668VWxDSur1v9zmZTF47XX8UjWMUd/hEtSUh
9e2ifUrol58TUj382CfdA7woDD/BmpB0C3R/aPwnfSP5Q1rpB2h/6uYR0LxA
50P7dPfvkn/J1Z+Q34bqCdB1Rd2CbaGrP6E+11Q/gfrfDcConhvF0eg7pfv5
8LFiG8KMryWTu2ereAGNb0Q6Dq0Hds4zOiqM8emQ8J67i70opHojdJ5uv91x
P+dkMplM7k123QfHPY6bV0vxvthPvyCMffl9QqoLHXHmbSHFB939Xavnlfr8
ZHJyx0m3fj75LaO+75yQ/Fex71kXUp0Eyl+OPpnP7ZpRj2FB+G/VrgppP+jq
NFx9CM1HtI+j50i/d8dtd19M50N+Ufe9ffedYutCquMR9WXPC6kedfRhOCOk
uOEAjOp4RHzyXmH8/VUh3U/SAea8nEzunq30veNaJ7//XrFNYeSTPSoMP+IF
Ic3LlKeW/sO9zb7p82k96a6HKQ8x6nhtCL9fbVkY9bwXhaTrfvCBYivCnfNi
1CiPz63zdr3fp/+KtF6l8w+d+5yQ+mdR/PqWasoYF7d2zZMnim0LSQfi1lVw
n6+b70B+JPd7pPtM63+qp0e6F+qPQPp/972i66Vxw31euT5PJpPJ5N+yb3Um
ia6+nfquhh5/WkjrgQFY9KWcG46S1lduXcqsT5VMdkd3X0P7FMoDcve/36i2
JhyAUZ/KqCP1pJD09lQ/nProkb6arovGbbo/FLfquh5g8u8j7a/dvAaa76jO
ANWZHIBRn9Y43gtC0s3OnS+2IaT+2udnqwnd+gPjfs7J5PUgzcv0XbTqw0J5
Xm7/kYi3bwkpn5f6dg3AqG8OzY+t+sVMij8/mUwmk9eHbl27vvm1SFfp6h4p
/y76bT4oHIBRHvHN1RaFbj002nfQ9br5wl3nBSSTfaDrP2ylZ6Pv0Y3juHXg
aR9BdQ5Jd7RznrLWDwkeFA7Aok7DueEoXT28e9/c/VTXvx8Xafx37z/pXuj9
JD0V6eji/l0Rxv57QUj9GqiOR/jX/ySkel/Rt+GMkO4P3YcvVrsqpOO4fnU3
PkLH6TofJJm8FrYab1vV23HjJtE3eEkY8b3HhVEvf1Ho9u+YlH4Eyb3BVu8V
zYOUp0n9aqmfBZHi6Tv349P+3dTfx/0e+/aduuNwxG0uCkOX8YqQ1i20zid/
Ee3jxlU/ge4bveet9BUUL3OPT8+R/GPRr+CskPICWq0n3fhg6vOTyWQyeT3p
xoncfjq0zqHj0Po5/t9lYdR7flYY8bR/F36p2ryQzt+tW9KK434vkslJYit9
oOvHpvxrGq/cuj1kse+fHo6SdMjkr6D9F13vf692VTju55/0SPWaWh2f9tef
rLaya5J/+6PV5oVUt43iv9QflvQ5P622KaR8/3E/52TyetCdf8kvSvMO6dNo
HqfzdP3GUa9lRjgAIz9k8Akh1bF06xO6/sZkMplM7k9Oip+K/m4r/QzVQ4v6
eaeFAzDKC56dqSakfbe7LnL9tHQfst93cj/T3b90fT70PZIfj44T9e83hd+t
tiCkOuTU/4t0+FQ/hPwtVIfhY9UWhFT/wX2+4+p72Le4j1tXk/rCuH+X5jt6
b8lPePhQsUVhfK+vCqnfzQCMfh96sUeEP6y2KnT39fS9kz+EdP5Ux8B9Xq3y
Irt6j5P7m7SudvWrrfTq5Je7odqykOJ6NI7R/EhxSfd6M78m2QW7fq8oLkDz
JuUV0u/pu6a+ObR+oHi9q8On+zku3T6tG3fuz35RGM/tVaF7XX1bbxPpulyd
Bh2f3hN3H0Tr/3ivzwpjX/gH4cUL1YS07m31Prv+rkl5f5LJZDI5Xk5K/Xw3
/47WdbROpvw7Wg9/tdpZIekJW92H1PMnk9ef9L249a/o9w89WGxVSP2Xp04V
mxdSHzeq0xt1epaEdP6ujpFIOmS3L/O434v9TtrX07qC/FGtni/N7+RP+FG1
NSHVvwo97CEh9eWhuvp0/m7cPJncS3T1LTT+EN1+lESqV0P+xhg/XhKS3j7y
g2eE1PeW6ge6cfxWdVSSyWQymfxbdu13cvtR0n6WjkP7CzqfmD+vCAdgocu7
dzhK2r/Tvob6+tF5unF8tz5/V+9TMumMA33ze9N32uo7In+LG/cJf8aK8B+r
KanP16CRUf4y6ZxDr3Re+N67xTaEdB9cv5Z7nHHpPdx50z0+keYv0oFTHhzp
ySl/hPo1UH5H/PsvhBEnfEoY9TweA94npPc8nsOV4Shdvbo7j7v6Pfq7FO9o
1c8imexi/KHfu7pH93xoP0L+xrsOFntOSLqF8P9dEr7252LzwkdOFrsqpPOn
dUvf1l3JvUGavyblfaPz/Ea1NSHVC3Xr57v9v9xxshXduqyk045x7mUhjdtu
npG7TnbXV+7fddfzbr19d14gPV7U87sk/OdqfwQ+LLyx2oKQ4ul0P7MeRTKZ
TCb7wEmpS0DrDVefQ8endQLV0R2APXCk2PJwlF37Ud31WN/6jCeTfSZ9L+44
Q/rAiC+dFsY4dFL4lWpnhNT/7vxsNSHVC4r4/5SQ8ggo35/uA+3Xxv2ck/85
W8XvaF9M+1z3PaHvkeJ3dPzY978gpO+L4r/0d6neRfiTtoSt8mKSyT6zVf3J
VnlzFEe+udqi8MO+96MMPf4JYeTVnRJSHK1V3NzNU8h9QTKZTCb7TDf+SMeh
fQT1faP59803ii0Lqf7zAIzi8nSerl6U9hcUnyVmHn2yD3T9FeNa39I63PWr
0Pfo+ifDz7EpjPHgivD4w8U2hVGvaFlI9UPIP3no/mILwl/9sti8MHTT60Kq
o+6Ok/R71z9D+003L6PVe+vGiVodn37vvrd0fKqXRf0rP15tURj5KE8LyR9O
+fVU/3MARv13oi7stvDCXLFNIfkhaT3T6j0n/6rr1826Acku2Cp+4Y5Xrr/u
Z9W2hDGuPCuMuMRvhd+rtiQkfb57vW5ecDLZBcelh2mlz3n7rWKrQoo/vvtO
sXWhO2/2La/ZPf9Yv1wSxn06JWz1fN3f03XR+orG267r69LfbaXP/3K108IB
GO0fo4/9qpDyeVv1eXHjjBlHSyaTyeT1ZKv6DLRuoXnQne/+pdqW8MjhYkvC
f6r2vHAA9vprxRaGo6R1V6v839TnJ5O7p6sDbOUfJh07xesjD3hKGOPHo0Ly
y32+2pbwE9WWhJQXQOPk7Ew1Yat64DkuXZ/74NZVaPVd0N8lf2+reCj9nvzV
VHcr9DIPCiPP4LKQ8nEibjwn7Op9Sib7RHdepnWsq3OgcYb4yWorQqrrOACj
urhR/+a8kM7fHd+orj75UV2/ZTKZTCaT15Ot8teIND9SfjHtu6n/He3fSZ9/
4nixLaG7P6Lf033ruo5oMnk9x4eu9Tlu/w46PuklKC+Gzof2C1RvPPYxi0Lq
g0l6mAFYXPerw1FSPzLKC6D71qrOg6vnp/HQPU4rtspfc49P7xvdZ/Ib032j
eYryX6gvLe3HQ+c+Jww90aJw5/o5Wlef/O3/+dcyGND5t3pPaD0T53VVSPef
xh/yz9D30nUfh+T+pDtft9pHkG6Qfk/jc+gXLgBfFNJ4RXlwdD60bsn1f7IP
bFUfnr47V99L3wUdh3QCVC80+masC928ob7RPX+qsxr39VVh1/p8Yqv6+eOq
++r2s6a/S3XtDtxW7Ekh6WFoH+rur9376dajmJR6yMlkMpkcL8c1v9NxXD+2
63d1/fDRV2pGSHE0qqvp1t9uNY9PSh2hZLIP4x7tg2j8oeOQ/43yfUifT374
8LcdF95UbV5I+/377ys2Lzz6ULE14ReqbQtjX3JZSPpGN8446X6GVu9n1+Nz
K92FG49z7wP9nt6fe+4utiWkOm+hh31eODCN4s6k/6frSr93cj/Q9QfSONN1
f5Zbqm0Lw494Rkh+daoHQvMvnQ/F06n+fyu9UDKZTCaTfaAb93R1rfR3aZ9L
5/OTahtC8gNEHvBB4Q3VloV0vV3r3Fydw7jfl+T+outXaaVboO/C9YPRet71
q1A93riOK8JjR4utC3fWyavFvue+4Sgp/yj0FwtC8jdOny22LKT70LUOk56X
27+sa/18q3mB7htd73+lP7rW+0+M+jFrQurjQPn40R/vSeGXqp0S3n6g2FNC
8ucPwKJe/kvDUZLunfp1tqr/6Y5vFB/Zb37+5HjZyt/l+ufd8ZzGH+qvQflB
MQ7NC93+15Qnld9v8nrSzTd096fjire69fNjnbIunPS4Ia1P6L6FLvuiMPwx
rwjd98HNs2h1H1rtT1vV0aXzpLgb3bfoQzUtpP71VAeSnqP7/rvX69YN7npf
k0wmk8m9wUmJm9B85+bN0fxLfR5Dx3dBOACj/TL1AyI/fNfr6tTnJ5O7/y7c
9T/ls5M+n+pdUL3uAVj8/SvDUUa/r3NC6gfn5keHfnlD6M4vOf7sfB+61lfQ
83L92G4dZvq7FG9qtS+meTmuT/u5R924Q8AnhPG9HhXGd39MSONG1o9K7ge6
uhe3fgXNX+7+IuLqy0Lqa//fqh0RUv4OzafudVGdfDrOpOwHk8lkMpn8W7rr
B9pfuHFGNx/wrTeLrQhpPRDxyT8KI59vRkj7Hfc86XrdfVbX/sxksg9spfcm
f4t7HMrbjTpDa8KPVVsQHj5UbFFIdSkHYPHvV4aj/Mdq08KvVVOSroP0OeTn
cf0qrepFuPXzJ0X3SPMsvYc0L9D1Ul390KdvC6neO52/6+ekPBHyA9A8Hu/p
Pbsm9eukemIz56oJz00XWxG6uv1Wfn63XvG43/fkZLPVuOr263TP55vVZoWk
1424wayQ8t3++kGxbSGN2zR/tboPyeRu6MYTW/XP6rqu6aerrQlpPfDZahtC
d//eSqfd9fOl35Oegfp5taof20o/3+r9cfNTiG79Cro/ly4W2xZSXIzyVeP/
XRTS/SF/l1svl66X5sfU5yeTyWSyD2y17nXXCe5896lqq8LvVLsg/EG1FSHV
141138+F5O+i+b1rf+y46jMnk5NIGseoDkzUvVsU3v3rYpvC118rtiAcgFEd
HorvU30/tx6XG79wddQUf3HjBZPOvunzaZ/r6vnd9QD9XSK9tzTPurqd6K9+
UTgACz3+L4aj/Ha1OWH4IbaE434fk8lxjnut/MYUV4rveFsY9fCWhKEfeUBI
fbUoT4fqRoaO4GUh5QvQvEn3gfZTNO6lXzGZTCaT18Ku/Uvu+oGOQ/tNdx9K
8XGq+0F1L6lOL9UZo/nardtM6wHaZ2Xdj2RS6epOyV9Hx6dxhnTLFNf4XrUl
IelbqC/Y2TPVhDQukX+G/KV0H0iH7z4vOs8vVrsqdPOPWvlX+zauunE9Vz9P
85d7H9zvjvwGJ44X2xJGfY7TQtLXkT9/ABb+gz8MR0m6pvj+loSky3XvP30X
7nvSdd/DZHI3dOMd7vFpvqPff7zaonAA9t1qC8NRxn5iSUh/l/yi9Pv9FqdL
jpet9ptuXi2t04i03qPj71w/X+ML36q2IaR1ad/qwxPdv0t+ldg3vCB0z8f1
n9Dz7bpfgxuXp3HbjfvQe0754KHH/z3wgDDys1eEmb+ZTCaTyb1Emn/HFUej
9aFbb4RIdWB+9ctip4QU/xqAkf9t6lSxeSGdJ/nN3HUI3c+MoyW7+H7d/lnu
+0bjlZsfRPsUN75Px7l4oZqQ6ld8udppIcW/Ynx6TEj1eb5abVl4S7VtIe2z
yG/pxk3ouaR/fm+Q5h0374y+L3cedPPlaZyJ72lOSPVqSH9LdQNinNgStooX
0PdIeTquXteNLyfHy1b5Pl3nE9Hx3fg+9cmi+Yv2Hbf+qtiUcAD2D9UeH44y
+tDOCkmHT9dL4wN9j66OJft3JJPJZLLPdOPs7u/d/S/lwR1/uNimMPyCp4Qx
//9JSPsIt2+O20eAjkP1Csb9XiT3JruuQ+gen35Pfgb39/R3SYd/9KFia0LS
95JukPR+dByq103jobu/oPGZmPqN5G5I3x3Nj+RP2Lmf5kEh1QEg/RLVAYt8
gnuF0V9iRejmAZH/wf1+33+v2KaQfp+6336y1fzbqr6fW1+IzsfNSyU/G/1d
On/aF0T9j6eF8b2+IqR6X0cOF1sS0n2m8a1VXmFysjguXZA7Drg6hK792+7+
gubrWEf8TviZautCd98xrudOdJ8L9c8iHZercyBdFq1P3Di4G4925006Dp2n
m0dJ8bJYn84LD9xW7Elh7F9fEl6YK7Yp7Po9TCaTyWSyz+y6viXR3Q/S/pp+
T/Es6jP19WrnhOFXOyyk+vy0HqN1Tqv7mXqbZBfjgLtfcPXzdD70e1cHTvEj
+ruhp1sXkn4++tw+JCR/PunzQ983JaR+u1SHytXVu7oF971q5U9IXh+6dcO6
Js37oYtfFVIeTfi/V4RU35K+d4pfU3/q+O4XhRQ3p/Gt6+firh/oux7Xe5IM
Tro+39VdUB4K/Z78seH3fUl45x3Fzgmjjt0FIdWrbJXfRHTzdnO/kEwmk8k+
010nUN/trvWcEYdcEg7AYt//y+F/TvUPnJsutiKk9Yxb/5bYt/1gcm9z0vX5
5H+j37v7F6qXS/37ot7klpB0gLHvOSEMf8m08IZqy0KqjxT630vCzAPaG+yb
X9fVI9H3SPt6ep9/XG1dSPolqqNL64GIh2hfXdI7Ud0e9/6Qbq3r+uTJ68Ou
/XutztPtX0PzcqvrpX6XN1WbF1K8IOIJs0KqQ+jWw6d4KN03ilNk3a29wUnR
57t/142jufoBmgdJt0B1wqne17GjxdaFkz6fuuNtrL8uCWn/4r4/bt+TVvkj
rr6O/q77PrvrPdqXxfm+KnR1LJP+PieTyWQyeT3Z9frZrdfq9iGidQvp80lv
MwCj/konTxTbFrr3wb2frn8s42795KT4t939gluHgfbFrepskO43/FsXheQ/
H4BR/3r6fewLLw9HSfUu6L65fQ3cfaX7/rh1zlu9t8m/j+585H6P7vhGeW2k
V/mnas8LHz5WbENIx6H6llTnivJxqD6/+1yo3yWtZ9x8QNL50Hos1w/9ZNfx
tVbrk1Z1schPHnGtdeEnqi0JqU+WG0+PPjZnheRvd79TGp/d9di439NkMplM
Tga7Xg+4dOdHt44ZXReth2l+jzp508LI439WSPsLim/S/O7ur8nfQsfvm58q
uTc46fp8V2dO+xfKr6G6QKTLpTre1JeT6hGFrv9F4ZnTxRaF5A8J/fJJ4Ttv
F1sT0vjcqv9ysi37Ni+48zgdh/wSP6q2JqS69NRfj/rm0HqA8vturrYodPOA
SLfm1hVx47bJ8XLS9fm0Xm2lz6d5Ob7XZSHV86G8mzjfK8KIT14W0nl23Z+i
b+N88u9j3/T59P1OSt0hYtT7XBXSvE/xPhr3aJ3s/r7V9br3n34fOpHLwlh3
vCJ0dQvuOrBV3wf3OO4+tFU92J9V2xLGc3hEGPvFM8JPVVsV0t8d1/uZTCaT
yWQX7Fs+mltnppXOhPxR1G/u7JliS8LI/3tcGPvso8LZmWJrQjpP2ke7ccNW
Op9xvSfJYN/i4C7dPFw6jls/3z1P6h/35WqnhbFvfkpIfjYaH8jf/uADxVaE
dP4UT6fvmu6nq6N2xyU6/9QN7g26OpBWeX/UHyfe60eFh+4vtiCk41MfjdDv
TwsHYNS/kvIB33qz2IrwY9UWhF+qNi88cbzYltC9n5My7ySDk6JvofHfrWfy
wfvFtoSUp0N9Meg7HYBRn/qoizMtfPedYutCui6aZ911lNs/KJlMJpPJv2Xf
/BLu33XjpLQOcXUvv76r2Ibwo9XmhTvr8U4J6e+SP5DWA+S3yT47yT6MM62O
07U+v1X/ux9WWxWeniq2IKTjkK4+9jeHds3wXz4nnDlXbFVI/ofIA9gUUr2U
cb+PyfGw1ThA+2ia391xg/RjU6eqCQemURwh/A0HhPSd0v2hed/VrZFuOevz
TBZbfXetdIluHQ93XnbzSijvhvQD9P1SntpttxZbFEb9vzkhrfPdemi5zt+f
7Js+n75fl7TepvVA1/VIqd93fJf3Cinf362X5dZv79t7GPUILwl/c0+xl4Q0
HtJ7QuOkexz3Prvvrbsudd9zIvWNinjaMWGsQ88Ko4/9lrCV3i+ZTCaTyT6z
b+summdp/UO/p3VUq33lznmC6n8bgL3+WrGF4Sjp75Kf0F1vu/60ZD/Ztzi4
Szf+S8eh99/169LvX/tzsXnhX14vtiikPpVRD+9pYdTfvSiM69D6GNSHmp47
xbPoelvlO7h+knG/j0mPbj+FVnlereq93FhtQXjwzmJrQlfvSnXqvlltVjgw
7cjhYkvDUf602qaQ8g0fOVnsqpCui9ZjkzLvJINd61vc49B4QvORm3f2+Wpb
wjvvKPaMcABG9UDIf07rhIibbQrdcZL2O63yHK/1PUsmk8nk/mDf/BL0d919
hJsH5+6Xad9E+h9ab1A8lOqQ0PlTHq57f/rmZ07uDU66Pp+O4+bLRx2gBSHV
mf9ctU0h5fvQOBP1hGaEP6i2IqQ6n66+gvS9NE6mfmNvs5VejuYvWifQ+0bH
eeBIsWUh+QMjP+95IfWboHojkdfysHAARv7PW6ptC93nRbrlcb9HSY+Trs93
87zou3b95xQXiPX1JWHMt68Iqd8N6fb/rdpV4c75ROpXdHW/Ge/bG+ybPp+O
487jbn49jScUj6Dj0Hfx5hvFloWRr3NQSOv8/ebPJ71E1CE7Jex6XGqlN6P3
uZVOwyXp5yP/e0pIupeY9+aEbvwrmUwmk8mkkvbFXa8TaH3i7ivJz0brbSLp
4qhvXfjJ5oR0fHd/ROvPXP/sDfZND+nGuVwdvrtPoePTcShuRf3iad9Nev7I
g39Q+MlqK0K6D+QHc/fjdB9avVduflDG0SaL7vNqpdNw569WfirS99I6h3Ts
VGeA+ldSve7Dh4otCt062K6fh66L/Jy53ugnJ0Wf7+bx0XtIx/l0tTXhzn3k
fyuMdf0vhBTvpvtA4yrF11rtF7K/fDKZTCavhX3T57s6H5p/3XxAOh93H/TZ
ahtCWocMwGI9sj4cJf1dtz8R6Z1Sn5+8nuNMq+OMS59P+TInTxTbFkb+zXlh
fGdXhNT3M8aP+4SkC6K6ItTXj/p+un2B6T6M+31Memz1fXW9rqD3kPLd3ONT
nn4c75IwvsvHgNqfN777V4W3/qrY40Kqa0r1Pei6aByj9YCrc06Ol63i+11/
v/S+0bxD639a37p1Rb5bbUEYdfTPCEnfSHr+mP83hOS3p/tGeTRu/edW8Zfk
eDkp+nw3f6dv+9NPVVsVRt7t/ULyD7TKWxxXvwz3ucR1XBFSflPU0d8WUv4v
jYcUl4n6/cpWeQGu7q5Vv7ZvVFsT0vv51WpnhVRfLvuzJJPJZDK5e9K6jvxX
Xa973fhRK9I6jfbFweeF4c9+SUj1+d3zJH+CG5fM/XVyN+OAuy9wdaS0ryG/
E41L5Af+SrUlIcWb6PuK70Pr2AzAvlPtwnCUbtyf/I2t9umu34OOQ+NMq7ro
yetD9/2h5951HQP6jlwdLx3/4WPFNoSxLnhZSHE3+v0ALNYLJ4aj/Hi1RSHV
v3Kvl5h+lcliq/G/1XN342h0HHecIb3HR6o9JPyXaseFpJe7udqikNbVNF65
8TUijXvkT87vOplMJpN9Zqv9KeWpda0nJD1b5PPNCO86WOw54QAs1i8zw1HS
+bt59K3WJ8nktXzvrY7TtT7fjZuErmFZGOv3y0KqZzgAC9390eEoaR9ExyH7
WrXp4Sjd50XjM+3jWumok23ZN/8D0e1rQ/trihdQf73wK7wopHpfVB8g6nmc
Fkbf3ReFUYf7sDD+36LwvXeLbQjJn0D3+a8fFNsWpv+hn5x0fT79XfKHkw6T
/i7Vtd7Zf/hLIc3j52erCUmfQOt2t7/AuMbh5HjZN31+1/XJ6f2nee3SxWLb
uyZ9p9Rnh+blqOu5KaTrdeMj49L/0LhE5xn39aKQ9BiUB0H3/42/FFvaNake
QqyztoRuvUr3O6L75tZVuP1AsWUh5XFTHtm77xRbF9J+jc6na/1AMplMJpPX
k63Wt13rP1vp+tz+jLQupXraUWfiKWHo5v4EvEf49lvFVoXk96P75vqlc9+d
3A3d94TW1W5+DY0DFNem41Ach/Z31Oed8tapv/y56WIrwu9XWxZSHjddl3sf
3O965/Fw9/pety739X6/k7ujO7+7+SbEVnUD6Pj0PtPfnTtfbENIupofVVsT
7tzP+pCQ/CGvv1ZsQUjXS+MqrZda9T253u9r8v9k3+Lj7jrB1d39a7VtIfWT
pXp0d95R7JyQ+sjT793vyO3DS3EE6i9G48C439NkMplMJncirUNo/dlqXer2
93Hj3eQ3CD3OupDq5ZLOltYDdD9bXVcyeS2cdH0+rdujPseWkOoQ0jgQ3/Fl
IdU5HICF3+LZ4SipzsAN1ZaFbn8u6mdKv3f1k8nJopt/5/ol6DjuPp38DO56
4wvVtoXUf5Pqe5AOP/wTfxSSzj/8Fs8IqV+w689367+N+33c72w1/7p+e3cc
oO/XHR8ovkbvJ+XTxTz8iJD6WVB8kHQI7v1382jofmb/zb3NvunzXf8/faeU
F0b1NqkPFOm0qe4f7esp33YARuv/VvdtXHTrMpF+g+KqdBy3z4KrG29Vx56O
30rnT7r9yAs5Iwx/z6tCyj99/71im0J3PTAp73MymUwmk12wb+tzWs/QusLV
A5Mul/o80vo84mDPCgdgsT++NBwlnSfti8lfTfeT4nTpH9vbbBWfcvs8unpy
ykOn/QjxJ9U2hOEPWxBSnkvs884L/59qzwhvrLYgpHj6P1ZbEWa+cLIPHJe+
l45D3zt9v7ROoL9L8yyNbzvXAdC+8wOwyPM7MBwl1YWg+vmUF0D9tek+uHHw
HK/2Nruub+PqxOg4pKOLeX5KSPHrARjtC9w446Rw3O9dMplMJpPXg636xLl/
l+qMfaaaMurozwppPfPNautCWrdQ3Dzz+/Y2++b/71qX614v+QHIr07HoX06
1deN470iPHBbsSeF8f/uE1I/jg/eL7YldPV4bp9TOg6NM1R3pdX7mRwv+zb+
uPVz6D1364YRP11tTfibe4q9JKT6AzF+3Cuk/oDhjzwidP0z5CdslTdB47A7
jrnzUerHguPyL7XSK9I8eOJ4MSX1udj5O/q98LZbi50WfqPampCut1VeQ+bR
JPvAcfu/r5X03T30YLFVIa3/qU4gfY+TonOedH0+7Qfd50Ln6dajcN/Db1fb
FNJ6MvI+Lwi/W21BSP0j6Pz79n4mk8lkMtkH9s0/Rn7aVvM4ra/cvEXqc0R+
cvK3u34kyi/IOtV7g62+x671+a4+0+0v754n6WNJp3rxQrEtYeQNPykcgB26
v9jCcJSHDxVbFP642rrQ1RXn957sgq3i2q1I4w/pcmlcou+rVb/X8GdsCykf
J+pvXRQOwKgPJvX3CT3OrJDihqnXTf4tu34fXJ0Yvbexvp4HPiakOiFfqnZK
SHUvxx0HSH1+MplMJpP/QVdP0iqPmH5Pea83VZsX3n9fMeXXq50TUp7v9Nli
y0L3PvRtP5hsy775/928V/fvthofyM9A+t7TU8UWhPSdfrTavJDqXcf+5ufC
AVjsb+aHo4z8nk0h3R8aH8gf8rNqW0L3+aZucG+wb+OPW+/LJcXvor7XmjB0
u08I47ucE0afz/NCqidGuv1/qXZBSPVJWtXZdvuDU7zYzSt048Kt6lQkg3Q/
Kc5FOkA3Lkb+f8pvHZhGfW0ir+6MkPr20vm78yD9PvWTyT5w3P7v1Od3y0nX
59N6wNXDuH4VIq1biFSvkvoxxXpzUUj7WXo/aT1G5zmu9zOZTCaTyT6wb/4x
d53jkvT2rfz2O/vTtM7V/1VtUUi6IPd8sj/dZLGV36+VLot+7/o/3X0Zrdsp
vhN5wHNCOj71s6A4F/WpdONZ5D/MOnXJPrBv/hz3PEm378Z9iK7+f3am2JqQ
+kdTvGwARv69qLszI6Q+3XTf3DyIcb8XSY/j0m+7cc+fVtsU/uqXxR4TDsD+
32ovDUc5daqakOrnUxx/3PGB1Ocnk8lkMvlfk9a39PtW+yPy75Hu6O23iq0K
Y/1yj5DyEL9QbVtIfolWeulkP9k3/3/X69Wu9ZbUx5Pybmj82Vl/ohb6iKnh
KCkeEbr+c8Lzs9WEoR9cElL9Yff+u/6HHH/2Bvs2/tD72SoOQvHHBx8otiKk
uEOsC04KI354Wjgw7dx0teEoKS7j1i2JeMtVIdUtp/gR1c8nfyzFed264q3e
z76xb+s9+r5a9Xmh66V6INRHm+brM6erCeN7vSSk74Wuq1V83x33JiVOlJws
jtv/nfr8bjnp+nx67m48lPY1NN+5+nxaF1E/tYhLzwgpTv3DaqtCul56b7vO
h00mk8lkchLZN/+Yu550/S20LiJ/Du1/af1D/u0BGPUVmjlXbFXo3n+Kx7V6
vsm27Fqf3+o8aR/h7r8eOVnsqpD0tFF/YkH4/WrLQnr/ox7MceEALPR688NR
unXvaTyhfYrbbzeZ7PP44/avd/v90ffYqk4m+UPI/xb18p8WUjw9/P3nhFQP
nOp+f67appCul9ZFbt5icrLofnetjk/zHc37n6+2JaR+VeT3Jr8u6VUojkzX
O+74QOrzk8lkMrkf6fbldI/v7jtaXVf4J7aEVE8g+nn+QvjuO8XWhVnHbH+y
b/5/Ok4r3WCrfhC0X448mCUh6W+jjsimkPJu6Lu+MFdNSOND9Nn4vZD6dHy6
2prQHVdpP+XGfbJ+9d5g38Yf930m3a+rO6I+O3fcXmxFGHk2p4XkP4y+wS8I
SZc1AKPxjfoCuPlQ7n2j47jrGXruOc7sTNc/3/X5UP2ukyeKbQvpOHT+MU8e
EZJ/Pt7fV4RUP8fV24dedVvovrfu99I33W9yb3Dc/u/U53fLSdfn0/HddSP5
i9xxmJ47xcsGYJE3tjgcZdzX80KqI0HnSd8L6eiu9T1LJpPJZHKS2Tf/mHt8
8v+QPt89vrtOo754se6+SUj5+OTvIv0S+bVaxSWT14eTrs8n/xJd1//+a7Gr
QqovR3Eo+j31cd45/vUH4cE7i60JKb+A9i/0e7rPNA5c7/cyuT/Yatxopc93
1w+uf4P8JO76IeLYq8IBWOTbnRuOMq7vipD8eHF9jwqp3ibVIadxu9U6KtlP
dq3Pp/U5zfu0jo06H+tC+i4GplGeS6wvNoV0veOOD6Q+P5lMJpP7kV37u1rV
3yZdDfkNwj+xLSTd3QCMdAK0zm/VbzTZT06K/7/rvBuX5G+M/ce8kM7/xmoL
wvh/vwVqPvLHqy0K43hTQtrvRJ3sFaF7f6hPH9038gvlOLO32bfxx33f3Pij
qz8nfybFBajuR+joDwO1zw713aB+hdR/8FvVNoTvv1dsU/jeu8U2hHTf6Hm5
7xXNI33TN/aN7rzsfl/0HOk4rv8/4nvrwu9VWxJSvlusu+eEkUd7Qfj6a8UW
hO56hvIO3PVVq+cy7vcxOdkct/879fndcq/q8936+XQf3PUM/d2zZ4otCWn/
+LVqK0K6/3Rd7vyb+vxkMplMJpV984+5+r2u74+rT6P1xj9VO7Zrkn6J6nVQ
/Y2u70+yLbvW57vfO/2e/Eit4rlUN+Yr1ZaE1Bcy+rs+IRyAxb//bjjKt94s
tiJ095vp10r2mZOSH+SeP60r3LrxNL6FP35TGDriWWH01VkX/qDaipDia5Rn
FPW/nxFGPO2UkHTItJ7JulJ7g13nz5K/jt6reH8vAk8IKZ/uy9VOCylPlurR
UfzL7Yc+KRz3+5hMJpPJZEvSeobim+Tf6LqvkHuc++8rNi+Mup4HhIfuL7Yg
pHVa1v3Y2+yb/99dr7q6CPf45B+g/rbRH29aSHV7os7Hs0LSn4TO/aKQrpe+
a8r3obwhug+0P3L9orSf+uD9YlvC6/2dJLth38afVvMaHZ++C7dP3w3VloXh
/3hOeM/dxV4URv/hx4Wh978kjPHsAWHELZ8XUlyS9GOUZzR3vtiGkOKk7nuV
/UODffPr0v2n74jWA5T3ceC2Yk8KB2CUt7JzX6orQsoLoPtA77Orb0wdfrLP
HLf/O/X53XKv6vNp/dBqnUn3jfZxlC9J9S3JL0R1Ken+uM/XzVdNJpPJZHI/
sG/+MVp/tvLDkx+Y/NhfrHZVSPUraL9MOr0BGK2X6Hrpumjd2Ld1ezLY6nts
pcty49ruvpW+o1t/VWxBePzhYptC8o9RHarwB58Rhq7/JaFbd47uJ40z1BeD
8nGyrnWyD+NP1/peYiu/ikuqM0/5QVEvZ0tI65xbqm0Lya/ixheor33olxeF
bl/g5HjZt3gKvee0Lr2p2rxwAEZxZ9KtUd3IiLMtClvd/0nhuN6TZDKZTCa7
IK03Ws2DtK+n/QL5T2i/T8eZnSm2Jow+nP8upL5d434+yfFwUvz/dHz39/Sd
kj+NdLNnThdbFJJfMfLpnxSSPofylKnuNNUH+OdqShpnXL8NHaeVzjnrA+xt
Tsr448YT3fef/P/0+53rD2i8g+oO0Toh6u4vCEMXf174kWozQrou6ocY/29N
2KqPMD1HN86yV/0n7nw6LtL3RfpAet9IPx99q04KaV4m/T/F/Wk9T/ELeg/d
eEer/h3jfv7Jvclx+79Tn98t96o+n66Lxk93/UD7UKrzNgA7+lCxteEoaV1H
5/PXD4ptC8nPRsfJ+SWZTCaTSWXf/GN0HHddR79396fu+pz+7oMPFFsR0vo8
/NCvCFvV7ezbuj3Zll3r8911NX0XVIfq5mqLQvJrvfGXYktC6sdK50/7F/f+
u30nu67Xl0x2Qdcv5I4/bp6+G19w6x7QcaiOE/0+/P1XhRRPpz65VA98ABb1
se4fjjLynx4XUr97N18yeX04Kes6+k6/UW1NSPXzqf4b6dCmzxZbFtJ50nqD
fj/u+EDq85PJZDK5H9m13r6VfomO32pdHfU4jwrJH3LxQrEtIdW/nZR1ZnJn
9s3/36oOD/3+f1T7X0Ja51O+POX//qTahjD25Y8Jw6//CPBeYeyHZoTUX+/r
1c4Jye/h7neI9HwfOVnsqjD3I/uTfRt/6Dg075NuvFXdVBoPTxwvtiV8841i
y8LQ3Z8TUl2CqH9/XhjrlFeFUUf/aSHVTaLrpXpo9Htah9A4RvFfYq5z/j7S
++9+d278mub3h48V2xDS90j6z/AvTgnjfbkspDg+XS/FNdz8HaJ7nKxvnLye
HLf/O/X5/XruUefsojDiny8KKc/O/bt0P79QbVtIx6fx3F3nEGk9FnxUSPHc
w4eKLQrd63J1esRJyRNMJpPJZHI/kOblcfnxaL1Bfjna78e6e144ALvzjmLP
DEfprn9SR7c32KpeOr0Prt+MjuO+b1HnaU5I/uS33yq2Kox6MIeE0Y9rRkh9
wVz9HvmvKL+YjtOqT2symVSSHqaV/8r1J1C9/fj7rwqpzlXwGeEdtxebFpJu
h+r7Uf4UrXOIre4zzS9uXyQ6ftd5UnT+7vtD95P0D3T+FM+iv0v9a9x56mfV
toTUN2cARrqUWF//XPijamtC8uumvj2ZTCaTyb3HccW1aZ35zWqzwh9UWxHS
8aPe7ZSQdL8/rLYqpOPTOpbWRe7+i+6PG9em89mrftFW+5RW617a17TqZ0H5
I/R7eq+ij9a8MOoRzgK1biHtI6hv13eqXRDSPoV0gPS9k59z3O9pMpn8+8dz
GsfceZmOQ32EKc9oAEb65NDvTwvDT7kidK+X9Pk0H5H/8IP3i20Jx/XcW/md
6PhufJnum+tXdEl+zpjfXhLS+0a/pz6zocdfEFL9HOoH4fqlk8n9QBqf3Xx2
d5yk39P5uPGat94stiIcgB07Wmx9OEoab91937jq/rn3M/KHLglj3/OK0NXJ
0LqC3jf3vrn15Mm/EflhG8KYlw4I43y1L8x3qy0IW9WlpPtJv5+U/JFkMplM
Jvcz+6Z7oXUj6XKpLi7lJ1Jd0AGYW8+WdL97NT416XT9qO7xW+3L6P2n86T6
FVSnJepGzwkHYJSHTrr9+Pf7hOTHc/urUl9m2j/S79335Frfv2RykknxhVbj
J/kTyJ9D64fPV9sSUl2IAVisL14ejpLWG6HH/4OQ4hGh918VuveN+jNSHMf1
a7nPyz2OWy+F2Gr+da/LzQtw14duHiv5n0f7sn/IWBffI6T3Nub500KqO0fn
Sfeh1fuZTCaTyWSyP3T317SuIz8JxWEpfhp1r1eF5DeI870ipPUV9Rui63Xr
D5Bezo2n0z6L9nduPb1JZ9/0+W7/TdrX0HdE50O6yvfeLbYhpHx22l/T9U6d
KjYvDF3es0Kqx0h++y9XOy2kv0s6xnG/p8lk8u8fz2m+c/tffKzagjDiLQ8L
Sc8c//6CcABGdUK+Wm1ZSH1JXD+Mqz/v2o/nPvdWcR+6Lopfu/1hiTQf0fxO
6zc6zierrQjnzhfbEFI/qQO3FXtSSHWw6Tsiv3er9ySZ3A8cV75Sq3E+6muu
Ckm3QPMdxSPcfuXj0kW7f5f8GNTHxM2boPvp5k+5dfjpPOn3ly5WE1L/Naqj
dc/dxS4Iad5376d7XdmfJZlMJpPJ/cOu/Sf/Vd7nKP+12raQ+uSGv+oXwtAN
XRKO+34n29Ddh9J+x11XE+n4rp6c8sejXtSWkNbtoZd/QLhzH8ndG/nZ3Pig
O/6QH7LVc0wm9zNdvTF9762+R4orUd282w8UOyuk+ELUHX9cSL8PvcBDQqqj
RfUZKB5B/h+6PzQeEul50X1243pdj8M0j1BcLPI2toSt4okX5optCt3rouce
evlFIenwaf3wjWozQrcPTs6zyWQymUxOLsdVF45IOjryn5CfkOqE07rx7Jli
S0Lqw0V6vOCmMNZZa0LS27t6b7pvbl3W/ZZH2Uqv0nXfKFpvu3UwiKTfo797
5nSxReFdB4vNCimfJfZhx4Whlz8ljHokG8IfV1sXUn1pt2/a9X4vk8mkT1ff
ReMz+cdCJ3hOSP0HQwd3vzD8N08ISYcfurAXhbQucvMOaP1w8kSxbSHdT/c+
d60X7Xr+dX8fOsFtIR3H1T26eQH0/kS94fPCARj1ibix2pQw8uQWhe46xH2O
yWTy2sc92rdS3NDVY1B/Oop3UD1DV58/7vs9Snf+ojhjjMenhK3ienQ/Kb7p
3v9bqm0L6fe0ryQ/D/ltSG/Wqo+ku47ab/UTkslkMpncz+zaL+3qlun3VJ+W
4mgRH3hOSPm2tP5x60gkrw/d/UurfGpaP5Ofk/y93662KZydqSak30cd3GXh
ACz8wb8bjpJ+T3rX6J92VUj3v1V9jNy/JJPd0Y3vtBqHye/njrcPHCm2LPxK
tSVhrBceFO4cRzsoHIB9q9r54SjpPrtxFreuvtv3010fuvNp131w3Hmc4pLu
36X7TNf1qWqrwrh/rwqpHgjp1qj/aawv1oT0XkXeybaQ6ty6+T7JZDKZTCb7
w1b59a3orivIjxf5lHNCd50feoGLw1FSfiXV1XfrSJDfw9Xnk58qdcs7c1z1
Kl1/F63b3fM5PVVsQRj6O2V8B38Skq419t/nhNTnK/SDzwvp/Om7o/0L5ftc
7/csmUwyW9W/pbgk9eN24ymRz3R6OMobqi0LIw/pmJB0ie59c3V6dJ/dOlR9
y4/r2g9J/l76u1T3g86H1odUDz/mSWU8t3uFtB6Oul9PC//zt38woP5WdP7u
fU4m9wPdfaLbv5j217SPpnwi2r/Q8SkOErqGA8LPVtsQ0rw2Kfp8On+6b1SP
NPiy0K3H7s6nlG/l+m1on/vmG8WWhfSe0P0kHf677xRbF7r3wY2Hdn2cZDKZ
TCaT/WfXfhJat7t/9/vVloXhv39GGDrip4TU12/czyHZLd19Ab2ftM53fx/1
VBaE9N7+39XOCKm/JPl1KS5G/jf6vtx6IBQvdv0JdPyu++4lk/uBXeejdT0O
U72FqNu3Kgy9wAkhxd1i3D0spHpZFHej66JxkvKhSFdA97Nr/+Sk6POJlEdG
9SppfUu6LKqTH/0Z5oUDMMofCT3+RWHE2S4L3Tw7N7+j1XuVTCaTyWSyP3T3
767On/YLrj+B9FG0LqK6c1THL+pzTwvPzxZbF1L8ulVdXCLdf3cfkQyOS59P
v6f1OenM6fui/nQ3VZsXfrPaupB09ZQXT/rV+P7mhbfdWuwJIfkHWu0rk8nk
7tm1jshdV5Cfk+Zfmq+jzscZIfX1jnrhp4VfrbYsjPHykJDymCgPi8Y9V1fv
1p3oOq+zlT6fzof8e3Tf3Lr35K+m9eSRw8WWhKRjpL6ZoYs/J6R6IDHf/lYY
+senhOG/PCU8/nCxTSHdn9TtJ5O7H4dbjc/0d7vWC1GcjsafqHe1IZx0fT49
Rzr/eJ5XhOTHIH3+zse/9n0TXZd7HNr3kU7mZ9W2hKGLOS9081PoPN330H0/
cz+bTCaTyWR/2Le8OVpX0PqE4kFUJ5z2++EPOLJrUj3bL1TbFk7Ken6/0X3/
3XWsW3eXjnPieLEt4ZeqzQspHzz0nlPC2G/9Uuj2Z4/v6TFh6AdfFn6n2pbQ
jVfS/aT4u+v/TCaT3Y23rp+Hjk/jBvXNoboHFEejvrrkz6F8pRh3HxeSbif6
KJ4RHn2o2JrQfS5032i+ozhOq/oMXcdT6Hyo70D0t9wSuu8n1dX/XLVNYeTJ
nRGSHp70+fRefaTas0LKE6H3n+7bznX4d6/nz742yWQymUz2h33Lp6P1atd1
3j5fbUtIernwl/xRePDOYjPCVutht14i+U/oOG69/Uln3+r3Et33x/096Q3i
O7ssfOvNYitCeq/I3zgAi3320eEoP1NtXUi6neiXOyuk+0P7u/T/J5Pdjbfj
0u2Tvpp+T36MO+8otioknXP4b14SUh0k0uFTHadYd6wJ6f7QdVH9JbcuE+nc
utbnu8ehcd6tR0H3k+Yd8qfFdVwRUh4cXRf1u4k69w8IKa9kAEb+yZjn54T0
3bn1ot3nm0zuJbaqY0/Hd/OV3O+U9vU0n1IfLoq/kD/BvQ9dx7Pc+0+/J31+
7HueE97962KbQtJfxfitJB0I6eHdOgARL10SUn73p6utCWkejHjvkrDVd+fm
ibv9jFKfn0wmk8nk3mPXdX7cfEnSBd36q2KPC0MvfFJIffdI599KL5e8Puw6
z9T9Pb3/FOe95+5iF4TvvF1sTRh+sud2TfKbkV+O6kJTvwnaj9N+lsYZeo5u
3ZK+5Sslk32m63fqOp5C+XHTZ6sJQ2d9XPiTahtCdzyPOpezwoj/3wTUviSU
PxXzwFkh+Rup7pCrf+463uGO88RW75s7v9BxSMdOvyc/4V8/KLYtpDpsMQ8/
KfxatWlh5KGeFZJf0c2rbfWeJJPJZDKZvP4cV9y51XnSOpbi/m4eK+V1xnrq
90CtI/r2W8VWhVTnltbztK4m/yrpjcelk+wbJ71+Lx3H1b9R/d7oG3ZJGP7F
deGbbxRbFg7AKL+Y+o7RdVEeNNVFpPsc/tFt4fV+L5PJSaZbL31c806rfjS0
3gj/zZLw49UWhVQPhOrfxr//QRh67Vkh6ejo/tO84+ZLdt3XuGt9vlv/ltar
rv+Q9JPuup3WgRQ3p7gh9ekmv/f3qi0J3e/IfY7J5H6g68+n49D+kXQ4c+eL
bQipriD9nure07hBebg0TraKK42rjgGN83Q+pCeJ+3SfMPwTy8LDh4otCqOP
mfKBI8WWhXScn1bbFJI/h3QpkbdxTkj3k/rIUB4BfRdunbpWfZ26ft+SyWQy
mUxef3bdZ5Do+hMiDnBVSPXwqf9jrEMfE1IeLtULyvz9frJVHjTt49x1MtUf
uzBXTRj+zleFpNukuPAAjPSrdP5UF+V//qbYVSF9F64O1s3Hd+u8ZX5xMrn7
8ZB+3+q7dsdt8iNRHSHyk7h6Hro/1GeEdPj0e6pTFHUdpoSxrnlCGPkKy0L3
fXD9vfQ+tFpPuvlfdBw6f9LJ03Hc66K/e+xosXXhAIzyQWg9TH5IOk9ab9Pv
qT/F++8V2xS6zz2ZTCaTyWT/2apPN5H8lq366/1btavCneuBvyAcgF28UG04
yhuqLQt/WG1VSDorui66D7Rup7yAvvVZaMW+6fNpX0P6wJ3r3itJb/D9asvC
nfX2ev4773O1Hj7V1Xf9DKSzpbrE9L3T3836+cnktY+HXeu0W7GVH4Pq2Ec9
pBUh+Q8jD+leIfkPSbdP/RPddQX55ULffVVIx+mbPt+df915iurPRF2NFSH1
rf71XcXOC6meDNUHpv7alE9KdXLovaX3hOZl12/vPt9kci+RxiV3PKR65tSv
iuqZ31xtUfiX14spjxwutiSMcVTrwA/AqN5713WfuqYb54r8iYvC0NGfFrr6
Fprf6Txpv0nvGz130r249fy/VG1eOO7nnEwmk8lkcu+wVR4osdW+mOrY3HZr
sUUh1cmh38c68ZKQ6khQf2qqt+/W1XH7P7r9kug4rl7afX/Gld9Bf5fuM+3L
Ii66JaTfUxyHzpPyxD9abV5IOrqoo3ZRSPEs8tPSfi309ZeE9P6kfyyZTHZN
mkdcPxUd58TxYltCOg7Vz6Q6VzE/PyocgO2suz4gjHXHCeHnq20J6bpcv5yr
26Hju/XK3POkedz9u26fa5ofd143qpGOhXQvdBzyq7vvP5HiAvSdToo/PJlM
JpPJ/cC+6U5b1Q+nfEPXn0PrauqHSHqn8HceE1J+ZcSNTwnd+mxu/6Nx+ffG
xVbvfys/PO0j3PMkvQr9nvzztN+h/JEBWOxr54ejpO+U9n1u/WHat5I/PPtK
JJO756R8FzR+0vhAv6frpX7Hrj+QdGgR/3xEeMftxZ4W/kO1+4X0d0nPRuOk
q8OnecSNq9J84fqH3X6jtG6kecrtF0/9aEi/Gnr854Xx/2aFFE/cOW9uThjf
wavCeO+mha3y7NzjJJPJ/pPm2YceLLYqpDwgqtvvxiP65g9x85uoDkCsF14R
uvOvqw8hHT6to6Le2O+E4be4LKTzpN+765ZkMplMJpNJl7TOGZdOhnT1se67
KDx0f7EFoft3Z84VWxVS/Yrog/SMkPRv1NeJ4oauHoziDq5fopXe3r3/bt4H
/V3aH7nxWepX5cYr3ToPtD+K92tVSDpMqn9Cfleqt0/12UKP/7KQvhf3fUgm
k0mXrj6f4iOuPtzNz6I4xQCM+jOSv5F0PvffV2xeSPeT1mM0/9Lv3Tqcbp9i
6s9Cz9H9Pb1XdP5ufI3eq9f+XGxeGPP8U8J33ym2LiR/NdVxpTpdbp0TWpdO
in87mUwmk8mkspX/p1W9XIrz0nrS9Qu5ffdoXRTr8VUh9eX8SLWHhKRbPv5w
sU2he57u9br6/+v9vrame11d+89df6mbbxLP/7KQ6ofQcW6stiCkvPUP6yCO
kvqKUp0T937SfoS+C7cfQavnnkzuZ7bKi3HrKrjxIBo3yP9D8/gbfym2JIz1
i/ZTvqnavHAAdtfBYrPDUdL4HOuXB4Wuf8+t/+D62dx4aNSj2ha660Y3v4Py
49z1GNXZ2Hk9eUp466+KTQmprlf8v/uEVB8s9PgrQldfutvxIplMTj5Tn//3
keYv0lmRPt/9uzSfur//VrXzwgO3FXtSSHqe2C9uCkO3MiN038NxP+dkMplM
JpN7h279B1qvkv/EXc9Q3QNav1HcgepZUf4++cGobhXpn6mfINVvd/2KpDej
37v6dvc47vvm+lfdvBLyo9L7ScchP6rrNzt8qNiiMPxeC0I6TuyTLgvJP3b3
r4vNCWN/dkVI+zXazz5ysthVYe5fkslk12xVP5/mF3ecp3Hv9gPFloXkF6J6
RJ+otiQM/f6TQhr/6bqiTtK2kMZ5uv80b1Kc7pZq20JaL/31g2JK9/2heBm9
D3R/qO4H/V2qyxF9X88I77m72ItCWs9E36d1IeWTRv3LDSGdPz1ftw+UG/dM
JpPJZDJ5/TkufT79nvRXrfoxuXVQ6Tjkj4p48hkh+Rt/UG1FSH+X1od031y/
Il3XfvP/0PW6+1CXrr+U/Oc7f3dXhKQDDP/3gjDW9buvCx11EJ8YjpL0NvF9
LAvp/rv72ewHmkz2h+PS57vnSesEGk+o3sXRh4qtCameefTxfkAY/rZXhLQu
onrs5I+iPCnyT4Z/6KKQngutT+i+uc+rVX1geu6t1gN0H75WbUVI9VuoLzzF
tWmepXjiuelqQtJP0vfr5ru5eRnJZLL/TH3+znT3KZR/Hf2WTwnpOLReIh3X
zn0PdR4ZgIWuamM4Sjo+7U+pXivdz73aHzCZTCaTyeT1Zys9G7FV3QmXrs7n
e9WWhA8fK7YhDP/W00LqL0k6qPOzxdaFdP7kr3DjaOSvcPcjfVuX0v35r/po
7Xb/Qn4/ep9pPxLPeVYYecAbQqovEfuJKeEALN6LS8NRkl+OjkP1K0hXOa73
IZlM7h92vZ6huICb50W/p3qY1K9k5/ysnwupPmfovpeEdB9o/KfrIt27+1zc
9QbFxT5XbVNI/Zhofej6Y2m9EevHFWHkbTwlpPk61heXhF+qdkpI/UDferPY
ipCeL61XqX6+e38yvpZMJpPJZH/Yyv/TdX11WmfSuqJVHy73ur5T7YLwX6od
F5Le6S+vF1sUxv/bEtL5kP/KXbfvt/Ub7Qta6Tdcf6z7dyN/d1ZIeeWvv1Zs
QTgA27nvm9ZhDr/m48Lws14WfvB+sS0hXS/lp9N77vqN99v7n0zuhpPSV4XG
z1ZxJSL5G6Oe/bqQ1g+ku6N66fF3Lgupv/MA7LZbi50ejjL8TMqfVNsQUv5U
q+cbdUo2hW5eBsUBafyn9RW9b/Q+uOsx8huTX5H4jWozwtBvPiak98ddn1A9
k1bfXTKZ7D9Tn78zKf6yc788XSfQfE3jMM2DFCeKdcGikJ4XxbnoPtC6Iual
eaFbbyGZTCaTyWSyFd36MF3700j35erPyY9B62fqc0T1E6Iuxaww/v0lIfkr
qP6V21fRrRfRqs7VpOvz6T68/16xTaF7PhQnffCBYitCOn/S4cdzvldI+453
3ym2Lox9z01C6mtJ+zK3Hl0ymUy2oqurIT+VqyumeYR06aSTobw/qnM1ACPd
Nenzo07/s0K3jzDF0Wheo/WMW7fKfV5R32tLSH0Kov/zjJCOT3lq77xdbE34
1WpnhaTLovma6lzRe0LxMuoz7sYrad3lHmdS4vjJZDKZTO4HToo+39XXuX4M
Oj6tS2NduSkMP+Jzwli/HxLSOjB0zc8Iad1L10X3wfUT9k230DVbXS/tW1vd
/511CGqko6D3lo5D9fCjjvJhIdUVcevw0D595/qNu6+TQ383dYPJpHJS9vU0
fpLu2u0bSPfBjeuR34bWD7ROoDwp6uMT65cXhORvpHUL6SrJX+TWpQ/9+JqQ
8hHoPpOuvlWeJtH1x/602qYwnttZIcUTB2AUx7y52qKQ+o1SvRR3fm+Vv5BM
JvvP1OfvTDpPWrdQfh/Fp+j4J08U2xbS/jHWBfcIQ4c1J6R5hOKGpOf/cbV1
Ie0TJ2W9mkwmk8lkcv/QrRfhxu/c9Q/p39x9PZHqlpP/agBG8YuIPywJqV8w
+X/c+9yqnm3X/c1d0vm7+bzu+VNfLdrv3FRtXkjHJ508vW+h43txOEryp9H7
5taLpvcq85GTyWTXdPX55KeiuJjrf6NxkvT5oZe5ICT/GPUpvvOOYueEkUf1
sjDWIweF3642J6Q+11SHkOaFH1ZbFUY9sOeEdH+oTpdbL/Tz1baElK9H71X4
Cy8KqU8o6eppnUl+5tF+CR+S1p8Uz6X7Q9+Fu2501wPpF00mk8lkcu+xVX9G
t96IW7eB/EVufijVG6H1eeinDggHYOF/en44SjrPVromN+9yr7KVHqxV3XV6
vtNniy0Lo2/neSHpH6g/V+QxPyQkPUa87yeFtI+j7/GL1a4KaX/n6ljG/X4l
k8n/oBsnouOQf4/mNXd9QuuNf6t2VUjnT3420iWSH+lT1VaFEeecFpKu/qPV
5oWk96bzceO5dB8or/C1PxebF36t2rSQni/NR/QcXT0/vW/u+or6dEf88HFh
PE/t30p1PCjv48BtxZ4UuvkmdL10nzMPLpncP0x9/s5096FRB/SiMOJsLwpj
X7ghpH5qVL+U/B50nlQHLOKcq0Ka19x1Xfo3kslkMplMds2udbNuXIz8Le66
yPXnkB/sZ9W2hF+vdk4YuqdfCKNf4gtCqssadREuCV09FV0X+cHceEQr3ZQb
n6W/S/sjeh9o/0Lrf9qPhF9qUXh6qtiCMPR6LwkPHyq2KKT+zuHfekQY+54T
wmNHi60L6bkcf7jYpjD1cslksm909flufpa7DnHjd1Qficb/0Ok8JSR9Ba1D
aB4ZgMW65g/DUVJ9DKqn9OVqp4V0nqQDoTjRmdPVhFRXip4L6bW+WW1dGHmW
TwrpfrpG60x6HyI+tiSk959I76f7HdHvaf3WN793MplMJpPJa2crfT75JVr5
tVy/Ivk56fhTp4rNCyl/lta9X6l2Rkh93skP3Cofedzv1/Vm3/T55DeO/dO8
0I0LhF9zSkg6zOgL94AwdBmXhPR3Kd/E/X7d+0n+cxoHWv3dZDKpHJc+n37v
rltovnDn04ePFdsQkp+NGH67J4QDMLdeGen2I/75rJDiVnG+i0LyQ95+oNhZ
YTy3K8JW7yfNp26fJnqfKe+M+m6TPn/nPt2vCCNv42lhPLc1IX0Xbt7opOhj
k8lkd0x9/s6MepBbQjp/yl8bgP1TteeHo6T8LMqDo/tM+VxU3yD2x1eEc+eL
bQjd9VX6N5LJZDKZTPaN5H+m37eKrxHdOAKtn6MewJaQjkN1yGO9fx9Q65pS
/VvS41F+KK1vKT7Yql5Wq+fbqk+6+3fffafYupD0itQHk3T11E+c6gZH3OZV
4QCM4raU99HqvhEjnnZV2PXzTSaTSVefT8dx42U0Hrr1sqjOEuV5kQ6HdOlU
tzD6Uf9RSPWySB8+AKN5iuJ09FzIX0dxSZpPKZ5F8xfVvac8OLoPoZf//XCU
cbzHhPRcKI5Jda7ofXZ1+PRc3HUC6Y5I99L1uJFMJpPJZHL37Lp/YtfnT/5M
WgfScWjfEevxJSHpxz5WbUE4AKP1c/ilXhHeWG1BSNdF68NWfQ26fr7jorvf
JLr1YGl/SvpG0mPQ90h908i/HbqJl4VRb/iCkPrJvvGXYkrKj3bzGtzvnfYp
fdPnJJN95qTU+XG/a9IbU90nOo47LlFcj35P64EYv38rjPy/3wlp/Ke+P9Qf
k/yN1A90AEZ+OdIT0jxCz53m95lzxVaF4e+cFVI/UJrHSW8Z9+MZIflpqX4L
3Z/oq3NWGPP2cSG9b6SrdNdLbl+Ja/3+k8lk/5j6/J3p1j2IuNO8kObxC3PV
hG+9WWxFSPcz6lNuC2m9RPor2p+68Sl3ndZqv59MJpPJZDI5KetJihO5dfhv
qbYtdM8z9G6LQsobHYBF/OHwcJRRz2BBSH4eqnNO+QtuP26qH+vS7Rvl1h1y
19X0noR/blYY9/uokPSW5I8ifSDpGN16vORnaxUnnRS/ejKZTBJdPXCrelk0
flJ8jf4u1T8crWv/IQdgpOeh31P9Q4rTkc6c+mK7daWIf/2g2LaQfv/9asvC
mD+PCek+U5yRjhPxyXUh+Tkjj/MZ4feqLQlzvk4mk8lkMjkudr0Oces/kH/G
1bmR/y38XpeFlF8Z6+KHheFHPCOk/NNYj08Lw484JaS8V7pvlL9J95/8ja5/
z9VL0P7uX6ttC+nvkt67VV1cV4dPfV3p91QnmfZfdD6kk4z362kh+dvp/tBz
If8n9WVzda3kf6b3it4H+o5ajW/JZHLvcVz9gyj+RX4w0ulR3Ir6OUY/zDkh
6dXdOhjkZ4t6J5eEdH/o/pMOn/oZ0f0hkr6U7g/dh+gP8NRwlDEfnhRS3gTl
zZH/me6nu86huCrFl+k47u+Tyf3AVrpiN4/JPT7Na/Rdn5sutiKkvDOq70R+
ADf/t2+keqQxr14Uxn06JaT9NfEL1baFdJ5U95JI9TbdOK+rK8v4WjKZTCaT
yVbsuv6V63dy6713fR9I731DtWUh6bWi/v39QvJ3xfp3Xkh1kCgO6N4HV89G
+zKKe7r9Dlq9nxSvoedL9z/qQ2wJ6e9SXJL8jRGPfUEY/ktl1F07LaR9Fu1n
3e+Lvl96vvutfloymdx7JD0Jjaut8h/deZN02qSrj/XIgpDqHO6sV9d6TVR3
i+pvkL+UdEHkr/tctU1h+NUuC6m+PfU5onxM0pPsXGfsiJB0XLQ+iXl+Wzju
7yaZTCaTyeT+Zd/0+XQcV0fk1oV7791iG0Kq40d9GKlOCK23aT35YR/4UdL+
xa1T6uoZKJ7+xWpXhe7+iPxdpBun/V0r/Qnpw12dP/ml4315URj7s+eEsa+6
JCRdH+V9RH3fC8I33yi2LHz/vWKbwoN3FlsTuv5qeo7kH3b375Ou20kmk7vn
pNQ7cutixTywLiQ/GNWjoHrFMb6+LKR5mfx7lM9I/a8p74Dq5/+02qYw/J3n
haTPp/hdnNcrwp3jxeonjHnpivCO24utCL9ebVVIeX+U70n9d9z4IM3jNF+7
+RTjHieSyT5y0ucvWp/HvnZNSHWigptC2t9Nih7b3T9eulhNGPf1VSGtH+j4
dD8pHy2e24yQ9oP0dyluSOdDcV63nkAymUwmk8mky3Hp8906D7Svp3o1dBxa
X0UdhatCul6KE5Gf5IP3i20JqS5W+LHuE1Jc7KvVloVunym6P7QOd/NSie5z
pziOu1+4qdq8MPxey0Kq7xS6wk0h9TEnP17kMVwS0v7R7TdB94HqVLjP0Y1r
J5PJ5Ljo1sdzj+8eh/K5SHcR653LQoqvhX/rIeHXqk0L6TwjHrUhJH0+cQAW
OqPTw1GSf5WOQ/oTqj9P8cTgY0KqZ0V+RYpLxjyvjHXBE8IHjhRbFtLz6luf
rGQymUwmk/uHfdPnkz+N1uHk76K/22rdRfsI8u/Repvq4FE9W/Jb0vm4enW6
/+T3c+ucu/e5Vf0Q8r/R+0PXRffz6EPF1oSRj/uSkPZNtN8hHWbo7h8Xkt+b
9kHTZ4stC1v5k93vyM2vJz+n639OJpOTy0nRA7vxUBoPY367Iox5Q+sSRz31
U0LyN36r2oYw8hovCmNeelBI8xTVsxqYRnXvSYcfdU1eFFKdEMqvpLoo1KeA
+mhT/sLOz+W88J23i60J6T2k9ZKbZzeuOn7J5F4i7S/cevg0v9BxaByg47t5
Op+utiakcZvqrrv6JZpP3X1xK7rrE8o7u7HalJCOf/JEsW0hxSXPz1YTkg7H
vZ+u38DVgyWTyWQymUy24qTkz9J6zK0L4fYtcs+T8lXp/MnvQfr80O8vCalf
FfUj/lG1NSFdF+Wf0u9pn0L+E1oPu/FN+v3Hqy0K6f5H3vC2kP7uzvp2tfCH
3TscJT13qqtP9dZof0TxR/f7pfeZ4mvj2p8mk8mky1bjVas6nNRPOfQX9wg/
VW1VGPPGnJD6IN9cbVFI+YBTp6oJqY4i5alRXf2o2/S0MNY1zwhJf0J5CqTD
p7gY9SMgP/AAbOd6+6qnOv5wsU0hvVeTUu8lmUwmk8nk3mPf9Pld10MjXYGr
Syf9ANUt/9Uviz0mJD9S/L8F4euvFVO20iF/vtqW0D0OPV/yW1JdF/Kbue8J
5XfQc3ePT/upH1RbEcZ+aFU4O1NNOACLfcnh4ShJ70f7StJbun11qf9pK38+
nQ/5ObseT5LJZH/Ydfy0Vf9xN++J1i2hN98U0noj8q7OCqMe/OPCnfuJqx/v
roPFZoV0HNK9h07/CWHo6I8LKf8x/K9/EpJun+rkD8AoH4HiiTvnb2rdEvq7
5G90+wq1mh8pzu7W2Xb/bjK5l0jfKeneaX4h0vHpe6S/6+Yvkz5/AEZ6GDof
up9d54+3Iu2bKM86dDqvCEm3E/vINWHMD5eF1M/Fva4Tx4ttCV2/E/kNaL7r
2/NNJpPJZDI5uZwUfb67v3brIBFpnfb2W8VWhbQ+jPjDS0LyO1G+KunKqJ7t
T6opwy+1KPxhtVUhrUvpvtE+pVV9A7qu8AvOCw/dX2xB6MbjaD9C+44BWDzn
nw9HSXWDI9/igtDVf5Jfi45D35H7PiSTyeSkk/yBrh/S/bs0blO9I6pP+OVq
p4U791PWukwDMPo99cek/s60PqG+pVGXa14Y8atzQjp/MoqLUb/p01PVhKT/
p+PHuuOK0H1PMj6VTCaTyWRyXOybPt89vutvcUl+M/LLkU4s+jKdFFKd29BN
nxWG/+yUkPIF6P7Qdbnr0mNHi60LXd1XKx0Xrbdb9aOkeoNRL+SikOq0fLva
ppDynUmXSPs10gdS/jjtd+i9Ij8kPXfS57t18l19TjKZ3D+cdH0+jasUF6Pz
pzrqlEcWdcWeFA7AyG8W8bFFIeWFxf+7SUh6/oN3FntWSHp7Ov4dtxd7WkjX
S/lutH6juPDPqm0Jv1ltVkj9EWL9eFVI7wO9h25dYlo3Un4lHSf9n8mk0o1b
ufXkXba6LtKHD8CoH5k7v9P9vN7P9UOS3oau67vVFoQxz2jdSNLVnD1TbElI
dcNuqbYtpH0c5RdE32kl7ePcvMXsz5JMJpPJZHJS6PqR3OPQfp/qFNHxyT9P
fgBaB5IfI+oJLQlJnxbxhEeFVF+CdHTUH4rqwFO8j/xdlN9K/a9dPww9X+pv
Rfsv2hfcVG1eGOvry8LwP13aNSmeRc+d8jLCbzctjH/XOrr33F1sS0jfhbt/
cfvQuc933ONWMpncv3TzjFr5FWm8Jb1BqzypWC8sCEP//oSQ+l3SfBfz4ctC
qtsf64hzwpjHLgipb3X8+4vCX99V7Hkh1auneFw8h1eF1JeZ/I2Ud0B+Szef
jub9nH+TyWQymUyOi33T59M6nNbVpAuiuK2r76V1ftSFXReGrvkZ4c71XVUv
Tboyqhdx5HCxJSHp+ly9B+nSQz8+LfxxtXUhPS/yg9H5kx/YfT/Jb0bPN3R2
M0K6P7S/C04JB2CR33xqOErqF0b6wLnz1YSfrLYibDUOuN87+f9b1SHpm24n
mUx2N86M+7xGSeMerVtovqZxkubfmAeeFVJf71i//LuQ4mWUL0ZxwJ119ap7
j7yBM0KqJx9xzkvCARjFByl/gfrX0HqPrjfWfS8J6bm38h+660Caf2md7x5/
3N9lMjlO0vfl7ltpfUv7L4qbUB4Q1cmkfRzFU2ge+Wy1DaGbX9C3dT6dJ837
FKejOlSkg6J+fJ+pti48fKjYopD2iV+oti2k+0B+FXp/3PllXM83mUwmk8lk
kthKn0/rIjeORsen/QLVO6Lzp3Uj+Z2ifsOykPTelK9K+rcBGPWXpHhT1LfY
EpKuLJ7DFeHUqWLzwn+tdlFIOvnwMx0Vfq/akpB0cfQ+fLXaWSH5u+g+xz7m
j8NRUh2PyAO+LPxEtSUh+Q+pj4ObF0y6UPLHxnPbFrp5GclkMjkuuusT8s+4
uqOudf5u3hbVM3TnQeqD+f1qy8LQy28II79wXUj9qSM/b0ZI+iXS/5O+Zef+
OPpcoj/2ovDj1ZT/f3tn1qTZUZ3r/gngYwPGBv8F7GPAjP0XbGMzCvwXADMZ
DOFLX0pCICG1hOOEw0EQDgeahSYmgRmFep5bPVZ1Tc0xEmiGE5kZvjj98pT1
qnNXfru+d908F/VV7r1XTitXrlzp5oOi+s0+VBiGYRiGi8ZFi8/v9VxaF7j7
9W6ekOYHOitsdvrXhC0O69+Fe0CaHXpm79Uke57yfpA/ivRG/tjmn7sopP13
0ifp343rdu9pfXuVdSHdd0b3hVH57TzxUWFrF3pfJ+XR/WSVS8Lmz/7Byyat
7+j9KW6E1jtuHJFLN56f4hgT1xGGu49zuX+c3vMdVTaEbp7YF54vsi6kfF8t
TnxdSPubzU55WNjsmluEtA9LefJpfqS897TvTOf7aD+U7sXeA0L52Vy7juwE
116l9kPnB912676Pa4+F4TKT+otrr9K+UjuPuyZs+zArwnbe95LwvnuLKH/9
qyJK2sfZA0LzEc2bFC9B64Wp53cizVM0rpLeKJ8V/Z7imt5U5ayQ4n8oPz/N
O3SOw50XKI5o6vVmGIZhGIZhL7txlP1J9jP5l9xzka5/gPK4UnxXy5P0QyHF
oe0BobxYlDeJ9EDrDtIznVOmddOfV7kopHur6d5k8ndROU/9ssh5IeW7oDwe
lHeC4v0orp5I+6qUh4TOU1D7pPvC6B5Mtz/m/q8wDHcrXTuH9utdu4L8TkcO
F7kkbPPhE8K2D7MqpHxNf13lgJDyMtF+2R4Qmq9JD22+3RSSPulcZNPfaSGd
jyM/M8VHUb59ek/K40FxKaQfOje30/0kDMMwDMPwv7lo8fm0f+36MWj/17Xz
3Xwj5Lc5sL+KsNmnmief7sOifXOKo6C4OPr9DdcXOSukPIpuvTz3bJHLQjcf
LNWju+9PeQKb3/eYkMp5+qkiF4Tb39ep+V7oPMUD9xdZFba8Iw8Kyf/s5r91
z1NQvATVV6/4ChpP3P4ehmE4Kr8ZlUN+Jzduk+Zrun+H7q+kPGOU377tlx4Q
0nm35qc8Bvyh8OGHimwI31dlVUjzO+VFafbR94V0DzjtY7Y4yoNCikeleqR7
PD9e5byw2XFPCOn8gtt+6Pe0f73T/TkM58Be58tOnyqyKWz3Ia8I231cZ4UU
R0HnmOj8+PbrKRUaz+l75zLO0PqL6r3tO50WUlwTPde1E0bFj4VhGIZhGIZ9
6drJU/vtaX+B7mOi+4v3gFDcPuV5oHwRFD/W/v6Y8D1VLgjdfS43rzvFn1O+
WYqfpzzA5AejfB2U/4r8XbfvK3IK+FPh1fcE/DfpPWldTPp089tTf6E4f1r3
Zf0VhuFc6N5rOeo9n32myKqQzqNR3AvNyzT+U34Jig8hO4Tiedp8uCmk96S8
9JTn8+SJKsI9IHS/J31vs8u+K3TbG/2e5nGyx5JXPwzDMAzDsC+ntq9c/yTZ
/3TPVMvb8R3hHpDmL/zW3qv56SqrQjr/S3kwHvxmkTUh3SPg+rWef67ImrCd
YzgnpHUHnYelPP/t+44IaT1F9wj8bZXDQlqnUD3S+pH8nOQfbnH660LKq0n1
4vqTwzAM585e/kY33y/F0fWKP6T3ccd5ul+GzouRXUF2SDuPeFJ46GAVIeU5
+eV/FTkn3APS4h9/tvdq0vk4Kp/ynjW/5rqQ4uSpPdA5RLo3gUj7lb3y3swl
bjYMw2snjVf331dkRdjOJV8vfFuVNSHtf9G8tmj3ZCU+PwzDMAzDMNxJ9spX
Q/tftN7vZU+2/Y2LQvIXtbjsw8IWH31GSHnX3bzuLS7+pPDFF4psCGkfjeID
ST+UP4Hy4lIc/p5OQvkijh6pIqR7ot9cRUl5iSk/BsXJkz7d8xFuv0gcYBiG
u5W99tHce3xoHCb/G+U7onNtLb7jkpDiVZo9siFs8fUrQopXp7yL7e93Cem8
G8W90LkAOudI5Rw/VkVIeVooHxfFBdF+Ftk/bl6pRfMbh2EYhmEYzp1T30NK
/hbys22fN1X9S3tAKO86+Qkp3/6vni5yUUj3fFG8BOmN1jvkf6P8JKSf91e5
LPzijUXOC+l+z7beelxIeVoo30vLQ3lESPV48ECR83uv5ruqbAnpfATdy0Dn
C9z2795HEIZhOHeOis9335PmWYqXJtK83OLrTwjbvZ3nheSfbPbIE8I9IG+t
srr3arb890eFVH7z/10WPvPrIitCip+ne0U/VuWcsOVjuySkeqT2cOtXiqwJ
qRzyJ9O5PPd+KPf9c74vDJeHic/fnonPD8MwDMMwDBeBtH6f2j9Adi/5Mah8
urea8kW0dcY/CeneajeOvf39a3uvJu03kf/N1fNnq6wLW/7+g8Kr7+3+bzY/
2Q+F7d7FbwtbnOApIeWzIn/d9vmENQ9VO6/wiJDi+tx4ePo9xX+6900v2vo0
DMOwFxctPr/Xd1G+R7qH1I3HoHmT7ifdA9LiYo7tvZrtfN4hIcX5t7xZF4R0
fq3F3RwQkp1A+2u97E+Ky+p1XjUMwzAMw3C3cuq4erccsv/JbnxHlQ1hs38f
E1IcOLHl0T8npPOtZG9Tftfmj/yJkPKK0HveeEOR/cK2nvgPIZ0Lbv7UHwsp
Hp7yhFCcOdUv3RdG65Tmf71F2PyO3xaSH5LWNeTXJT/qX1S5LKT4/zOni7x8
jh4nwjAMr+bU9sOo/PlUPvnH3Ph/yuNB+cdo/4v0Q/fCtPLuFX6gynFhi0//
kZDuGyJ76dTJIhtCuseT9m1bvpAVIcWlU/40yu/h5vvqdV+2227JvpraTx6G
4eIw8fnbM/H5YRiGYRiG4U6S1uO98nj38gOQn6HZuRtCsm/pXuO2T3W/kPJo
kb1N9z9Sviw6F0D3Nt57T5GLwk9UUTY/1kkh7Q/uAaHvanlwLwsp/o38hBSn
R3k/PlPlspD26ag99FoPkl/OzWflrmevtf+HYRj2Zq/9sqlJ/reXXiyyKSQ7
hO61ofuI6X0oDv/tVdaFFD/T7ov+TyHNd+Rfbd9xWkj5MGm+o/xXLZ/WqpD0
Q/OsO49THrOp7YQwDMMwDMO5c9Hi88l/QvFUf1hlTUhxbhRPTvYkPZfi5Mme
b+/1IyHltSC/319VOSek+61a3otDwuafe1hI8e2Up5f8eB+tclZI6xHKf0v5
OppeHxWSPveYQucj6L5Xig+cyzo6DMPQ5aLF59Pv3fHZ3d+kPE5u+XRvCz2X
/JzkZyO7iM67tfN5Xxe2/cODQjoX+aYqZ4Vt3/Wk8NDBKkLy05J/r9k1m0K3
/ZAd6Oapo+dS+6TnUnvLPmMYLg8Tn789E58fhmEYhmEY7iRdO3Dq/LSUr+C6
KqeElD+h2cVbQtpHe+qXRc4L2z7YcSHljaf7JY8eKbIipPxLFD9P/ivKe78H
pOWl2L/3arb49wPCto/0r0LKu9X2x74vpPc5crjIpb1X851VNoXufQq0nnLz
dFH59D5uXpRe+TTCMAxHcS5xBeQ/pPd087e7dhHlP2zz4WFhm4dvElIcDs1r
Lb/UhpDyfZF+6F5mN68p6Yf2EynPJOnfzZO/aH7jMAzDMAzDUZx7fD6VQ/5D
+j3Zn83Pd0RI8e10vyflY3/g/iKrQnpPOi/c8o5cEL65yiXh/ieKnBVSfALl
v6Vzxy3P7nFhW9/cI9wD0r7je3uvZrPfnxRSfmDKA0zx/3RfAK1fqL5oPUjt
jdZfrr80DMNwavaKBx4Vn0/luH4t8rNRnitXP7QP5e5PvfB8kXUh5dmge0Xp
XCT9nvKw0Xu6eTwobt9tt1Rf9Fzaf3T7hXuOI3H4YRgmPn97Jj4/DMMwDMMw
3ElObQ+7fi3iP1bZFFJcPflb2n7XuvDDVU4LKX877ZdRnvm2j/SYkPLn7wGh
fZ/mV/u5kPaV6BwE7eM0e/9JYfOrfVPY7o28T0jvSX4/ur+gvdcZIcXz0zqR
zmvQ76m/kD+Q1k1ufGAYhuHcOfU+mnt+0N2ncM959dpfe2uVVWGLM3pc+J4q
B4UUL/ShKieFLe/+USHF59C89sjDRTaF5P9074kmkv1J+5Lxc4ZhGIZhGG7P
RYvPd8+Bkt3uxrMdO1pkVUj3ctK52j0gLX/Gd/ZezRZff0pI9nmLdzskpOe2
fBvf2Xs1Kc883U9KzyU/Xvu/S0LKf0L3k5KftuU1OSSkPL1vqbIibP7dC8Lm
71wRuu2Z8tz2yocThmE4ilPH5/cqx/UfuucB3Xsb6fwg+bvafuaWkN6Hzn+5
fjk6X0b6dO8bpe+l8mk+pfLJn+ne003lTz2PU3umdkvfG3sjDHcfE5+/PROf
H4ZhGIZhGC4CaT1OeR7c8slPQnZprzjnj1U5J9wDQvtTlOeK4vwpXo7i52kd
1PLof1fY8t0+JKTvauuMM3uvpnuf4weqHBe2/ah/E9L+HZX/8SrnhZR3i8oh
f6kbV0/7s64/lvJcuf0oDMNwLly0+Hwqn85t0fk1+r2rn89X2RLS72n+ev65
ImvCE8erCJs+nhRSXs2DB4qcF1IeflcP9F1k71GcFemTyiG7tNc50zAMwzAM
w7Bx6vg68tuQvefmlW15MlaElI99D0jz592192pSOXSv5aGDVYTND3dESOcI
2r2Xh4X0nhS339YBJ4Wkh7bOuHvv1aTfb69PvYeU/JNufntad1BcB7VPKof8
hC6pfPquMAzDqblo8flE8h8Safx348OJVI573yWVT+9P+1zkZ3PnHcpnQt/r
+kvpu6geqX1SOW759J70XPd+bXff3G3nvfpXGIY7z8Tnb8/E54dhGIZhGIY7
SXcfyvVH9bIbKQ8D/Z7sZ8rP0L77SSHlh39dlX8Wfq7KhpDi4lr8+ykhxeHT
Ph3liWp5HX4qpHMB7bu/JaT3aTwvbHmkDgrv/EaRs0KqR7oXm9Yv7nkNN66e
fu+WQ/dBuO3Z7UdhGIZTs1cc/qLR3W+icZvyU7n7Kdvfd/Py8ybRfHpdlU2h
qzd6z177iUT3/oLsQ4VhGIZhGO4Mp/ZvkF3n5l+l9/z7KheEN3+5yA+EB/ZX
ET79VJELwhbP/hMh5eFv9vV/CCkPP8Wxv7nKJWHLN39RSPqnfLOU/4TOHbTv
eFTY4ve/J2z+1H3CFlfyDeFttxb5sfBTVVaEdC9Yr3bo+jPDMAznwl7xeKPi
8+n3NG5THg+KP3f9jbQPRXYL3aNN5dN7un5L2tdz9Ul5PEjP7n1J7r0Dvc57
Ur8gOyH7g2EYukx8/vak8ZzmnbauPSWk+BzaF5t63Uff5caxEGm+c+N23H3k
ReOodhuGYRiGYejel+eWT/Zey1u/LnTLf3+Vy0LKS0/7axSXvgfklpuL/Ofe
q0nnBWjdRN/1Z1XOC5td/HPh+6qsCmm/j9Ydn6hyUejmoXX9jW77dNtJGIZh
uDs4dd7+XvOOey+AGwdC5ytbvsxLQrpfe3R9hmEYhmEYhmM4ar/SjWty7wOl
/B7Nz3dB2OLf7xS2uO8TQopjb3HlDwubPf594QernBBSfg+KY98DQv7P5n89
IaT3p3h7eu6vni5yce/VfEOV08Lt85wcFlLc/skTRdaFFL/n5kl2731InF4Y
hmEYhmE4R7rnVohkb/e6r4RI8S20DnXjk8nOd8+Vu+9J98HRerblrVwTUvl0
TmrR7vly66V935PCL95Y5IDQjY+i+qV23iv+iuqR4nOmvg9xLhzVbsMwDMMw
nC972W9EN+8EkfYdyE6m860PP1RkQ/jsM0VWhceOVhG+pcqK8ENVTgppn6id
AzgmpHsb6f7ij1c5L6T8VHQ/dVs3fFdI8fxP/bLIeSG9J9Uv7XtSHoyd7idh
GIbh7mIvv4p777D7PmRHkZ1A8fb0XMoP2eJT/kXo3oPjnssLwzAMwzAMdwen
3sd084S4cQsUT/KVW4pcFt5wfZFvA58Qbm9Xq7S4+HN7r+a99xS5KKT88H9T
5ZDwtVXuF1LefnpPyodPcfuUD5/0Q/cU0P2nn66yKvxslXWhm4/X9au750qm
9tuHYRiGYRiG4U6S9jvcfQ0irRPpvDat+3ZrHC/lZdoDQvtHtD6i9c6i6YfW
fRTfQvnzW9z640I3P0Avuvm73HPfVI/Uf0mfo+Prd1u/DsMwDMNwviT//9T5
eSgfEd2fReXQeorswAfuL7IqvK7KKWHbR9oUkt7ovACde6X9o5YX6mfCFjf3
beEeU6icFqd/XviuKltCNx8U1Yt7Dr3X/VxhGIbh7ubUeR7ccsi+cvPMPPpI
kS2h+54tzmVF6N5PHYZhGIZhGC4nR+1j0nMfebjIptDN40flHz1SZEX4F1Uu
C5sf8RTw34WUf578eJQ//z1VLgjbfaEnhcePVRG6cftUPvlLiXTvJ9Uv1SOV
T/7GXu3QvQctefLDMAzDMAzDObKXHeuey576XmN6LtnzvdYX7j3L9J6Uf5Lu
F3Pjc+i5tM/VSz+9SO/f4k3OADU+/8zpIltC91w2kd7fXVe6fpte7zk6vj7x
+WEYhmEYzpXueclezyX71i2H3pPivd31wh23F9kStv20K0JaH+27rchPhLRu
onur6dwBxfW5+nxnlU0h5fulcmi9RnrrtS4IwzAMdzcXza9CdgWd46P50c23
3yu//YsvFNkQZv4NwzAMwzBcTs5lf5P8fnTPo2s/U36P5sd6XEhx7K+p8gNh
+/tDwnZP5nmh+570+7vvKnJeSPdytnsGfiik8mm988Eq68JPVrkkfPqpIheE
vfzJLt04n8Tnh2EYhmEYhnPk1PcOu+9D5bhxEW6cvJtP3s3jRHnaKS6F1k3t
HrO7hbd+pciakPRDeqD3H+UfoPU+6a2tF08L6b48Nw/AKPbKe0n7mFT+6Pj6
xOeHYRiGYbgodO937nW+lfIX0f4L2flUPsWnuXHjFB9O5dB7kr1KpHMEL71Y
ZFPY9uU2hC3v/paQ8va7eqZ8+LSPSfYt5ed399EW7fx1GIZhOJaL5ldx4/N7
PZfsCrITyK/YK84/DMMwDMMw3B2cy/4mxT/Qfj3d+0nlv73KurDF198nbPdW
HRH+WZXzwpa3/jEhxaWfPFFkXUh6oHXKnd8oclZI93628wH/R/jqKueElP+f
8pPQc0kPVF9u3o+p71Ob+rlhGIZhGIZhuJOkePJe+/hT76dMnc+f7Hzaf3Hj
q19bZVX4+ipfFz77TJFVIcWNzMUP4PJPq5wT3nhDkf1Cihei+KJ3V3n5pH06
irOi9tPrXjn6Lredz4Wj22MYhmEYhvPj1HYFxfP3Kp/WWS0f/KbQPb/8zK+L
rAjd93Tz8NP733ZrkXUhPZfs6iOHi1wStnPPPxK2ddi60D3/S+tQWheQfuj8
snvOIgzDMFxOTp13xS3HzYtCdgXlse8VP0/v2SsfThiGYRiGYbg7OMoOdPM/
uPEbZPe6eS1++V9Fzgmb3+6MkOL8KR6A8vi1vPj/LnxrlVUh3Y/5l1XOCr90
U5HHhO0cwAnh56psCFu+xu8J23ro58Lnni1yWUj6p/MX5EeldVmv+8Ko3VK7
Sl79MAzDMAzDcJFJ9i3lC3LLd/Mckj1PcQUU/0DnC4i0j+Pmb3TjsffdVmRD
2M5zXxRSfP7bqqwJ3Xp343Ombp+uPt9f5ZiwnRf/J+GB/UXOCWkd3drpGWFr
L8rb9xU5JWzxRMrmDzgrbP3gSWFbH58SNv+Bkr6X9Dk6vj7x+WEYhmEYLgoX
zR5++KEiG0Iqh/K+0u8/UGVN+Loq/yzc/0SRs0K6B83dp3tTlbPCll/qgLDZ
3ZtCKp/yZdH916RP9963XvtZdF7Y1UMYhmG4nJw6rt4t3/Vbkr3kxmnQeUDK
d0H349C8HP9VGIZhGIbhcnJqO9DNW0j+KLqf0fVTuf4oOi/wD1XWhPS9FFff
4uIvCpuf7y4hxRs8+M0ia0KKG3lvlSPCFt9xQkh6pjwhFPdCeqZ6pziBXu2t
V/y8ez66V/8KwzAMwzAMwylI8fOUX6jXediHHiyyLqS45ZYX8PQ1s60vlW2/
5cxkvPuuIueFLb7jtHAPCK1PSc9UX26czNR0z+nT/XEtPv2nwg9WWRfSuXv6
PbXbF54voqTy6Vw8PZfygNF+Ir0n6XN0fH3i88MwDMMw3GlObVf0ytdK5wWo
fNp/cc9l33VnkXPC48eKXBa6947Rc48eKbIipHvH3PUOndd217nuvWlEiuuj
+EB3H6rXefwwDMNwd7BX/DzZJ73iJah8eh+yf6h8ikei31McSy97IAzDMAzD
MJwXR+1Xun5Ct3z3PKxrh99yc5FV4aur7Be2+y2PCylPPuXloOe2OIKHhX9Q
5V4h6ed9VVaFLb7+x8L2/KNCystH6xdqD+49m65f1I3b79X+EycQhmEYhmEY
LjJpXUbrCLd82qeguIIWL78lpHjmP6qyIXxHlZdPKofWaxTnTL+n8p9/rsia
8D1VLghfU+XfhBQPQ+svWgeNygtKpLgRWt/R+Yt2L8FPhPRcVw9Z94VhGIZh
GIZhGIZhGIZhGIZhGIZhGIaLzF7x0rRf7+6zUzwAxXVQ/pCW//67QsprR+9J
cTJ/W+WSsOXjf0zY7h29LKT4mfZep4Tt/74mvPeeIheFpOcPVdkQ0u97ne8I
wzAMwzAMwzAcRVr33X9fkRXhq6pcL3xblTUhra97rZenZuLzwzAMwzAMwzAM
wzAMwzAMwzAMwzAMw/Da2eu+8l750un3p08V2RRS3PiB/UXOCds+/8+Fv/5V
kUtCute+xRdsCA8fqiL8ZJVLwv1PFDkrbPcMPCmkcijvJemZ7hVNHH4YhmEY
hmEYhruVic/fnonPD8MwDMMwDMMwDMMwDMMwDMMwDMMwDMPpSPvvbjx/r/z8
FCdAv6e89zd9scgF4YPfLLImvPUrRX4kfHOVS8Lb9xXZFL70YhHlx6qcE1Ic
vqu3d1e5Iuz1+zAMwzAMwzAMw7kw8fnbM/H5YRiGYRiGYRiGYRiGYRiGYRiG
YRiGYTgd3bgCt5xe++/ue7rnDm68ocg54fuqrArd76K89/9YZVP46CNFtoRf
vaPIFSGdUyC65yDCMAzDMAzDMAznwsTnb8/E54dhGIZhGIZhGIZhGIZhGIZh
GIZhGIbhdGzyC6Ebn0/77H9XRUn77O5z31BlS/iRKleEv/1NEaX7XddV2RRS
HD6VT/pxyyHS99Jzd7r9hWEYhmEYhmEY9mbi87dn4vPDMAzDMAzDMAzDMAzD
MAzDMAzDMAzDcDq68fNUDu2zu3ELdF6A8sn/SZVfCKkcev8Xni+yLnT1cOZ0
kS0hnReg96fy31jlitDNh0959RO3H4ZhGIZhGIbh3Jn4/O2Z+PwwDMMwDMMw
DMMwDMMwDMMwDMMwDMMwnI60P94rPt99H4pXf3eVK0Iq58UXimwI6fcUP096
oPehOAeKh6e4DnrPd1bZFH64ypbQ/d5FixsJwzAMwzAMwzB0mfj87Zn4/DAM
wzAMwzAMwzAMwzAMwzAMwzAMwzCcjrT/Tvvg9Hs3zpzo5pP/QpUrwtOnimwK
6T0pPuFdVbaEFG9P7+nGIVD+f9I/lUPf6+o5DMMwDMMwDMNwLkx8/vak9S99
F8Xnt/vcHhe699m57zO1Puleud/+psgVobt+p3LoubR+d/0wpLfcoxeGYRiG
YRiGYRiGYRiGYRiGYRiGYRiGYRiGYRiGYRjOkaPygVNcLsUVu/HPFIdM5feK
B3bf8+YvF1kRtnj8fxF+oMqa0D03TfHYbjmj2OLuzwhbnP7PhBRvT+VTe6D+
4p67/+odRZRTx6W7/YJ+75L0RvqhvAeLdq4kDMMwDMMwDMMwDMMwDMMwDMMw
DMMwDMMwDMMwDMMwDBeBU+fNdvON97p/zY1vv+/eIpeEe0A+XWV179WkeGZX
/9dar6+UH6lyRUhx7H9c5ZTwj6r8REjlu/U49XkWipOn92/nEpTUHige3r1f
wNUbtTeqX/qu3LsXhmEYhmEYhmEYhmEYhmEYhmEYhmEYhmEYhmEYhmEYhkqK
46U4eYrXnTqu2I3TduO3H7i/yKrw9VUeFJ48UWRd6OqT4r1Hxe27emt/f1K4
/4kqwm89WuSKkPRGceMtH/+W8F1VlFPHk/c6L+CeT3HP17j3DtA5mkU7VxKG
YRiGYRiGYRiGYRiGYRiGYRiGYRiGYRiGYRiGYRiGL4dT5wmn8qeOv6XnUhw1
xbG7cdq37yuyKfzSTUUuCn+/yk3C554tcllIedFJn4uWh/zzVbaEv3mpiLKd
zzgjfFWVJ4TvrnJF6L6nqzdqbxR/Tt975nQRJZ07oDj5XvdE0LmYXu1q6vEn
DMMwDMMwDMMwDMMwDMMwDMMwDMMwDMMwDMMwDMMwDHeSo+JjKZ85xZl/ocoV
IcXPUx7+XnHL7vmCT1VZEb6uyj8Ljx0tsiqkcwS93nPR+NEqZ4WtPh8X7rut
yIaQzk2cPlVESeVQ/HyvduXG27v3QbjvQ89dtHMfYRiGYRiGYRiGYRiGYRiG
YRiGYRiGYRiGYRiGYRiGYbgIHBWfT8+l+GS3fIpP/nCVLeELzxdZF769ivKz
VZQfrKL8RJWLwpYv/x4hvY97HwGdaxgVt0/nC6he2nmFY8KWL/964aurnBO2
73tS2OLrTwtv/UqR48Lfq3Je+MD9RVaFjz5SZEs4tZ6p/VMefirH7afueNLr
HEEYhmEYhmEYhmEYhmEYhmEYhmEYhmEYhmEYhmEYhmEYLgIpnnZU3D7lvXfj
eCkv+luqrAjfVOWs8NTJKsLrqpwSUhz4bbcWOSHcA/L6Kut7rybF1RObKKeu
R+Jvf1PkipDe/0+rnBO2uPv9ws9V2RA+8nCRTSHlw6d8+0+eKXJF6PYX0gOd
p+h1buVdVbaEbjtx2yHl4e9170AYhmEYhmEYhmEYhmEYhmEYhmEYhmEYhmEY
hmEYhmEYLgIXLT6f4pCJbpzwV+8ockVI+duJv3mpiJLiwD9Z5ZKwnT/4D+GD
3yyyJnTjmRct/pni2+n8xR23Fzkt3HdbkZ8I6bnufQGj2v9upTvOLJr+6X2o
3eZegDAMwzAMwzAMwzAMwzAMwzAMwzAMwzAMwzAMwzAMwzC8dlIc+P33FVkR
vqrK9cK3VVkTUrw9xQO7celTM/H5y8m5x+f3+q7R7xuGYRiGYRiGYRiGYRiG
YRiGYRiGYRiGYRiGYRiGYRiGc2Li87dn4vOXk73i8xMP/8r0PPq9wjAMwzAM
wzAMwzAMwzAMwzAMwzAMwzAMwzAMwzAMw/CVMPH52zPx+cvJXvH2VI9Ealdz
r9+cXwjDMAzDMAzDMAzDMAzDMAzDMAzDMAzDMAzDMAzDMAyXgYnP356Jz19O
9oon/7sqL5/UX3rF84+Kh8+9A2EYhmEYhmEYhmEYhmEYhmEYhmEYhmEYhmEY
hmEYhuEyMPH52zPx+cvJucfn9+LUepvL+YIwDMMwDMMwDMMwDMMwDMMwDMMw
DMMwDMMwDMMwDMMwfDlMfP72THz+crJXfDjV46h8+FPHt7tx+C4Tnx+GYRiG
YRiGYRiGYRiGYRiGYRiGYRiGYRiGYRiGYRguMhOfvz0Tn7+cnEs8/KIx8flh
GIZhGIZhGIZhGIZhGIZhGIZhGIZhGIZhGIZhGIbhMjPx+a+Mf1flF8LTp6oI
31jlceGfVPmFkJ7b5Bcvm6RPen/i1Pqk9kDv8+4qV4RNr0pXn269zKXduhx1
TiF8ZRzdXq7m1PdBuPOI+z6Lpjcal756R5ErQir/t78pckXovudvXiqyJaTf
03hOdsIo/ffi6PFgt40n0X/f751a/zSO0fjjjmN0ftb9XnpPsg+pHBqfqRyy
Y933TP9dTI7W307XS6/7wka9v3teeNR70rjnjleu38AtZ2r9jNL/XDhaf8uu
/7lz7vXea9ym8ZbsWPr9KH2O4uj2O9f2H/0vpv6nHmdoPHH3ZZZtnbhbObr/
7bb+G/1v/11zWf+6681HHymyJXS/q9f4PBeObr8ZfxZDj8tKd1+y1363639z
xx/6PY2rH6lyRbhbx71eHN1+M34uhh53Sv+LVr9unAmNt669F/9b4+j2u+wc
Xf9pP33oxhnSOEZ07b1PVrkkfF2VW4TPPlNkVTh3+82trzYPnBHecH2RJ4RT
62fqcYPaFc2nRHcd5O6LkR7eUWVDeF2VTeEoPY/i6HEx9Lhb24k7P849Pp9I
7++On/9YZVP4+SpbQirHXTft1jh84ujxYK7jSfT/yvQ/qh7d59K44drDFA9P
48zt+4psCt9ZRdnL702/d/1XNK+l/y4mR+tvp+tlLvH57ndNXb9u+fR7Gg/P
nC6yJfxQlQ2hez590epx6nqZC0frb9n1P3fOvd5bfpRNIa1/e8Wn7dbzSi5H
t9+5tv/ofzH1T+/Zy/7p5d/7cJUt4amTRTaEo/Ua/m6O7n+7rf9G/32/d5Se
yW9G69w/qrIhdO20XnmN5sLR7Tfjz2LoMfz/SfXVK08jPZfsQ9q33XdbkQ2h
+z5kf/bKs5r+u5gcrb/ov8/3jtJzr/1TKp/i/T5XZUO4W/cp3HoJd4aj63/Z
28+ixQFS+a5/771VVoSvqXKf8MTxImvCXt81Kj+5a3+2fO1ngJo/n/Jnkh/g
C1WUbn6nqeMk3Xz4rt/D7adU/n33FrkkfO7ZIpeFmd/DReaitYde5bv7Vq4/
Yer3H1VfFGf1mSqXhTTu0bjdK//M1HoYxdHjwaKPJ9F/X/1PXY+94gd6xX/S
c6n8T1dZFd5ycxGlO4+4+32u/4rKT/9dTI7W307Xy9zj86em609w2xXluXr+
uSJrwiOHi1wSuvci7dZ6HD1+LPv4M1p/y8651zutc48dLbIqdO8zIrr2cMbP
xeRo/UX/i6HH/0mfbv91f0/2Ia1z315lXdj261aEu3X8mTtH97/d1n+j/7Hs
tf791qNFrgg/W2VdePRIkRUhlUPvv2x5WUe3393W/pdN/3OnW19TxymRvUfx
qAcPFDkv7JVXjb7XvQd5t3J0+834uRh63Cn9T12P7j6su6507SvKv0HrXHe/
de4c3X6XnaPrf9nbj/u9vfIA0zhD4xvt25K9R/nzX1/l60KK6+t1fmHR/If0
XXfcXuS0sNmnjwtdO3lq/YyyH6aO26R+ceMNRc4Jn/l1kRXhbh2HiaPH12Xn
6Pq/1vbQq/zE578yUrwBrV/eUkVJ/hbXnqFxfi7t3+Xo8WO3jT/R//bfNbU+
3fuS6D3dfTH6vTvO3HN3kQvCT1S5KKR1Fs0vrh3r3iPg6m3uHN3/ln38dOsl
8fnb0z237u73kb33/iqXhX9d5bzQvUdpt3L0+LHs489o/S07517vX7qpyEXh
r54uonTPJZEd6O4DuvH8c+Ho9jvX9h/9L6b+6T1dO9YdH8gOpH0Nsvc+VuWc
cC76XzaO7n+7rf9G/2Pprn/p97Qf8cD9RVaF76lyQejGkS5b3Ono9rvb2v+y
6X/u7FVfvdoD7RfQuaSWZ+iokMYxd1+Dft/yj14R7tZ+ShzdfjN+LoYee+t/
VH312v8luvYV3Yu0/4kiZ4XJDxkuQv+dC0frb1HGVTfObep97S9/qcgl4R9X
2Sd89pkiq0KK/3f1Nmof390fb9/3pJDi8+ncBPkraH6h+bHXeV53H8q9J9qN
s3W/a/v4hyeFr62yKuzVTubC0ePosnMu9d4r7t0t3x03RsXnT11frh97+/tx
lO76a9ni7oijx4+5jj/R//b6H1Vfrn+GxgG6B4qe6/6e7D3KJ/P0U0UuCN35
xfX/uPZ88nfNi6P1t9P1kvj87enGJ7j6J3/LB6qsCSlei8bbZauv0ePHso8/
o/W37Jx7vX/xxiLnhffeU+SikMZPGvfoPBTZh/S9u3VcHd1+59r+o//F1D+9
Z697JN19QLIbX1flsrCtF88IR+s1/N0c3f92W/+N/sfSXf/Sd5Gd9qkqK0LK
w0bl0PsnP8a8OFp/y67/uXPqdZkbN0LvQ/H57Tz6IaFrZ7r7v24+t6x/F5Oj
9Rf9e981tT57jQ9Uvru/SePedVVOCek9p9bbKI5uv8vO0fW/7O3Hzevo6oHK
p/OJ7rlIKp/y57+qyjeE/1BlTTj3+cidL1o9nBE2f8B3hG+tsiqke+cp79N9
9xa5JKRzE/tuK7IhdP0Vrp3v6p/82OTPcffF6DwF6Y3KceMr5sLR4+uyc9Hq
3V1f97o/xX2fqZ87qr7cfTpaj9z85SIrQvf+a9LDsq2DiKPHj0Vvz9F/X/1P
XY/uOSn6/dT3StP7HDpY5ILw8KEiF4XuvqFrB5J9647zvep30Ti6/y37+OnW
S+Lzt6drr7r+cBofnn+uyJrwhuuLnBVSOa5/YO4cPX4s+/gzWn/LzrnX+8er
nBeSv5Teh+zJ5AncnqPb71zbf/S/mPqn93TtWPf8tfvcr9xS5LKQ9jtG6zX8
3Rzd/3Zb/43+x7LXfjGVQ/save4Nif9tXhytv2XX/9w5dTx5r33SE8eLrAlb
3NBhIX0vvY/7nu5+0Oh+NhVHt9+Mn4uhx53S/9T12Gv/l8p37UDKn9/yRp8S
TtXOFpWj2++yc3T9L3v7cf1ppAe6V/eDVdaFlN+b4rqJz/y6yIqQ9m1/v8pN
wlu/UmRNSOOzm6doVPukOCKq3w9XOS18Q5VHhG+psiKk/E5/X+WCkOKOqH5P
niiyLly0/ABu/JKbv7T5V54Ukn+b6t09FzAXjh5fl52LVu+9zt3T73vpYXS9
TVWPvdZH7vkv109FeQVHtedRHN3uFqXdRv87o/9R9eh+lxvn6Y5vtC6jOPw3
V7kkpPPO7vxI3+Xaq4u2Ppqao/vfso+fbr3MPT5/VP1S+WQ/u+fW31ZlTXjH
7UVOC+k9e/m75sLR48eyjz+j9bfsnHu9331XkfNCNz6/13ml3K89L47WX/S/
GHr8n/TZy46l9Sb9np773LNFLgvv/EaRs8LReg1/N0f3v93Wf6P/vt87tZ7d
fYejR4qsCCk+v1e+ptHtdCqObr8ZfxZDj8tKqpde+9du3DuNV++vclnY4laO
Cd3zob3itZLfY14crb/ov8/3jtJzL38XjUunThbZELZ18xkhlb9o+1C9OLr9
LjtH1/+yt59e9tXnq2wJKe79vVVWhBRfR3EpFB++fZ4NlW8+UOTy3qtJ9hiN
t4s2Trpx161dPCls3/1z4UeqXBFSe3PjH3qNG1SPRPe5tP/l9i/3fIEbn79s
54JHj6/LzkWrdzc+340jmloPvfTv6mfUd9HvKR716aeKXBC67/PoI0W2hIvW
zqfm6PFjruNP9P/K9L9o9UvjYa91E5Vz5nSRLeEfVLkkpHUTjWO94o3dewHm
Eufci6P737KPn269zCU+37XfRr0n2clE0vNnqlwWtnwzJ4X0Prv1HDpx9Pix
7OPPaP0tO+de7xSfT+tf8me69iqR7MlRep6ao9vvXNt/9L+Y+qf3HBWfT/Ye
xedTnOpovYa/m6P7327rv9H/9t+1aOtfsruI999XZEX4p1XOCd3yR7fHnebo
9pvxZzH0uKykenH9nFSOO86Q34/OJVG+UPe7et17QvfNje5nU3F0+834uRh6
3Cn9L1r9Up5G154kv9k7q2wK31TlrJDK36323uj2u+wcXf/L3n5o/HH7u5sH
kuKQe+Vv/PKXilwSvrrKncJ/qLImdO26Xnbg1PxClSvClof+hLDp77DwpReL
bAqpXqhdkd0+dT91/TlTj2PUTiiuoD1fz1McO1pkVUjnKeh9euVnGDX+u+cd
SM8tL+OWkOqL9NzunTgtpPtHqL+446Tb/l099+pH7rzj5r2n5z77TJFV4Yeq
nBTSc931O82P7j7XqPWRW++95lOKT6B7W9xz0G490vw1tX7c+cKl2++IB/YX
OSek+93cc3NTz1OjSPOO+72nTxXZFLZ785QfrXJW6Pajqef9qcclN18Ktf93
VdkSUjnUH/93lQsvm1Q+fW+zR7aEbr3MJQ6W2o87rpK90ey+M0K6/4vOZbh2
0XVVNoX0e3c8d+frXvOdew6ayn9HlQ0hrX8p/wDp0x03iFPbCb3qkfoR2bG0
3iE9UP6EZpedEdL7uPfZjZ7/d3p9TfVL9vD29xGoX+LFF4psCEfraSr28vu5
4z/5dam+KP7HzUPirlvJT+KOn+56vxfdcZjsB5rX/leVC8K/rXJJ6LYT1x9I
fmP6/ahzba5fiNqbu0/R/KxHhWTX0XNdPVO7Gj3+XSvdc3w0XtE6t93nfELo
rtd62VFT5ylyxzEqh9otjf+uv5TuS6L8+bvV/+Ouo915kMohO5ne50s3Fbko
pPxsZP+747bb/t123ms922s95dYj9a/m/zsldNuDqzeaB93xamr7bWpSe6Z8
g3fdWeSckOrFvf/XtR/cfufWb69+0SuugPRDfnKKJ3HXg1PPa73qkZj991fW
zl09u/akOy7Rc3vN16+rcllI/the++9kx7p5h1z2mo/c8mk8dOf3v69yQUjz
F5VDenb9J73m5V71OPX+Bb3PIw8X2RTS/jvp3/X70XeNXj9dK2k/1LUraH75
bJV14V9WOSuk5/ayE0btN7l2EfUXols++TH+pspF4SerXBJSP6JxzJ033XX6
ouUVd8dPV29z33/v5a8gvfXan3L7NfWjZvd9TUj5xul9aN3h1uPUdPVP8ag3
fbHIQeHU9x1PPe/0ilug51J76OU3uOH6ImeFdF+Y2z5Hxef0orvPRfXoxpPQ
/N7uLzoppN/T+rHXuN0rLqXXvpI7npB+aD6icij+Z99tRU4KqRz3uTQOuPEJ
o/rX1PY5jVcUh0/zvjsO9PLzTL0vMGp95Pp/fq/KeeFDDxZZF7rzlDsuzYXu
PguR2j+tI15V5azQtVum3l+Y2q5zx3N6z17nIn+/ysWXTbeduHZOL3uA6K4r
R7U3YvN/nBHSuSQa91w7qtd5kFF6dv0z7rj08ENFNoRtnXRM6PaLXn6P0eun
a61fqhd3/4v8M9S/3PFkLvrvZR+64wnVI5XT8l6cE1J8fvT/yvRPvyd/AvU7
ijNx4yvc9Tu9j/u97vw1al5zeehgkQvC++4tckno+oXcdTG1h176pOe6dO15
am/uuHf8WBUh2WP0nr3Wv6PHv2ul+72kN1rn3vzlIkeEbhwpcWr/QK/1C/UL
+j35u1y/JT2X4hspPn+3tn93Huy1/0j1Qnr+8yoXhc8/V2RN2Ctu0/0u0o/7
vb3am1u++73kx6Zzze44Rr+n93ftjVH2+dT+c+ItNxdZFdI5TdKz27966XPq
/uI+l9qbO07S+EDrpmb3XRb2Wqdk/z377y/HX+r2l1H779Rf3P13dxwm+6pX
/3LbuVtfVL6rH3c8pHvq31tlRUjluO2kV7/rZXe59ejua7j1S3naaZ5y4/N7
tbe5kPwAbjuh8mkfkPLn9xpnRtlpvcbPXus4Kp/sFlrn0v31bn4Pdxyg+drd
j6PyXXu113jbqx3Off+917zfqxzSs+v3o/xCe0AeuL/I6t6r2WveHEXX7npf
laPClm/mp0KqF1pPUZySe86oF6ee32n8dP0nVI90zs5tz9R/p9a/S3cfzbUz
qR7d8Zbs5JY/5riQ3pP6lzs/uvp02zmV766XXX+Ie/6a2vlnqlwW0n6ru2/r
rotdfY7uly+XvfyNdF6V8uq7461rb7j13otT1xd9r+uP/esq54Wvr7IudP2E
O9V+d5rueoFI7ZnySN++r8gpYa88WqPa+aj+6I4nNF6RX5To2v9u/lvXPnH1
5tp7i1bvlKfuxPEia0J3fqHfu/fiTa2fqccB1x77XJUNYTvXckhIftq5xAmM
In2v268pv1b7+5PCXuum0fq7VlJ/d/3hbn/8WJVzQrI3Ro3bU5PGH7deXP3Q
upj8b22ePyMkP4Zrt9Dvp9YD/b5Xe+sVR0R8T5ULwsOHilwUuvY52QnuOqKX
H2ku8yl9b7PjjgtbHNCGsNf82Kv9Lxqp3dLvqX3SPfItj+B+Ya9z8e73Uv3S
76e2n+n3NG738g/QeUyKMxk9z0/FXu3HXVe670nzFN2D4PoZqL254yGND64d
MnW997IfKJ8q2Xvu/WLuOmuU/8HtR736nRtn0s6HrQjJf+7qmd5/7n5a9/3d
cZLK/6//W+SskPLjTW0/TE3XLs3++/brVnf+6tXv3PWdu//uzl9k71F9uec+
6Ltce5X8G1SPvcbVXvaAOx7S7+m+JIrPd7+rl19ltP2+0/YM9Qvah6J1rptP
bC7xYy63vx/22s93UPzDh6ucFrp2y+h2eq3jm1sv9Ht6LumN4ldpn53i9ul8
dK/4fOrvbn+k8l17z9X/1OPh3Pff3d9P7a9w72ujfurG51N9ueMhtZNe5xdc
Uv8l/bfz+keF+5+oIqTnkh07Sg+LRrddkT+T7iVv9yqvCkd/97Wyl13k8qUX
i2wK6ffNntoS0v0U1B/n3l9cu8ilG2dC/Y7i85uf4JjQzedA8xTNv6PrbVFI
eqP4/C9/qYiS/HXUfmi8pfmd/D+j9bfTpHU9rV/affLrQncd4Y4Du5WufU73
67X8gqeEU9+XNBe661a3fIoHpntDPlFF6e6Du/1u2dqDu99KfrYPVFkTus+l
39P7LNs8RXqgPN5/UOWwkOa1Xvbt3O38XqT2Sfm1bryhyDnhso1LU5PaOfl5
7r6ryHkh5S1x5525s9f3uvuYVF+U5+HzVbaE9D7Uf3uNb73sq7mw3VN6QUh2
oBtX7NqHy0Y3DxXdp0znklx/Js1ry7b+dffpKL6r5SE+KKT63a12Wq/vIn+m
u19M/liy90brb6fZq77ccuj3FLfw3LNFLgvd+Drqj1PvL8yddE8cxRW75/XI
fujlrxitv6lIdjjFLdx1Z5FzQnf/aNn0TCR/Ds1HtH9E6yY6Z0H1Phe/XPbf
52XvLdr+O/Uvis9v+4EnhG57oP7u9js3/8DcSfVFcfh0/4vr78o81Uh2r3ue
tO0fnhaS/ebG54eNbjtv543WhHfcXuS00B2v5j5/jdrHpPUp5f9s+4TKqf2r
bvlzbw/Ltv++aPVF3+Xut95/X5EV4Wuq/JuQ8um5djv1l1F+JHe/puXFPyV8
VZUnhLRufeH5IuvCN1TZEtJ70nzn2i1UTq/zXK6fxD1XTvZSy+94UUjxDPRd
rt5cuvFs9D4uSZ/0nqQfd5+62VNbQrqf1/0ud19+6v5Cv3f3E91+58YD0PhM
9ynT/Tg0jtFz3XtDqF1RPfbqd275Lnudq6V8qt98oIjS9av0mq976X8U3Xqh
cyvkrz55osi6kPq1e860V7sdRWpXNK5SOaQ3ihuh/TuqF2o/vewoKt/9Xve5
pH9av/eKhyf7gfrRW6ooya529UN01xduPbp2Qq/vIrrjJ+WRfnuVdWGveYTs
EBqf3Ti0RaNbX5RXkM4lUX93xw16H3dcdfXQq53T9/byl9L4SXlm6Nyfa++R
HkbnlblWUr24fjYqh9Y1dN9r2w/fEFJ9jdbftZLaG30vtUMqx7WT6bmU/5b8
HvSe7jl0d9wge6NX3phe836v8ZnOm9P+uHvfAc1rvfLU9Zpfeq0L3O+i59I4
+cUbi5wX0jrLXcfReEvv6ba3RWOvfkd27223FjkhdPfXSP9Tr0fc8Y30447D
vewH0hvtn9J+Ry/9LBrdduiOz+54S6R5ivKeuXn/yG5x+x21f/f8SK9xidir
Pe+7rciG8NDBIheE5Hei8imPBH1XLzu21/g5df1S+TSPt/vqV4U07tF6md6n
1zqLynfHDbedu/VF7c2NJ6Hx6m+qXBTSPpRrf45qz73mBdffkv33vv1i7vvv
N1xf5Kyw3ee8KXT9Bu7+u7tepvrtNY+4dO1M+t6/qHJZSHFH7nlMd57qpede
416veiS9uXnd271w54SkZ7LrXHugV7sdRXc8pHohvTV/w7rQzTtE7zN1ex5V
773ieWj+ovJp//1TVZRT70O5+/v0vVSP2X9vdJ/ba//dXS+445W7b+XGCVCc
ZMuDckG4B+QPq6ztvZru+9P4Rr+fmq491vR3Wtj8Af8qbHnZjwrb3w8JKR91
OydxRNjyRR0Utv1DJeXDod/TezZ/1UEhld/uGzwgPHK4ipB+3/yah4Xt/qhD
wt+r8l1hy0v3EyG9D30v6d+tl1Gk+j14oIqQ9N/8LoeER49UEdLvWxzx14T0
nu53Eandkh7ouW690+/d9tb2OQ+8bLb73g8J6bnNf/MzYYsD+rqQ+u9rqxwV
uuNMy5tyWLho/dF9n15s65XvCJu9/AMh3WNO7Yf6Ra/+4vaLqUntk8Yxap+k
z1beY0Lqd/Rcd9wb1T57kdoVtWeyo2jep3Jo3GtxQEeEbn3R97r2m9vOe9lv
7rhBeiO7jt6fxjfqX1QO5Qmnfk3v2cbd/UJ3fFi0cY++l+wK0uerq9wppPvI
qL3Rc6nf0fu0/Y0nhFSOq89e455bX0TSG40D7dz5R4VuP3Lra5R+3PGZxmHS
J40D1M6pnLaP97iwrZ8eEFL/pfmRfj96/u9jJ+v4TN9L9Uv1SGzjyreFLb/g
E8JFW19MPY41P80BIemBxhl6Lv2exp/mH3pISPYMtR8ijTNkd9H7U3vuZf+7
7ZDql/Ts2g9Nf48Jm1/q+0IaP2ncI72585Q7H/WyH3qtZ93xn+qxPeceYdsH
+7mQ+pfrv3LXEXMh9UeyH6gfUfm0H0R+Xbe/j7KT3fU1tUN3XU/tlton1SP5
hVq+qIeF9NzR8/9UdOuL+pE7b27vB1B/LNntVF9kT9L7u/1ulP/B7b+95jXy
M7R93UeEpB/Xf0jvSeWMsv/pub1IeqB5ivpLG6e/J6TyaZyndYdrb9D3Lpr/
lsohPxjpk8Zbmo9a3uj/FNL4Sd+1aO25V/vP/vv2dhc91633XvOXu//urqfa
/PMzYdPT3UJ6LrUH1x/r+scWbb3cyy9B83vrNz8UtvzfPxJSOyd7j+YpNy5i
1Lp1ajuf2ieNn7QPReX8VZX9QtK/2/7nQndcJX1uvz+4X9j6vfqR3PjAqfUz
dT8ivdF8SusLap9UDj237Qd+W0j7HVQv7jhG7+P6D117Zur6pedm/317P4Zb
X+76gt7HHefb+fKjQtJDWz/tEz77TJFVoXu/AJ0jGBWfT6RzNHRvMs3Xrt+G
9onI3u7lZ+61/qX3JP1QP6XxgcZPGh9ov9K1b+m5pDeX17rOf6WkeqT6ovGH
9EbjJPULWoe67dydB6lfkF/L9Ru47+mW7/oH3PmO9E/7qqS3Fi95ROjaz64f
z9Wny17jgEuaL8iP7a5np96XnHocm3o95a4XqBya72ieIv27+2uj2m0vkh1F
cW5uPDm1Z/Kzkf+T3r+X/Ub9wm23NG6Tn8T1HxLJr0LfRc8le4/mqe3jiF7+
Pi/pmewoei7pedR+Ez23l9+g5cP4sZDql/pFL//G9v7Aa4/TnrpeXD2QPkk/
VF/0Pq6fzd336aUfl/RcGp9pfqFyqL5cP0DLo/AjYa/4n7mT7C5qh+55Z1rv
vLHK40J3vh6tv2ula+fQ72lepue66xSyJ919Q9d+oHs5t89HofZVr/bj2ue9
7AdaT5F9RXagu2/oxmXResSdjxZt35Dqi76X5pG2n/BTIfUXGp/d9kO/Hz3+
XSupndO8Q3ogtvxAPxTSeWqqx172M40zvfTp+oFd+5DGMfq9O/6Q38+dH+dC
dz4ivVH9kt5cPyrtt9L7kD/EtTfccxk0npD+e8XzuJx6fdfytP1c6NaXaw+4
7W1UP+pF1w/g9i9338H1n8/Ff+vuv5Me6Pdk71H7p+9148l7tcNe/chl9t8b
R+2/k37c8crdf6fnuvYkzVNUPo2H7vn0Xn7yXn7sXvaG6x9w/Q/kn3HtSVoX
kx3ijnu96qvX+s71q9M8Re9D4x6Nt65fbrfuv7vjMI0/bpwD+c/dPMaj1i+9
SO2c5k3SP9Uj2W/kX6L9Jnefl/oLjavuOteNd6J5Z+r6zf779naFu09EdPNJ
Un25/Yjep/Wb48J2n/ZJId0DSPHtdG+Fe2/a1KT3cfP803189Ps2n1wR0u/d
cxDuPZ5u+UQ6f+HeY0L3XJCe6X4KKt+994HKJ731oqv/XnTviab3p3ug3Pdx
9e+W795L1eteFfc+lKnr3e0XROq/9Hv3/jv3nppe+nT7r3v/kUt6T/e+PCKN
A6RP9x5e9/4al1OPt1P3U/c9aZ7tpZ9FI9lv7rzz0otFNoVUv679RveWuvXu
jgMuabx1xxP6XtdOdvVG+nHLIT249825eqP7+HrZ/+69e+747L6n2x7oXmYa
B9z3oflr7nomuvcJ9tKb+/tR+nHrl+YLd1wl/bjj2KOPFNkSut9LdsXo+f9a
6d7fTfp3/Q/uc3er/eb2U9f+cedrd3ygctx7damcd1XZErrv436Xq/9R47Pb
Htzxmcqh9+9l5y9avyO66yy3fX64ypbQrS/X7poLXT8b2RW9xg2yE3r5FXu1
Z/q96wd22xv9ntZBrp28bHZCr3HM3Zd066UXqX+59iTRLcddH83Fr+uOw259
0Xjrjj+j+leveiQ9uOtisrfdeqfvde2NXvs4Luk9XbuUxsP/af001bqDyl+0
eaQXs/++/bjXa//d/d6p673Xc3vt49D7uH6JqeOOXHu+13qk1z67W+/uuo/a
w9z956696tYXvU+L398Suv690euknbYP6fenTxXZFLr+RqpHshtH2b29/D/0
ve5+Df3etavJX+fGS1B/d+0Zdxzrta5320/231+Z/qdeT7mk+qJ5ivRP3+Xu
K1E5tC88yu4l/j8YuZ4l
   "], {{Rational[4362, 247], Rational[-5330, 21]}, {
    Rational[18586, 247], Rational[9530, 63]}}],
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  ImageSize->{508, 319}]], "Output",
 CellChangeTimes->{
  3.575539653880105*^9, 3.586581224330914*^9, 3.5869672257602973`*^9, 
   3.58696923552244*^9, 3.5870926583733797`*^9, 3.587093285051573*^9, 
   3.58709335219763*^9, 3.5871248203998847`*^9, 3.5885695634477153`*^9, 
   3.588579719595878*^9, 3.5885797901218395`*^9, {3.588580233899251*^9, 
   3.58858026835563*^9}, 3.5885804858112144`*^9, {3.588580636089305*^9, 
   3.58858066005835*^9}, 3.5885807333587027`*^9, 3.58858096431511*^9, {
   3.588580998829385*^9, 3.5885810265269313`*^9}, 3.589076629895391*^9, 
   3.5890766643847475`*^9, 3.5890767822136617`*^9, 3.5890768461687064`*^9}]
}, Open  ]],

Cell["Show combined images", "Text",
 CellChangeTimes->{{3.587093425431419*^9, 3.5870934298779507`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"figa", ",", "figb", ",", "figa"}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", "\"\<shapes-stanford\>\"", "]"}]}]], "Input",
 CellChangeTimes->{{3.4304770076200123`*^9, 3.4304770111270123`*^9}, {
   3.4304780198210125`*^9, 3.4304780221070127`*^9}, 3.4304902087750125`*^9, 
   3.4309105005336*^9, {3.5017429815565577`*^9, 3.501742984363718*^9}, {
   3.5017430408199472`*^9, 3.5017430441321363`*^9}, {3.501743126068823*^9, 
   3.501743154524451*^9}, {3.5017432380612288`*^9, 3.50174324088439*^9}, {
   3.501743299564747*^9, 3.5017433024129095`*^9}, {3.5017436890130215`*^9, 
   3.501743695403387*^9}, {3.5017442970598*^9, 3.501744299938965*^9}, {
   3.5017444174116836`*^9, 3.5017444226359825`*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{
  3.5755396540871305`*^9, 3.586581224587961*^9, 3.5869672260943365`*^9, 
   3.5869692357234635`*^9, 3.587092658594407*^9, 3.587093285273599*^9, 
   3.5870933524076543`*^9, 3.587124820526922*^9, 3.588569563616741*^9, 
   3.5885797197459283`*^9, 3.588579791898164*^9, {3.58858023402826*^9, 
   3.588580268478606*^9}, 3.5885806691165237`*^9, 3.588580969520706*^9, 
   3.5885810306884313`*^9, 3.5890766300864706`*^9, 3.589076664518764*^9, 
   3.5890767823436832`*^9, 3.5890768463007545`*^9}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJwUmXk8VG8bxlXWkpJKKFmKsWRfxn4ZhhnGLJYZCVmmslUoJZVCi1CWtFBJ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      "]]}}, RasterBox[CompressedData["
1:eJzs3WevZleV6Pt9db8A2cRunW/QfZqmoQn7K5AzfaQr3dcXOgDdhMNLXjqU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    "], {{Rational[4362, 247], Rational[-5330, 21]}, {
     Rational[18586, 247], Rational[9530, 63]}}], {{}, {}, 
    {GrayLevel[0], Dashing[{Small, Small}], LineBox[CompressedData["
1:eJwUmXk8VG8bxlXWkpJKKFmKsWRfxn4ZhhnGLJYZCVmmslUoJZVCi1CWtFBJ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      "]]}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  AxesOrigin->{20., 0},
  Frame->True,
  FrameLabel->{None, None},
  Method->{},
  PlotRange->{{18, 74}, {-250, 150}},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.5755396540961323`*^9, 3.58658122459194*^9, 3.586967226103339*^9, 
   3.586969235733465*^9, 3.587092658603407*^9, 3.5870932852825994`*^9, 
   3.5870933524166546`*^9, 3.5871248205319014`*^9, 3.588569563621738*^9, 
   3.588579719759901*^9, 3.588579791907091*^9, {3.58858023403123*^9, 
   3.588580268482663*^9}, 3.588580669139507*^9, 3.5885809695406637`*^9, 
   3.588581030696433*^9, 3.589076630095421*^9, 3.5890766645288095`*^9, 
   3.589076782353709*^9, 3.5890768463107634`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Section 4.5 Application: The tides of Io", "Title",
 CellChangeTimes->{{3.430557142957*^9, 3.430557151575*^9}, {
  3.5017457331459394`*^9, 3.5017457453136353`*^9}, {3.5017458852716403`*^9, 
  3.5017458857216663`*^9}}],

Cell["\<\
Notice that the object in bound rotation in all cases is called the \
\[OpenCurlyDoubleQuote]planet\[CloseCurlyDoubleQuote], and the other the \
\[OpenCurlyDoubleQuote]moon\[CloseCurlyDoubleQuote].\
\>", "Text",
 CellChangeTimes->{{3.587094688904015*^9, 3.5870947351435633`*^9}}],

Cell[CellGroupData[{

Cell["Tides on Io", "Section",
 CellChangeTimes->{{3.5017459099690533`*^9, 3.501745914664322*^9}, {
  3.5870951803899875`*^9, 3.587095180598011*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.43055715363*^9, 3.430557156328*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(32\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.575539815431489*^9, 3.586581226401206*^9, 
  3.5869672267474203`*^9, 3.5869692360204983`*^9, 3.5870948041498427`*^9, 
  3.587094875186367*^9, 3.587098749588394*^9, 3.5870988776887627`*^9, 
  3.587100307142278*^9, 3.588569564086836*^9, 3.588579719928026*^9, 
  3.5890766302794805`*^9}]
}, Open  ]],

Cell["Parameters ", "Text",
 CellChangeTimes->{{3.587093552070613*^9, 3.587093557286237*^9}, {
   3.587094376198496*^9, 3.587094415673232*^9}, 3.587100338895089*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"G", "=", "GravitationalConstant"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DD", "=", 
   RowBox[{"421700", " ", "Kilometer"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "center", " ", "distance", " ", "between", " ", "Jupiter", " ", "and", " ",
     "Io"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", 
   RowBox[{"1821", "Kilometer"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"radius", " ", "of", " ", "Io"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]0", "=", 
   RowBox[{"3.528", " ", 
    RowBox[{"Gram", "/", 
     SuperscriptBox["Centimeter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"average", " ", "density", " ", "of", " ", "Io"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]1", "=", 
   RowBox[{"2.0", 
    RowBox[{"Gram", "/", 
     SuperscriptBox["Centimeter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "average", " ", "surface", " ", "density", " ", "of", " ", "Io", " ", 
    RowBox[{"(", "guessed", ")"}]}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", 
   RowBox[{"8.93*^22", "Kilogram"}]}], "  ", 
  RowBox[{"(*", " ", 
   RowBox[{"mass", " ", "of", " ", "Io"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"m", "=", 
   RowBox[{"1.89*^27", "Kilogram"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"mass", " ", "of", " ", "Jupiter"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4305571681029997`*^9, 3.43055718919*^9}, {
   3.430557223604*^9, 3.430557343773*^9}, {3.430557390789*^9, 
   3.430557400842*^9}, {3.430557457889*^9, 3.430557470174*^9}, {
   3.4305584975480003`*^9, 3.4305585269700003`*^9}, {3.430558608527*^9, 
   3.430558616646*^9}, {3.430558659685*^9, 3.4305586695220003`*^9}, {
   3.4305593321099997`*^9, 3.4305593404449997`*^9}, {3.430559648876*^9, 
   3.430559710543*^9}, {3.430732202724*^9, 3.43073222817*^9}, {
   3.4307324801429996`*^9, 3.4307324905950003`*^9}, {3.430732718527*^9, 
   3.43073271935*^9}, {3.5870935605096254`*^9, 3.587093610618637*^9}, {
   3.587093676638558*^9, 3.5870936836584005`*^9}, {3.587093891546343*^9, 
   3.587093893242546*^9}, 3.5870939744452906`*^9, {3.5870940538838215`*^9, 
   3.587094067280429*^9}, {3.5870941382209425`*^9, 3.5870941648231325`*^9}, {
   3.587094220188777*^9, 3.5870942343604765`*^9}, {3.5870943518355722`*^9, 
   3.5870943679225035`*^9}, {3.587094440440203*^9, 3.587094611993788*^9}, {
   3.5870946491142406`*^9, 3.587094664587098*^9}, {3.587094792468443*^9, 
   3.5870947968899713`*^9}, 3.5870948662192907`*^9, 3.587100086349785*^9}],

Cell["Distance variation in an orbit", "Text",
 CellChangeTimes->{{3.5870953168233557`*^9, 3.587095319237646*^9}, {
  3.587100410831721*^9, 3.5871004127009425`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]D", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"423400", "-", "420000"}], ")"}], "Kilometer", " "}]}]], "Input",
 CellChangeTimes->{{3.587100090097234*^9, 3.5871000950098257`*^9}, 
   3.587100466446392*^9}],

Cell[BoxData[
 TemplateBox[{"3400"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.587100101215568*^9, 3.5871003074573135`*^9, 
  3.5885695642228417`*^9, 3.5885797200409784`*^9, 3.5890766303984547`*^9}]
}, Open  ]],

Cell["Gravitational range", "Text",
 CellChangeTimes->{{3.587095333542364*^9, 3.587095336421707*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hg", "=", 
  RowBox[{
   RowBox[{
    FractionBox["3", "2"], "a", " ", 
    FractionBox["m", "M"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["a", "DD"], ")"}], "3"], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{
        FractionBox["3", "5"], 
        FractionBox["\[Rho]1", "\[Rho]0"]}]}], ")"}], 
     RowBox[{"-", "1"}]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.430558205119*^9, 3.430558212443*^9}, {3.430558328349*^9,
   3.4305583752209997`*^9}, {3.430558462657*^9, 3.430558468552*^9}, {
  3.430559362685*^9, 3.430559369196*^9}, {3.430559463399*^9, 
  3.430559471958*^9}}],

Cell[BoxData[
 TemplateBox[{"7054.661579333851`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.57553981600556*^9, 3.586581227232294*^9, 
  3.5869672275955167`*^9, 3.5869692370326195`*^9, 3.5870948048919325`*^9, 
  3.5870948757994385`*^9, 3.587098750175464*^9, 3.5870988782728324`*^9, 
  3.5871003077283497`*^9, 3.5885695643958673`*^9, 3.5885797201909556`*^9, 
  3.589076630555478*^9}]
}, Open  ]],

Cell["Centrifugal range", "Text",
 CellChangeTimes->{{3.587095340135154*^9, 3.587095343957611*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hc", "=", 
  RowBox[{
   FractionBox["1", "3"], 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     FractionBox["M", "m"]}], ")"}], "hg"}]}]], "Input",
 CellChangeTimes->{{3.430559481256*^9, 3.430559487722*^9}, 3.430560694248*^9}],

Cell[BoxData[
 TemplateBox[{"2351.6649675873036`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575539816113572*^9, 3.5865812273203106`*^9, 
  3.5869672277235327`*^9, 3.5869692372266436`*^9, 3.587094804996945*^9, 
  3.587094875910452*^9, 3.587098750283477*^9, 3.587098878380845*^9, 
  3.5871003078453603`*^9, 3.5885695646068664`*^9, 3.588579720246964*^9, 
  3.589076630638508*^9}]
}, Open  ]],

Cell["Maximal total range ", "Text",
 CellChangeTimes->{{3.587095515991254*^9, 3.587095526358497*^9}, 
   3.587095589942128*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hmax", "=", 
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], "hc"}], "+", 
   RowBox[{
    FractionBox["2", "3"], "hg"}]}]}]], "Input",
 CellChangeTimes->{{3.430559491553*^9, 3.430559527473*^9}}],

Cell[BoxData[
 TemplateBox[{"5486.996042085001`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755398163265963`*^9, 3.5865812274402866`*^9, 
  3.58696722777954*^9, 3.5869692372766495`*^9, 3.587094805038948*^9, 
  3.587094875958459*^9, 3.587098750338482*^9, 3.587098878432852*^9, 
  3.5871003078833647`*^9, 3.5885695646389093`*^9, 3.588579720271966*^9, 
  3.5890766306665444`*^9}]
}, Open  ]],

Cell["Variation in maximal range", "Text",
 CellChangeTimes->{{3.587100211288777*^9, 3.58710023782096*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]hmax", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "3"}], ")"}], 
   FractionBox["\[CapitalDelta]D", "DD"], "*", "hmax"}]}]], "Input",
 CellChangeTimes->{{3.430558476949*^9, 3.430558494987*^9}, {3.430558590943*^9,
   3.430558591414*^9}, {3.43055930815*^9, 3.430559325796*^9}, {
  3.430559380337*^9, 3.430559382969*^9}, {3.430559538873*^9, 
  3.430559622572*^9}, {3.587098732629359*^9, 3.587098758004403*^9}, {
  3.587098857683364*^9, 3.587098858697485*^9}}],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "132.7184245417762`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5870988786518774`*^9, 3.587100307996381*^9, 
  3.5885695647109194`*^9, 3.5885797203309736`*^9, 3.5890766307334967`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Tides on the Moon ", "Section",
 CellChangeTimes->{{3.430557142957*^9, 3.430557151575*^9}, {
   3.4305600651689997`*^9, 3.430560066352*^9}, {3.4305604260769997`*^9, 
   3.430560428531*^9}, {3.430560462762*^9, 3.430560464113*^9}, {
   3.430560881123*^9, 3.430560886545*^9}, 3.5870949930175047`*^9, {
   3.587095121160882*^9, 3.5870951292218466`*^9}, {3.5870951704377923`*^9, 
   3.587095195176762*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.43055715363*^9, 3.430557156328*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(12\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.575539816744646*^9, 3.586581229310588*^9, 
  3.586967228330606*^9, 3.5869692386838183`*^9, 3.5870948763825083`*^9, 
  3.5870988222961187`*^9, 3.5870988929045877`*^9, 3.5870989264726176`*^9, 
  3.5870992956349115`*^9, 3.587100308260413*^9, 3.5885695650710125`*^9, 
  3.5885797204949946`*^9, 3.5890766311646147`*^9}]
}, Open  ]],

Cell["Parameters", "Text",
 CellChangeTimes->{{3.5871003454688764`*^9, 3.587100346796034*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"G", "=", "GravitationalConstant"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DD", "=", "MoonOrbitRadius"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "center", " ", "distance", " ", "between", " ", "Earth", " ", "and", " ", 
    "Moon"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", "MoonRadius"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"radius", " ", "of", " ", "the", " ", "Moon"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]0", "=", 
   RowBox[{"3.346", " ", 
    RowBox[{"Gram", "/", 
     SuperscriptBox["Centimeter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"average", " ", "density", " ", "of", " ", 
    RowBox[{"Moon", "'"}], "s", " ", "material"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]1", "=", 
   RowBox[{"0.9", "\[Rho]0"}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"average", " ", "density", " ", "of", " ", "the", " ", 
    RowBox[{"Moon", "'"}], "s", " ", "surface", " ", "material", " ", 
    RowBox[{"(", "guessed", ")"}]}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", "MoonMass"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"mass", " ", "of", " ", "the", " ", "Moon"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"m", "=", "EarthMass"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"mass", " ", "of", " ", "the", " ", "Earth"}], " ", "*)"}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.4305571681029997`*^9, 3.43055718919*^9}, {
   3.430557223604*^9, 3.430557343773*^9}, {3.430557390789*^9, 
   3.430557400842*^9}, {3.430557457889*^9, 3.430557470174*^9}, {
   3.4305584975480003`*^9, 3.4305585269700003`*^9}, {3.430558608527*^9, 
   3.430558616646*^9}, {3.430558659685*^9, 3.4305586695220003`*^9}, {
   3.4305593321099997`*^9, 3.4305593404449997`*^9}, {3.430559648876*^9, 
   3.430559710543*^9}, {3.430560073427*^9, 3.430560152791*^9}, {
   3.430560288135*^9, 3.4305603889300003`*^9}, {3.430560480717*^9, 
   3.4305606107720003`*^9}, {3.430560919084*^9, 3.430561031179*^9}, {
   3.430728150259*^9, 3.4307281673459997`*^9}, {3.430728237061*^9, 
   3.430728262091*^9}, {3.430730718789*^9, 3.43073071956*^9}, 
   3.430730751823*^9, {3.4307310235290003`*^9, 3.430731023895*^9}, 
   3.430732931185*^9, {3.587094889244052*^9, 3.587094971272894*^9}, {
   3.58709502457229*^9, 3.587095098433153*^9}, 3.5871001260625496`*^9}],

Cell["Distance variation in an orbit", "Text",
 CellChangeTimes->{{3.5870953168233557`*^9, 3.587095319237646*^9}, {
  3.587100410831721*^9, 3.5871004127009425`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]D", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"405606", "-", "363104"}], ")"}], "Kilometer", " "}]}]], "Input",
 CellChangeTimes->{{3.43072757454*^9, 3.4307275769300003`*^9}, {
   3.5870992656653147`*^9, 3.587099291728441*^9}, {3.587100127422714*^9, 
   3.587100130161044*^9}, 3.5871004500164194`*^9}],

Cell[BoxData[
 TemplateBox[{"42502"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{
  3.575539815664519*^9, 3.586581226782197*^9, 3.5869672270864544`*^9, 
   3.586969236443548*^9, 3.5870948045698924`*^9, 3.5870948755164056`*^9, 
   3.587098749887429*^9, 3.5870988779907994`*^9, {3.5870992565462217`*^9, 
   3.587099295850936*^9}, 3.587100308472437*^9, 3.588569565206978*^9, 
   3.5885797207110677`*^9, 3.589076631578607*^9}]
}, Open  ]],

Cell["Gravitational range", "Text",
 CellChangeTimes->{{3.587095333542364*^9, 3.587095336421707*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hg", "=", 
  RowBox[{
   RowBox[{
    FractionBox["3", "2"], "a", " ", 
    FractionBox["m", "M"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["a", "DD"], ")"}], "3"], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{
        FractionBox["3", "5"], 
        FractionBox["\[Rho]1", "\[Rho]0"]}]}], ")"}], 
     RowBox[{"-", "1"}]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.430558205119*^9, 3.430558212443*^9}, {3.430558328349*^9,
   3.4305583752209997`*^9}, {3.430558462657*^9, 3.430558468552*^9}, {
  3.430559362685*^9, 3.430559369196*^9}, {3.430559463399*^9, 
  3.430559471958*^9}}],

Cell[BoxData[
 TemplateBox[{"42.591824697753914`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575539817278711*^9, 3.5865812298156176`*^9, 
  3.5869672290826936`*^9, 3.586969239255887*^9, 3.587094877015586*^9, 
  3.587098822741171*^9, 3.5870988933406415`*^9, 3.587098926909667*^9, 
  3.5870992961039677`*^9, 3.587100308733469*^9, 3.588569565382973*^9, 
  3.5885797208710656`*^9, 3.5890766317126637`*^9}]
}, Open  ]],

Cell["Centrifugal range", "Text",
 CellChangeTimes->{{3.587095340135154*^9, 3.587095343957611*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hc", "=", 
  RowBox[{
   FractionBox["1", "3"], 
   RowBox[{"(", 
    RowBox[{"1", "+", 
     FractionBox["M", "m"]}], ")"}], "hg"}]}]], "Input",
 CellChangeTimes->{{3.430559481256*^9, 3.430559487722*^9}, {3.430560635594*^9,
   3.430560656432*^9}}],

Cell[BoxData[
 TemplateBox[{"14.371960709502439`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755398175617456`*^9, 3.5865812298886075`*^9, 
  3.5869672292277126`*^9, 3.5869692394969144`*^9, 3.5870948771275983`*^9, 
  3.587098822848184*^9, 3.5870988934466543`*^9, 3.587098927017682*^9, 
  3.5870992962089806`*^9, 3.58710030884348*^9, 3.588569565447009*^9, 
  3.5885797218092113`*^9, 3.5890766317676706`*^9}]
}, Open  ]],

Cell["Maximal total range ", "Text",
 CellChangeTimes->{{3.587095515991254*^9, 3.587095526358497*^9}, 
   3.587095589942128*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hmax", "=", 
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], "hc"}], "+", 
   RowBox[{
    FractionBox["2", "3"], "hg"}]}]}]], "Input",
 CellChangeTimes->{{3.430559491553*^9, 3.430559527473*^9}}],

Cell[BoxData[
 TemplateBox[{"33.18520336833676`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755398176097517`*^9, 3.586581229928633*^9, 
  3.586967229339726*^9, 3.5869692396069307`*^9, 3.587094877178605*^9, 
  3.5870988228921895`*^9, 3.58709889349366*^9, 3.5870989270646887`*^9, 
  3.587099296252985*^9, 3.5871003088834867`*^9, 3.5885695663461323`*^9, 
  3.5885797218612075`*^9, 3.589076631788669*^9}]
}, Open  ]],

Cell["Variation in maximal range", "Text",
 CellChangeTimes->{{3.587100211288777*^9, 3.58710023782096*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]hmax", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "3"}], ")"}], 
   FractionBox["\[CapitalDelta]D", "DD"], "*", "hmax"}]}]], "Input",
 CellChangeTimes->{{3.430558476949*^9, 3.430558494987*^9}, {3.430558590943*^9,
   3.430558591414*^9}, {3.43055930815*^9, 3.430559325796*^9}, {
  3.430559380337*^9, 3.430559382969*^9}, {3.430559538873*^9, 
  3.430559622572*^9}, {3.5017459393357325`*^9, 3.501745943741985*^9}, {
  3.587098909427571*^9, 3.587098913826101*^9}}],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "11.00754821315019`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.587098927261712*^9, 3.587099296447009*^9, 
  3.587100308979496*^9, 3.5885695664221087`*^9, 3.588579721907174*^9, 
  3.589076631840655*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Tides of the Earth in bound rotation (static Earth)\
\>", "Section",
 CellChangeTimes->{{3.430557142957*^9, 3.430557151575*^9}, {
  3.4305600651689997`*^9, 3.430560066352*^9}, {3.4305604260769997`*^9, 
  3.430560428531*^9}, {3.430560462762*^9, 3.430560464113*^9}, {
  3.430560881123*^9, 3.430560886545*^9}, {3.430728109715*^9, 
  3.430728112642*^9}, {3.5870990263516016`*^9, 3.587099029885024*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.43055715363*^9, 3.430557156328*^9}}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(12\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5755398180007973`*^9, 3.5865812302406545`*^9, 
  3.5869672299247956`*^9, 3.5869692403950253`*^9, 3.5870948775636506`*^9, 
  3.587099184373562*^9, 3.587099499529376*^9, 3.5870995719040594`*^9, 
  3.5871003092395277`*^9, 3.588569567125245*^9, 3.5885797220691943`*^9, 
  3.5890766320047007`*^9}]
}, Open  ]],

Cell["Parameters", "Text",
 CellChangeTimes->{{3.587100353708864*^9, 3.587100354940011*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"G", "=", "GravitationalConstant"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"DD", "=", "MoonOrbitRadius"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "center", " ", "distance", " ", "between", " ", "Moon", " ", "and", " ", 
    "Earth"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a", "=", "EarthRadius"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"radius", " ", "of", " ", "Earth"}], " ", "*)"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]0", "=", 
   RowBox[{"5.51", " ", 
    RowBox[{"Gram", "/", 
     SuperscriptBox["Centimeter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"average", " ", "mass", " ", "density", " ", "of", " ", "Earth"}], 
   " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Rho]1", "=", 
   RowBox[{"4.5", " ", 
    RowBox[{"Gram", "/", 
     SuperscriptBox["Centimeter", "3"]}]}]}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "average", " ", "mass", " ", "density", " ", "of", " ", "Earths", " ", 
    "surface", " ", "material"}], " ", "*)"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"M", "=", "EarthMass"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"m", "=", "MoonMass"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4305571681029997`*^9, 3.43055718919*^9}, {
   3.430557223604*^9, 3.430557343773*^9}, {3.430557390789*^9, 
   3.430557400842*^9}, {3.430557457889*^9, 3.430557470174*^9}, {
   3.4305584975480003`*^9, 3.4305585269700003`*^9}, {3.430558608527*^9, 
   3.430558616646*^9}, {3.430558659685*^9, 3.4305586695220003`*^9}, {
   3.4305593321099997`*^9, 3.4305593404449997`*^9}, {3.430559648876*^9, 
   3.430559710543*^9}, {3.430560073427*^9, 3.430560152791*^9}, {
   3.430560288135*^9, 3.4305603889300003`*^9}, {3.430560480717*^9, 
   3.4305606107720003`*^9}, {3.430560919084*^9, 3.430561031179*^9}, {
   3.4307281355810003`*^9, 3.430728143531*^9}, {3.4307281808970003`*^9, 
   3.4307282280290003`*^9}, 3.4307290678380003`*^9, {3.43072920635*^9, 
   3.430729208389*^9}, 3.430729262683*^9, {3.430729646909*^9, 
   3.4307296497799997`*^9}, {3.430729728416*^9, 3.430729762175*^9}, 
   3.430732903475*^9, 3.5017460155910945`*^9, {3.5017460941325865`*^9, 
   3.501746115189791*^9}, {3.587099002457734*^9, 3.5870990061171737`*^9}, {
   3.5870990391701393`*^9, 3.5870990445617857`*^9}, {3.587099079454973*^9, 
   3.5870991777447653`*^9}, 3.587099476407602*^9, 3.5871001505894957`*^9}],

Cell["Distance variation in an orbit", "Text",
 CellChangeTimes->{{3.5870953168233557`*^9, 3.587095319237646*^9}, {
  3.587100410831721*^9, 3.5871004127009425`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]D", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"405606", "-", "363104"}], ")"}], "Kilometer", " "}]}]], "Input",
 CellChangeTimes->{{3.5017461245133247`*^9, 3.501746127722508*^9}, {
   3.587100153422833*^9, 3.587100155190045*^9}, 3.587100457313295*^9}],

Cell[BoxData[
 TemplateBox[{"42502"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"km\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: kilometers"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Kilometers\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755398182818327`*^9, 3.5865812310637784`*^9, 
  3.5869672308519077`*^9, 3.5869692407950726`*^9, 3.5870948779516983`*^9, 
  3.5870991847116013`*^9, 3.587099499821409*^9, 3.5870995721960955`*^9, 
  3.587100310306656*^9, 3.5885695678513317`*^9, 3.588579722178248*^9, 
  3.5890766321166716`*^9}]
}, Open  ]],

Cell["Gravitational range", "Text",
 CellChangeTimes->{{3.587095333542364*^9, 3.587095336421707*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hg", "=", 
  RowBox[{
   RowBox[{
    FractionBox["3", "2"], "a", " ", 
    FractionBox["m", "M"], 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["a", "DD"], ")"}], "3"], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{
        FractionBox["3", "5"], 
        FractionBox["\[Rho]1", "\[Rho]0"]}]}], ")"}], 
     RowBox[{"-", "1"}]]}], "//", "ToSI"}]}]], "Input",
 CellChangeTimes->{{3.430558205119*^9, 3.430558212443*^9}, {3.430558328349*^9,
   3.4305583752209997`*^9}, {3.430558462657*^9, 3.430558468552*^9}, {
  3.430559362685*^9, 3.430559369196*^9}, {3.430559463399*^9, 
  3.430559471958*^9}}],

Cell[BoxData[
 TemplateBox[{"1.0544136576359167`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575539818793894*^9, 3.5865812322729096`*^9, 
  3.586967231294963*^9, 3.58696924120112*^9, 3.5870948783757524`*^9, 
  3.587099185021638*^9, 3.587099500088442*^9, 3.5870995724681273`*^9, 
  3.5871003117068243`*^9, 3.5885695680863485`*^9, 3.588579722313223*^9, 
  3.589076632255719*^9}]
}, Open  ]],

Cell["Centrifugal range", "Text",
 CellChangeTimes->{{3.587095340135154*^9, 3.587095343957611*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hc", "=", 
  RowBox[{
   FractionBox[
    RowBox[{"M", "+", "m"}], 
    RowBox[{"3", "m"}]], "hg"}]}]], "Input",
 CellChangeTimes->{{3.430559481256*^9, 3.430559487722*^9}, {3.430560635594*^9,
   3.430560656432*^9}, {3.430728334856*^9, 3.430728346727*^9}}],

Cell[BoxData[
 TemplateBox[{"28.916662124836858`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.575539818970913*^9, 3.5865812323759427`*^9, 
  3.5869672315159864`*^9, 3.586969241409144*^9, 3.587094878592774*^9, 
  3.5870991851886578`*^9, 3.5870995002514615`*^9, 3.587099572637146*^9, 
  3.5871003121428795`*^9, 3.5885695719428377`*^9, 3.5885797223962345`*^9, 
  3.5890766323747487`*^9}]
}, Open  ]],

Cell["Maximal total range ", "Text",
 CellChangeTimes->{{3.587095515991254*^9, 3.587095526358497*^9}, 
   3.587095589942128*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hmax", "=", 
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], "hc"}], "+", 
   RowBox[{
    FractionBox["2", "3"], "hg"}]}]}]], "Input",
 CellChangeTimes->{{3.430559491553*^9, 3.430559527473*^9}}],

Cell[BoxData[
 TemplateBox[{"10.341829813369563`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{3.5755398190129194`*^9, 3.586581232408944*^9, 
  3.5869672317830205`*^9, 3.5869692416091685`*^9, 3.5870948790908346`*^9, 
  3.587099185225662*^9, 3.5870995002974653`*^9, 3.587099572680153*^9, 
  3.587100313021983*^9, 3.5885695721218753`*^9, 3.5885797224162793`*^9, 
  3.5890766323967485`*^9}]
}, Open  ]],

Cell["Variation in maximal range", "Text",
 CellChangeTimes->{{3.587100211288777*^9, 3.58710023782096*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[CapitalDelta]hmax", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "3"}], ")"}], 
   FractionBox["\[CapitalDelta]D", "DD"], "*", "hmax"}]}]], "Input",
 CellChangeTimes->{{3.430558476949*^9, 3.430558494987*^9}, {3.430558590943*^9,
   3.430558591414*^9}, {3.43055930815*^9, 3.430559325796*^9}, {
  3.430559380337*^9, 3.430559382969*^9}, {3.430559538873*^9, 
  3.430559622572*^9}, {3.5870995678285704`*^9, 3.5870995939557056`*^9}}],

Cell[BoxData[
 TemplateBox[{RowBox[{"-", "3.430390015071499`"}]},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"m\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Meters\""}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.5870995728781767`*^9, 3.5870995943237476`*^9}, 
   3.5871003132770123`*^9, 3.5885695728149505`*^9, 3.588579722472245*^9, 
   3.5890766324447565`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{928, 903},
WindowMargins->{{256, Automatic}, {Automatic, 32}},
WindowFrame->"Normal",
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 106, 2, 65, "Title",
 InitializationCell->True],
Cell[676, 26, 290, 8, 58, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1003, 39, 144, 2, 65, "Title"],
Cell[CellGroupData[{
Cell[1172, 45, 136, 2, 75, "Section"],
Cell[CellGroupData[{
Cell[1333, 51, 101, 2, 41, "Input"],
Cell[1437, 55, 385, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1859, 69, 74, 1, 36, "Subsection"],
Cell[1936, 72, 734, 20, 82, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2707, 97, 79, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[2811, 102, 416, 11, 61, "Input"],
Cell[3230, 115, 603, 13, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[3894, 135, 142, 2, 75, "Section"],
Cell[CellGroupData[{
Cell[4061, 141, 103, 2, 41, "Input"],
Cell[4167, 145, 411, 9, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4615, 159, 102, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[4742, 164, 1281, 32, 105, "Input"],
Cell[6026, 198, 647, 15, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6722, 219, 97, 1, 36, "Subsection"],
Cell[6822, 222, 107, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[6954, 227, 182, 5, 41, "Input"],
Cell[7139, 234, 827, 18, 41, "Output"]
}, Open  ]],
Cell[7981, 255, 90, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[8096, 260, 113, 3, 56, "Input"],
Cell[8212, 265, 261, 3, 41, "Output"]
}, Open  ]],
Cell[8488, 271, 103, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[8616, 276, 307, 9, 56, "Input"],
Cell[8926, 287, 633, 13, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[9632, 308, 116, 1, 65, "Title"],
Cell[CellGroupData[{
Cell[9773, 313, 77, 2, 41, "Input"],
Cell[9853, 317, 508, 11, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10398, 333, 32, 0, 36, "Subsection"],
Cell[CellGroupData[{
Cell[10455, 337, 743, 19, 99, "Input"],
Cell[11201, 358, 827, 18, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[12077, 382, 97, 1, 36, "Subsection"],
Cell[12177, 385, 69, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[12271, 389, 254, 7, 61, "Input"],
Cell[12528, 398, 724, 16, 41, "Output"]
}, Open  ]],
Cell[13267, 417, 298, 8, 30, "Text"],
Cell[CellGroupData[{
Cell[13590, 429, 239, 7, 66, "Input"],
Cell[13832, 438, 829, 18, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14698, 461, 237, 6, 58, "Input"],
Cell[14938, 469, 718, 15, 41, "Output"]
}, Open  ]],
Cell[15671, 487, 51, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[15747, 491, 114, 3, 41, "Input"],
Cell[15864, 496, 737, 15, 41, "Output"]
}, Open  ]],
Cell[16616, 514, 217, 3, 30, "Text"],
Cell[16836, 519, 511, 12, 62, "Input"],
Cell[CellGroupData[{
Cell[17372, 535, 265, 6, 41, "Input"],
Cell[17640, 543, 730, 15, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[18419, 564, 230, 3, 75, "Section"],
Cell[CellGroupData[{
Cell[18674, 571, 126, 2, 41, "Input"],
Cell[18803, 575, 563, 12, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19403, 592, 99, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[19527, 597, 1134, 24, 119, "Input"],
Cell[20664, 623, 648, 14, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[21349, 642, 137, 2, 41, "Input"],
Cell[21489, 646, 886, 19, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[22424, 671, 97, 1, 36, "Subsection"],
Cell[22524, 674, 177, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[22726, 680, 238, 7, 57, "Input"],
Cell[22967, 689, 1007, 21, 41, "Output"]
}, Open  ]],
Cell[23989, 713, 108, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[24122, 718, 224, 6, 41, "Input"],
Cell[24349, 726, 1010, 21, 41, "Output"]
}, Open  ]],
Cell[25374, 750, 144, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[25543, 757, 265, 7, 56, "Input"],
Cell[25811, 766, 824, 17, 41, "Output"]
}, Open  ]],
Cell[26650, 786, 289, 5, 30, "Text"],
Cell[CellGroupData[{
Cell[26964, 795, 621, 15, 41, "Input"],
Cell[27588, 812, 469, 7, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[28118, 826, 236, 3, 75, "Section"],
Cell[CellGroupData[{
Cell[28379, 833, 103, 2, 41, "Input"],
Cell[28485, 837, 538, 12, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[29060, 854, 99, 1, 36, "Subsection"],
Cell[29162, 857, 1205, 30, 122, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[30404, 892, 97, 1, 36, "Subsection"],
Cell[30504, 895, 113, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[30642, 900, 276, 8, 57, "Input"],
Cell[30921, 910, 758, 16, 41, "Output"]
}, Open  ]],
Cell[31694, 929, 99, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[31818, 934, 342, 11, 59, "Input"],
Cell[32163, 947, 417, 6, 41, "Output"]
}, Open  ]],
Cell[32595, 956, 131, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[32751, 963, 539, 16, 78, "Input"],
Cell[33293, 981, 750, 16, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[34104, 1004, 208, 3, 75, "Section"],
Cell[CellGroupData[{
Cell[34337, 1011, 125, 2, 41, "Input"],
Cell[34465, 1015, 540, 12, 21, "Message"]
}, Open  ]],
Cell[35020, 1030, 345, 10, 30, "Text"],
Cell[CellGroupData[{
Cell[35390, 1044, 746, 25, 70, "Input"],
Cell[36139, 1071, 532, 13, 56, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[36732, 1091, 182, 2, 65, "Title"],
Cell[CellGroupData[{
Cell[36939, 1097, 104, 1, 75, "Section"],
Cell[CellGroupData[{
Cell[37068, 1102, 127, 2, 41, "Input"],
Cell[37198, 1106, 454, 10, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37689, 1121, 100, 1, 36, "Subsection"],
Cell[37792, 1124, 726, 19, 122, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[38555, 1148, 96, 1, 36, "Subsection"],
Cell[38654, 1151, 51, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[38730, 1155, 257, 7, 54, "Input"],
Cell[38990, 1164, 696, 14, 41, "Output"]
}, Open  ]],
Cell[39701, 1181, 184, 4, 30, "Text"],
Cell[CellGroupData[{
Cell[39910, 1189, 181, 4, 41, "Input"],
Cell[40094, 1195, 701, 14, 41, "Output"]
}, Open  ]],
Cell[40810, 1212, 302, 5, 30, "Text"],
Cell[CellGroupData[{
Cell[41137, 1221, 233, 6, 57, "Input"],
Cell[41373, 1229, 919, 21, 41, "Output"]
}, Open  ]],
Cell[42307, 1253, 162, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[42494, 1259, 198, 6, 58, "Input"],
Cell[42695, 1267, 922, 21, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[43678, 1295, 30, 0, 75, "Section"],
Cell[43711, 1297, 30, 0, 30, "Text"],
Cell[CellGroupData[{
Cell[43766, 1301, 361, 11, 57, "Input"],
Cell[44130, 1314, 656, 14, 41, "Output"]
}, Open  ]],
Cell[44801, 1331, 332, 6, 30, "Text"],
Cell[45136, 1339, 781, 21, 65, "Input"],
Cell[CellGroupData[{
Cell[45942, 1364, 314, 9, 68, "Input"],
Cell[46259, 1375, 283, 4, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[46591, 1385, 99, 1, 75, "Section"],
Cell[CellGroupData[{
Cell[46715, 1390, 129, 2, 41, "Input"],
Cell[46847, 1394, 436, 10, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[47320, 1409, 104, 1, 36, "Subsection"],
Cell[47427, 1412, 133, 1, 30, "Text"],
Cell[47563, 1415, 414, 11, 41, "Input"],
Cell[47980, 1428, 275, 9, 30, "Text"],
Cell[CellGroupData[{
Cell[48280, 1441, 329, 8, 56, "Input"],
Cell[48612, 1451, 973, 25, 56, "Output"]
}, Open  ]],
Cell[49600, 1479, 109, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[49734, 1484, 452, 13, 56, "Input"],
Cell[50189, 1499, 1194, 32, 78, "Output"]
}, Open  ]],
Cell[51398, 1534, 232, 7, 30, "Text"],
Cell[CellGroupData[{
Cell[51655, 1545, 980, 30, 56, "Input"],
Cell[52638, 1577, 264, 4, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[52963, 1588, 239, 3, 75, "Section"],
Cell[CellGroupData[{
Cell[53227, 1595, 129, 2, 41, "Input"],
Cell[53359, 1599, 552, 11, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[53948, 1615, 152, 2, 36, "Subsection"],
Cell[54103, 1619, 3524, 66, 142, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[57664, 1690, 97, 1, 36, "Subsection"],
Cell[57764, 1693, 493, 13, 30, "Text"],
Cell[CellGroupData[{
Cell[58282, 1710, 411, 9, 57, "Input"],
Cell[58696, 1721, 423, 5, 41, "Output"]
}, Open  ]],
Cell[59134, 1729, 107, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[59266, 1734, 838, 15, 58, "Input"],
Cell[60107, 1751, 398, 5, 41, "Output"]
}, Open  ]],
Cell[60520, 1759, 184, 4, 30, "Text"],
Cell[CellGroupData[{
Cell[60729, 1767, 446, 14, 56, "Input"],
Cell[61178, 1783, 420, 6, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[61635, 1794, 280, 8, 56, "Input"],
Cell[61918, 1804, 405, 5, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[62360, 1814, 304, 8, 56, "Input"],
Cell[62667, 1824, 400, 5, 41, "Output"]
}, Open  ]],
Cell[63082, 1832, 465, 11, 30, "Text"],
Cell[63550, 1845, 1043, 24, 41, "Input"],
Cell[64596, 1871, 105, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[64726, 1876, 2260, 42, 65, "Input"],
Cell[66989, 1920, 22941, 381, 247, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[89979, 2307, 155, 2, 36, "Subsection"],
Cell[90137, 2311, 107, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[90269, 2316, 675, 11, 41, "Input"],
Cell[90947, 2329, 86625, 1424, 344, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[177621, 3759, 106, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[177752, 3764, 266, 6, 41, "Input"],
Cell[178021, 3772, 178, 4, 41, "Output"]
}, Open  ]],
Cell[178214, 3779, 330, 6, 30, "Text"],
Cell[178547, 3787, 1179, 27, 102, "Input"],
Cell[179729, 3816, 222, 3, 30, "Text"],
Cell[179954, 3821, 906, 24, 102, "Input"],
Cell[180863, 3847, 121, 3, 30, "Text"],
Cell[180987, 3852, 320, 7, 41, "Input"],
Cell[181310, 3861, 1530, 35, 96, "Input"],
Cell[CellGroupData[{
Cell[182865, 3900, 599, 11, 41, "Input"],
Cell[183467, 3913, 135628, 2224, 344, "Output"]
}, Open  ]],
Cell[319110, 6140, 104, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[319239, 6145, 778, 13, 62, "Input"],
Cell[320020, 6160, 674, 12, 21, "Message"],
Cell[320697, 6174, 180302, 2963, 279, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[501072, 9145, 223, 3, 65, "Title"],
Cell[501298, 9150, 289, 5, 30, "Text"],
Cell[CellGroupData[{
Cell[501612, 9159, 149, 2, 75, "Section"],
Cell[CellGroupData[{
Cell[501786, 9165, 118, 2, 41, "Input"],
Cell[501907, 9169, 502, 11, 21, "Message"]
}, Open  ]],
Cell[502424, 9183, 165, 2, 30, "Text"],
Cell[502592, 9187, 2651, 60, 167, "Input"],
Cell[505246, 9249, 165, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[505436, 9255, 248, 6, 41, "Input"],
Cell[505687, 9263, 542, 12, 41, "Output"]
}, Open  ]],
Cell[506244, 9278, 101, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[506370, 9283, 663, 19, 59, "Input"],
Cell[507036, 9304, 717, 15, 41, "Output"]
}, Open  ]],
Cell[507768, 9322, 99, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[507892, 9327, 248, 7, 56, "Input"],
Cell[508143, 9336, 713, 15, 41, "Output"]
}, Open  ]],
Cell[508871, 9354, 128, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[509024, 9360, 221, 7, 56, "Input"],
Cell[509248, 9369, 713, 15, 41, "Output"]
}, Open  ]],
Cell[509976, 9387, 107, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[510108, 9392, 503, 10, 56, "Input"],
Cell[510614, 9404, 562, 12, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[511225, 9422, 407, 6, 75, "Section"],
Cell[CellGroupData[{
Cell[511657, 9432, 118, 2, 41, "Input"],
Cell[511778, 9436, 532, 11, 21, "Message"]
}, Open  ]],
Cell[512325, 9450, 94, 1, 30, "Text"],
Cell[512422, 9453, 2440, 54, 165, "Input"],
Cell[514865, 9509, 165, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[515055, 9515, 344, 7, 41, "Input"],
Cell[515402, 9524, 763, 16, 41, "Output"]
}, Open  ]],
Cell[516180, 9543, 101, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[516306, 9548, 663, 19, 59, "Input"],
Cell[516972, 9569, 737, 15, 41, "Output"]
}, Open  ]],
Cell[517724, 9587, 99, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[517848, 9592, 272, 8, 56, "Input"],
Cell[518123, 9602, 742, 15, 41, "Output"]
}, Open  ]],
Cell[518880, 9620, 128, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[519033, 9626, 221, 7, 56, "Input"],
Cell[519257, 9635, 737, 15, 41, "Output"]
}, Open  ]],
Cell[520009, 9653, 107, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[520141, 9658, 505, 10, 56, "Input"],
Cell[520649, 9670, 581, 13, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[521279, 9689, 408, 7, 75, "Section"],
Cell[CellGroupData[{
Cell[521712, 9700, 118, 2, 41, "Input"],
Cell[521833, 9704, 510, 11, 21, "Message"]
}, Open  ]],
Cell[522358, 9718, 92, 1, 30, "Text"],
Cell[522453, 9721, 2440, 51, 167, "Input"],
Cell[524896, 9774, 165, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[525086, 9780, 294, 6, 41, "Input"],
Cell[525383, 9788, 716, 15, 41, "Output"]
}, Open  ]],
Cell[526114, 9806, 101, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[526240, 9811, 663, 19, 59, "Input"],
Cell[526906, 9832, 712, 15, 41, "Output"]
}, Open  ]],
Cell[527633, 9850, 99, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[527757, 9855, 279, 7, 57, "Input"],
Cell[528039, 9864, 719, 15, 41, "Output"]
}, Open  ]],
Cell[528773, 9882, 128, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[528926, 9888, 221, 7, 56, "Input"],
Cell[529150, 9897, 719, 15, 41, "Output"]
}, Open  ]],
Cell[529884, 9915, 107, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[530016, 9920, 458, 9, 56, "Input"],
Cell[530477, 9931, 593, 13, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
