(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    797085,      14136]
NotebookOptionsPosition[    789482,      13875]
NotebookOutlinePosition[    789933,      13892]
CellTagsIndexPosition[    789890,      13889]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 11 Computational elastostatics(p. 177)\
\>", "Title",
 CellChangeTimes->{
  3.4199117085635786`*^9, 3.5054112034859476`*^9, {3.5311196372977996`*^9, 
   3.5311196398729467`*^9}, {3.5873809022632866`*^9, 3.587380905046653*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], "\n", 
 RowBox[{"<<", "Lautrup`"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.4143073450928373`*^9, 3.414307349419837*^9}, {
   3.467103613614896*^9, 3.4671036161540413`*^9}, {3.4792048675030003`*^9, 
   3.479204871837*^9}, 3.5054112034859476`*^9, 3.5890386686060276`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"InterpolatingFunction", "::", "dmval"}], "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{3.5054112034869475`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Plot3D", ",", 
    RowBox[{"PlotRange", "\[Rule]", "All"}]}], "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.414737806818797*^9, 3.4147378101887975`*^9}, 
   3.5053675081702843`*^9, 3.5054112034869475`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Section 11.3 Application: Gravitational settling in two dimensions (p. 182)\
\>", "Title",
 CellChangeTimes->{{3.5019976051019335`*^9, 3.5019976185967054`*^9}, 
   3.5054112034879475`*^9, {3.587380910311288*^9, 3.587380922853896*^9}}],

Cell[CellGroupData[{

Cell["Shear-free solution (p. 183)", "Section",
 CellChangeTimes->{{3.501997649581478*^9, 3.501997652836664*^9}, 
   3.5054112034879475`*^9, {3.5873809338712826`*^9, 3.5873809379817877`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{3.5054112034879475`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(14\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5755448491837215`*^9, 3.586581475168389*^9, 
  3.587380840415523*^9, 3.5890784987894344`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Verify the theory", "Subsection",
 CellChangeTimes->{{3.587381020023099*^9, 3.58738102381361*^9}}],

Cell["Displacement field", "Text",
 CellChangeTimes->{3.5054112034879475`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["u", "x"], "=", 
   RowBox[{
    FractionBox["\[Nu]", 
     RowBox[{"1", "-", "\[Nu]"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"b", "-", "y"}], ")"}], "x"}], "DD"]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["u", "y"], "=", 
   RowBox[{
    RowBox[{"-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["b", "2"], "-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"b", "-", "y"}], ")"}], "2"]}], 
      RowBox[{"2", " ", "DD"}]]}], "+", 
    RowBox[{
     FractionBox["\[Nu]", 
      RowBox[{"1", "-", "\[Nu]"}]], 
     FractionBox[
      RowBox[{
       RowBox[{"3", 
        SuperscriptBox["x", "2"]}], "-", 
       SuperscriptBox["a", "2"]}], 
      RowBox[{"6", "DD"}]]}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.4144124909654245`*^9, 3.4144125183284245`*^9}, 
   3.4144125818144245`*^9, {3.4147322569427977`*^9, 3.414732313245797*^9}, 
   3.5054112034899483`*^9}],

Cell["Parameters", "Text",
 CellChangeTimes->{3.5054112034899483`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"DD", "=", 
   FractionBox["EE", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", 
       SuperscriptBox["\[Nu]", "2"]}], ")"}], 
     SubscriptBox["\[Rho]", "0"], 
     SubscriptBox["g", "0"]}]]}], ";"}]], "Input",
 CellChangeTimes->{3.5054112034909477`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"EE", "=", 
   FractionBox[
    RowBox[{"\[Mu]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", "\[Lambda]"}], "+", 
       RowBox[{"2", "\[Mu]"}]}], ")"}]}], 
    RowBox[{"\[Lambda]", "+", "\[Mu]"}]]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\[Nu]", "=", 
   FractionBox["\[Lambda]", 
    RowBox[{"2", 
     RowBox[{"(", 
      RowBox[{"\[Lambda]", "+", "\[Mu]"}], ")"}]}]]}], ";"}]}], "Input",
 CellChangeTimes->{{3.4144129900544243`*^9, 3.414413018935425*^9}, 
   3.4147322938397975`*^9, 3.505411203491948*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DD", "//", "Simplify"}]], "Input",
 CellChangeTimes->{{3.414732483995797*^9, 3.4147324920937977`*^9}, 
   3.505411203491948*^9}],

Cell[BoxData[
 FractionBox[
  RowBox[{"4", " ", "\[Mu]", " ", 
   RowBox[{"(", 
    RowBox[{"\[Lambda]", "+", "\[Mu]"}], ")"}]}], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Lambda]", "+", 
     RowBox[{"2", " ", "\[Mu]"}]}], ")"}], " ", 
   SubscriptBox["g", "0"], " ", 
   SubscriptBox["\[Rho]", "0"]}]]], "Output",
 CellChangeTimes->{3.575544849272729*^9, 3.5865814752093945`*^9, 
  3.5873808404575405`*^9}]
}, Open  ]],

Cell["Effective force", "Text",
 CellChangeTimes->{3.505411203491948*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   SubscriptBox["f", "x"], "*"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Lambda]", "+", 
       RowBox[{"2", "\[Mu]"}]}], ")"}], 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       RowBox[{"x", ",", "x"}]], 
      SubscriptBox["u", "x"]}]}], "+", 
    RowBox[{"\[Mu]", " ", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       RowBox[{"y", ",", "y"}]], 
      SubscriptBox["u", "x"]}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Lambda]", "+", "\[Mu]"}], ")"}], 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       RowBox[{"x", ",", "y"}]], 
      SubscriptBox["u", "y"]}]}]}], "//", "Simplify"}]}]], "Input",
 CellChangeTimes->{3.5019977564685917`*^9, 3.505411203492948*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.575544849310734*^9, 3.586581475229437*^9, 
  3.5873808404775724`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   SubscriptBox["f", "y"], "*"], "=", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Lambda]", "+", 
       RowBox[{"2", "\[Mu]"}]}], ")"}], 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       RowBox[{"y", ",", "y"}]], 
      SubscriptBox["u", "y"]}]}], "+", 
    RowBox[{"\[Mu]", " ", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       RowBox[{"x", ",", "x"}]], 
      SubscriptBox["u", "y"]}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Lambda]", "+", "\[Mu]"}], ")"}], 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       RowBox[{"x", ",", "y"}]], 
      SubscriptBox["u", "x"]}]}], " ", "-", 
    RowBox[{
     SubscriptBox["\[Rho]", "0"], 
     SubscriptBox["g", "0"]}]}], "//", "Simplify"}]}]], "Input",
 CellChangeTimes->{3.5019977564685917`*^9, 3.5054112034949484`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.57554484935074*^9, 3.5865814752494397`*^9, 
  3.5873808404975595`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Stresses and strains", "Subsection",
 CellChangeTimes->{{3.5873810329257545`*^9, 3.5873810436300497`*^9}, {
  3.5873811417583833`*^9, 3.5873811461659555`*^9}}],

Cell["Strains", "Text",
 CellChangeTimes->{3.5054112034949484`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["u", "xx"], "=", 
   RowBox[{
    SubscriptBox["\[PartialD]", "x"], 
    SubscriptBox["u", "x"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["u", "yy"], "=", 
   RowBox[{
    SubscriptBox["\[PartialD]", "y"], 
    SubscriptBox["u", "y"]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["u", "xy"], "=", 
   RowBox[{
    FractionBox["1", "2"], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", "x"], 
       SubscriptBox["u", "y"]}], "+", 
      RowBox[{
       SubscriptBox["\[PartialD]", "y"], 
       SubscriptBox["u", "x"]}]}], ")"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.4147324128937974`*^9, 3.4147324160777974`*^9}, 
   3.5054112034969482`*^9}],

Cell["Stresses", "Text",
 CellChangeTimes->{3.5054112034969482`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   SubscriptBox["\[Sigma]", "xx"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", "\[Mu]"}], "+", "\[Lambda]"}], ")"}], 
     SubscriptBox["u", "xx"]}], "+", 
    RowBox[{"\[Lambda]", " ", 
     SubscriptBox["u", "yy"]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Sigma]", "yy"], "=", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", "\[Mu]"}], "+", "\[Lambda]"}], ")"}], 
     SubscriptBox["u", "yy"]}], "+", 
    RowBox[{"\[Lambda]", " ", 
     SubscriptBox["u", "xx"]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   SubscriptBox["\[Sigma]", "xy"], "=", 
   RowBox[{"2", "\[Mu]", " ", 
    SubscriptBox["u", "xy"]}]}], ";"}]}], "Input",
 CellChangeTimes->{3.505411203498948*^9}],

Cell["Boundary conditions", "Text",
 CellChangeTimes->{3.505411203498948*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["\[Sigma]", "xx"], ",", 
     SubscriptBox["\[Sigma]", "xy"]}], "}"}], "/.", 
   RowBox[{"x", "\[Rule]", "a"}]}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{3.5054112034999485`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.5755448494087462`*^9, 3.586581475280405*^9, 
  3.587380840524581*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["\[Sigma]", "yy"], ",", 
     SubscriptBox["\[Sigma]", "xy"]}], "}"}], "/.", 
   RowBox[{"y", "\[Rule]", "b"}]}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{3.5054112035009484`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.5755448494447503`*^9, 3.586581475298405*^9, 
  3.587380840542562*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["u", "x"], ",", 
     SubscriptBox["\[Sigma]", "xy"]}], "}"}], "/.", 
   RowBox[{"x", "\[Rule]", "0"}]}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{3.5054112035009484`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "0"}], "}"}]], "Output",
 CellChangeTimes->{3.5755448494777546`*^9, 3.58658147531643*^9, 
  3.587380840562565*^9}]
}, Open  ]],

Cell[TextData[{
 "Fails to meet the boundary condition at ",
 Cell[BoxData[
  FormBox[
   RowBox[{"y", " ", "=", " ", "0"}], TraditionalForm]],
  FormatType->"TraditionalForm"]
}], "Text",
 CellChangeTimes->{{3.5873810862864227`*^9, 3.5873811007302055`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["u", "y"], ",", 
     SubscriptBox["\[Sigma]", "xy"]}], "}"}], "/.", 
   RowBox[{"y", "\[Rule]", "0"}]}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{3.5054112035009484`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["a", "2"], "-", 
        RowBox[{"3", " ", 
         SuperscriptBox["x", "2"]}]}], ")"}], " ", "\[Lambda]", " ", 
      SubscriptBox["g", "0"], " ", 
      SubscriptBox["\[Rho]", "0"]}], 
     RowBox[{"24", " ", "\[Mu]", " ", 
      RowBox[{"(", 
       RowBox[{"\[Lambda]", "+", "\[Mu]"}], ")"}]}]]}], ",", "0"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.5755448495137587`*^9, 3.586581475335411*^9, 
  3.587380840582575*^9}]
}, Open  ]],

Cell["Average vanishes", "Text",
 CellChangeTimes->{{3.587381129110796*^9, 3.587381132318206*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Integral]", "0", "a"], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["u", "y"], "/.", 
     RowBox[{"y", "\[Rule]", "0"}]}], ")"}], 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input",
 CellChangeTimes->{{3.4144130349014244`*^9, 3.4144130754114246`*^9}, {
   3.4144131117534246`*^9, 3.414413149539425*^9}, 3.5054112035019484`*^9}],

Cell[BoxData["0"], "Output",
 CellChangeTimes->{3.575544849853799*^9, 3.58658147556948*^9, 
  3.5873808481325274`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Results", "Section",
 CellChangeTimes->{{3.4144134983194246`*^9, 3.4144134998984246`*^9}, {
   3.501997911477457*^9, 3.5019979251332383`*^9}, {3.5019980764458933`*^9, 
   3.5019980773959475`*^9}, 3.5054112035019484`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CleanGlobal", "[", "]"}], ";"}]], "Input",
 CellChangeTimes->{3.5054112035029488`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(17\\)\[NoBreak] symbols\"\>"}]], "Message",\
 "MSG",
 CellChangeTimes->{3.5755448501258307`*^9, 3.586581475736497*^9, 
  3.5873808495617104`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Get simulation data", "Subsection",
 CellChangeTimes->{{3.4147363405707974`*^9, 3.4147363410967975`*^9}, 
   3.5054112035029488`*^9, {3.5873811545669537`*^9, 3.5873811567262945`*^9}}],

Cell[BoxData[
 RowBox[{"<<", "\"\<data/ces.settling.m\>\""}]], "Input",
 CellChangeTimes->{{3.5053672942250476`*^9, 3.505367336497465*^9}, 
   3.5054112035029488`*^9}],

Cell[CellGroupData[{

Cell[BoxData["a"], "Input",
 CellChangeTimes->{3.5054112035039487`*^9}],

Cell[BoxData["1.`"], "Output",
 CellChangeTimes->{3.575544850360859*^9, 3.5865814758165216`*^9, 
  3.587380851604917*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["b"], "Input",
 CellChangeTimes->{3.5054112035039487`*^9}],

Cell[BoxData["1.`"], "Output",
 CellChangeTimes->{3.575544850403864*^9, 3.586581475842495*^9, 
  3.587380851635915*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["nx"], "Input",
 CellChangeTimes->{3.5054112035039487`*^9}],

Cell[BoxData["20"], "Output",
 CellChangeTimes->{3.57554485043787*^9, 3.586581475860512*^9, 
  3.5873808516599617`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["ny"], "Input",
 CellChangeTimes->{3.5054112035039487`*^9}],

Cell[BoxData["20"], "Output",
 CellChangeTimes->{3.575544850479875*^9, 3.586581475880476*^9, 
  3.587380851707966*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Y"], "Input",
 CellChangeTimes->{3.5054112035049486`*^9}],

Cell[BoxData["2.`"], "Output",
 CellChangeTimes->{3.5755448505228806`*^9, 3.586581475898497*^9, 
  3.587380851877984*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Nu]", "=", "nu"}]], "Input",
 CellChangeTimes->{{3.4144153528564243`*^9, 3.4144153542504244`*^9}, 
   3.5054112035049486`*^9}],

Cell[BoxData["0.333333`"], "Output",
 CellChangeTimes->{3.575544850556884*^9, 3.586581475916482*^9, 
  3.5873808519169817`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["lambda"], "Input",
 CellChangeTimes->{3.5054112035049486`*^9}],

Cell[BoxData["1.5`"], "Output",
 CellChangeTimes->{3.575544850597889*^9, 3.5865814759334955`*^9, 
  3.5873808519569798`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["mu"], "Input",
 CellChangeTimes->{3.5054112035049486`*^9}],

Cell[BoxData["0.75`"], "Output",
 CellChangeTimes->{3.575544850699901*^9, 3.5865814759555264`*^9, 
  3.5873808519819584`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["rho0g0"], "Input",
 CellChangeTimes->{3.5054112035049486`*^9}],

Cell[BoxData["1.`"], "Output",
 CellChangeTimes->{3.575544850747907*^9, 3.5865814759724874`*^9, 
  3.5873808519989824`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DD", "=", "scale"}]], "Input",
 CellChangeTimes->{{3.4144152459124246`*^9, 3.4144152464164248`*^9}, 
   3.505411203505949*^9}],

Cell[BoxData["2.25`"], "Output",
 CellChangeTimes->{3.575544850803912*^9, 3.5865814759895306`*^9, 
  3.587380852020963*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["omega"], "Input",
 CellChangeTimes->{3.505411203505949*^9}],

Cell[BoxData["1.`"], "Output",
 CellChangeTimes->{3.5755448508379183`*^9, 3.5865814760545197`*^9, 
  3.587380852037966*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["accuracy"], "Input",
 CellChangeTimes->{3.505411203505949*^9}],

Cell[BoxData["0.`"], "Output",
 CellChangeTimes->{3.575544850884923*^9, 3.586581476075515*^9, 
  3.5873808520780067`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualize ", "Subsection",
 CellChangeTimes->{3.4147363306177974`*^9, 3.505411203505949*^9}],

Cell[CellGroupData[{

Cell[BoxData["iter"], "Input",
 CellChangeTimes->{3.5054112035069485`*^9}],

Cell[BoxData["4000"], "Output",
 CellChangeTimes->{3.57554485093893*^9, 3.5865814760925045`*^9, 
  3.58738085230203*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["precision"], "Input",
 CellChangeTimes->{3.5054112035069485`*^9}],

Cell[BoxData["0.009857`"], "Output",
 CellChangeTimes->{3.575544850987935*^9, 3.5865814761085453`*^9, 
  3.5873808523410034`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["energy"], "Input",
 CellChangeTimes->{3.5054112035069485`*^9}],

Cell[BoxData[
 RowBox[{"-", "0.072641`"}]], "Output",
 CellChangeTimes->{3.575544851027939*^9, 3.5865814761275473`*^9, 
  3.5873808523580313`*^9}]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"uxi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"ux", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"uyi", " ", "=", " ", 
    RowBox[{"ListInterpolation", "[", 
     RowBox[{"uy", ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"uxxi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"uxx", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"uyyi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"uyy", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"uxyi", " ", "=", " ", 
    RowBox[{"ListInterpolation", "[", 
     RowBox[{"uxy", ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"sxxi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"sxx", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"syyi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"syy", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"sxyi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"sxy", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{3.5054112035089493`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"uxi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.5054112035089493`*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nHk4VW/UsI+p0CSppJKKSCoJRbFJSEVppEEkzSIkJVLJnOGY53mep2Oe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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJw1mXngTVX3xu89e59QCiHzTIY0GKIUMqUyhaIXjYQyJ2k0ZMicprdoLg2S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         "]], Polygon3DBox[CompressedData["
1:eJw1mnfgz1UfxT/33o+RJMloyMompfGkTVMKpaVCGS0aGkakjJKUStEwCiEq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         "]], Polygon3DBox[CompressedData["
1:eJwt13ncjWUaAODvO+dIliRLRpFBGEKTtawjW9nK3tgauylLZclkKWPf96US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         "]]}]}, {}, {}, {}, {}}, {
     {GrayLevel[0], Line3DBox[CompressedData["
1:eJwt0rsvpFEYwOGPcRsMK6Jd0RAFpSgoN4pViSjoiGK3WIlsQakQOh1/AS0R
+k3o3MfMuM2ystkIu2Tjfg3PSRS/eU7mO5nznsxX0zvQ8S0niqJR3SqsL/Vb
87lR9IE/WM0NNvKQrTxnO5/Zw9JYFCX01bqew2zmONs4xS7OsJ+L/M5l/ucJ
77Wta/3TioFSXGWaa8xwnTthJu5yk3vcYlL74Xd4wBSzTPMnMzzkDo+4y1+c
dY9TPuhYN+GOmjBbkiNc4iAX2MdpdnKSnzjGJg6xjrX6Yh1nNx+d8Zl/2cIs
G8K9+JEX9v3hncp1FfaqyoxlrGSCFSwNe1jCBItZwng4j0UsYL4KrWMsCP9x
+I6vytNLeKYnGSOa83H2/j68Acy/Svc=
       "]]}, {
      Line3DBox[{690, 1004, 473, 689, 1113, 912, 691, 1114, 913, 692, 1115, 
       914, 693, 1116, 915, 694, 1117, 916, 695, 1118, 1010, 1217, 696, 1119, 
       917, 697, 1120, 918, 698, 1121, 919, 699, 1122, 920, 700, 1123, 921, 
       701, 1124, 922, 702, 1322, 1005, 923, 1006}], 
      Line3DBox[{704, 1011, 1218, 703, 488, 705, 1125, 924, 706, 1126, 925, 
       707, 1127, 926, 708, 1128, 927, 709, 1129, 1012, 1219, 710, 1013, 1220,
        711, 1130, 928, 712, 1131, 929, 713, 1132, 930, 714, 1133, 931, 715, 
       1134, 932, 716, 1135, 933, 717}], 
      Line3DBox[{719, 1014, 1221, 718, 1015, 1222, 720, 504, 721, 1136, 934, 
       722, 1137, 935, 723, 1138, 936, 724, 1139, 1016, 1223, 725, 1017, 1224,
        726, 1018, 1225, 727, 1140, 937, 728, 1141, 938, 729, 1142, 939, 730, 
       1143, 940, 731, 1144, 941, 732}], 
      Line3DBox[{734, 1019, 1226, 733, 1020, 1227, 735, 1021, 1228, 736, 520, 
       737, 1145, 942, 738, 1146, 943, 739, 1147, 1022, 1229, 740, 1023, 1230,
        741, 1024, 1231, 742, 1025, 1232, 743, 1148, 944, 744, 1149, 945, 745,
        1150, 946, 746, 1151, 947, 747}], 
      Line3DBox[{749, 1026, 1233, 748, 1027, 1234, 750, 1028, 1235, 751, 1029,
        1236, 752, 536, 753, 1152, 948, 754, 1153, 1030, 1237, 755, 1031, 
       1238, 756, 1032, 1239, 757, 1033, 1240, 758, 1034, 1241, 759, 1154, 
       949, 760, 1155, 950, 761, 1156, 951, 762}], 
      Line3DBox[{764, 1035, 1242, 763, 1036, 1243, 765, 1037, 1244, 766, 1038,
        1245, 767, 1039, 1246, 768, 552, 769, 1157, 1040, 1247, 770, 1041, 
       1248, 771, 1042, 1249, 772, 1043, 1250, 773, 1044, 1251, 774, 1045, 
       1252, 775, 1158, 952, 776, 1159, 953, 777}], 
      Line3DBox[{781, 1160, 954, 779, 1161, 955, 783, 1162, 956, 785, 1163, 
       957, 787, 1164, 958, 789, 1165, 959, 791, 1166, 569, 793, 1167, 960, 
       795, 1168, 961, 797, 1169, 962, 799, 1170, 963, 801, 1171, 964, 803, 
       1172, 965, 805, 1174, 967, 807}], 
      Line3DBox[{806, 966, 1173, 804, 1265, 1057, 802, 1264, 1056, 800, 1263, 
       1055, 798, 1262, 1054, 796, 1261, 1053, 794, 1260, 1052, 792, 1259, 
       568, 790, 1258, 1051, 788, 1257, 1050, 786, 1256, 1049, 784, 1255, 
       1048, 782, 1254, 1047, 778, 1253, 1046, 780}], 
      Line3DBox[{809, 1058, 1266, 808, 1175, 968, 810, 1176, 969, 811, 1177, 
       970, 812, 1178, 971, 813, 1179, 972, 814, 1180, 1059, 1267, 815, 585, 
       816, 1181, 973, 817, 1182, 974, 818, 1183, 975, 819, 1184, 976, 820, 
       1185, 977, 821, 1186, 978, 822}], 
      Line3DBox[{824, 1060, 1268, 823, 1061, 1269, 825, 1187, 979, 826, 1188, 
       980, 827, 1189, 981, 828, 1190, 982, 829, 1191, 1062, 1270, 830, 1063, 
       1271, 831, 601, 832, 1192, 983, 833, 1193, 984, 834, 1194, 985, 835, 
       1195, 986, 836, 1196, 987, 837}], 
      Line3DBox[{839, 1064, 1272, 838, 1065, 1273, 840, 1066, 1274, 841, 1197,
        988, 842, 1198, 989, 843, 1199, 990, 844, 1200, 1067, 1275, 845, 1068,
        1276, 846, 1069, 1277, 847, 617, 848, 1201, 991, 849, 1202, 992, 850, 
       1203, 993, 851, 1204, 994, 852}], 
      Line3DBox[{854, 1070, 1278, 853, 1071, 1279, 855, 1072, 1280, 856, 1073,
        1281, 857, 1205, 995, 858, 1206, 996, 859, 1207, 1074, 1282, 860, 
       1075, 1283, 861, 1076, 1284, 862, 1077, 1285, 863, 633, 864, 1208, 997,
        865, 1209, 998, 866, 1210, 999, 867}], 
      Line3DBox[{869, 1078, 1286, 868, 1079, 1287, 870, 1080, 1288, 871, 1081,
        1289, 872, 1082, 1290, 873, 1211, 1000, 874, 1212, 1083, 1291, 875, 
       1084, 1292, 876, 1085, 1293, 877, 1086, 1294, 878, 1087, 1295, 879, 
       649, 880, 1213, 1001, 881, 1214, 1002, 882}], 
      Line3DBox[{884, 1088, 1296, 883, 1089, 1297, 885, 1090, 1298, 886, 1091,
        1299, 887, 1092, 1300, 888, 1093, 1301, 889, 1215, 1094, 1302, 890, 
       1095, 1303, 891, 1096, 1304, 892, 1097, 1305, 893, 1098, 1306, 894, 
       1099, 1307, 895, 665, 896, 1216, 1003, 897}], 
      Line3DBox[{911, 1009, 685, 910, 1321, 1111, 909, 1320, 1110, 908, 1319, 
       1109, 907, 1318, 1108, 906, 1317, 1107, 905, 1316, 1106, 904, 1315, 
       1314, 1105, 903, 1313, 1104, 902, 1312, 1103, 901, 1311, 1102, 900, 
       1310, 1101, 899, 1309, 1100, 898, 1112, 1308, 1007, 1008}]}, {
      Line3DBox[{251, 474, 1113, 252, 488, 280, 1222, 503, 295, 1227, 518, 
       310, 1234, 533, 325, 1243, 548, 340, 1254, 563, 1161, 355, 578, 1175, 
       370, 1269, 593, 385, 1273, 608, 400, 1279, 623, 415, 1287, 638, 430, 
       1297, 653, 445, 1309, 668, 460}], 
      Line3DBox[{253, 475, 1114, 254, 489, 1125, 281, 504, 296, 1228, 519, 
       311, 1235, 534, 326, 1244, 549, 341, 1255, 564, 1162, 356, 579, 1176, 
       371, 594, 1187, 386, 1274, 609, 401, 1280, 624, 416, 1288, 639, 431, 
       1298, 654, 446, 1310, 669, 461}], 
      Line3DBox[{255, 476, 1115, 256, 490, 1126, 282, 505, 1136, 297, 520, 
       312, 1236, 535, 327, 1245, 550, 342, 1256, 565, 1163, 357, 580, 1177, 
       372, 595, 1188, 387, 610, 1197, 402, 1281, 625, 417, 1289, 640, 432, 
       1299, 655, 447, 1311, 670, 462}], 
      Line3DBox[{257, 477, 1116, 258, 491, 1127, 283, 506, 1137, 298, 521, 
       1145, 313, 536, 328, 1246, 551, 343, 1257, 566, 1164, 358, 581, 1178, 
       373, 596, 1189, 388, 611, 1198, 403, 626, 1205, 418, 1290, 641, 433, 
       1300, 656, 448, 1312, 671, 463}], 
      Line3DBox[{259, 478, 1117, 260, 492, 1128, 284, 507, 1138, 299, 522, 
       1146, 314, 537, 1152, 329, 552, 344, 1258, 567, 1165, 359, 582, 1179, 
       374, 597, 1190, 389, 612, 1199, 404, 627, 1206, 419, 642, 1211, 434, 
       1301, 657, 449, 1313, 672, 464}], 
      Line3DBox[{261, 479, 1118, 263, 493, 1129, 285, 508, 1139, 300, 523, 
       1147, 315, 538, 1153, 330, 553, 1157, 345, 568, 1166, 360, 583, 1180, 
       375, 598, 1191, 390, 613, 1200, 405, 628, 1207, 420, 643, 1212, 435, 
       658, 1215, 450, 1314, 673, 465}], 
      Line3DBox[{265, 481, 1119, 266, 1220, 495, 287, 1224, 510, 302, 1230, 
       525, 317, 1238, 540, 332, 1248, 555, 347, 1260, 570, 1167, 362, 585, 
       377, 1271, 600, 392, 1276, 615, 407, 1283, 630, 422, 1292, 645, 437, 
       1303, 660, 452, 1316, 675, 467}], 
      Line3DBox[{267, 482, 1120, 268, 496, 1130, 288, 1225, 511, 303, 1231, 
       526, 318, 1239, 541, 333, 1249, 556, 348, 1261, 571, 1168, 363, 586, 
       1181, 378, 601, 393, 1277, 616, 408, 1284, 631, 423, 1293, 646, 438, 
       1304, 661, 453, 1317, 676, 468}], 
      Line3DBox[{269, 483, 1121, 270, 497, 1131, 289, 512, 1140, 304, 1232, 
       527, 319, 1240, 542, 334, 1250, 557, 349, 1262, 572, 1169, 364, 587, 
       1182, 379, 602, 1192, 394, 617, 409, 1285, 632, 424, 1294, 647, 439, 
       1305, 662, 454, 1318, 677, 469}], 
      Line3DBox[{271, 484, 1122, 272, 498, 1132, 290, 513, 1141, 305, 528, 
       1148, 320, 1241, 543, 335, 1251, 558, 350, 1263, 573, 1170, 365, 588, 
       1183, 380, 603, 1193, 395, 618, 1201, 410, 633, 425, 1295, 648, 440, 
       1306, 663, 455, 1319, 678, 470}], 
      Line3DBox[{273, 485, 1123, 274, 499, 1133, 291, 514, 1142, 306, 529, 
       1149, 321, 544, 1154, 336, 1252, 559, 351, 1264, 574, 1171, 366, 589, 
       1184, 381, 604, 1194, 396, 619, 1202, 411, 634, 1208, 426, 649, 441, 
       1307, 664, 456, 1320, 679, 471}], 
      Line3DBox[{275, 486, 1124, 276, 500, 1134, 292, 515, 1143, 307, 530, 
       1150, 322, 545, 1155, 337, 560, 1158, 352, 1265, 575, 1172, 367, 590, 
       1185, 382, 605, 1195, 397, 620, 1203, 412, 635, 1209, 427, 650, 1213, 
       442, 665, 457, 1321, 680, 472}], 
      Line3DBox[{277, 682, 1322, 683, 278, 501, 1135, 293, 516, 1144, 308, 
       531, 1151, 323, 546, 1156, 338, 561, 1159, 353, 576, 1173, 1174, 368, 
       591, 1186, 383, 606, 1196, 398, 621, 1204, 413, 636, 1210, 428, 651, 
       1214, 443, 666, 1216, 458, 685, 686, 687}], 
      Line3DBox[{459, 667, 684, 1308, 444, 652, 1296, 429, 637, 1286, 414, 
       622, 1278, 399, 607, 1272, 384, 592, 1268, 369, 577, 1266, 354, 1160, 
       562, 1253, 339, 547, 1242, 324, 532, 1233, 309, 517, 1226, 294, 502, 
       1221, 279, 487, 1218, 250, 473, 681, 688}], 
      Line3DBox[{466, 674, 1315, 451, 659, 1302, 436, 644, 1291, 421, 629, 
       1282, 406, 614, 1275, 391, 599, 1270, 376, 584, 1267, 361, 569, 1259, 
       346, 554, 1247, 331, 539, 1237, 316, 524, 1229, 301, 509, 1223, 286, 
       494, 1219, 264, 480, 1217, 262}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0u3dUzv//Bl42SZGdZCRFikSp9CgqMiNUSppCGyEp7YW2hvbee8/H3b3S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    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.5755448525861287`*^9, 3.5865814781258163`*^9, 
  3.5873808545513144`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"uyi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.5054112035099487`*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nHlcTfv3/xOVzCKRSkgyRCVk6phVMkbIUDIPiTJFSKgMGVLRbGqWNB2N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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJw1mXngTVX3xu89e59QCiHzTIY0GKIUMqUyhaIXjYQyJ2k0ZMicprdoLg2S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         "]], Polygon3DBox[CompressedData["
1:eJw1mnfgz1UfxT/33o+RJMloyMompfGkTVMKpaVCGS0aGkakjJKUStEwCiEq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         "]], Polygon3DBox[CompressedData["
1:eJwt13ncjWUaAODvO+dIliRLRpFBGEKTtawjW9nK3tgauylLZclkKWPf96US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         "]]}]}, {}, {}, {}, {}}, {
     {GrayLevel[0], Line3DBox[CompressedData["
1:eJwt0rsvpFEYwOGPcRsMK6Jd0RAFpSgoN4pViSjoiGK3WIlsQakQOh1/AS0R
+k3o3MfMuM2ystkIu2Tjfg3PSRS/eU7mO5nznsxX0zvQ8S0niqJR3SqsL/Vb
87lR9IE/WM0NNvKQrTxnO5/Zw9JYFCX01bqew2zmONs4xS7OsJ+L/M5l/ucJ
77Wta/3TioFSXGWaa8xwnTthJu5yk3vcYlL74Xd4wBSzTPMnMzzkDo+4y1+c
dY9TPuhYN+GOmjBbkiNc4iAX2MdpdnKSnzjGJg6xjrX6Yh1nNx+d8Zl/2cIs
G8K9+JEX9v3hncp1FfaqyoxlrGSCFSwNe1jCBItZwng4j0UsYL4KrWMsCP9x
+I6vytNLeKYnGSOa83H2/j68Acy/Svc=
       "]]}, {
      Line3DBox[{690, 1004, 473, 689, 1113, 912, 691, 1114, 913, 692, 1115, 
       914, 693, 1116, 915, 694, 1117, 916, 695, 1118, 1010, 1217, 696, 1119, 
       917, 697, 1120, 918, 698, 1121, 919, 699, 1122, 920, 700, 1123, 921, 
       701, 1124, 922, 702, 1322, 1005, 923, 1006}], 
      Line3DBox[{704, 1011, 1218, 703, 488, 705, 1125, 924, 706, 1126, 925, 
       707, 1127, 926, 708, 1128, 927, 709, 1129, 1012, 1219, 710, 1013, 1220,
        711, 1130, 928, 712, 1131, 929, 713, 1132, 930, 714, 1133, 931, 715, 
       1134, 932, 716, 1135, 933, 717}], 
      Line3DBox[{719, 1014, 1221, 718, 1015, 1222, 720, 504, 721, 1136, 934, 
       722, 1137, 935, 723, 1138, 936, 724, 1139, 1016, 1223, 725, 1017, 1224,
        726, 1018, 1225, 727, 1140, 937, 728, 1141, 938, 729, 1142, 939, 730, 
       1143, 940, 731, 1144, 941, 732}], 
      Line3DBox[{734, 1019, 1226, 733, 1020, 1227, 735, 1021, 1228, 736, 520, 
       737, 1145, 942, 738, 1146, 943, 739, 1147, 1022, 1229, 740, 1023, 1230,
        741, 1024, 1231, 742, 1025, 1232, 743, 1148, 944, 744, 1149, 945, 745,
        1150, 946, 746, 1151, 947, 747}], 
      Line3DBox[{749, 1026, 1233, 748, 1027, 1234, 750, 1028, 1235, 751, 1029,
        1236, 752, 536, 753, 1152, 948, 754, 1153, 1030, 1237, 755, 1031, 
       1238, 756, 1032, 1239, 757, 1033, 1240, 758, 1034, 1241, 759, 1154, 
       949, 760, 1155, 950, 761, 1156, 951, 762}], 
      Line3DBox[{764, 1035, 1242, 763, 1036, 1243, 765, 1037, 1244, 766, 1038,
        1245, 767, 1039, 1246, 768, 552, 769, 1157, 1040, 1247, 770, 1041, 
       1248, 771, 1042, 1249, 772, 1043, 1250, 773, 1044, 1251, 774, 1045, 
       1252, 775, 1158, 952, 776, 1159, 953, 777}], 
      Line3DBox[{781, 1160, 954, 779, 1161, 955, 783, 1162, 956, 785, 1163, 
       957, 787, 1164, 958, 789, 1165, 959, 791, 1166, 569, 793, 1167, 960, 
       795, 1168, 961, 797, 1169, 962, 799, 1170, 963, 801, 1171, 964, 803, 
       1172, 965, 805, 1174, 967, 807}], 
      Line3DBox[{806, 966, 1173, 804, 1265, 1057, 802, 1264, 1056, 800, 1263, 
       1055, 798, 1262, 1054, 796, 1261, 1053, 794, 1260, 1052, 792, 1259, 
       568, 790, 1258, 1051, 788, 1257, 1050, 786, 1256, 1049, 784, 1255, 
       1048, 782, 1254, 1047, 778, 1253, 1046, 780}], 
      Line3DBox[{809, 1058, 1266, 808, 1175, 968, 810, 1176, 969, 811, 1177, 
       970, 812, 1178, 971, 813, 1179, 972, 814, 1180, 1059, 1267, 815, 585, 
       816, 1181, 973, 817, 1182, 974, 818, 1183, 975, 819, 1184, 976, 820, 
       1185, 977, 821, 1186, 978, 822}], 
      Line3DBox[{824, 1060, 1268, 823, 1061, 1269, 825, 1187, 979, 826, 1188, 
       980, 827, 1189, 981, 828, 1190, 982, 829, 1191, 1062, 1270, 830, 1063, 
       1271, 831, 601, 832, 1192, 983, 833, 1193, 984, 834, 1194, 985, 835, 
       1195, 986, 836, 1196, 987, 837}], 
      Line3DBox[{839, 1064, 1272, 838, 1065, 1273, 840, 1066, 1274, 841, 1197,
        988, 842, 1198, 989, 843, 1199, 990, 844, 1200, 1067, 1275, 845, 1068,
        1276, 846, 1069, 1277, 847, 617, 848, 1201, 991, 849, 1202, 992, 850, 
       1203, 993, 851, 1204, 994, 852}], 
      Line3DBox[{854, 1070, 1278, 853, 1071, 1279, 855, 1072, 1280, 856, 1073,
        1281, 857, 1205, 995, 858, 1206, 996, 859, 1207, 1074, 1282, 860, 
       1075, 1283, 861, 1076, 1284, 862, 1077, 1285, 863, 633, 864, 1208, 997,
        865, 1209, 998, 866, 1210, 999, 867}], 
      Line3DBox[{869, 1078, 1286, 868, 1079, 1287, 870, 1080, 1288, 871, 1081,
        1289, 872, 1082, 1290, 873, 1211, 1000, 874, 1212, 1083, 1291, 875, 
       1084, 1292, 876, 1085, 1293, 877, 1086, 1294, 878, 1087, 1295, 879, 
       649, 880, 1213, 1001, 881, 1214, 1002, 882}], 
      Line3DBox[{884, 1088, 1296, 883, 1089, 1297, 885, 1090, 1298, 886, 1091,
        1299, 887, 1092, 1300, 888, 1093, 1301, 889, 1215, 1094, 1302, 890, 
       1095, 1303, 891, 1096, 1304, 892, 1097, 1305, 893, 1098, 1306, 894, 
       1099, 1307, 895, 665, 896, 1216, 1003, 897}], 
      Line3DBox[{911, 1009, 685, 910, 1321, 1111, 909, 1320, 1110, 908, 1319, 
       1109, 907, 1318, 1108, 906, 1317, 1107, 905, 1316, 1106, 904, 1315, 
       1314, 1105, 903, 1313, 1104, 902, 1312, 1103, 901, 1311, 1102, 900, 
       1310, 1101, 899, 1309, 1100, 898, 1112, 1308, 1007, 1008}]}, {
      Line3DBox[{251, 474, 1113, 252, 488, 280, 1222, 503, 295, 1227, 518, 
       310, 1234, 533, 325, 1243, 548, 340, 1254, 563, 1161, 355, 578, 1175, 
       370, 1269, 593, 385, 1273, 608, 400, 1279, 623, 415, 1287, 638, 430, 
       1297, 653, 445, 1309, 668, 460}], 
      Line3DBox[{253, 475, 1114, 254, 489, 1125, 281, 504, 296, 1228, 519, 
       311, 1235, 534, 326, 1244, 549, 341, 1255, 564, 1162, 356, 579, 1176, 
       371, 594, 1187, 386, 1274, 609, 401, 1280, 624, 416, 1288, 639, 431, 
       1298, 654, 446, 1310, 669, 461}], 
      Line3DBox[{255, 476, 1115, 256, 490, 1126, 282, 505, 1136, 297, 520, 
       312, 1236, 535, 327, 1245, 550, 342, 1256, 565, 1163, 357, 580, 1177, 
       372, 595, 1188, 387, 610, 1197, 402, 1281, 625, 417, 1289, 640, 432, 
       1299, 655, 447, 1311, 670, 462}], 
      Line3DBox[{257, 477, 1116, 258, 491, 1127, 283, 506, 1137, 298, 521, 
       1145, 313, 536, 328, 1246, 551, 343, 1257, 566, 1164, 358, 581, 1178, 
       373, 596, 1189, 388, 611, 1198, 403, 626, 1205, 418, 1290, 641, 433, 
       1300, 656, 448, 1312, 671, 463}], 
      Line3DBox[{259, 478, 1117, 260, 492, 1128, 284, 507, 1138, 299, 522, 
       1146, 314, 537, 1152, 329, 552, 344, 1258, 567, 1165, 359, 582, 1179, 
       374, 597, 1190, 389, 612, 1199, 404, 627, 1206, 419, 642, 1211, 434, 
       1301, 657, 449, 1313, 672, 464}], 
      Line3DBox[{261, 479, 1118, 263, 493, 1129, 285, 508, 1139, 300, 523, 
       1147, 315, 538, 1153, 330, 553, 1157, 345, 568, 1166, 360, 583, 1180, 
       375, 598, 1191, 390, 613, 1200, 405, 628, 1207, 420, 643, 1212, 435, 
       658, 1215, 450, 1314, 673, 465}], 
      Line3DBox[{265, 481, 1119, 266, 1220, 495, 287, 1224, 510, 302, 1230, 
       525, 317, 1238, 540, 332, 1248, 555, 347, 1260, 570, 1167, 362, 585, 
       377, 1271, 600, 392, 1276, 615, 407, 1283, 630, 422, 1292, 645, 437, 
       1303, 660, 452, 1316, 675, 467}], 
      Line3DBox[{267, 482, 1120, 268, 496, 1130, 288, 1225, 511, 303, 1231, 
       526, 318, 1239, 541, 333, 1249, 556, 348, 1261, 571, 1168, 363, 586, 
       1181, 378, 601, 393, 1277, 616, 408, 1284, 631, 423, 1293, 646, 438, 
       1304, 661, 453, 1317, 676, 468}], 
      Line3DBox[{269, 483, 1121, 270, 497, 1131, 289, 512, 1140, 304, 1232, 
       527, 319, 1240, 542, 334, 1250, 557, 349, 1262, 572, 1169, 364, 587, 
       1182, 379, 602, 1192, 394, 617, 409, 1285, 632, 424, 1294, 647, 439, 
       1305, 662, 454, 1318, 677, 469}], 
      Line3DBox[{271, 484, 1122, 272, 498, 1132, 290, 513, 1141, 305, 528, 
       1148, 320, 1241, 543, 335, 1251, 558, 350, 1263, 573, 1170, 365, 588, 
       1183, 380, 603, 1193, 395, 618, 1201, 410, 633, 425, 1295, 648, 440, 
       1306, 663, 455, 1319, 678, 470}], 
      Line3DBox[{273, 485, 1123, 274, 499, 1133, 291, 514, 1142, 306, 529, 
       1149, 321, 544, 1154, 336, 1252, 559, 351, 1264, 574, 1171, 366, 589, 
       1184, 381, 604, 1194, 396, 619, 1202, 411, 634, 1208, 426, 649, 441, 
       1307, 664, 456, 1320, 679, 471}], 
      Line3DBox[{275, 486, 1124, 276, 500, 1134, 292, 515, 1143, 307, 530, 
       1150, 322, 545, 1155, 337, 560, 1158, 352, 1265, 575, 1172, 367, 590, 
       1185, 382, 605, 1195, 397, 620, 1203, 412, 635, 1209, 427, 650, 1213, 
       442, 665, 457, 1321, 680, 472}], 
      Line3DBox[{277, 682, 1322, 683, 278, 501, 1135, 293, 516, 1144, 308, 
       531, 1151, 323, 546, 1156, 338, 561, 1159, 353, 576, 1173, 1174, 368, 
       591, 1186, 383, 606, 1196, 398, 621, 1204, 413, 636, 1210, 428, 651, 
       1214, 443, 666, 1216, 458, 685, 686, 687}], 
      Line3DBox[{459, 667, 684, 1308, 444, 652, 1296, 429, 637, 1286, 414, 
       622, 1278, 399, 607, 1272, 384, 592, 1268, 369, 577, 1266, 354, 1160, 
       562, 1253, 339, 547, 1242, 324, 532, 1233, 309, 517, 1226, 294, 502, 
       1221, 279, 487, 1218, 250, 473, 681, 688}], 
      Line3DBox[{466, 674, 1315, 451, 659, 1302, 436, 644, 1291, 421, 629, 
       1282, 406, 614, 1275, 391, 599, 1270, 376, 584, 1267, 361, 569, 1259, 
       346, 554, 1247, 331, 539, 1237, 316, 524, 1229, 301, 509, 1223, 286, 
       494, 1219, 264, 480, 1217, 262}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0unc01////19kREkyU1KUEoWW2U1TkT1D9k62UJKVSGRk7+yRvdfd3nsT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    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.5755448528041525`*^9, 3.586581478217821*^9, 
  3.587380854659336*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"uxxi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.5054112035099487`*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1vHk4Ve0X8C9KIpUQigZJJVHRTLtBhDJVilSoJCElokklUSkcc2aZ5/Fw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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJwtmXnAVVMXxu89ex80z0KaNWhERalQFM0JKUmkTGmUDEVKidKkIpUyZiyk
SYUGZMrYoERUIhEqY+H7Pd9z/ljv+r37nnvuOXtY+1lrV+49sMuAJJfLPcGf
gH8NXwL/dT6XqwG3hdvC1eE2cBv4dLg73B1uDl8FXwk3grvB3eBicAP4NLgs
fCZ8JlwGbgw3hivC58It4ePgpnBTuDjcEG4AHws3gZvA1eAL4AvgCnBLuAVc
D+4CXwgXhU+DT4X3YYXgsvgfscLw8fgfsIJwGfzOvN87xe/CInw0vi73uRDu
DP+KlYYL4v/BH8GKYvuxPdhO7IDeDV8Yvz/vPoz4X7BS8DH4n7GS8FH4bzD1
e8CXB1rA58Bd4fHwvfAseAP8Afwg/B78Dvww/AH8PjwJXgOvhm+E58Fz4Xvg
FfByeAb8LrwefgB+G34TvhteBi+Bp8JvwevgMfByeCk8DV4PvwVPh9+B34Zn
wx/CG+Cx8KvwMng8/Bq8Cr4GngXPhPvAM+EH4SvgqfAUuCc8BZ4MXwLfB4+D
J8Cvaz7CfeGH4YfgJxmgVvz/MYM2Eb+a9jdorxQ8DzUf78fegF+nvSM8Cr4L
bg3fDt8Kj4aXwovhg5pncDH8HuxouAj+Iq4ZBg+Du8A3wzfDF8O3wLdk9x8M
D4Y7wUPgIXB7eBA8EL4QHgoPhQfD0+Hp8BT4FfgVuDN8E3wTPBleBC+CH4JX
wiuysX4ZfhmeB6+F18Lt4IHwAPhSeDh8O3w9PBG+H74BngRPhK9TH8ET4AHZ
WEyFr4UnwOPz7uPr4Ovgy+A74TvgbvAIeDh8AXwj3A/uDt8Bj4DPh/vBN8Bt
4P7wjXBbeADcH96NpXAB/W703LufMbyNax6B59BeAK4D19GYwLXgWvAxcG24
NpyHq8EnwUXgU+FT4P+wqnAV/Q7tNeGa8B/Y8fCx+G+xo+BC8lxzMnwyvFe/
DZfC/67YBJfGf6/fhkvg52brbg38N3YifCL+T+wE+Dj8SK55HH4cvgt+QnEW
vhN+DH4MvhWeA8+Gb4Fnw7Pgv7By8An4cbQ/Dz8Hj4KfhJ+EFUBOwlWF74Vf
gJ9Xn+gzPRfuH3xlfCX8v1gVuDJ+ENdPg6fBN8FPw0/B/eFH4XmKIdF9vfT/
ATKXOw8+hbax0eOxhPaE9g6KubSNie7HxbSntF8ON6Itn3rtnQr/h50L18cf
wc7RGOP/xVrC9fCHsbM1xvh/sBZwXXxIHSMawDF1rGwIF4T7aY+Bd2OV4DL4
O6PHZiHPMyh6XB+F74ju3wXwyOh+eQkeET1+L8D74frcpyJ+YPTYz6O9X/Q+
MQf+Gz4rcczpHz2f5tJ+fXR8nwXfGD3XH4FviN5vZsOjosdvEVyM578ZPpu2
IqnHozn8M3YKXAl/CDtDcx3/O9YEroH/CasDV8D/hjWGq+N/xGrD5fEHsYaa
K/hfsdPgKnoWrBxcCn8AawBXxRfnGYbB58C/YKfClfFFaR8KnwUXTj1XmsF7
sCpwWfwf2Jlab5ojqeP+GZpHqeN4Y82R1HH/dLhQ6n2rKVwg9Z7RRH2JVYaP
1VhhzbT+8T9gJ8Pl8N9hVeHjNG5YU61hjTNWEz4Bvw+rBZ+oMcROgo+Pfo9n
GIL5WpvR8eJlxmJ4dIx4Pvgel3PNZXl/71K4Kxz1TvDpiidcV1x7jeJldBx4
Fz5M+6fww8G/fwV8ed79dAl8cd5jci18Td7vejF8Ud7j1gvumff7dYUvyfu9
e8I98h7DvnAfxSh+Y6Hmf2rtofVeMYsz2i9egu+AX4JfzHtsn4OfyeK/YtGj
ec/BZ+Gn4Z+0p8IV8CNofxFeCP+G1YCr5x1bFG/r44dzzUJ4AXwEOwWuhw8a
Z7gRfBirD9fF3x0dl18JnnfX0N4377l8JXyF4ijvcx//t+XaW6J12DNcX4L2
22lvQVs1eKb6FG4UvDd9hv9T6ynnuX1rtD57lvZh0ZrsafimaF34BHxztD6b
Dw+J1o6Pw7dH7wHPwYOjteNjwWuiD/e/Wn3A/7fCn+BL034+3Jr2wlpD8FnZ
3tQCPjtvvSdt2SlvnXkV3CvvtX413Ft7FNwBbgcXyt6lOVxQ6w9ulvfa7QR3
0P4Dd4Y7qt/gjnB7uCTcCj4PLgW3hlvBA+EH4AfgIfAMeEbe2mwUPAp+Ct4K
b4GnBuvPK9VPOWvfb7E/sQPqczHXVNA+Evz5rznrY+UWW/Cb1KfwZnijxhL+
At4GPwl/Dm9W/8KbdE/4EP4XbHf2O99j32HPcs12/Bd5f6ZrdmE/Ztfpmr95
jjfxf+Ff5vpPNEbaA+CN8KfwQfwP2N7sffTcO7E/snvms/dQe7GsTZ/t031z
/v432ec/Z9f8lLXvza4/lN1Hz/Vb1leLE+v/F7UWsS/h7XnnXcodEvxy2ovA
5bI9/XDO/fsv71Me/yR+QLRmzSse4svSXhx/W7SuDfglia9fltjEiheKXeoz
/eaRYL3wIv4/WTaOKxPnOuXzHsv/sv75JesDff8Qn1XHV8v686+sT2cG97n6
fgH2NbxDOiTra93nedq/wn+Z9+/qHf/R9xK/S+T5X4B3wF/l3Yd/ZvPgvcR9
tT/73d+zsVAcfpXPhkbHxp3ZPd9PHKuT6N/JZxrplezZFiUenwPZPBofnYsW
wZfAn0fbuXxnW+IYq/iXwI3ghvDoaD16NP7rxHGygGJG4jipGPA9Vg8uqD0x
eP1qj96bOGYW1r4ZvH61Z32XOGYWUrwMjgna+56PzjGku3Yk7v9jFNuyvam4
tE8WcxRvpIkugrtov8hiSEntrVmMKq19D24Ol4D3Z+9VVOs9i+HFpFmC45v2
0P+yeKL99yja2+e85/6bxRzpgSnReb702D9ZH2pP/z1bI1ordYL3QWlLacm7
4JE8Z+ng/bd6prOUZw3SHkp775z14bFwj5x10SPRWkzXTo+uC0i/VQjeU6Qf
Tgjei6VPlFcov1gqfRC850o/bNX34BQ+LnjPldb6PPGeeJS0kvRczppWOlHP
c5V0QnAMl6ZtqXwmZ71aK3iPk+5tEZyHSrs2CX4v5ZEFgmO4tFzB4D1C+0PN
4L1e2rhYsB6QTiscPKbSh9WD9yPp5KLBGkMar3iwbpH2KxKsMaTlcllMUTxR
0aNutva/TVyHkeZ/EPsI7svnk4LrAD3hp7lmG7yV708PrlFcTfsDWmPqB3ha
cI2iNzw5uG5wBTwjuG7QB16XOO+WZp4SXKPoRfvaxLUUaeaHsI/ha2i/GV4A
t4PXJK7JSIe/lThH+78+h+fquzzbBcFzSTqzTfC+pv1tKf/3SJybDKR9Pu2t
4OuDtZB00/vROZW0x4W035uzju0SXJ+RRn0jOq9W3tE5uHYhfXspPDlnTdst
uL4hndw9ONeW5hoUrD/P130S14WUL0wMrs9cHp1jKn5+rPwhcT98lLfeUT+/
B4/h+lVwV67/MLpeIC00NrgOcyk8J/E4fsj1V8P3wPfk3Qcj9b5wb3gsPBbu
BY+GR8NXwnfDd8NXwWPgMfDqxHUt5TL3BNdquqnPgmsyl8G3BetP1S3uC86X
9U53BNdeOkfn/Mr9e8B3wkvgTvDo4PrVxfCwYG3ZXlouWMd2hMcF13m6Sx8G
a9QO8KjgetRF8MjgGteF8F3Bda0u8IDgnLc1PCRYD7eBBwdr4AvgzxLXDaT9
tiSOFcolv8jiueL/9sT1RuWP0v3S/9KTH2R7hPamzYnrFco3NyauMyi/KBNc
J3kTvylxfUO55/rENQTpzIOJ664r4d/gdvBr8PHBtZe3tJ4S1zqkaT9OXA+R
1t2ZuNYqjX0ocS13Fbw7cT1W+fseuBm8TFogca1V+fuGxLUC6eEDieuuK+Af
E9ddlys2JK4/SJPvy/Ya7XFfJq79Kp/6KXGN91X4q8R1XeW8JYPrS2vxpbDr
4XXSNcE1ovVaB9H1IOn83sG1ROWk0v6KAy/Qt28nrucor1e9QetuEu3vJK6f
SMNfG1wXPZfv9gquQyqHVW6htbNS81yxJue8+7rgGsh58FXBtUfl5tcEx5aW
cAiuga/GR+wyeA0+Ca6Bv4GvH1wn/Ah/cnAN8AN83eC64of4csH1sbfxFYPr
eO/gawfXGzfgawTXBt/HVw6uK76LLxScO2iPOJx4v9BefyRxPfx1Pq8WXJN8
D181eK/R/lg/teZWTD4ltdZX7G0YXD/8FF8vtXZXzP8kOieUdtoUneNpH9wQ
Xcc8xPWfReeH0mxDg+uWB/BfR+cz0hWLo2uFO9Tf0TUNzf+Pomuav8EPRp8x
aD4siq5pfgnPiD5j0JzZGJ1Pao9eEl2j/FpzLDrPkVbZEp1/ShfdkMUixfbb
s1ikuLQ1OqeSHvs8OreXZpsZXUvR+uoXXI/9WWsuuGb7C35HdL4nrdU3uP62
H/9ldN4l7fRFdB4oXdefzxZzza/4bdF5nXTdtOizFs2fh6NrO5oPC6Nrx1sV
26Jr0NvgBdH14s/1TtH16O2Ki9E16y/0jNG12q/gT6M1unKBedE1N82lZ6Nr
sps036PrbJqfz0TXbTfC86PruZoPz0XXmjfrXaPr3VvgOdH1Gc3J+6LPfrTe
740+j1Gs+Dg6P1GeonxAuZH09tTocx2tl1nRtSOt/e3Rmlvae0L0uZHiz6To
syLFwNnRNR+tl8nRZ0Ka8xOjz5wU6+6PPkNS/Nwcrdelzz+IrrMfpH1VdO17
D9wjuGa7F98zuNa6T7EluH77g9Z6sE7QPt4uOI/+Bt82eH+XxmgavIeqf84I
rp+rn6/AnoJ/xJ8eXDNXPzcL3mc1pmcGaxVpwkuC68Pf4TsG5+m78R2CzxF2
BeezyndfSpyvKb9aha+bOt+WllB+J53bJHVeqTMs5UiNgzWAxlR54K6cc1LV
rzRGGqv2wbXinfgVic/AdP+uwbX67/EXB9eu1YedtIbhb/HvRdevFQeaB+sH
zeezg3WI5urw4LMMjcWr0TpD7/VudL1b62UEti7nOLMyWsfo/n2Ca+M/4ScE
19wUQ16LPudQvy2L1jQao/HBmko6ROcO6rPa9MmKaA2kvl0erXv0vq9Hn6Po
HVdH1000N9ZGn+VobqyLrqFoTFsHn7kovp0ffOaiuHQONi7ndXpWsNbSWn4z
uu6v528VfC6jNXte8FmM4t5b0ecEii3ro88VFIvWRNfxNSffiT5jUFyaG10/
V8x/IPrsVvtUpdTnf9JdJ6U+U5S+qpr6jFB66cTUZ2/SWuVTn6VJU1VJff4n
7VQ59bmgNNXpqXN46fld0XUo5ZLlUp8FSqtXSH1WJ832dPS5kWp0FVOfO0rv
PRZd/9d+Vzp1vVL7b6nU9Uft3TVSn4lK0zZOXVNQXtAwdT1IuUaj1PUU5SM1
U59TSpd+E10XU85bPfUZqjTz8anPSqX9SqauY2p/Py71uaZ0YJnU54XKEY5N
fW4q3Vg29VmpNPy46PMq6agzUtcslDfVSV3Pkr6tlVrHSxs/Hn0OIg3wRPTZ
inRCg9R1MeVQp8LzE9fcTk6t46V1/wd7pOlk
         "]], Polygon3DBox[CompressedData["
1:eJwtmnfgV9Mfxu8ZX6IkSaWSkjJKWsrM+NkjyagoUajM7PoRDaXsllW0rBRK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         "]], Polygon3DBox[CompressedData["
1:eJwt1nf8lWMbAPDzO+e0h4xQIclqSHsP7b2HaEoLDaOFEm0VpYnKKBIK72tU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         "]]}]}, {}, {}, {}, {}}, {
     {GrayLevel[0], Line3DBox[CompressedData["
1:eJwt0ssqRVEcwOHl7rhLSEIegMiUkiklhpRkRmFKyRBlTh6AF3BMTZiZuZzj
figkl6TcR74Vg9/+/u29aq29281j04NTWSGEBX0qzm+60XZ2CBXcYxNP2Mp7
dsX17GN+TgjDrGGtJswdnGUPlzjAVY5yg9Pc4jx3+cN7fulA73rSvgMdxns8
4iFTPGKaKR4zzRMeR3VqPuUZz3jOc17wgpe8ZIYZXnHT+zzwW9f60LPWnG2f
y9zhHJOc5DpHuMJ+LrKbM2xnm8bNVRxiLnvj+9mrk3dsCfHB38dvjN/Amju+
qlwvulWD52WsZynrWMJaFrOaRXEvJljJQpbF9SowFzOfCeaxIOtv6zzaNuTQ
kULS5fH/n/gFnvtHLQ==
       "]]}, {
      Line3DBox[{702, 1015, 480, 701, 1129, 922, 703, 1130, 923, 704, 1131, 
       924, 705, 1132, 925, 706, 1133, 926, 707, 1134, 1025, 1232, 708, 1135, 
       927, 709, 1136, 928, 710, 1137, 929, 711, 1138, 930, 712, 1139, 931, 
       713, 1128, 1140, 932, 1022, 1338, 1014, 1021, 1023, 933, 1016}], 
      Line3DBox[{715, 1026, 1233, 714, 494, 716, 1141, 934, 717, 1142, 935, 
       718, 1143, 936, 719, 1144, 937, 720, 1145, 1027, 1234, 721, 1028, 1235,
        722, 1146, 938, 723, 1147, 939, 724, 1148, 940, 725, 1149, 941, 726, 
       1150, 942, 727, 1337, 1020, 943, 1024}], 
      Line3DBox[{729, 1029, 1236, 728, 1030, 1237, 730, 509, 731, 1151, 944, 
       732, 1152, 945, 733, 1153, 946, 734, 1154, 1031, 1238, 735, 1032, 1239,
        736, 1033, 1240, 737, 1155, 947, 738, 1156, 948, 739, 1157, 949, 740, 
       1158, 950, 741, 1159, 951, 742}], 
      Line3DBox[{744, 1034, 1241, 743, 1035, 1242, 745, 1036, 1243, 746, 525, 
       747, 1160, 952, 748, 1161, 953, 749, 1162, 1037, 1244, 750, 1038, 1245,
        751, 1039, 1246, 752, 1040, 1247, 753, 1163, 954, 754, 1164, 955, 755,
        1165, 956, 756, 1166, 957, 757}], 
      Line3DBox[{759, 1041, 1248, 758, 1042, 1249, 760, 1043, 1250, 761, 1044,
        1251, 762, 541, 763, 1167, 958, 764, 1168, 1045, 1252, 765, 1046, 
       1253, 766, 1047, 1254, 767, 1048, 1255, 768, 1049, 1256, 769, 1169, 
       959, 770, 1170, 960, 771, 1171, 961, 772}], 
      Line3DBox[{774, 1050, 1257, 773, 1051, 1258, 775, 1052, 1259, 776, 1053,
        1260, 777, 1054, 1261, 778, 557, 779, 1172, 1055, 1262, 780, 1056, 
       1263, 781, 1057, 1264, 782, 1058, 1265, 783, 1059, 1266, 784, 1060, 
       1267, 785, 1173, 962, 786, 1174, 963, 787}], 
      Line3DBox[{791, 1175, 964, 789, 1176, 965, 793, 1177, 966, 795, 1178, 
       967, 797, 1179, 968, 799, 1180, 969, 801, 1181, 574, 803, 1182, 970, 
       805, 1183, 971, 807, 1184, 972, 809, 1185, 973, 811, 1186, 974, 813, 
       1187, 975, 815, 1189, 977, 817}], 
      Line3DBox[{816, 976, 1188, 814, 1280, 1072, 812, 1279, 1071, 810, 1278, 
       1070, 808, 1277, 1069, 806, 1276, 1068, 804, 1275, 1067, 802, 1274, 
       573, 800, 1273, 1066, 798, 1272, 1065, 796, 1271, 1064, 794, 1270, 
       1063, 792, 1269, 1062, 788, 1268, 1061, 790}], 
      Line3DBox[{819, 1073, 1281, 818, 1190, 978, 820, 1191, 979, 821, 1192, 
       980, 822, 1193, 981, 823, 1194, 982, 824, 1195, 1074, 1282, 825, 590, 
       826, 1196, 983, 827, 1197, 984, 828, 1198, 985, 829, 1199, 986, 830, 
       1200, 987, 831, 1201, 988, 832}], 
      Line3DBox[{834, 1075, 1283, 833, 1076, 1284, 835, 1202, 989, 836, 1203, 
       990, 837, 1204, 991, 838, 1205, 992, 839, 1206, 1077, 1285, 840, 1078, 
       1286, 841, 606, 842, 1207, 993, 843, 1208, 994, 844, 1209, 995, 845, 
       1210, 996, 846, 1211, 997, 847}], 
      Line3DBox[{849, 1079, 1287, 848, 1080, 1288, 850, 1081, 1289, 851, 1212,
        998, 852, 1213, 999, 853, 1214, 1000, 854, 1215, 1082, 1290, 855, 
       1083, 1291, 856, 1084, 1292, 857, 622, 858, 1216, 1001, 859, 1217, 
       1002, 860, 1218, 1003, 861, 1219, 1004, 862}], 
      Line3DBox[{864, 1085, 1293, 863, 1086, 1294, 865, 1087, 1295, 866, 1088,
        1296, 867, 1220, 1005, 868, 1221, 1006, 869, 1222, 1089, 1297, 870, 
       1090, 1298, 871, 1091, 1299, 872, 1092, 1300, 873, 638, 874, 1223, 
       1007, 875, 1224, 1008, 876, 1225, 1009, 877}], 
      Line3DBox[{879, 1093, 1301, 878, 1094, 1302, 880, 1095, 1303, 881, 1096,
        1304, 882, 1097, 1305, 883, 1226, 1010, 884, 1227, 1098, 1306, 885, 
       1099, 1307, 886, 1100, 1308, 887, 1101, 1309, 888, 1102, 1310, 889, 
       654, 890, 1228, 1011, 891, 1229, 1012, 892}], 
      Line3DBox[{894, 1103, 1311, 893, 1104, 1312, 895, 1105, 1313, 896, 1106,
        1314, 897, 1107, 1315, 898, 1108, 1316, 899, 1230, 1109, 1317, 900, 
       1110, 1318, 901, 1111, 1319, 902, 1112, 1320, 903, 1113, 1321, 904, 
       1114, 1322, 905, 670, 906, 1231, 1013, 907}], 
      Line3DBox[{921, 1019, 689, 920, 1336, 1126, 919, 1335, 1125, 918, 1334, 
       1124, 917, 1333, 1123, 916, 1332, 1122, 915, 1331, 1121, 914, 1330, 
       1329, 1120, 913, 1328, 1119, 912, 1327, 1118, 911, 1326, 1117, 910, 
       1325, 1116, 909, 1324, 1115, 908, 1127, 1323, 1017, 1018}]}, {
      Line3DBox[{259, 481, 1129, 260, 494, 287, 1237, 508, 302, 1242, 523, 
       317, 1249, 538, 332, 1258, 553, 347, 1269, 568, 1176, 362, 583, 1190, 
       377, 1284, 598, 392, 1288, 613, 407, 1294, 628, 422, 1302, 643, 437, 
       1312, 658, 452, 1324, 673, 467}], 
      Line3DBox[{261, 482, 1130, 262, 495, 1141, 288, 509, 303, 1243, 524, 
       318, 1250, 539, 333, 1259, 554, 348, 1270, 569, 1177, 363, 584, 1191, 
       378, 599, 1202, 393, 1289, 614, 408, 1295, 629, 423, 1303, 644, 438, 
       1313, 659, 453, 1325, 674, 468}], 
      Line3DBox[{263, 483, 1131, 264, 496, 1142, 289, 510, 1151, 304, 525, 
       319, 1251, 540, 334, 1260, 555, 349, 1271, 570, 1178, 364, 585, 1192, 
       379, 600, 1203, 394, 615, 1212, 409, 1296, 630, 424, 1304, 645, 439, 
       1314, 660, 454, 1326, 675, 469}], 
      Line3DBox[{265, 484, 1132, 266, 497, 1143, 290, 511, 1152, 305, 526, 
       1160, 320, 541, 335, 1261, 556, 350, 1272, 571, 1179, 365, 586, 1193, 
       380, 601, 1204, 395, 616, 1213, 410, 631, 1220, 425, 1305, 646, 440, 
       1315, 661, 455, 1327, 676, 470}], 
      Line3DBox[{267, 485, 1133, 268, 498, 1144, 291, 512, 1153, 306, 527, 
       1161, 321, 542, 1167, 336, 557, 351, 1273, 572, 1180, 366, 587, 1194, 
       381, 602, 1205, 396, 617, 1214, 411, 632, 1221, 426, 647, 1226, 441, 
       1316, 662, 456, 1328, 677, 471}], 
      Line3DBox[{269, 486, 1134, 271, 499, 1145, 292, 513, 1154, 307, 528, 
       1162, 322, 543, 1168, 337, 558, 1172, 352, 573, 1181, 367, 588, 1195, 
       382, 603, 1206, 397, 618, 1215, 412, 633, 1222, 427, 648, 1227, 442, 
       663, 1230, 457, 1329, 678, 472}], 
      Line3DBox[{273, 488, 1135, 274, 1235, 501, 294, 1239, 515, 309, 1245, 
       530, 324, 1253, 545, 339, 1263, 560, 354, 1275, 575, 1182, 369, 590, 
       384, 1286, 605, 399, 1291, 620, 414, 1298, 635, 429, 1307, 650, 444, 
       1318, 665, 459, 1331, 680, 474}], 
      Line3DBox[{275, 489, 1136, 276, 502, 1146, 295, 1240, 516, 310, 1246, 
       531, 325, 1254, 546, 340, 1264, 561, 355, 1276, 576, 1183, 370, 591, 
       1196, 385, 606, 400, 1292, 621, 415, 1299, 636, 430, 1308, 651, 445, 
       1319, 666, 460, 1332, 681, 475}], 
      Line3DBox[{277, 490, 1137, 278, 503, 1147, 296, 517, 1155, 311, 1247, 
       532, 326, 1255, 547, 341, 1265, 562, 356, 1277, 577, 1184, 371, 592, 
       1197, 386, 607, 1207, 401, 622, 416, 1300, 637, 431, 1309, 652, 446, 
       1320, 667, 461, 1333, 682, 476}], 
      Line3DBox[{279, 491, 1138, 280, 504, 1148, 297, 518, 1156, 312, 533, 
       1163, 327, 1256, 548, 342, 1266, 563, 357, 1278, 578, 1185, 372, 593, 
       1198, 387, 608, 1208, 402, 623, 1216, 417, 638, 432, 1310, 653, 447, 
       1321, 668, 462, 1334, 683, 477}], 
      Line3DBox[{281, 492, 1139, 282, 505, 1149, 298, 519, 1157, 313, 534, 
       1164, 328, 549, 1169, 343, 1267, 564, 358, 1279, 579, 1186, 373, 594, 
       1199, 388, 609, 1209, 403, 624, 1217, 418, 639, 1223, 433, 654, 448, 
       1322, 669, 463, 1335, 684, 478}], 
      Line3DBox[{283, 696, 697, 1140, 284, 506, 1150, 299, 520, 1158, 314, 
       535, 1165, 329, 550, 1170, 344, 565, 1173, 359, 1280, 580, 1187, 374, 
       595, 1200, 389, 610, 1210, 404, 625, 1218, 419, 640, 1224, 434, 655, 
       1228, 449, 670, 464, 1336, 685, 479}], 
      Line3DBox[{285, 699, 700, 698, 1338, 686, 695, 693, 1337, 694, 300, 521,
        1159, 315, 536, 1166, 330, 551, 1171, 345, 566, 1174, 360, 581, 1188, 
       1189, 375, 596, 1201, 390, 611, 1211, 405, 626, 1219, 420, 641, 1225, 
       435, 656, 1229, 450, 671, 1231, 465, 689, 690, 691}], 
      Line3DBox[{466, 672, 688, 1323, 451, 657, 1311, 436, 642, 1301, 421, 
       627, 1293, 406, 612, 1287, 391, 597, 1283, 376, 582, 1281, 361, 1175, 
       567, 1268, 346, 552, 1257, 331, 537, 1248, 316, 522, 1241, 301, 507, 
       1236, 286, 493, 1233, 258, 480, 687, 692}], 
      Line3DBox[{473, 679, 1330, 458, 664, 1317, 443, 649, 1306, 428, 634, 
       1297, 413, 619, 1290, 398, 604, 1285, 383, 589, 1282, 368, 574, 1274, 
       353, 559, 1262, 338, 544, 1252, 323, 529, 1244, 308, 514, 1238, 293, 
       500, 1234, 272, 487, 1232, 270}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0u3c4lu8fPk6S0lJGqUiigRChhFdWQ8Pem1BE2WTP7L333nuPuI3H8zw2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    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.575544852890164*^9, 3.5865814782658243`*^9, 
  3.58738085473433*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"uyyi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.505411203510949*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nXk8VH/0/4U2rbSKUqJNizaJNK2kUpL2aN+TpU17WhEpCakk+5LsY3ft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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJw1mXngTVX3xu89e59QCiHzTIY0GKIUMqUyhaIXjYQyJ2k0ZMicprdoLg2S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         "]], Polygon3DBox[CompressedData["
1:eJw1mnfgz1UfxT/33o+RJMloyMompfGkTVMKpaVCGS0aGkakjJKUStEwCiEq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         "]], Polygon3DBox[CompressedData["
1:eJwt13ncjWUaAODvO+dIliRLRpFBGEKTtawjW9nK3tgauylLZclkKWPf96US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         "]]}]}, {}, {}, {}, {}}, {
     {GrayLevel[0], Line3DBox[CompressedData["
1:eJwt0rsvpFEYwOGPcRsMK6Jd0RAFpSgoN4pViSjoiGK3WIlsQakQOh1/AS0R
+k3o3MfMuM2ystkIu2Tjfg3PSRS/eU7mO5nznsxX0zvQ8S0niqJR3SqsL/Vb
87lR9IE/WM0NNvKQrTxnO5/Zw9JYFCX01bqew2zmONs4xS7OsJ+L/M5l/ucJ
77Wta/3TioFSXGWaa8xwnTthJu5yk3vcYlL74Xd4wBSzTPMnMzzkDo+4y1+c
dY9TPuhYN+GOmjBbkiNc4iAX2MdpdnKSnzjGJg6xjrX6Yh1nNx+d8Zl/2cIs
G8K9+JEX9v3hncp1FfaqyoxlrGSCFSwNe1jCBItZwng4j0UsYL4KrWMsCP9x
+I6vytNLeKYnGSOa83H2/j68Acy/Svc=
       "]]}, {
      Line3DBox[{690, 1004, 473, 689, 1113, 912, 691, 1114, 913, 692, 1115, 
       914, 693, 1116, 915, 694, 1117, 916, 695, 1118, 1010, 1217, 696, 1119, 
       917, 697, 1120, 918, 698, 1121, 919, 699, 1122, 920, 700, 1123, 921, 
       701, 1124, 922, 702, 1322, 1005, 923, 1006}], 
      Line3DBox[{704, 1011, 1218, 703, 488, 705, 1125, 924, 706, 1126, 925, 
       707, 1127, 926, 708, 1128, 927, 709, 1129, 1012, 1219, 710, 1013, 1220,
        711, 1130, 928, 712, 1131, 929, 713, 1132, 930, 714, 1133, 931, 715, 
       1134, 932, 716, 1135, 933, 717}], 
      Line3DBox[{719, 1014, 1221, 718, 1015, 1222, 720, 504, 721, 1136, 934, 
       722, 1137, 935, 723, 1138, 936, 724, 1139, 1016, 1223, 725, 1017, 1224,
        726, 1018, 1225, 727, 1140, 937, 728, 1141, 938, 729, 1142, 939, 730, 
       1143, 940, 731, 1144, 941, 732}], 
      Line3DBox[{734, 1019, 1226, 733, 1020, 1227, 735, 1021, 1228, 736, 520, 
       737, 1145, 942, 738, 1146, 943, 739, 1147, 1022, 1229, 740, 1023, 1230,
        741, 1024, 1231, 742, 1025, 1232, 743, 1148, 944, 744, 1149, 945, 745,
        1150, 946, 746, 1151, 947, 747}], 
      Line3DBox[{749, 1026, 1233, 748, 1027, 1234, 750, 1028, 1235, 751, 1029,
        1236, 752, 536, 753, 1152, 948, 754, 1153, 1030, 1237, 755, 1031, 
       1238, 756, 1032, 1239, 757, 1033, 1240, 758, 1034, 1241, 759, 1154, 
       949, 760, 1155, 950, 761, 1156, 951, 762}], 
      Line3DBox[{764, 1035, 1242, 763, 1036, 1243, 765, 1037, 1244, 766, 1038,
        1245, 767, 1039, 1246, 768, 552, 769, 1157, 1040, 1247, 770, 1041, 
       1248, 771, 1042, 1249, 772, 1043, 1250, 773, 1044, 1251, 774, 1045, 
       1252, 775, 1158, 952, 776, 1159, 953, 777}], 
      Line3DBox[{781, 1160, 954, 779, 1161, 955, 783, 1162, 956, 785, 1163, 
       957, 787, 1164, 958, 789, 1165, 959, 791, 1166, 569, 793, 1167, 960, 
       795, 1168, 961, 797, 1169, 962, 799, 1170, 963, 801, 1171, 964, 803, 
       1172, 965, 805, 1174, 967, 807}], 
      Line3DBox[{806, 966, 1173, 804, 1265, 1057, 802, 1264, 1056, 800, 1263, 
       1055, 798, 1262, 1054, 796, 1261, 1053, 794, 1260, 1052, 792, 1259, 
       568, 790, 1258, 1051, 788, 1257, 1050, 786, 1256, 1049, 784, 1255, 
       1048, 782, 1254, 1047, 778, 1253, 1046, 780}], 
      Line3DBox[{809, 1058, 1266, 808, 1175, 968, 810, 1176, 969, 811, 1177, 
       970, 812, 1178, 971, 813, 1179, 972, 814, 1180, 1059, 1267, 815, 585, 
       816, 1181, 973, 817, 1182, 974, 818, 1183, 975, 819, 1184, 976, 820, 
       1185, 977, 821, 1186, 978, 822}], 
      Line3DBox[{824, 1060, 1268, 823, 1061, 1269, 825, 1187, 979, 826, 1188, 
       980, 827, 1189, 981, 828, 1190, 982, 829, 1191, 1062, 1270, 830, 1063, 
       1271, 831, 601, 832, 1192, 983, 833, 1193, 984, 834, 1194, 985, 835, 
       1195, 986, 836, 1196, 987, 837}], 
      Line3DBox[{839, 1064, 1272, 838, 1065, 1273, 840, 1066, 1274, 841, 1197,
        988, 842, 1198, 989, 843, 1199, 990, 844, 1200, 1067, 1275, 845, 1068,
        1276, 846, 1069, 1277, 847, 617, 848, 1201, 991, 849, 1202, 992, 850, 
       1203, 993, 851, 1204, 994, 852}], 
      Line3DBox[{854, 1070, 1278, 853, 1071, 1279, 855, 1072, 1280, 856, 1073,
        1281, 857, 1205, 995, 858, 1206, 996, 859, 1207, 1074, 1282, 860, 
       1075, 1283, 861, 1076, 1284, 862, 1077, 1285, 863, 633, 864, 1208, 997,
        865, 1209, 998, 866, 1210, 999, 867}], 
      Line3DBox[{869, 1078, 1286, 868, 1079, 1287, 870, 1080, 1288, 871, 1081,
        1289, 872, 1082, 1290, 873, 1211, 1000, 874, 1212, 1083, 1291, 875, 
       1084, 1292, 876, 1085, 1293, 877, 1086, 1294, 878, 1087, 1295, 879, 
       649, 880, 1213, 1001, 881, 1214, 1002, 882}], 
      Line3DBox[{884, 1088, 1296, 883, 1089, 1297, 885, 1090, 1298, 886, 1091,
        1299, 887, 1092, 1300, 888, 1093, 1301, 889, 1215, 1094, 1302, 890, 
       1095, 1303, 891, 1096, 1304, 892, 1097, 1305, 893, 1098, 1306, 894, 
       1099, 1307, 895, 665, 896, 1216, 1003, 897}], 
      Line3DBox[{911, 1009, 685, 910, 1321, 1111, 909, 1320, 1110, 908, 1319, 
       1109, 907, 1318, 1108, 906, 1317, 1107, 905, 1316, 1106, 904, 1315, 
       1314, 1105, 903, 1313, 1104, 902, 1312, 1103, 901, 1311, 1102, 900, 
       1310, 1101, 899, 1309, 1100, 898, 1112, 1308, 1007, 1008}]}, {
      Line3DBox[{251, 474, 1113, 252, 488, 280, 1222, 503, 295, 1227, 518, 
       310, 1234, 533, 325, 1243, 548, 340, 1254, 563, 1161, 355, 578, 1175, 
       370, 1269, 593, 385, 1273, 608, 400, 1279, 623, 415, 1287, 638, 430, 
       1297, 653, 445, 1309, 668, 460}], 
      Line3DBox[{253, 475, 1114, 254, 489, 1125, 281, 504, 296, 1228, 519, 
       311, 1235, 534, 326, 1244, 549, 341, 1255, 564, 1162, 356, 579, 1176, 
       371, 594, 1187, 386, 1274, 609, 401, 1280, 624, 416, 1288, 639, 431, 
       1298, 654, 446, 1310, 669, 461}], 
      Line3DBox[{255, 476, 1115, 256, 490, 1126, 282, 505, 1136, 297, 520, 
       312, 1236, 535, 327, 1245, 550, 342, 1256, 565, 1163, 357, 580, 1177, 
       372, 595, 1188, 387, 610, 1197, 402, 1281, 625, 417, 1289, 640, 432, 
       1299, 655, 447, 1311, 670, 462}], 
      Line3DBox[{257, 477, 1116, 258, 491, 1127, 283, 506, 1137, 298, 521, 
       1145, 313, 536, 328, 1246, 551, 343, 1257, 566, 1164, 358, 581, 1178, 
       373, 596, 1189, 388, 611, 1198, 403, 626, 1205, 418, 1290, 641, 433, 
       1300, 656, 448, 1312, 671, 463}], 
      Line3DBox[{259, 478, 1117, 260, 492, 1128, 284, 507, 1138, 299, 522, 
       1146, 314, 537, 1152, 329, 552, 344, 1258, 567, 1165, 359, 582, 1179, 
       374, 597, 1190, 389, 612, 1199, 404, 627, 1206, 419, 642, 1211, 434, 
       1301, 657, 449, 1313, 672, 464}], 
      Line3DBox[{261, 479, 1118, 263, 493, 1129, 285, 508, 1139, 300, 523, 
       1147, 315, 538, 1153, 330, 553, 1157, 345, 568, 1166, 360, 583, 1180, 
       375, 598, 1191, 390, 613, 1200, 405, 628, 1207, 420, 643, 1212, 435, 
       658, 1215, 450, 1314, 673, 465}], 
      Line3DBox[{265, 481, 1119, 266, 1220, 495, 287, 1224, 510, 302, 1230, 
       525, 317, 1238, 540, 332, 1248, 555, 347, 1260, 570, 1167, 362, 585, 
       377, 1271, 600, 392, 1276, 615, 407, 1283, 630, 422, 1292, 645, 437, 
       1303, 660, 452, 1316, 675, 467}], 
      Line3DBox[{267, 482, 1120, 268, 496, 1130, 288, 1225, 511, 303, 1231, 
       526, 318, 1239, 541, 333, 1249, 556, 348, 1261, 571, 1168, 363, 586, 
       1181, 378, 601, 393, 1277, 616, 408, 1284, 631, 423, 1293, 646, 438, 
       1304, 661, 453, 1317, 676, 468}], 
      Line3DBox[{269, 483, 1121, 270, 497, 1131, 289, 512, 1140, 304, 1232, 
       527, 319, 1240, 542, 334, 1250, 557, 349, 1262, 572, 1169, 364, 587, 
       1182, 379, 602, 1192, 394, 617, 409, 1285, 632, 424, 1294, 647, 439, 
       1305, 662, 454, 1318, 677, 469}], 
      Line3DBox[{271, 484, 1122, 272, 498, 1132, 290, 513, 1141, 305, 528, 
       1148, 320, 1241, 543, 335, 1251, 558, 350, 1263, 573, 1170, 365, 588, 
       1183, 380, 603, 1193, 395, 618, 1201, 410, 633, 425, 1295, 648, 440, 
       1306, 663, 455, 1319, 678, 470}], 
      Line3DBox[{273, 485, 1123, 274, 499, 1133, 291, 514, 1142, 306, 529, 
       1149, 321, 544, 1154, 336, 1252, 559, 351, 1264, 574, 1171, 366, 589, 
       1184, 381, 604, 1194, 396, 619, 1202, 411, 634, 1208, 426, 649, 441, 
       1307, 664, 456, 1320, 679, 471}], 
      Line3DBox[{275, 486, 1124, 276, 500, 1134, 292, 515, 1143, 307, 530, 
       1150, 322, 545, 1155, 337, 560, 1158, 352, 1265, 575, 1172, 367, 590, 
       1185, 382, 605, 1195, 397, 620, 1203, 412, 635, 1209, 427, 650, 1213, 
       442, 665, 457, 1321, 680, 472}], 
      Line3DBox[{277, 682, 1322, 683, 278, 501, 1135, 293, 516, 1144, 308, 
       531, 1151, 323, 546, 1156, 338, 561, 1159, 353, 576, 1173, 1174, 368, 
       591, 1186, 383, 606, 1196, 398, 621, 1204, 413, 636, 1210, 428, 651, 
       1214, 443, 666, 1216, 458, 685, 686, 687}], 
      Line3DBox[{459, 667, 684, 1308, 444, 652, 1296, 429, 637, 1286, 414, 
       622, 1278, 399, 607, 1272, 384, 592, 1268, 369, 577, 1266, 354, 1160, 
       562, 1253, 339, 547, 1242, 324, 532, 1233, 309, 517, 1226, 294, 502, 
       1221, 279, 487, 1218, 250, 473, 681, 688}], 
      Line3DBox[{466, 674, 1315, 451, 659, 1302, 436, 644, 1291, 421, 629, 
       1282, 406, 614, 1275, 391, 599, 1270, 376, 584, 1267, 361, 569, 1259, 
       346, 554, 1247, 331, 539, 1237, 316, 524, 1229, 301, 509, 1223, 286, 
       494, 1219, 264, 480, 1217, 262}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0e3kw19/3vyUURREtRNZIJSmp5FJkTwllLZW97JJ9Ddn3fd9F2UmWY6mU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    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.575544852973173*^9, 3.5865814785408587`*^9, 
  3.587380854832341*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"uxyi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.505411203510949*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nXk4lV309yXNszR5okmRVEiTyknKmKIJDcaQODKGInNmccyzEklzpkq5
m0cqiUKDBopUmtVTeZ/3fe/v2r/Ldf2ef87Vx3as83Hb973XXms/k6x3rLEV
FxMTU+wjJtb7v9f9pkfKw5XcNfAqJtbRb+yBjgvbZ2mkTdU7LQAPknuyoESn
H3f0bUO3nN5l4m7pnuILq0dzcuek5pQL7xB39JJ4b283kbt7Tb5YTu8B8fhJ
p3Ubz8hxkbOuVQmb6onrK5U5nhymwGmnWLeWCZuI/6k47B19RZHrJfZ//2sm
HvDq0l/JSzM5i5K70XJ6L4l3v+wu6kpX5i5syxqnJ3pNPPCv5Fxvf1XuHxmn
AmFTK/Gq6zc02+yWcD416nNEcm3E8+bckpp4yIh7FDrgYpnwHXHLhnMqt39v
5OaqP17ZVPae+FGPwLi+V124+A8FDd3dH4n371ITquyN4PBveB2sVq56T3sa
h3/jtf7Ukut7AozIM7jr5B0f9OMsyTP4LnXdZumf28gz+O27P6e/O+ZCnsF3
zLAYcuK4B3kG/9ldImb+1Js8g7t96paYbOZHnsH/lkxvvJwZQJ7B7RtOTVHf
FkSewUcbvjZ7cyqEPIO/eXbasnjvXvIMLvnDcOQh273kmeJRqJ/0YXQIeQaX
uJTWq+/LWPIMfx1WGpnhXWrkGVwyr/pT7mRL8oxX9atTLxgou5Bn8IUvKu+P
M/cmz+CZ+zS6dmkEkGfwGzahc+ylQ8gz+Nbw1SF7pcLJM3hZt0XxmQXR5Bl8
X//zbwad20eewa8rbj13WRhPnsG5wUfXTpNOIM/gfsEuQ089TCDP4F71lYMa
r4rIM7hjfc28r+bR5BncpnW6pWKvGPIMT0v7Dh7j5LeYPIMPW95YeHypPXkG
P5fhsXXeai/yjNeOTTK5b/wDyTO47PL2hb+XhpFn8NoBWmE6p6LJM3ir0T2N
K1Lx5Bnc8a+meEx1AnkGP/TTyMajXzJ5Bp+Q9+yYZnMKeQZ/NXRo3h/TNPIM
Ptezs05eNY08g3v5W527dDaZPIMfC863fHwkljyDtwTtzZ+kGEae4UNX2dXj
m+NS8gw+dsgMcdVlQvIMPi+/0WGMgR95Bpd4vl39dkcoecZrwVj5AMk+MeQZ
/Lb5fiP7+njyDK5uqCm+cHQSeQZ3Du39pnx6KnkG90ofP1wjKZ080/t0Pw9x
25pJnsEfrRH0KzHOIs/gCs7RZq1qmeQZPPtPnvRvr1TyDP7C/e+OX/XMM/iB
mKs/J98KJM/43I9Ve1sbdTPP4DXqJWIru1zIM/gJhe/5R7sCyDN4u/uUW2rG
EeSZxo9b9tP5RBx5xuvNz06ztjgnkmfwPcHurTE/U8gzuNM/vfqfmp5BnsGn
F11/fy8uizyDT+h1dUrlpBzyDL6t44RV2rYc8gyeZGx8WtsjizyDt36zja3i
mGdwJfnMJNPXMeQZXGFc8dFdNbvIMz7fUgOXm+9lmWfwiZOkJe5ku5Fn8FLd
OuvY1CDyDO6l1fK142AUeQZvm5g/49oUEXkGP/ukJqNUJpk84/WwVvn8iotp
5Jm+vm+FycqmTPJMcS7ueusyMoc8g1dUL7b5NSmXPIO/WtalktnGPIPvbb7m
ec+YeQbPrztiMlWceQb32Hf6uJwBm5/B7cPiAmtNPcgzPodRU0BR6WA2P4M3
+oRwZ8LcyTO465F/nir6BJNn8JBPtiGKjdHkGVz3R/UhrpF5Bu8z7qia8DXz
DG4YkTo90zOdPOM1c8n9gYPNssgzPY9wlcOb9Jhn8C+DGkdY9GOewau/Ohqu
VWKewd1Tfh2+J8/mDfCy05Hmy+LZ/Ax+W3uk2Hm1SPIMXu8dNO3QYWfyjHjF
nDW3uFjPI8/gr2/4JK8tYJ7B69rFpLfGMc/g8Ru2jOh4zTyD++zMv2N1j3mm
949TXRp6kXkGXzVrSGONLPMMHjj0xEG3UnY941W6/vqFQRuyyTP4qVXj3L3L
s8kz+IOZy84edmXzM3i1AZc4dUE6eQYPlpS1/JSXSJ7BlXWPlIy7GEae6fOm
XRu19oAteUZcLrErjDobZ5Nn8LcbJ3A7f7N5A7z/AjvHj81s3gA/LT+h6EcV
mzfAh/VRcjQewDyDz71gnXzwQhJ5Bl/vptLpWp1KnsEzI4sU5vVj8zP4yPKo
KfVGmeQZry/Hl/9UNGX3QXDRyC0LPO+kk2fwoh1HjDdEpJBn8LELVFxb1dhz
HbjEzLp3UtXs+Rl8nX9HL704M/KMn39Ra7xfl+9k8gx+tdPf7uNOV/IMbm/r
0vXSJJA8g9ulyFxe78fug+AuasknFPzZfRA8L1rjkV99AnkGP/21MGlCaDJ5
Bq/2S9OX3syeN8A/qd29UrEmjTyDZycM2xk0PI080+c6mjSkyjiFPIOXK/S6
fXJCInkGT5887VOUZSx5BpfISq/cMTiQPIOr/qrceMhShTzj5yyNjMkot+1P
nsHzI/8qPPvKnuvADWLKDdvK2XMd+J6dB/U77rLnOvAXbh6fbGZFk2fw1xoD
Cg+9iyPP4MGWpY+spySQZ/DQwq2rchISyTP4kez8J+aaSeQZXBBRpbClLpE8
gxf9OK69fBhbp+C1srxP2O9xceQZPLNMdn9naBh5pvun5kOuYrMPeQa37Sz1
z7PZQOtwvJ/YmfOTl+eI07oRXMXjkNmCdjvyDN697ErQwpKd5Bn8bOnYyH6H
Asgz+LVTF3zeBISSZ3Dv+oG5pvciyDP4gkiv+MQJMeQZfEqZ7+OQafvIM/gD
cdmE4N9sPQjuz4lN5vrvI8/g/U0Udq2sjiLP4G1Bvt/u3Wbrbrzqt6uHt1f6
k2fwjovn5hxN2EGewSe62L3oPOFAnvF9S0dua3xaJ0uewfUGlD33fmFGnsHv
PC45sTDegTzTejTHTGHCdTfyDD7qvG/Ycn1v8gwu8eTvT7/FfuQZ/N9VRdem
RASQZ/Dszj6aqiWB5Bn8j2HA9YuqQeQZ/IrLxRka2YHkGVz99tQxRRL+5Bn8
Umycle5Wb/IM3ih5XvLSkB3kGa/6xXb5coGm5Bl8vsWO40GebiyPxn/dpehh
94gLiuQZPDrHYNWcOwvJM3hJXu/vh+/rkmfwsV/kTP+5toY8g2d1zHjw7Kcp
eab7c438yOnHzMkzuOTt3VdyGqzIM/iL6iTVGBcb8gzeJ1BcMtbChjyDx3/9
fEb4xZI8g58rN88K7LeePIOv67vi0vWJknRdgi+o3TTE67Y5eQTvHJH2TTTc
kf08/rVCQdJVtbevoOVs36ppegfotZkTL/6UwuZt8KtSRiPuKo/mbBIFCcKm
DzTeYsE/Cb8TAuj3CL71xx1r8YgYDv/G94m/CxLqdSym9wePvlX7UueAHI3D
q/5m6ab3A83o3/g5vxWlAv7tcqGfh/fr2++pQvLEH5UYD+7Xvrr30xleFA/e
52b7TBPlz3IUP74vTi3w7MSTHoLG+ykq0/VE9Fox885nuw+juV2a7mVlwo/E
g+eUpv3RCubwb3z98WPH+uMJI2k8XieNk+33YPkOig/fJxbq0uvbjz40Hl9P
nhWxX/xiAL0/4hXLznktcfgM/Z3g+3rPtpm9wMuL4sP7rFuVui3za2slxuPr
XnN8NOXbo+j3gvdZbCs+4maYPL0vvk9msbz65XmegsG7dgzuJXaS/i4fnwxc
J5VgyK1SG+QuJnaRfq7dmS+6L8SliGO87MR5Y+K6NnPJR55ZiIndpPGzT6jo
/RgsQxzjd9Ykt1TV2HI7RphNExO7z36/K5uk9++bTBzj+3TIpHsrCrmR1ZfH
i4k9pPFJ+bVJ239OJY7xYTWCHLVZbtx/g0aKiT2m8fOGf3E+dF6BOMbH5ard
3njOk9u0PHmAmNgTGt9x0UlvyZQZXPf/++858T18Ph/j8T7TzDUC6518aDzl
sfm89IOnav92d7+g96mXiL5WZKNMHOM9/NRyzpsHcAcrHL53d7+i8TnzY9eY
9lUljvEr76823vwtkNuZlv2pu7uFxh/6t/dTWaOFxDG+2CreVrs+mNP1qu3o
7n5D47WnLrJtFNMljvF/kgy2GGaGctLr+7/t7mb7Dhdr8p4LwtYRx/gf/Y5d
WbVrL9ehuuRVdzfbj7h6p1X6XoA1cYwX9u6s76oJ/W/V7Pasu5vtU0hKzp35
e4sbcYz3zrgrY+wSyuG6xX3D5f6Aji2vN9P1Sc/7w5QejzN2ousQ3HSnbUTH
Sk+63sDni2U3hwl96boCnzHBJ/bLyUC6fihv31He7+PYULoewFemqNwenBZO
1wN4MZ8/x+8d3C1MOfCF3j76/YJP3X85PGFAPP0ewcXGqvwK8BHR7wtcRnfo
KUevBPq9UP7/pVB40TmB/NN+wqbDw4Jy4skzuMPj3/+qu0eTZ8o/z/gnJPb2
VvIMbnjBLXePjQd5BjcQ3n+flLWHPINL3O/93tUyhDxTfj723c6xBhHkGfze
x4blJsNiyTN4UsESryeO8eQZ3IHPn8MzuIb0bK/5F5PIM/jFBYs2Dd2eQp7B
vbXHy5ffSSXP4IOX+Z8XrUgjz+DDG9Kjeq1IJc/gY+4/NalqTiTP4Jl5mhH7
K2LIM55rutQihAYy28kzeNSWw/dScn3IM/jm25tuSqoGk2fwh//qbvngHUGe
wf12xg/+GbyPPIPbbhxwYVOBiDyDCxIbnhV6J5FncCc+fw7P9HO9fupX66ST
Z/D+007Y53dmkGfw3QmbdlZ0ZZJn8IBmW2XDK5nkmfxofQscejedPIOfr/pl
9UQtmTyDy8+fHyiYyzzjOTE0wCDEX92ZPIOfM5D9OarYjzyDF3t9ruilvJc8
gx+ord0ywDSGPIOPmaFq2H+MiDyDf0123jzdOIk8g++al68vskglz7QvyufP
4RncqGvIiA6tLPIMbnlkeXLQhWzyTHlvDy/FfqtyyDO4WI7YQBfnbPIMvmBY
w+vUkxnkGXzLH3+/rSuZZ/B3tcUzWx5GkWc8dytmn9kojHIhz7TfqJ6nuFw7
gDyDC1dM2KO3NJw8gx+4PKZg0fd95Bl8w5vS9f77E8gzuPu8O+JefVPIM+Vp
hhzeNEQunTyDL+fz5/AMHtJ6aYNTVTZ5Bs/VGnev9kcOeQYfYv2wpLJPLnkG
Lx/xvLY0i3mmfYHxvkUf1zLPtC5UTUkRi0oiz+DDtb8ZGfaLJM9Yx2zoeCvZ
tMqVPIPfX31tyaNBgeQZfPGYovzb6yPIM7iEt5hne14ceQb/oLX/vKV5InkG
HzX3zaczN5ln8Dp/burAB8wzeDqfP4dn8NkfxZQvDcohz+DXL303D33KPIP/
6HVjy6JAdj2DT9Eu3j2iKIs8g5/ef/LYxmFs3gDvcG7XjpjD5mfwqk/Lz7q+
CiPPWBdeHWErscqceQYf37Z676hJzDP49OPpUgttmWfw5uGK7e9ymWdwS8uN
Y+7pMc/gSxYcthubyjyDr91u9SXflXkG9+fz5/AMPu1pXnrXJHY9g0edvLpC
JofNG+DyjZ81Hr3LIs+UVz8vdujNwwzyDG5SL95LNJHdB8E/fz+i0eUmIs/g
AwdfHT3dkz3XYZ0tXRr16PJ5Nm+A2yS5bFsWzuYN8JrqNJGHB5s3wFtTFy2t
/sjmDfDxkZYZnU5s3gD33V7fkRKYTJ7BtyQt9SqSSiPP4Bl8/hyewYPjZ8sd
kM8kz+DOWkpX12/IJM/gIx49s1xomUGeqT5IU+KarSJ73gB/nO635WhEEnkG
r9hn4bZaM448g3sstHYrTgsiz8hbNMtsfD2+lN0HwQ0SdVfFWe0hz+D5B33L
9Hew+yD4JLH8uNol7D4IHrTeM/stF0+ewQumNWhPC0okz+AP++h0/OWSyTN4
FZ8/h2fwwdmHEt/MSSPP4F8W+d8aPimNPIOb2FzxMPqZQp5pH2H7HcfKM0nk
Gfzrc6+KtytE5BlcSzP4dcigKPJMPocvfxLS5UeekQcaHXBU4LmMPdeBr9zt
wEX230WewaPVFgrvzWHPdeDHPQbLFv4NJ8/g38s+qp03jCXPlIfX+eqe7BNP
nsGD9KLsthsnkGfwED5/Ds/grf5P2wtmsOdn8PS7l5O2dCaSZ3CtobsyYs4m
kGfwcpNGu9zEePIMrlyk2lL6OZo8gyvG13QOuh9CnsHPD3gyo98eT/KMvJqR
vNXR/b3ZOoXy3pY6M91eu5Nn2sceX/jHfJ0feQavnPMu8MfIYPIMHpYj/dZY
Now8g9v3tsu0z4skz+Als03T3SxjyDP4ZD5/Ds/gixtuVVx5wdaD4OuG62qc
XLGPPIPP3zPOJ6V3DHkG/+P0YP7QlRHkGfz6067PMaoh5Bm8NSzPKf/SbvIM
PiBjdPwCx23kGXnKX14D+2fbbyDPlE+2qbSQdrAjz+CuHdP1h4a5kGfw5zPf
PVIJ9yTP4EPChZLSU3eRZ3D/levMO1X2kGfw2VdGJYUeDyDP4Fl8/hyewY3F
a759mBxEnsElt7k8ML/C8irgfj/zvKpWBpBn8GClTYt9SnaTZ/DbmX4umiM8
yTPVY74+a+25cTt5BrfaJOHu56RLnpGvE/fMDj36fS55Bj8za+XLlroV5Bk8
9/r8b0bWRuQZvGGHwii/5+vJM/hW68giaZlN5Bl86OV/Th7/YEGewaMGWrj6
DbYmz+DNfP4cnqnOtF3q+zErG/IMfvLRvHuesVbkGdxg+whDrSYz8gzua6+v
4tmiRZ7Bjy+TrdkWpCuAZ/CDxdusRxhZU56Tni9enrc4/0WffG6M0uw+Izwi
+OQ49m53XzPymTH5YMuOpgLByYEOH5QvWpDPJ3we2v+zlYOsihX5VD/wYGuF
MENQv/7Z/UVrLMnnOT5P+33k3ch62U3k0/Pv2d8z9YIEAyNOW3xNMyaf5ny+
y7JH/lCcjz+28sbw0cp+5HM5H/++a/PGXnm2m3wG8/HPeZDRy9xwF/m8ysff
Nf5SUburF/kcyMc/v+2s8UY7d7bu4+M/ttD369m5znR97ol1OyuSuyRQPvf5
VUkSy7/Rdbo5wCPQdDv5XFyiY/ak7IJg3hSl2c/Wu5HPp93eVeXCCsG8bKOT
l3x3kk8/gyJBgtxZwWClZsuAAB/yKZvSdNq5qUxwVm52tHbMbvJZ+XLwNH1R
iaDeYqew+Z4v+SxAvqtH/s2bjz9yr1TU5tQY8qnLx7/52MS7Dw1iyacyH7/d
pZ+l127Eks8aPv6uw9YVB2fGkk9XPv5/OjTya+XZfU2Sj//lh1mbf/23vsA8
UMLHPy039/BcNfY8/M3iWIec3m3B6G+zv+86yu5r1/toF4jkbglmTY9vO+/P
7mt43fVE8uOfT7vJs52x1zjnpuuCTY1y8RJnAsnzvB/DapvKrgmcljQfC2sL
Ic99swqj9URXBcuKB/WdnR1GnuuXaWqXC68I2kdv+GraJ5I8y/J5Lbse+bdO
Pn6HxvHWIway54eLfPxbd5tzbs+TyHMcH/8as7tRNyyTybMlH/+3pXel5Kcl
k2dlPv6uF+2rUtoTybMYH//amleJsm0sz1nDxy/mVPja7AzLV5SP/bqvqaxG
sK9J0+3FIPacluN40UkkVyPYvtx/q32zN3kOq4zW0xPdFyQWmqY8lgwiz5S3
UJ883G9ZGHk+flO/r3PTXUGB65GJjm1R5HmP8yuuqaxa8O9OvZED+seR51VS
vj56oiqB+6O6WqsfLM95h89rOfTIv5Xw8X/Wtvqgc549D2fy8dcs8f2tWp1O
nkP4+D8Mr5g6XYetO5z4+IM1DP9+up5Ongv5+LfcLN5h0sHWd958/DIfbqYp
t7J1tB4f/6+tt4oDh7H8m3tg/GE5vXqBa+ezo6/nsnXHpodb9jaV1QkOZ89b
qCfD8m9a8oo2Irk6gaP8Pqs6o1DyPGPXd4Ge6KFgVoTX6If5UeSZ1nmSnme7
zsWR538n7PtZJqwVfH0z6ruyRQJ5fuW2qV7Y9EDwR/uneOFjlucMQ91pj/yb
Cx9/7IoPHR2v2PrOlI/fTb5RNvtvFnleysfv8eXF3YE62eRZgY9/3pqbm/9Y
s3zFcHyOXubHfr9l+YouPv5pwcenhY9m+bdmPv6Bj25VqR5mefvZ7bOuCJsa
BTfbZFs+xLJ19Jgl/+bI6TUKXvrN0/cYwdbR3ftu+DaVNQgc0wwerTsYRp7f
vEwwE8k1CJ40zouzqIolz/fmWs7TEz0W1Igv2HnrDMtz4vXvXu7k18ssz5nT
1PWxTPhI8PVqTMm1JJbn1OLzWlo98m9KfPz9TBs6FiSzfIUUH/+k5yvjZqSw
vNBvPv6FRQc7CrNZXug1H79P6zlHtS0s/1bFxz9Ra63yLgeWfyvh47e/fUBM
YxzznMnHX1vUq2KjkM0b79ZUPSkTPhdUHSk1X2LC8kK1+alnhU3PBGckTPee
/cA8V3RtTZbTeyaYe/rej329WF4oz0DFvansqSBw5cQRGRtYXigq+89qkdxT
wecL/WZLdLB8hfunW0p6oieCJ2FaghNzWF4Iry3W0muuzWF5Ibz2zL+95ePf
mrZv4YDfzPN9Pv6vE/vUtd9jns/w8RuOrGmcncQ85/LxzxL+fF/an3kO5+O3
ODEsJv0Wu55d+Pgn6V0JnKPL8pymfPw2xsPXDCmPIM+1fLzZcd2xBxyYZ51+
GQ5NZS8EmwSuM5bJsPxbhXKvx2XCF4KtZVezgjczz8ob7bVFci8EmfM79i7K
Zp4PBleXCJuaBS9OfrnYZsTyb2OPzZmiJ2oWbDCXMO9bzDxH16fFy+k1Cz6b
/duVfIF5NufzWgE98m94Hdh72w3Hef8jn8zH31wUJD3iCcu/nePjz2xRsop3
YvPGLD5+49xRrotPs/2RA3z8uS06qzZvSCPPo/n4pXf4/fn6OoHl7fn4awdy
Y+dnsDyniqh3uJzea8G2z99uHL3F5o18Pl7xV5t678li1/O4lntbm8peCZ7d
H7luaRnLv8UOnVdbJnwlmGH6/uyrJcyz+IJMTZHcK0G3y9FH596y63mnlfhJ
YdNLgUBP1emiFvPcHrlNVk/0UvCoodnm2SLm+fz/kn9T5uPvNflkscMWNj/j
VWVQhWLeI5Z/G8vHf+mwjfN3LbYPFc3Hf+Xuzj4Bfuw+KMbH7yqx/c37eSnk
2YOPP3ZOfH6KFMtzvuXj14iXOWAyiz3X7dN+IKUnahV4uIg9fHyP3Qd7uywI
ltNrFTx0dpH7fJnl37z4eA/UplqdEGfzc8dlCcumshaB7s38kIO2bH627Nh+
t0zYImj9cXrksjI2Pz8cVbNYJNcikDJQ2xD8kc3PuoL5R4RNrwVX3ff8q82x
+Vm6R/0qPMfy8R/iHIYbRLDnDXE+fr9MZ6uUZpZ/w2vtvMkFs9XSyHM7H7+i
7OxeCZ4p5Nmcj3/1ymXKnfbsue4BH/+u3S5pnqJ9bH+Ej99Du/Tbg+Usz/n+
V+5+YdNbQcOnx+sWb2LPdVZy/Ybrid4Kphe01RYdYvm3OkPhHjm9t4JRnffv
6E4OIc/6fLxT+txqeFIXSZ4rcxduaip7I9hkemnl40PseUP1ds6tMuEbQYbm
5jHlh9nzRsGXPgtEcm8EMx0vRn/UZnlObz6vtbdH/q2Djz9eeqt+2UH2/GzJ
xz/ra/S9kq/s+fkhH7+iqntqxeok8oxXndOG+q9LWZ3AeT7+M7t3Zr77HUee
lfn4lSc/chL7GkGeD/Lx73gV51owmeU5DU7smCWSeydwXblUUW4cW6dwj+sy
hU3tgkQt1TfXr7E6gTniiwfpidoFU85cabv4zZ88F8444COn1y6YLKXysu1Q
KHkez8er/rjjNKfPnp/j9jhvaCprE7g3zzU6uY49P0sUPrxaJmwTPPLXvm1+
SkSe6/m8lkKP/Js+H//qsum73sfFkedKPv49J9pscr7EkWdVPv6Tq3zqFAPi
yHMBH7/SQqHmUQ+2HqS6DJU1B3SkIslzLB//qFLJxSeygsmzOB//ijVXFk5N
9iDPh200KsqE7wUOUZn+RSKWf5OJOThdJPff78dfZaPPm20sL1Q2MFXY1CHo
WKw50EHgQZ77Nrv01RN1CK6PfFQ9vYrVvewa8MhDTq9DYG76Xf/tuyDy/IGP
N6vEzeGOH1sPWm/OM24qeycYZOKgo3A6ijyr8Xmtgh75t0I+/gFNnO7CmmDy
PJ6P36guJfTe3FC2HuTjHzrmd+35daHkWYKPf9lq7/VnuoPJszcff7777JGD
FQLJM177ZwQpLzPxIc+WfPwr4w/1qu9tT577jh2i21T2UbDdbVy9d4I6eUY9
nRO35e2imNXk+cP2BjmR3EdB1LlnG8e/NSfP1nxd4uP010+SJRxYPvlCfi89
0QeB2imf4wVDWd2awZtBLnJ6HwSrsj+4ixS8yTPHxzvZfqSwn/we8hzH57v+
9si/SfDxG01f4VJa7UCevfn4Y0PnjVjb5kye3/Pxm0+yf9Ho70Kerfj4Tcsm
/p523pk81/Hxj9rtLr+1fBt51ufjf3OrYLjSXkvyeZXPd8UZrtetWyNN/qfx
PKb4w9zeTz2IT+S/z9NJvlJlXjC9z0X+6wWluRrP56+n8XjNex49cOliXeL9
eQ8y45K/3BHa0zjkCV+9XNz17qYT5QdRh7bs/MWPDWYS1O+Arwec2KIlHGZI
9ZbgBxzmKMzdPYXqmcF97XUSZBS3UD0zuPTiceuu/rKlembwqbF9NarNhFTP
DD7rt+yTt8puVM9MeYsBY6o9n3hSPTO4Qqn5o8o7PlTPDB7yW/Zj5Bw/qmcG
X3jKxeT0zACqZwZfFNVmMmJhENUzgzcc+ujh0xVMdePgyVOOmLVIhFDdOPiJ
XXnXjXKDyCf+noa/0Bo/Yt1m8gn++UTopqXBquQT/IvzOpMQJyfyCa6unHqm
zNaTfIKPN3iiMSfCl3yCa3Wu1s07HEg+wfW11+QNnxFKPsG//5G/cbwunHyC
F7mbHDz7OYp8ggeVBeWerowln+CqCX8ON7TFkU+aVyrb5206GU8+wSueL3A0
bY4jn+Ad7aJTn1eyvj/wQU9Dda18Ysgz5qcTYXNDHJK2kmfw3Ld/byYfYXX4
4G0Dy22z8zzIM/gmHad5M3r7k2dwp2a9S4NyQ8gzeOy7Jvtib9ZXAr68a2JA
gVUseQb/qjHoivSRePIMfuBp6M/I8gTyDP6nNE5yjWcSeQYftUqvxakX65MC
37drosE/s1k/GvjogNeP25ckkWfwksnf74vfY/3CtK91M/jd1Y0R5Bnzvagw
aL2L5nbyDG4/54f/kSIN8gzulSv/ZprhLvJMXGb5e9OyYPIMbnW8/5Z+UyLJ
M3iD66TI0DGsTwp82OrwiXtGsT4pWjdcOuBx4N8k8gw+9HbZQrNVqeQZXMnY
YtzGyenkmfZDTk6rM12fQZ7Bj7tWjgkrZv2VVDdXGdT+LDWFPIPf+Nd4m8Ml
1v8OviI8f/UBnxDyjPunvtRLzdTeO8gzuMansJj5eUvJM/jVFftlXQb4k2fw
htGds91Uw8gzeJH4wDuVBbHkGVy3rDrY5p6IPIMnTHr4yHsG6/ujOuIs67Wl
emnkGdzMcvGVxOIM8gz+0d+z2dkxizyDC6OnD9CXzibP4C5a9buKjrJ+YfCz
5wZrt4tYXza4TMq4yj+7mWfyJvR98lsugDzjeWRrScyld1au5Bn8s3d2xfwb
zDP4yc6Zo/ItWX8leNWbkQ8nt7L+SnC/psbQkMHx5Bl8+czrenMa2TkD4OHu
V9XNHFkfK7j1ggdNY5JZHyt4wdAEJe9P7JwB8BJ71yqD7eycAfB+4pdHNYaz
vmxwY89ZrwodWf87eL2w/KJ1HPMMLrj0ImPQHjZvgFd4tL8wmuNDnvF8N98p
rOJthRtxPN818P3v8I/x43OkHpaVC4hj/Fq+jx6/F4wfPEr28/6SIOIY79Kj
jx7jF5jVFSTeiSKO8cE9+usxvmSR4u6D6iLiGN+z7x7jl8m9j42el0wc4yV6
9ONj/BanL+tNHqURx/ieffoYP7ylzFDiXSZxjO/Zv4/xEYM/zdSQziGO8T37
+jE+yHP4zwLZXOIY37PfH+MnnbUb9aWGXW8Yf7fHOQAYv9Vrpe8+Jda3jvEe
Pc4HoPzNqbCu/g/ZfQ3jy3ucG4DxlgXPUk36RxOn/aUe5wlg/MDs68Frz7gT
x/gn/DkDuG6x/pgim7Eo4AC7bsGPbWrre3bMfLoOKQ+9bOtn3xh2HYJ/dKh8
nvCKXW/gCXsVw6rvs+uK3ufSJQ3xa+z6AT+e9nfKkensOgG3VXOvabnFzhkA
X1E7tTDUg50zAP74kNJ0q5fsnAHw03Y33v0tZL8vcK1Su0fSO9l9kOq4C975
Z7xg5wyA99Ed+/fY0HDyTH1Cc5+mLLliR56xnpP//u3r+2nMM/gHl+1zyyJV
yDP4+deDU6oGM8/UV1T84cpXSeaZuJV+cJ068ww+6XT9xc1/2TkD4OusCttv
TWLnk4AnHzmic82Nzc/g9XukI6c8YOcMgJscHmb69yM7Z4D6NjYmm4p7sucN
8BI7l8kD56aSZ+LOHUqH77NzBsBrigfK1keEkmdwHaV1Ky3/bCHPWB8nnKmQ
Fx/gRp7BqzgDp7kB8uQZ3DYxOk/nHbsP0r7rTFkf1U2R5Bn8mPWHbz8k2X0Q
vEm2w1wpht0HwWumZHppWLLzdsCzXj6/sHwze94Av7ii9FAfN3auEbi0vfMh
KwP2XAe+zj3XzzEnlTyDr+/94skq+yTyDP7E4u5U8TVsnQLeLyHuxDGPIPIM
fm9I9RDHdF3yjHzDkAz/JScWuJBncLuCbhu75LHkGbz/6IJ778axc43Av/W1
6jupgD3XgbcIjQo3JrPnOvDrLXmcpBV7rgPPUjwxT/NhInkGVxrpX/XEK5k8
gz9oW2g0RD+FPIO7DXW+9/FzMnkGX7RmsmK4chJ5BtdwCS6sHy8iz9S35zRZ
T+NLJHkGP3EhJLv6nB95Bi9S6XvYZ9Ay6qND/mbpyyX/PjdyJM/Uz2Rjs+10
WGclPIN7fsqwChu9mzyDLxzqEvT9OVungPu171u+/lgEeQaXzBl2d83DWPIM
rnl9zIEBTuw8NHB/g2Qf32IReQYvn+L3bY4uWw+Cl/x8PuxvvYg8g096J9V7
wMR48kx9RfV1ByOORpNn8DEaGuZBh1geg/oJjqXsz7/mSZ7BZVbscJLZb0me
kQ8zM1WzaHBk625w1wTNftbWQ1g/Ic+3DhlgumYhW3eDx3kvGjfyqB95Bn/x
ZInS9+nB5Bnc/rfOfMneYeQZfOeiEPOunxHkGfy+RI5q560o8kz9gknfn0tG
RpNn8OdHXF7LHo8iz+AZv4KyY8+Fk2eqe/JYLZEvF0KeqU/udHD/tyd3k2fa
f3jjXdFSuZ08gx8KXi5WtseJPCOfp5qvYp7nbkLzA/jrjc63Uqbak0/qP0gY
JhvS25V8gg8KkX1/u/9O8kl5SoFpsEnkLvIJ/uX6pOTUIXvIJ3i/Fz7Wyv/N
S/AJvs50qaxtGDuXEvz81wmFU177k0/w6r5KL4yqdpNP8MDGP5+Or/Ikn+B7
amP9rQQO5BN8p8qhgjIzQ/IJHv5wjoqihLvAj6+Ho75w0911t0du4VB/Bn6t
UPPC1U5bDvVS4DWqq80UjIWcG18fAz65698+oplu3Cy+ngN82uiBny41eHLY
1wfP5esbsQ8NLpPd+qh6RgCHfVNwt/onrdNnBXHY56N49J891aoN5rAvBf53
8/xTsU3BHPZRwG+tst7bWyyIQ94fvEn2V37wqigO9XbIsy0YsuVcpqkHh/ow
8DPP7O43+e7mUM8Efnz5qzuR0wM51N/Q+Kg3V9XzQ7gC/vkDfBtffyjB76eC
n18+v/fB3DgO+5fgSzuGl13yjOew3wZe5tvy1ed1HIf9IfCkZ53vc5sj6bwC
8NbviUYZ1yM51Lch31WltmH3MulgDvVYlH/btL9GvW84h/oh8AjFukuN/z23
e/P3e3ATvq6vnt+nBO9/d/QK+bZkTo3fVwP/OfGNT1NeEifi94HAE/QyrENz
4jjsu4AvuCye2SkTwaE+DHmk+oBkYcvOGA71TODDHT5ZO5fHc9jfBQ/l69+w
H0nnEyzU7xpRl8ph/wy8dLOyT45kAof9HvCi/nOP+amHc6iXoroepfs3k6Yk
ctjXBB/B131hHw782bVipbstiRz2jcAVsiz35hmGc9ifQ57Bkq9fwn4S+PpZ
9s2bv4dz5j3yDHf5PENBj/zAdj4/INsjD+DH5wHu9Fjvr+DX+2E91vXi/Lpe
q8f6XZ9fv5v3WHdv49fd5/+X9bX0/7KO9u6xXvbk18v1PdbF5/h1sVqP9e9j
fv2L/Tasf9fz9UhxPda/g33+//rXh58/sV6TGnV2cu2GIO4TP3+CK0yqa3cv
j+JQrwnOTfNSH6sl4lBfSOvWP1NjqrSSOdTDgecNsloRWp7God4F/Nbv6utX
b2VzqM8AH+0Yp73yWhaHegKqz21+aHCkPp3D/jf41Dyz6cLEJA77teCvos6M
sneK5LC/CP5+eZTZHAkX7hI/T2I9tTN1zPUHFyI41KGCZxyPFqVPiedQNwke
4lj8aqR4Eoc6P/AfZ1WuztNP5VBfAr5l8uB+O69ncKiHAF8wIX6GY+80Dvv3
4AMvSKmXzf3v7w775TyXfPfB8ua7CM6Hnw/BR/w+FSl9xIUL5udDOlfDRafi
3qB9nICfD8Fvp5XPipmewKEeEfxlzgdPY/VkDvUZ1Aefa+MunZvMoZ4AvEzt
97ijxiIO+9/gWtF9jrqtDuewXwt+yMRRPVnHhZPn5z08/x8+e2/GttI47hU/
74Enfvcx6fRN4FCvAC4VEid9+OU+Dvvr4Mcnhb/1CAvlsB8MPrtSJ+u7vxN3
h5/f8Dy8WGXlQbUBcRz248F1SqIjYg8Hctg/Bq95s2xT5wg7DvvHeN4rvayQ
0/1mM4f9Y9yXf16PFEgFTuewHwwu5PeL8ZwDnrmqddy1qFgO+814/tEV1k6L
nv2jEs/n4HE/MxSvnZlI+8e0b7xx4KX9Nc6Cdr5uEu8vlvXY6NJNH86U78eg
54RZ+q/lW0051IOC38zJ1p6nHc6hfhE8VUI469jOKC6T79/AfU3jeNkk+f4W
HPoowFt3TFASS3XlUFcKnqSwYKPxf/f983wdIfivjBILg2EJnDRf9wYeYy+r
O+h5ItfI94fg/tWuvffyx22W3BO+zwE8/dUUi3JPDw59BeDOyWvKVG8HcKhn
BVdwbBMtyUnkUH8J7rJydoStSgqHekGqo/cd6/FlbhqH+jbwX79iG3+lpnPz
+X4V6r8pdy2dlWzOoW8EPFFymozKc08OfQ7go3ICx7/LD+JQlw9+wNwjJ35+
JIe6W7pfqztI7N2SwqFOFFy1OLrNUjmdQ10j+FfL3WM+6GVyqMMDV+w6NqbX
3iwOdWPg5pJ2/Y6YZ3Fn+H4b3K9HrmtSiPhrysnwfSPgzvc/qQT03cmhTwN8
88dTZ8/nBXPoK6A+cvljSreHRnOogwffMbB1v9ggEYd6Ynr/voVBf1TSONS/
gqe80hCbJJfJoV4TfLfdlylpS7I51BdSfdYI40Ola3M41MOBuziUtra+zOZQ
vwWe8fnQpsoxmZw7/3eK++/pLe72W2uMOPT5gFffeLpx8XRPDn0p4Le29Htz
qTmYQx8FeN3YxUu7W6M51P1TnWn3NaXGChGHOnVwvdORwtOFyRzqfYmXesyS
nZ7FoT4VPLV8dVyjag6Hekpw4y9HDAKG53Ko/wP3iunKtXbN4VCvBn5/17MH
F3wyOdRXgdvsK3dSyk/mJPh5B/ffZL4u+g0/L4E3LFlZ5yXM5HT5eQn3tXMj
0/Zfmynk9Pl5B/zvy/9f33uPn6/Aw+b0HiVTnc6hXhn8Y/RDrT8OmVwYP19R
32Rsr5dVCXacGj9f0f1u6QKN8CpfTsTPO+A5fF1rOT+PgQf4nFbI2JvKoR4a
XLIu97DD9/+uT34eA++YKPtov246d4Ofx3CfKll1/+Sx8RZcLT+Pgf+t/Gej
U4oXZ83PY+Cdwsj99gnBnCM/74BP4us/c/j5jfKEd/u6WUQncajPBhfE5iSc
v/vf9cPPb+DjLQx3DpZK4VD/SvfrFRbmr4Ylc0P5+Q33xxMbdRoKDhtz7vz8
Bu509fEf6WZXToWf38BTlk71nj/cnyvg5zfwsx2GvkWnQrlh/LxD92u+rjKM
n/coj1e57cSG6ngO9eXgzyMSF2c2iTjUQ4OPPaQz0V86gUP9LvhEO/u4x4tE
HOpNqc63WK0ofnYcF8PPe7jvp5t4+EZOXsChXw58fbXiia4Ee64XP++Bz2pU
6Dh9xpPz4uc98AKBhOv4If7cD37eA386MXbjkpvBXDE/74Dn8/WNO/j5ELx5
yqY/FcsjOdTTgytk/qlNaIviUP8Nbp6tvfDz1mgO9crUx6NrmyknH8WhvhZ8
9tr79kGtYRzqQcGFU/QGdPUP5tAfSOeiBU2wWadqyqGfDfxr9NLZW6PsOX1+
3gNXuPN8oeJcVw79QuDKNtMEtvN2cuhvAV904fu88NO7ODP0V/Jcia8/RP8A
eK62p0Ph2AAO9e7gn5q8/pHwCOBQnw2uVrG7QcHPn0M9MbjIudlF4p/dHOpf
wV2zv2Q6WnlwqNcED3M+Ff/9lj03np8PwW/w9YF4rsJzWn/tQ59vdG+kuk3w
y9Z3LsbV21DdJnj++w9DJittp7pN8Pt6elLKM12obhN8levX1MTF7lS3Cf7a
Ptfwg+NOqs8Ez+7RzwsuO8fx6YXX/lSfSfV7C43SKjMCqT4TfNJDo01eU1gd
LPjb+22PpDqCqT4TvHO2ocx812DyAE8pbW0/Xq5xJw/g81d73ZTz9CEPVHe3
3dhRztmfPFD+yido/IzCINbPiPq39WptySdC6fOC2/XotwX31R0aN8aR1VeD
x9216DdBh50jAZ4m8zZpu348fV7w5lvaz8R2sn4B8PaOCzsXWkXS54WP6ynj
306wDqLPS/VsigZ/5Yv20uelc8Jq3KZeqWfnP1DdWt6fzsfD2LkN4Bt69LfS
c37gub+9PrD+VnDHZ7fFvmkm0+cCL79vufPbVtbXA563TPTzzrNoih+fb8Fg
97b3+6MpfnAPYcR3nzesfh5cKnnin/de7BwP8L09+kPB9+jILz2xkvUfUV1/
TIMKt571eYEP4SJ9jk+IoXgQr8fGe6dmn2Z9E+D7HvZZ7BfP/IBL9eiXBJ99
edrcoAmsjw/8iZbKhkinaHp//Pxfi8vMj9xl/a3gPfsBwZu85DTU1Fk/IPJC
Pc/jAu95Hhf4oh7ncYH3PI8LvOd5XOBSPc7jAsd5XJh/wHueuwXe89wt8J7n
boH3PHcLvLjHuVvg73ucu0X9BHyeDRzjq/nzuOATn2NRYvPjGROYT/Alff92
GFszn+D70jfWCP7HuVvgu1yeP1A3YD7BNR0W2CZlMp/gxXVrlvj7snMOwcuO
TVFO+R/n74GvmtP3Gzea9fFR3ci30IbFg1kfH3i1oEgrzJxdh+A/I33MD5xk
/WXgzwapWF64EkoeEFforqWVxrmsLw98tajPzBbp/9H/yHOv1j3FHtmsLw98
m4X+GKPL7Fws8M0h5a7iDqzPi9YLNvO5wV2srxx8TviSls+17Dwr8Ffxy2ZZ
zmXzG9WBXJeeJfmUnWcFrnkx5aDHBnY+EvVbJ+frnPyH9cGBc8HVr8vXsj44
8Gdn9t59uID1wYF/UVO/K5PBzokCN1Kpb9rSl/WpgYvOmdzdG8POWQW3fPZT
48+qGIof3P3vGIuArQEUJ37On8ctng0bWR8Z+MwPsVG9PrLzOWndMTL0mq44
O7cKPHXIGKm5vqyfi/J1OnWpZkvCKR4633Rv9MBxT9g5G3g/8artt60K99HP
pT7IceEF3tHsPgveEu12//l/9328P/iw9BX3HVXc6H3wfcdOfJy5cUIkjQe3
Orj3mlvaVo7y7D36Guh5C+vXfjoB47M20njwb3zdPv0d8nziYmuH4hI2b+Pv
pr7KKfzHa2uaZ8CXrDqU+DzDjd4f/Atfr07rcJ5LySy97umfQPMD+Galj82T
RyTRz8X1qxh3Rdnojj39XPCBl6dO2WHlRX/X1MfcNXT54tBAiofydXxdN+IB
z29PNBzfL5XiAa9qrbg+7Sj7+wV3e6tmJnkzneKk/NXipcdOPXOkOMHH/t0R
0mm+m+IEVxF/ceGcF+ujpP6nlhmF6dKsX4zqrvl6acQPXrZk3N8pdhkUP53z
ULD8yv4frJ+X9tGqV3gbX2LnFoLfKj1hPGJRFjv3g7/ej7j/7cMN2UGfC/x8
U8yjvsGsDxc86d3L6x+vsnPwwDVM+nipSbL5BzzBNmrbsyDWh0j7dHzdMj4v
ePXmxtWJB9n5ruDfRnzWGN6X3ZfB70REPNNxZvdliv/yU/UHtqwfH1whTHHM
7MAM8oC/S822qZJRhazvG7x3v46zJtas7xt81qZhyp/s2P0FXOZ33OQ/I9j9
Bfz5rBrdDefY/QVc3qLfpJEyKeQHfBhfxws/4O/LzmUsrGV99BRPwsVfSp+Y
H3BT9TGGD78wP+BJhbaNnt7MD81zXSYGtVPZuRDgR5YcG5zuwP5+ERfOJUD8
4DZ8fSnGI97+S19Yv33GPFP/NN9fj/cBT+LrJ/G+4C9Df9gNz2bvj88dPtUo
IFPErmfwPa9mas6d6k+/R/A8Pv+GnwueydcT4ufSeUgTzQprdNnzD3hBzCvX
aD02b1D9m5uZKHOBE8UDHvOt+Pf+FjZvgKtJXfw1KpXNG+DyfP4Ncfasx0Oc
4B1rRk5aOCOFne/E8xtlTyfv6c3O/abzp/sbxQ9+zH6/uC5OPXo3wiubzc90
/39uWKtgz+Zn2ueqkshvWB5I8YOv7JB3TfPbS38XVP/G59/wucD38PVv+Fzg
nyVKe/uqsvsLnR8jqPf9+0lEnwv8XK9mkzW34+n6B7842n9rzf599HlxvXsN
+nBqWagVfV7w6TNixKP+u87xecFtpp8Wtzntw84l4HmTuaNa3wvsOYfe/5qJ
/6A17Lwp8EI+/wYPPevTqC8e51s4Hei/dkc0eaBzXH6Xiw1/F0UewG9P8shq
6IwgD+DTzPO1X+XvpXmA3kdTpjh6BjtPAM8px3vkl8AbW7x8vP/YkB9wnYuc
f2u4kPyAP/wWveSMtjs7TwB5uXfXM764eZEfcF/PSQ6aM//HuWc8n9mjL7hn
HRq8gRcOjjdM8GL/vwn6/wB84napjgkgb+Axs1edFVj5kTdwLe+bVc+zvckb
nYf6p7VghKIbzZ/g5iYnzxq52NLzHrxu4/dt/w+zgOpL
   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJw1mXngTVX3xu89e59QCiHzTIY0GKIUMqUyhaIXjYQyJ2k0ZMicprdoLg2S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         "]], Polygon3DBox[CompressedData["
1:eJw1mnfgz1UfxT/33o+RJMloyMompfGkTVMKpaVCGS0aGkakjJKUStEwCiEq
FQ17Z1dP2kiaD+2heBrynJfzef647j33fr7f3+d7x3mf97lqdrmp3Y0xy7Jr
9U+u+pSUZQ+rbi+8Ux0dhGcI9xKOJbLsRdXbhL9ReUntwzU+UO2Wam/X8y+p
jApZNk5952jsWJX71D5P479p7Djh4cLnC/8ufLLwQ8KXCO8QHii8THiM8BLh
51VG6PtGqe80jdUXHqb2QPX9V+171PeR8FN6vrreb4PqhsJRdW+Nn6fxR4S7
C88Uvl31dH32ZfVdobE/hFupXUv13Rpvl/y3rhd+Rfh84UeFewjPEt6oupFw
Ut1HuKfGZwkPF14oPF31F8Lfqrysdr3c83N78Hx+ob6jhMuo7if8ueojhfdS
fbvwISpdhTvo+R/UbpB7/gYJ/6H2x3qugXBQ3Uv4OZUHgt/xVL3L3Sofqj1e
49U0H72EVwqPEN5P+Bk9Pzh4DViLycJDhO8XPl74ZpVX1L5fzy/SWG/hVcIP
ClfQ5/sLvyk8SriK8Bxq4c/0HUv1/ACNvy38mPoP1Hjj3Os7TON/q32nxv8t
/LjGD9L4HcJvCY8WPkB4iPAHwuOEDxHepLqxcK66rz5/q8ZXCD8gXF7jfYRX
Cz8kvL/wpcLPCd8sHISn6zP3B++BFhp7V+31KhM1PqnYE2PAmffKUJWP1X5a
YzX4PtX3BZ8JzsZdKu+o/YT6D9b47cJrhUcKVxKeyp7U8yPUd6LG+qqsUfth
jVfUeD/hdcKPClcWnqbnh+v5keprrrEpwncLPyB8gvAY4V7CA4QbCzfMvXas
4Z+cJ/ac8EThvfR9zYSnCU8QLi18m/ARah+rvo/0N48Vfpbfp75SGq8m3E24
o/CPaldV6SJ8ufD3av9LZSrnS7iknr9T+FS1z4YD9H3H557vSeoro/GmzKfw
I8Hzz54+Tu0Wmff6AOEWwi2FtwkfIzxB7SfVV4L9INxc7bPUtxV+EX6C+VPf
bn6LypPMt3Cm50vrmWZqN2cPFmeiqXCzzGeFM3qU8PGZz+4OzWF1tauob4XG
ZuR+V975bI0NhjOELxHeqeeHwAHC7YX/yx5VaSl8nvDPat+i8UbBZ/o94ZuF
Gwaf6XeFBwqfLXy+8C/C9VXaC1+ZQSIF5whfmpmL/lRfPbVrqG+NxnYK1xA+
QHhlbk47Uu3jMnMdZ+Jo4RMyn5W7hE8L5rRvhf/S55uoXVd9a3lW5Wn2r3Cu
+ftD43WFqwuv1tjfwocL1xNex9ypPMV5FU56vonw45w34X/UPjF3fJgqXFbj
LZPX58povqoN53Ieg/m/TfLfvyaaD85MXt8rovmcOTpV+MD/z13y2nSO3u8v
qDyo7xqtvtM11jZ5/xC/dmmspso1/H0984vaNVSu5vuFf869puy9GtFrfVHy
eSAI8ntqqXQX7qLnf1W7ffL56hm93y5JPl83Re/HF1Ue1bNj1deKvSV8ldqd
1PeT2nVU+gnfEhzvDsvNJXAK78ue4izVLPbavfqODZxf4Zr6e2epb4Hwa8F8
e47wEuG5wfxzhvA84VnB/NNKeLHwnGB+bSm8UPj1YH49mRgtPE24nPDZwouE
Zwfz9enCc4VnhoKvhI8PXhNiVy31LRNernKo2qdqfLbGXgqOD6cJzyH2BfNx
c+HXhGcE83UL4deFXwyOR8P0ezdm5mK+r3XyfruavatnT8kdq4nZ+2r8JOZc
+FnhfYQvSN4LN+j5VzV2U/L8DIt+/in19dezg9XXFG0j3EP4FuGDiUXChxf8
+B/Op/p6EEvVfpzYyv7R+I3q2x+twv4Qvpk9KrxQz+3DZ9XXDa4Tvk7ta1Ue
FJ4vvLfGvxLuIjyC/an2TeqrpM8v0HhZtb9WX1fmRrik8KfCHYXfKfi7WcGH
rxY67RP1Xc7cxj3bMtss3CF3PJoP/6p/MftJpTPvo77SGqud3G6u8dHERvar
xm9Q337JMaI+3BQdO+AguKh+ET/mqZRR+0v1ddZYqWRuPEL9Q4UnEuM1do/6
jkYLCDcR/he/Uc+8ljtWEDOI1V+p73S1y6p+K7fG47Mdo7XfFtWnCJeM5iM4
DC6Dozar73uV1mpXVL0+92+ES5tG/3aeOZmjG/1ZYgSxopTqN+FqPX+98GnC
T7D3Ve/FXoejc+ub0sQu4SuE3yUmBr/Tj9Exm711WXQs5x2OCf6bvNseTaZ2
h2ithgYZQnyN1ia8Q30937SIX6/EPWEh26S+y3LPKVwGpzHX83Ofdc78dckx
jFhGjHg7OkYQK2oKvwOncq5Vagmvh2uitejAzNq9TrEn2AuPqaxGtxGr9EwP
4VXEPGKdcPfcMYJYQYz6MPozPYX7ZP6u79BYaq8N1u8TVAYEz8FRcL/wHcFz
cCSxQvg24TuEGyWvwa3C/TOvDWcQ7oRDOZujOOPCvYUPRZsL3yjcCw4VHg/f
Cg9iTwrPjHu2craRGKuxZehEYqPw1cJvoNtYK84I64HuFP6dM5o7hh8q/Gdw
bGcOmcs/hHvyeZUywXPEXA3LHTuIIfvo7/+k8obaC9RXVQd1m+q/hSuoLI0u
tL9R/1X67HJ0ovB3rHluTVFb+K9grTFcpVvwnt0XbYmOVvtX+jQ2VqW32neq
7zC0KTpV7e2ccY2tE64q/BucJvyIyg1q36a+6uRqwl2DOXBv4UnCdwkPFT5G
eKTw9cK3Ch+SfCbRCkdkPquc8TP47dFnH049U7hcNNeyJnBvq+i1Yg+zlwdF
60f2BO9ybvReYfx54cHRex8O52+fE83t76kcW+i/n9TXIzle3g3vljAHsVfa
RHMTnEOsvqXgIjQ13H5GtNa+kt8s3E94jvA/qjuxt1V/knsO2Otto+dmcfTZ
26p3WAbXavwZ4f7qn5s7Jo3X2OTMseqy5Nz4Vo2/nrs8E/wbGeuS/OwdcG3u
HHRc8HeSm5JTk1t0i861eeaF4JyNz96QHM+HRsfHa5Lj8V3R+cDVyfH6zmj9
3y05Hg+I1vedkvVBn2h9iMZi7TtFa689mp/9G50LdEyOx72j9SFnnLPYOvrs
35isR+6N1htoJHKbrtHa6fpkfXJPtH5kzp4PzqmZS2Iesa98dG6AHiRX7RKt
3ckh0ML7RecW1ybrl4HR+RAaGG28f7Q2Zk2f0/fPzLzWu9V/hdp1VW8msCZr
83rCnwrH5HyoofCW3JzfRrhSdCxgz7UVrhy9F8kJyA2qqH5fuGSy/mwi/KXw
fdHabUlmrQJnE8t2BHM5Off8YA+EXLyZxi8lFqh8l5sD4cIW0foUjufs7Szy
g5XRev5HNFpuTiW2nx7NtcwnudW+0dpiT04ovDs41sPpxOp/inyA/KpBcD5D
/vaByjFq71LfqiI+EAt+CuZCzsCfrEfms4EnMk9jSzN7JU+qb01wDk0uDUeh
hc6M5q7fVF8oXFX1h8J5cn7aWPhz4e2qLxA+mHcR/kt1B2KB6o3CKdk/aST8
mfAO1RcLV1P9cW4PZ2zwGcfb2aX+jmrXVr0JvonORSdkzi3wSIhF7aK9E3J6
uPWi6Fy/RLLeP0z4i9wxmlh9uPBXuWMgsXB9Zm8Lzwk92zc63yUfIdesrnqD
nv1d9UVwq+qPcnsyxMYLor0aPAG8gAujvQJiUINCPxCb1qn9vdr/URkr/Huy
P1OmWB9i4N/F+hAb0eBocTgBrwgNj5aHM/B+yAFmFnoDL2ND8PqWJ2Ym7+fn
ivPK+pKDoE/QoGhRND/aH47BuyEnIDfgjOLd3JN7f3cWDsxNcn6GB8B5fz6Z
b8h5yX1fTM5F8BTwFl4SPi1zjkKuMis598WjgJ9egTMz58Scf3Iocik4By9r
ZnL+hScBf3yp33M0sU91f7gimV/wFPAWyBHJfclZyR3x2BYFe2R4b3AguTI5
LtwIp5Ebw5FwHWvO/pqYeS8g3vEyyHHJNfEnxxT7E34iJk4q4gOxEE8Fv4IY
RiyDo58o9ivcDeeTG8HhxAI4l1yPPQcXo0HQInho7XOfx3kF37A/yRHJFYkh
eHPkhOSGcDjeHjnlnIK/8PLw6BYGe4x4d+Rc0wt+hd/IUecW55/zDidPLPQ/
XD09md/Jccg3ZgifmNnTwNu4ILffuEK4Dr81Wc+UD86nLsrtN65Ck2r8vNx+
29Lg/Lldbr/tDeHawg8n6y3OAGfh/Nz+HTktue2Fuf3IlcJ1yfdzcxWc1UD4
4tz+5Wrh+sx1svfTsNCHzyT7w4cWemsKHCdcp9CTk5P9zNqF/lpEXBR+TnVD
fd9jyfq7bHD+8b766/C+aC7WK1m/VQjOF19Ijm94XHhdjyfrwXLB+dnoZG3K
+Sd/gbOrBccIuJwz107tBtFncTYaJbMnhTdFDsncrC3OMx5vyeCciNyIGFM6
mGOIPQs4c8X57462SM7H8NDwN36NPq8HqR6EPir+Ph4Zfs9sdInwFuFOwqM0
Xlk4D86HH03W6zE4330kWS9nwfnwtOTz26jQyyOT9f4/mfMpcszDgmMYuedr
yfkrnh9+EBoJ7xTPB+2ERsIrxfNBO6GR8ELxjNBO+MPkG4Mz54J4Xnj1eLR4
YXj+5FuDMvvHeFh4t2gYtAweEF7QyCLe4InhleMR4ZURj/CH8a/xp/HIuPvg
TgDvDI+L/JCcnjwOjwuvixhUKzkmtQmOgcQqNEjb4JiDNkHznBWsWXYXMbZ1
cAwi9hJDzw2OycTWVSp9M+d85H5oJPxQ/E20E/4ma9mu4HM085nBGgktzedX
qtRVqZecj+KnkBOQGxDTTwz2BLYXZ+L56JyMs4Jn/mqwxu9XxCPuXGIRz/CQ
8KY488MKvnshOh6Si6HhzgnWEGg77rQeC/acuOviDqpvcH7M3RQe1L3Bdxh4
U3h45KNoArQBnibeJpqMuy7mhFx6ZTFX3InhB+EXkW+j4dByaMoDkjUc+SL5
FtqOnGVKsKdKLoOfxGd4Fr8JjXJGsAbdVWie04M9kL8KzXBycM6FlpiF7sx8
J0W8hrNWF/prhPCMaH1Czv5LMsetKfTKg8mcCXfCwfclczBcDEcOT+ZcuBdO
fCB5/vHyuF+BT8fpffoQezXeJJnDGgefUbiNO7vFwXc43OVx57IkmMO5i4HD
lxb8gzd6nT5/SDAnrihi9AnBnELshrPhbmLC/ckeHF5cT+EqyZzTqtifcNFY
9X2l9tcqPybvN7iYvh+S9wx7Z6vKz8l3ctzNoUmrCn9NjArWoGWSNRzaBQ2z
s9CcpwRr7h2FJoRr4By0Incs3LV0hROTNe9JwTk0Wph3rpycB/BbpiZrcTQf
2o87G+5u0KDVeFfhC4M1bblkDi0VzKFw60P8hsx3uKw3HjZ3K3j2eNvfCF8S
nANVFN4mfHFwjlQh2dPG2yanqSH8be77jM6Z33MrMTk4Zy4vPCn5PrZWsD/B
fuMO+bNif01I9hbxGPGX5rLGmT1L/Lu9cu9l5pC5PDD3d/NOvNu85LPDGcJ7
OSD3b+cdeJcyueeeNWFt5id7J3gceB0lcnsH3CGRr5XKzT2sAWuxMNkfwCPF
K12UnK/gCeMNL0jOr/Bk8GaIsWWKeEh8XJecL3CHh79QOvfackY5q8R4Yj0x
Gu9xSTIXwol4zVVy7y3WlLVdncw9cBDeGJoCbYGmwLt6Lzm2EGPIV1Yk+xV4
ZnhnbyT7IXhmeGeR/ZbZs8S7zHN7m3gq5LPLk/N5PCK8on1zxwo4H+5fmuzH
4KnjrS9L9m/wiPCKUm4vFM+UfLlsbq6Cw+CyNcn5DZ4f3t/a5PwGT5B8tTLn
I/MZ46wdnHtvsufYe2gctA6aCW96fbJfgQeN38OeY+9xZ45X9nHyXTl3wviv
FXPHRmIisXFD8v0/d+r4te8K35vZA8Pv+Sg5FhOT8XvfSfar8TjxO95K1gJo
AvytD5L9Gu50ydfa5uY6OA6u25h8f8H/ccDv3kQMyXzHhd/ydrJXyh0Gftf7
yf4Jd/bkg/vkjgXEWGLth3Be5jto8lM83JrBZwxvF02KNuVM4g1OTJ4bnuHZ
NrlzGTgZbm6dW9uT45DrnJs7d0Dzo/0Pys0NcAbccVIy11+s735WeHEy93Ln
wd1HpdzaAw6EC/fPrTXQKGgVOAAuwFPAW0Ajo5XR1HiXTYrvOys6tpTLrS3Q
MGiZN4u/jweKn4BGRCuiEbkrwQ/Hb8czwTtBM6Id0YjcrWxJzr3Iwfi/AZ8k
51rkXPhNnyV7cXgIeAlwGFw2LTp2zlbfZNZOfZfDx8neHJ4W3tY3ybkWORfe
yLfCyzN7Png/eNIHBXvMywoPCW5qkdlb4v6Z3JccmLvS7cm5ODk53gsaF61L
jsNdwuZkv5E7Svw2Yv4W4WejtcCnwuMz/58d/MRzi/1zlcZfFn4q2X8+ONiP
fjo59lYN9svHJ/vjvDPvPiZZq1UMvg8bl/xbuIPA3yZmE4t+CI7lfCffzXcs
L/bM0OB3YC99mZz7kgPj136e7I/iieKNfiE8JbPHitfKO48O/k38lsuL33db
9Np8lbw2eK54r3OStTMamrsW/HnuT2pk9q/+nXx3wZ0FfvTu5NyBOy38bXIw
cjFyUu7yyFHIVchRuMtDE32q9tRorVRJY99l1hfojP1ya1E0Nlr762QvBU8F
rxLNtYn3jdZi5LzkvuR43C0yp8xtzcx3CVv1/KuZPWG8YdaAtSBn5K4STbRZ
7SmFVuIyEb+PO1D8+ZDbDyQfxx+skFv7kxOQG+zNHsmsOdGeaMBPhCdHa8Pu
xfwOKXJB9gh7hT3EXcuu5LPHGeTua1tyrrjHMxL+H9qvwLI=
         "]], Polygon3DBox[CompressedData["
1:eJwt13ncjWUaAODvO+dIliRLRpFBGEKTtawjW9nK3tgauylLZclkKWPf96US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         "]]}]}, {}, {}, {}, {}}, {
     {GrayLevel[0], Line3DBox[CompressedData["
1:eJwt0rsvpFEYwOGPcRsMK6Jd0RAFpSgoN4pViSjoiGK3WIlsQakQOh1/AS0R
+k3o3MfMuM2ystkIu2Tjfg3PSRS/eU7mO5nznsxX0zvQ8S0niqJR3SqsL/Vb
87lR9IE/WM0NNvKQrTxnO5/Zw9JYFCX01bqew2zmONs4xS7OsJ+L/M5l/ucJ
77Wta/3TioFSXGWaa8xwnTthJu5yk3vcYlL74Xd4wBSzTPMnMzzkDo+4y1+c
dY9TPuhYN+GOmjBbkiNc4iAX2MdpdnKSnzjGJg6xjrX6Yh1nNx+d8Zl/2cIs
G8K9+JEX9v3hncp1FfaqyoxlrGSCFSwNe1jCBItZwng4j0UsYL4KrWMsCP9x
+I6vytNLeKYnGSOa83H2/j68Acy/Svc=
       "]]}, {
      Line3DBox[{690, 1004, 473, 689, 1113, 912, 691, 1114, 913, 692, 1115, 
       914, 693, 1116, 915, 694, 1117, 916, 695, 1118, 1010, 1217, 696, 1119, 
       917, 697, 1120, 918, 698, 1121, 919, 699, 1122, 920, 700, 1123, 921, 
       701, 1124, 922, 702, 1322, 1005, 923, 1006}], 
      Line3DBox[{704, 1011, 1218, 703, 488, 705, 1125, 924, 706, 1126, 925, 
       707, 1127, 926, 708, 1128, 927, 709, 1129, 1012, 1219, 710, 1013, 1220,
        711, 1130, 928, 712, 1131, 929, 713, 1132, 930, 714, 1133, 931, 715, 
       1134, 932, 716, 1135, 933, 717}], 
      Line3DBox[{719, 1014, 1221, 718, 1015, 1222, 720, 504, 721, 1136, 934, 
       722, 1137, 935, 723, 1138, 936, 724, 1139, 1016, 1223, 725, 1017, 1224,
        726, 1018, 1225, 727, 1140, 937, 728, 1141, 938, 729, 1142, 939, 730, 
       1143, 940, 731, 1144, 941, 732}], 
      Line3DBox[{734, 1019, 1226, 733, 1020, 1227, 735, 1021, 1228, 736, 520, 
       737, 1145, 942, 738, 1146, 943, 739, 1147, 1022, 1229, 740, 1023, 1230,
        741, 1024, 1231, 742, 1025, 1232, 743, 1148, 944, 744, 1149, 945, 745,
        1150, 946, 746, 1151, 947, 747}], 
      Line3DBox[{749, 1026, 1233, 748, 1027, 1234, 750, 1028, 1235, 751, 1029,
        1236, 752, 536, 753, 1152, 948, 754, 1153, 1030, 1237, 755, 1031, 
       1238, 756, 1032, 1239, 757, 1033, 1240, 758, 1034, 1241, 759, 1154, 
       949, 760, 1155, 950, 761, 1156, 951, 762}], 
      Line3DBox[{764, 1035, 1242, 763, 1036, 1243, 765, 1037, 1244, 766, 1038,
        1245, 767, 1039, 1246, 768, 552, 769, 1157, 1040, 1247, 770, 1041, 
       1248, 771, 1042, 1249, 772, 1043, 1250, 773, 1044, 1251, 774, 1045, 
       1252, 775, 1158, 952, 776, 1159, 953, 777}], 
      Line3DBox[{781, 1160, 954, 779, 1161, 955, 783, 1162, 956, 785, 1163, 
       957, 787, 1164, 958, 789, 1165, 959, 791, 1166, 569, 793, 1167, 960, 
       795, 1168, 961, 797, 1169, 962, 799, 1170, 963, 801, 1171, 964, 803, 
       1172, 965, 805, 1174, 967, 807}], 
      Line3DBox[{806, 966, 1173, 804, 1265, 1057, 802, 1264, 1056, 800, 1263, 
       1055, 798, 1262, 1054, 796, 1261, 1053, 794, 1260, 1052, 792, 1259, 
       568, 790, 1258, 1051, 788, 1257, 1050, 786, 1256, 1049, 784, 1255, 
       1048, 782, 1254, 1047, 778, 1253, 1046, 780}], 
      Line3DBox[{809, 1058, 1266, 808, 1175, 968, 810, 1176, 969, 811, 1177, 
       970, 812, 1178, 971, 813, 1179, 972, 814, 1180, 1059, 1267, 815, 585, 
       816, 1181, 973, 817, 1182, 974, 818, 1183, 975, 819, 1184, 976, 820, 
       1185, 977, 821, 1186, 978, 822}], 
      Line3DBox[{824, 1060, 1268, 823, 1061, 1269, 825, 1187, 979, 826, 1188, 
       980, 827, 1189, 981, 828, 1190, 982, 829, 1191, 1062, 1270, 830, 1063, 
       1271, 831, 601, 832, 1192, 983, 833, 1193, 984, 834, 1194, 985, 835, 
       1195, 986, 836, 1196, 987, 837}], 
      Line3DBox[{839, 1064, 1272, 838, 1065, 1273, 840, 1066, 1274, 841, 1197,
        988, 842, 1198, 989, 843, 1199, 990, 844, 1200, 1067, 1275, 845, 1068,
        1276, 846, 1069, 1277, 847, 617, 848, 1201, 991, 849, 1202, 992, 850, 
       1203, 993, 851, 1204, 994, 852}], 
      Line3DBox[{854, 1070, 1278, 853, 1071, 1279, 855, 1072, 1280, 856, 1073,
        1281, 857, 1205, 995, 858, 1206, 996, 859, 1207, 1074, 1282, 860, 
       1075, 1283, 861, 1076, 1284, 862, 1077, 1285, 863, 633, 864, 1208, 997,
        865, 1209, 998, 866, 1210, 999, 867}], 
      Line3DBox[{869, 1078, 1286, 868, 1079, 1287, 870, 1080, 1288, 871, 1081,
        1289, 872, 1082, 1290, 873, 1211, 1000, 874, 1212, 1083, 1291, 875, 
       1084, 1292, 876, 1085, 1293, 877, 1086, 1294, 878, 1087, 1295, 879, 
       649, 880, 1213, 1001, 881, 1214, 1002, 882}], 
      Line3DBox[{884, 1088, 1296, 883, 1089, 1297, 885, 1090, 1298, 886, 1091,
        1299, 887, 1092, 1300, 888, 1093, 1301, 889, 1215, 1094, 1302, 890, 
       1095, 1303, 891, 1096, 1304, 892, 1097, 1305, 893, 1098, 1306, 894, 
       1099, 1307, 895, 665, 896, 1216, 1003, 897}], 
      Line3DBox[{911, 1009, 685, 910, 1321, 1111, 909, 1320, 1110, 908, 1319, 
       1109, 907, 1318, 1108, 906, 1317, 1107, 905, 1316, 1106, 904, 1315, 
       1314, 1105, 903, 1313, 1104, 902, 1312, 1103, 901, 1311, 1102, 900, 
       1310, 1101, 899, 1309, 1100, 898, 1112, 1308, 1007, 1008}]}, {
      Line3DBox[{251, 474, 1113, 252, 488, 280, 1222, 503, 295, 1227, 518, 
       310, 1234, 533, 325, 1243, 548, 340, 1254, 563, 1161, 355, 578, 1175, 
       370, 1269, 593, 385, 1273, 608, 400, 1279, 623, 415, 1287, 638, 430, 
       1297, 653, 445, 1309, 668, 460}], 
      Line3DBox[{253, 475, 1114, 254, 489, 1125, 281, 504, 296, 1228, 519, 
       311, 1235, 534, 326, 1244, 549, 341, 1255, 564, 1162, 356, 579, 1176, 
       371, 594, 1187, 386, 1274, 609, 401, 1280, 624, 416, 1288, 639, 431, 
       1298, 654, 446, 1310, 669, 461}], 
      Line3DBox[{255, 476, 1115, 256, 490, 1126, 282, 505, 1136, 297, 520, 
       312, 1236, 535, 327, 1245, 550, 342, 1256, 565, 1163, 357, 580, 1177, 
       372, 595, 1188, 387, 610, 1197, 402, 1281, 625, 417, 1289, 640, 432, 
       1299, 655, 447, 1311, 670, 462}], 
      Line3DBox[{257, 477, 1116, 258, 491, 1127, 283, 506, 1137, 298, 521, 
       1145, 313, 536, 328, 1246, 551, 343, 1257, 566, 1164, 358, 581, 1178, 
       373, 596, 1189, 388, 611, 1198, 403, 626, 1205, 418, 1290, 641, 433, 
       1300, 656, 448, 1312, 671, 463}], 
      Line3DBox[{259, 478, 1117, 260, 492, 1128, 284, 507, 1138, 299, 522, 
       1146, 314, 537, 1152, 329, 552, 344, 1258, 567, 1165, 359, 582, 1179, 
       374, 597, 1190, 389, 612, 1199, 404, 627, 1206, 419, 642, 1211, 434, 
       1301, 657, 449, 1313, 672, 464}], 
      Line3DBox[{261, 479, 1118, 263, 493, 1129, 285, 508, 1139, 300, 523, 
       1147, 315, 538, 1153, 330, 553, 1157, 345, 568, 1166, 360, 583, 1180, 
       375, 598, 1191, 390, 613, 1200, 405, 628, 1207, 420, 643, 1212, 435, 
       658, 1215, 450, 1314, 673, 465}], 
      Line3DBox[{265, 481, 1119, 266, 1220, 495, 287, 1224, 510, 302, 1230, 
       525, 317, 1238, 540, 332, 1248, 555, 347, 1260, 570, 1167, 362, 585, 
       377, 1271, 600, 392, 1276, 615, 407, 1283, 630, 422, 1292, 645, 437, 
       1303, 660, 452, 1316, 675, 467}], 
      Line3DBox[{267, 482, 1120, 268, 496, 1130, 288, 1225, 511, 303, 1231, 
       526, 318, 1239, 541, 333, 1249, 556, 348, 1261, 571, 1168, 363, 586, 
       1181, 378, 601, 393, 1277, 616, 408, 1284, 631, 423, 1293, 646, 438, 
       1304, 661, 453, 1317, 676, 468}], 
      Line3DBox[{269, 483, 1121, 270, 497, 1131, 289, 512, 1140, 304, 1232, 
       527, 319, 1240, 542, 334, 1250, 557, 349, 1262, 572, 1169, 364, 587, 
       1182, 379, 602, 1192, 394, 617, 409, 1285, 632, 424, 1294, 647, 439, 
       1305, 662, 454, 1318, 677, 469}], 
      Line3DBox[{271, 484, 1122, 272, 498, 1132, 290, 513, 1141, 305, 528, 
       1148, 320, 1241, 543, 335, 1251, 558, 350, 1263, 573, 1170, 365, 588, 
       1183, 380, 603, 1193, 395, 618, 1201, 410, 633, 425, 1295, 648, 440, 
       1306, 663, 455, 1319, 678, 470}], 
      Line3DBox[{273, 485, 1123, 274, 499, 1133, 291, 514, 1142, 306, 529, 
       1149, 321, 544, 1154, 336, 1252, 559, 351, 1264, 574, 1171, 366, 589, 
       1184, 381, 604, 1194, 396, 619, 1202, 411, 634, 1208, 426, 649, 441, 
       1307, 664, 456, 1320, 679, 471}], 
      Line3DBox[{275, 486, 1124, 276, 500, 1134, 292, 515, 1143, 307, 530, 
       1150, 322, 545, 1155, 337, 560, 1158, 352, 1265, 575, 1172, 367, 590, 
       1185, 382, 605, 1195, 397, 620, 1203, 412, 635, 1209, 427, 650, 1213, 
       442, 665, 457, 1321, 680, 472}], 
      Line3DBox[{277, 682, 1322, 683, 278, 501, 1135, 293, 516, 1144, 308, 
       531, 1151, 323, 546, 1156, 338, 561, 1159, 353, 576, 1173, 1174, 368, 
       591, 1186, 383, 606, 1196, 398, 621, 1204, 413, 636, 1210, 428, 651, 
       1214, 443, 666, 1216, 458, 685, 686, 687}], 
      Line3DBox[{459, 667, 684, 1308, 444, 652, 1296, 429, 637, 1286, 414, 
       622, 1278, 399, 607, 1272, 384, 592, 1268, 369, 577, 1266, 354, 1160, 
       562, 1253, 339, 547, 1242, 324, 532, 1233, 309, 517, 1226, 294, 502, 
       1221, 279, 487, 1218, 250, 473, 681, 688}], 
      Line3DBox[{466, 674, 1315, 451, 659, 1302, 436, 644, 1291, 421, 629, 
       1282, 406, 614, 1275, 391, 599, 1270, 376, 584, 1267, 361, 569, 1259, 
       346, 554, 1247, 331, 539, 1237, 316, 524, 1229, 301, 509, 1223, 286, 
       494, 1219, 264, 480, 1217, 262}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0e3c012/4PkpGUtFCZaSM7JARd3ZEooxQlJTMkFnKyMjee3vbe2+PnZCd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    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.5755448530571833`*^9, 3.586581478591857*^9, 
  3.587380854911333*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"sxxi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.505411203510949*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nHk4VV3495GieVBPc6k0GJpQqWQnEdIsIYqKkjkUDWRW5lllnud5Jps0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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJw1mXfgV1Mfx++557aLrNLW1pRob1RKyMomZa+GrVSEUEoaJJKGpAihiEpG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         "]], Polygon3DBox[CompressedData["
1:eJwtmmXAVcUahc/MnE/FVkRBREJpFES95rW7uxOwRRRUFBPFwm7FBMUOwAQT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         "]], Polygon3DBox[CompressedData["
1:eJwt13f81fMeB/Df75yjqaWEEiqhJTRop0W0p7a2rqa0qET7SkbbCJWRES6V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         "]]}]}, {}, {}, {}, {}}, {
     {GrayLevel[0], Line3DBox[CompressedData["
1:eJwt0jsvQ2EYwPHX/X6JCI0I8QFIiJFBjAxiZBA2BowMYsNgJrHzCdrZwGii
6lZtRUTcFnex+L2J4X9+T9L2PG9PTtfMwvh8UQhhTZ+K86tutV8cQiOP2MlL
9vCBg/zmKCtKQphkKxOaM/dzmcPc4Di3Oc09LjLFVR7yl/fxnjrRu57j7EBp
pnnKU2aY4RnPeM5zXvCClzHzFa+YZZbXvGaOOeaZZ4EFJv2fB/7oRh960Y6z
HXOTB1xhkvPc5RS3OMZ1DnGJfezVrLmZEyzjSHzWdg3wnt3xuZfKOTrift+5
45ca9KYndfi8nu2sYxtrmWANW1gdd7GKTayM94i/U4W5luWsZhkraXUop7Wh
lI4UUi6P/+/EH9gfR28=
       "]]}, {
      Line3DBox[{703, 1015, 481, 702, 1130, 923, 704, 1131, 924, 705, 1132, 
       925, 706, 1133, 926, 707, 1134, 927, 708, 1135, 1026, 1233, 709, 1136, 
       928, 710, 1137, 929, 711, 1138, 930, 712, 1139, 931, 713, 1140, 932, 
       714, 1129, 1141, 933, 1021, 1338, 1016, 1023, 1024, 934, 1017}], 
      Line3DBox[{716, 1027, 1234, 715, 495, 717, 1142, 935, 718, 1143, 936, 
       719, 1144, 937, 720, 1145, 938, 721, 1146, 1028, 1235, 722, 1029, 1236,
        723, 1147, 939, 724, 1148, 940, 725, 1149, 941, 726, 1150, 942, 727, 
       1151, 943, 728, 1339, 1022, 944, 1025}], 
      Line3DBox[{730, 1030, 1237, 729, 1031, 1238, 731, 510, 732, 1152, 945, 
       733, 1153, 946, 734, 1154, 947, 735, 1155, 1032, 1239, 736, 1033, 1240,
        737, 1034, 1241, 738, 1156, 948, 739, 1157, 949, 740, 1158, 950, 741, 
       1159, 951, 742, 1160, 952, 743}], 
      Line3DBox[{745, 1035, 1242, 744, 1036, 1243, 746, 1037, 1244, 747, 526, 
       748, 1161, 953, 749, 1162, 954, 750, 1163, 1038, 1245, 751, 1039, 1246,
        752, 1040, 1247, 753, 1041, 1248, 754, 1164, 955, 755, 1165, 956, 756,
        1166, 957, 757, 1167, 958, 758}], 
      Line3DBox[{760, 1042, 1249, 759, 1043, 1250, 761, 1044, 1251, 762, 1045,
        1252, 763, 542, 764, 1168, 959, 765, 1169, 1046, 1253, 766, 1047, 
       1254, 767, 1048, 1255, 768, 1049, 1256, 769, 1050, 1257, 770, 1170, 
       960, 771, 1171, 961, 772, 1172, 962, 773}], 
      Line3DBox[{775, 1051, 1258, 774, 1052, 1259, 776, 1053, 1260, 777, 1054,
        1261, 778, 1055, 1262, 779, 558, 780, 1173, 1056, 1263, 781, 1057, 
       1264, 782, 1058, 1265, 783, 1059, 1266, 784, 1060, 1267, 785, 1061, 
       1268, 786, 1174, 963, 787, 1175, 964, 788}], 
      Line3DBox[{792, 1176, 965, 790, 1177, 966, 794, 1178, 967, 796, 1179, 
       968, 798, 1180, 969, 800, 1181, 970, 802, 1182, 575, 804, 1183, 971, 
       806, 1184, 972, 808, 1185, 973, 810, 1186, 974, 812, 1187, 975, 814, 
       1188, 976, 816, 1190, 978, 818}], 
      Line3DBox[{817, 977, 1189, 815, 1281, 1073, 813, 1280, 1072, 811, 1279, 
       1071, 809, 1278, 1070, 807, 1277, 1069, 805, 1276, 1068, 803, 1275, 
       574, 801, 1274, 1067, 799, 1273, 1066, 797, 1272, 1065, 795, 1271, 
       1064, 793, 1270, 1063, 789, 1269, 1062, 791}], 
      Line3DBox[{820, 1074, 1282, 819, 1191, 979, 821, 1192, 980, 822, 1193, 
       981, 823, 1194, 982, 824, 1195, 983, 825, 1196, 1075, 1283, 826, 591, 
       827, 1197, 984, 828, 1198, 985, 829, 1199, 986, 830, 1200, 987, 831, 
       1201, 988, 832, 1202, 989, 833}], 
      Line3DBox[{835, 1076, 1284, 834, 1077, 1285, 836, 1203, 990, 837, 1204, 
       991, 838, 1205, 992, 839, 1206, 993, 840, 1207, 1078, 1286, 841, 1079, 
       1287, 842, 607, 843, 1208, 994, 844, 1209, 995, 845, 1210, 996, 846, 
       1211, 997, 847, 1212, 998, 848}], 
      Line3DBox[{850, 1080, 1288, 849, 1081, 1289, 851, 1082, 1290, 852, 1213,
        999, 853, 1214, 1000, 854, 1215, 1001, 855, 1216, 1083, 1291, 856, 
       1084, 1292, 857, 1085, 1293, 858, 623, 859, 1217, 1002, 860, 1218, 
       1003, 861, 1219, 1004, 862, 1220, 1005, 863}], 
      Line3DBox[{865, 1086, 1294, 864, 1087, 1295, 866, 1088, 1296, 867, 1089,
        1297, 868, 1221, 1006, 869, 1222, 1007, 870, 1223, 1090, 1298, 871, 
       1091, 1299, 872, 1092, 1300, 873, 1093, 1301, 874, 639, 875, 1224, 
       1008, 876, 1225, 1009, 877, 1226, 1010, 878}], 
      Line3DBox[{880, 1094, 1302, 879, 1095, 1303, 881, 1096, 1304, 882, 1097,
        1305, 883, 1098, 1306, 884, 1227, 1011, 885, 1228, 1099, 1307, 886, 
       1100, 1308, 887, 1101, 1309, 888, 1102, 1310, 889, 1103, 1311, 890, 
       655, 891, 1229, 1012, 892, 1230, 1013, 893}], 
      Line3DBox[{895, 1104, 1312, 894, 1105, 1313, 896, 1106, 1314, 897, 1107,
        1315, 898, 1108, 1316, 899, 1109, 1317, 900, 1231, 1110, 1318, 901, 
       1111, 1319, 902, 1112, 1320, 903, 1113, 1321, 904, 1114, 1322, 905, 
       1115, 1323, 906, 671, 907, 1232, 1014, 908}], 
      Line3DBox[{922, 1020, 690, 921, 1337, 1127, 920, 1336, 1126, 919, 1335, 
       1125, 918, 1334, 1124, 917, 1333, 1123, 916, 1332, 1122, 915, 1331, 
       1330, 1121, 914, 1329, 1120, 913, 1328, 1119, 912, 1327, 1118, 911, 
       1326, 1117, 910, 1325, 1116, 909, 1128, 1324, 1018, 1019}]}, {
      Line3DBox[{260, 482, 1130, 261, 495, 288, 1238, 509, 303, 1243, 524, 
       318, 1250, 539, 333, 1259, 554, 348, 1270, 569, 1177, 363, 584, 1191, 
       378, 1285, 599, 393, 1289, 614, 408, 1295, 629, 423, 1303, 644, 438, 
       1313, 659, 453, 1325, 674, 468}], 
      Line3DBox[{262, 483, 1131, 263, 496, 1142, 289, 510, 304, 1244, 525, 
       319, 1251, 540, 334, 1260, 555, 349, 1271, 570, 1178, 364, 585, 1192, 
       379, 600, 1203, 394, 1290, 615, 409, 1296, 630, 424, 1304, 645, 439, 
       1314, 660, 454, 1326, 675, 469}], 
      Line3DBox[{264, 484, 1132, 265, 497, 1143, 290, 511, 1152, 305, 526, 
       320, 1252, 541, 335, 1261, 556, 350, 1272, 571, 1179, 365, 586, 1193, 
       380, 601, 1204, 395, 616, 1213, 410, 1297, 631, 425, 1305, 646, 440, 
       1315, 661, 455, 1327, 676, 470}], 
      Line3DBox[{266, 485, 1133, 267, 498, 1144, 291, 512, 1153, 306, 527, 
       1161, 321, 542, 336, 1262, 557, 351, 1273, 572, 1180, 366, 587, 1194, 
       381, 602, 1205, 396, 617, 1214, 411, 632, 1221, 426, 1306, 647, 441, 
       1316, 662, 456, 1328, 677, 471}], 
      Line3DBox[{268, 486, 1134, 269, 499, 1145, 292, 513, 1154, 307, 528, 
       1162, 322, 543, 1168, 337, 558, 352, 1274, 573, 1181, 367, 588, 1195, 
       382, 603, 1206, 397, 618, 1215, 412, 633, 1222, 427, 648, 1227, 442, 
       1317, 663, 457, 1329, 678, 472}], 
      Line3DBox[{270, 487, 1135, 272, 500, 1146, 293, 514, 1155, 308, 529, 
       1163, 323, 544, 1169, 338, 559, 1173, 353, 574, 1182, 368, 589, 1196, 
       383, 604, 1207, 398, 619, 1216, 413, 634, 1223, 428, 649, 1228, 443, 
       664, 1231, 458, 1330, 679, 473}], 
      Line3DBox[{274, 489, 1136, 275, 1236, 502, 295, 1240, 516, 310, 1246, 
       531, 325, 1254, 546, 340, 1264, 561, 355, 1276, 576, 1183, 370, 591, 
       385, 1287, 606, 400, 1292, 621, 415, 1299, 636, 430, 1308, 651, 445, 
       1319, 666, 460, 1332, 681, 475}], 
      Line3DBox[{276, 490, 1137, 277, 503, 1147, 296, 1241, 517, 311, 1247, 
       532, 326, 1255, 547, 341, 1265, 562, 356, 1277, 577, 1184, 371, 592, 
       1197, 386, 607, 401, 1293, 622, 416, 1300, 637, 431, 1309, 652, 446, 
       1320, 667, 461, 1333, 682, 476}], 
      Line3DBox[{278, 491, 1138, 279, 504, 1148, 297, 518, 1156, 312, 1248, 
       533, 327, 1256, 548, 342, 1266, 563, 357, 1278, 578, 1185, 372, 593, 
       1198, 387, 608, 1208, 402, 623, 417, 1301, 638, 432, 1310, 653, 447, 
       1321, 668, 462, 1334, 683, 477}], 
      Line3DBox[{280, 492, 1139, 281, 505, 1149, 298, 519, 1157, 313, 534, 
       1164, 328, 1257, 549, 343, 1267, 564, 358, 1279, 579, 1186, 373, 594, 
       1199, 388, 609, 1209, 403, 624, 1217, 418, 639, 433, 1311, 654, 448, 
       1322, 669, 463, 1335, 684, 478}], 
      Line3DBox[{282, 493, 1140, 283, 506, 1150, 299, 520, 1158, 314, 535, 
       1165, 329, 550, 1170, 344, 1268, 565, 359, 1280, 580, 1187, 374, 595, 
       1200, 389, 610, 1210, 404, 625, 1218, 419, 640, 1224, 434, 655, 449, 
       1323, 670, 464, 1336, 685, 479}], 
      Line3DBox[{284, 694, 695, 1141, 285, 507, 1151, 300, 521, 1159, 315, 
       536, 1166, 330, 551, 1171, 345, 566, 1174, 360, 1281, 581, 1188, 375, 
       596, 1201, 390, 611, 1211, 405, 626, 1219, 420, 641, 1225, 435, 656, 
       1229, 450, 671, 465, 1337, 686, 480}], 
      Line3DBox[{286, 700, 701, 696, 1338, 688, 699, 697, 1339, 698, 301, 522,
        1160, 316, 537, 1167, 331, 552, 1172, 346, 567, 1175, 361, 582, 1189, 
       1190, 376, 597, 1202, 391, 612, 1212, 406, 627, 1220, 421, 642, 1226, 
       436, 657, 1230, 451, 672, 1232, 466, 690, 691, 692}], 
      Line3DBox[{467, 673, 689, 1324, 452, 658, 1312, 437, 643, 1302, 422, 
       628, 1294, 407, 613, 1288, 392, 598, 1284, 377, 583, 1282, 362, 1176, 
       568, 1269, 347, 553, 1258, 332, 538, 1249, 317, 523, 1242, 302, 508, 
       1237, 287, 494, 1234, 259, 481, 687, 693}], 
      Line3DBox[{474, 680, 1331, 459, 665, 1318, 444, 650, 1307, 429, 635, 
       1298, 414, 620, 1291, 399, 605, 1286, 384, 590, 1283, 369, 575, 1275, 
       354, 560, 1263, 339, 545, 1253, 324, 530, 1245, 309, 515, 1239, 294, 
       501, 1235, 273, 488, 1233, 271}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0unc0l/8fPo5SqZSG2dBAg0SLCg8zK2WUhggpe6WEyJ5R9ipkj8re62Hv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    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.575544853144193*^9, 3.5865814786428647`*^9, 
  3.587380854992378*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     RowBox[{"syyi", "[", 
      RowBox[{"x1", ",", "y1"}], "]"}]}], "-", "1"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4147351353577976`*^9, 3.414735162924797*^9}, 
   3.5054112035119495`*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nHVcFPv3/7HBwq6LioKFiogoiuiIDXa3InoVlFIMQMQAERURl2uCYgAS
BiguqYOElHQILB0btInNz493Xmd+3308rv/MwydvZs88mX3vznmf9xlhYrX6
7/YKCgpanRQUOvw+3t0YHO46wWY2jlrfW3pm7All9mnOvjHK8CkDrvrXprIj
i0OZh9KiNnXDOOJaHqs8mqpDGPWoflPCLdKIZ51wTjw/IYTJSBzzTN0wh3iF
7uZ7o2Y/Zs5rJr6xEBUQN05Xm1e1JphZeM1ELLQQET8551S9cOkDpp3C//5V
EI/1+9pzWP1dZkdYhpu6YRU//kKn4wszrzAvTG8NNhTUEFewrX9k2mzG/DXU
3N9CJOb5xhRrXf0rrF223hSBuoy/3hMnPC2FQezbM0qxQot64opdPHIeXotg
p+oVLhUJG4k7mJW4pFxPYC83+Re1tTUTD1nieurq0BwW/4fXkw4bk684HmDx
f+LxQzQM99qw8Ey8x6LdBo/tWHgGn3NxYNESt9MsPBPf4bjR5f15Fp7BVXur
tZu53JOFZ/A7J3xnb97sxcIzuHHQavO2rvdZeAZfeWr81IheQSw8g5t+m1a+
4GkoC8/ghY/DIysyw8kz+OKjkhvxFSx5Bjd3nKf5624ieQa3LV+mkLgmnTyD
f/QpY8o088kz/FV4TPQ61vqMPINrKd5wSRsQRp5xHLR7fzunY2HkGfzOjazt
AVrPyTN4S63/Ymd9IXkGVxjf6NP6NJw8g1u3rNPxC4skz+BZC7e+HLk9hjyD
bywyEa7/7Qee6XV//Bx3fG4ceQZXfWC656BBInkGX2mf7NOomUKewVVOm1V9
PZNBnun8zsYbpLdzyDONr550+7PHW/IMT9Zdh38yuxJPnslf/NTga37x5Blc
Z+SkyHaF8eQZR9/bWg0LJieQZ/Cts30W9BMmkGfwQadbspusE8kz+Mri78Vh
lq/JM3iYhfEiSUwSeQafvsCnPONICnkG330zv8uiiDTyDB4S2bVniyCDPBPv
l3jvoWo2eQYvS/s5SOiYS57BXy7VNXBcWUCewedGhf2KKi0iz/DxpmDOP60b
08kzuMPKyy2WZunkGXx6+KSFA7zSyTN43qLXB3a+SyfPOGb1X/gi5mAGeQZ3
k57XThuXSZ7BC4+2eMUqZ5Fn8BDLwNtLQ7LIM7hqWGTYJOds8gzea+rz+jjP
HPIMPjdwge2xmlzyTNeb9GpKxoV88gxeN7B1VPiFt+QZ/OoRyez0/CLyDO4R
PGZH+bIS8ozrTm7rNPOXTy55Bm8YOnu5ekAueQZvv/r6GzGbS57BCy+XqR+T
5pJn8K6KQ6Ymj8ojzzhanmd75DrkkWfwN1GzvJa25JFn8PkbmteNdcknz+A5
7sGLbxkVkGfwkluBUumct+QZXDg+a+UT80LyDP4PIz2tW1xEnsGnTRJc1Dkm
Is/gQfldrdQNS8kz+NiAzjOlLWXkGdfXfneF3otbheQZfM6GpGw3v0LyDL6n
rPzuo+hC8gwekpr/JaeikDyDlwVY168cXESewcMkHj9DzYrIM47v+w3Y0SO3
iDyD37Y4dLd0czF5Bu8Vt2jEuG4i8gz+5rFSoW61iDyDF6/LP+5WW0KewacN
3h23a0AZeQbPk33NuLG9nDzT6wZPmtJ+RwV5Bvc/FcJ2qKogz7iO6f73XQ7e
KyXP4ArOoh63A0vJM/jNuaPtzF6Ukmfw61sbL00qKyXP4N/cznfd1KuMPIN7
lCQ0FG0oI8/gbwb6xniFlZFnHCforlpiM76cPIMbadec10goJ8/gd0zulnrN
qiDP4I8+RNX1CKwgz3SeiwddT6lVkmdw3e6twct9KskzuPOvx4+YYVXkGVzv
wLabWjuryDPi1R58NfzG+kryDP7eecCv4E2V5Bl8cYNs6M+dleQZvN+k0/b/
WFWSZ/CmjjPOnHOuJM/gV6VOJxzvVZJn8Jo5Ax/1Takkz+Blu3Q/r/9WSZ5x
HDJbQ8zMqCLP4IWMx/HtTlXkGXzPvsD1vYqqyDP40s3ekke61eQZPMJxyqpE
j2ryDN7d/NEqK3E1eQb3fRDYOfJTNXlGXPMj2vWoKa4mz+B9cgK0N5dWk2dw
rXu1GZXV1eQZ/NtOz0LLpmryDN7b0FiQ96uaPIMrHu8euGNADXkGr9c8//SM
Tg15Bi+001j3bnMNeQYPmmS9QHahhjzj2LjV/XRlQg15Bp/tGHYyp1MteQaP
nWhcfWtJLXkG7973wTXZ2VryDF50/Oj89Oha8gy+nR00S+V+LXnG648RMB4v
NovJM3jFlNPlZ7eKyTONZ2IHTtspJs/gIxWqbKNNxeQZfHRCb3XXQ2LyTPNl
zofD986IyTN4wGPnyIteYvIMbnpx7plT4WLyDJ66bVTPNSIxeQZv0pub9aSj
hDzjaJvTfv1PbQl5Bg9cMs9+xS4JeQa/U64Q/fOshDyDDzEd0DvbR0Ke6e8p
0Dy5bp+EPON1un9zYO3zpeQZ/E75qXPXCqTkGXzNpOOHDhdJyTO4mb3PGcUy
KXkGrzZJGr20VkqewW+dNmv52Swlz+DGedHixJ9S8gw+qiXI2U5ZRp7BozsE
/rg8WkaewYMmJa86bCAjz+A9eyrXLdwhI884Rtw549zOUUaewY9csd5Ue1lG
nsG9R3RQnXBVRp7BW6bEFw9eIyPPOJ/pmli1mEX15Bm8Y4XT9XmL68kz+Fyn
Sak+hvXkGfzgh3HGKUvqyTP4h/ERObNX1JNncP/rMd32rq0nz+Cl+d+b+m6p
J8/gogfDU57sqifP4J/WhGaNtaonz+Dme/+JjXaoJ8/g62JLp+dcqCfP4KcP
fdcTetWTZxxfyYzObPSrJ8/g7nEOGkvv1ZNn8KzCdUo3zevJM35P8Vrj3bkP
G8kzeI5yW+ju3xyewe3TNhpF/ubwDB6p7tZz3qNG8gz+YRDrr/64kTyD91Qd
0eLypJE80/vy1szeu0MbyTP42qGHtWueNZJnem64H5tsFd5InsGfZ7T2Fkc3
kmfwTpPnPP4Q20iewbuZTR688XUjeQbvp5JyuTy5kTzjeHZFpbNHfCN5Bg/K
qDv3JaCRPOPnw5+k/O9InsFf/vqXwzP4kgmpfzg8g+dr/MvhGXzblX/PA8/g
t4Wv/3B4Bo/Tif3D4Rlcp7/wD4dn8IBNd/9weAYPTDj5h8MzuFhr8x8Oz+Af
jSb94fAMbvVK4w+HR8rPjXv2h+P/OH6dN1f0P77rH8bTQtTEVLbZvgm3iGac
YxLs8hfkEjdddXSwpeg1c32mz+F6swK2NrLzm9GG9xgcT37v6zZk0x3KA4Ir
lJ1os9t2h8F5wOdH5mRVWmdSPOAtfYcmzI3KpfPj95rD3yv2LGqi+wFcNXK+
g9Jvjv/j2P7bhebppTyn5+Che4r/d704P+U/nw06FbA7lH4fvKafU+pNlzQa
j/OY2qT8H5/4vU/6iRafipvY4qxrk8cZChgcFerOdj50K5CxN7ARCi2aie8e
q5Lr/yKLxuPn/V9kpm2waGbxfxwTe44zKbBqJg/4PdWctdo9rvPnx89dZ45z
PxaQSedHvN/3/9/48XtZ/goaC6ybaTz5mTVf67hZKP0+fh5hM7I5voXP9+I8
b5LTAv//+PF7q45k/fHf3d6qezuFED4fa1/1QlvDhl2u081GQSGWXrfhYO79
Xj9DGHCMV/gevc5JxZa9Gly2Q0EhmcZ//Pvy5JUHQhhwjM/s6hC0xfQka9V7
02gFhSw+r+4VevXWkicMOMbP/pw6yIx1Yfumx6koKOTR+HeZftuXjn7EgGP8
1sXjps9ZfYn9PaivgkIhjdfQmGkxcHUQA47xMeZOWXNXXWG3zL+qpKBQQuPH
L2vJdLrjz7T9+VdOPIHL/2M8xXmzJjsj1JvFeIqHy2PnlOp8b2ur5O+HeWqT
R3lcY8Ax3nJPL5V2WoGsb7TZ57a2aj6fXzjrZY9zJxhwjP8U3NMy7WwIe+TG
7XdtbbU0/ubNRw65Kq7EMX7c/qQHr52fs4uP5ja0tUlo/On+70ctCLpHHOOV
PDPHrS6MZoesU5S2tfHrFKuLe2yrvhxKHOMDfQ8c+NEnjm3QnlXd1savX7yf
sXnn6JcxxDH+wYDXVVbuSezLXgfL2tr4dY1F/V6/nbX2NXHKw6fNbOnwKYPu
W8rT3Bh/rrJPGN234MPShSPXmYTR/Ql+0S91S1RDGN2H4Mls9Jqrwud0v4EX
MDvPNScL6b4C37K1S6ddUyLo/gGfGX21Nk8piu4Hyo9y+Xb83cE7dTHR7WXN
0t8dvKzQSCh99Ir+juBLH8RaFYxIoL8X+HOPQR20pifR3wVctzRhumBSGvkH
7/CUTW0dnEWewUOs9VQ1ZuWSZ3xun2k4+PHy/XjyDC5IDZm6Ny2ePIN/U+61
6Y1yAnkGP9kkGfTIJoE8g8eFz9Ib/SOBPIOP3jhlrXFIInkGjzF3HXvs+mvy
DP6My7fDM+Xt9l1ruMykkGfK53dK/ikdk0aewd+lC4NbTqaTZ4pzeu3aPlOz
yDPlUTYNmdTSIYc8g/ed0F1t7JQ88gy+/5u4cFRcAXmmPKJBl07Ge9PJM/i5
ocnWD93TyTP4jWwLg07Z6eQZPEfTZaCydgZ5Bi+oYcaZhGeQZ/B1+yqeTd+b
SZ7Bn11dpJKjlkWe5fPt8Ax+MLh11TOzbPIMnrv289CyFTnkGfzx8Uvzpu3J
Jc/g2yM6FKhH5JFn8PuJV9wTlxSQZ3DBpCkP9YcVkmfwKcebT2eqF5Nnyg8P
eXth+INc8gz+I6HdxM+RueQZXDojwWleUS55Bn+39MfDEV3yyDN486CNC80X
5pFn8DMDhh8IvZFHnsG3NHn2UeyQT57l8+3wDF7RZW1QT+0C8gw+eEiQwvoO
b8kzuJPwlWz+j7fkGdx3aomZjVoReQa3+irZc/VIMXkGz97YciakSUSewb+s
OC9VcSwlz/ieXmhbVybwLSTP4N4xRRajhIXkGbyfYMaXmpxC8gyeNEPtfe+f
heQZ3DJWf/Vh3SLyDN5hyvt8Reci8gzepWLn3i41ReRZPt8Oz+CnNiwLHf6p
mDxT/FM2aeg8EpFncJ3WAPahcwl5Bj8brqL65XgpeQa3XqTSxfZyGXkGF8/P
PXRIWE6ewasS2d0mVhXkGc89i1w/ekcElJJn8A+CNbePRJSSZ/CipyaLarNL
yTN4yseZy+a3lpJn8Jx9xjvGjS8jz5TfDY4x7mpVRp7BI6LtHh+JLyPP8vl2
eAYX57y71ze0nDyDG3hc0Y0aXUGewRdvyx4Vf7KCPIMfi70auUhcQZ7Bq/QT
DjetqiTP4KmKfzd0ZSvJM3jxm/lpt0ZXkWc8R9ofXfjccFMleQY/2OdTvnRH
JXkG7zIgJfzYvkryTOsMN0vbetpXkmfwZUP670q8VEmewQNfSlPVHlaSZ/B5
C2YO0M+sJM/y+XZ4Bs+pGp9sMrmKPIObssK3BdZV5Bnc+hvj1xhRRZ7BN1WN
HL5DsZo8g3/sMvXJgE3V5Bm8x7dV2ZP8q8kz+IkPS103SqvJM+Wxmz5YdCut
Js/gYy0LJExVNXkG76n3/qNdXTV5Bj+Tb/f87udq8gzuXlIRFdClhjyDn5Lt
Hy8bWkOewQ2nl41eM6OGPMvn2+EZvMV276LUMzXkGdw7e6HNfWENeQZnJ+8Z
YdtYQ57B29Z11Y4YW0uewWPivy3UM6klz3SeiaLTVf/Ukmfwrv4X1MIjaskz
5ZnLb1xZvlVMnsHXxYSc62osJs/gCRsOH5L9LSbPVKdQZZdx11JMnsEPnnvd
/r29mDyDb1cqXDj0gpg8g5/c90O7u4+YPMvn2+EZ/GUHpdLCAjF5BvfrEO2z
8ZuYPIP73iied2eEhDyD91JWMx29REKewUftOp/T64CEPNPrGrh4vL8oIc/g
VhZfb4+4ISHPyBs1zTVosSmQkmfwPVtefn1RKCXP4Aa3tXesKpGSZ3B/yynr
gyql5Bk8RXN3znKZlDxT3nvgr0arD1LyDF4Ubrf5R5uUPMvn2+EZfLFBYONj
NRl5Bn+8+6RDjJ6MPIMH2H9cOWitjDyD79pgpOtrKSPP4Cditn08ckZGnolP
79+tVSAjz+AStS1vUy7KyDPycAcs6wO1F9eTZ8qf30x4sdiwnjyDGys1JH01
qifP4E+VDwx2XlZPnsGFeYOlE1fVk2fwj3vd5vdZX0+ewYeunLNvzNZ68iyf
b4dninMY+yTSop48Uz78nlfHDXb15Bn81+y4tHEu9eQZfP/NhhkJnvXkGdx+
TtGANT715Jni33HN8opvPXkGX1+7QXzFq548U5684oeV8cNG8gzusXWuuu9v
Ds/gBtVfdis/aiTP4AW3/Yqf/ebwDH5LNO904ONG8gy+it07/deTRvIMPjLd
pH9YaCN5ls+3wzN4m3brul/CRvIM/r1D0InkqEbyDD6mu59nT7aRPNO6Q3pE
tV98I3kG11y6T2yU1Eiewd+NvZRx4TeHZ/D9J06fmRXbSJ6Rr5OK/s0TwjNx
1X/z5/AMvnP8vxyewQdX/HseeKY8/KnkPxyewccMSPzD4Rk8q+XlHw7P8vl2
eAYf8u7+Hw7P4DKZ6x8Oz+ArAkz/cHgGb3wx/w+HZ3CTaxP/cHgGPyZaWvT/
5zlxNK7xnP5S/SDlMzdfMGiLsAhmdC4P1L3S7Ep5S6+RvrVWIn9G88c99Z7n
PSk/WcLloav2yj7aeN+gPKTevZzd0RZejIuPU91Gj9uUb4zi8rSjjVz23e7L
5w8P/4r8MdHwNDMz3kU1tt09yjdu5/JdC+Tyh+25+NeX+JvPuuxLPudz8fd8
PbntRPYD8unExX+9f59D+mMfkc8ELv6L4u7R0mtPyWdXLn7LI6lapwQR/HMf
F7+lvZJyzsVY8unofjBSoP6KUb++3qBZGEr3J45Mw4E0x7VP6P7UD1u0qUT4
gvHrN/3sz5Bguj9LufWOS29T4nu4B9D9eXxJEOOpHsn080l8PuyHH92fw66J
nlqKhIxg2PWogIG+dH++rOo+2kgQxhxUatPopXyf7k9/Lq8VJ5d/s+Xifx2w
peB7vxfkczEXv9P+e4k5fi/JpxYX//Zw0Y21N2PJZzYXf87Y1b8sL8SRzwNc
/B3EbXn2ponksw8X//RKk4z3rink89OORw3qhqnMgppTVlut48jn604L/QXq
Kcwje2F08e5X5BPHujNV8z4ejiWfe7h1orEXuhw2Xc6Sz2mtyrkiYSJjEFqS
eubbC/LZ+VaAm6EggdmRbjXqn58x5LNgrsHCcIt4Jn7Z/R1GFjHkcxiXv3ok
l2dr4eL/onp50tseyeQzlou/yf2hypzXyeTTg4u/7P2ZLwkfU8inMRf/VNOv
+X+/SiOfWlz8S/y8BmzbmkE+Fbj43++aumBb52zyGT7o4yWRMJuJ7qK2qknz
Dfn02R9rLlDPZtbpHfARDk0jn2dfuhkaCrKY8BsX3UrHpJJPHKdvjl7bbnUK
+XycbNTZUpTBjL99tMz2bjL5dLSsZkXCdMbulvl+T5Vk8rm8n4OdoeAN8zBq
4d53b5LIZxrqu+XyaWFc/Iecr2vbZmaRT28ufqus6O4Ph2aTT2fEf/W+w+a7
2eTTHPlXwa6h9/flkM8ALv7y/FbPLNdc8mnLxX9mmm3VsX759DllyMUv2Php
t0avt+TZ5tTlQHXDAsZ7b9SsJ4p8Pm1L3jYXkTCfubzXuXXI8RzyPG+Mxi6B
ej5zqoeO+s5f2eR5vP1nxlCQx3y98uuI/qVs8oxjRNQy1SDtbPL8ffilr0KL
XKb2TPwmtj6LPFcf3FJgIcph4tZoeV58yectz3J5qjly+TRrLv5Uo/1T1lzL
J88bufjN7j/yisrOJ89zuPinKIuD1s8sIM9jufj3fLx6fFN5AXnuxcW/tcnq
yj+Jb8nzFy7+tV30bft85/OWFVz88Vv339ay4/OWk+o04y1ExcyqCc2/ijvy
+bSBs777qBsWM4ssW5emHXlLntsuJTmIhEWM5YvZpvYfC8izpMpzk0C9iNl8
a4yi+GQBec6cajzNUFDISL0WVjDDC8gzjj0m9r+3MyefPPuIvjQLLd4yapc7
xty+xect53F5qpty+bQJXPxhYSVdzm3m82n9uPhTR2vr9D5eTJ5/cPEXtNSM
ykwqJs81XPwHRt48d85ARJ7fcPHvLbfcUVIvIs9hXPz1tolO0rQS8uzNxb/J
q3/2yUI+b1m/+k2J0KKcUVB1+DBCkc+n5fpdj7QQlTHxZ5evOHi0hDxHf9l9
Vd2wjFmwLKBg9jsReb6/ZLKNSFjK5I0PaXfVVkSeL9z+uUKgXsow+i1LEnqJ
yLPNu5QJhoISJuLZOgvzmGLyjOM0u1sqQxyLyTOOY+XyaVIu/qczhI0KA/l8
WhYXv3XHhH46w8vJcwQXv+pqi/Uui8vJ8x0ufu3p25IWXS0nz65c/DYNMx/F
tK8gz9Zc/MuPKt1QWlJBnjdy8efu6q+w2IfPW+Zy8YY3vfC/pcjn0xZ18TIT
CSsZYaXG9dCXFbxnrXaFQotKRnZuseccuwryrLV570KBeiXjXXTNP2tGBXn2
dUoPsxBVMAvjVeZ1+x0nPA96NEXNUFDBlCwYvtjCv5w8uxXcuKxuWMEkWdxq
f3trOXnezuWpSuTyaTgmKw0ZlllTyeeHufg1hK6TuosqyXMUF7/93oWBCWWV
5FmTi/+b/3jZtpZK8nyPiz9bEvXBvnsVeR7Axa/huzt/wOQqPg/Pxe+/JqNQ
cxOft5ws6OCqbljDfLZfqHbajc+n+XHxVhSHbl5rwOfTBtdm7hYJfz935ntq
J3ytIs/uPaflCi2qmT76B1x0wqvIc/vp3gYC9WqmdVVAzoxjVeT5yM72IRai
KkbDfsFShUVV5LnuvOkwQ0EV0y5gsMmKv6rIcwyXpyqQy6dpcfFPGnDuu3AG
n0/DUXfd4b++juDzaYO4+AvELm3ifnw+zY2Lv/ebjkM39Kkhzwpc/IIddwwT
+teQ50Nc/I6OHZef/auGPEu5+A8VDGnuOKyGPF9amNPPUCBmLNS1/17Tnc+n
dbCe7qRuKGaqdtkPY2JryfNRLl7JVrUi2ZFa8twQ19FYJKxllD1XOx6bXEue
jRv2ZQgtfn/Oe4cn7XzH5y3z+mfrC9RrGatzGhODw/m85WJGN9hCVMNE6lVO
qHXm85ZDuDzVHrl8mjsXv1G7zm8XXuDzae25+P+KuDjRcQ+fT8PRZHS4+Kmh
mDzXcfG/qzn200NHTJ63c/EftNbPMxgpJs85XPzLpO2PVPcQ8+sdXPyHjJuW
3H7H5y0bv925ayGSMu+ubDJSOcvn03aqd+llKJAyR6xnK+vp8fm0/GUWjuqG
UkYrYb/3jGYJeTbi4j0dmpfZ4YGEPL+8M2OLSChhBMFPV+aZSMizdqpPitBC
wqTljFodrCYhz/4fOk0XqEuYgiMy7W51fN7SlstTqcnl0xq4+Jt1YhqfFErJ
szEXv9ax8tiGB1LynMfFXzHbZM7y41LyjOPVGfMtTmyUkucYLv61iz4+StKR
kmctLv6h1gZrz3WTkmdfLn7Pdks19Ar4vOWSJ1aaAvV6ZkGHXcfrvtaRZ7Yw
39tCVMdsGtIxqvRhHXme0l6/m6GgjhGPS5HIjOvIc8D4e3bqhnXMxNzxD+8P
qCPPKly83Tbc1/+cISPPHo6W60VCGWM+sKd1/HkZee4YkJcgtJAxGcsV1ZyM
ZOS5gMtTFcnl04y4+O3Pakm01fh82ksufkf3GysmS+rIszYXf9+gqSbNT+rI
sz8X/0KDMwLH43XkGcfB3pnqcUvryLM7F3/UnWc92vrXkef2XPwnbSeWfs3i
85aBu2ZHCy0amQXTde9k7uDzaUMv+o4TqP/+nuJRXerVl8+nXRZ2vW4hamAm
mMryapMayHPnCuvOhoIGRtvlUIdWhwbybK/09pC6YQPjcJYt3TSlgTw3cfHG
bVJZ5l/P5y1Ntt5fJRLWM2m/9rVO9Ofzljpcnmq1XD4tgIt/ufOb5FZzPp+m
wsXfUcv/lXAsn0/z4OLflzj8+XZJA3nuyMX/+J8u4/oHNpBnWy7+DZ6u0X7m
DeSZ7msnrTPiCQ3k2ZiLX9l5n3BENZ+37Dyox2KRsJnpOvGfv+cJ+Xwa6uMm
/v1WefB+Pp/WtK9IXaDezPR90LYrUZXPp5lwdYZdSzUO3S1o4vPDL/zaGQqa
mD4fK4sK3JrI8xJJN2t1wyZGr9ZVw29eE3lmuXiHNhid1/3O5y09uPyVtlw+
rSMXv0pO9G4HIZ9Ps+XiPxCzMl9kwefTGrn4+yy9onlkNJ9P28nF//1FeLxd
RRN5zufi16zTbNfq1USejbj4D/vE33+wvok8h3H5luREleMv72YSP8vlVfIE
X33uJ+cQz+byCQLtlmGBl3OJt3F5g0OLV7kXrefrYRK4/JjHg/MtX9o/ZjB+
NMfX/9x0I3M5Xz+myv2euL+VTndROp0nlvt50dfoLpda+bwrjmrfLvh8U+Kv
S5HzHLNt3PDl4c3EaV+yuPz/5BWpPrOd0iKnFXwfA/xcNa04/OLVAzQO/M72
FwEdvG2o7gpcQZaSnXR7D9UJEb+jW6Fl9eMl6lrAK5pO2y6os6Z6afCVR/er
fh7mTPXS4B4+zTMr/xJQvTR4zJwcu6OF3lQvDW4d8fz1wwN+VC8NvnjH/MN/
JTymemnwlx26bH994jnVS4OvPlml3MfjBdVLg/dS9p0642gC1UuDR4YMv/y3
/huqM8f7dZfJNM/9Lx5TvSv4lKFdBmqeeEz15+CXls+7KHR5QvXn4Pcm1fpV
2IZQ/Tm45XADs1bnUPIJvrp3R9Oh35+ST8p3xmhcGPEzjHyCm55f8ePrg3Dy
CT7B00p5o0Y0+QRvH5U6bVkWSz7pumw61fSujyOf9P3dqG+wUflr8gmu1c74
07elb8gn+P3hgkr/u9lUb0zXJfBysf5SQJ4x/+Uumf0xoVsseQbfXn3tjaCV
Jc9U93e9ne+42bHkGfzDib8X1F+LJc/g/XNG2o0a+oo80/lTtEPVCl+RZ3CX
M4/a93obR57BCy4v6xU+JoE8g7fv9/niqppE8gy+gfVd+9eEZPIMHvl+YrNx
vzTyTPzsTol1cQZ5Bs/a2ydopTSbPBN/5aIkbswjz+CestOb3ykUkWd8nvQ5
X73M6WwyeQb3bnmSoHU0mTyDh5k/absQmUyewfVsvj9e0zuFPIP3Czp4yvSf
FPIMvvL5rPbblqWSZ/Bfuy2V/lqURp7BXf02zY11f0OewaPWn1O/xmSQZ3DV
dlm6H3SzyDN4dyc/9uvVbPIM3t9y3v5Z63PJM3j1AXufaWfyyTN44E7l2FF9
C8kzuFS7STO9v4g84/N5e5jzYgWPLPIMruR0JOzN2SzyTHVPigGnLQKyyDP4
SIudd71zssgzeG6gsnLnPtnkGdxT/4Nv2f5s8gz+3vrRX5+qs8kzuNhvV4cT
J3PIM/iIPTpvri/PJc/gW0o8NELX5pFn8Ak37WTu1/LJM7izjnXo1HFvyTO4
35zsCS6disgzuH7Ux5YeI0TkGXyxkVdo+yX8/m5832HMVa0cJXnkmZ5Tr//4
ZlGeR57BB0+tO1X4K488g3c3Lhl8eXQ+eQafFfLEJMgknzxT/U7ls5SuEfnk
GXxN9pQ3P8YWkGfwQu+sgHXRBeQZ3McrZ3as/VvyDD7g3ilfB7NC8kzxHOnt
LT5fRJ4pPz01STBXVkyewUuGNpT2OFRCnsFffyo33zq6jDyDt8we2j3Er5w8
4/tjfonh154L+fkE3x9/Ov27vx7+qf7xzcaaNbP5/h60Ts7t06e6duRpgpJV
arcUEcf4a3L79Gl8TMQFx5NFxDH+itz+fapfq13TPDmiiDjGt8rt68f4ZXuO
fj/SpZg4xl+U2++P8UvykzQYq2LiGJ8i1wcA49M7h6xv+VhMHOPHyfUHwPj+
K48tN7wnIo7xi+T6BmD89WzrPN0jJcQx/rZcPwGMVzRNMR1mU0oc4x/K9Rkg
nzN0Hh71LiOO8YZy/Qcw3s9q0JybxeXEMX6aXF8CjHe8cqJKdrWCOMY7yfUr
oOsd3nmLrHclcYyfwfUxwH2L5xuB+eQdvZQq6f4Et/ToEdjYie9jAC7pw/b+
pzffxwD820utGufhfB8D8GEX9Wp/6vB9DMCXRO5Y/nAN38cAfN7N2R/0jvF9
DMDzzo9aGv+I72MAbj+jr2BzUyX9fSl/uVswzl+P72NA+Tav8Uq9PPk+BuDB
quFbxn6qor8L+N6WCw5XV/N9DMA33564zewR38cAvEBV9euYeL5fAZ4Xk2c7
+1nq8/0NwB2z/I9mz+D7GICHhfnfM53H9zEAZ/orG0Qs5/sYgHexHZBpYMz3
MQCv2xC0fa9dNXkGN7WZODnwRjV5pvW3MIsXqXHV5JnqHZ7dbpzdWk2ewf3z
l786qcP3MQBfFDn58137GvIMXjXdqubu6xryDP55YlcX1a58HwPwv78mJB1e
yPcxALf86mAVr1tLnvH8PflJyMsCc76/Abi1sWJb2r5a8gy+eL6lw7SDteQZ
PNs5IabMvpY80/rYVKv03a615Bl8WZ+hZjKvWvIMzr4UC+vCaskzeC+XGTZp
ebXkGdzBvNLkws9a8kz7kJKG2cRNEJNn8MqDnRb2MBGTZ7pe1ceV5jfF5Blc
os1O/JwkJs+0vpq12XdFnZg8g9vu6tzlDsv3hUA+46XS6bBLN/j+BuADF5aN
UbkuIc/gM8OHvV53S0Kewee9mNO2876EPIOf9jQfVfdQQp7BrYI0p2RFScgz
jTdQzQp8IyHPVHfz8rPz8WoJeQaP1dp75r2ClDyDG03tdLmPmpQ8gxu0KKqO
NJSSZ/BJR3afCLCWkmfwc1HTz79zk5Jn8KIEt+u/bkrJM/i6attH4dv5vhDI
D8X7jFSe/orvbwDuNnPAgjWxMvIMntrzztoz8TLyTHyv51utJBl5Bl/02GBM
l3QZeQYvCwrY7JcvI8/gjXm54+IrZOQZvMvz9WU9mvl+EbR/aMQEt9R2deSZ
6sgS1kY/GVhHnsHFHt5jvk+qI8/gI7PeLR5lWEeeaZ3/cNU5rU115Bn804sW
tYjNdeQZPG6yXkOgSh15Rr7tU3BV2ZJKvr8B+DLfJfGlFXy/COL5MzT2VPH9
IsBfnYhLG1PD94ugfKkkctEXMd8vAvx4VH7HSfV8vwjwoKQLrQ9a+H4R4Lqd
DlfHtPL9IsBlY56767VrIM/g+gaNa126N5Bn8BNbp10dMriBPIPfe+f6dPvo
BvIMvubIvsV6mg3kGTy1tr/BLY0G8gw+f8CxwNDvfP8N5PMeWXwZUPiZ7wsB
LmhdqDKyle8LAW5f++HBzVa+LwS4o6lqD+MvfF8I8NKcdT2ufuX7QoD30Fr6
J98Ln+DL9Lc92PuT7wtB+dSoktRTCk3kE/xAo9TIrGMT+aR8qvX9KfmKTeST
1tNGTe59sUcT+aT9ASuf34xXbiKf4Mnnj1kN69ZEPsH19lXGHf3UyB7n8rzI
1+X1kyrJtEypjo3ylp63l7iMmkr1WJQXXR4wPPi2BXuQq78BV5ybZZ0XeJrV
5OpFwNf+6LxnwJDLLOoGwC25ekisc1N9YmlmWSevRyzWZcHPpOfM+BQdxmId
EXx4rlaVm3sMi3Uv8GtdB5YeiI9nsU4D/k3NYmKSdhqLdQW6rhXt6uZm57Ko
26O84qVHS3WXhLCoPwMvlS64kmgQyqJeClwv93ld+q2nLOp7wPf4rb62wyGM
9ee+f4ALuHrFjtx6LeUJtz533T4mjsX6KPjVVT1WaJYmsljPA3fs1b3UsyKV
xfoT+KnclV0sg7OonwB41opv3fbsz2dRP4d8V9J9o1kNdbEs6r3A93vn92q6
/IpFfRL4qYqiI+yRONaW+7wHF3P1gQXcOijV610vDTXZmc7qcOt24Jum/bPt
lySLFXDrTOBbNk6LtwjOZbGuAz7Xq2ZD+cK3LOrPkEc62Ld/THNzCot6KfCq
CGnH6cmpLNaPwcdy9XVY7wRfMmB7zeT1eSzW58Bn3H0fbBBUwGI9CXy0w6Qs
zRVFLOqxkG8RurfXm3Atm8W6KdUFcHVlWOcD73Vi+4jYp0Us1qUoLzTOZlsr
K2Kx/kfrgVx9FNarwHvvVJ1SureM3f4feQb//8gPDPuPPEDafzzvn5V7rnfj
nuvnyT2/J3PP79vlnrsXcs/dMf/xfD1E7jk6mHuOtpV7Xl7MPS8XyD0XT+We
i3Xknn9Pc8+/WM/D828eV+/k8R/Pv3bc/InntV97um7Z0KuSfcfNn/R8tyzh
tO3QShb1oOCDojyWiSZXsqhfBF/VT3+OzopKFvV24Jf85xRYHq5kUU8DrnDM
bf3USVUs6j/Azc1ck+6eqWJRrwB+dKKCtKi2isX6Ovg5478KVX4/J2I9GLzX
nc/6F25Vs1i/BD+x+rwoJaqafcXNk3iecusoM1u9pJpFnSu4R5ypW8ct1Szq
MsEVfAaq1h2oZlFHCL72VnJtB49qFvUr4Pct609M/buGRb0FuNRB++S3xzUs
6gPARbt9g4s/1LBYzwZXdvRf1G1yLWvHzYfgrnWz2r+bUss6cfMhnl/W7Ih7
MPt0Lctw8yF4lqP3nP6etSzqHcG7em+w8Q6sZVH/Ae7z8pH3nJNiFvUK4HXe
6qmbQsUs1tfBpwS4zSsuELNYDwYX16WYVSWL2THcvEf7O+/P6jrvqYSt5uY9
8JW5GzTDXklY1EOAu7cUmd20l7JYvwf3Vjjd2O+SlMV6M63zZzQcPmwnZdO4
+Q3fh3N2VkwaXShjsd4Pvm10vqTbxjoW69Pgy0af3nBlYR2L9Wl83/NcsHFu
q1oDu5Tj+JyKuiOQbn+fwxZy58H8X8mcGeuxq4DFejOtz96ofTXf04bWj6k+
lFtfxvci8A/6Tcbh2Xks1qfxfYkpf3zscCvfRwv8wg6njYd+c6w3gx8fvOa+
4u/vpXVcHSfO33VzyNzeJ73Yjdx+D1zXuJVjv4x6/JhFfSr4gONPnjI/hCzq
KcGX2a79fMcwivXm9ofAw+FRQ77UdoplsU8DXDr6m/6i8bEs6lzBfXeauzQ/
iGdjuLpG8HOFhzIlMxPZIVwdHvhspUEP9JclscXc/hN83h2aWHjQ/EQyW8Lt
owC3fvH0y/o7ySz2M4C3f25/JbQmmUV9LXj+ljHdH0jSWNSDgn+pM9t43ySd
Rf0irR+9dd6isSKTRb0d1fnPHOS4oiKL1eX2w+Bzc4N78Pfi81ks9qXQes1Z
sa7sehaL/RXg73bEqfmFZ7HYJwB+KPLo2RuyLBZ1wOC3u4WcGqOSw6JuFTxa
b1+ZRmoOizpL8ITgKWM/BeWyqAsEd1vlc/mvxDwWdWzgPnfn/rirWsBGcPt5
8Pnuu05YsbMyjx3K7UsBV3Qffb9z4+/vLdz+CvDxIQ/ejlDMZ7HPAfzaQKer
ITr5LOrywd+zug/ijuSzqG+mfRJ700pajApY1OOCfzqQ6GoqKWBRP0p9SgY4
NcSHvGVR7whedOFjeu+gQhb1eeD9xky1UHlbxKKeDHxkUEP/lTNFrA33PsXn
dVsSs24GU8RiHxF4gM6zPa6Li1jshwHXcL9b/nNHEYt9HeCfTbpHx7sUsdiH
AL5qkle3ka+KWNTNg4cts5mq3LeYRf0x+OQ992Z3URSxqJcFtxl59qNGpIhF
fSe4aGr4hn6XS1jUI4JbLtddGX25lEX9HPiQTf0/7YsuY1HvBT7d3OXk+m/l
bEdu3sHntZCr05Zw8xJ4aYRe7jr/SnYxNy/hc7BIraZv0Mxq1oibd8DLuXrj
TG6+Ar8ePKreTljNon4a/PmPu6HOkmr2LDdf0X6XTgLruv21rA43X4HPPvL2
brxVLSvg5h36vOPqbMO5eQz8s9msAbte17Koz6Y86uIkU836Whb1xOBPj0zZ
+vdAMZvEzWP4XKuKqHALui5hc7l5DPx6aFvgr5sS1oSbx8BflzxZ7uMjYfdz
8w74Sa4e1Yeb38B3Zw70ccqXsKgXB7+U/ClWqVnCor4ZPOL150m7e0pZ1OOC
O/iO3fZ9spTtyc1vlH/zKO/lEitjbbj5Dfzbj7i7Ua9k7GRufgMv+VLasDZB
xvpz8xuNj3GtckiWscrcvAOuxNV5nuXmPfDgeXe9flbLWNS7U15u3USDLR9k
LOqzaV++4Qj/ys51LOqJwaULrMduHFrHov4V/GjY3oCWqXXsRW7ew/eEk+1X
LhJX1LPYjwdePuRrx52V9Ww7bt4DvxW80Mu1qp49ys174Jn6mZONa+rZVm7e
A2817VO8UlLPPuPmHfBYrt7SipsPwSPGFLctelfPor4fPF/cJcXnSz2LenTw
++/bj9jYvoFF/TR41qa26JQeDSzqfel6g96H2AxpYFGfCr677LGHwpgGFvV8
1J/D5t/8GOr2wHdx+TEjbt4DT8v5Nz+G/Uvg/t3/zY9hvw34vfb/5sc2Yf8m
x1W4ekjsZwAv8d38Jz+G+ntwyYPCP/kx1IuDfx1S8yc/hvpmcIN+t//kx1CP
C97/H60/+THUj4LHb3v9Jz+mgvo27I/n6gnxvYryVGNsBj2IM6PvPVSHGJpZ
Ef5sJV8Hw/FBfd2N13TaSXWklAd7VagbGWhHdaTgJzUnsJ1GuVIdKbhG1KTM
dTaeVC9K/R3l9gvT+S89sLhg/ZDqRcF339F2ctd5RvWi4E2SYRm6rpFULwqu
Oc7q4f26WKoXBTd6MLouaNdrqqelPi5ShbsCnRCqpwVPWZd1M+NXCHkAX2P3
PdCk91PyQH3+5g5Ldz32jN9fyfFBZ/frr1r9nK6X6gDl9vOCb+sytvJE0iu6
XvCkrc9PZlYm0PWCuz1Snt9cnkzXCz6k+nBAyze+DxvVg2WqHFxdHEvXBe6w
5JWF7q5XdF3g5j6Hp1wZGEfXBf5+oN7KAoV4ih+8Rm7/LLh7UOP5K/VvKH5w
N8XOCg4rsyh+8I7JqmFx3jkUJ65jiJ15Y4+iFIoTfJCup1Pq+VSKk+rT7ql0
nW2VRvGAj5Hbfwqubz7wtc6APIoH/MI54aiYgQVUf0v9wEwHdhxrUkjxIN7g
5NWKL1yzKR7wk74dPEM+Z9PrUr8luf2Y4HP2TRrV3q6IXpf27ekc2td5vYjO
j9c39T6R93kG349OPp+G81D/0fEKWSqLyqh+GHmh+XL9u8Bb5Pp3gRfI9e8C
fy3Xvws8U65/F/hWuf5d4M+5/l2YT8Cr5fp0gTNyfbooLyfXpwvcXq5PF3il
XJ8u8BS5Pl3yeTZa/+DGF3H9u+AT19H3m7LuMmW+Txf4A/3bwX2G8H26wMXj
3MzaxvF9usAd7w2InzGH79MFvv1Swo1V2/g+XeD3bpts3XWK3ydI+wejfJze
jeL7bhEfNnfVVGu+7xb4qb5+2dvjqsgb+JqMFWqXB/B9t8Bj9gf9UDDl+26B
p49K91z9hN/3h7g8lz5LVjDi+2iBp7p+Glq/nu+jBe7U7Vprf7Nq8gC+K0Gp
suVkNXkAT3Q45XfkdjVdL/UTChptPHAbv18PXPHy2Qdnb/H79cDLMpQWzy7l
9+uBBy6efWDwQL7/FfiVEUX+O+bx++zI956tfRJP8vvsqB7ju9rDDPdafj8j
x/82bKe3+24tXRf4NfvtWmYRtRQ/9UN6MmrjNHt+Hxx4g+6nGZoB/D448E5X
5neoT+f3wYFfyZ+lfaea76+F1zHdoZTdEMLvUwOvmB2vNfclv08N/Jt62byK
DL7PFfjny/rNQ235/WLgnttKZ3Zw5/eLUT/UpJoV3gK+3xTON97e1nPlW37f
FviEFYP8Cir5/k7g8+s1bSw28vukqM9ZRVp5u7V1dB78Xmjr0++tLXyfIvDH
/cMNjH5/j8X7H/NrCLevAf/HfLBg05PE5OePaTytr3F1+3ifUP6wU9XpH5FR
dB7q8/fP2U+ZSrE0z4A/WPogTDA5ls4vX6+O84MXBR1ervkwkeYH8AVe7ypc
riXR6+L+rXuqeu34mWR6Xaon7z/Z4d3DZHpfg3vUFP9V3JxM8YCf5eq6EQ94
8rFTkzcrZlA84B/NBX+7lmfS+xd8wsVjPb/14Ptv4D6Nf8jmfHHPojjBx838
eG36vSyKE7yj9tRzlnFZ9D4FXyucpP/kHd/PQb5emuZ3jttsO2imopNL8YMP
+5G99U2vPIof/LplfJjh4Hx6/4L/PerpGrd1/L4k3O/2ppEP7cR5dF30fSRg
TZXkSx5dF7iKYpztpoH5dF3U5/jT5eIO8/PpfU111NZL2/ue4vsqyNct43rB
H/c7dmeyyVu6Xqp/HnY6Rja+kK4XvGu7DlMdhhTR9RKfEG3nqF1M8wN4f6lB
uJWViDzgfWnxy7xz/wVFfN8Pjquei5NuXV9EHuh7urPKBY+DReSB5qcjnT+/
9yoiD+BdGLWVevl8f1fwrR/u39VT5fshyNfxwg+4S+3JMsZNRH7ATdfHjtaZ
U0J+wHcq3Qqa3aeU/ICHvG1Q6t6xjPyAG6Tr2dr2KKf5DbzCLVI2oi/f90A+
n4b4wXO5+lKMR7yrJodt7a7P79+Xz6fhPOAbuPpJnBf854Y7mTub+X6buO4v
ad+/njWvpfODz+2148o/1vy+dfl8Gl5Xvp4Qrwse97xwt3lrLfkn3/pvdzkP
E1M88H32+Y19629IKB5wMx3l6RreEooHXNwv79ulO/z+bvl8GuKUr8dDnOBh
BUZrbX9I+H5Q6C/767hx9CB+Xza4RXhFj0RdKcWP+6J3h6rD/V7JKH7wUr3K
ovQ4GcVP/R4ujA61S5RR/JQ3i3q0/u8UGb0v5PNpuC75+jdcF/jWHROOS37x
/RvBr3ztLv7ci98HDX7q5yn3N+r8PmjwdBfh96QZdXS9uN9N9Ixa9Cr5vovg
jg0ZQ5Wr+L6L4E+uOE/ZWM33XQRvTj3oPquW77sIfvGzduUnCd93UT6fBg/g
07j6NNp3z/E97+Ms0tr4/eDg2x0+Xh+h1EAewMeIhow72Jffpwx+YaB9qPkw
fp8y5T/SvxTpjG0gP/+VLwIPcNX63L2V399N+8V9jX/taeX7JYLvbOps0NzK
90sEN3e39Yn4wvdLBJ+32jY0+yvfL1E+zwZv8nVo8AYeYdh3Yy+FJn4fPZ5r
R3pFLOnQRN7ARyxVHNzSuYm8UTznIkTFXfl9x+Tn5PpmzZ78vmNw7d3jIgcr
8/tzad3TuE9teyN+fzHlVa6K+h7ax/etwt+hhFu3/X9oyFxR
   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJwtmXfgT9Ubxz+fe84lMrOzRRmJElIystKwV6hESktGGsiMNMz2lrbQlsgu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         "]], Polygon3DBox[CompressedData["
1:eJw1mgf8l1P/xu8zSksqWYmmtsrm8UdGZbczoonHLlsSLTKzZURLpUQqQtpF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         "]], Polygon3DBox[CompressedData["
1:eJwt13f8luMaAPDf733fhnaOjCKcBqGEhnZpl5b2JgqVfpEmFdLUkIbVlGQ7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         "]]}]}, {}, {}, {}, {}}, {
     {GrayLevel[0], Line3DBox[CompressedData["
1:eJwt0kkuREEAgOEyz3RHxI7YknYA4QDSVlbCzkZYsJN2AGEv4QRcwHABvTVr
85yIiCkSsxi+Cou/v+qk3qt6r15T/0j3cF4IYVxviuMnXWopP4QEs2zkFlt5
wQ4+souhIIQ+JJjUkHGKY2znJNOcYS/nOMhFZpjlC6/5EdeJ/3WvFRva5ipz
XOMO17nLDe5xk/vx2jhfB8Y5HnKHR9zlMfd4wn2e8oBnnPc8N/zUuV71oCl7
24zviMsc5QIHOMseTrOTE2xjhi1s1qBxFXv5bY10vC/becZUfA42xHdv3hXf
VaNn3aneHqtZxyrWspJJVsS5LI/rsIwVLGUpS2LGRSyOx8WieNYs5M/fEYYv
2UZY8HP7/z38AsN+Sdc=
       "]]}, {
      Line3DBox[{694, 1010, 475, 693, 1119, 916, 695, 1120, 917, 696, 1121, 
       918, 697, 1122, 919, 698, 1123, 920, 699, 1124, 1016, 1221, 700, 1125, 
       921, 701, 1126, 922, 702, 1127, 923, 703, 1128, 924, 704, 1129, 925, 
       705, 1130, 926, 706, 1328, 1011, 927, 1012}], 
      Line3DBox[{708, 1017, 1222, 707, 490, 709, 1131, 928, 710, 1132, 929, 
       711, 1133, 930, 712, 1134, 931, 713, 1135, 1018, 1223, 714, 1019, 1224,
        715, 1136, 932, 716, 1137, 933, 717, 1138, 934, 718, 1139, 935, 719, 
       1140, 936, 720, 1326, 1008, 937, 721}], 
      Line3DBox[{723, 1020, 1225, 722, 1021, 1226, 724, 505, 725, 1141, 938, 
       726, 1142, 939, 727, 1143, 940, 728, 1144, 1022, 1227, 729, 1023, 1228,
        730, 1024, 1229, 731, 1145, 941, 732, 1146, 942, 733, 1147, 943, 734, 
       1148, 944, 735, 1327, 1009, 945, 736}], 
      Line3DBox[{738, 1025, 1230, 737, 1026, 1231, 739, 1027, 1232, 740, 520, 
       741, 1149, 946, 742, 1150, 947, 743, 1151, 1028, 1233, 744, 1029, 1234,
        745, 1030, 1235, 746, 1031, 1236, 747, 1152, 948, 748, 1153, 949, 749,
        1154, 950, 750, 1155, 951, 751}], 
      Line3DBox[{753, 1032, 1237, 752, 1033, 1238, 754, 1034, 1239, 755, 1035,
        1240, 756, 536, 757, 1156, 952, 758, 1157, 1036, 1241, 759, 1037, 
       1242, 760, 1038, 1243, 761, 1039, 1244, 762, 1040, 1245, 763, 1158, 
       953, 764, 1159, 954, 765, 1160, 955, 766}], 
      Line3DBox[{768, 1041, 1246, 767, 1042, 1247, 769, 1043, 1248, 770, 1044,
        1249, 771, 1045, 1250, 772, 552, 773, 1161, 1046, 1251, 774, 1047, 
       1252, 775, 1048, 1253, 776, 1049, 1254, 777, 1050, 1255, 778, 1051, 
       1256, 779, 1162, 956, 780, 1163, 957, 781}], 
      Line3DBox[{785, 1164, 958, 783, 1165, 959, 787, 1166, 960, 789, 1167, 
       961, 791, 1168, 962, 793, 1169, 963, 795, 1170, 569, 797, 1171, 964, 
       799, 1172, 965, 801, 1173, 966, 803, 1174, 967, 805, 1175, 968, 807, 
       1176, 969, 809, 1178, 971, 811}], 
      Line3DBox[{810, 970, 1177, 808, 1269, 1063, 806, 1268, 1062, 804, 1267, 
       1061, 802, 1266, 1060, 800, 1265, 1059, 798, 1264, 1058, 796, 1263, 
       568, 794, 1262, 1057, 792, 1261, 1056, 790, 1260, 1055, 788, 1259, 
       1054, 786, 1258, 1053, 782, 1257, 1052, 784}], 
      Line3DBox[{813, 1064, 1270, 812, 1179, 972, 814, 1180, 973, 815, 1181, 
       974, 816, 1182, 975, 817, 1183, 976, 818, 1184, 1065, 1271, 819, 585, 
       820, 1185, 977, 821, 1186, 978, 822, 1187, 979, 823, 1188, 980, 824, 
       1189, 981, 825, 1190, 982, 826}], 
      Line3DBox[{828, 1066, 1272, 827, 1067, 1273, 829, 1191, 983, 830, 1192, 
       984, 831, 1193, 985, 832, 1194, 986, 833, 1195, 1068, 1274, 834, 1069, 
       1275, 835, 601, 836, 1196, 987, 837, 1197, 988, 838, 1198, 989, 839, 
       1199, 990, 840, 1200, 991, 841}], 
      Line3DBox[{843, 1070, 1276, 842, 1071, 1277, 844, 1072, 1278, 845, 1201,
        992, 846, 1202, 993, 847, 1203, 994, 848, 1204, 1073, 1279, 849, 1074,
        1280, 850, 1075, 1281, 851, 617, 852, 1205, 995, 853, 1206, 996, 854, 
       1207, 997, 855, 1208, 998, 856}], 
      Line3DBox[{858, 1076, 1282, 857, 1077, 1283, 859, 1078, 1284, 860, 1079,
        1285, 861, 1209, 999, 862, 1210, 1000, 863, 1211, 1080, 1286, 864, 
       1081, 1287, 865, 1082, 1288, 866, 1083, 1289, 867, 633, 868, 1212, 
       1001, 869, 1213, 1002, 870, 1214, 1003, 871}], 
      Line3DBox[{873, 1084, 1290, 872, 1085, 1291, 874, 1086, 1292, 875, 1087,
        1293, 876, 1088, 1294, 877, 1215, 1004, 878, 1216, 1089, 1295, 879, 
       1090, 1296, 880, 1091, 1297, 881, 1092, 1298, 882, 1093, 1299, 883, 
       649, 884, 1217, 1005, 885, 1218, 1006, 886}], 
      Line3DBox[{888, 1094, 1300, 887, 1095, 1301, 889, 1096, 1302, 890, 1097,
        1303, 891, 1098, 1304, 892, 1099, 1305, 893, 1219, 1100, 1306, 894, 
       1101, 1307, 895, 1102, 1308, 896, 1103, 1309, 897, 1104, 1310, 898, 
       1105, 1311, 899, 665, 900, 1220, 1007, 901}], 
      Line3DBox[{915, 1015, 689, 914, 1325, 1117, 913, 1324, 1116, 912, 1323, 
       1115, 911, 1322, 1114, 910, 1321, 1113, 909, 1320, 1112, 908, 1319, 
       1318, 1111, 907, 1317, 1110, 906, 1316, 1109, 905, 1315, 1108, 904, 
       1314, 1107, 903, 1313, 1106, 902, 1118, 1312, 1013, 1014}]}, {
      Line3DBox[{253, 476, 1119, 254, 490, 282, 1226, 504, 297, 1231, 518, 
       312, 1238, 533, 327, 1247, 548, 342, 1258, 563, 1165, 357, 578, 1179, 
       372, 1273, 593, 387, 1277, 608, 402, 1283, 623, 417, 1291, 638, 432, 
       1301, 653, 447, 1313, 668, 462}], 
      Line3DBox[{255, 477, 1120, 256, 491, 1131, 283, 505, 298, 1232, 519, 
       313, 1239, 534, 328, 1248, 549, 343, 1259, 564, 1166, 358, 579, 1180, 
       373, 594, 1191, 388, 1278, 609, 403, 1284, 624, 418, 1292, 639, 433, 
       1302, 654, 448, 1314, 669, 463}], 
      Line3DBox[{257, 478, 1121, 258, 492, 1132, 284, 506, 1141, 299, 520, 
       314, 1240, 535, 329, 1249, 550, 344, 1260, 565, 1167, 359, 580, 1181, 
       374, 595, 1192, 389, 610, 1201, 404, 1285, 625, 419, 1293, 640, 434, 
       1303, 655, 449, 1315, 670, 464}], 
      Line3DBox[{259, 479, 1122, 260, 493, 1133, 285, 507, 1142, 300, 521, 
       1149, 315, 536, 330, 1250, 551, 345, 1261, 566, 1168, 360, 581, 1182, 
       375, 596, 1193, 390, 611, 1202, 405, 626, 1209, 420, 1294, 641, 435, 
       1304, 656, 450, 1316, 671, 465}], 
      Line3DBox[{261, 480, 1123, 262, 494, 1134, 286, 508, 1143, 301, 522, 
       1150, 316, 537, 1156, 331, 552, 346, 1262, 567, 1169, 361, 582, 1183, 
       376, 597, 1194, 391, 612, 1203, 406, 627, 1210, 421, 642, 1215, 436, 
       1305, 657, 451, 1317, 672, 466}], 
      Line3DBox[{263, 481, 1124, 265, 495, 1135, 287, 509, 1144, 302, 523, 
       1151, 317, 538, 1157, 332, 553, 1161, 347, 568, 1170, 362, 583, 1184, 
       377, 598, 1195, 392, 613, 1204, 407, 628, 1211, 422, 643, 1216, 437, 
       658, 1219, 452, 1318, 673, 467}], 
      Line3DBox[{267, 483, 1125, 268, 1224, 497, 289, 1228, 511, 304, 1234, 
       525, 319, 1242, 540, 334, 1252, 555, 349, 1264, 570, 1171, 364, 585, 
       379, 1275, 600, 394, 1280, 615, 409, 1287, 630, 424, 1296, 645, 439, 
       1307, 660, 454, 1320, 675, 469}], 
      Line3DBox[{269, 484, 1126, 270, 498, 1136, 290, 1229, 512, 305, 1235, 
       526, 320, 1243, 541, 335, 1253, 556, 350, 1265, 571, 1172, 365, 586, 
       1185, 380, 601, 395, 1281, 616, 410, 1288, 631, 425, 1297, 646, 440, 
       1308, 661, 455, 1321, 676, 470}], 
      Line3DBox[{271, 485, 1127, 272, 499, 1137, 291, 513, 1145, 306, 1236, 
       527, 321, 1244, 542, 336, 1254, 557, 351, 1266, 572, 1173, 366, 587, 
       1186, 381, 602, 1196, 396, 617, 411, 1289, 632, 426, 1298, 647, 441, 
       1309, 662, 456, 1322, 677, 471}], 
      Line3DBox[{273, 486, 1128, 274, 500, 1138, 292, 514, 1146, 307, 528, 
       1152, 322, 1245, 543, 337, 1255, 558, 352, 1267, 573, 1174, 367, 588, 
       1187, 382, 603, 1197, 397, 618, 1205, 412, 633, 427, 1299, 648, 442, 
       1310, 663, 457, 1323, 678, 472}], 
      Line3DBox[{275, 487, 1129, 276, 501, 1139, 293, 515, 1147, 308, 529, 
       1153, 323, 544, 1158, 338, 1256, 559, 353, 1268, 574, 1175, 368, 589, 
       1188, 383, 604, 1198, 398, 619, 1206, 413, 634, 1212, 428, 649, 443, 
       1311, 664, 458, 1324, 679, 473}], 
      Line3DBox[{277, 488, 1130, 278, 502, 1140, 294, 516, 1148, 309, 530, 
       1154, 324, 545, 1159, 339, 560, 1162, 354, 1269, 575, 1176, 369, 590, 
       1189, 384, 605, 1199, 399, 620, 1207, 414, 635, 1213, 429, 650, 1217, 
       444, 665, 459, 1325, 680, 474}], 
      Line3DBox[{279, 686, 1328, 687, 280, 681, 1326, 682, 295, 683, 1327, 
       684, 310, 531, 1155, 325, 546, 1160, 340, 561, 1163, 355, 576, 1177, 
       1178, 370, 591, 1190, 385, 606, 1200, 400, 621, 1208, 415, 636, 1214, 
       430, 651, 1218, 445, 666, 1220, 460, 689, 690, 691}], 
      Line3DBox[{461, 667, 688, 1312, 446, 652, 1300, 431, 637, 1290, 416, 
       622, 1282, 401, 607, 1276, 386, 592, 1272, 371, 577, 1270, 356, 1164, 
       562, 1257, 341, 547, 1246, 326, 532, 1237, 311, 517, 1230, 296, 503, 
       1225, 281, 489, 1222, 252, 475, 685, 692}], 
      Line3DBox[{468, 674, 1319, 453, 659, 1306, 438, 644, 1295, 423, 629, 
       1286, 408, 614, 1279, 393, 599, 1274, 378, 584, 1271, 363, 569, 1263, 
       348, 554, 1251, 333, 539, 1241, 318, 524, 1233, 303, 510, 1227, 288, 
       496, 1223, 266, 482, 1221, 264}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0vHc4V338P242FLJFZiFRlCLzqUiS2UBEGVH2iERlFFL2SmST7D3Kziay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    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.5755448532322035`*^9, 3.586581478694872*^9, 
  3.5873808550803475`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"sxyi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{3.5054112035119495`*^9}],

Cell[BoxData[
 Graphics3DBox[GraphicsComplex3DBox[CompressedData["
1:eJx1nQlUTV34/2/InAwhU5rMQyQy1U1IJZUhVKSSMU3SKIlKpfk2p3kms8rc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   "], {{
     {EdgeForm[None], GraphicsGroup3DBox[{Polygon3DBox[CompressedData["
1:eJwtmXncTVUbht+z19qSeRYhY0RFRYY0SELGSCVDhqQyK01KUZG5UJJoQJNG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         "]], Polygon3DBox[CompressedData["
1:eJwtmnfgT9Ufxu8551qJSKQiO8mIpKW0jChEC6WkEo1fGqg0ZDU0EO1hNCkR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         "]], Polygon3DBox[CompressedData["
1:eJwt13f8llMfB/Df777v0i4VhdBWaWsP2km7tBVt0ZC0jNAUKmnrIZUVpcdM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         "]]}]}, {}, {}, {}, {}}, {
     {GrayLevel[0], Line3DBox[CompressedData["
1:eJwt0rsrhWEcwPHH/bhLQhL5AygyMsjIICOD2BgwMsgkBjNl5y9wrAZGi7vD
OS7lEhFJci2fpwzf8/md3rf3+b2n0zQy0T+eFUKY07fi/KFrbWaHUMEdNvKU
LbxnZ7yPvSzICWGQNazVmLmd0+zmAvu5zGGucZJJznKbP3yIal+fetauhQ64
x8N4jUc84DEPecIjpngcv+vUnOJZ3JtpnjHDNM+Z4QXPecmk93nkr670pRet
2G2Pi9ziDNc5zlUOcYl9nGcXp9jGVo2aqzjAPPbw3VkdvGMzT9gQf1PXb/mm
cr3qRvV2LGMdS1nLElazOD6fRaxkISuYYGm8TwXmIuYzwTzmM5e5dGzIpjXC
ho+n///DH2oiSDQ=
       "]]}, {
      Line3DBox[{703, 1015, 479, 702, 1131, 923, 704, 1132, 924, 705, 1133, 
       925, 706, 1134, 926, 707, 1135, 927, 708, 1136, 1027, 1233, 709, 1137, 
       928, 710, 1138, 929, 711, 1139, 930, 712, 1140, 931, 713, 1141, 932, 
       714, 1130, 1142, 933, 1024, 1340, 1025, 1026, 1023, 934, 1016}], 
      Line3DBox[{716, 1028, 1234, 715, 493, 717, 1143, 935, 718, 1144, 936, 
       719, 1145, 937, 720, 1146, 938, 721, 1147, 1029, 1235, 722, 1030, 1236,
        723, 1148, 939, 724, 1149, 940, 725, 1150, 941, 726, 1151, 942, 727, 
       1338, 1017, 943, 1022, 1339, 1018, 944, 728}], 
      Line3DBox[{730, 1031, 1237, 729, 1032, 1238, 731, 507, 732, 1152, 945, 
       733, 1153, 946, 734, 1154, 947, 735, 1155, 1033, 1239, 736, 1034, 1240,
        737, 1035, 1241, 738, 1156, 948, 739, 1157, 949, 740, 1158, 950, 741, 
       1159, 951, 742, 1160, 952, 743}], 
      Line3DBox[{745, 1036, 1242, 744, 1037, 1243, 746, 1038, 1244, 747, 523, 
       748, 1161, 953, 749, 1162, 954, 750, 1163, 1039, 1245, 751, 1040, 1246,
        752, 1041, 1247, 753, 1042, 1248, 754, 1164, 955, 755, 1165, 956, 756,
        1166, 957, 757, 1167, 958, 758}], 
      Line3DBox[{760, 1043, 1249, 759, 1044, 1250, 761, 1045, 1251, 762, 1046,
        1252, 763, 539, 764, 1168, 959, 765, 1169, 1047, 1253, 766, 1048, 
       1254, 767, 1049, 1255, 768, 1050, 1256, 769, 1051, 1257, 770, 1170, 
       960, 771, 1171, 961, 772, 1172, 962, 773}], 
      Line3DBox[{775, 1052, 1258, 774, 1053, 1259, 776, 1054, 1260, 777, 1055,
        1261, 778, 1056, 1262, 779, 555, 780, 1173, 1057, 1263, 781, 1058, 
       1264, 782, 1059, 1265, 783, 1060, 1266, 784, 1061, 1267, 785, 1062, 
       1268, 786, 1174, 963, 787, 1175, 964, 788}], 
      Line3DBox[{792, 1176, 965, 790, 1177, 966, 794, 1178, 967, 796, 1179, 
       968, 798, 1180, 969, 800, 1181, 970, 802, 1182, 572, 804, 1183, 971, 
       806, 1184, 972, 808, 1185, 973, 810, 1186, 974, 812, 1187, 975, 814, 
       1188, 976, 816, 1190, 978, 818}], 
      Line3DBox[{817, 977, 1189, 815, 1281, 1074, 813, 1280, 1073, 811, 1279, 
       1072, 809, 1278, 1071, 807, 1277, 1070, 805, 1276, 1069, 803, 1275, 
       571, 801, 1274, 1068, 799, 1273, 1067, 797, 1272, 1066, 795, 1271, 
       1065, 793, 1270, 1064, 789, 1269, 1063, 791}], 
      Line3DBox[{820, 1075, 1282, 819, 1191, 979, 821, 1192, 980, 822, 1193, 
       981, 823, 1194, 982, 824, 1195, 983, 825, 1196, 1076, 1283, 826, 588, 
       827, 1197, 984, 828, 1198, 985, 829, 1199, 986, 830, 1200, 987, 831, 
       1201, 988, 832, 1202, 989, 833}], 
      Line3DBox[{835, 1077, 1284, 834, 1078, 1285, 836, 1203, 990, 837, 1204, 
       991, 838, 1205, 992, 839, 1206, 993, 840, 1207, 1079, 1286, 841, 1080, 
       1287, 842, 604, 843, 1208, 994, 844, 1209, 995, 845, 1210, 996, 846, 
       1211, 997, 847, 1212, 998, 848}], 
      Line3DBox[{850, 1081, 1288, 849, 1082, 1289, 851, 1083, 1290, 852, 1213,
        999, 853, 1214, 1000, 854, 1215, 1001, 855, 1216, 1084, 1291, 856, 
       1085, 1292, 857, 1086, 1293, 858, 620, 859, 1217, 1002, 860, 1218, 
       1003, 861, 1219, 1004, 862, 1220, 1005, 863}], 
      Line3DBox[{865, 1087, 1294, 864, 1088, 1295, 866, 1089, 1296, 867, 1090,
        1297, 868, 1221, 1006, 869, 1222, 1007, 870, 1223, 1091, 1298, 871, 
       1092, 1299, 872, 1093, 1300, 873, 1094, 1301, 874, 636, 875, 1224, 
       1008, 876, 1225, 1009, 877, 1226, 1010, 878}], 
      Line3DBox[{880, 1095, 1302, 879, 1096, 1303, 881, 1097, 1304, 882, 1098,
        1305, 883, 1099, 1306, 884, 1227, 1011, 885, 1228, 1100, 1307, 886, 
       1101, 1308, 887, 1102, 1309, 888, 1103, 1310, 889, 1104, 1311, 890, 
       652, 891, 1229, 1012, 892, 1230, 1013, 893}], 
      Line3DBox[{895, 1105, 1312, 894, 1106, 1313, 896, 1107, 1314, 897, 1108,
        1315, 898, 1109, 1316, 899, 1110, 1317, 900, 1231, 1111, 1318, 901, 
       1112, 1319, 902, 1113, 1320, 903, 1114, 1321, 904, 1115, 1322, 905, 
       1116, 1323, 906, 668, 907, 1232, 1014, 908}], 
      Line3DBox[{922, 1021, 691, 921, 1337, 1128, 920, 1336, 1127, 919, 1335, 
       1126, 918, 1334, 1125, 917, 1333, 1124, 916, 1332, 1123, 915, 1331, 
       1330, 1122, 914, 1329, 1121, 913, 1328, 1120, 912, 1327, 1119, 911, 
       1326, 1118, 910, 1325, 1117, 909, 1129, 1324, 1019, 1020}]}, {
      Line3DBox[{258, 480, 1131, 259, 493, 286, 1238, 506, 301, 1243, 521, 
       316, 1250, 536, 331, 1259, 551, 346, 1270, 566, 1177, 361, 581, 1191, 
       376, 1285, 596, 391, 1289, 611, 406, 1295, 626, 421, 1303, 641, 436, 
       1313, 656, 451, 1325, 671, 466}], 
      Line3DBox[{260, 481, 1132, 261, 494, 1143, 287, 507, 302, 1244, 522, 
       317, 1251, 537, 332, 1260, 552, 347, 1271, 567, 1178, 362, 582, 1192, 
       377, 597, 1203, 392, 1290, 612, 407, 1296, 627, 422, 1304, 642, 437, 
       1314, 657, 452, 1326, 672, 467}], 
      Line3DBox[{262, 482, 1133, 263, 495, 1144, 288, 508, 1152, 303, 523, 
       318, 1252, 538, 333, 1261, 553, 348, 1272, 568, 1179, 363, 583, 1193, 
       378, 598, 1204, 393, 613, 1213, 408, 1297, 628, 423, 1305, 643, 438, 
       1315, 658, 453, 1327, 673, 468}], 
      Line3DBox[{264, 483, 1134, 265, 496, 1145, 289, 509, 1153, 304, 524, 
       1161, 319, 539, 334, 1262, 554, 349, 1273, 569, 1180, 364, 584, 1194, 
       379, 599, 1205, 394, 614, 1214, 409, 629, 1221, 424, 1306, 644, 439, 
       1316, 659, 454, 1328, 674, 469}], 
      Line3DBox[{266, 484, 1135, 267, 497, 1146, 290, 510, 1154, 305, 525, 
       1162, 320, 540, 1168, 335, 555, 350, 1274, 570, 1181, 365, 585, 1195, 
       380, 600, 1206, 395, 615, 1215, 410, 630, 1222, 425, 645, 1227, 440, 
       1317, 660, 455, 1329, 675, 470}], 
      Line3DBox[{268, 485, 1136, 270, 498, 1147, 291, 511, 1155, 306, 526, 
       1163, 321, 541, 1169, 336, 556, 1173, 351, 571, 1182, 366, 586, 1196, 
       381, 601, 1207, 396, 616, 1216, 411, 631, 1223, 426, 646, 1228, 441, 
       661, 1231, 456, 1330, 676, 471}], 
      Line3DBox[{272, 487, 1137, 273, 1236, 500, 293, 1240, 513, 308, 1246, 
       528, 323, 1254, 543, 338, 1264, 558, 353, 1276, 573, 1183, 368, 588, 
       383, 1287, 603, 398, 1292, 618, 413, 1299, 633, 428, 1308, 648, 443, 
       1319, 663, 458, 1332, 678, 473}], 
      Line3DBox[{274, 488, 1138, 275, 501, 1148, 294, 1241, 514, 309, 1247, 
       529, 324, 1255, 544, 339, 1265, 559, 354, 1277, 574, 1184, 369, 589, 
       1197, 384, 604, 399, 1293, 619, 414, 1300, 634, 429, 1309, 649, 444, 
       1320, 664, 459, 1333, 679, 474}], 
      Line3DBox[{276, 489, 1139, 277, 502, 1149, 295, 515, 1156, 310, 1248, 
       530, 325, 1256, 545, 340, 1266, 560, 355, 1278, 575, 1185, 370, 590, 
       1198, 385, 605, 1208, 400, 620, 415, 1301, 635, 430, 1310, 650, 445, 
       1321, 665, 460, 1334, 680, 475}], 
      Line3DBox[{278, 490, 1140, 279, 503, 1150, 296, 516, 1157, 311, 531, 
       1164, 326, 1257, 546, 341, 1267, 561, 356, 1279, 576, 1186, 371, 591, 
       1199, 386, 606, 1209, 401, 621, 1217, 416, 636, 431, 1311, 651, 446, 
       1322, 666, 461, 1335, 681, 476}], 
      Line3DBox[{280, 491, 1141, 281, 504, 1151, 297, 517, 1158, 312, 532, 
       1165, 327, 547, 1170, 342, 1268, 562, 357, 1280, 577, 1187, 372, 592, 
       1200, 387, 607, 1210, 402, 622, 1218, 417, 637, 1224, 432, 652, 447, 
       1323, 667, 462, 1336, 682, 477}], 
      Line3DBox[{282, 697, 698, 1142, 283, 686, 1338, 687, 298, 518, 1159, 
       313, 533, 1166, 328, 548, 1171, 343, 563, 1174, 358, 1281, 578, 1188, 
       373, 593, 1201, 388, 608, 1211, 403, 623, 1219, 418, 638, 1225, 433, 
       653, 1229, 448, 668, 463, 1337, 683, 478}], 
      Line3DBox[{284, 684, 699, 701, 1340, 700, 696, 688, 695, 1339, 689, 299,
        519, 1160, 314, 534, 1167, 329, 549, 1172, 344, 564, 1175, 359, 579, 
       1189, 1190, 374, 594, 1202, 389, 609, 1212, 404, 624, 1220, 419, 639, 
       1226, 434, 654, 1230, 449, 669, 1232, 464, 691, 692, 693}], 
      Line3DBox[{465, 670, 690, 1324, 450, 655, 1312, 435, 640, 1302, 420, 
       625, 1294, 405, 610, 1288, 390, 595, 1284, 375, 580, 1282, 360, 1176, 
       565, 1269, 345, 550, 1258, 330, 535, 1249, 315, 520, 1242, 300, 505, 
       1237, 285, 492, 1234, 257, 479, 685, 694}], 
      Line3DBox[{472, 677, 1331, 457, 662, 1318, 442, 647, 1307, 427, 632, 
       1298, 412, 617, 1291, 397, 602, 1286, 382, 587, 1283, 367, 572, 1275, 
       352, 557, 1263, 337, 542, 1253, 322, 527, 1245, 307, 512, 1239, 292, 
       499, 1235, 271, 486, 1233, 269}]}, {}, {}}},
   VertexNormals->CompressedData["
1:eJx0e3c01m/8fqhUVqGISJEKDYU0PC9SKVrIyki2hqwQRfbM3nvv+djrue0t
kqxEGWXEWyih8rs7v875nu/nnG//POfc533u535f9+t1XdfrenRA54mCPvWm
TZsubNm0iQZ//t5wKGDXVCbhz5y/n782/v4jQO612cEz48qkiQDBcbvCENjC
dZ366B8CDj8Lc668o0waU8pMv+WeAoFCudK1PwmII13/OmOuRLKUzl19JJoP
1nuoz/sTBLRnavZXfVQhRRbG0FVEFYN17c1v9WME7GTsK8wqUiUFrpab7kUV
gKTyIgXeEvAg6xKvL5ciqdRpAAS31oB17/7SV4gA1l3nR4cNFUj3vqprDX+o
A+GuI2xuqQQI8ThrfgrRIsV57VwYUGyEjzRJvXavCFCbLVz48eg2aYOHGFQP
aobye0+lBi0IeGcS93jf6hZSSKRFE82vNqgrIY5MyBBwLNjuo+kjc5JmUdcx
utQuoLK+5pBMRcD3gAaDwx8cSDGx6593rHVDJu3+Wo+heTC/qXV6Pf4kKUGg
9Q6NeD/M6zyZHH0zB5wDlzI5lnMpzrsu3cefpJ1j/x9Pta1F6y+ehoDFbY90
ioIyieeOldVpjOeY3c/a9wdCQYr+sO1AZwhc3vrW9dwvAh6fP3E33jEEPNmf
acskpsDdQDLzxx8ERLAcdpJJD4Z7gXvC0JV8kB0vvVs7R4C0vlxNM2cYjPTp
m7rkF8N7xmJbi48EUEtedOj6HAZmXJ91/boqgPUsXU5RNwGx5pczFu0D4cNv
N4cDjDVwDm44LlcTQGJlmiO2BcKW1xk8CT11IFd/+Wx6CgHjFhwTMRzRsG9D
ybyOvxEkpQN1JH0IeOma9L7h9ivIbP5p5Hu5GUSVzGJirAion9aYTRawBF0e
mY1Am1ZgE466sUuDgNP3l99Hbo+DV9XufTKjrZBSpVyyS5GA5jDZ1po6TzhD
ddw1e6kZEn5GFSeZEED5sDkgRS4NPc7+eH9DrBo09Ao3FmoJOHozvV1s9zCi
7g3SYBB2IckELhw8V/sVmoZVD2YRyfCLugLFPbxDqpairBxaJYDGh3u7pHcK
mEx29ih2BUE/OaJddIUAAa/0qeP4edXbc9tC0pLghKmjCOcSAXYHnpxK25cM
T9y5e9DDHEjzueVy4CsBOXVnqjuFUoGlKCb2unUhhM6IvQvEOG88eWIrn5IK
Qq+TfL9/LQXtA5tD2XoJ8Hv+86xXcQKstbTk6ctVQZB8WdvHFgKU059xnd6b
AIxGE+3dp2pA6Pwzwr2cgFGNkq5D39MgYGI48lFrLTzW6Jt5mkZA53BORrd4
OJD8RV2y/tSBlyZH5VoiAcGUMHH7TCeISLimbqVWD65CPQXOCQRkaN5kvWgc
CCM72KvnPtdCw9VbWodyCdBJSNf63J2Bbl+R0OR+Ug3B1+iyjuP+6vis7UL7
iIK8377yTehJh0sOm5OgioDqkLKVisJRtPpnlnbJRoU0zHT6kUbHLKi/XGLg
F8qFUNvpx1QL8qRV5b2K9d8IGDkudm/lRy50c+qR74sEgJdFQtYA5oGKUPnx
1ztzYTL068OzTAlQ6d6bcRPXLU3VmqrvaDaIiKYdVFPLgBD4Ufx5CuNQZmwd
y5wHBkuxZ77H5APjMLNb2CgBtp0fOy4z5oHoCfFaPSiCg1V2BSL9BOgGHK2P
z0iH4Vdt6svepbDM7yhRjvlkFyPv23iRNPhlaygnKVQBBXKu5tEdBNzi6RDw
7M6C8k8lfL4LlbCoukQ/WUdAUG7yTveoSIhYJx2OTasC+wpdp6pWfL8+ymdF
vthC3IufdkMNFfCITaxhuoeA84tNteD4CvX/4lUk1othaSjXeAzXw5ln7443
HK5AR1ICaBubc0Dvc5L1yWYCnl9dOHFSvQmN0X+8aLTHEVbu0xFtjgT8EpSt
zhQZQTtUNa1CFA0oy4xtq6JiX2EvjTv3SZpCCOaMtE7nuEm67tHzWHuaAN7p
wQULw0I4yM44VZvoA6YtDYYJXwhwlt7WldVPBp9B3iJ23Wj4aBlkH/eZAAva
KXnKLjIUKRV0HB1Khsv8tIdqJzDP135kMpkkwzajyx00XzOBrkGZRnYE38uV
G/lXH5KBbeDsgUb+fPDwcOgyHCKAfd9v1qL0HNhHvtj8WoIMojz6aTofCHjl
YpUs8iEThp231uxTLYQgWfNL0nj9KwN7e5lqFrT7+V3bnlAIGpem2g4PEyCo
uhhd5BEGXNPGISezyLC34vy7zfje7bQ+/npySh1NPts9RX8+F/OYvgoZ11Wl
8Mceyt40VOGuHVG6OxnOMmk7qOL1wIB1Ab7zteguO9VphZt+EFKdVm5dRICi
on7yl+A2lBv00HeC0xVl/W5R+nGdgI9y7pa1Re+Rn/r2P4dEHlPGH3y6GjLw
FdyHmF9Us5YAuwP7H6G2K6Tgumw5m08E3JDbu7PrcQlM70p7TLrqCq++nPjC
gO83z2T5gn9dMZwcu7JVflsI6ByKvxKL1x1K1e6fGiiC+jy9984msZAm0n1Y
EO8jTtlrTt5fDHHbdh+9diUJVKNMii5gnGUCTmu/YS4CTtoM/Z9pqTCUUKxV
gdevOPYYv1LPBw7tH0fPhafDEpWqW8skAWaD6blPbmTD+sbmO1c90yG7WfoC
/QwBjp0ju27JpUPRyJqD0sk0OLq447H6PAGhcfNLPyEA1DfH0vN4JMH8W6mC
UazXKssuobKpnsjwOPs8T3M48GnLmn7HulPeaXR3S0ABShYUoBnzcYSwod+5
upj3RNtucHrr1qN7YRQlxfv2aJ5+II+EeeZF6BfOMa8OJJr0YnEjxB85mUe8
TjhJQP+hFeZvjIPo98Q+SeKLEYWG6uC9n2ZzkLZbxe3LlTIQ/dQmQB49T7pP
JVzP+p6A+ObHoe+cy0B2h0duXPYz4N5YKj2J620pc91JuKkUfEV2JtrreQLL
SCD3Cn7+xfTDlLzSEnA8o31k7mMAjCx5aj3GuDFsbu+wvl4CKfPj54onQ6DI
YaRuHa/XeL7c9CChCPbtWojZ+j4cdMzjOQjsQ5x2B25qI/JheMTkXPveCKAw
rfeEzGI+tLA3zS7OAmqRUHtftlD4NeIfs3uRgNJephdczUlAkZ0WF4wOgBPm
1wyzMZ475b8liNE7waDr5eymFGdIMZNlvIl1n2l/yJ/c1HAkn6X9bie7Ohqc
6NaM+Y35qmD597psMZJ2i+AbVPRAKvM/yreNEyB5kIv1Cm0Dqq02San8EYzU
lqM51SMImFpJ/95R2IHSWdBEUkIQ4jtVX9l/jADLFVm67Rl9aI06u/P1b31K
wzV1L4W5OSjWGqZEvSyHTj2WPaafDpOYhZ9wSWKeXE3ln3gQVw5eyWoHM0z1
IMP+SkxfHwE/WPlvne8tA69dNmsxMxZw+0YP29VBAs6VfH5+ObMUCsXoDDUf
PYOHqwbh9bh/j6pepnvqWgInz5CLN72zhYTbt4qZcP03tu7llrlRBJ8O7sr6
SLEBdZv4V/SYf85Opg4XlucDf3AMo1+pMTjS2Gx7iuvTRTvZr7g0A3Ss3prZ
xeugH8P28knfCXBvs2lC1TGQkyBR0rbLEUVtNTC4iOtzhwvPGLeDBDB9jvFO
7fFH5jGBdRPYR4mX2SoxBiQgfZWXfvu+R6DVptk/XvhepjutPDs5SxH/G6Fv
Dt1xqMfNayAE1xXDydDF0NR6RNfy4pzv53h0fSxSRyuSgFoqOcctie1IQoSm
IycvDE3wh9wNuYB5209q62OdXrTRtHvisYceJW7LZ1ZNpXkIt1k7Xkguhx8e
f4xNvjBTTg2q08RhPGeWRBq3FZXD2W1/Zg7zqKDqsM+R4XidyknfSW68DHbL
jsoIEgbIO1Ct9CjGmcbQUF4hqxSijU7XNt2yRESVAn8XxnnDPoo9dLgYCt9+
MsivdkDsdT7jbzGfUH7awsrZQti2+slPx9ENLbbKHNXC/LmWOCy2pzsPhDZ/
uc9n5o/suIvX9mH9lRbLmBH8mgqVvaRQwydhSP1bqcpfnLN+Dj4g9IMhK04w
3TE7FiX7T4WS1wiIDmQLPPLdFkXfHNuZGpGErm6cuv0G+6h3erf3R11LQeOj
VE8fhKUhqVsgIbqAfZR97eNZg1Kk+ODwM0WVDJTcY8cQi/WX57XX4mPXOvRY
YTLbQTgdOVXm7PGKJyD/bDtlo6MVXbIa+HzYPg6p3av9OIl94Mj75Dv+23vQ
G/meNc01Q0rSkvD1r4XzoBJ+O8J5rAxWMzl3d/OeoVwI4nkggnHTaxK8LtJU
Bqe/QbqplDV6vn1T1QO8Xi1gpPN2tRRKaC4VhAu7o9an8YLWmDfSZ5euGtaU
ADyYcSn7EIBEd/t8u4P5ofsPC4nLpAjeyqXNyD0JR/2kyG0euB8v1L8gaXoU
wKtvk+LRc7HIdQf12Gasv/YSF31L2XNAw0+20OVEEmp61hywgXmS96W5v+yr
JJg27Ke6IJaGPIobn4djndq1HymfoHaHG899erKyMlF84ZCkxjIBJl3brjc8
8kSMMRcr7fty0KPoinEz/HzYK8/DJYFp6LlUY8OP93lorEuo9iHW92fcL67e
iCxBda6jD6L25KEPhr1H32E/v3JVLvV3Qw0KvcEy5ieUha4/9aqm5GD+1M+3
5pRuRraWKqrs1AmoRmCI7pc59u1D+7++r3+NNgm9SnmcZ0xx3N0c8p6JAOHw
I8us8yVAFs24SyV7maJoRrFnxr5rZ/DjwE2oBE4w8i50mLug49PX7E7idS7n
nZeZt5aAsMqlDdpLwWjYtyp6M+aBO37FVU5jRXDPS+Out24sot15sZEZ1+2e
gb6BtzfJUBsQrUR1JBnV+RUrp2MfYhy93riDkgtU1CyU8+/S0YardIQfft8a
mot+e8kZIO/KtcFsnYOa93uK7sZ1znmCRoOGMx5Clb7Mtn/PR84JRi8X8L3s
OPno7NGYR6DQ1rDnFJmMLrmVbGLB+tjQMXuJjz4ArXF85pVfLkQdzgKLBNbT
ZAs3PROPdLT0wYHnh04R2vX2Coc37rtGCnnTUeNi5PMk7xGHQCGacT2/dA77
xkcXxVloZKtRiYmdP9PWfLS0YlS3vQLrcj+3+HOOBvRGaatMXUcSOj+Z6MgT
TMBQ7LR2YFozyohUzXwY9phyb6Dn4WeM/++C/kjd4kIgR84KXM+6TRHYGTef
h9/3YGz90B27QrCaWb+myOyPtmir2p/H68y7WPlLPpLhbeb5zwodcehWRoZg
FT5/qQ9DBf10AYzR5x0+o5WG/tznN9PCdWsgp7i6qSUXDKpH3iZI5qInWRlf
+//O3RdWf27SzIDl5jNH4w+TUeC5p56y+F7as28lOFunQLnc8luPgiLkHPJj
5wr2b5w9Uvp6xhHAkPpOmSevBDHEHLhGwuuPXNnqO0p1UMmHKvLEQil6lQLx
LBi3m/4Wvjn3A5H8R7cLl43K0OvUT/LquB9PW/v4vsH+zVuLz9NcogwJbjBd
6Mb+eeuVe4YvfQvRLa5mV4+xEhT1lUgUwD6ZKjWhRdOxDLUspcznkQuRDk+c
1zqe+8bpuckHXtWgu8vz99kDstDUfMsniTysF2m0BURINaooTzpg+caKsrz+
e5oT+/bW7sUjeidzQTjFoFrznQpFsXP5NsL9JXZ4kbm3PgcmVPU3r8SFItbi
G9p8mMcqfW40Jn/Kxvpdl7jrXioK/Lijkhv7gSmjkM9G5lkQs+q3Q1U/H3Hr
UmdrY5w9VFSEFiXTgItt64tF3mJUwESO9sQ8k5h5oOXQcCKMauU6il4tRwyN
0RRN/L5/dFfuxAvEw5P+3a9inavQ6AvDSzN4HtStO3XeWCgQOokzH4RSKcjv
Z9iR5no8F2989rM6bYP712exOwKhTZmlP05jHHxjjEu9dgcgHbnxPfu2IuR8
t+j6Mzy/mFpd2yarkoQqHc/Y2q5WITpVhp5njQS4JvNpewXmo0d5vG83hZaj
C6d0lEvwOpOGaljbRzLipdp/9ZhREQrZ/Z7btwv36bBp3HRqGdKWfswmrJyN
1ik55rc78VwZPjFsoZGCICrl6D5zK0o3+Qv1CNbZIA2H9hOXEsDL9xVjqJwW
RS+W67DKOgF0y7R8PlsT4Ez/ndRnxdFoTvbsfcDPH3kzdTE4KB5GIs7SEw+y
UVqlCi8z1qkqp/m+psI4MJQ90l1QXYQW3tEwHsD1yVxwbujKt0jQs320tjex
Agmcvj9RgHlPbeNsssD3EBhtszjyqgkh1aFyxRoK1hHj4OR7ykHQYvPETL+r
FhW1KUv34fn347DC4NkbbtAofeLNxFIdctw2VSGagucmlfkcizpzxEEvPkux
qEeVk02x9VintC3bzGLmXVFh5y/nql91iBkV0E8m4zopbQ9aig9HjGENyUWG
eH+t3JOuBVgfdwwy3t5IQ16v+NNObaYgpHgy9DE+T2bIzVlUmolqVQ6xmM+U
oPNlCdxTWB8zrYVludSyUVvTp+s8ApnopqFiZQnmt6W3c59S6eNB/4PYi5/6
RpTZc+eNVzGePzY2LuVU6JPwJxP+pCz+y+vQsrHt7CN90grTu0Z7/2R0yIdS
qYv1mrx+QmYnlT6JZf+nrNuGRejPkRD7BcwbdiLGLJFSeqS56XQWn60UZG5p
9fwp7jtJORND/9r7pHOuVjXbrOtR785fL8sxDre1KQVtsZokzw2LEONnzUhK
eLyK6SmeF95ctD/Cd5c0zTe5zyW5DYU6vBgKwnPWxMTvHy40t0iqvmJW56I6
0H7PFYsOPBdUh7Ixc9KTKJx3sml51DvRmbbbUSmHCBjuL7efu6pAodPsUOE+
1oFk3I2tmc5g/iE7n2ea0KUMCOSWuXc1IzeaR5M0ZgSENJpo7Vu3pPBfjnTO
9q9BifHRNPsqCRhUkX8xlWNFwZNrEFdQNtIyHTE8v0RAkeWWuMUaE4qiTPbG
kdowMK0wT2nHPjw6UVLyRaMj6atCi/LfT51/eI5xX/vgeLsW1dydbKjQaYDF
QKnAo3IE8JO/nBvccIXnxLZThz+6Avu8pToTfr6GVizx0jNXYH7QlJAbokza
fNq51Aevn80uoI4qCaXMnx4jX17rgQLVTmd/v3nYs3nf40bXXvTJY2/2PYkY
kmd09wnDW/NgKiLuUPstFbVnm7YRP6tB8nyF6T1EAKPJ+YMmTN1o0J0qMW8q
BQpleC/tX5wHljcdLKdLAqHD182njikAXdl8PscevxfztcSEqb5AWDxq8F58
6T4l9P1xuhjsk1kEFDW9fR+jRfaDknGsauiCgfsEgc95hdud69RpedJAjPHc
yq5E4Ey8aT2JfeDvTRORpLfKpANHihd5g13huMGlUG/8fKxkmCZRV4ykrHqy
n30ugtqTw7Ofm/BcFnWedjxQn/RqZzD10ZRgVBq5WKmEv1fx9ajeboMkoAs9
oBt8WJ0E9Sxi3Hh/MFh2vpxbg8YFupG9Xy3UdDLwUT8iQN9WUG3z8WQKq33w
hd71Znh7jIVr2oSATe7V+x+UPgVeGhueiYynEMvEIdyIz2P7uETn16cWBLzV
Sy8YUklGD6nzy3Sx72U13+ea7Qxv/H2lVRqd0KxNF68Mfv54otkMNHiAkpKZ
2gUOL6hJuChXhc/5yYXDW/RbG9L7MxedyFsAwf1urVmiBMi18LdUPnwKs94z
HbOeyqTQ7tsK43if8JaBy36p+iS+Gepi4TwXZDmtsz0Er0t63o09djES1svs
FDW+WZBY7kZo38T3Ur08y/4tWZmk/Uhy/qDNU0iR3SX8d5+jVD43OCjOsObG
vlxfqkuZ/83vG4vXx1OvrC3ZmJN8zG1bQ10joOoS4WGJ99mm1Sy8RyYK+JnT
LBN6OqD/3Te9eAECNG0PVAtf8ETui7RdjBPtUHJ5csL1HOYHlsrB3cmlSCpi
5itjUy30FenJyL4igFbjwhcL+lA4PP8l/gJrEITvomTn4DkrvHDEPdRGmWQl
8SSXC+vYm4lYiim+rwq+w23TGSEgW8Q+kLSUCM2rlw69wnwO84ybJ5uVSIpr
vu1TStnAVWwzY4b7bpjIMWReCwbTp59Zc59mQ9Ix4a/RWH+v0kku8Gcqk8Q6
ZV8PTJGB9Y3Ktjjs32ovMsiqjYXAeWGRoal1Mnz+8NI6APsf9cbYOceTqiRP
NSY5Gc9ScDtFN3Pjw198dt07bRQG80+FJs2iS6HS+cH4PJ7vKNcNfA/eUiGl
cs1mFTJXQbLPqHz4awLo665Lsb8KBVN6RQ563iq4Xvy8Vwv7uhcZOrC2VZF0
jz1/nHS/Bhqpo2ZTqv7ndwfN//zukK6xvUveNBCYbRWsOg1qgAG9P3IA8w/8
y883/8vPr//LzxcKH8hc3alJSu15OyO6pRGq+8J+NfsTMM9DnAkciITgbwGr
WsMNMBfb9TbWj4DPLfmhqiuKpIIMVp50mma4Q8xbONjiOfdTjCfvlkBgl1bt
5aA0wcc7D9m1nmNdHtuzzV79HIk36pGbcnErvAo4V7dLjYDF+ut5b+QcwXMy
ur8ypQX26n6d1jMgIHevVpx4hA5pc96WfaoKnQCC4wtH+XHfmXvMOe0IhPOG
u2Wts1shglrp3VlVAqL29A4vWVmTAldEpnukugHO2odcX5qH5JqtprPnwsGM
iYpK9XArhDzqPzynSYAFpSLe6bgN6YXX1G4J3x7ICzrx7kD8PPT+6DbUeXsP
XaJNA5HERpC1bzp40IuAQ5TQ2rRvTynhPGclTwv2gfUPJY7BzXjuM1u+M4N9
aclA9cBLpQBYKqD6ELuC50dWWV2dPSnQEKveVnMmAW60WNwzxvPUis6+7LcP
k2G/R5qxTV0GbFPRP7UX+4q7oo4Gq3wpIHLCh6d1TwHclTPfX/4Z1xvbz+bT
d1OhX+x3TQxvMWQ/VE05gX3seIf700DjFFirP7nQsbkcLv8c0wx5Q4AQ9+ZP
TIkJ4DpHfbrldRVkWDfYs2DdVPlPnn/sX54fNcf+vH5vGlBdyrTMDa0F6zU2
/5EM3KejU9xJh2LhS4Cid1JBHby3KP0ViH3FBSOZMM13ryCm8FWC15l6GBsP
C7icSMD7Os70fIo3nItcn5aPr4OY/aX5O9Lxe02lB/maOqCsLXevvC2tgSQV
Ab4Q/L1ahprneW4UIL3WC/1Cp8uhs8PqkR72e6+fk0wEHjWixE4LMa3pFLDM
NtXod8Hzzvn6oqoPuTDGVbbayukLKtu1w8WwH/40euS+j2wuBEtnwIGJGAjP
r7l9eB7z4bsvHy4q5ICtQPKfColUmNFyUOvCc1ZbhZ690moOdDMtPxmVzQHf
ug0JJzzfRXj0Wx7blQc11T++R64VgI5gBpGEceb25hxSfpoDjtUaWmTmYtgV
zjkU3If9Wxa5eYddOlT9moym/VgKoZoKaWPYP7P8y/PX/+X55H95/v3suB52
iSxocclQ2lVWCetfmloEGghoGa/Sa52PBwb3iBNGtlUgEpQkkIL9Nkui0ztp
TR/IfSNxAGwrgR+pW41h3zspp9HPwGiKdh2p2E9zvwzWA/70FGE+OSIZDcKS
6YixV3CyT6UQnuyesNTA8++1pDSJU28oqDJg4EbswzQocBtnbsM8MKT5/vLI
xXZkKb9ZbMsue9B88f7AW2kCBAoGTq7eLYSqwfxsO0ZPWBYJnSYwv1lR2B4V
Yb7jWrmv1jISDi6FbrNb8LqxxknXnRpk2PsgbYJWPRHyi5pE4rF/E6PdeyjH
jgymnqNWIuNpIG/GJJSDffKMykfL/FIyrK1XhOUtZMO9Udr93zDOFmtMOhmu
BXC1vMItMTsflukNWO7iOYXWT/r8tVt4DuqoL+a8T4bplnJJPcyr/v/J84P/
5flKg0t2V69lQX/g7MhsWCEMebfayOH9gz4wnoxQjAPTshRx1mkyfHEJFm3C
54xpM/H6yuAAu8k1z+ks8oH9/bXOd7h+lo3I+5i/h6FHKTQx4gIZ4GX6I2UI
64W/jSxHwtEi9GKL895RqjjgUwgvacFz60yZkMSz4HpU3WW29amYE3icMOi5
GEfA9itS4C/zGoXNtdw95O6E5E7ZOfCxYb9kI5ZbrFMCKT9SDpLinODJwkSN
zEcCfH5trTaZKwadMktq+SeBkGHqm++H1/UWjbQOnSyG8/QPBy9NRQJ9zcJu
RoynarGA2e/1IrhZg6JekxKAs6s5twHf+6L4s6wvFUVAlMYaL3slQ3G2Ybzt
CMahgu4bx6FCuCEls3CELw1aOG14MvH+EzMUmhXWfHjupO6oE5qO5yV7ZzXM
P5b/R55vl59yPqI9HfwvXQ99pp4GL+QeaQ7OYd6mMas6tycC2H8FFGaYJ0NX
jsbsJsyHXjmrYFSjiiSe/tZ35I2FrvKnu8ywjt9Zkqrf2ZKMnjNvH4hq8Yfv
e7uGD2G9dnetjX7jVY6OygvRNnPqQ5ebzX32fswz25O/H8tvRMasV0x3kJ2R
wLU/ggKemPey6C4clOtCT+KcmfVZXqG70jdVfLcS4PSo9u7552UQOq8mxGRt
BQH5Il3yuF/kf7rcSPheCkG7Y8+uSrrB3iw2N19cb2tyzfOHuUohQ0VNf2zE
DxQrzzz3xnWl05TFyOZUAvtevuO9Mx4Emqmy18oxnv5q3PvVs4thqry1oYoq
DOLOZTk5YzxvPLC/y1mM6zK/uqd/JhxMHj7slsL15viWLkfkVT54ZZVfDygP
B8WXtqbWXwno/pfnb/5Pni+505BM3ZgMudx/fNIFA+GyjafCCsYTFSqJuqT7
w9Et77KWvNxhjPHY5D3sD53eb9NLofdAb7Ysp3+h0oflZFNHI+zTzKkYe95a
Z6Gtt7pXGSvskGXWbDkN1qOLOy7XbVGrQsX6FT6hi76Ib/fqRA7mq1LRxDOL
Ik3oGU/7+Z6kYKRgfGrfbicCHmaxpuR0vUZ6q61cy8IBSLI8w45gwP48hSYs
IKoc/limhAWP3QfHhALPCcyTbqITvttpy7GPI11msDcDhvje0twBAm41Pf2w
haMM3mfP5ZarW0NKqPFbW4z/5gs27J/YSqHcNki3kMcW3tW3ESYY53odg2em
+cXQf2NjvJ7eFgKiaHnu477LS2ErtjUqhMldPPWiTU+hpOtQ+008n8YJvK2l
JeVDhP5b43ghPdD6Rn/PBuus1788/8q/PH/lX55PdaBjdM+5ePCM6fXW3u6A
eDkbIRXPs+6wyKvF4wCNOZui6R1foXUR7tfTGGcDsaGhmdYQxOM8cYvJMxT9
tJc4dhTXc1NWnULQlzykcgQFUyajkbVaaGAXvl/h7c8jPglXozC2AubIa/Fo
Mu9ywH08j8R1rjqGpjWizO8c2gNX49BO9Vt1290I2HG0fDtJrhPZjCQ9F6gO
RqfChFeH+AjwC34oNVJYDtrfH4rzXriDKIsCiRF/8/zP+oXP2cphump4yYBN
D1WI9rxxxzj/EJX4cIcH1/+DN3Olf0yRMY39wN+cec58dOtGfQl8/JW22LzV
Frkb9mVqY96QFHt7PWe0COzu1oYPqzuh+Td0j/SwPq6VTZwO1CcDNU8SV8Ve
TyTlGE0jgnmg3UbqJirMhdJvfY+p5QOQ/KE88m6sy9f+5fkl/8nztzuJzFgG
h8HUfYOMQ40xyMXB/rwm9u2lGUHvZ7fRAqcFdzKfZyLqY02WSsDrVc+jKrj4
YhDL+wZxzr2pSKE2LI4O7yMn5a16jrYATS15tyk9S0cHSqTGbfA51/klFD+4
VSGa5a2f5GbTkUKw1DfReowzrX4Hi1oDapC0jOjYk4reBC7ycGB/uxQ26V/C
3o723G3YrxsQhfoUD5vcwDrIESzGVttaBmIu77gvq1miJXvxFMNB3C+/mzu/
CZXBWPuvDv0bLsjZi1f8F+YT9z2KkWuipTDMJ67wmtYPSTGyB9/GvGEk9+iI
algxpKepZ4bxh6Ba3bTuv7xdVR95mxHr1snZuPcdI5GIRqlZuAbzAzU3Scu0
NR/+TJ9uZ9SPR/ya1NFHMM4nOnm+mH/JgrLN31xt8pOQQ4+7/9+/qxH8l+d/
+Zfne/7L82mdDe0+Tb2CVd4r1lckMtHuj0bif/P8CAaHENNEWzQs/ixbhD4H
HZKo7UjAPPNyf6ugHXcCGtQYXlvnyENDC/PPZXDdao8c4HxzqgCNyG1mYrfL
Q9lc0Rzf8XstKRYVFBVXoGv8l4bd43PQa67EgefYv9HJVmsM6dehnXfMtlUm
paOcp3yNqgnYP4QMsreJtCAxpQqmOHzv3FOpnmlGBBhu3319tqEEjnyrm342
4YhOMrMeF8d1+PHkFQYNkRIwU09afVAbgDyusq+O/133+toxe6sYvnSl7O0J
i0SF1P0X2jDf8vTnZLaFF4KZ2ULnRnYCau/dHbVznIDo6PD31KIFsFRrbMY5
nYLus8k2LmGc96fxTh63ywGDPSF8r39noOC8NrZe7GdmThdJaDikg95x5Phm
Sy76UWNl5Ij55OC/PD/kP3k+U/5eZY745yDd9+zwsC0ZPZHh0H+M76vzoGR2
Yos7erzT4srvpEI0UzlKfQPvLzsk483mkYhsZ+Zad54rQmo3F+WpcD0c9BHq
0+3PR3lB3hrqzYWIO9a4ueQdAYpVEQeuaZWh2gNTucd/F6DdLeVBTzFvuEoZ
Bd+gqUEhJet5339nIwV/idv6ZKzXYYs7mn0a0CaWkdlSxQRkJNaiu47rXNH8
wVqaUyEo8B6ZE+DxQWkvd9w0wOfREXFltKUthLuSLCq/PkShBYs6RTms+zM8
knz0V8jwrPTaZ6WAZHTsm835Z7i/BO33Pc9OzYfYrst2GV2ZKMyRtlwX4+wc
uz+avSYbuFl2cRyXzkfT64JRLzE/i++vYSnXTYcB/gEdoTYyMn5x+WYwvi+7
E2kk6XvJsCfR9KrytyLE/ppFfhvm+cP/yfMZ/+X5FplpwYq7FGC9Z2/P15ZS
VCrSHuKHfd37YmrWYxQfJDPYLxEoW4ZO3ShjP4f78ZPCTcbBsQTUG2nnTnez
DPlqKvKsYjz3rMkfeLAvD43X3T9S41+KPl3c2LcZz91rfrdMZoyLkbQcnymR
X4weZ8uyVPzNn+mXxdPTKxB3n0X8J3Eymn41DAEUzFcNPm8ClGrQz7Mnval4
0tHiDsfYwUICGjY/VYp8kwOHApZpODuCUIWbe7Ef5sME1elYF44cCJVUKXGJ
SkLnyam3qnB/fYu4Jm37KwtIly87kMey0fKHB4F5GH+u/Yl/qO9mwLubER4T
DWTU42YT+hHXv36Rt65mVAok+d2VayooQV9fZNk/xDzvrlZrM3giER5E7/p5
jK0CpVbfLzzUTQBvoIrKRf9YYGtSk6FrqUIMIQrxuni+ePyfPN//X56vf/fI
cv8xIxR6s5s/xg4hj8l9yc21BNiqOJ/vTfVBmyS2ufAIIuT9fS0G6nDftb7i
TQ+JRUa6b7WzKNXoS0jvu3d4rglffKz+jZSJuj1DrrpfrUSX+j5MOeG67R3P
VuQzLUCHnM90C6SUIvVBN6XRVgLOXO7cMaxQhCKlT3UaDpLRfl6Dc3143lEN
+iDSNF2CPN3urLh1pqMT2m0tVvgeveNfZZkyJsD3+ga7XfsjUMrpOatfP//m
/8Kv4/LjYWhDjqT8JR0JuEfNfcN8+LW89mff/nhw4S3g+ilLRqLbj578hXHe
pjByiV4xBtrHb+XI55ei1pFsHVvM821/812WCMjNOrPA61+F7vn8adZuwz5Z
t6I+9EYI2J25tdlAqwa5z7m0L1XgeU0r+43p7UCYpLIs+LNUiwI3aNPJObiP
/uX5Df/J84/Gwe3aHH3EfuG+5jvderRwOKzRBPPkatiqiWy7E3qX5czhzFGP
snc3R31Owv1L55jl7hiEXAoF0ho3apHAvKIwB96fpSt3yuBIItrfs0ajLor5
oHjs2XAZniNijoaybslAbnuyLU6rV6KrAZ6j0IzrdlPoOtOBLPR5KVTo3dMi
ZDp1tgP+zo/KRfkJXalIla1ocN9UKgK9ieA07JdcMpOPx1rpkzZf++R0kioR
bVGnmp/Huszx0zO0Z58+SesBc4b4xwJ0nbyVmgrXs1J4qaWMlx6ptPrg9ZWH
FUi8ZV1dspeAuwVZB5WodUmH6y3N30nUopfVGaEqxbiu/E/cl0rTJp3oeJXh
97MBCTStR9wMwP7wwXtyYI0G6YmExBm/Sy2ofat2/cOHWI9YPxR861Ilyaxf
JJuPtyH6iqkosWtYj/6T5/P8y/Pj62YHyjxPUeDGZp7D0p3oXW6MA+NhAlYC
Lzf8PH6Loum+80GuTQdivWDA33SKANYOVBlUfo8iYXlR+5VFK1IQ/fbskcbf
3xHuUOkvmFB+7RadOHuqAV1czEI6EQQUPDvDWJzwlNIonclVxFqBlMv9xbVx
feqIqunnfLCkiHPlTX+3SUQBOZfjMjFuWY/LH9mpBUGvVaWy054gUH5Tw8X1
Ny9d3KPjf8AfzL7LMPksJELoe35p7VUCqqkNn9K+9QTaN8Wtsw+z4c43V+1S
rNevE3QWX+5zBzqp/TZ+S2T4WddWtIF16lDoS7rEOUe45+19ccCvFPYcK+mc
xjxJR3jR3nIwhzYRhp3Te6pAcPODbxaY9z48CRsX3K4Dc4fGe6fu1UDznfdv
s6oIOP1/5ISTXw/lZoSFw5XS9NgfHxrA7M5LZxqsL06CeQ7dv1whulyG+15r
E+RdtV3MfE6Ax0FVKidRY/idTX/udV0LNJXTbTurj3EbcBMN3+EHa77MDi9M
2kDzkZCR7U2sF73QaHDHAu4/jtse0NQB3y9OeIsdx37blFnpoXAy/Fxo8YhW
CgBJrYdhFrjfY8e5OWesE8Gox/M02ToB7Ba6oyexP+Sxnx6zj4wHwXg19tSX
mcByaz9HEebhlktidO8k44Gj6HzEw9ECyOGquzKKdVB4tvXUyOM42Ku7Kf2V
ewn0b/VnVcG4Ca0wn1tvjgQqT7qhuOOVoGdGJziKceuE0KfmT4PhSu2vHY3M
NUD1M3PIsZoAhX85G9N/crbJ79/zVNlSIZZCw/R6phaOcPaRktMI6GEyUbQQ
jgDFc59X3nnUw9Vhl0D3GMyHcmxxrhluMP+uQzK2pwGOj8re1ArEfr7lRwqy
coe6ljeHtciNwHVsc++UB+ZVJs5D2/OpYTWHW7uovBE47GqzefFczMF9dPJO
SQ5sVzWRXeL0hbPvucpuYj70Odz1oorIgplDX3/TeMfCbOUB40MYn7vxe2UL
JzNALsOpfUEsDQKyBZlYsc98m2VMxxSeAUcXToap3cmDHzeO6uti/uzacG4O
c8wA8b39Cy51hcCcLl2njvU6s/TpG0P+NKgurhos7S2Fa7NPrWgwDwTsPS78
TD8R0pR6nYJoq6B8G/O5x5j/mf7lZn/+k5udo9o0OvEsEyzPPXrw3KEKxI5F
zzFg3Wkx5lvavxAHM5uFHlzTpcCtRc4bu/B6l8h8UlqwF2SvSG7n+UWBDy5Z
uqxY1/yzEtom8/XR7UvNBwQ1KHCWeS7zCNYjevqSJvv2KMRrfu/CZ3yeTXbJ
DMXtBNxxZ2UwmS5CF/yFg7MySyApIIdBBeuaxFL+e+8tjaimzffJvvEUsMsW
nBfCODPwntAx+kAGY1sG4RBGT1gq1C/dh/3hkXckgZpNZNjx9Ovv8LAICDHb
fOowXg9Y2HpbZT4fyFt/bjpmkgQ//OQM+XH/5uXSBTZm5oNvWVLPt4OZYOPQ
5T2D/ZVmufETrth88Na+nalolg8/o28ENmKcFQXXrOeLcmG7xWIuLSoEhtfu
DZ3YD3iTXDiPLaRDAn+DTMCWUoiyz3lJ9+5//g72439yszO3CWm73kzYcv0m
Z4p5EWyX8hz0wLoADrqEj38cNLKb2JN/FwHs8BV3wv5t+ia5O67PETZuT2fb
nS4Cg3rfFD5cDxw2+uKk2iBURXHY/f0hGXRHhyfP4/fN2bL/bo1bLtq4JV/P
+jwXJu4zbx/B/tD191LTc9salKrNYCFMnwK5VwSFB4sIGDwx5G0d3obK2tnH
L2y1B/6MWZE/1wkI4Xn5PmBTCWyYOO6QjXMCpacyjGrYPz8wkRsZFCyGbWZb
ZJLvBsF5bakzNnj/3W4V23ewF4F5hkisbGYMiMu+HLiC8Twxwv+hG88LcumK
3go7k4H2m31NKt6HStio9qVyITRZDB0MUMgAUuiD217Yl8Y8Zua3/VIAV/OF
Nj3WzoW1hG2mzzBu17IEmd1e5MAQLffxDvsCYDA+x30G+zTzf7kZzaYt/ys3
26jzPDcakw4jfBThNJ4MsCvhSknGujnbpRbJHxsJ/u5ijVds0iFV249jZAnz
/32xVJ2v16BhK5LeqEmBTxuP/SIw750inekbxno8KCG5e7tiArw4uD534QcB
jYsVT8Lai5F3bYoAl1wosEmfqb+J74ujQTfSjKUBXWAf52U3sAeNz3zsOpHY
Zz5xuGwX0olUy1wnWJyc0NpW0ZXBAwRIhYZM/hYtg696Z0L2WFtBY/F3vyPv
CVBOmH3BZVqKdWK/uoWmBxyJo2KiwTg0ke9ON6uXQKkyA2MJUxDIsu1/dhnj
EKv1nNUwsBjk1fZpnhKIBN+7HjuO4/PQnlvKs1wogt0boiZ7iuNBKY10Twiv
j1kca78lWwjLxwXqrQ4kg4jDoLk6vsd5WhbjZLp86NccvTn3ORXO3Pj5MAfz
dte/3Iz1P7mZ3qyXhPRAMsQwfjdX9g6BWRjN1FshYIitTtx+NhBKzjZJha8E
gtIJuaOnsP7SJjeT9485opUczicrJ73BlUX15MofAnT79N+UuqSjweXkqZfw
DLxr1bjNMc7vc5fThrdWoNgt2YLftuki4UtbB9xxf2nP33PwrmhEUV1jL0+I
u6LSm1Yn0jE/v+b5vf4t6zXKa2PRuMD4Cr1IMfR8vpOAd7ftQjQty6HydmhM
6Nh9YMtgktHpJyDw+3ejgzVl4C36dphb/SncuDI074R9qUPnBnVjUSkcyDk6
EGDtCEsi8qovsU5V1Eo4p8eWgLyJhMnlUA+gthFq0sT4B7G1kH/4FMPXsh99
kVT+cHlCpHkG1/na+gn0Oq0Qinmz+hTMg6Dg7on9yrh/33vn33N9nA+f90s9
Vc8Ogc9XLvdcmfuf3OzN0/+dm8kItjqx2MZD/Npmg7q35ohTelNU1joB/SuZ
05rPXIDaDeKnTV4iD9qeXMUNPO+8v2mOggMRF0lxzWuPJ8rI+3ZnO8a5j+4j
u7FsDpr+3P7+bb0/upK2Z9QO69SEz/fBBzurkGkpe7uFbDCioe7hoe4kYNxk
bnWcaEQjuqKdo6khyPnZcyfChYDz317sO3TuNWII+Pm990QA4i0TK+ViI+Dz
AWqdqJxy6D2XSDl44Q56/jG0P6ePgMRYi2VV5nI4fVm11uy4KsqhfsMQjfnT
8ZPufcRRBtktGxWhX+6g6ZkuwQhc/522w6ZR9SXArZOUbX/5GkLXFNK9Mc75
T2ofWFwrhtIyYvvH6XWKtpjE9QTMMy/YTmXeESmEw+33QswXJKH7sqPpDOb5
P6xtbg678iG+9uVK3SAXWq5L6/v7/1Bk/+Vmmf/JzTYnJ3kVyIRDzkv6K2wH
o1BpX//gG+wnLeEhqYZeF+7p/Tyv7BWH9uSly7H9IuC2RKDYB7NIpGn+mdr3
SwIihm5YSGHeUJb0Yivdm4/6FzX6bgkmIYuGyG20f/OKXze/nFetQuT7s0qh
W5LQzosO7xfwvFB+25HZSLgRiT9+ZJIUE4809rJQ+XkRwC/Z//M8uQPV3vfg
Eq8MRq0vyi7UHiNgn2+JSMt4Gfz50TX7NzdDam3me3DdyjHHqFkZlYFgaUi3
NLcTcmJmpahjfWGeSVDLf1UKy3ceibfYeSDphK57mbiep8vc7u1mKAFHpzMf
K5xfoXPfTSbImB/mvUzK7SSKwF23vfKgnz9KY3agvjqJeX7p+zUaJjJ87RPc
WbEtENlfqw6Tx+8lOUnPqHo1FyJff7z/42Qw6rKaPL0T+xyhf7nZx//kZkFu
saqj4X4wOBWSOIznTVneX5mGGH/FjGfhmyhmSO2SAmfEmUzk67P/4DG8ri6u
Lyy2KR5RxRU39F/KQkZC+wo1//4uGdy28C4+HzHFvCTtV8tCQ6UuQuuYx+om
/pzSFq1E8Uqnn3tty0TfsxzUvmEfkju8lSmquB79DjHd6mSQihi9F5k0Q7F/
1m8YbH3Uhs4Qroc0/KIQU8HYMQesg5mgsGpPU4rnhYe37Scc0dH8SD0XrFPZ
TbKNV0pKYEmX6rpEmj96xcL4MQqv/yBX1AfSlYD1l5uKQyphqCZoS6YJrlv5
zEuCBzyKIFaQd1c4Wwziv3GWWg6fM2HZ/C61JxlaJu5sl1yKRx9f66luYJyP
/Q5d/66RD9ckDjlqriSi4Mn6izFY3x/q1W4EPMuCLdbbGKxoU5BTygTfX97g
+5ebBfwnN2Psioi33v4SfizYmHd2FSBL3TXpyVnsh/WzGdR7nNENSbdrSjfI
iIZ9TTgN3+Pj30zT98sSEAP97I3NKmQU8OREBy3mK1E47bA3Mh8lsyxkzfsU
ILabc62muE91ttmdKt1XjqJb39y8Y5GHRqzzBBZbcJ3cyRakRNaiiIzg8cNK
GWi92093Szrmz8sr8rS3m9Gh2eX2Aw0xqH5Fsd3KAuMsHG7dPl0IfX9s+SR4
fNDkqKOuIf7eOwJ1HZED2CfkcH7fbBSJbj3MZY+ZIMDt3B6pZ5aFkMPG/oPf
KBE9qDfJ3hj7+/uRSPxWPjKccPAPEqhIRRbpa+m3MD9MwYOAR6exro3dJI/h
utqSa6TehfdhvD7Dp/Y4GxzWpqhSV7ORwS+/ZTH8vU9MZYJWmNNBYNqYlGuZ
h6wsOq3NsQ4e+5ebbf9PbjYoeoRRIusebOL+ZMXFXIpGu7m9U3E9OAZwHJBt
9USVo/pwdb0E8aV1+VzG6ztGiSt/EhPQQZ4/UQ2JJajScOnuNozn4Xra9JS3
eegPUhK6n1SMZE1ffW7pIeCKuwO/SmAJWrhcvtdYsxB9tbp01QPjTBXQuyBL
oqCs89MvQudyEa+Au1xAKQEjv2gbTczqUcFAS/WyYgK6NybKJxyN546YK+d2
ruaC4qvK0yIdQYju3C97K8yHyoGs/C2GecBVu0RwtyWi0wfaa+lwnYTOmfia
yeeBc5E+HLPKRHwc3dQFuD6v6bjx63jngLTroWK/B/lof9IwrRXWu6+X6m6f
l8+E+D/C2+0WyOilShpTAX5fDxPJaYddaXC4f2dbJCpCFzulfxRi/hHX7zuz
MJsEh9+G0NUvFCMNFwZyAsbB8l9u1vyf3CxeVrL9+DdtdI350qThCwqyr+SA
7EYCHh2OzTDp8EK+hesf7KWq0d0TaW1xeN7ZX/yjyYg5Ds10d+ZHS1ciCquG
fwzWtfyPu5p79mejeZdaXcaZMnSH3GeYh+egQlarWw9vFKKRnhzr0uVipM55
us2ojQBy2ylTWu8yNHzv0sCf22T0c6BMRgh/b5ZZ7XsZeQr6ZN120e9gOuoQ
Vnp7DM+n2VytW49kJ8IWP+O16/sjUOVSN2k31gXylcrAFx+T4CD18Klzh9KR
slvxz0LMY8TrEPldlGQ4qBaV8eRoASov2p/Yi+vwS6wh507rZGArabqi4lqM
5H7Wh5lg3JLIgu1DaYmgZUihzzpYjm7JP3ddwnXCpyFOVeAfDzLVWX5Co5Vo
zlaK/Tc+f4bl1iWB/BjYWOYsHsuvRiZuB98BPv+vf7lZ3X9yM5ttKz/0b2si
PUFmF5aFOnT30euEXrz+vd1WuvCZM5roPl64U7gOoWNzZ2yyMZ8wxuQNdYSg
Km23ir3kGnTt4O9rfGW4L5a8O7SyUtC82MnKeftq9KRvb/nfuXLqy1m3O1W5
qNeq+vBISBmSH+xvlsXzINnmqvG5ogJU12OwczaIjJgsNRWqsH/Qc/1gMRha
hCqp47uP9KSjiC0a7h24TkSMDegTSsxhabhpe6lvAirnk8lhx77Ie6CtiHOb
I9S73tK7z1GAhIcqhowxT5ppHPtG+8kDVDyPduZwlqHftr/j//6uFG4kPrJY
7gcw5e+b+x7XT0BS8l6syyYFB6QMhvxByf5G9gBvLUpsK7rHUUyAVlNf+Zyt
P6D2dQXasDpk+z1kJzPmN9cdQS8N5/zB55Zg5jeBerRn/N4eh0Tcp5vif/7N
zRT+k5vJu3CZat9Shr2NU17fd3agvPh5xSRxAk6K/Xmd26CAevg5Jtqo2lBS
pSbLfiUCpMdpr9dbmCLGh6QCV6omNHbkUex3VwL2fH7hbJXggfqiei66/6pB
x3xvmy2WECDQvX/Df3Msuux/Vdz/YBkqzaKaXcW+d4JlE5f0rmT0labyxCnM
25xnc7XPYv4vzuV+bvIgAb3dlfJcgrUX2EV3xOhrzkPBtJ7qpk0GUKs0dKqK
OgjsRMo3fLGfnL4x9c7S1R95sNAEw4t6WFz7yGoUR4D/Mbu96vfT0atDc6+Z
iqpBlb5J5zbCc1bmIRbXqTJk5ru4OFZXBtPhAq88yQRo5DdI5tvUo0gjGeXE
g+kAR07pmIcRYH3/TVX8mViof0CmudQRgQ6gmtbeVdwvXpcBVaag4toUDZHe
JDTeRtpmjeeRyRH7nWp+N9CJSNsDXY8CwId80L4Ln7NXNt9FZ0aPYmiTMbI4
GAlBs/c+HcfzyCGm0B9XR5VJ7h/Ppx5fDQTJuPW1MPw8x51pT+MLNeji0B7f
A9ZkSOOxfymP77dZNuDZ+KUS9FvPoHz+UgUc2XEt0BPXQ47ZZo2z17zBIq2P
pUW/C7ZG+/usbsb3WHL3idLuYOijFWqM0GuDq9t3vVe7SYDgnKVD0u5UJMQZ
0aLC1AZmDD/KE7Hf6FbJPemfWoBUY+tJUqdqoDWm2fdlJvarBfXXAvsykYuB
+w6DkSbo3R3vikwJOL5NerJGMwhsZV7rFbSFwIyF0uTfv9vsMj9cptgcCFnP
M6o8dZVJLS+fKMXh9wrSlpXwehMICmV1ETviUoCOP6XhJp7XZJq/c6qxBMKs
hf9G9qV80IlO7jg6T8D1TUHjFuLBkJXkXc6cWwxSfuaUFewP3dcdI7lngmFv
7PhGUWcFxKPj/lJvsA8MUzQ6fMIfYoYGXogy1ECB/EDLdQoBl4bKLzb7+UGa
18VnXL11IPr8w+EvmE9W1Ykln7BwMFo7ERAr1AjKlh9Eqn3w86f8RlGqJ0wI
SuSr3WqGmc2lPRpWGLdet93qKWbA/Y2nemdGK4Tv+9MYq0bA79qDDwKORsFF
GpZH0hFtYCKbHHxOFvuE+G16Dj5uoDmxcI1BrBUYDORst9zDOn4oKmf4ayKk
hk3WH70VDLf8L+1sw/x8nUprW4xOIjSk+PtETt4hydYxvyzF/j+B1TemUTcR
NHJeWFjyJ4MndROt5jK+l2eJ28iZCcC59a2WrUgu3PF+YzrwlYCnLy7P33JN
gmr9lraBLUVw7KMiwzzGzStP5PS+1STQNLih/i2lDN520DLmvMX9WN8h9fBk
HGRLfFP5AdXwU12h3g371Um3mjbXI7Gwodn0Zsf7GvDalRLIguutb4ZfgGKX
As3SddISCXWQc+Bp8bMEAuzgp/PddyGw7eAL6V87GyBPb9e7nnDswwWaQ/nc
XoI3x9xLk7EG4PwZc2A5APvVxOxx+6AQ4GQfO7rpSz2MTT1YcYv8W4f2ok8b
ElDmUHHQ/R214G25yMWB+1TvXLoHT9cIan3AgKSoHpPUj5jSKdF/hQgFuefv
TbKA72VHa/bxQKBaT7FYwP34VOPrp4riTDhI85rFZ0SB9Njvgc4Axm3nm37l
Z8mZIOXC/NR7WyLwLXx53YV9yJOqR9nVoRkgvhEn5PkxE2h+eLNaYl6in2qc
aavKguoHqv15eQWwShO8VQvjSWpSu3T7SxaoS9OdaIsvBhbLhxftsV5EpHlw
mzOmAv8e6t+H7pTDnOLtqKPdBLTvctZ+PZMMP2RVfQ2cqkB9npfxKMZ5g8ai
Zcu9TNC2FGq8/poCKz4vo1zKCXiVnNnmWxoBbFF8+p6sNWAnLFZhX0WAs+DZ
kIc99nBDQGVCRBLB5hM/Sh3+5qK7NNcCPXXQTM2pu7PHKkGvzCk6B/eFNGPr
RPPpYgRf7lGK1Ivgru2HCz3Yn5RulT7JnF+PfIuro4x/hsEdbo/fJOzT2MRY
rFYbRtCDE+XGHpyKlC0hrULcTF/BxZkzIdKsAEKKbwZQufnBoU8Han5hn2Zz
sjewjr4AmGf569rsb5P4KmhzurDexSmntwst5IPg1srMrVJxID0Gqn95Pm7v
jg9fJPMhsrZN/MfeNDhCmmreh/3tmRVWHZ8zBfB7eWqzDGsu2MbwPvmbL9Ed
NDz8cDUfDHlWD988TQZG2qdap/C8+chR+0Xc0SxY32CLPz9ZBD0bt0XZ8Pob
heIN9+0ZwBr843l1XQlUvFx6U/oO9yl5t21ASBbsec+xL6q1FC7V5HVG4Xtp
6xbYq84SAbGGdB2BmaXwK6Qz1wLrsqWYHc86px7cPJQtkLhYBFoVetMvsW/X
pQ9a43gRi2xk3zpubM4HnsVA7Zu4764yjA6/w36Msr3q3MOuRAiYWS5trCEg
VqDVQeRZCzL59v/q+vJwqr42bEOFioRkKlISSooiylNJJRQakUSZIplSQiHz
PEvmeeYcwzGEZZZ5LFOmFMpwCpUovuV7z/v9rtf1/f5d177W2XutZz3PfT/7
3vcREcxnV0HblQLODxrieZaqlOx/fkCBj1pqLGqNyg2/5+/e7zENR488UJ71
yIcCKpYd5BEPYBRUSiXj35U50uw6IJQPkcsPfF7dVpS9/Xik1RvjtOfinjcG
yXkwwaaSfCQuHHaPCEV74euTiNMTSjx5oNvUKX2sLB5u3TQzTMB8JFtC9a/V
mzzYrHGBdgdTKnCr9x7wxTjZoOcN9Y3NmNd8UGVoysmExENN0wYY74V90+mP
b82Gnee7f1jQEuBqRcz7HRhvd7fl1c4YZEDnHqeWo0QCRHM6yDVjHvT9SZP3
C4l0qGDZVyIVTYDBO1U3fuLfdaq1bTkhGgK2MVusD6dnA/s3sXcea98F01AJ
e9o/QRc1+F8e9koBX9pQNnFcF/h8+z/tjM5A73+NP3unHwlis3eHv2K8Oqs9
dr1kcxVy2Ru/lbXcFno/DchsyiaDQ/mjsLuqTUgyVvvI/ngP9J3n86HXZ8kA
gc4u0kX96O/nj6/OhxuXx5mqUk2wzoAh+ZPr7XkSUG15kN4s8hIcl1oZUvA6
CMdwq6jFkWDz4w16cUHnZPUebvw+g+OQN/M8v6ccCZKqynwH5gJhNDI62QeP
Ny1Y6h9UKQDy3k06pxUjYNvEpuQEnB+y/bdoMhgXQEjM5RyvI3Fw+GKXdiKe
ny4cFaYdz4fTl0TpfDgTIfeWPJEdX996cfzUOwcCyDyiEsoJSYIAxpKHafhc
yHvp3PklmAny3R/izwongcgks0QYjreWoAjVcuEUMLw7wyX6KgFka04deP0d
r+cTn+cp93xh39byguDKGHj2Q0H5AMaluks9X9L9/VC0U8qYqlAguLIErJRi
HKJ/a09t44tcdIB3vlr6qgmElJDK3fE53cUdbBrgXY2UPx+y/jXsgn4n3f9m
jPHVbJZyxu6QZqQXz8X09VYA2po5r/z9MM5Xi1yiJta9qEDuvqm6qGF5XYe9
xK+AGWj0enpIMaUIKp848HCWPwE7tbyTDficLlJ8AKLX+QD4MXxm5TMtgu4R
vq3vRk/IJhU6d0bi8/hfP4Exip8AK8VPwLGFdvvviUIQr18Yi7Z2g1c58llp
GJ//oPgJRK/zEwjO1hpkfE8Cl4yIPGkzf9g/Qy16Ee/LCYrPQOc6nwGVk3m8
aVdJEHE0z4S0GgQvO5dt1vr///UfuLXOf0BY4YHySmg+uJUrNRS6hALth6il
p2P/+BIw7/1fX4IE9buL/F8IIB7xpyxYKxT4OhbY9k//41fwap1fQTiyZxa0
zoA7hUciz24Ogpy4EIlGXO9cKf3b0XX92xEan7GLwgmwTA56eyfHG6RUJQyt
MI71pfgbeK3zNzDwusaQ1/4CqF5wzzP+tgGZVY/A0FUymLn8x/dg5fP/+h70
ZguWavwMRzdFfPhXJyzQtzlrhws4ro5R/BBuUPwQlih+CFqvWHyOYz75u+3l
z7RTPihDzVhVGOeNSYpPAtc6nwTz57zbTy5XI5qtd4RDAkJRcQD9Zd9wMjBR
/BPoKf4JyhT/hLLAtzxRAc0oQIbBw3hvMDpToq1RjOOziuKrIL3OV+FnnPbn
lLvvkbtFjUBgm17565dnmWeWZuAFxW+Buv4/fguxFL+F5uajltbEYrBZNliY
g8uoiDDLHY/jNkdq8LJZZjEc5xk/obBvY/lvxQHJPDx+5bRmseloEbAzmXHL
HbqD6M88lFrzDfhzR+WSTEYhbJK9+TJA7CHi3an+fQDH27W5rsCq7wVA1dtU
bmL8FLFSsb3Iw/nWVtwk1Pp2HpxXEZa8LeCEHnaHCkpOkoHe3372Og0BzKR4
hI0WvZDyds7r4jivviFFb6mWTwEat82PdzcHo/xaG+MeHA+fz7yjYwsIgfaw
WXH5yQgUfDaVsHftu05ypiFR8wlSiT/MWy4ej8xaiGIEjD+NXbwisjoT0YjD
G+cfcsnIsZpV5QnOM9nXzikmahUi0ZiySF+hVHRwwfFvQjfmTSYpY0lpVYhz
YfOi1MUUJDI57fQJ540hn6jKYrpGtHx/9yVr52hk3W12iksVx7ks7RbG652I
TXpbaaS3QbmqmEd3aeos8AnPns6mLoZTrEqJXGMWiFFx5XZpL35e3QHmAPZi
EFUJ4L3mcbS8mGWUKQCP72uJ5pWVKILMs4d5Dnc7oR65A+4LOH4k+RQ/nlgm
gZEY07bMGz4ol+mTDxs+72liJVEjDflw6WTcuadXglHRFo/Vq/icGi5XXP6+
QASL6XHbBJrXiMWkTeTgF8ynxoIUJuuyYF+gqtxv81hEvn7+4TjGOQyWmi4X
9BLh0SYG3aEbiehtcw9h0zwZeByO5S9ZeYFM6N+lhPcpaC7SbPXuTzLMJT0/
bWXmioZQo1XAhQw0w+XAO4yvZ7gmLx1GlYJ2tc9coGvKQjJiJ+69w797X6mg
362FhHK+/BT9mJyFaKduzsVhXKc5xqx75Fgl2tlxUeTbw3Qk6xu7KwrzNfNN
UUodn+vRzbtTBEdiLMo2dr3vakyGmifUad5b21EOxzPdi3JG5SfdOeVgaRZE
VnaWVTkWQm0HYWnbPkckcbT0FtcgzsNRRP6GtEKoVz74bjXxTPnlmPGIGRyf
87bbTAMqSUAk/I37SBuAlAKQ8SO8nqZX07f3hxTASWFqVWdiOLq9IiJ7CufD
JkLQm1KuPFCY0v5AJxqHwh/GzrthfNLkQd1eJEqAAUb5vTvUkpAHncCuFzie
+/wUXL8vpsNG5+jMkMg0NLeg2qKB69EPcdec3fpxIDooZLdhMguxvxhmW+s/
awlwUwmIPwNeTtfXbTkEFBvizmGE86RNn6/KzV5fxPXNdop/ey7aoRpzLxHP
UzVW5ut0MBWVZdM+PFeYi2INnvZ9xfcZa5c+/GagAJ0YnqPNbCUira27qEVa
yaC994bPzY/laFC1eCZaKht1ZNlr1mMehDq+bNl7qRZVXjo/yVcZj0Tza2xU
3clw4Gh25ptrjYjV0Xtx/16Tcg/Fzq4mFVxPTbclKe0qAI5Y3Q2fFD1R4v1E
GSMcb8YeTHt6VQtg+zZzVzMVxXKqlawhTzxevud1T6BlPuDC1mpI/xptp0ri
FMLj/klNuRv186DgKpHm5MUE1DDf9Dga58/0vUt1xD4COB98kFcqkYZWMujS
L+B1rhNa7mdhyoSrVIMZPYHZyJXd+xbjZ1zHc7o7zoWkwH3LtF+jJCJinwyl
Ssbz3/7wxeN4eyR0+zsNtXbmoUF7ccH9eJ4rSaRhmiY52NPAuHVAtQDps7yy
WrtPdWcxox2FAajyeN+EkygJaa+Qcn/g+iga8fCPckcKKviw00xbhYQ6Tkgc
nsL1mlFTmZbaLh+9fLt94fmmAvRQWM73F+Yj4r/UloXHS5D7hyo+4gciqso+
fiEI83e/Q7mt+teqUEbsTV9iTgpa9dU5XxBPBu63rTsunaxG1VOJRz06LMpP
P42yWcTjZ70n/FcqicCRYR6d6BaIWmnnzk/jfX863MQRvUSEGAL7eTnNq+Un
Tr4rqMX8RWDlS8Lf40SYlPbWrLwajwTqVmmO4jhMUrpPMrhJgDMM07U6E+no
/Vbjmnn8vGqHqzyofmaASV2UIJsuEf00/NLyHsePUEPy2WszyXDajf+laXc+
IlG7k8LweXmVfnCbom0CqCVeXmGKK0Tj1bSqLHgdyldtdE7khYKHiFW2X2Yx
espq0p+J82dBQWeSYv9DZK23JWKpvQTxO7Y+GOnEOIfxzmv7z3id7exI1J9K
0IjPOJcJzgPH7vvt20NMQudfXRaS31CC2E50Hz2C41a90FdQ0CwX3ejasIF2
vBC1ighpDWC+yTpobG+rR0I3kt/HbP6Vh/JHPZc968mwVCIzvkouQ6ff2h+c
HchAFoP5ZwLf4Hz+lNqjwomEnpST3j0iWpe/vnvW9Rx+Xg7yCN9McDooo21f
igReobMbJmszMA5ZukDqqPiRDrMMq0lXLNXL4TlDTDzm3UuEtLuBrWkwoiZ9
2uJjCtKbpbqQinHORgnmOxXeqeDvdtSzxpaIYn53Fz/E6yw6nAwX2xIhoCOv
cfYlCTHf0PR5jOumR+Xh2F2fYiHJVOOL1mQJOqpe6LUXP29Cd2OBGHsUKB3b
M8WhWI7azcy37qrG+9Vppqls7A+hD6ZOxcpUIKPNJ243YR6d+FXBJb3HGqWG
JlrPkyrQ8ITTbDHm3dmuZek+PH5oReB5zVudChQydfppFl4HdR56nrGQOGRa
w5xj+6gcUV28M09XSYapg/6bOT9ko2G6uE9zxBK0mWPbdq0ajBvbpuWaXxBR
C38PFR1DAbpaFaDCgvdL/VZfygkGEuqL+UWr0oz5Tdu8wRPMT9No7tea0r1C
+ZExMyOfLMvjDqW5lmMc1WFroLvcFAWC/pwMFnLRaOWR/It5jKM4rxg3zdDg
570QtsC2mIX0qDuY0nDeO7dJS+dIeQRoX0/SdZgoQN8rcmy+4fxsqdGdsTs4
DCrSHis1HSxFTDdUI7c2k4EYsFgVKRkEZP7ncavvK1C/E4H1ZiEZWLx37+ma
9odI1zh+06kqNG5r3vUxkQyBrRqKki4voUiCedBRuAYVXAsWVMV4TNL8wu1g
YzPUemDOzSa3Bu39dtNuTzDG+ZW1s6ePvkSuxu93qu2rQR7PC62D8PVFFiZ6
b31CEGlf+AW7Y1VI1/b9ch+uX9q6DSPsVElIblskXY9hOaIpdZBgQpi/7BNb
2Kqbgt7ROZKdewpQEHVXlTeOB8ZXB5R+tqeij9K3n2sZJ6HvkeYMRzD+cWpL
Xdg0FgsFO/tsd7ZplS87vaHzwjjHh+HAeGxHIOSclP9qoZAIfEb8aaJ4PSOE
m0uMWQPBLXxbANuhHNh85v4JFzxP9vWxnc5SwXCX46abPnUBRNR/56jD+dZC
PeDUwa/BsCX+p4CxUAmEFrJa3e8iQ787vbDUYX84o9clMGeGgHvursh8Ba5f
FF3lRYqu0oKiq5SQMXBoT/aA3p5ff23p6qE9et/DDBvM+7g/vNBKModcw4ch
NCcboK89PmG/NuaPOg3FhAMRYPi1kfEUbSMohnC0cauR4fLBbasaXq5wM723
zGzqLZyrzeZHumSgHrebbBKJRD6PTvNyHKmCoofRvQVpZHAhuOSKO/Qgujs2
TLqxicBRJtATWDoDhfYeS1O1cSCtHhJbbpMB/bS7rE7jdVD38c6YPp4AV+zP
RVln5MKK2ug1fVwX2HfsWCIOJwC1Ab/b/juFsNvtTH04zm8CepcqmOVjIS/t
vL3X9BtQ3Kkp+aORDG0UPaTaOj1kiW58/6trLnBAM35x6VoNWBefmPQMw+t5
uCjIU8AXrEwP18u2VEPfDn1W20icJyP3Scn9iUaO73MUpd0r4ZMH0y3qHDLw
0wueVPydj+5nHO/fIlUA3cKTNIfeYnz7sk+6iqMX8U0dsLnCHQpV99NvcyfM
AMdw3vm3P9OhWvDghputObBqKUhqwM/lEm049as4A+jlTYW8FvLgy1UJyQ2Y
V172KD+1aTIFwi+lMNwiFEJRx/jcCN73YoouMZmiSxyi6BINfVa84sas0Zvy
9D/kc6Ww5/nr+7Ut+H7aTGLzLmUh1aai5zuaSfDU7azNMs4PXjK8d6cfVSDf
1uz3Ln2pkFKSo2GWS4ave+/0ejO9R38XNES20AaBwvNjstH0s5Af8rVTa4kA
qR/bSk11CWB/+FHpHozfjvJduuyrmQ0dCcT+RNk8+MmY0qCF8fApin6PuE6/
R7PRsebGrjyUffbQcfrFNCj5Jsfth/NG9lLJ6pvBasQdy884E/AKxmI23L6F
eZYDd7rJQkQXMrvHIvTxvDm8UUuOmHk8CzVicVGeSUT46zGx0GaUBQE8zZ/m
8Tw9FN2ayTrdWu41muW457WIo+jtdyM/EzjQ0KYS7UWGRnWmn/N1HShstO24
u/Mz1PL3gZll2SwYUPRXmRT9VRFFf/XGYeJIiF476rksqHl1xh0dZ98HCzOz
/6/PYP8vfoO+/+I3WPMvfoPK6/h+HYXXb17H60/+C39/ReHdLut4913P//Dr
xXX8WoTCo6+s49FDFL7Mso4v01N48UYKL75M4cWlFP4rs47/nqPoqajX6als
/st/qRz+L/+No/BfOjGr3M6lIghTLzI4+lIH+R668WwM8y9NXzc3m/ZC+LQ0
z7fsb4rmDpklGeN1q/uVU6rNTgKWmVVHm1dPEZuMRocy5rN2H/T9PBLyIHqU
uewXrRNq+8Af4jlBhvw0LbYxKwJ8TWy3vCDthRQjGLKu4XUbzRivHzDVQYmp
uyXscuLQtYujNTtxPhd528ocHB6PxL6+6dTMTUIHlp7ES2M+df9Y/uPdwQXI
4nD50HBzCuLb42TwA+/viNqHOumUSmTYgxrUhFLQ2yauXsD58F7UEddV+gZU
+8GEiashBv14YNFdqUWGR7vPPyl52YG2JdJc/J5rgK78JR87WT8L04WZFgoK
hZA8duJenow3GgiK7juzppN0LunNOVUANkO+T9NeBSFjBRtFwPVC/gzncENU
LsRdVox6LBqOqvaQU7vWvrsv3Sd/0ywHjrJ7p/C+ikFHBZluxWNcxPYpVdPD
NR79MjZCTc2ZqPCslUQq5jufC6sc56LyEP8f9rSHUlkoSE7Q0QXve9HW2Ynr
NAjZ3i6wOleUjpSCJ/c8KiJDr5H8HUe2OjROvdxFao1HdElez0Je4HEO39XP
bi3I79l+lU8OLxA9gcq3gn/NL7q49/PvPCBSpTP2fY5BvV9/pa35grIdz3kp
JU+EpOvBgjlcicgvqeM0Nb5/h2oat+DjWbArIp2dKJmKatTlnzz9subHvjGP
HvOovGtTSnoLRNSYxNjxDdcFtWqFY6lvS5DepnRUQMhBuQOn5QHnyYROmY1T
/VWI/amNjFpAMtKQ130cEo35EdA+yHaqRy1pagannD2QBgrQXbYkw+rJjsDe
lWw4q3LZ89PJTPSCJqHeFuft7iumd5a40+F4FPexKl8CMk2+HSOMx9nqeGO1
f5CQYJuoqr1pHnKXF+o+i3HvrhNzDaZHEeJW2GAx3pqFgrmq2r/nkyGLP/9+
uFUl4s+pEznRFoRkuaV+ThExjtrh32U5lQQap4fGtLeQkJtVsxz92vdoZ4yi
Gv2KEPO08kWOWCKq0/Ksz8J4u/1e8vPTJUVI379c49vpOJTBXiG4EeO97XED
drLOBFQ74/lcVzEJzeSnK659nxgr4jB7tDcQ+sJXp5VG9OCX7A/tEJwHSBYv
9OJl+pGHoVTEgTYfWY2NGf3HxGbAk/K+23fd+24W7yjpH6R69Id213tz4xTo
9BKQsjQigxLhnO2x4GgQuq+qtbLNHNF58Dqfx/lHOuUKo2hkNLD47it/lqtd
/ohKtf0GHqeWYBWUEAyD667zt2d3nEBj7mUFExiXMgYxpmpylqCfz9zrdqYR
YIReCQ3gfSyh0dQ1CHiLGl1yBBZYC0F+g11xrjwZ3smqLTCxJKCA6Pn7w3OV
8Nvc9r0qPncbbv3nvbM+5b3zW8p750X9xluvH1YjzllvkqBhCTyr/m4+bEUG
xek/RvN+fuDFknvvvE0VtDYHu66d39e+IOvUEw9PX2/pjy30Ac1GRpfQ32TI
6Y//nSsfDbkhRpalQhXAnbx0vKOEDDKdHX7mJxKhME2ViXdrFcg2Svfnp6y9
R7hp2yiRAfze6X9+ObpAWtQPnd84nzRu08zcNZwO8VMcByUqIyFt8au3zXcy
GOTM2+wQSIIdWwmVbIsl8E54lOFeExnMklkKrjenwLyUljhPVBkcID6W3Ilx
YHr/+bO7lOOggOWa19fTCPbcdN5Cg3lu3YthHgYggsGBg0qXEuzA8UrzJTF8
3i+YXpkuEyKCZD3byV9NoWD7enOC+5qOLu8StagPAWTme4vaF+IhPLTh8lod
N5Prv5BSnQZmD/eqnbQogGX7yZIGnJ9dX8iRzl1IAwc1qoIDEySwUlb33o3j
8I8TmbW6KQHGs9kevLxSCPvyirk78Dk9Xj3KHvvVHcKv7jn87VMBLBLP6tzF
eVs88aK8umI+0FC7hL+ZfgRJM+6jgrjeCT/a8iVXIR8sldmCt773B5UanRze
z2Q4439dcTIsD+Jq6UoeCUZBYOnT2YS192h6HiXMknngFRTE1XI2EZI/u3ZY
4TxZP5/z9ef+LNCP4fqTSEWA+YWVgTQ8HnP25pjURBowDDxqDzIjwOL7XI5g
PM+5CO8Y8rVEEIk596t5KAfeFT5oSsDrILxvU+TzlpegEZ+v+PtvBijUkk5p
4X3MrrnrXmYWgSoOmAtO+iQCWfZHluVPMvzWGZwIIeB1YUO6tZnqcINZcx4N
4/lT/xyariaBlMqRP2pXXMGdMcSSCY/zSTgpDawWQOnrzPFnssHwypsrbe09
zsvKwf2qxwpgw+NoGh2lSHBaCZbNW9PnfzcYlqTC66jAEyfsHwf+irB77f3R
rusq2zSf5EAgg2Sox7EkUN6eJbeWP+l8ve+GH0wHdwE59c0RifCmSSggEdfB
Uv4bIioaCUBYvNKXFBMPbL5e7kWYX29W2cv5mcMWbCvvHLIZjYQfuyy7/PD5
lRW/tqGfHI08cp9VB3L4QbE8uVMF183lkcQjRccK0eDrmBlHk+twTOHwuTMY
p/2apV9YMi+Cy2zlwnVmEpCr9G7MH+e3J1N6kR99iuDgckGN9sJTmBbctJkN
j6vLaysUVhWC9mS7T2GrGwz93cW2A9f9SBttegsiCdzy9IPsbf0hkT7mag6e
f+kXS2vqVhIwXN7fTUcOgpaTkk2reB1m80L0E/blg4LQbav8/aFgeflLaRbe
X80J1q0aOJ/ffrrhS4dcIDDya40x4OctLQA2rhtxQI30PuyP8oQCa7oLkWt+
Avp87UGbTGDo2Y4/+w0eg7NdDUfPKhn0NrDOxwXEIl+lzjq6vsfo4IayijC8
DtsuByVe8SlE7yrkip2RD3IRbmu5h+//yj06vdLlGrTL9Enej8shqOP4F+e7
fv/ozBvX6cwlPOOePzmVCumxSk9HuIJRTmhpkswPMjTzXf2tQV0Md1Zfv+M0
N0dD2oXZFfg8KlH00vzr9NKpH2KD26KToaeyeNH0SQL6mf7CmmUO847n995W
7gwGyXtNo23kZLQ6FK53dO17kG/iJ8y81/w0aj2T/WyQ22y3PzfGIcV635Vu
MBVC9dCdvkiiD/KKvcrSi8d/UXTCj9bphK2PbP7Av5wIe22iec/oZyJS3hD/
C4xDOuwEz7NM+MN+jh5j180E9LO+MuwtjsMwePpQ5o4NEpU+45dhQ0TEo83t
b/D1rxSGzqxIFMCzD1IroSwOyIvnCW0Yxg+yzy+bKLbng5+y+11t2hC0t9md
Zz8et1TyFNy3mAd8F+SEkitikdBvu8AEfN4NKHpa/nV62sToJ4qWEglgMzw2
e4aUi/ZvMnG+jOPkaYD6U+sRH/ijP3Al2iofHY7xdZtd+38B/o0JKlH2yECS
jWfKpwAdof6hkovnt+QeVOWIi0HRpye+ZXCSUGdH04nneH3CBa7SuI0TweZm
KY3rogPae8yImI7zrWXyd7vlMCKokOFZX9drdOOiwt82fE4/7wzijZkjwDO2
8WOLJ5MRdynT5AWcD+35tm423ZUDYlTTVpVPspH1LScXFfy7vyg61VcUnaoj
Rad6WN6XVbA7FkxpvLRqTItQ7aCm6wEcV0cqpHmtZLzAz+HwgTH6EsQcbOTR
3E0GCxUFg2/PXiClIemT6eElaHZuqdMI89CLO6QX966GI1eNzkEl3hLUcnR4
OaSTDKezl8LvDaWj99fzxX+VF6E/Kzljiq04/5tpnOkYx/fznXdugmSHlMoj
X9vi85WToRzc8jQd7o3cE9lwOAYd0ynZcwrXu2DZaOOe2DRI2sc1nSKYgYY7
ZmsPf8X5re56y+EfKVAMehO5P3JRlUfHZse1vnoBqcxALBFQqCYstxSiDJXU
WG78XAcpelHZdXrRoK+Eg+2bI+DbX72+n4PlaCJAnVqzkgy2wyUizFPucEqn
fu+oZgWy647MOVaKeUr9sxWj3fYo2oPVkD+8An0929wVg+t735OkdLfFQBR4
n5c1ZRGhq82rNyxwnS1rjI665ZiIiPwZYdIny5AOs+iFxrXv4k/9iZvzyEHm
aUJT8lNFqPVtlP6TBjJc22S2sbA5CpxjvnYbpsehG2G+ocm/yCBnYzRQTRsF
lXxmd76LEdDyz7NRyjj+57fMyLCjCHi4pyWPblMhCm3SV3w7QIao0TdCf4PD
QCLd3Nz6cBkqLpT5q/+WDBnbzm2/JRUELmnGG4xPVaLWe5O3x/PI4EPRc7qt
03M+HE0zk3V5CWUbc8U/nq1B12x6E2++wvhNVv4UvbEZEoHlGSPvGlRvF7M/
JwTXu2KOIzxHXiIaZm51z7Bq9M34pxB3DBnMS5h2L3mHoJIBuv7fSpXoXtDj
dyu5ZGC2oP/USpWEcsSiklaYSlFT21aTAoyTV5roHaT1UpBkYLxWtFoeYswd
qd+K9zeQopPsWaeT/Lxrd2swzvv66Ul+tgcbIG+k4uf0HVw3K8wjKj8HAmvO
L1ueuXigK7iosYL5bHwjP8+UQiC4dCWIPLPIhNaLO5wYcN4zjFwQ1FYOgtDD
IdpDv3OB+UTZEtUkjpMX9qRW52Do17fR2/i6EN68SH/5GK/zgJBHi5p/EDjz
7X55aE8pNKWkM35qJsO+UxmSDnz+8Eiap31UrwI2jfFp2rwhwzdKX+7Cur7c
qV5n6jBaH7Dzra6rra6De6Fyuf52+FzLujdpydlC6NUrP81z3sJd2y/FdPp4
fsXE2hNGAUB2Dw1f/dAAdxe0hTuv43x4haf3tEYw3OefrTQebYBiz7+K+/F4
6oXnbCc4naBuLuPPhw1voe4t/x1BU5znQ7uvKY/EgQzbcKDSmXRQYNzqL7Wm
0/77N+FnbjyY3Z6uO2BLhHc8EY5r/h7PDL1UonITQOrjkglvbgH0qH2eNsP1
Czkp1xwviwev/ZErCWXFUHfrp+GvdjJI9aq16jDEABU76y3F6jJIdJS0TK0h
wzClL6e6ri9Hs9zxJYzRA4a+JlwjKtQA9YCDQlcYrl8f/NokIjzgDCHsSIlQ
DcheEgtlDce88oS6LW+bBdpyU5DqZXEVzEUn7mjAuNrJfObsI0I6GHd+/Lql
MxuuSw77/cH4MOA+Nb/Imwz46SArf3prHlyrc52SwXlYM/QeG+fldNi4vUZv
3JIEJe92bQ7EePVj4ZfnjcNJUFLMayqUWgwnIgScT7aRoYvSf0tZ13/zFn83
u1HJEM3d1r9VPVgKBxbKxwoxPt/2QUq8aXM8eiqquv2xXxF0JdHWcuL5PQra
9rGMFKNuDV9PlqNEGOhnkLLE82iG8bIfCH+LtoqV0E4xh4E/HV+emB4ZHsXO
+2QyEKE5C/ZrbyWAtMszWQK+/93MylJxrAToT/a8T1ucC8Oy98719eH12XJk
N397OuzZaHTlOX0BCOh/OroJj/NR+nL56/py4iK0bPGyBFQq8cO5zD4DNotF
Hlh7Txdv6Wfy82UlUvPYwlHPFQOSIS2RvDlk8IxrtEw91Iw49X4TtjyQAC5t
6kA6STLknm0g2PHkwugdw8RukSwQSFr9vgfXHb43E4QHZ7JBidXltwsNAcYr
j36Pxji/gdKvs17XrxtbJCTq3apBiV3ncr1rLOCxu1RcDY6H/JCGQSJLK8pm
74sVcHNBbofSLaW5cL0jcT44u5sA7BoKXDF6SWD3QFsrCJ9fHUofL3tdH8/a
z7KBkdiKbEjTI9c7fNByfRfwMZNB/l/8x1wo/mPb/8V/7DvFfyyZ4j9GS/Ef
61nnP1ZD8R+jvvwf/7HAdf5j7Cz/f/8xev3/9R+jofiMKazzGYun+IwlUHzG
/lB8xnQoPmPbKD5jvyk+YzUUn7GLFJ+xpxSfMTGKz5j/v/iMxa/zGZOm9PeY
1/X3tlD8xx6v8x9zmjHZNMVWDLoW6ZtVdt5FwVrfDgfi9bRa2dtps7sIQk8o
n/u87wEa8Foecsbr6XDXzP4r5kfMLtvb5K9Yoc2Sdc7mGE9OtDT+GVzIB3t/
a1kjPXvk4DQgpojx2NwpZW9291zwC2+RTS9zRtHSsjLsGD/k3iph15jMBtd9
ph3cRT4ozP6U8FGc9/wLovS/GF6FYzrzpIOCcSjpS/ElPbye+5nOP9spG4nu
eV9NV8pLREFSLN2iGA9fLzGjHW8iID2qMYfPAikoK8KJbwHnmUgLv7dR9qWo
wOL4PeOyFGQ8YHJbFue9dK37EWL2NUhW4dDqtUNJ6GzC8q1zGOd3Ow5xX77a
hIYLw/dahr1GV9X7za+eJsMO2rCOx2aFYDK46cyTdE9UXXTV/xPOt1TEoiuy
nQXwgTOh6vRCAOLZFDsyhc9LOtu2S5/z8iDzMbUnLQpFmh72cZlrPoGcgY2s
x4kQyiblLzIcgXZhomeC10Fd/Jh9bkgmLJiNZk3PxSLe9LKpYoy3Xxd1taVp
xyJmFfExee9M9JXK7/A2HG8R1ucuDX4hoDO3uRIOPshC4vFfvTzx724b/nGt
Y7IEqbyWyQ4Ww9dnXtjFi+NkdlOH2/t3VSgmjuZb2acUdE4ky6UZ5/nvDlc6
L/S8RcQTLP0s+6IQW9yXl0N38Tni5B9SVcqHnEJ2R5nUaORHH3fTFZ+LJxOL
KazUuQAy53TlyPGo/F04LSfGvYKCycn1XDlwa/7hOHqWjOI2Zxss4PNyaYu3
pVJdGtjxy65qi6SjWzQXb3/Cz+v+i3PJMo2AbsrGmgRy56L7Eow62zEPkitI
44hsL0JCRt+4neMI6Nzrs8cYMF76VGRnzN9WgTQf6n8tFcpAsie3CRhnkYHH
bRu7dGktkjbitHx/IRZNWESUdDmTIbkww5YuMwcIoxtXPz3LQKEl0zvX+oR3
3uxzYtmRAaNdflFHenNQXGzn8WQ8rj7qYbLXPhlySE6j5T+JSPym+uNRnG91
XJAC100S2jkhQfTtzkMSzgzda/ez8ZefZObWMiRo36OlrEtApLRHn0Ux/ty9
UaWpgacK/XbpHheVTUSRC92OnLgORsTbMD06kwzmZ6aKShMK0PLe1fHJNZ3S
08MSC6/i4ehJWIj9XIg0UiR+NOP89jkh7/runkJ0TbJPrtMgFx0qDTblxnjs
+IvYoWsSpahclHPP3NVUtJ3v7LZOHM/Wtjf3uDJHgrpj9V734HKU2/7JULaK
DEdr7T/qXMhD+z9Kv0vxTkV/5d63KAz/831E8rrvIywnNTLFP8ZDyidPHseQ
AFDx/VMkiXHdOOW7gL/rvgtgGzthHNGfCCcHinOIN6ogZab/XnQiGYz2cTzx
0syAjS/myyON/OChSCTj9XmcT+ZqTV43pcMiq9cDiSMxoK1H9P2Fz3szRSc/
v04nPzRsu2D3Mg2gVK/8tWE5iI6POVRg/GZg1dGwNzEG6tqp7ur4Iqj8NnAy
pIwMdjolvCv3ifDk9wj/bykv+Nxn9+omzp/K4y2eK1uJcKO5nN9nUwToHNi5
eR7HYe8z83L+ZwTwkamO2rOSABZ7nrRV4LjtpOjJWdbpyecbeHOoaTJgB/nK
6Kp/IchTlX2q6iBD2hUjw17hWHjQFK8ual8IdRGO92/i+n7et/fR5+8vgU/0
VrlhagHsnPlCOoXP0YtpmxcW9vkw8cfzT4ySKyisHKSTwflhdRfjnx7WfOjq
Xhp4yxMKgnRaHX04Puu/zJxz1M+Dngg1ublTsVAPVU1r/PdJAk+Lv2oeGOft
yGTbmAQ2/V5/SbiOv6fotDvW6bTH2qflGAQwvgoS52INIYDgkjArO56Hhl18
k9vvSMi4FyA/JZADHu4dqp4zZNipW2Q+8MYELlm8e5hyPB10LPMJEj8w/nzM
/uaQQyxq6soUo1ZIgImo7OQZPP5jqH7ozCQJRPqX/rx1cgAaK97OIVxnr1Sb
W2ubkuDe5vT3Her+oN6TG7pWL7JoX/ow4nWx1tpgMlETBjXUfwyNcR77a7Ip
X/ZCAZx3P+T6+2o0GKTxUa3xjveP/ZMThvKhVPf3oMGfOOBKQEupeJ6L/6KX
3jcx6siUkQLHFEt0lasT4GL/Ttc9mM/2udGtXKYPA9UfEpwXmmLhU89FgjOO
c/NoOjH+jRaoj8WIpdglDDS3n75a9ZcM/KzXQx2Tk9GpzLrv4Rxu0P7xzo3H
mA8WvPfudR8sRm9S9yo/jzqPZCO2VbPi8xvp+M2aEFMEzmqaAW7uVlCw+wHh
C44Hi7FTW/R3FoEu/Usu9qvOcOld5sVTuJ6e0Dgk7ShTCJeuiElZ/fWGWYZM
SW2MJzkmbZnlPEhwzt3/o/LfAKDvSZKcwevZnWpfOBVXADJml8XvZQbDjhcr
Bzfhfb8Zp51mk50HBn1Sbq9FQ2Ff/g7zrjU/ZIp++O46/bBXp7uI+IFEOOiQ
8/lcqw/wSvYO1mNepphkcEXtkB9kEJh3PBp1AOdKndJajFtkFyudD2R6oknV
Vw6du6SRwPlHi8qrZGCffr3faxrn2ZnQ1Df1TmjVmFrbGp933l/5M+9Pl6Ko
N9E5mmwBiMzhr9OGednEhYOcJix1aKaff0ldIARdumlU9vLlP322onV9tgqK
rvWP6//qWrf7VMc5r/XZqGTvcgnjfes/4lve+48vwd51fTZmim7TeJ1u81IC
y26+L/7guZA5KXcvBdHMTkdfw7ji9dnJQDWXQlh6KPvlt/YL5KnxsW2tz3Zn
pfFMxQIJFnv3i2Tu9UMPRVpGtwz902ezWNdn+0PRMQqv0zEaZX+5o0nlDPez
/P7WPSWgXKvbB+Jx3f9zrrFustQJKX0bEhfPJCL5XQMjPDieGyPcSmT2FoDj
jufar8rc0G6/J2E2uE5JZVaVzcXlQ20jr+W0SxhKjrpqJIrHB+WNV2O68sBG
m3rQVzcObdvAzpiK84Aepc+2f12f7QFFB9i5TgdoS/Cc+d1oBio7Oa+ncxWg
m9++rGzB4062S0fcdnugsGT/4ktDBYilp5HNDMfhWMmr0t72OHS0N1PgqSAJ
idAteShgvLSozmR/rYEI3XoBp01c/FFqcZQr/xTexwfVs11aeJw8oEC8HYte
89lfJOM8fEaSqicwnwBMwQmt7VypSF1wK+kRjueHvmMDtHbZsM3Ocnb8ag4S
Clzl3omfa57SZ4tY12drpOju3Nbp7hQlN+h6NIqhrbQ0IwJpJWjzFOGYVRfG
t7GVgYm1Xshg48i+23UlqLZn2yWVTjIQs8rffbOJxXigsaaArQTF6l7S5MH5
/95IeTDtcCa63eAmXBNUhJ548KQxt5Ch6Ku1fXZ8OrxKsPMZUg1BG+J0d+xZ
+18YdPihDV06DKfMy6/uTkLiNzRyTfC+dwjuMl9gT4O/pTY5g9ty0MZvYtNb
Md9f2txzmzCZDPV6y/SnBfKRy9V3wVpreuYO1baqhATgTuEftFIrQqdcupnG
8HMJU/psZ9b12W7/iy6uZNmH/s1jPZT6fTfDSEoFuvD2t7EaxjNcl94ncL/y
QHRqr+/KW1Wg6zXdl+/h+ququMh/hyUCaWSzWKlLINQQfiKVrgLjqNt7np3/
kYK+bUmmG75XiuJQeYwZxifc5RGHPjARkJREuBpBvgiVqFaOjzX+00cqX9dH
enmGy/DDySgglZPMMq3SkV2r9A0SztszwucLfipEglywyM7tj/NQP1sTtwiO
h9awv/5UP8PBoc3MKVqmGEkl2gav+Uk+1RwzGboeCgp9wuJ2T8vRTwajfS2Y
10dImXI//hQI12kmSz4/q0Qb+hjO9hIxj6b039zX9d+iKPq3wnX6NwK9QDus
3EWNI/XW2fE1aF+0v0ZXML7/ukNFOkwOKGl3WaXQ2Rp0X4FL9vkrjK/EhW52
PfRHaqnJYnotVWjSz/rX9mQcD9UX9I6rRiPasmURHYMKNGG0A7qLcF0QdwjZ
n5OEyl85JZdcLEFHz5jdU8VxNc4kcDwgPAWNCEREbuLIRQ8Ke5LW/OpNKe+F
49e9F37q9t3xrF4V5on3O90NMoF+55/lY9FkQJTvf63Xff/7fwB2figG
    "]],
  Axes->True,
  BoxRatios->{1, 1, 0.4},
  Method->{"RotationControl" -> "Globe"},
  PlotRange->{All, All, All},
  PlotRangePadding->{Automatic, Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.5755448533182135`*^9, 3.5865814787458763`*^9, 
  3.5873808551613817`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Fig. 11.1L Displacement field (p. 184)\
\>", "Subsection",
 CellChangeTimes->{{3.501997888493143*^9, 3.5019978927653875`*^9}, 
   3.5054112035119495`*^9, {3.5873813296379805`*^9, 3.5873813355257325`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"VectorPlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"uxi", "[", 
       RowBox[{"xx", ",", "yy"}], "]"}], ",", 
      RowBox[{"uyi", "[", 
       RowBox[{"xx", ",", "yy"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"xx", ",", "0", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"yy", ",", "0", ",", "b"}], "}"}], ",", 
    RowBox[{"Frame", "\[Rule]", "True"}], ",", 
    RowBox[{"VectorColorFunction", "\[Rule]", 
     RowBox[{"Function", "[", 
      RowBox[{
       RowBox[{"{", "}"}], ",", "Black"}], "]"}]}]}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<ces-displacement\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4147364425767975`*^9, 3.414736495884797*^9}, {
   3.414736557422797*^9, 3.4147365676447973`*^9}, {3.4147366000297976`*^9, 
   3.4147366300777974`*^9}, {3.4147367758197975`*^9, 3.414736844094797*^9}, {
   3.4318342077212086`*^9, 3.4318342105932083`*^9}, {3.4671045034277906`*^9, 
   3.4671045794351377`*^9}, {3.46710461321907*^9, 3.4671046443868527`*^9}, {
   3.4671046798358803`*^9, 3.4671047317148476`*^9}, {3.479204936175*^9, 
   3.47920496374*^9}, {3.479205273781*^9, 3.479205275218*^9}, {
   3.4847099985222*^9, 3.4847100000042*^9}, 3.5054112035139494`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5755448539302883`*^9, 3.586581478904913*^9, 
  3.587380855384386*^9}],

Cell[BoxData[
 GraphicsBox[{{}, 
   {Hue[0.67, 0.6, 0.6], 
    {Arrowheads[{{0.005892695735326355, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 0.07791651932302897}, {0., 0.06494062353411388}}]}}, 
    {Arrowheads[{{0.011437248239840749`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 0.15544972763819864`}, {0., 0.13026455807608706`}}]}}, 
    {Arrowheads[{{0.016611667437966326`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 0.23257541353685227`}, {0., 0.19599601503457625`}}]}}, 
    {Arrowheads[{{0.021395520327244892`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{-1.840603305469524*^-19, 0.3092710799715891}, {
       1.840603305469524*^-19, 0.2621574914569823}}]}}, 
    {Arrowheads[{{0.025774846631762185`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{1.8406033054695242`*^-19, 
       0.38552135646567104`}, {-1.8406033054695242`*^-19, 
       0.32876435782004315`}}]}}, 
    {Arrowheads[{{0.029733129351428612`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{-1.8406033054695237`*^-19, 0.4613080575500417}, {
       1.8406033054695237`*^-19, 0.3958347995928153}}]}}, 
    {Arrowheads[{{0.033259197568309196`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 0.5366188838736483}, {0., 0.46338111612635147`}}]}}, 
    {Arrowheads[{{0.03635285348747873, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 0.611453617661261}, {0., 0.5314035251958815}}]}}, 
    {Arrowheads[{{0.03900707033183582, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 0.6858045223241016}, {0., 0.5999097633901838}}]}}, 
    {Arrowheads[{{0.041217285894000345`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 0.7596665748022505}, {0., 0.6689048537691777}}]}}, 
    {Arrowheads[{{0.042997974042391446`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 0.8330557110456281}, {0., 0.7383728603829429}}]}}, 
    {Arrowheads[{{0.04436078444856977, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{9.20301652734762*^-20, 
       0.9059847575203321}, {-9.20301652734762*^-20, 0.8083009567653816}}]}}, 
    {Arrowheads[{{0.0453100989444296, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 0.9784585386901207}, {0., 0.8786843184527359}}]}}, 
    {Arrowheads[{{0.04587368524026631, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0., 1.0505076272276102`}, {0., 0.9494923727723891}}]}}, 
    {Arrowheads[{{0.0023213397854996635`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.06887274091396248, 0.}, {0.07398440194318036, 0.}}]}}, 
    {Arrowheads[{{0.006328012222636344, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.0688893500270027, 0.07791661888896366}, {
       0.07396779283014016, 0.06494052396817919}}]}}, 
    {Arrowheads[{{0.011658775511689427`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.06893847085382646, 0.15544979415076823`}, {
       0.0739186720033164, 0.13026449156351746`}}]}}, 
    {Arrowheads[{{0.016754908452777315`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.06901729042229375, 0.23257485456070806`}, {
       0.0738398524348491, 0.1959965740107205}}]}}, 
    {Arrowheads[{{0.021496271447803358`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.0691237628418384, 0.3092695178055223}, {
       0.07373338001530444, 0.2621590536230491}}]}}, 
    {Arrowheads[{{0.02584734037600772, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.06925416019792237, 0.38551798159534306`}, {
       0.0736029826592205, 0.32876773269037113`}}]}}, 
    {Arrowheads[{{0.029785088500370115`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.06940258788057772, 0.46130262351368956`}, {
       0.07345455497656513, 0.3958402336291674}}]}}, 
    {Arrowheads[{{0.03329503706005229, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.06956472789378627, 0.5366109307469993}, {
       0.07329241496335658, 0.4633890692530005}}]}}, 
    {Arrowheads[{{0.03637542961503806, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.06973596578937047, 0.6114426915354843}, {
       0.0731211770677724, 0.5314144513216583}}]}}, 
    {Arrowheads[{{0.03901880079279212, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.06990919812410258, 0.6857905615567003}, {
       0.07294794473304025, 0.5999237241575851}}]}}, 
    {Arrowheads[{{0.041220146938672646`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.07007920108644373, 0.759649660488957}, {0.0727779417706991,
        0.6689217680824711}}]}}, 
    {Arrowheads[{{0.04299340104219194, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.07023967706789415, 0.833035743692657}, {
       0.07261746578924869, 0.738392827735914}}]}}, 
    {Arrowheads[{{0.04434979287285577, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.07038184291002954, 0.9059614353635371}, {
       0.0724752999471133, 0.8083242789221766}}]}}, 
    {Arrowheads[{{0.04529346750650858, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.0704997322324374, 0.9784315763522489}, {
       0.07235741062470545, 0.8787112807906077}}]}}, 
    {Arrowheads[{{0.04585151868627704, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.07058503268648098, 1.0504761735552}, {0.07227211017066187, 
       0.9495238264447995}}]}}, 
    {Arrowheads[{{0.004645400936973129, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.13774248556247487`, 0.}, {0.14797180015181086`, 0.}}]}}, 
    {Arrowheads[{{0.0074834266086054105`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.13777588928422058`, 0.07791455849640265}, {
       0.14793839643006512`, 0.0649425843607402}}]}}, 
    {Arrowheads[{{0.01229604962161982, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.1378741732912687, 0.15544488293658218`}, {
       0.14784011242301703`, 0.13026940277770352`}}]}}, 
    {Arrowheads[{{0.017172355070609137`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.13803205832179108`, 0.23256669208585898`}, {
       0.14768222739249465`, 0.19600473648556954`}}]}}, 
    {Arrowheads[{{0.021788879478894643`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.13824556247071476`, 0.3092567597855934}, {
       0.1474687232435709, 0.262171811642978}}]}}, 
    {Arrowheads[{{0.026056448197112062`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.1385070171408902, 0.38549967522762896`}, {
       0.1472072685733955, 0.3287860390580853}}]}}, 
    {Arrowheads[{{0.029932698519385348`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.13880472532270244`, 0.46127768826970633`}, {
       0.14690956039158326`, 0.39586516887315065`}}]}}, 
    {Arrowheads[{{0.033395180413041244`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.13912981957119522`, 0.5365791912445997}, {
       0.14658446614309045`, 0.4634208087554001}}]}}, 
    {Arrowheads[{{0.03643697786207101, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.13947324032785918`, 0.6114032699981413}, {
       0.14624104538642654`, 0.5314538728590013}}]}}, 
    {Arrowheads[{{0.03904900726538152, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.1398210808488312, 0.6857433632398515}, {0.1458932048654545,
        0.5999709224744337}}]}}, 
    {Arrowheads[{{0.041225188695500056`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.14016244854690316`, 0.7595952157922412}, {
       0.14555183716738254`, 0.668976212779187}}]}}, 
    {Arrowheads[{{0.04297721845637361, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.14048526996845165`, 0.8329733756398674}, {
       0.14522901574583405`, 0.7384551957887036}}]}}, 
    {Arrowheads[{{0.04431545229012773, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.14077218463837568`, 0.9058902790008684}, {
       0.14494210107591002`, 0.8083954352848455}}]}}, 
    {Arrowheads[{{0.04524312956081025, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.14101083677410386`, 0.9783505764449327}, {
       0.14470344894018183`, 0.8787922806979239}}]}}, 
    {Arrowheads[{{0.04578589171999758, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.14118516981745893`, 1.0503832305516818`}, {
       0.1445291158968268, 0.9496167694483176}}]}}, 
    {Arrowheads[{{0.006986724752927476, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.20659322375406122`, 0.}, {0.2219782048173673, 0.}}]}}, 
    {Arrowheads[{{0.009099468282043924, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.20664417139032024`, 0.07790778726429544}, {
       0.2219272571811083, 0.06494935559284741}}]}}, 
    {Arrowheads[{{0.013293539801365087`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2067939763891127, 0.15543083677155087`}, {
       0.22177745218231587`, 0.13028344894273483`}}]}}, 
    {Arrowheads[{{0.017843476686114603`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.20703459593508428`, 0.2325445270096825}, {
       0.2215368326363443, 0.19602690156174607`}}]}}, 
    {Arrowheads[{{0.02226136056290597, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.20735933229635004`, 0.3092253493900938}, {
       0.22121209627507848`, 0.26220322203847757`}}]}}, 
    {Arrowheads[{{0.026391752911407464`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2077559918709809, 0.38545741108285064`}, {
       0.22081543670044762`, 0.32882830320286355`}}]}}, 
    {Arrowheads[{{0.0301667109200455, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2082072671738764, 0.461224496848371}, {
       0.22036416139755216`, 0.39591836029448596`}}]}}, 
    {Arrowheads[{{0.0335501859962252, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.20869953216676376`, 0.5365144352850433}, {
       0.21987189640466479`, 0.46348556471495644`}}]}}, 
    {Arrowheads[{{0.036528403156433346`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.20921848343653915`, 0.6113264055402778}, {
       0.21935294513488943`, 0.5315307373168648}}]}}, 
    {Arrowheads[{{0.039088943577255376`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.20974434606124123`, 0.6856543901321674}, {
       0.21882708251018732`, 0.600059895582118}}]}}, 
    {Arrowheads[{{0.041223832769544624`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.21026077314878794`, 0.7594949681122797}, {
       0.21831065542264064`, 0.6690764604591484}}]}}, 
    {Arrowheads[{{0.04294135705306183, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.21074991694803313`, 0.8328609760458008}, {
       0.21782151162339541`, 0.7385675953827703}}]}}, 
    {Arrowheads[{{0.044250278741966076`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.21118635497141558`, 0.9057644055456807}, {
       0.21738507360001294`, 0.8085213087400329}}]}}, 
    {Arrowheads[{{0.04515234126966938, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.21155231175049333`, 0.9782096425995668}, {
       0.21701911682093522`, 0.8789332145432898}}]}}, 
    {Arrowheads[{{0.04567016088703918, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.21182437161766057`, 1.05022326722116}, {
       0.21674705695376795`, 0.9497767327788394}}]}}, 
    {Arrowheads[{{0.009345162325251413, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.27542511943882364`, 0.}, {0.2960034519897477, 0.}}]}}, 
    {Arrowheads[{{0.01098586597811592, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2754941887966887, 0.07789798439377534}, {
       0.29593438263188276`, 0.0649591584633675}}]}}, 
    {Arrowheads[{{0.014586129293808365`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.27569791585581516`, 0.15541029021359573`}, {
       0.2957306555727562, 0.13030399550068997`}}]}}, 
    {Arrowheads[{{0.01874780316467417, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2760247726684631, 0.2325117862852305}, {
       0.29540379876010825`, 0.19605964228619802`}}]}}, 
    {Arrowheads[{{0.022907264680831024`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2764649716038331, 0.30917833335466227`}, {
       0.29496359982473835`, 0.2622502380739091}}]}}, 
    {Arrowheads[{{0.02685144006647249, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.277001658760984, 0.38539371677032513`}, {
       0.29442691266758747`, 0.32889199751538906`}}]}}, 
    {Arrowheads[{{0.030485193562853397`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2776109371575949, 0.4611432316285023}, {
       0.29381763427097657`, 0.39599962551435464`}}]}}, 
    {Arrowheads[{{0.03375727855838015, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2782743600055166, 0.5364150258420282}, {
       0.29315421142305487`, 0.4635849741579716}}]}}, 
    {Arrowheads[{{0.0366445082548708, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2789733071402012, 0.6112076116739794}, {0.2924552642883702,
        0.5316495311831632}}]}}, 
    {Arrowheads[{{0.03913111643729614, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2796814515157671, 0.6855166340237937}, {0.2917471199128043,
        0.6001976516904918}}]}}, 
    {Arrowheads[{{0.04120652076151205, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.28037726269688124`, 0.7593397159604055}, {
       0.29105130873169016`, 0.6692317126110227}}]}}, 
    {Arrowheads[{{0.042874515382828875`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.28103826095195744`, 0.8326876142573162}, {
       0.29039031047661396`, 0.7387409571712547}}]}}, 
    {Arrowheads[{{0.04414117851370127, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2816318192601197, 0.9055711984987562}, {
       0.28979675216845163`, 0.8087145157869574}}]}}, 
    {Arrowheads[{{0.04500666333199186, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.2821346108019292, 0.9779949857494827}, {
       0.28929396062664225`, 0.8791478713933739}}]}}, 
    {Arrowheads[{{0.04548923000146114, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.28251684420875955`, 1.049982167769187}, {
       0.2889117272198118, 0.9500178322308125}}]}}, 
    {Arrowheads[{{0.011716993760749292`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.3442422682759155, 0.}, {0.3700434460097987, 0.}}]}}, 
    {Arrowheads[{{0.013030617609957322`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.3443303353552387, 0.07788404015756593}, {
       0.3699553789304755, 0.06497310269957692}}]}}, 
    {Arrowheads[{{0.01610495149139549, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.3445896105884306, 0.155380426578499}, {
       0.36969610369728356`, 0.1303338591357867}}]}}, 
    {Arrowheads[{{0.01985221637647809, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.3450053088652812, 0.2324635660954166}, {
       0.36928040542043294`, 0.19610786247601195`}}]}}, 
    {Arrowheads[{{0.02370767907222183, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.3455642943618402, 0.30910826445762557`}, {
       0.36872141992387397`, 0.26232030697094577`}}]}}, 
    {Arrowheads[{{0.027421399057159165`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.34624444532393517`, 0.38529856642610955`}, {
       0.3680412689617791, 0.32898714785960465`}}]}}, 
    {Arrowheads[{{0.0308742894099989, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.34701572717871443`, 0.4610209218239016}, {
       0.36726998710699976`, 0.39612193531895545`}}]}}, 
    {Arrowheads[{{0.034001509814163486`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.34785471389072825`, 0.536265659256216}, {0.366431000394986,
        0.4637343407437837}}]}}, 
    {Arrowheads[{{0.03676911606849361, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.34873776885520097`, 0.6110297928777553}, {
       0.3655479454305132, 0.5318273499793873}}]}}, 
    {Arrowheads[{{0.03915775395113345, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.34963296588540926`, 0.6853113161228896}, {
       0.36465274840030487`, 0.6004029695913957}}]}}, 
    {Arrowheads[{{0.04115415696103337, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.35051396161170867`, 0.7591095539201522}, {
       0.3637717526740055, 0.6694618746512758}}]}}, 
    {Arrowheads[{{0.04275661401762742, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.35135372997042175`, 0.8324326552849536}, {
       0.3629319843152925, 0.7389959161436174}}]}}, 
    {Arrowheads[{{0.043967532423857085`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.35211381009839177`, 0.9052898466533601}, {
       0.3621719041873224, 0.8089958676323536}}]}}, 
    {Arrowheads[{{0.044785875665503724`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.352765247657243, 0.9776866599952823}, {0.3615204666284712, 
       0.8794561971475743}}]}}, 
    {Arrowheads[{{0.04522305020443093, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.3532732165068064, 1.0496406714615085`}, {
       0.36101249777890776`, 0.9503593285384908}}]}}, 
    {Arrowheads[{{0.014124603601013532`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.41302002454357944`, 0.}, {0.44412283259927754`, 0.}}]}}, 
    {Arrowheads[{{0.015193062311919968`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.41312896332135396`, 0.07785891651571471}, {
       0.444013893821503, 0.06499822634142814}}]}}, 
    {Arrowheads[{{0.01780242009069814, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4134500596169818, 0.15532843212728667`}, {
       0.44369279752587526`, 0.13038585358699906`}}]}}, 
    {Arrowheads[{{0.02112259215353559, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4139636556103061, 0.23238179896944305`}, {
       0.4431792015325509, 0.1961896296019855}}]}}, 
    {Arrowheads[{{0.024634932355998333`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.41465169599128165`, 0.30899350053155245`}, {
       0.44249116115157533`, 0.26243507089701895`}}]}}, 
    {Arrowheads[{{0.028075159541910838`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.415485899568419, 0.3851476766095616}, {
       0.44165695757443796`, 0.3291380376761526}}]}}, 
    {Arrowheads[{{0.03130852283658349, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4164288254269461, 0.4608331418851995}, {
       0.44071403171591095`, 0.3963097152576574}}]}}, 
    {Arrowheads[{{0.03425727557671419, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4174515741484794, 0.536041364999044}, {0.4396912829943776, 
       0.46395863500095574`}}]}}, 
    {Arrowheads[{{0.03687749636222891, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4185261076086713, 0.6107690040710017}, {
       0.43861674953418567`, 0.5320881387861409}}]}}, 
    {Arrowheads[{{0.039145249393231434`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.419614611839755, 0.6850156980464943}, {
       0.43752824530310197`, 0.600698587667791}}]}}, 
    {Arrowheads[{{0.041044795768529686`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4206871907521072, 0.7587835840956839}, {0.4364556663907498,
        0.6697878444757441}}]}}, 
    {Arrowheads[{{0.04256724257083575, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.421713075989975, 0.8320769418149659}, {0.435429781152882, 
       0.739351629613605}}]}}, 
    {Arrowheads[{{0.04371082038715049, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.42264999065981146`, 0.9049034622759744}, {
       0.4344928664830455, 0.8093822520097393}}]}}, 
    {Arrowheads[{{0.04447235393202581, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4234642399249507, 0.9772690917114499}, {0.4336786172179063,
        0.8798737654314067}}]}}, 
    {Arrowheads[{{0.04485550196019172, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.42411854441815333`, 1.0491854377092542`}, {
       0.4330243127247037, 0.9508145622907452}}]}}, 
    {Arrowheads[{{0.016572933909587313`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4817529469544661, 0.}, {0.5182470530455338, 0.}}]}}, 
    {Arrowheads[{{0.017446139022260155`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.48188604350335745`, 0.07781994961951737}, {
       0.5181139564966423, 0.06503719323762548}}]}}, 
    {Arrowheads[{{0.019640304317988366`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.48227816141575575`, 0.1552484983763281}, {
       0.517721838584244, 0.1304657873379576}}]}}, 
    {Arrowheads[{{0.022530536219636133`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4829018090436038, 0.23225826350999}, {0.517098190956396, 
       0.19631316506143853`}}]}}, 
    {Arrowheads[{{0.02566752427071404, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.48373241738141853`, 0.3088229925877456}, {
       0.5162675826185813, 0.2626055788408258}}]}}, 
    {Arrowheads[{{0.028794148111856193`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4847328829744047, 0.3849274289699089}, {0.5152671170255951,
        0.3293582853158053}}]}}, 
    {Arrowheads[{{0.03176893069746725, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.485858211109081, 0.46056323532978033`}, {
       0.5141417888909188, 0.39657962181307665`}}]}}, 
    {Arrowheads[{{0.03450451126952548, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.48707404031877416`, 0.5357233698558066}, {
       0.5129259596812257, 0.4642766301441932}}]}}, 
    {Arrowheads[{{0.036947978610024045`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4883465962637802, 0.6104039998276772}, {0.5116534037362196,
        0.5324531430294653}}]}}, 
    {Arrowheads[{{0.03907104429243736, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.48963451526136864`, 0.684607462617036}, {
       0.5103654847386312, 0.6011068230972495}}]}}, 
    {Arrowheads[{{0.04085413881590014, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4909042605448137, 0.7583375086598023}, {0.509095739455186, 
       0.6702339199116258}}]}}, 
    {Arrowheads[{{0.04228135359882215, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4921228335986677, 0.8315954105799761}, {0.5078771664013321,
        0.7398331608485949}}]}}, 
    {Arrowheads[{{0.04334494320056341, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.49324634034051207`, 0.9043860064522672}, {
       0.5067536596594877, 0.8098997078334466}}]}}, 
    {Arrowheads[{{0.04404023197847869, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4942378412656535, 0.9767168070473888}, {0.5057621587343463,
        0.8804260500954677}}]}}, 
    {Arrowheads[{{0.04436072708495548, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.4950602428181708, 1.0485913969951195`}, {0.504939757181829,
        0.9514086030048797}}]}}, 
    {Arrowheads[{{0.01902538869188552, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5504813282568655, 0.}, {0.5923758146002771, 0.}}]}}, 
    {Arrowheads[{{0.019739958234394134`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.550639582024055, 0.07776769315116851}, {0.5922175608330875,
        0.06508944970597433}}]}}, 
    {Arrowheads[{{0.02156811474972651, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5511052624776809, 0.15514025046348665`}, {
       0.5917518803794617, 0.13057403525079905`}}]}}, 
    {Arrowheads[{{0.02403924442872607, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5518428026222445, 0.2320882471729857}, {0.5910143402348981,
        0.19648318139844287`}}]}}, 
    {Arrowheads[{{0.026780663106217958`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5528197267987354, 0.3085863275445702}, {0.5900374160584072,
        0.2628422438840012}}]}}, 
    {Arrowheads[{{0.02955872233038946, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5539903250873933, 0.3846211909041262}, {0.5888668177697493,
        0.329664523381588}}]}}, 
    {Arrowheads[{{0.03223556631299473, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5553017254408522, 0.4601878210350886}, {0.5875554174162904,
        0.3969550361077684}}]}}, 
    {Arrowheads[{{0.03472033436976964, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5567141883529063, 0.5352822574661165}, {0.5861429545042364,
        0.4647177425338833}}]}}, 
    {Arrowheads[{{0.03695359283808146, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5581885170094131, 0.6099005060430015}, {0.5846686258477294,
        0.5329566368141412}}]}}, 
    {Arrowheads[{{0.03890290152203498, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5596804333046692, 0.6840471952180864}, {0.5831767095524734,
        0.6016670904961989}}]}}, 
    {Arrowheads[{{0.04054632461059717, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5611534464480676, 0.7577292506877609}, {0.5817036964090748,
        0.6708421778836673}}]}}, 
    {Arrowheads[{{0.041859752160823514`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5625727736466958, 0.8309436941968257}, {0.5802843692104468,
        0.7404848772317453}}]}}, 
    {Arrowheads[{{0.042828805262026856`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5638945759040704, 0.9036922800537607}, {0.5789625669530721,
        0.8105934342319532}}]}}, 
    {Arrowheads[{{0.04344768546511978, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5650804777654873, 0.9759849140679969}, {0.5777766650916554,
        0.8811579430748596}}]}}, 
    {Arrowheads[{{0.04369833070224139, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.5660936326328141, 1.047815832584023}, {0.5767635102243285, 
       0.9521841674159763}}]}}, 
    {Arrowheads[{{0.021502504905085637`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6191825569476451, 0.}, {0.6665317287666401, 0.}}]}}, 
    {Arrowheads[{{0.022078164670427614`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6193691651450055, 0.07769060173708578}, {
       0.6663451205692799, 0.06516654112005708}}]}}, 
    {Arrowheads[{{0.023566120309290724`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6199170837616625, 0.15498078700446147`}, {
       0.6657972019526229, 0.13073349870982423`}}]}}, 
    {Arrowheads[{{0.02561735014943525, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6207800690204656, 0.23183934919392576`}, {
       0.6649342166938199, 0.19673207937750278`}}]}}, 
    {Arrowheads[{{0.027937177449102353`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6219157780005016, 0.3082440709716163}, {0.6637985077137838,
        0.2631845004569551}}]}}, 
    {Arrowheads[{{0.030328756960518385`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6232679878530591, 0.38418576732233845`}, {
       0.6624462978612262, 0.33009994696337575`}}]}}, 
    {Arrowheads[{{0.03266733566591961, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6247747970566867, 0.45966275784547794`}, {
       0.6609394886575986, 0.39748009929737904`}}]}}, 
    {Arrowheads[{{0.034864253392861456`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6263911076108721, 0.5346750691911559}, {0.6593231781034133,
        0.4653249308088438}}]}}, 
    {Arrowheads[{{0.036856239677933285`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6280724578352195, 0.6092186657013209}, {0.6576418278790658,
        0.5336384771558217}}]}}, 
    {Arrowheads[{{0.038606516987297654`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6297722854804746, 0.6832994165136557}, {0.6559420002338107,
        0.6024148692006298}}]}}, 
    {Arrowheads[{{0.040089403626331614`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6314532129103457, 0.7569261372375036}, {0.6542610728039396,
        0.6716452913339245}}]}}, 
    {Arrowheads[{{0.04127339285265379, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6330789470465594, 0.8300924332038122}, {0.6526353386677259,
        0.7413361382247586}}]}}, 
    {Arrowheads[{{0.04213572069942719, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6346088859831872, 0.9027957927897298}, {0.6511053997310983,
        0.8114899214959841}}]}}, 
    {Arrowheads[{{0.0426699239196955, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6360052989928302, 0.9750493256578964}, {0.6497089867214552,
        0.8820935314849603}}]}}, 
    {Arrowheads[{{0.04284588743390822, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6372338758437858, 1.046837622134702}, {0.6484804098704996, 
       0.9531623778652972}}]}}, 
    {Arrowheads[{{0.024040599832741785`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6878166471436118, 0.}, {0.7407547814278165, 0.}}]}}, 
    {Arrowheads[{{0.02448011259903324, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6880428839177398, 0.07757485852954049}, {
       0.7405285446536884, 0.06528228432760236}}]}}, 
    {Arrowheads[{{0.025624595179567732`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6886994859255489, 0.1547450462632427}, {0.7398719426458794,
        0.130969239451043}}]}}, 
    {Arrowheads[{{0.0272362446120043, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6897169995964972, 0.23147959871489227`}, {
       0.7388544289749309, 0.19709182985653628`}}]}}, 
    {Arrowheads[{{0.02910002596377435, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6910374828397222, 0.30776091675056416`}, {
       0.737533945731706, 0.26366765467800723`}}]}}, 
    {Arrowheads[{{0.031064728940961024`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6925900696805716, 0.383583856721923}, {0.7359813588908565, 
       0.3307018575637911}}]}}, 
    {Arrowheads[{{0.03302527889190773, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6943051693801954, 0.4589511021960573}, {0.7342662591912328,
        0.3981917549467997}}]}}, 
    {Arrowheads[{{0.03489820127475764, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.696133367480645, 0.5338652324032528}, {0.732438061090783, 
       0.466134767596747}}]}}, 
    {Arrowheads[{{0.03661918506677905, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.6980259797264684, 0.6083228100604919}, {0.7305454488449598,
        0.5345343327966507}}]}}, 
    {Arrowheads[{{0.03814668187132967, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.699935690940712, 0.6823297084738792}, {0.7286357376307162, 
       0.6033845772404062}}]}}, 
    {Arrowheads[{{0.03945035784569666, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7018253794274519, 0.7558955449208681}, {0.7267460491439762,
        0.67267588365056}}]}}, 
    {Arrowheads[{{0.040491915898425226`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7036578388507012, 0.8290111948888028}, {0.724913589720727, 
       0.742417376539768}}]}}, 
    {Arrowheads[{{0.04123900500610911, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7053977283396607, 0.9016692220361139}, {0.7231737002317674,
        0.8126164922495999}}]}}, 
    {Arrowheads[{{0.04168158819390093, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7070129437479287, 0.9738835504206752}, {0.7215584848234995,
        0.8832593067221813}}]}}, 
    {Arrowheads[{{0.04177940511337507, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7084728832149181, 1.0456310106328224`}, {
       0.7200985453565101, 0.9543689893671768}}]}}, 
    {Arrowheads[{{0.02657793504807152, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7564515737944387, 0.}, {0.8149769976341323, 0.}}]}}, 
    {Arrowheads[{{0.02687913491579344, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7567315011381026, 0.07741380445240043}, {
       0.8146970702904685, 0.06544333840474241}}]}}, 
    {Arrowheads[{{0.027681345626473487`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.757513970621732, 0.15441678570980288`}, {
       0.8139146008068391, 0.13129750000448281`}}]}}, 
    {Arrowheads[{{0.02884334953518178, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7586991796792052, 0.2309796967803291}, {0.8127293917493656,
        0.19759173179109943`}}]}}, 
    {Arrowheads[{{0.03022626626396095, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7602114551410302, 0.3070950269040882}, {0.8112171162875408,
        0.2643335445244832}}]}}, 
    {Arrowheads[{{0.03172941967483751, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7619662315896574, 0.38276422441911356`}, {
       0.8094623398389135, 0.3315214898666006}}]}}, 
    {Arrowheads[{{0.033273430057350485`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7638886743721192, 0.4579947737382018}, {0.8075398970564518,
        0.3991480834046552}}]}}, 
    {Arrowheads[{{0.03478543751379706, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7659264902401938, 0.5327914520367031}, {0.8055020811883772,
        0.4672085479632967}}]}}, 
    {Arrowheads[{{0.03620320949192158, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.768028361897445, 0.6071504203678286}, {0.803400209531126, 
       0.535706722489314}}]}}, 
    {Arrowheads[{{0.03748225695662886, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7701467817449283, 0.6810768359475761}, {0.8012817896836427,
        0.6046374497667093}}]}}, 
    {Arrowheads[{{0.038585311877299334`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.772246290215542, 0.7545773986452512}, {0.7991822812130289, 
       0.673994029926177}}]}}, 
    {Arrowheads[{{0.03946837677687792, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7742881203854968, 0.8276404652637408}, {0.7971404510430742,
        0.7437881061648302}}]}}, 
    {Arrowheads[{{0.04009073549066934, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7762426404081094, 0.9002551834119754}, {0.7951859310204614,
        0.8140305308737383}}]}}, 
    {Arrowheads[{{0.040435063112520674`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7780862902018795, 0.9724326912677244}, {0.7933422812266915,
        0.8847101658751323}}]}}, 
    {Arrowheads[{{0.040454414578760346`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.7797894829105787, 1.044145112297462}, {0.7916390885179923, 
       0.9558548877025373}}]}}, 
    {Arrowheads[{{0.02905264119753809, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8251554560021532, 0.}, {0.8891302582835605, 0.}}]}}, 
    {Arrowheads[{{0.029219010974769977`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8254900347266227, 0.07717703600134394}, {
       0.8887956795590909, 0.0656801068557989}}]}}, 
    {Arrowheads[{{0.0296794124339709, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8263982657926505, 0.15392909765651316`}, {
       0.8878874484930633, 0.13178518805777256`}}]}}, 
    {Arrowheads[{{0.030378642408086264`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8277498881015769, 0.23024749695748528`}, {
       0.8865358261841368, 0.1983239316139433}}]}}, 
    {Arrowheads[{{0.03125459692065132, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8294511925387172, 0.3061432568957869}, {0.8848345217469964,
        0.2652853145327845}}]}}, 
    {Arrowheads[{{0.032262425743714136`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8314045793729283, 0.38162371786172933`}, {
       0.8828811349127855, 0.33266199642398486`}}]}}, 
    {Arrowheads[{{0.03335527029610693, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8335293084359184, 0.45669794761177795`}, {
       0.8807564058497953, 0.40044490953107903`}}]}}, 
    {Arrowheads[{{0.03447498316077861, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.835771418757275, 0.5313692857461106}, {0.8785142955284388, 
       0.46863071425388914`}}]}}, 
    {Arrowheads[{{0.03556452496743864, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8380772441160227, 0.6056306132823598}, {0.8762084701696911,
        0.5372265295747828}}]}}, 
    {Arrowheads[{{0.03657652961045206, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8404001949832843, 0.6794831205402931}, {0.8738855193024294,
        0.6062311651739922}}]}}, 
    {Arrowheads[{{0.03746529184162845, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8427073123741742, 0.7529272181157564}, {0.8715784019115396,
        0.6756442104556719}}]}}, 
    {Arrowheads[{{0.03818039908598051, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8449600656175502, 0.8259474303474554}, {0.8693256486681636,
        0.7454811410811155}}]}}, 
    {Arrowheads[{{0.038673860410117854`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8471348691451969, 0.8985305368345077}, {0.8671508451405169,
        0.8157551774512062}}]}}, 
    {Arrowheads[{{0.03891537439297443, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8492193954227694, 0.9706788451761778}, {0.8650663188629445,
        0.8864640119666789}}]}}, 
    {Arrowheads[{{0.038858142094431655`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.851182586054618, 1.0423662023396996`}, {0.8631031282310958,
        0.9576337976602998}}]}}, 
    {Arrowheads[{{0.03155928785592121, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8938241712223156, 0.}, {0.9633186859205409, 0.}}]}}, 
    {Arrowheads[{{0.031535619967254666`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8942676124267511, 0.07679605064087879}, {
       0.9628752447161054, 0.06606109221626406}}]}}, 
    {Arrowheads[{{0.03159576317839433, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8953534441404007, 0.15318815966389618`}, {
       0.9617894130024558, 0.13252612605038952`}}]}}, 
    {Arrowheads[{{0.031790140141771825`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.896903468538286, 0.22919344234860914`}, {
       0.9602393886045706, 0.19937798622281938`}}]}}, 
    {Arrowheads[{{0.032128964721494034`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.8988046927233967, 0.3048269854503277}, {0.9583381644194597,
        0.2666015859782437}}]}}, 
    {Arrowheads[{{0.0326149854017264, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9009535650861723, 0.38009395153491476`}, {
       0.9561892920566842, 0.3341917627507994}}]}}, 
    {Arrowheads[{{0.033229716781249925`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9032720736136324, 0.4550007520334922}, {0.9538707835292243,
        0.40214210510936477`}}]}}, 
    {Arrowheads[{{0.03393087143148959, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9057076885163111, 0.529544880563046}, {0.9514351686265454, 
       0.47045511943695373`}}]}}, 
    {Arrowheads[{{0.03466894138234686, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9082075635618363, 0.6037138743761667}, {0.9489352935810202,
        0.5391432684809758}}]}}, 
    {Arrowheads[{{0.035396814620797086`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9107274733134407, 0.6775045638797063}, {0.9464153838294159,
        0.608209721834579}}]}}, 
    {Arrowheads[{{0.036061223440272266`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9132354316933267, 0.7509082675735027}, {0.9439074254495298,
        0.6776631609979255}}]}}, 
    {Arrowheads[{{0.03660541359687446, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.915693147392631, 0.8239044841443818}, {0.9414497097502255, 
       0.7475240872841893}}]}}, 
    {Arrowheads[{{0.036976245787218975`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9180802958583769, 0.8964792918143849}, {0.9390625612844796,
        0.8178064224713287}}]}}, 
    {Arrowheads[{{0.03712059089380176, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9203944671512271, 0.9686154260542584}, {0.9367483899916295,
        0.888527431088598}}]}}, 
    {Arrowheads[{{0.036985027720896485`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9226064920200965, 1.0402818304506527`}, {
       0.9345363651227602, 0.9597181695493466}}]}}, 
    {Arrowheads[{{0.034028220866008256`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9625344097445531, 0.}, {1.0374655902554462`, 0.}}]}}, 
    {Arrowheads[{{0.033700239579803414`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9632135405616569, 0.07627611503016665}, {
       1.0367864594383425`, 0.06658102782697621}}]}}, 
    {Arrowheads[{{0.03333281104054442, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.964496716142519, 0.15215240773049743`}, {
       1.0355032838574805`, 0.13356187798378827`}}]}}, 
    {Arrowheads[{{0.03301969320988773, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9662245862681313, 0.22773640090868086`}, {
       1.033775413731868, 0.2008350276627477}}]}}, 
    {Arrowheads[{{0.0328178341834154, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9682940789120666, 0.3030441338341355}, {
       1.0317059210879327`, 0.268384437594436}}]}}, 
    {Arrowheads[{{0.03276774770674412, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9706092814085809, 0.37806635105959013`}, {
       1.0293907185914186`, 0.336219363226124}}]}}, 
    {Arrowheads[{{0.03288274576909151, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9730964056941933, 0.4527985937609486}, {1.026903594305806, 
       0.40434426338190843`}}]}}, 
    {Arrowheads[{{0.0331402481012319, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9757046198131095, 0.5272232017281605}, {
       1.0242953801868897`, 0.4727767982718392}}]}}, 
    {Arrowheads[{{0.03350161973743153, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9783826086207441, 0.6013158633264692}, {
       1.0216173913792552`, 0.5415412795306734}}]}}, 
    {Arrowheads[{{0.03392512902714565, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9810877030203464, 0.6750674400390537}, {1.018912296979653, 
       0.6106468456752318}}]}}, 
    {Arrowheads[{{0.03435248528238847, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9837879989407352, 0.7484576453608326}, {
       1.0162120010592641`, 0.6801137832105955}}]}}, 
    {Arrowheads[{{0.03471735427795157, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9864471303932347, 0.8214552935076758}, {
       1.0135528696067646`, 0.7499732779208953}}]}}, 
    {Arrowheads[{{0.034964636590075475`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9890468042585328, 0.8940483523981306}, {
       1.0109531957414666`, 0.8202373618875831}}]}}, 
    {Arrowheads[{{0.035019346446266066`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9915788929423244, 0.9661974096650212}, {1.008421107057675, 
       0.8909454474778353}}]}}, 
    {Arrowheads[{{0.034825976967571524`, 1.}}], 
     {GrayLevel[0], 
      ArrowBox[{{0.9940346458399173, 1.0378770598480043`}, {1.005965354160082,
        0.9621229401519951}}]}}}},
  AspectRatio->1,
  Frame->True,
  Method->{"TransparentPolygonMesh" -> True},
  PlotRange->{{-0.05050762722761054, 
   1.0505076272276106`}, {-0.05050762722761054, 1.0505076272276106`}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.5755448539542923`*^9, 3.5865814789488993`*^9, 
  3.587380855429454*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fig. 11.1R Deformation of block perimeter", "Subsection",
 CellChangeTimes->{{3.501998124308631*^9, 3.5019981435727324`*^9}, 
   3.5054112035139494`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"rx", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], " ", ":=", " ", 
   RowBox[{"x", "+", " ", 
    RowBox[{"uxi", "[", 
     RowBox[{"x", ",", "y"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ry", "[", 
    RowBox[{"x_", ",", "y_"}], "]"}], ":=", " ", 
   RowBox[{"y", "+", " ", 
    RowBox[{"uyi", "[", 
     RowBox[{"x", ",", "y"}], "]"}]}]}], ";"}]}], "Input",
 CellChangeTimes->{3.5054112035159492`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig1a", "=", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"rx", "[", 
       RowBox[{"x", ",", "b"}], "]"}], ",", 
      RowBox[{"ry", "[", 
       RowBox[{"x", ",", "b"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "a"}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", 
    RowBox[{"AspectRatio", "\[Rule]", "1"}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4147368728457975`*^9, 3.4147368778687973`*^9}, {
   3.4671051604103675`*^9, 3.4671051647946186`*^9}, 3.5054112035159492`*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJw11Hk01VsbB3BJSIWUkjKmeA1NKifTN4kMcclUNyJlnolCt5JreE9KRGTo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     "]]}},
  AspectRatio->1,
  Axes->True,
  AxesOrigin->{0, 0.77},
  Method->{},
  PlotRange->{{0., 1.028111}, {0.761989, 0.821509}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.5755448541823187`*^9, 3.586581479012911*^9, 
  3.5873808554914412`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig1b", "=", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"rx", "[", 
       RowBox[{"a", ",", "y"}], "]"}], ",", 
      RowBox[{"ry", "[", 
       RowBox[{"a", ",", "y"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", "0", ",", "b"}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", 
    RowBox[{"AspectRatio", "\[Rule]", "1"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.4147368831837974`*^9, 3.4671051734431133`*^9, 
  3.5054112035169497`*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwt0HtUjHkcx/FuBnWYZaolUUIXZ1dFkUGfiG4sxamsQ6UmlUN3NqXEVqai
1Rgt3SOXakhOmaxK293upOmmdHVtmsel53nslmK1vz1nv+d8zve8/n0v9Q3Z
7a+moqLiRfbfryrLtzDtZKHy/0Uq+WaGxA1bPMvfXV6J+5xy3gJiDXbmpcEr
fKgn2NfMJE4wiEiryXLBW2F/0EgHC1lF6MHYvH1YWcFdVEBcKZ8l/HDjMLYc
FI5oEevWSnpyK6JRqeuwSI04bdjp1YZzMdC0V2z+1M5iqHfSSuF7EkdEXYLX
xJQoKz3wmzjs1vJWrSIe3B9Rq3k/HkNeey4FEltvor1n8xLQ7fHATCpn4ZxY
H7+uIwWz6iUFxcSWggOz4hxTMVDFnZ1LXBcQHSurSkUeb8WNBOLFbm6Gp2+e
g52/aKEbsaBvqtcpNg3bPPLfjLSxMNXDDyY66Zixr43HIY6SmtZqNYoxvKmo
Y34riwuFdR+HzbNBb7O0VyfW/OovjgvIxiG/3dc/yljcGG7lG+ZlY8lxH9su
4h4qRHRkbg60DO/Wi4mH3ufAjGzpXdtFPGKDObeMLUvycJNnNK3xJ4sitlMh
FRdgVcyOq9UtLHLbf1IYf3sdI92Z8/LqSd/ktsT8mRLcjoOGbjULftP+mD55
GdTanGM/V7AISd3ku9SvAmXU41Wht1nUGBqcG3d+ABPXxO2HC1l06UQ9zzKu
huG1n3dGZrHwy1te/d6vFnqT1pU66Sy+S/jsmuhdh736pUGNSSxEPuEBwxsb
0LA3V2JxkvTh8t2neU3wS6TMXUNZ+M/0uRCi24LCA46cQQGL46Uax+5NPkZL
+YANZy8Lu/5tZ0Jny6CeU+D9jwuLM46+GVLeE7QYK08m2bIIL5swXve1DWrS
D9JaCxapF/Yc/V2zHdc2hghuLWMxX61Z1ZXTgZcVwc5WOix2hNqtn7OwExf7
g5rqOSwCL2t94Zp1QZYbptSfZNBWn5VUbNoNbZ3iZFOKwcqccZUUPMWU9YI6
7gADFwU9r2dbD9RF99yNnjCwSx5q5jj3wkaft+xdDYNDzxq3JgqegdKOGiq8
y4Bt55prh/Qh+M6zKN0CBkmHwzLfpfeDbhCNNqQz8Djh01ZdNoBw+Y7+Ez8z
OKDcvDa8dhD+Gad8pyIYqIQG2x8bHUK8dvDZ0/4M8u/91T3+dRiBCof+nZ4M
CtcJDSaFzzFocejUmCMDo0cmF/YZvMDmbvHD4vUM9Lj3SzolL+A2plriuJLB
rvKcO7GOL2H5m03V5CIG0vF1QWbPXsIp7Jc81bkMtnoL1M2DXmHJghlGzFca
teee5lZwXmOLwrVziqbhfPhyykDma8xffsz10wsaE4MC++aNb7D/6G0nv04a
xefdt+8bfIPAsGDVsUYaXucvL5FEjsAzobwjWEqDr1n93ERPgVZevN1gEY3R
oV5+foUC3mqr3K9l07jYzO902jWKR2+fl4al0Qh2eD+ZMzGKmCDR9KHTNBb/
ap7in6GENnfP9u+Jp6e3C1ddUUKIhdkf42nonGounshSwt3r8Y544lkLFh5I
KVBC7mP79sopGhZLuqLKJEqkvioyfBJLQ9KgncmpV8Jl9VQaP5pGSb3emGxM
iaaAOwmqxH435RmZrBIP56vGt5ygkesrFwT+rYTXiPisB3GLlIrQ+KxEn9uX
vvAoGtoONj62HAqKa58yJMdp3NlwNe6BPoWS/PbRCOKz71aIkg0oWAkv2W0g
Tu4elf1oROGtFaP6xzHS51ZQ5ZQJhfUdsTJFJI2QBxpBtmsoRLJrdpYSN4rN
K7lrKSy2+dx9nDhzr7b5CxsKs93kEzOIKcZJnGBLYfyeIr01gkZ0hPSMx2YK
hZ7LVl8iLrdefMV0K4UAjrB3P3F+WnLPlAMFcZJe4nJily4t21ZnCv8CmtX9
8Q==
     "]]}},
  AspectRatio->1,
  Axes->True,
  AxesOrigin->{1.04, 0},
  Method->{},
  PlotRange->{{1.028111, 1.176552}, {0., 0.821509}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.575544854219324*^9, 3.5865814790339155`*^9, 
  3.587380855512415*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Shear-free solution", "Subsubsection",
 CellChangeTimes->{{3.4147325861377974`*^9, 3.4147325896247973`*^9}, 
   3.5054112035169497`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"rxsf", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"x", "+", " ", 
   RowBox[{
    FractionBox["\[Nu]", 
     RowBox[{"1", "-", "\[Nu]"}]], 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"b", "-", "y"}], ")"}], "x"}], 
     "DD"]}]}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"rysf", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", 
  RowBox[{"y", "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["b", "2"], "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"b", "-", "y"}], ")"}], "2"]}], 
    RowBox[{"2", "DD"}]], " ", "-", 
   RowBox[{
    FractionBox["\[Nu]", 
     RowBox[{"1", "-", "\[Nu]"}]], 
    FractionBox[
     RowBox[{
      SuperscriptBox["a", "2"], "-", 
      RowBox[{"3", 
       SuperscriptBox["x", "2"]}]}], 
     RowBox[{"6", "DD"}]]}]}]}]}], "Input",
 CellChangeTimes->{{3.4144152997744246`*^9, 3.4144153741374245`*^9}, {
   3.4144155821164246`*^9, 3.4144155824294243`*^9}, 3.5054112035179496`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig1c1", "=", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"rxsf", "[", 
       RowBox[{"x", ",", "b"}], "]"}], ",", 
      RowBox[{"rysf", "[", 
       RowBox[{"x", ",", "b"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "a"}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", 
    RowBox[{"AspectRatio", "\[Rule]", "1"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.4147369483507977`*^9, 3.4671051783943963`*^9, 
  3.5054112035189495`*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJxF1ns0FWsfB3CUQkJOp4tLtUuKIqUipa+UUihJqKgoarOEpKQSKiSvkKhI
CpvkErZku5TIpQsiEdJ2yb6MjF7qkJNzHutd727WmjXrs56Z5/f9zvwzDCcP
K2cJMTGxu+ScuP7/uDfaIaPO5sFpbFZtyNzVEHc9q/DbW6BevJMx4T/OPsgy
U98J+a/2hr/XrVHyT/LRCRtw7IsF8/dh+dCiuN/3H8Th6o73E/6hPFxvE+sI
2/N1qktEzx+FouKo14Tr2ta9eJrggqhQ+3dLRPsx0fR5ZP1SYtbti2ylZDd8
nF2Xu1S0/wkk63Ss0iA2ynTaU2btCR3NRc80RPO8EDg52UZTNO8U3A9YpC0j
nuOWbrLggA+kWs8cXi6afxrH17SraYnmn4VXtHm9NvGgxoBekKMflKP3F6wQ
5TmH0JP3WTqiPP7Ibv6YsYr4FV9Xs/fYRRyze12qK8oXgJ7i7+2rRfkCob74
+zo9Ub7LaAlIPL+B2DD/0FRrzmVkStsKDEV5r8DE29QRorzBeMFvPmksynsV
ffbxKtuJ/zyZOlrw/CpaKaWeHaL8YYg40MQ2F+W/hi2fBs5aivJHoIPZEGZH
PKBDUbOrI9A/PuvRflGf6yg7FfvBXtQnEtkSRrscRX2iwEkzrHYV9YmB/6ye
dxeJpxl818j1jIHvStWeIGK2b1NYW3UMZOtCxIOJJX9cN9c6fRNTwgecw4nT
B6UaGhtjYaAs8fb2RJ4vPz+oht9GVvCmmYXEcWof9bb13IZ18aURzkSeI4W3
PA3uINbrh6CMOIrrvb+CfwdCJ/fhKuLV7f2fjpskoNnufloLsV9DZ2/+eCLY
UYyQMeKF8qUmn/begxTXbaNYAQ+vLeJZU7LuYSSnT0KSOOLLMnamVhLS3rvn
yxFznfg+/7QnwXTli7yFxJccHEdT9B8gWu+SpgVxrdUescFvKdju6zL+iPhw
85QD95VScU7mEJ1L/MOWU2C1mdijhSokVnNguBXEpKIvd9vUSuKAY3Sznx4L
TnPUazqI156/9kjyQhp+urV5KTzhITm1wkZZKgMsLYZzALHB0jO5b3QyEDd3
ZlUIcUOGpqz/vgzYtgavuk48nhNVzs3IgFEdpZVIvJ9zaDnL4hGK4vOzS4gV
6sfEV97IhHmpQesY8YUR3ayt83LwVWnJ66BCHmSOSOcs25oDrneexzXi2Led
jxVO5ODWZrP5McSPH1xlt5XmwHZb6r1U4pMs2qFQ+zEUuT/VaolZlibOZnWP
4Wez3FjxKQ9yqYPe3tPzMMc2vSqT2Hp/+7AzIw9Zod1vnhDHy1f52K3Jw9SV
mp3PiZf6xZ8xdMiDTF8Xo5nYeNfWc1Oz8mDe9V+tceLTo/FB8eb5eKkSb2dV
xEOnxbaoimtsBKVNHpHg8NB0Lqnb5T4bm9mUrTxxzcNRXZlCNvS131cqE+dL
Zn6w7GajlPnw5Wri0FIFlU79Auj9c/TIMeJVWm2skS8F+JK8QL6eOETWvUTL
qBBa7x8uSS/mQed1NC9uuAjuDZMcAkt5qNKUi6iV4uCynuDbdWKHsKurx1Q4
CPRfGJ1IHLY9IPCgCQfLMyOHi4l7q92VF8dyYBZ8YNNfxLcqTXfl6RXj4ZpH
Pz3KeBArG3/yxq8EvQGr+pnPyPd97BosLv4MdrXcVxfLeVB09Eyq3/ACtEOZ
T1IVD/MEtbRdfyV402uMfd/wsDuV2ZPpXo3bls5+KY08HDzT2qu0+BWWRJWX
zm/jIfrCf8L2ZL+B7eEGnY9d5H1PcnxQpVmPJ8bbLVWFPFjUi/c+HGiAr2yk
dscQD3P1ftQ06zSiNay7IvsXD40ajWy5iCb8sWztpAXSfGywL/3Y2/semZ2r
0zT+5IOT/toySe0DvJOygpQZfKgFlNcdv9gCR1/LwlZtPh54tSXOqmjFpAHm
yLENfEwfaPN5x2hDuaxrWp8ZHxoZwisafu2Y8VWsq/8AH/sCVPkuxR3wvFXt
c9uND58uvd25szuh4Ph0v9MFPlLtfWS77D/D+ZmGTXEEHxXTHMVaLLi4darJ
yiOJjwztsV8VP7m4ZLzexY3NR3wnK2VRQhcq1wZMflvNh4R1VZ20WTcaZ8Sw
73ziw5jpfrPsRze2s3TU3g3xET5NXbIvtgc7FquEb5IWwNzbauTGll7YRl2u
2cEQYNbxyBh2Xy90t7rWnNEX4Onfe98mX/mCcc8187p2CfDO5vrVuav6wJx1
npZzFWDIsG1AvKkPc5fvnb32kgDhwlstnoE8JL6USzC5J8AKNSbTSo2P+uOf
lupzBMj49TxOopKPlfape22aBUhICN++zovsy3UpCh0SIK6GmdKtKISj691g
bwUhpDmcIMUXQvT0Fd+M1RHC2uzGL4UTFPx9daWHLIVIZ15gBMn147PhTaMO
DyG8Ldprzpb046icYqRmtBDbynbM7T34Fbn4eJTOE2K4btMU1fGvUOEWLShv
EmJ2aubpjowBrFi2LmfRiBBbpDQi5cxpqDOVNzcqUVh0lL+3cyeN0Ji8ohPK
FCSepyln76Yxx9d24zQVCs/PqKdZ2NKoPfbKa4sqhQ28RWXhTjSOCKwDC+dT
0K2a1y9zloa/z9/GSWoUFl6eaTqVRcMq96bleW0K4tym6S3pNJK7e0KVVlDg
rr/RxHpEQ+0N6p8SJw7NOLQ1l0Z+9YLg7zoUlI7In75SQmOaY0iChy6FGcYy
KZOaaDxQ4bm56FMYvFvLbGqmQTUoXJFcR6FuNHRFcisNJ0PTzBTi8Nypxcad
NLo2ctV6DChIMSQbAwWkTy+77LAhBd75yrjd/TQ69pmbim2kUNV6yYFB02gI
+fb5HvHlSAnB82Ea22Sd130GhSP95dmRf5H+Trpj/kYUjE0DTx3+SeOcl/zr
eZsoMFKMDHR+0bDXH00vI/7f39sg3AsHYw4aU/gXbdwDKw==
     "]]}},
  AspectRatio->1,
  Axes->True,
  AxesOrigin->{0, 0.74},
  Method->{},
  PlotRange->{{0., 1.}, {0.7407407962962685, 0.8518517407407963}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.5755448542683296`*^9, 3.586581479062916*^9, 
  3.5873808555434523`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig1c2", "=", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"rxsf", "[", 
       RowBox[{"a", ",", "y"}], "]"}], ",", 
      RowBox[{"rysf", "[", 
       RowBox[{"a", ",", "y"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"y", ",", "0", ",", "b"}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", 
    RowBox[{"AspectRatio", "\[Rule]", "1"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.4147369505587974`*^9, 3.467105182531633*^9, 
  3.5054112035189495`*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJwdygs0VHkcB/AJm2gOK4dsdFBnt1UasSts8u2lcJJsefTCem1FazJne1nP
1E6UrSgl6SFnqQij2cp6JEUjE0NJ7TQeTXSv5X/vpFYGe3d/53zP93zO92cb
Hvd9lA6Px/Pl8l/La0wzXLM0uOK9qyftQyXGC/tmMSc0CIoafVk6TYIFGRUX
SjhTBcbHuvkSpAT6lc7hLI0ZnnKaL4HjSEabNlODjpiA4GE/CbJtdGbfP6aB
5fTHkqMlEmw5TAq9j2igvbrWzGNHFfq9Wmu2/qLB34kX0poe3QbU0a7anRqY
T+8Q+vXcgdfT+z5vAzUQPLhebNdeDfHF5Q6LPDVYWKfg76qthXHi+a0jjhqI
3R6KOsfrsea6dK7WWoOQrztlzWkNqB+ULBDzNUhaiXTlkkbEz1Toxo2xiO4K
fHNa+RCKhIr9BQMsfvYprYaoCQcnhE6+nSy8lu6LETk+hvwJk76mgcVkeGT1
UJcMW1KyS86Ws9B2tOgWJjyBvaeZ2c6LLJ78iJrl3nLcDAq2SMtk0fxZir9s
4ilECQnhlgdYqGJXBpcOt0F4uC7JOIqF7PUMtY+yHVbdjU7rN7HYYasKk9Qq
YLTi+uIZK1gI+lqSNko7UPZ+Yz1fwGLZp3JV6OVOnL3Xdf7kHBYR/FLnb688
gyQy2TNOn8WQR25K3uXnSNo+KD47ymDQz9e78VQXTkU5mi7vY1DV7nYyOu8F
/P9c5mzfxoDk2a8S5XTD62O03b5aBhmBaQ9Cj7/E0dXxkbjBQPh7nFKU/wp7
lR4u/ucZpDc1fuN18S9cE9Mb+44wsH+lY9wpVYJvcdddHs9AoaAq98lfY/7o
unqdHxhsEBOvRc9VSO2XZkh9GRjGb3YpSujBceuBocplDCqVuVtM9Hohh6ut
4UJuH+vhL8nsRdDeXfk95gyaZxsEbrPuQxbZdmtKj0FPzL3NRjf6wE8VGBQw
BLf7LW+aremHa2jDp3QVwWiYvtWBzn7k2e6vfdRKEDBmVRET8QarR2c+/vUe
QWzc3E0zeWrkmYfHnCrm9pTQtvJsNVy9z22ec4ZA5CUU+ji/BT9sxHgyleCn
3XtqTz57i6xG/YiFQgILciL/SOwAgr7jTTzbTvC5YMi72WQQ8om7ApkPwTyZ
neGJ0kEkFOU4feFG0FvVkli07h2EIX6JA18SqB2qnSPIO8yKagmdNCX4SlHJ
2/AbhSSX9dFlnHWtC1vNT1OgDFpjQzjrqMqkqhwK98vkB2tnEUTfsVDuzaMg
/Kc9O8mEoKXNOSG3iII880XTpBH3f+6MdqCawrFKtcOUAYF9Cm1aMEhhND16
6S3Omxx9InfTFMKCBtxDOV8IaGxbOkzBVTvoXTeDIOej4eRTDYUBz6GIZH0C
myu9JbpTFDy72dwpPYKijKq1QjMa5TdEBbc4kz2yYA8LGlZJ76+Fct7ROH6U
b0lDM+9DRZ0uwSVZuUuJDY2rsWMtyToEde6RVP8iGkY4pHDgbHU3QFkpoHHI
ZPyFahpBfnH4UKojDX+pVg3Oer3yEBsXGjXixKERHgFlJ2gYcaNht22SvcT5
vbp4ZZ07jZzFyWN+nK3NoMwCDd7/R7DnD5ITsorGvwH0fcA=
     "]]}},
  AspectRatio->1,
  Axes->True,
  AxesOrigin->{1., 0},
  Method->{},
  PlotRange->{{1., 1.2222218888890555`}, {0., 0.8518517407407963}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.575544854300332*^9, 3.586581479081901*^9, 
  3.587380855562475*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig1c3", "=", 
  RowBox[{"ParametricPlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"rxsf", "[", 
       RowBox[{"x", ",", "0"}], "]"}], ",", 
      RowBox[{"rysf", "[", 
       RowBox[{"x", ",", "0"}], "]"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "a"}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", 
    RowBox[{"AspectRatio", "\[Rule]", "1"}]}], "]"}]}]], "Input",
 CellChangeTimes->{3.4147369540957975`*^9, 3.467105184594751*^9, 
  3.50541120351995*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJxFlnk0lN8fx31LWqhooSxFtqJUQkr5VJLSZiQUIY0WFMaSpbKO0LdsZS88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     "]]}},
  AspectRatio->1,
  Axes->True,
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{{0., 1.2222218888890555`}, {-0.03703698148150925, 
   0.0740739629630185}},
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.575544854336337*^9, 3.586581479101919*^9, 
  3.587380855629429*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig1c", "=", 
  RowBox[{"Show", "[", 
   RowBox[{
    RowBox[{"Graphics", "[", "Dashed", "]"}], ",", "fig1c1", ",", "fig1c2", 
    ",", "fig1c3"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4144154295234246`*^9, 3.4144154803274245`*^9}, 
   3.50541120351995*^9}],

Cell[BoxData[
 GraphicsBox[
  {Dashing[{Small, Small}], {{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJxF1ns0FWsfB3CUQkJOp4tLtUuKIqUipa+UUihJqKgoarOEpKQSKiSvkKhI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      "]]}}, {{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJwdygs0VHkcB/AJm2gOK4dsdFBnt1UasSts8u2lcJJsefTCem1FazJne1nP
1E6UrSgl6SFnqQij2cp6JEUjE0NJ7TQeTXSv5X/vpFYGe3d/53zP93zO92cb
Hvd9lA6Px/Pl8l/La0wzXLM0uOK9qyftQyXGC/tmMSc0CIoafVk6TYIFGRUX
SjhTBcbHuvkSpAT6lc7hLI0ZnnKaL4HjSEabNlODjpiA4GE/CbJtdGbfP6aB
5fTHkqMlEmw5TAq9j2igvbrWzGNHFfq9Wmu2/qLB34kX0poe3QbU0a7anRqY
T+8Q+vXcgdfT+z5vAzUQPLhebNdeDfHF5Q6LPDVYWKfg76qthXHi+a0jjhqI
3R6KOsfrsea6dK7WWoOQrztlzWkNqB+ULBDzNUhaiXTlkkbEz1Toxo2xiO4K
fHNa+RCKhIr9BQMsfvYprYaoCQcnhE6+nSy8lu6LETk+hvwJk76mgcVkeGT1
UJcMW1KyS86Ws9B2tOgWJjyBvaeZ2c6LLJ78iJrl3nLcDAq2SMtk0fxZir9s
4ilECQnhlgdYqGJXBpcOt0F4uC7JOIqF7PUMtY+yHVbdjU7rN7HYYasKk9Qq
YLTi+uIZK1gI+lqSNko7UPZ+Yz1fwGLZp3JV6OVOnL3Xdf7kHBYR/FLnb688
gyQy2TNOn8WQR25K3uXnSNo+KD47ymDQz9e78VQXTkU5mi7vY1DV7nYyOu8F
/P9c5mzfxoDk2a8S5XTD62O03b5aBhmBaQ9Cj7/E0dXxkbjBQPh7nFKU/wp7
lR4u/ucZpDc1fuN18S9cE9Mb+44wsH+lY9wpVYJvcdddHs9AoaAq98lfY/7o
unqdHxhsEBOvRc9VSO2XZkh9GRjGb3YpSujBceuBocplDCqVuVtM9Hohh6ut
4UJuH+vhL8nsRdDeXfk95gyaZxsEbrPuQxbZdmtKj0FPzL3NRjf6wE8VGBQw
BLf7LW+aremHa2jDp3QVwWiYvtWBzn7k2e6vfdRKEDBmVRET8QarR2c+/vUe
QWzc3E0zeWrkmYfHnCrm9pTQtvJsNVy9z22ec4ZA5CUU+ji/BT9sxHgyleCn
3XtqTz57i6xG/YiFQgILciL/SOwAgr7jTTzbTvC5YMi72WQQ8om7ApkPwTyZ
neGJ0kEkFOU4feFG0FvVkli07h2EIX6JA18SqB2qnSPIO8yKagmdNCX4SlHJ
2/AbhSSX9dFlnHWtC1vNT1OgDFpjQzjrqMqkqhwK98vkB2tnEUTfsVDuzaMg
/Kc9O8mEoKXNOSG3iII880XTpBH3f+6MdqCawrFKtcOUAYF9Cm1aMEhhND16
6S3Omxx9InfTFMKCBtxDOV8IaGxbOkzBVTvoXTeDIOej4eRTDYUBz6GIZH0C
myu9JbpTFDy72dwpPYKijKq1QjMa5TdEBbc4kz2yYA8LGlZJ76+Fct7ROH6U
b0lDM+9DRZ0uwSVZuUuJDY2rsWMtyToEde6RVP8iGkY4pHDgbHU3QFkpoHHI
ZPyFahpBfnH4UKojDX+pVg3Oer3yEBsXGjXixKERHgFlJ2gYcaNht22SvcT5
vbp4ZZ07jZzFyWN+nK3NoMwCDd7/R7DnD5ITsorGvwH0fcA=
      "]]}}, {{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJxFlnk0lN8fx31LWqhooSxFtqJUQkr5VJLSZiQUIY0WFMaSpbKO0LdsZS88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      "]]}}}]], "Output",
 CellChangeTimes->{3.5755448543633394`*^9, 3.5865814791159353`*^9, 
  3.587380855644459*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fig1d", "=", 
  RowBox[{"Graphics", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Dashing", "[", 
      RowBox[{"{", 
       RowBox[{"0.03", ",", "0.03"}], "}"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"a", ",", "b"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"a", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], 
   "]"}]}]], "Input",
 CellChangeTimes->{{3.4144154930084248`*^9, 3.414415495256425*^9}, 
   3.5054112035209494`*^9}],

Cell[BoxData[
 GraphicsBox[
  {Dashing[{0.03, 0.03}], LineBox[{{0, 1.}, {1., 1.}, {1., 0}}]}]], "Output",
 CellChangeTimes->{3.5755448543853436`*^9, 3.5865814791258955`*^9, 
  3.5873808556534233`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"fig1a", ",", "fig1b", ",", "fig1c", ",", "fig1d", ",", 
    RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"HoldForm", "[", "x", "]"}], ",", 
       RowBox[{"HoldForm", "[", "y", "]"}]}], "}"}]}], ",", 
    RowBox[{"AxesOrigin", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], ",", 
    RowBox[{"AspectRatio", "\[Rule]", 
     RowBox[{"1", "/", "1.2"}]}]}], "]"}], "//", "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", 
   RowBox[{"\"\<ces-outline\>\"", ",", 
    RowBox[{"ImageSize", "\[Rule]", "220"}]}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.3901079882785797`*^9, {3.4144158703594246`*^9, 3.4144159067464247`*^9}, {
   3.4318342227912083`*^9, 3.4318342243672085`*^9}, {3.4671043550433035`*^9, 
   3.4671043865551057`*^9}, {3.467104753451091*^9, 3.4671047861389604`*^9}, {
   3.467105093579545*^9, 3.467105113667694*^9}, {3.479208698292*^9, 
   3.4792087061289997`*^9}, 3.479208737426*^9, {3.479208769506*^9, 
   3.479208769644*^9}, {3.4847100694642*^9, 3.4847101414312*^9}, {
   3.5014894211346197`*^9, 3.501489454621535*^9}, 3.50541120352195*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.5755448545903673`*^9, 3.586581479250942*^9, 
  3.58738085578644*^9}],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJw11Hk01VsbB3BJSIWUkjKmeA1NKifTN4kMcclUNyJlnolCt5JreE9KRGTo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      "]]}}, {{}, {}, 
    {GrayLevel[0], LineBox[CompressedData["
1:eJwt0HtUjHkcx/FuBnWYZaolUUIXZ1dFkUGfiG4sxamsQ6UmlUN3NqXEVqai
1Rgt3SOXakhOmaxK293upOmmdHVtmsel53nslmK1vz1nv+d8zve8/n0v9Q3Z
7a+moqLiRfbfryrLtzDtZKHy/0Uq+WaGxA1bPMvfXV6J+5xy3gJiDXbmpcEr
fKgn2NfMJE4wiEiryXLBW2F/0EgHC1lF6MHYvH1YWcFdVEBcKZ8l/HDjMLYc
FI5oEevWSnpyK6JRqeuwSI04bdjp1YZzMdC0V2z+1M5iqHfSSuF7EkdEXYLX
xJQoKz3wmzjs1vJWrSIe3B9Rq3k/HkNeey4FEltvor1n8xLQ7fHATCpn4ZxY
H7+uIwWz6iUFxcSWggOz4hxTMVDFnZ1LXBcQHSurSkUeb8WNBOLFbm6Gp2+e
g52/aKEbsaBvqtcpNg3bPPLfjLSxMNXDDyY66Zixr43HIY6SmtZqNYoxvKmo
Y34riwuFdR+HzbNBb7O0VyfW/OovjgvIxiG/3dc/yljcGG7lG+ZlY8lxH9su
4h4qRHRkbg60DO/Wi4mH3ufAjGzpXdtFPGKDObeMLUvycJNnNK3xJ4sitlMh
FRdgVcyOq9UtLHLbf1IYf3sdI92Z8/LqSd/ktsT8mRLcjoOGbjULftP+mD55
GdTanGM/V7AISd3ku9SvAmXU41Wht1nUGBqcG3d+ABPXxO2HC1l06UQ9zzKu
huG1n3dGZrHwy1te/d6vFnqT1pU66Sy+S/jsmuhdh736pUGNSSxEPuEBwxsb
0LA3V2JxkvTh8t2neU3wS6TMXUNZ+M/0uRCi24LCA46cQQGL46Uax+5NPkZL
+YANZy8Lu/5tZ0Jny6CeU+D9jwuLM46+GVLeE7QYK08m2bIIL5swXve1DWrS
D9JaCxapF/Yc/V2zHdc2hghuLWMxX61Z1ZXTgZcVwc5WOix2hNqtn7OwExf7
g5rqOSwCL2t94Zp1QZYbptSfZNBWn5VUbNoNbZ3iZFOKwcqccZUUPMWU9YI6
7gADFwU9r2dbD9RF99yNnjCwSx5q5jj3wkaft+xdDYNDzxq3JgqegdKOGiq8
y4Bt55prh/Qh+M6zKN0CBkmHwzLfpfeDbhCNNqQz8Djh01ZdNoBw+Y7+Ez8z
OKDcvDa8dhD+Gad8pyIYqIQG2x8bHUK8dvDZ0/4M8u/91T3+dRiBCof+nZ4M
CtcJDSaFzzFocejUmCMDo0cmF/YZvMDmbvHD4vUM9Lj3SzolL+A2plriuJLB
rvKcO7GOL2H5m03V5CIG0vF1QWbPXsIp7Jc81bkMtnoL1M2DXmHJghlGzFca
teee5lZwXmOLwrVziqbhfPhyykDma8xffsz10wsaE4MC++aNb7D/6G0nv04a
xefdt+8bfIPAsGDVsUYaXucvL5FEjsAzobwjWEqDr1n93ERPgVZevN1gEY3R
oV5+foUC3mqr3K9l07jYzO902jWKR2+fl4al0Qh2eD+ZMzGKmCDR9KHTNBb/
ap7in6GENnfP9u+Jp6e3C1ddUUKIhdkf42nonGounshSwt3r8Y544lkLFh5I
KVBC7mP79sopGhZLuqLKJEqkvioyfBJLQ9KgncmpV8Jl9VQaP5pGSb3emGxM
iaaAOwmqxH435RmZrBIP56vGt5ygkesrFwT+rYTXiPisB3GLlIrQ+KxEn9uX
vvAoGtoONj62HAqKa58yJMdp3NlwNe6BPoWS/PbRCOKz71aIkg0oWAkv2W0g
Tu4elf1oROGtFaP6xzHS51ZQ5ZQJhfUdsTJFJI2QBxpBtmsoRLJrdpYSN4rN
K7lrKSy2+dx9nDhzr7b5CxsKs93kEzOIKcZJnGBLYfyeIr01gkZ0hPSMx2YK
hZ7LVl8iLrdefMV0K4UAjrB3P3F+WnLPlAMFcZJe4nJily4t21ZnCv8CmtX9
8Q==
      "]]}}, 
   {Dashing[{Small, Small}], {{}, {}, 
     {GrayLevel[0], LineBox[CompressedData["
1:eJxF1ns0FWsfB3CUQkJOp4tLtUuKIqUipa+UUihJqKgoarOEpKQSKiSvkKhI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       "]]}}, {{}, {}, 
     {GrayLevel[0], LineBox[CompressedData["
1:eJwdygs0VHkcB/AJm2gOK4dsdFBnt1UasSts8u2lcJJsefTCem1FazJne1nP
1E6UrSgl6SFnqQij2cp6JEUjE0NJ7TQeTXSv5X/vpFYGe3d/53zP93zO92cb
Hvd9lA6Px/Pl8l/La0wzXLM0uOK9qyftQyXGC/tmMSc0CIoafVk6TYIFGRUX
SjhTBcbHuvkSpAT6lc7hLI0ZnnKaL4HjSEabNlODjpiA4GE/CbJtdGbfP6aB
5fTHkqMlEmw5TAq9j2igvbrWzGNHFfq9Wmu2/qLB34kX0poe3QbU0a7anRqY
T+8Q+vXcgdfT+z5vAzUQPLhebNdeDfHF5Q6LPDVYWKfg76qthXHi+a0jjhqI
3R6KOsfrsea6dK7WWoOQrztlzWkNqB+ULBDzNUhaiXTlkkbEz1Toxo2xiO4K
fHNa+RCKhIr9BQMsfvYprYaoCQcnhE6+nSy8lu6LETk+hvwJk76mgcVkeGT1
UJcMW1KyS86Ws9B2tOgWJjyBvaeZ2c6LLJ78iJrl3nLcDAq2SMtk0fxZir9s
4ilECQnhlgdYqGJXBpcOt0F4uC7JOIqF7PUMtY+yHVbdjU7rN7HYYasKk9Qq
YLTi+uIZK1gI+lqSNko7UPZ+Yz1fwGLZp3JV6OVOnL3Xdf7kHBYR/FLnb688
gyQy2TNOn8WQR25K3uXnSNo+KD47ymDQz9e78VQXTkU5mi7vY1DV7nYyOu8F
/P9c5mzfxoDk2a8S5XTD62O03b5aBhmBaQ9Cj7/E0dXxkbjBQPh7nFKU/wp7
lR4u/ucZpDc1fuN18S9cE9Mb+44wsH+lY9wpVYJvcdddHs9AoaAq98lfY/7o
unqdHxhsEBOvRc9VSO2XZkh9GRjGb3YpSujBceuBocplDCqVuVtM9Hohh6ut
4UJuH+vhL8nsRdDeXfk95gyaZxsEbrPuQxbZdmtKj0FPzL3NRjf6wE8VGBQw
BLf7LW+aremHa2jDp3QVwWiYvtWBzn7k2e6vfdRKEDBmVRET8QarR2c+/vUe
QWzc3E0zeWrkmYfHnCrm9pTQtvJsNVy9z22ec4ZA5CUU+ji/BT9sxHgyleCn
3XtqTz57i6xG/YiFQgILciL/SOwAgr7jTTzbTvC5YMi72WQQ8om7ApkPwTyZ
neGJ0kEkFOU4feFG0FvVkli07h2EIX6JA18SqB2qnSPIO8yKagmdNCX4SlHJ
2/AbhSSX9dFlnHWtC1vNT1OgDFpjQzjrqMqkqhwK98vkB2tnEUTfsVDuzaMg
/Kc9O8mEoKXNOSG3iII880XTpBH3f+6MdqCawrFKtcOUAYF9Cm1aMEhhND16
6S3Omxx9InfTFMKCBtxDOV8IaGxbOkzBVTvoXTeDIOej4eRTDYUBz6GIZH0C
myu9JbpTFDy72dwpPYKijKq1QjMa5TdEBbc4kz2yYA8LGlZJ76+Fct7ROH6U
b0lDM+9DRZ0uwSVZuUuJDY2rsWMtyToEde6RVP8iGkY4pHDgbHU3QFkpoHHI
ZPyFahpBfnH4UKojDX+pVg3Oer3yEBsXGjXixKERHgFlJ2gYcaNht22SvcT5
vbp4ZZ07jZzFyWN+nK3NoMwCDd7/R7DnD5ITsorGvwH0fcA=
       "]]}}, {{}, {}, 
     {GrayLevel[0], LineBox[CompressedData["
1:eJxFlnk0lN8fx31LWqhooSxFtqJUQkr5VJLSZiQUIY0WFMaSpbKO0LdsZS88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       "]]}}}, 
   {Dashing[{0.03, 0.03}], LineBox[{{0, 1.}, {1., 1.}, {1., 0}}]}},
  AspectRatio->0.8333333333333334,
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox["x", HoldForm], TraditionalForm], 
    FormBox[
     TagBox["y", HoldForm], TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->All,
  PlotRangeClipping->True,
  PlotRangePadding->{
    Scaled[0.02], 
    Scaled[0.02]}]], "Output",
 CellChangeTimes->{3.5755448545953665`*^9, 3.586581479253898*^9, 
  3.587380855789477*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Fig. 11.2L Vertical pressure (p. 185)\
\>", "Subsection",
 CellChangeTimes->{{3.5019983477094083`*^9, 3.5019983500045395`*^9}, 
   3.50541120352195*^9, {3.587381343710704*^9, 3.587381347533243*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"y1", "=", "0"}], ";"}], "\n", 
 RowBox[{
  RowBox[{"\[CapitalDelta]py", "[", 
   RowBox[{"x_", ",", "y_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"syyi", "[", 
     RowBox[{"x", ",", "y1"}], "]"}]}], "-", "1"}]}], "\n", 
 RowBox[{"fig2a", "=", 
  RowBox[{
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"rx", "[", 
        RowBox[{"x1", ",", "y1"}], "]"}], ",", 
       RowBox[{"\[CapitalDelta]py", "[", 
        RowBox[{"x1", ",", "y1"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "0.7"}], ",", 
     RowBox[{"AxesLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"HoldForm", "[", "x", "]"}], ",", 
        "\"\<\[CapitalDelta]\[NegativeThinSpace]\!\(\*SubscriptBox[\(p\), \(y\
\)]\)\>\""}], "}"}]}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Black"}]}], "]"}], "//", 
   "\[IndentingNewLine]", 
   RowBox[{"Figure", "[", 
    RowBox[{"\"\<ces-pressure\>\"", ",", 
     RowBox[{"ImageSize", "\[Rule]", "200"}]}], "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.4144157527934246`*^9, 3.4144157533324246`*^9}, 
   3.4144162646024246`*^9, {3.4144163324044247`*^9, 3.414416340249425*^9}, {
   3.414732612113797*^9, 3.414732613311797*^9}, {3.4147349308167973`*^9, 
   3.4147349556477976`*^9}, {3.4147349988177977`*^9, 
   3.4147350032957973`*^9}, {3.4147350455697975`*^9, 3.414735080127797*^9}, {
   3.4147371357307973`*^9, 3.4147371464647975`*^9}, {3.4147379728177977`*^9, 
   3.414737973823797*^9}, {3.4147383979847975`*^9, 3.4147384043357973`*^9}, {
   3.4147391335527973`*^9, 3.4147391336487975`*^9}, {3.4318342409862084`*^9, 
   3.4318342425382085`*^9}, {3.467104431196659*^9, 3.4671044346278553`*^9}, {
   3.467104999291152*^9, 3.467105001924303*^9}, {3.4671050411795483`*^9, 
   3.467105045492795*^9}, {3.479208811977*^9, 3.4792088810620003`*^9}, {
   3.5014895186821985`*^9, 3.501489559046508*^9}, {3.501489598321754*^9, 
   3.501489724121949*^9}, 3.5054112035329504`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.575544854848399*^9, 3.586581479440962*^9, 
  3.587380855942457*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJw11nk4VN0fAPBJUdpMknpbNLQoqaYkUnxHG6qXaVGWtyLZdy2Sl0aUNc3Y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     "]]}},
  AspectRatio->0.7,
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox["x", HoldForm], TraditionalForm], 
    FormBox[
    "\"\[CapitalDelta]\[NegativeThinSpace]\\!\\(\\*SubscriptBox[\\(p\\), \\(y\
\\)]\\)\"", TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.5755448548524003`*^9, 3.5865814794429827`*^9, 
  3.5873808559445095`*^9}]
}, Open  ]],

Cell["Check integral (should vanish)", "Text",
 CellChangeTimes->{{3.4147371191227975`*^9, 3.4147371210177975`*^9}, 
   3.5054112035339503`*^9, {3.587381358535565*^9, 3.5873813637732573`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{"\[CapitalDelta]py", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "a"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4147350180067973`*^9, 3.4147350365717974`*^9}, {
   3.4147350856287975`*^9, 3.4147350856757975`*^9}, {3.4147371603017974`*^9, 
   3.4147371645567975`*^9}, 3.5054112035339503`*^9}],

Cell[BoxData[
 RowBox[{"-", "0.0007336499999999867`"}]], "Output",
 CellChangeTimes->{3.5755448549704137`*^9, 3.58658147948997*^9, 
  3.587380856009505*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Fig. 11.2R Shear stress (p. 188)\
\>", "Subsection",
 CellChangeTimes->{{3.501998374797958*^9, 3.501998376526057*^9}, 
   3.5054112035339503`*^9, {3.58738138488593*^9, 3.5873813900856037`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"y1", "=", "0.5"}], ";"}], "\n", 
 RowBox[{"fig2b", "=", 
  RowBox[{
   RowBox[{"ParametricPlot", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"rx", "[", 
        RowBox[{"x1", ",", "y1"}], "]"}], ",", 
       RowBox[{"sxyi", "[", 
        RowBox[{"x1", ",", "y1"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"x1", ",", "0", ",", "a"}], "}"}], ",", 
     RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Black"}], ",", 
     RowBox[{"AspectRatio", "\[Rule]", "0.7"}], ",", 
     RowBox[{"AxesLabel", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"HoldForm", "[", "x", "]"}], ",", 
        SubscriptBox["\[Sigma]", "xy"]}], "}"}]}]}], "]"}], "//", 
   "\[IndentingNewLine]", 
   RowBox[{"Figure", "[", 
    RowBox[{"\"\<ces-shear\>\"", ",", 
     RowBox[{"ImageSize", "\[Rule]", "Small"}]}], "]"}]}]}]}], "Input",
 CellChangeTimes->{
  3.4144154060164247`*^9, {3.4144157318664246`*^9, 3.4144157580264244`*^9}, 
   3.4144160420034246`*^9, {3.4144161587654247`*^9, 3.4144161632024245`*^9}, {
   3.4144162083344245`*^9, 3.414416211046425*^9}, {3.4147384249747972`*^9, 
   3.4147384253737974`*^9}, {3.4147392006387973`*^9, 
   3.4147392011827974`*^9}, {3.4318342494692087`*^9, 
   3.4318342513632083`*^9}, {3.4671043394404106`*^9, 
   3.4671043445637035`*^9}, {3.4671043978997545`*^9, 3.467104398243774*^9}, {
   3.4671050124999075`*^9, 3.467105017043167*^9}, {3.479208789686*^9, 
   3.4792087916210003`*^9}, {3.4847102468262*^9, 3.4847102476042*^9}, {
   3.5014897496644106`*^9, 3.501489762367137*^9}, 3.5054112035349503`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Figure", "::", "noexport"}], "MessageName"], 
  RowBox[{":", " "}], "\<\"Image not exported\"\>"}]], "Message", "MSG",
 CellChangeTimes->{3.575544855206442*^9, 3.586581479635988*^9, 
  3.587380856161484*^9}],

Cell[BoxData[
 GraphicsBox[{{}, {}, 
   {GrayLevel[0], LineBox[CompressedData["
1:eJxd2Hk4VO/7B3BbiSJJlGTXJkqLUnEjSUqk9DGiRSRL0sdSSShL1BCiRSqR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     "]]}},
  AspectRatio->0.7,
  Axes->True,
  AxesLabel->{
    FormBox[
     TagBox["x", HoldForm], TraditionalForm], 
    FormBox[
     SubscriptBox["\[Sigma]", "xy"], TraditionalForm]},
  AxesOrigin->{0, 0},
  Method->{},
  PlotRange->{All, All},
  PlotRangeClipping->True,
  PlotRangePadding->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{3.575544855209442*^9, 3.5865814796380525`*^9, 
  3.5873808561635294`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{1039, 740},
WindowMargins->{{169, Automatic}, {Automatic, 14}},
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 244, 5, 65, "Title"],
Cell[814, 29, 405, 8, 62, "Input",
 InitializationCell->True],
Cell[1222, 39, 194, 5, 41, "Input",
 InitializationCell->True],
Cell[1419, 46, 298, 7, 41, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1754, 58, 244, 4, 104, "Title"],
Cell[CellGroupData[{
Cell[2023, 66, 191, 2, 75, "Section"],
Cell[CellGroupData[{
Cell[2239, 72, 103, 2, 41, "Input"],
Cell[2345, 76, 311, 8, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2693, 89, 104, 1, 36, "Subsection"],
Cell[2800, 92, 78, 1, 30, "Text"],
Cell[2881, 95, 1011, 35, 96, "Input"],
Cell[3895, 132, 70, 1, 30, "Text"],
Cell[3968, 135, 301, 10, 62, "Input"],
Cell[4272, 147, 569, 17, 96, "Input"],
Cell[CellGroupData[{
Cell[4866, 168, 152, 3, 41, "Input"],
Cell[5021, 173, 410, 12, 58, "Output"]
}, Open  ]],
Cell[5446, 188, 73, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[5544, 193, 782, 26, 41, "Input"],
Cell[6329, 221, 119, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6485, 228, 874, 29, 41, "Input"],
Cell[7362, 259, 120, 2, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[7531, 267, 165, 2, 36, "Subsection"],
Cell[7699, 271, 67, 1, 30, "Text"],
Cell[7769, 274, 794, 27, 97, "Input"],
Cell[8566, 303, 68, 1, 30, "Text"],
Cell[8637, 306, 832, 28, 82, "Input"],
Cell[9472, 336, 77, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[9574, 341, 270, 8, 41, "Input"],
Cell[9847, 351, 164, 4, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10048, 360, 270, 8, 41, "Input"],
Cell[10321, 370, 164, 4, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10522, 379, 262, 8, 41, "Input"],
Cell[10787, 389, 163, 4, 41, "Output"]
}, Open  ]],
Cell[10965, 396, 258, 7, 30, "Text"],
Cell[CellGroupData[{
Cell[11248, 407, 262, 8, 41, "Input"],
Cell[11513, 417, 573, 18, 64, "Output"]
}, Open  ]],
Cell[12101, 438, 98, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[12224, 443, 391, 10, 57, "Input"],
Cell[12618, 455, 118, 2, 41, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[12797, 464, 226, 3, 75, "Section"],
Cell[CellGroupData[{
Cell[13048, 471, 121, 3, 41, "Input"],
Cell[13172, 476, 289, 8, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[13498, 489, 189, 2, 36, "Subsection"],
Cell[13690, 493, 167, 3, 41, "Input"],
Cell[CellGroupData[{
Cell[13882, 500, 71, 1, 41, "Input"],
Cell[13956, 503, 121, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14114, 510, 71, 1, 41, "Input"],
Cell[14188, 513, 119, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14344, 520, 72, 1, 41, "Input"],
Cell[14419, 523, 119, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14575, 530, 72, 1, 41, "Input"],
Cell[14650, 533, 118, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[14805, 540, 71, 1, 41, "Input"],
Cell[14879, 543, 121, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15037, 550, 152, 3, 41, "Input"],
Cell[15192, 555, 127, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15356, 562, 76, 1, 41, "Input"],
Cell[15435, 565, 124, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15596, 572, 72, 1, 41, "Input"],
Cell[15671, 575, 125, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[15833, 582, 76, 1, 41, "Input"],
Cell[15912, 585, 123, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16072, 592, 150, 3, 41, "Input"],
Cell[16225, 597, 123, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16385, 604, 73, 1, 41, "Input"],
Cell[16461, 607, 123, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16621, 614, 76, 1, 41, "Input"],
Cell[16700, 617, 121, 2, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[16870, 625, 98, 1, 36, "Subsection"],
Cell[CellGroupData[{
Cell[16993, 630, 74, 1, 41, "Input"],
Cell[17070, 633, 120, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17227, 640, 79, 1, 41, "Input"],
Cell[17309, 643, 129, 2, 41, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17475, 650, 76, 1, 41, "Input"],
Cell[17554, 653, 146, 3, 41, "Output"]
}, Open  ]],
Cell[17715, 659, 1658, 53, 222, "Input"],
Cell[CellGroupData[{
Cell[19398, 716, 314, 9, 41, "Input"],
Cell[19715, 727, 82494, 1344, 310, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[102246, 2076, 314, 9, 41, "Input"],
Cell[102563, 2087, 82669, 1347, 305, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[185269, 3439, 315, 9, 41, "Input"],
Cell[185587, 3450, 83228, 1355, 311, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[268852, 4810, 313, 9, 41, "Input"],
Cell[269168, 4821, 82120, 1338, 310, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[351325, 6164, 313, 9, 41, "Input"],
Cell[351641, 6175, 82693, 1347, 308, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[434371, 7527, 313, 9, 41, "Input"],
Cell[434687, 7538, 84358, 1375, 314, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[519082, 8918, 415, 12, 41, "Input"],
Cell[519500, 8932, 81876, 1334, 313, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[601413, 10271, 315, 9, 41, "Input"],
Cell[601731, 10282, 84020, 1369, 305, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[685800, 11657, 214, 4, 36, "Subsection"],
Cell[CellGroupData[{
Cell[686039, 11665, 1356, 30, 82, "Input"],
Cell[687398, 11697, 257, 6, 21, "Message"],
Cell[687658, 11705, 40611, 886, 384, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[728318, 12597, 159, 2, 36, "Subsection"],
Cell[728480, 12601, 486, 15, 62, "Input"],
Cell[CellGroupData[{
Cell[728991, 12620, 604, 15, 41, "Input"],
Cell[729598, 12637, 3333, 62, 371, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[732968, 12704, 551, 15, 41, "Input"],
Cell[733522, 12721, 2364, 47, 393, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[735923, 12773, 142, 2, 27, "Subsubsection"],
Cell[736068, 12777, 1016, 34, 96, "Input"],
Cell[CellGroupData[{
Cell[737109, 12815, 556, 15, 41, "Input"],
Cell[737668, 12832, 3400, 63, 381, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[741105, 12900, 554, 15, 41, "Input"],
Cell[741662, 12917, 2053, 41, 384, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[743752, 12963, 551, 15, 41, "Input"],
Cell[744306, 12980, 3538, 66, 363, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[747881, 13051, 285, 7, 41, "Input"],
Cell[748169, 13060, 8079, 137, 290, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[756285, 13202, 611, 19, 41, "Input"],
Cell[756899, 13223, 200, 4, 384, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[757136, 13232, 1225, 25, 82, "Input"],
Cell[758364, 13259, 256, 6, 21, "Message"],
Cell[758623, 13267, 13498, 237, 325, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[772182, 13511, 208, 4, 36, "Subsection"],
Cell[CellGroupData[{
Cell[772415, 13519, 2148, 47, 122, "Input"],
Cell[774566, 13568, 255, 6, 21, "Message"],
Cell[774824, 13576, 5205, 95, 275, "Output"]
}, Open  ]],
Cell[780044, 13674, 192, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[780261, 13680, 420, 9, 41, "Input"],
Cell[780684, 13691, 156, 3, 41, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[780889, 13700, 203, 4, 36, "Subsection"],
Cell[CellGroupData[{
Cell[781117, 13708, 1647, 37, 102, "Input"],
Cell[782767, 13747, 255, 6, 21, "Message"],
Cell[783025, 13755, 6405, 114, 275, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
