(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     11623,        362]
NotebookOptionsPosition[      9592,        287]
NotebookOutlinePosition[     10299,        313]
CellTagsIndexPosition[     10214,        308]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Package: CleanGlobal", "Title",
 CellChangeTimes->{{3.5052826178940787`*^9, 3.505282621269272*^9}, {
  3.588479794202591*^9, 3.5884798128149595`*^9}, {3.588480702006892*^9, 
  3.588480702174966*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"BeginPackage", "[", "\"\<Lautrup`CleanGlobal`\>\"", "]"}], ";"}], 
  "\n", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "first", " ", "unprotect", " ", "all", " ", "symbols", " ", "in", " ", 
    "this", " ", "context"}], " ", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"Unprotect", "[", "\"\<CleanGlobal\>\"", "]"}], ";"}], "\n", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", "messages", " ", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"CleanGlobal", "::", "usage"}], " ", "=", " ", 
   "\"\<CleanGlobal[] removes all symbols in the Global context and resets \
the line counters\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"CleanGlobal", "::", "removed"}], " ", "=", " ", 
    "\"\<`1` symbols\>\""}], ";"}], "\n"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], ";"}], 
  "\[IndentingNewLine]", "\n", 
  RowBox[{"(*", " ", 
   RowBox[{"clean", " ", "global", " ", "context"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"CleanGlobal", "[", "]"}], " ", ":=", " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"SetDirectory", "[", 
     RowBox[{"NotebookDirectory", "[", 
      RowBox[{"SelectedNotebook", "[", "]"}], "]"}], "]"}], ";", "\n", "  ", 
    RowBox[{"length", " ", "=", " ", 
     RowBox[{"Length", "[", 
      RowBox[{"Names", "[", "\"\<Global`*\>\"", "]"}], "]"}]}], ";", "\n", 
    "  ", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"length", " ", ">", " ", "0"}], ",", " ", 
      RowBox[{"Remove", "[", "\"\<Global`*\>\"", "]"}]}], "]"}], ";", "\n", 
    "  ", 
    RowBox[{"Unprotect", "[", 
     RowBox[{
     "\"\<In\>\"", ",", " ", "\"\<Out\>\"", ",", " ", "\"\<InString\>\"", ",",
       " ", "\"\<MessageList\>\""}], "]"}], ";", "\n", "  ", 
    RowBox[{"Clear", "[", 
     RowBox[{
     "In", ",", " ", "Out", ",", " ", "InString", ",", " ", "MessageList"}], 
     "]"}], ";", "\n", "  ", 
    RowBox[{"Protect", "[", 
     RowBox[{
     "In", ",", " ", "Out", ",", " ", "InString", ",", " ", "MessageList"}], 
     "]"}], ";", "\n", "  ", 
    RowBox[{"$Line", " ", "=", " ", "0"}], ";", "\n", "  ", 
    RowBox[{"Message", "[", 
     RowBox[{
      RowBox[{"CleanGlobal", "::", "removed"}], ",", " ", "length"}], "]"}]}],
    ")"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"End", "[", "]"}], ";"}], "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetAttributes", "[", 
    RowBox[{"CleanGlobal", ",", " ", 
     RowBox[{"{", 
      RowBox[{"Protected", ",", " ", "ReadProtected"}], "}"}]}], "]"}], ";"}],
   "\[IndentingNewLine]"}], "\n", 
 RowBox[{"EndPackage", "[", "]"}], "\[IndentingNewLine]"}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5052826539141393`*^9, 3.5052826645687485`*^9}, {
   3.5052827148876266`*^9, 3.5052827598822002`*^9}, {3.5052831472283554`*^9, 
   3.5052831824423695`*^9}, {3.5052855270404725`*^9, 
   3.5052855667067413`*^9}, {3.5052861089927588`*^9, 3.505286124039619*^9}, {
   3.505288676188594*^9, 3.5052887049602394`*^9}, {3.5052924538666644`*^9, 
   3.5052924939279556`*^9}, {3.505292528247919*^9, 3.505292543076767*^9}, 
   3.5052939938967495`*^9, {3.508230748739792*^9, 3.5082308189228067`*^9}, {
   3.570774675594291*^9, 3.5707746921543503`*^9}, {3.5707774432558537`*^9, 
   3.5707774460152035`*^9}, {3.5707775895923*^9, 3.570777605248287*^9}, 
   3.5884797942035933`*^9}],

Cell[CellGroupData[{

Cell["Clean the global context (see also Utilities`CleanSlate`)", "Section",
 CellChangeTimes->{{3.505109789211809*^9, 3.5051097960712013`*^9}, {
   3.5051102683912168`*^9, 3.505110308255497*^9}, {3.5052924348955793`*^9, 
   3.505292435573618*^9}, 3.5884797941985855`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "CleanGlobal"}]], "Input",
 CellChangeTimes->{{3.505109832741299*^9, 3.5051098373445625`*^9}, {
   3.505109949688988*^9, 3.505109949848997*^9}, 3.5051137289691505`*^9, 
   3.5884797941985855`*^9}],

Cell[BoxData[
 StyleBox["\<\"CleanGlobal[] removes all symbols in the Global context and \
resets the line counters\"\>", "MSG"]], "Print", "PrintUsage",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.570777702221535*^9, 3.5884797941995883`*^9},
 CellTags->"Info3570781302-1927465"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsection",
 CellChangeTimes->{{3.505113753566558*^9, 3.505113754782627*^9}, 
   3.5884797941995883`*^9}],

Cell["All names in the global context are removed", "Text",
 CellChangeTimes->{{3.5051099658709135`*^9, 3.5051099775025787`*^9}, 
   3.5884797941995883`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"x", "=", "5"}], ";", 
  RowBox[{"y", "=", "3"}], ";", "z", ";"}]], "Input",
 CellChangeTimes->{{3.505109956555381*^9, 3.505109958863513*^9}, {
   3.5051103577063255`*^9, 3.505110362120578*^9}, 3.5884797941995883`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Names", "[", "\"\<Global`*\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.505109982778881*^9, 3.50510999832977*^9}, 
   3.5884797941995883`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"x\"\>", ",", "\<\"y\"\>", ",", "\<\"z\"\>"}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.5707774819327264`*^9, 3.5707776173458843`*^9, 
  3.5707776634586563`*^9, 3.5707777022845774`*^9, 3.5884797941995883`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.505110012612587*^9, 3.5051100154167476`*^9}, 
   3.5884797942006063`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], ":", 
  " ", "\<\"\[NoBreak]\\!\\(3\\)\[NoBreak] symbols\"\>"}]], "Message", "MSG",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.5707777024415874`*^9, 3.5884797942006063`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Names", "[", "\"\<Global`*\>\"", "]"}]], "Input",
 CellChangeTimes->{3.5884797942006063`*^9}],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.5707774821217527`*^9, 3.570777617537857*^9, 
  3.570777663494647*^9, 3.570777702460597*^9, 3.5884797942006063`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "CleanGlobal"}]], "Input",
 CellChangeTimes->{{3.505109832741299*^9, 3.5051098373445625`*^9}, {
   3.505109949688988*^9, 3.505109949848997*^9}, 3.5051137289691505`*^9, 
   3.5884797942006063`*^9}],

Cell[BoxData[
 StyleBox["\<\"CleanGlobal[] removes all symbols in the Global context and \
resets the line counters\"\>", "MSG"]], "Print", "PrintUsage",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.5707777025595913`*^9, 3.5884797942015915`*^9},
 CellTags->"Info3570781302-1927465"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsection",
 CellChangeTimes->{{3.505113753566558*^9, 3.505113754782627*^9}, 
   3.5884797942015915`*^9}],

Cell["All names in the global context are removed", "Text",
 CellChangeTimes->{{3.5051099658709135`*^9, 3.5051099775025787`*^9}, 
   3.5884797942015915`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"x", "=", "5"}], ";", 
  RowBox[{"y", "=", "3"}], ";", "z", ";"}]], "Input",
 CellChangeTimes->{{3.505109956555381*^9, 3.505109958863513*^9}, {
   3.5051103577063255`*^9, 3.505110362120578*^9}, 3.5884797942015915`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Names", "[", "\"\<Global`*\>\"", "]"}]], "Input",
 CellChangeTimes->{{3.505109982778881*^9, 3.50510999832977*^9}, 
   3.5884797942015915`*^9}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"x\"\>", ",", "\<\"y\"\>", ",", "\<\"z\"\>"}], "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.5707774822798033`*^9, 3.570777617700878*^9, 
  3.570777663633663*^9, 3.57077770261557*^9, 3.5884797942015915`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.505110012612587*^9, 3.5051100154167476`*^9}, 
   3.5884797942015915`*^9}],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"CleanGlobal", "::", "removed"}], "MessageName"], ":", 
  " ", "\<\"\[NoBreak]\\!\\(3\\)\[NoBreak] symbols\"\>"}]], "Message", "MSG",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.5707777027595916`*^9, 3.5884797942015915`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Names", "[", "\"\<Global`*\>\"", "]"}]], "Input",
 CellChangeTimes->{3.588479794202591*^9}],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output",
 GeneratedCell->False,
 CellAutoOverwrite->False,
 CellChangeTimes->{3.570777482465765*^9, 3.570777617892912*^9, 
  3.570777663674694*^9, 3.570777702777628*^9, 3.588479794202591*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{707, 867},
WindowMargins->{{476, Automatic}, {Automatic, 84}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Report"}, "StandardReport.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3570781302-1927465"->{
  Cell[4761, 129, 306, 6, 40, "Print",
   CellTags->"Info3570781302-1927465"],
  Cell[7282, 210, 308, 6, 40, "Print",
   CellTags->"Info3570781302-1927465"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3570781302-1927465", 10032, 300}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 205, 3, 99, "Title"],
Cell[787, 27, 3425, 85, 581, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[4237, 116, 272, 3, 88, "Section"],
Cell[CellGroupData[{
Cell[4534, 123, 224, 4, 40, "Input"],
Cell[4761, 129, 306, 6, 40, "Print",
 CellTags->"Info3570781302-1927465"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5104, 140, 123, 2, 37, "Subsection"],
Cell[5230, 144, 157, 2, 30, "Text"],
Cell[5390, 148, 252, 5, 40, "Input"],
Cell[CellGroupData[{
Cell[5667, 157, 166, 3, 40, "Input"],
Cell[5836, 162, 300, 6, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6173, 173, 155, 3, 40, "Input"],
Cell[6331, 178, 298, 7, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6666, 190, 117, 2, 40, "Input"],
Cell[6786, 194, 232, 5, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7055, 204, 224, 4, 40, "Input"],
Cell[7282, 210, 308, 6, 40, "Print",
 CellTags->"Info3570781302-1927465"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[7639, 222, 123, 2, 37, "Subsection"],
Cell[7765, 226, 157, 2, 30, "Text"],
Cell[7925, 230, 252, 5, 40, "Input"],
Cell[CellGroupData[{
Cell[8202, 239, 166, 3, 40, "Input"],
Cell[8371, 244, 293, 6, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8701, 255, 155, 3, 40, "Input"],
Cell[8859, 260, 298, 7, 21, "Message"]
}, Open  ]],
Cell[CellGroupData[{
Cell[9194, 272, 115, 2, 40, "Input"],
Cell[9312, 276, 228, 5, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
