(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     26307,        721]
NotebookOptionsPosition[     23201,        610]
NotebookOutlinePosition[     23909,        636]
CellTagsIndexPosition[     23824,        631]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Package: General", "Title",
 CellChangeTimes->{{3.5052826178940787`*^9, 3.505282621269272*^9}, {
  3.588479400766636*^9, 3.5884794032788935`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"BeginPackage", "[", "\"\<Lautrup`General`\>\"", "]"}], ";"}], 
  "\n", "\n", 
  RowBox[{"(*", " ", 
   RowBox[{
   "first", " ", "unprotect", " ", "all", " ", "symbols", " ", "in", " ", 
    "this", " ", "context"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Unprotect", "[", "\"\<`*\>\"", "]"}], ";"}], 
  "\[IndentingNewLine]", "\n", 
  RowBox[{"(*", " ", "messages", " ", "*)"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"HoldSubscripts", "::", "usage"}], " ", "=", " ", 
   "\"\<HoldSubscripts[] makes subscripted heads work as a single \
symbol\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ReleaseSubscripts", "::", "usage"}], " ", "=", " ", 
    "\"\<ReleaseSubscripts[] make subscripted heads work as in standard \
Mathematica\>\""}], ";"}], "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"DoCollect", "::", "usage"}], "=", 
   "\"\<DoCollect[args][exp] works as Collect[exp,args]\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"DoSeries", "::", "usage"}], "=", 
   "\"\<DoSeries[args][exp] works as Series[exp,args]\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DoLimit", "::", "usage"}], "=", 
    "\"\<DoLimit[args][exp] works as Limit[exp,args]\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "permit", " ", "input", " ", "powers", " ", "of", " ", "derivatives"}], 
   " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"MakeExpression", "[", 
    RowBox[{
     RowBox[{"RowBox", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"SuperscriptBox", "[", 
         RowBox[{
          RowBox[{"SubscriptBox", " ", "[", 
           RowBox[{"\"\<\[PartialD]\>\"", ",", " ", "x_"}], "]"}], ",", " ", 
          "n_"}], "]"}], ",", " ", "f_"}], "}"}], "]"}], ",", " ", 
     "StandardForm"}], "]"}], " ", ":=", "\n", " ", 
   RowBox[{"HoldComplete", "[", 
    RowBox[{"D", "[", 
     RowBox[{
      RowBox[{"ToExpression", "[", "f", "]"}], ",", " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"ToExpression", "[", "x", "]"}], ",", " ", 
        RowBox[{"ToExpression", "[", "n", "]"}]}], "}"}]}], "]"}], "]"}]}], 
  "\n", "\n", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"hold", " ", "and", " ", "release", " ", "subscripts"}], ";", " ", 
    RowBox[{"alternative", " ", "way"}]}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"HoldSubscripts", "[", "]"}], " ", ":=", " ", 
  RowBox[{"Block", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", 
    RowBox[{
     RowBox[{"SetSystemOptions", "[", 
      RowBox[{"\"\<DifferentiationOptions\>\"", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"\"\<DifferentiateHeads\>\"", "\[Rule]", "False"}], "}"}]}], 
      "]"}], ";", 
     RowBox[{"SetAttributes", "[", 
      RowBox[{"Subscript", ",", " ", 
       RowBox[{"{", 
        RowBox[{"Constant", ",", "HoldAllComplete"}], "}"}]}], "]"}]}]}], 
   "]"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"ReleaseSubscripts", "[", "]"}], " ", ":=", " ", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{
      RowBox[{"SetSystemOptions", "[", 
       RowBox[{"\"\<DifferentiationOptions\>\"", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{"\"\<DifferentiateHeads\>\"", "\[Rule]", "True"}], "}"}]}], 
       "]"}], ";", 
      RowBox[{"ClearAttributes", "[", 
       RowBox[{"Subscript", ",", " ", 
        RowBox[{"{", 
         RowBox[{"Constant", ",", "HoldAllComplete"}], "}"}]}], "]"}]}]}], 
    "]"}]}], "\[IndentingNewLine]", "\n", 
  RowBox[{"(*", " ", 
   RowBox[{"convenience", " ", "functions"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"DoCollect", "[", "args__", "]"}], "[", "exp_", "]"}], " ", ":=", 
  " ", 
  RowBox[{"Collect", "[", 
   RowBox[{"exp", ",", " ", "args"}], "]"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"DoSeries", "[", "args__", "]"}], "[", "exp_", "]"}], " ", ":=", 
  " ", 
  RowBox[{"Series", "[", 
   RowBox[{"exp", ",", " ", "args"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"DoLimit", "[", "args__", "]"}], "[", "exp_", "]"}], " ", ":=", 
  " ", 
  RowBox[{"Limit", "[", 
   RowBox[{"exp", ",", " ", "args"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"End", "[", "]"}], ";"}], "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{"SetAttributes", "[", 
   RowBox[{
    RowBox[{"Evaluate", "[", 
     RowBox[{"Names", "[", "\"\<`*\>\"", "]"}], "]"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{"Protected", ",", " ", "ReadProtected"}], "}"}]}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"EndPackage", "[", "]"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", "initialize", " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"HoldSubscripts", "[", "]"}], ";"}], 
  " "}], "\[IndentingNewLine]"}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5052826539141393`*^9, 3.5052826645687485`*^9}, {
   3.5052827148876266`*^9, 3.5052827598822002`*^9}, {3.5052831472283554`*^9, 
   3.5052831824423695`*^9}, {3.5052855270404725`*^9, 
   3.5052855667067413`*^9}, {3.5052861089927588`*^9, 3.505286124039619*^9}, {
   3.505288676188594*^9, 3.5052887049602394`*^9}, 3.5052926300987444`*^9, 
   3.505292704356992*^9, {3.5054117637479925`*^9, 3.505411804240309*^9}, {
   3.505411872817231*^9, 3.505411956732031*^9}, 3.5054770223136406`*^9, 
   3.5054770711484337`*^9, {3.5068499255358057`*^9, 3.5068499455669518`*^9}, {
   3.5068500637747126`*^9, 3.5068500664468656`*^9}, {3.506850208541993*^9, 
   3.5068502206386847`*^9}, 3.5068505537507377`*^9, {3.507180741027769*^9, 
   3.507180753779498*^9}, {3.5071807847472696`*^9, 3.5071808019962564`*^9}, {
   3.5071808684030547`*^9, 3.5071808952175884`*^9}, {3.5075275829201264`*^9, 
   3.5075275950488205`*^9}, {3.5075277224381065`*^9, 3.507527740901162*^9}, {
   3.570774493820386*^9, 3.570774516179124*^9}, {3.570777736584923*^9, 
   3.5707777422736044`*^9}, {3.5753701807137733`*^9, 3.575370190732031*^9}, {
   3.5753707141587744`*^9, 3.575370723730966*^9}, {3.575865152779868*^9, 
   3.5758651557792854`*^9}, {3.5758661384136887`*^9, 
   3.5758661427892003`*^9}, {3.575866358677327*^9, 3.5758663692586546`*^9}, {
   3.5758674443606567`*^9, 3.575867472113141*^9}, {3.5758710140399113`*^9, 
   3.5758710175393257`*^9}, {3.575871056583189*^9, 3.5758710609748263`*^9}, 
   3.5758710960631437`*^9, {3.5759466803070292`*^9, 3.575946819524511*^9}, {
   3.575946925300785*^9, 3.5759469699533978`*^9}, {3.5866596504939833`*^9, 
   3.586659674055917*^9}, {3.5866597327743225`*^9, 3.5866597355656366`*^9}, {
   3.5866607037741985`*^9, 3.586660729945484*^9}, 3.588479246525019*^9, 
   3.5884792918627377`*^9}],

Cell[CellGroupData[{

Cell["Allow powers of derivatives", "Section",
 CellChangeTimes->{{3.505109746774382*^9, 3.5051097548468437`*^9}}],

Cell[CellGroupData[{

Cell["Example", "Subsection",
 CellChangeTimes->{{3.5051137855253854`*^9, 3.50511378665345*^9}}],

Cell[TextData[{
 "This is a convenient notation (not allowed by standard ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ")"
}], "Text",
 CellChangeTimes->{{3.5051009733905735`*^9, 3.5051009770697837`*^9}, {
   3.505101020790284*^9, 3.505101029320772*^9}, 3.5051016764457855`*^9, {
   3.505101820923049*^9, 3.5051018263433595`*^9}, 3.5051019075990067`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   SubscriptBox["\[PartialD]", "x"], "5"], 
  SuperscriptBox["x", "6"]}]], "Input",
 CellChangeTimes->{{3.505030164153721*^9, 3.5050301819207373`*^9}, 
   3.505101676446786*^9}],

Cell[BoxData[
 RowBox[{"720", " ", "x"}]], "Output",
 CellChangeTimes->{
  3.570774420330018*^9, 3.5707777564163857`*^9, 3.5759479187275996`*^9, 
   3.5866597761767645`*^9, {3.5866607455704803`*^9, 3.58666076168647*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Hold subscripts", "Section",
 CellChangeTimes->{{3.5050295648324423`*^9, 3.5050295656954913`*^9}, {
   3.505029597519312*^9, 3.5050296238608184`*^9}, {3.505029677824905*^9, 
   3.505029681328105*^9}, {3.505101046518756*^9, 3.5051010569213505`*^9}, 
   3.5051016764237843`*^9, 3.505109423627899*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "HoldSubscripts"}]], "Input",
 CellChangeTimes->{{3.5051012049088154`*^9, 3.505101209520079*^9}, 
   3.5051016764247847`*^9, 3.505108430331086*^9, {3.505113270671938*^9, 
   3.5051132831586514`*^9}, {3.5051136445103197`*^9, 3.5051136872687654`*^9}}],

Cell[BoxData[
 StyleBox["\<\"HoldSubscripts[] makes subscripted heads work as a single \
symbol\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5866607618204865`*^9},
 CellTags->"Info3586667961-5218291"]
}, Open  ]],

Cell["Implication for attributes", "Text",
 CellChangeTimes->{{3.5051094152624207`*^9, 3.505109417542551*^9}, {
  3.505109481176191*^9, 3.5051094841423607`*^9}, {3.5051133775510507`*^9, 
  3.5051133813582687`*^9}}],

Cell[BoxData[
 RowBox[{"HoldSubscripts", "[", "]"}]], "Input",
 CellChangeTimes->{{3.505029769162129*^9, 3.5050297731133547`*^9}, 
   3.505101676428785*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "Subscript", "]"}]], "Input",
 CellChangeTimes->{{3.5051086238771563`*^9, 3.5051086380989695`*^9}}],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"Constant", ",", "HoldAllComplete", ",", "NHoldRest"}], 
  "}"}]], "Output",
 CellChangeTimes->{
  3.505108638598998*^9, 3.5051086686757183`*^9, 3.5051087296412053`*^9, 
   3.505108901791052*^9, {3.505109385551721*^9, 3.5051094016576424`*^9}, 
   3.505109491614788*^9, 3.5051095992369432`*^9, 3.5051097591450896`*^9, 
   3.5051098439779415`*^9, 3.5051098780158887`*^9, {3.505110008431348*^9, 
   3.505110030340601*^9}, 3.5051102407756376`*^9, 3.505110366426824*^9, 
   3.5051109714324284`*^9, 3.5051111060011253`*^9, 3.50511130277538*^9, 
   3.5051131706092143`*^9, 3.5051132479076357`*^9, {3.5051133155495043`*^9, 
   3.505113321469843*^9}, 3.5051133567248597`*^9, 3.5051136502046456`*^9, 
   3.5051137004045167`*^9, 3.505114750671589*^9, {3.5051980110625544`*^9, 
   3.505198020489094*^9}, {3.505198532988407*^9, 3.5051985524695215`*^9}, 
   3.5707744204900703`*^9, 3.5707777565654144`*^9, 3.5759479189235463`*^9, 
   3.586659776324771*^9, {3.5866607457135835`*^9, 3.5866607618714957`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsection",
 CellChangeTimes->{{3.5051137685094123`*^9, 3.5051137696364765`*^9}}],

Cell["This is what is normally wanted", "Text",
 CellChangeTimes->{{3.5051094976071305`*^9, 3.5051095066796494`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubscriptBox["\[PartialD]", "x"], 
  RowBox[{
   SubscriptBox["f", "x"], "[", "x", "]"}]}]], "Input",
 CellChangeTimes->{{3.505029781466833*^9, 3.505029784145986*^9}, 
   3.5051016764307847`*^9, 3.5884723350443716`*^9}],

Cell[BoxData[
 RowBox[{
  SuperscriptBox[
   SubscriptBox["f", "x"], "\[Prime]",
   MultilineFunction->None], "[", "x", "]"}]], "Output",
 CellChangeTimes->{{3.505104528093891*^9, 3.505104536101349*^9}, {
   3.505108145839814*^9, 3.505108171347273*^9}, 3.5051082966554403`*^9, 
   3.5051083555828104`*^9, 3.505108425802827*^9, 3.5051086686937194`*^9, 
   3.5051087296582065`*^9, 3.505108901808053*^9, {3.5051093856957293`*^9, 
   3.5051094018106513`*^9}, 3.5051094916327887`*^9, 3.5051095992569447`*^9, 
   3.5051097591650906`*^9, 3.505109843992943*^9, 3.50510987803689*^9, {
   3.5051100084603496`*^9, 3.505110030357602*^9}, 3.505110240793638*^9, 
   3.5051103664508257`*^9, 3.5051109714494295`*^9, 3.505111106017126*^9, 
   3.505111302794381*^9, 3.5051132479226365`*^9, {3.5051133155735054`*^9, 
   3.505113321501845*^9}, 3.50511335673786*^9, 3.5051136502246466`*^9, 
   3.505113700422518*^9, 3.50511475069059*^9, {3.5051980110775557`*^9, 
   3.505198020506095*^9}, {3.5051985330064077`*^9, 3.5051985524865227`*^9}, 
   3.5707744205120726`*^9, 3.5707777565824413`*^9, 3.575947918940549*^9, 
   3.5866597763407745`*^9, {3.586660745728505*^9, 3.586660761886527*^9}}]
}, Open  ]],

Cell["but this would normally be undesired", "Text",
 CellChangeTimes->{{3.5050299553677797`*^9, 3.5050299617521443`*^9}, {
   3.5050301077524953`*^9, 3.5050301089275627`*^9}, {3.5051012957190094`*^9, 
   3.5051012991102033`*^9}, {3.5051016207586007`*^9, 3.5051016210226154`*^9}, 
   3.505101676433785*^9, {3.505108470487383*^9, 3.505108481551016*^9}}],

Cell[BoxData[
 RowBox[{"ReleaseSubscripts", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5050297124338846`*^9, 3.5050297184332275`*^9}, 
   3.505101676434785*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"y", "=", 
  RowBox[{
   SubscriptBox["\[PartialD]", "x"], 
   RowBox[{
    SubscriptBox["f", "x"], "[", "x", "]"}]}]}]], "Input",
 CellChangeTimes->{{3.505029728735817*^9, 3.5050297513061075`*^9}, 
   3.5050297969737196`*^9, 3.5051016764357853`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   SuperscriptBox[
    SubscriptBox["f", "x"], "\[Prime]",
    MultilineFunction->None], "[", "x", "]"}], "+", 
  RowBox[{
   RowBox[{
    SuperscriptBox["Subscript", 
     TagBox[
      RowBox[{"(", 
       RowBox[{"0", ",", "1"}], ")"}],
      Derivative],
     MultilineFunction->None], "[", 
    RowBox[{"f", ",", "x"}], "]"}], "[", "x", "]"}]}]], "Output",
 CellChangeTimes->{{3.505104528114892*^9, 3.50510453612035*^9}, {
   3.505108145858815*^9, 3.505108171370274*^9}, 3.505108296677441*^9, 
   3.505108355606812*^9, 3.505108425831828*^9, 3.505108668722721*^9, 
   3.505108729686208*^9, 3.505108901855056*^9, {3.505109385718731*^9, 
   3.505109401830652*^9}, 3.5051094916567903`*^9, 3.505109599281946*^9, 
   3.5051097591930923`*^9, 3.5051098440189443`*^9, 3.505109878060891*^9, {
   3.505110008493352*^9, 3.505110030378603*^9}, 3.50511024081664*^9, 
   3.505110366473827*^9, 3.505110971475431*^9, 3.5051111060411277`*^9, 
   3.505111302817383*^9, 3.505113247943638*^9, {3.5051133156075077`*^9, 
   3.505113321539847*^9}, 3.505113356766862*^9, 3.505113650251648*^9, 
   3.505113700444519*^9, 3.505114750713591*^9, {3.5051980110975566`*^9, 
   3.5051980205290956`*^9}, {3.5051985330294094`*^9, 3.505198552506523*^9}, 
   3.5707744205360746`*^9, 3.570777756603433*^9, 3.5759479189595203`*^9, 
   3.5866597763627777`*^9, {3.5866607457494698`*^9, 3.5866607619054976`*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Release subscripts", "Section",
 CellChangeTimes->{{3.5051094351985607`*^9, 3.5051094383587418`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "ReleaseSubscripts"}]], "Input",
 CellChangeTimes->{{3.5051012387577515`*^9, 3.50510124275298*^9}, 
   3.5051016764267845`*^9, 3.505108435081358*^9, 3.505113659217161*^9}],

Cell[BoxData[
 StyleBox["\<\"ReleaseSubscripts[] make subscripted heads work as in standard \
Mathematica\"\>", "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.586660762007526*^9},
 CellTags->"Info3586667961-5218291"]
}, Open  ]],

Cell["See what happens", "Text",
 CellChangeTimes->{{3.505109450470434*^9, 3.5051094531105857`*^9}}],

Cell[BoxData[
 RowBox[{"ReleaseSubscripts", "[", "]"}]], "Input",
 CellChangeTimes->{{3.5050297124338846`*^9, 3.5050297184332275`*^9}, 
   3.505101676434785*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "Subscript", "]"}]], "Input",
 CellChangeTimes->{{3.5051086238771563`*^9, 3.5051086380989695`*^9}}],

Cell[BoxData[
 RowBox[{"{", "NHoldRest", "}"}]], "Output",
 CellChangeTimes->{
  3.505108638598998*^9, 3.5051086686757183`*^9, 3.5051087296412053`*^9, 
   3.5051089018350544`*^9, {3.5051093856697283`*^9, 3.5051094017806497`*^9}, 
   3.505109542381692*^9, 3.5051095993889523`*^9, 3.505109759308099*^9, 
   3.5051098441299505`*^9, 3.5051098781898985`*^9, {3.5051100086093583`*^9, 
   3.50511003049361*^9}, 3.5051102409276457`*^9, 3.505110366596834*^9, 
   3.5051109715894375`*^9, 3.505111106164135*^9, 3.505111302933389*^9, 
   3.505113248057644*^9, {3.5051133157565165`*^9, 3.5051133216918554`*^9}, 
   3.505113357204887*^9, 3.505113650365655*^9, 3.5051147508245974`*^9, {
   3.5051980112185636`*^9, 3.5051980206381025`*^9}, {3.5051985331294155`*^9, 
   3.5051985526105294`*^9}, 3.5707744207070985`*^9, 3.570777756761441*^9, 
   3.5759479191275787`*^9, 3.586657636222147*^9, 3.5866579463960023`*^9, 
   3.5866597765298223`*^9, {3.5866607459125257`*^9, 3.5866607620605555`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example", "Subsection",
 CellChangeTimes->{{3.5051137767018805`*^9, 3.505113779374034*^9}, {
  3.5884723416122*^9, 3.58847235302763*^9}}],

Cell["This is the normal desired behavior", "Text",
 CellChangeTimes->{{3.5051016370695333`*^9, 3.505101642645852*^9}, 
   3.5051016764397855`*^9, {3.5051095453768625`*^9, 3.505109551903236*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Sum]", 
   RowBox[{"n", "=", "0"}], "5"], 
  SubscriptBox["x", "n"]}]], "Input",
 CellChangeTimes->{{3.5050298558440866`*^9, 3.505029874691165*^9}, 
   3.5051016764407854`*^9}],

Cell[BoxData[
 RowBox[{
  SubscriptBox["x", "0"], "+", 
  SubscriptBox["x", "1"], "+", 
  SubscriptBox["x", "2"], "+", 
  SubscriptBox["x", "3"], "+", 
  SubscriptBox["x", "4"], "+", 
  SubscriptBox["x", "5"]}]], "Output",
 CellChangeTimes->{{3.505104528134893*^9, 3.505104536137351*^9}, {
   3.5051081458818164`*^9, 3.505108171390275*^9}, 3.5051082967034426`*^9, 
   3.5051083556278133`*^9, 3.5051084258518295`*^9, 3.5051086687627234`*^9, 
   3.5051087297192097`*^9, 3.5051089019040585`*^9, {3.5051093857437325`*^9, 
   3.5051094018516536`*^9}, 3.5051095423966923`*^9, 3.5051095994079533`*^9, 
   3.5051097593231*^9, 3.5051098441509514`*^9, 3.5051098782129*^9, {
   3.5051100086283593`*^9, 3.505110030509611*^9}, 3.505110240942647*^9, 
   3.505110366612835*^9, 3.5051109716064386`*^9, 3.5051111061811357`*^9, 
   3.50511130294739*^9, 3.5051132480756454`*^9, {3.505113315774517*^9, 
   3.5051133217078567`*^9}, 3.505113357226888*^9, 3.5051136503856564`*^9, 
   3.5051147508385983`*^9, {3.5051980112345643`*^9, 3.5051980206521034`*^9}, {
   3.5051985331454163`*^9, 3.5051985526265306`*^9}, 3.570774420724068*^9, 
   3.570777756781417*^9, 3.575947919144543*^9, 3.5866576362470517`*^9, 
   3.5866579464169807`*^9, 3.5866597765457935`*^9, {3.586660745928494*^9, 
   3.5866607620755615`*^9}}]
}, Open  ]],

Cell["while this is not", "Text",
 CellChangeTimes->{{3.5050299245760183`*^9, 3.505029947391323*^9}, {
  3.5050301133448153`*^9, 3.505030114447878*^9}, {3.5051016501582823`*^9, 
  3.5051016764427853`*^9}, {3.5051095577515707`*^9, 3.505109559254657*^9}}],

Cell[BoxData[
 RowBox[{"HoldSubscripts", "[", "]"}]], "Input",
 CellChangeTimes->{{3.505029894959324*^9, 3.50502990065765*^9}, 
   3.5051016764427853`*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SubsuperscriptBox["\[Sum]", 
   RowBox[{"n", "=", "0"}], "5"], 
  SubscriptBox["x", "n"]}]], "Input",
 CellChangeTimes->{3.5051016764437857`*^9}],

Cell[BoxData[
 RowBox[{"6", " ", 
  SubscriptBox["x", "n"]}]], "Output",
 CellChangeTimes->{{3.5051045281558943`*^9, 3.5051045361553516`*^9}, {
   3.5051081459018173`*^9, 3.5051081714082766`*^9}, 3.505108296723444*^9, 
   3.505108355645814*^9, 3.505108425874831*^9, 3.505108668788725*^9, 
   3.505108729744211*^9, 3.5051089019240594`*^9, {3.505109385763734*^9, 
   3.505109401875655*^9}, 3.505109542421694*^9, 3.5051095994259543`*^9, 
   3.505109759342101*^9, 3.505109844169953*^9, 3.505109878230901*^9, {
   3.5051100086473603`*^9, 3.505110030532612*^9}, 3.505110240960648*^9, 
   3.505110366631836*^9, 3.50511097162844*^9, 3.505111106200137*^9, 
   3.505111302966391*^9, 3.5051132480976467`*^9, {3.5051133158045187`*^9, 
   3.505113321745859*^9}, 3.5051133572568903`*^9, 3.505113650404657*^9, 
   3.5051147508575993`*^9, {3.5051980112555656`*^9, 3.505198020672104*^9}, {
   3.505198533162417*^9, 3.5051985526495314`*^9}, 3.5707744207471004`*^9, 
   3.570777756804418*^9, 3.575947919165552*^9, 3.5866576362740793`*^9, 
   3.586657946441002*^9, 3.5866597765738153`*^9, {3.5866607459494934`*^9, 
   3.5866607620965214`*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Convenience functions", "Section",
 CellChangeTimes->{{3.5071809147377048`*^9, 3.507180935831911*^9}, 
   3.5071809947312803`*^9}],

Cell["\<\
These functions are convenient to execute by tailing them to expressions\
\>", "Text",
 CellChangeTimes->{{3.50718095538503*^9, 3.507180990345029*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Sin", "[", "x", "]"}], "//", 
  RowBox[{"DoSeries", "[", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "6"}], "}"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.5071810066279607`*^9, 3.5071810382357683`*^9}}],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"x", "-", 
   FractionBox[
    SuperscriptBox["x", "3"], "6"], "+", 
   FractionBox[
    SuperscriptBox["x", "5"], "120"], "+", 
   InterpretationBox[
    SuperscriptBox[
     RowBox[{"O", "[", "x", "]"}], "7"],
    SeriesData[$CellContext`x, 0, {}, 1, 7, 1],
    Editable->False]}],
  SeriesData[$CellContext`x, 0, {1, 0, 
    Rational[-1, 6], 0, 
    Rational[1, 120]}, 1, 7, 1],
  Editable->False]], "Output",
 CellChangeTimes->{{3.507181033755512*^9, 3.507181038999812*^9}, 
   3.5707744207830853`*^9, 3.5707777568424606`*^9, 3.5759479192815638`*^9, 
   3.5866597766268024`*^9, {3.5866607459864984`*^9, 3.586660762136526*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"a", " ", "x"}], "+", 
   RowBox[{"b", " ", 
    SuperscriptBox["x", "2"]}], "+", " ", 
   RowBox[{"a", " ", "y"}]}], " ", "//", 
  RowBox[{"DoCollect", "[", "a", "]"}]}]], "Input",
 CellChangeTimes->{{3.5884723905504055`*^9, 3.588472410963996*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"b", " ", 
   SuperscriptBox["x", "2"]}], "+", 
  RowBox[{"a", " ", 
   RowBox[{"(", 
    RowBox[{"x", "+", "y"}], ")"}]}]}]], "Output",
 CellChangeTimes->{3.5884724133022842`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  FractionBox[
   RowBox[{"1", "-", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{"-", "x"}]]}], "x"], "//", 
  RowBox[{"DoLimit", "[", 
   RowBox[{"x", "\[Rule]", "0"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.588472454046476*^9, 3.588472478847587*^9}}],

Cell[BoxData["1"], "Output",
 CellChangeTimes->{3.588472479932744*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{788, 867},
WindowMargins->{{Automatic, 400}, {Automatic, 34}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Report"}, "StandardReport.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3586667961-5218291"->{
  Cell[9465, 256, 214, 4, 40, "Print",
   CellTags->"Info3586667961-5218291"],
  Cell[15599, 404, 222, 4, 40, "Print",
   CellTags->"Info3586667961-5218291"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3586667961-5218291", 23641, 623}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 152, 2, 99, "Title"],
Cell[734, 26, 6970, 173, 901, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[7729, 203, 114, 1, 88, "Section"],
Cell[CellGroupData[{
Cell[7868, 208, 96, 1, 37, "Subsection"],
Cell[7967, 211, 361, 8, 30, "Text"],
Cell[CellGroupData[{
Cell[8353, 223, 219, 6, 40, "Input"],
Cell[8575, 231, 220, 4, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[8856, 242, 304, 4, 88, "Section"],
Cell[CellGroupData[{
Cell[9185, 250, 277, 4, 40, "Input"],
Cell[9465, 256, 214, 4, 40, "Print",
 CellTags->"Info3586667961-5218291"]
}, Open  ]],
Cell[9694, 263, 214, 3, 30, "Text"],
Cell[9911, 268, 156, 3, 40, "Input"],
Cell[CellGroupData[{
Cell[10092, 275, 141, 2, 40, "Input"],
Cell[10236, 279, 1031, 16, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[11304, 300, 99, 1, 37, "Subsection"],
Cell[11406, 303, 117, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[11548, 308, 245, 6, 40, "Input"],
Cell[11796, 316, 1166, 18, 40, "Output"]
}, Open  ]],
Cell[12977, 337, 352, 4, 30, "Text"],
Cell[13332, 343, 161, 3, 40, "Input"],
Cell[CellGroupData[{
Cell[13518, 350, 273, 7, 40, "Input"],
Cell[13794, 359, 1410, 28, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[15265, 394, 107, 1, 88, "Section"],
Cell[CellGroupData[{
Cell[15397, 399, 199, 3, 40, "Input"],
Cell[15599, 404, 222, 4, 40, "Print",
 CellTags->"Info3586667961-5218291"]
}, Open  ]],
Cell[15836, 411, 100, 1, 30, "Text"],
Cell[15939, 414, 161, 3, 40, "Input"],
Cell[CellGroupData[{
Cell[16125, 421, 141, 2, 40, "Input"],
Cell[16269, 425, 976, 14, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17282, 444, 143, 2, 37, "Subsection"],
Cell[17428, 448, 195, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[17648, 454, 223, 6, 48, "Input"],
Cell[17874, 462, 1287, 22, 40, "Output"]
}, Open  ]],
Cell[19176, 487, 253, 3, 30, "Text"],
Cell[19432, 492, 155, 3, 40, "Input"],
Cell[CellGroupData[{
Cell[19612, 499, 171, 5, 48, "Input"],
Cell[19786, 506, 1123, 17, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[20970, 530, 136, 2, 88, "Section"],
Cell[21109, 534, 161, 3, 30, "Text"],
Cell[CellGroupData[{
Cell[21295, 541, 244, 6, 40, "Input"],
Cell[21542, 549, 674, 18, 58, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22253, 572, 295, 8, 40, "Input"],
Cell[22551, 582, 214, 7, 40, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[22802, 594, 286, 8, 56, "Input"],
Cell[23091, 604, 70, 1, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
