(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[     31825,        778]
NotebookOptionsPosition[     29844,        708]
NotebookOutlinePosition[     30739,        739]
CellTagsIndexPosition[     30613,        733]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Package: Units", "Title",
 CellChangeTimes->{{3.5051195192313347`*^9, 3.505119523718591*^9}, {
  3.5884810399108953`*^9, 3.5884810429992123`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"BeginPackage", "[", "\"\<Lautrup`Units`\>\"", "]"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"Unprotect", "[", "\"\<`*\>\"", "]"}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"Acceptable", " ", "SI", " ", "units"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Capitalize", "[", "string_", "]"}], ":=", 
   RowBox[{"StringReplacePart", "[", 
    RowBox[{"string", ",", 
     RowBox[{"ToUpperCase", "[", 
      RowBox[{"StringTake", "[", 
       RowBox[{"string", ",", "1"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetAttributes", "[", 
    RowBox[{"Capitalize", ",", "Listable"}], "]"}], ";"}], "\n", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"define", " ", "accepted", " ", "unit", " ", "names"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"regular", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<meter\>\"", ",", "\"\<gram\>\"", ",", "\"\<second\>\"", ",", 
     "\"\<ampere\>\"", ",", "\"\<kelvin\>\"", ",", "\"\<mole\>\"", ",", 
     "\"\<candela\>\"", ",", "\"\<newton\>\"", ",", "\"\<pascal\>\"", ",", 
     "\"\<joule\>\"", ",", "\"\<watt\>\"", ",", "\"\<coulomb\>\"", ",", 
     "\"\<volt\>\"", ",", "\"\<farad\>\"", ",", "\"\<ohm\>\"", ",", 
     "\"\<weber\>\"", ",", "\"\<tesla\>\"", ",", "\"\<lumen\>\"", ",", 
     "\"\<becquerel\>\"", ",", "\"\<sievert\>\"", ",", "\"\<liter\>\""}], 
    "}"}]}], ";", 
  RowBox[{"irregular", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\"\<lux\>\"", ",", "\"\<lux\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\"\<hertz\>\"", ",", "\"\<hertz\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\"\<siemens\>\"", ",", "\"\<siemens\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\"\<grays\>\"", ",", "\"\<grays\>\""}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\"\<henry\>\"", ",", "\"\<henries\>\""}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"basenames", "=", 
    RowBox[{"Join", "[", 
     RowBox[{
      RowBox[{"Map", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{"#", ",", 
           RowBox[{"StringJoin", "[", 
            RowBox[{"#", ",", "\"\<s\>\""}], "]"}]}], "}"}], "&"}], ",", 
        "regular"}], "]"}], ",", "irregular"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "attach", " ", "prefixes", " ", "to", " ", "all", " ", "names", " ", "and",
     " ", "capitalize"}], "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"prefixes", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<yocto\>\"", ",", "\"\<zepto\>\"", ",", "\"\<atto\>\"", ",", 
     "\"\<femto\>\"", ",", "\"\<pico\>\"", ",", "\"\<nano\>\"", ",", 
     "\"\<micro\>\"", ",", "\"\<milli\>\"", ",", "\"\<centi\>\"", ",", 
     "\"\<deci\>\"", ",", "\"\<\>\"", ",", "\"\<deka\>\"", ",", 
     "\"\<hecto\>\"", ",", "\"\<kilo\>\"", ",", "\"\<mega\>\"", ",", 
     "\"\<giga\>\"", ",", "\"\<tera\>\"", ",", "\"\<peta\>\"", ",", 
     "\"\<exa\>\"", ",", "\"\<zetta\>\"", ",", "\"\<yotta\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"names", "=", 
    RowBox[{"Capitalize", "[", 
     RowBox[{"Flatten", "[", 
      RowBox[{
       RowBox[{"Outer", "[", 
        RowBox[{"StringJoin", ",", "prefixes", ",", "basenames"}], "]"}], ",",
        "1"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "remove", " ", "erroneous", " ", "or", " ", "slow", " ", "names", " ", 
    "in", " ", "Quantity"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"problems", "=", 
   RowBox[{"{", 
    RowBox[{
    "\"\<Yoctolumens\>\"", ",", "\"\<Hectomoles\>\"", ",", 
     "\"\<Yottahenries\>\"", ",", "\"\<Dekasieverts\>\"", ",", 
     "\"\<Dekalumens\>\"", ",", "\"\<Dekagrays\>\""}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "s_", "]"}], ":=", 
   RowBox[{"Apply", "[", 
    RowBox[{"Or", ",", 
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"s", "==", "#"}], "&"}], ",", "problems"}], "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"goodnames", "=", 
    RowBox[{"Select", "[", 
     RowBox[{"names", ",", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"f", "[", 
         RowBox[{"#", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "]"}]}], "&"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"End", "[", "]"}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"declare", " ", "all", " ", "prefixed", " ", "units"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Map", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Set", "[", 
       RowBox[{
        RowBox[{"Evaluate", "[", 
         RowBox[{"Symbol", "[", 
          RowBox[{"#", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "]"}], "]"}], ",", 
        RowBox[{"Quantity", "[", 
         RowBox[{"#", "[", 
          RowBox[{"[", "2", "]"}], "]"}], "]"}]}], "]"}], "&"}], ",", 
     "`Private`goodnames"}], "]"}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"Non", "-", 
    RowBox[{"SI", " ", "units"}]}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Angstrom", "=", 
   RowBox[{"Quantity", "[", "\"\<Angstroms\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Atmosphere", "=", 
   RowBox[{"Quantity", "[", "\"\<Atmospheres\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Bar", "=", 
   RowBox[{"Quantity", "[", "\"\<Bars\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Barn", "=", 
   RowBox[{"Quantity", "[", "\"\<Barns\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Celsius", "=", 
   RowBox[{"Quantity", "[", "\"\<DegreesCelsius\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Curie", "=", 
   RowBox[{"Quantity", "[", "\"\<Curies\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Day", "=", 
   RowBox[{"Quantity", "[", "\"\<Days\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Dyne", "=", 
   RowBox[{"Quantity", "[", "\"\<Dynes\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Electronvolt", "=", 
   RowBox[{"Quantity", "[", "\"\<Electronvolts\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Erg", "=", 
   RowBox[{"Quantity", "[", "\"\<Ergs\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Fahrenheit", "=", 
   RowBox[{"Quantity", "[", "\"\<DegreesFahrenheit\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Fermi", "=", 
   RowBox[{"Quantity", "[", "\"\<Fermis\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Foot", "=", 
   RowBox[{"Quantity", "[", "\"\<Feet\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Gauss", "=", 
   RowBox[{"Quantity", "[", "\"\<Gausses\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Gigayear", "=", 
   RowBox[{"Quantity", "[", "\"\<Gigayears\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Hectare", "=", 
   RowBox[{"Quantity", "[", "\"\<Hectares\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Hour", "=", 
   RowBox[{"Quantity", "[", "\"\<Hours\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Inch", "=", 
   RowBox[{"Quantity", "[", "\"\<Inches\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Kilobar", "=", 
   RowBox[{"Quantity", "[", "\"\<Kilobars\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Knot", "=", 
   RowBox[{"Quantity", "[", "\"\<Knots\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Maxwell", "=", 
   RowBox[{"Quantity", "[", "\"\<Maxwells\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Megabar", "=", 
   RowBox[{"Quantity", "[", "\"\<Megabars\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Megaparsec", "=", 
   RowBox[{"Quantity", "[", "\"\<Megaparsecs\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"MetricTon", "=", 
   RowBox[{"Quantity", "[", "\"\<MetricTons\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Mile", "=", 
   RowBox[{"Quantity", "[", "\"\<Miles\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Millibar", "=", 
   RowBox[{"Quantity", "[", "\"\<Millibars\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Minute", "=", 
   RowBox[{"Quantity", "[", "\"\<Minutes\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Oersted", "=", 
   RowBox[{"Quantity", "[", "\"\<Oersteds\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Parsec", "=", 
   RowBox[{"Quantity", "[", "\"\<Parsecs\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Percent", "=", 
   RowBox[{"Quantity", "[", "\"\<Percent\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Phot", "=", 
   RowBox[{"Quantity", "[", "\"\<Phots\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Poise", "=", 
   RowBox[{"Quantity", "[", "\"\<Poise\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Pound", "=", 
   RowBox[{"Quantity", "[", "\"\<Pounds\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Rad", "=", 
   RowBox[{"Quantity", "[", "\"\<Rads\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Rem", "=", 
   RowBox[{"Quantity", "[", "\"\<Rems\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Roentgen", "=", 
   RowBox[{"Quantity", "[", "\"\<Roentgens\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Stilb", "=", 
   RowBox[{"Quantity", "[", "\"\<Stilbs\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Stokes", "=", 
   RowBox[{"Quantity", "[", "\"\<Stokes\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"TonsOfTNT", "=", 
   RowBox[{"Quantity", "[", "\"\<TonsOfTNT\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Torr", "=", 
   RowBox[{"Quantity", "[", "\"\<Torr\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Year", "=", 
    RowBox[{"Quantity", "[", "\"\<Years\>\"", "]"}]}], ";"}], 
  "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"convenience", " ", "functions"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ToSI", "::", "\"\<usage\>\""}], "=", 
   "\"\<ToSI[q] converts a quantity q to fundamental SI units\>\""}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"ToUnit", "::", "\"\<usage\>\""}], "=", 
   "\"\<ToUnit[u][q] converts a quantity q to unit u\>\""}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"NoUnit", "::", "\"\<usage\>\""}], "=", 
    "\"\<NoUnit[q] removes the unit from a quantity q\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Begin", "[", "\"\<`Private`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ToSI", "[", "q_", "]"}], "=", 
   RowBox[{"UnitConvert", "[", "q", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ToUnit", "[", "u_", "]"}], "[", "q_", "]"}], "=", 
   RowBox[{"UnitConvert", "[", 
    RowBox[{"q", ",", "u"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"NoUnit", "[", "q_", "]"}], "=", 
   RowBox[{"QuantityMagnitude", "[", "q", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"End", "[", "]"}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{"clean", " ", "up"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Remove", "[", "\"\<`Private`*\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"SetAttributes", "[", 
  RowBox[{
   RowBox[{"Evaluate", "[", 
    RowBox[{"Names", "[", "\"\<`*\>\"", "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"Protected", ",", "ReadProtected"}], "}"}]}], 
  "]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"EndPackage", "[", "]"}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.39010152249358*^9, {3.3901015654805803`*^9, 3.39010161915958*^9}, {
   3.39010170832158*^9, 3.39010175050758*^9}, 3.3901018704945803`*^9, 
   3.39010277672758*^9, {3.3901029758965797`*^9, 3.39010298144958*^9}, {
   3.3901034768705797`*^9, 3.39010347963258*^9}, {3.394797956936*^9, 
   3.394798057884*^9}, {3.394798278824*^9, 3.394798313049*^9}, {
   3.394798444698*^9, 3.394798468559*^9}, {3.3947985093059998`*^9, 
   3.394798521179*^9}, {3.407833442978342*^9, 3.4078334614613423`*^9}, {
   3.407833653476342*^9, 3.4078336541143427`*^9}, {3.407833902386342*^9, 
   3.4078339087533426`*^9}, {3.4078343209953423`*^9, 
   3.4078343318933425`*^9}, {3.4078344584873424`*^9, 3.4078344600903425`*^9}, 
   3.407834812372342*^9, {3.4161884829351673`*^9, 3.416188505227167*^9}, 
   3.416188557151167*^9, {3.416188655460167*^9, 3.416188658088167*^9}, {
   3.416188965066167*^9, 3.416189029171167*^9}, {3.416189110061167*^9, 
   3.416189152861167*^9}, {3.416189257656167*^9, 3.416189259854167*^9}, {
   3.4161893125231667`*^9, 3.416189318212167*^9}, {3.416189365158167*^9, 
   3.416189427007167*^9}, {3.4161895171071672`*^9, 3.416189528821167*^9}, {
   3.416189981532167*^9, 3.416190036460167*^9}, {3.416190093974167*^9, 
   3.416190104941167*^9}, {3.416195208092167*^9, 3.4161952150491667`*^9}, {
   3.416195797768167*^9, 3.416195803057167*^9}, {3.4407360376032076`*^9, 
   3.440736072595207*^9}, {3.4407361912512074`*^9, 3.4407361920332074`*^9}, {
   3.4479972069288*^9, 3.4479972245978003`*^9}, {3.4479973416198*^9, 
   3.4479973427418003`*^9}, {3.479957134593*^9, 3.4799572229189997`*^9}, {
   3.479957327894*^9, 3.479957438552*^9}, {3.47995901967*^9, 
   3.479959026726*^9}, {3.480839757399793*^9, 3.4808397852723875`*^9}, {
   3.480839828799877*^9, 3.4808398315890365`*^9}, {3.481345934264535*^9, 
   3.481345939753849*^9}, {3.481346030088016*^9, 3.4813460542273965`*^9}, {
   3.505119587719252*^9, 3.5051196785714483`*^9}, {3.5051199984187427`*^9, 
   3.505120092897146*^9}, {3.505120154622677*^9, 3.5051201656683087`*^9}, {
   3.5051202646919727`*^9, 3.5051203734371924`*^9}, 3.505120418516771*^9, {
   3.50528441469585*^9, 3.505284414851859*^9}, {3.5052898780073338`*^9, 
   3.505289882162571*^9}, 3.505290111059664*^9, 3.505291120235385*^9, {
   3.5052912008759975`*^9, 3.5052912272205043`*^9}, {3.5747746325513706`*^9, 
   3.5747746431854753`*^9}, {3.574774687021719*^9, 3.5747746916408005`*^9}, {
   3.574774744356967*^9, 3.5747747518659787`*^9}, {3.574774814366681*^9, 
   3.574774837266967*^9}, 3.5747748677923355`*^9, {3.574775381358003*^9, 
   3.5747754148673573`*^9}, 3.5747757271507244`*^9, {3.5751904832411847`*^9, 
   3.5751904846483603`*^9}, {3.575191018700494*^9, 3.575191031790062*^9}, {
   3.575191066531483*^9, 3.5751910727462096`*^9}, {3.5751911432270555`*^9, 
   3.575191160054225*^9}, {3.57534057864165*^9, 3.5753406084083867`*^9}, {
   3.5753406481383753`*^9, 3.575340725728161*^9}, {3.5753411064339676`*^9, 
   3.5753411850828424`*^9}, {3.5753412706636467`*^9, 3.575341301276389*^9}, {
   3.5753413382310295`*^9, 3.5753413653454337`*^9}, {3.5753414853445244`*^9, 
   3.5753415813155556`*^9}, {3.5753416205445194`*^9, 3.57534166930162*^9}, {
   3.5753417722015624`*^9, 3.575341772725587*^9}, {3.5753418146979017`*^9, 
   3.575341815372977*^9}, {3.5753419167876754`*^9, 3.57534196380058*^9}, {
   3.5753420561611876`*^9, 3.5753420564732156`*^9}, {3.5753421398577356`*^9, 
   3.5753421780215187`*^9}, {3.5753422217200065`*^9, 
   3.5753422400732713`*^9}, {3.575342290357598*^9, 3.575342333989092*^9}, {
   3.575342487865385*^9, 3.5753426093346825`*^9}, 3.5753427156579895`*^9, {
   3.57534281562354*^9, 3.5753429041516566`*^9}, {3.5753429426284876`*^9, 
   3.5753430081537156`*^9}, {3.57534304188002*^9, 3.5753430846923585`*^9}, {
   3.575343152778925*^9, 3.5753433176015725`*^9}, {3.575343357743679*^9, 
   3.575343407861945*^9}, {3.5753438624370255`*^9, 3.5753438791151147`*^9}, 
   3.5753440504606476`*^9, {3.575344175071241*^9, 3.5753442160573883`*^9}, {
   3.5753442726924987`*^9, 3.575344358371259*^9}, {3.5753444723565702`*^9, 
   3.5753444998360605`*^9}, {3.575344584224646*^9, 3.5753446277080793`*^9}, {
   3.575344761324855*^9, 3.5753448528353686`*^9}, {3.5753449061090794`*^9, 
   3.5753449082773094`*^9}, {3.575344944362838*^9, 3.5753449615409946`*^9}, {
   3.5753451070173125`*^9, 3.5753451338036237`*^9}, {3.575345257146124*^9, 
   3.5753452945838118`*^9}, {3.575345368260105*^9, 3.57534538785056*^9}, {
   3.575347403928751*^9, 3.5753474236411524`*^9}, {3.575347566527112*^9, 
   3.5753475678552623`*^9}, {3.5753476703921394`*^9, 3.57534767377763*^9}, {
   3.5753479799990635`*^9, 3.5753480870294514`*^9}, {3.575348159510592*^9, 
   3.5753481771178017`*^9}, {3.575348214439459*^9, 3.5753482282482166`*^9}, {
   3.575348440006777*^9, 3.575348451573226*^9}, {3.575348730790325*^9, 
   3.5753487498166747`*^9}, {3.5753487813586397`*^9, 3.575348785039135*^9}, {
   3.5753488516704693`*^9, 3.575348861285694*^9}, {3.575348937638299*^9, 
   3.5753489503959074`*^9}, {3.575349122670535*^9, 3.575349143433146*^9}, {
   3.5753491849503527`*^9, 3.575349188685829*^9}, {3.5753492213759437`*^9, 
   3.575349287304205*^9}, {3.575349368231382*^9, 3.5753493731979947`*^9}, {
   3.5753495540936675`*^9, 3.575349560796522*^9}, 3.575349670694356*^9, {
   3.5753498284621167`*^9, 3.5753498918691163`*^9}, {3.5753499560931864`*^9, 
   3.5753499565181966`*^9}, {3.575350187764241*^9, 3.575350221939581*^9}, {
   3.5753503380771074`*^9, 3.575350404791542*^9}, {3.5753505165235558`*^9, 
   3.5753505234594526`*^9}, {3.5753505675359197`*^9, 
   3.5753505784763427`*^9}, {3.575350664598108*^9, 3.575350671116926*^9}, {
   3.5753508603027344`*^9, 3.5753508799071465`*^9}, {3.575350910047985*^9, 
   3.575350910215949*^9}, {3.575351035509733*^9, 3.575351062668149*^9}, {
   3.5753511332319703`*^9, 3.5753513129115543`*^9}, {3.5753513529725466`*^9, 
   3.5753513864927564`*^9}, 3.575351429925207*^9, {3.575351468556079*^9, 
   3.5753514869903717`*^9}, {3.5753515247871695`*^9, 
   3.5753516348049574`*^9}, {3.575351665829828*^9, 3.575351667536044*^9}, {
   3.5753517024304233`*^9, 3.575351751100588*^9}, {3.5753518668601255`*^9, 
   3.5753518773174376`*^9}, {3.5753519935540257`*^9, 
   3.5753520504601884`*^9}, {3.575352093083477*^9, 3.5753521608109818`*^9}, {
   3.575352227292328*^9, 3.57535225896234*^9}, {3.5753523033168783`*^9, 
   3.57535235581947*^9}, {3.575352410446328*^9, 3.575352424631163*^9}, {
   3.57535246214787*^9, 3.5753524962991347`*^9}, {3.5753525588260064`*^9, 
   3.5753526151550837`*^9}, {3.575352706636572*^9, 3.575352736186268*^9}, {
   3.575352779284642*^9, 3.5753528429866796`*^9}, {3.575352983519328*^9, 
   3.5753529967099447`*^9}, {3.5753652816636252`*^9, 
   3.5753652820856967`*^9}, {3.5753679300351763`*^9, 
   3.5753679585057054`*^9}, {3.5753682334602304`*^9, 
   3.5753682589364552`*^9}, {3.575368376009153*^9, 3.5753683928262606`*^9}, {
   3.57536851869709*^9, 3.5753685320967655`*^9}, {3.5753687414970655`*^9, 
   3.575368801271611*^9}, {3.5753688434648757`*^9, 3.575368860410996*^9}, {
   3.5753689098292127`*^9, 3.575368957058132*^9}, {3.5753690228543653`*^9, 
   3.575369202694934*^9}, {3.5753693258634543`*^9, 3.575369386058958*^9}, {
   3.5753694493809094`*^9, 3.575369454035494*^9}, {3.5753697580396824`*^9, 
   3.575369761447153*^9}, {3.5753708444590983`*^9, 3.575370850266856*^9}, {
   3.5753709045656767`*^9, 3.5753709361636443`*^9}, {3.5753756290833607`*^9, 
   3.575375644332297*^9}, {3.5753758246588945`*^9, 3.5753758270982447`*^9}, {
   3.5753763922582135`*^9, 3.5753764155151296`*^9}, 3.575376538082476*^9, {
   3.575376576629356*^9, 3.5753765817870064`*^9}, {3.5753772308884935`*^9, 
   3.5753772614653125`*^9}, {3.575425789079517*^9, 3.575425790590705*^9}, {
   3.5754342260827184`*^9, 3.57543427870103*^9}, {3.5754343879881434`*^9, 
   3.575434393800852*^9}, 3.5754344472912583`*^9, {3.5754344866239777`*^9, 
   3.5754344923626685`*^9}, 3.5754345235744123`*^9, 3.5754366311842957`*^9, {
   3.575436987102001*^9, 3.5754370004516015`*^9}, 3.5754371632641373`*^9, 
   3.5754387289299936`*^9, {3.5754398177736397`*^9, 3.5754398277008305`*^9}, {
   3.5754448612186613`*^9, 3.5754448961118484`*^9}, 3.575445358452324*^9, {
   3.5754454091464076`*^9, 3.575445428953783*^9}, 3.5754454870557537`*^9, 
   3.575445538482927*^9, {3.5754523340124593`*^9, 3.5754523426995025`*^9}, {
   3.575452381708182*^9, 3.5754524015975704`*^9}, {3.5754524382829723`*^9, 
   3.575452477164635*^9}, 3.575453967053401*^9, {3.575454277659671*^9, 
   3.5754544651801696`*^9}, {3.575454501249498*^9, 3.575454562288822*^9}, {
   3.5755313643014746`*^9, 3.5755313892914743`*^9}, {3.5755316823006306`*^9, 
   3.575531693527979*^9}, {3.5755335377702603`*^9, 3.5755335418637524`*^9}, {
   3.5755358145214386`*^9, 3.575535861125029*^9}, {3.5755358941429925`*^9, 
   3.575535916344656*^9}, 3.575535980408343*^9, {3.575536449062575*^9, 
   3.575536458243677*^9}, {3.5755368873951683`*^9, 3.575536897764412*^9}, {
   3.57553693425679*^9, 3.575536938212266*^9}, {3.5755370307893734`*^9, 
   3.5755370850588846`*^9}, 3.5755371308423796`*^9, {3.5755380191149583`*^9, 
   3.5755380541781807`*^9}, {3.5755387313374133`*^9, 
   3.5755387919936934`*^9}, {3.5755388421637106`*^9, 3.575538855228278*^9}, {
   3.575543216160781*^9, 3.575543239797619*^9}, {3.575543799043716*^9, 
   3.57554383516005*^9}, {3.5755450358101125`*^9, 3.5755450493707404`*^9}, {
   3.575545158041778*^9, 3.5755452005798817`*^9}, {3.5755453316256046`*^9, 
   3.5755453418528323`*^9}, {3.5755466512029366`*^9, 
   3.5755466623342714`*^9}, {3.575548242084819*^9, 3.5755482898255453`*^9}, {
   3.575602061890298*^9, 3.5756020823837566`*^9}, {3.5756021247058334`*^9, 
   3.5756021354371243`*^9}, 3.575602172471565*^9, {3.575602333745917*^9, 
   3.5756023408477674`*^9}, 3.5756024961264005`*^9, 3.575602541027788*^9, {
   3.575603202761186*^9, 3.5756032167228622`*^9}, {3.5756034946582108`*^9, 
   3.575603514456586*^9}, 3.575604415871743*^9, {3.5756101026180735`*^9, 
   3.5756101049193454`*^9}, {3.5756114127612686`*^9, 
   3.5756114263518977`*^9}, {3.5756117199421234`*^9, 
   3.5756117239006033`*^9}, {3.575612440264552*^9, 3.5756124580646896`*^9}, {
   3.575612724038601*^9, 3.5756127317485266`*^9}, 3.5756856955830975`*^9, {
   3.5756857576898956`*^9, 3.575685793195397*^9}, {3.575686079283311*^9, 
   3.575686081065534*^9}, {3.5756863096162367`*^9, 3.575686350674376*^9}, 
   3.5756864419678183`*^9, {3.57568653514856*^9, 3.575686536811759*^9}, 
   3.5756869376371193`*^9, {3.5756883693048797`*^9, 3.575688402259011*^9}, {
   3.5756884343800044`*^9, 3.575688469107401*^9}, {3.5764877085794554`*^9, 
   3.576487711318758*^9}, {3.588569726702454*^9, 3.5885697337734118`*^9}, 
   3.58857038156367*^9, {3.588570433534291*^9, 3.5885704356765575`*^9}, 
   3.5885706084075603`*^9, {3.5885706389743648`*^9, 3.5885706646516047`*^9}, {
   3.5885707262914667`*^9, 3.58857072990095*^9}, {3.5885708105481415`*^9, 
   3.588570812924493*^9}, {3.5885708496831293`*^9, 3.588570857084095*^9}, 
   3.5885709182618318`*^9, {3.5885709535943227`*^9, 3.58857101213879*^9}, {
   3.5885710595637903`*^9, 3.5885710608669786`*^9}, {3.588571276387369*^9, 
   3.5885712783776255`*^9}, 3.588571393843292*^9, {3.588571519901291*^9, 
   3.5885715258260503`*^9}, {3.588571565103992*^9, 3.588571578130686*^9}, {
   3.5885717932079678`*^9, 3.5885719282061763`*^9}, 3.5885720762289743`*^9, {
   3.588572129915782*^9, 3.5885721404781017`*^9}, {3.588572214023465*^9, 
   3.588572400819188*^9}, 3.588572648341652*^9, {3.5885728957620316`*^9, 
   3.5885729334918485`*^9}, {3.5885730350857725`*^9, 
   3.5885730444519625`*^9}, {3.588573355348472*^9, 3.5885733857023015`*^9}, 
   3.588573480684375*^9, 3.5885738982043724`*^9, {3.588574891398584*^9, 
   3.5885749269871006`*^9}, {3.58857499778209*^9, 3.5885750172205267`*^9}, {
   3.588575254173684*^9, 3.5885752935056615`*^9}, {3.58857561622162*^9, 
   3.588575622532471*^9}, 3.588575888488249*^9, {3.588575950408068*^9, 
   3.5885759553887014`*^9}, {3.588576027054807*^9, 3.5885760280009813`*^9}, {
   3.5885783321736746`*^9, 3.5885783365471883`*^9}, {3.588578376988325*^9, 
   3.5885783797467227`*^9}, {3.5885784323714037`*^9, 3.5885784360218253`*^9}, 
   3.588578477769126*^9, {3.5885785078799505`*^9, 3.5885785123695726`*^9}, {
   3.5885786152385893`*^9, 3.588578622026454*^9}}],

Cell["\<\
Converts Quantity units to explicit units\
\>", "Section",
 CellChangeTimes->{{3.505120487140696*^9, 3.5051204970122604`*^9}, {
  3.5756862805676003`*^9, 3.5756862981348543`*^9}, {3.588482381759256*^9, 
  3.5884823833744965`*^9}}],

Cell[CellGroupData[{

Cell["Unit conversion", "Section",
 CellChangeTimes->{{3.5051201845103865`*^9, 3.5051201873825507`*^9}, {
   3.5052899978161864`*^9, 3.5052899999023056`*^9}, 3.505290058989685*^9}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "ToSI"}]], "Input",
 CellChangeTimes->{{3.5051197575869675`*^9, 3.5051197585680237`*^9}}],

Cell[BoxData[
 StyleBox["\<\"ToSI[q] converts a quantity q to fundamental SI units\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5884823920745945`*^9},
 CellTags->"Info3588489591-6490567"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "ToUnit"}]], "Input",
 CellChangeTimes->{{3.5051197875646825`*^9, 3.5051197901278296`*^9}}],

Cell[BoxData[
 StyleBox["\<\"ToUnit[u][q] converts a quantity q to unit u\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5884810906792717`*^9},
 CellTags->"Info3588488290-1683493"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "NoUnit"}]], "Input",
 CellChangeTimes->{{3.505119820670576*^9, 3.5051198302401237`*^9}}],

Cell[BoxData[
 StyleBox["\<\"NoUnit[q] removes the unit from a quantity q\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellChangeTimes->{3.5884810908052635`*^9},
 CellTags->"Info3588488290-1683493"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsection",
 CellChangeTimes->{{3.505120750526761*^9, 3.505120754727001*^9}, {
  3.505120813735376*^9, 3.5051208150624523`*^9}}],

Cell["This converts 64 miles per hour to meters per second", "Text",
 CellDingbat->"\[FilledSmallSquare]"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"64.", " ", 
   RowBox[{"Mile", "/", "Hour"}]}], "//", "ToSI"}]], "Input",
 CellChangeTimes->{3.5051209692972736`*^9}],

Cell[BoxData[
 TemplateBox[{"28.610617221234442`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", 
          
          RowBox[{"\"m\"", "\[InvisibleSpace]", "\"/\"", "\[InvisibleSpace]", 
            "\"s\""}]}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: meters per second"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", 
       FractionBox["\"Meters\"", "\"Seconds\""]}], "]"}]& )]], "Output",
 CellChangeTimes->{3.505284451725968*^9, 3.5756858761548133`*^9, 
  3.5884810909133005`*^9}]
}, Open  ]],

Cell["Convert 64 miles per hour to SI units and drop the unit", "Text",
 CellDingbat->"\[FilledSmallSquare]"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"64.", " ", 
    RowBox[{"Mile", "/", "Hour"}]}], "//", "ToSI"}], "//", 
  "NoUnit"}]], "Input",
 CellChangeTimes->{{3.5052910935278573`*^9, 3.5052910997282124`*^9}, 
   3.575686245967249*^9}],

Cell[BoxData["28.610617221234442`"], "Output",
 CellChangeTimes->{3.5052844522079954`*^9, 3.5884810909683037`*^9}]
}, Open  ]],

Cell["Convert 64 miles per hour knots", "Text",
 CellDingbat->"\[FilledSmallSquare]",
 CellChangeTimes->{{3.575686199160384*^9, 3.5756862146553392`*^9}}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"64.", " ", 
   RowBox[{"Mile", "/", "Hour"}]}], "//", 
  RowBox[{"ToUnit", "[", "Knot", " ", "]"}]}]], "Input",
 CellChangeTimes->{{3.57568621955394*^9, 3.575686248487565*^9}}],

Cell[BoxData[
 TemplateBox[{"55.61459071082289`"},
  "QuantityUnit",
  DisplayFunction->(TooltipBox[
    StyleBox[
     RowBox[{#, "\[InvisibleSpace]", 
       StyleBox[
        RowBox[{" ", "\"knots\""}], "QuantityUnitTraditionalLabel"]}], 
     ShowStringCharacters -> False], "Unit: knots"]& ),
  InterpretationFunction->(RowBox[{"Quantity", "[", 
     RowBox[{#, ",", "\"Knots\""}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.575686239455434*^9, 3.5756862501187706`*^9}, 
   3.5884810910763187`*^9}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
ScreenStyleEnvironment->"Working",
WindowSize->{932, 735},
WindowMargins->{{Automatic, 273}, {Automatic, 16}},
ShowSelection->True,
FrontEndVersion->"9.0 for Microsoft Windows (64-bit) (January 25, 2013)",
StyleDefinitions->FrontEnd`FileName[{"Report"}, "StandardReport.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Info3588488290-1683493"->{
  Cell[26705, 603, 195, 4, 40, "Print",
   CellTags->"Info3588488290-1683493"],
  Cell[27057, 616, 195, 4, 40, "Print",
   CellTags->"Info3588488290-1683493"]},
 "Info3588489591-6490567"->{
  Cell[26342, 590, 204, 4, 40, "Print",
   CellTags->"Info3588489591-6490567"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Info3588488290-1683493", 30319, 722},
 {"Info3588489591-6490567", 30509, 727}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 152, 2, 99, "Title"],
Cell[734, 26, 25015, 543, 1801, "Input",
 InitializationCell->True],
Cell[25752, 571, 240, 5, 88, "Section"],
Cell[CellGroupData[{
Cell[26017, 580, 180, 2, 58, "Section"],
Cell[CellGroupData[{
Cell[26222, 586, 117, 2, 40, "Input"],
Cell[26342, 590, 204, 4, 40, "Print",
 CellTags->"Info3588489591-6490567"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26583, 599, 119, 2, 40, "Input"],
Cell[26705, 603, 195, 4, 40, "Print",
 CellTags->"Info3588488290-1683493"]
}, Open  ]],
Cell[CellGroupData[{
Cell[26937, 612, 117, 2, 40, "Input"],
Cell[27057, 616, 195, 4, 40, "Print",
 CellTags->"Info3588488290-1683493"]
}, Open  ]],
Cell[CellGroupData[{
Cell[27289, 625, 147, 2, 37, "Subsection"],
Cell[27439, 629, 106, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[27570, 634, 152, 4, 40, "Input"],
Cell[27725, 640, 652, 16, 40, "Output"]
}, Open  ]],
Cell[28392, 659, 109, 1, 30, "Text"],
Cell[CellGroupData[{
Cell[28526, 664, 238, 7, 40, "Input"],
Cell[28767, 673, 114, 1, 40, "Output"]
}, Open  ]],
Cell[28896, 677, 153, 2, 30, "Text"],
Cell[CellGroupData[{
Cell[29074, 683, 211, 5, 40, "Input"],
Cell[29288, 690, 504, 12, 40, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
