(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     41779,       1260]
NotebookOptionsPosition[     36742,       1107]
NotebookOutlinePosition[     37213,       1125]
CellTagsIndexPosition[     37170,       1122]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["\<\
 Chapter 19. Computational Fluid Dynamics\
\>", "Title",
 InitializationCell->True,
 CellChangeTimes->{
  3.4322089825922174`*^9, {3.5055409664353323`*^9, 3.5055409887456083`*^9}}],

Cell[BoxData[{
 RowBox[{"<<", "\"\<Lautrup`\>\""}], "\n", 
 RowBox[{
  RowBox[{"Off", "[", 
   RowBox[{"InterpolatingFunction", "::", "dmval"}], "]"}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.390123669809355*^9, 3.390123678824355*^9}, {
   3.431676807166*^9, 3.4316768157*^9}, 3.4317553707712126`*^9, {
   3.5055408978724103`*^9, 3.5055409042887774`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Finite elements to 2nd order", "Title",
 CellChangeTimes->{{3.432368900184414*^9, 3.4323689021684136`*^9}, {
  3.432369155139414*^9, 3.4323691579834137`*^9}}],

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.4323691409964137`*^9, 3.4323691438674135`*^9}}],

Cell[CellGroupData[{

Cell["Forward derivatives", "Section",
 CellChangeTimes->{{3.4323691633284135`*^9, 3.4323691680674133`*^9}, 
   3.4323700621354136`*^9}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fd", " ", "=", 
   RowBox[{
    RowBox[{"a0", "*", 
     RowBox[{"f", "[", "x", "]"}]}], "+", " ", 
    RowBox[{"a1", "*", 
     RowBox[{"f", "[", 
      RowBox[{"x", "+", "dx"}], "]"}]}], "+", 
    RowBox[{"a2", "*", 
     RowBox[{"f", "[", 
      RowBox[{"x", "+", 
       RowBox[{"2", "dx"}]}], "]"}]}], "+", 
    RowBox[{"a3", "*", 
     RowBox[{"f", "[", 
      RowBox[{"x", "+", 
       RowBox[{"3", "dx"}]}], "]"}]}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4323691708324137`*^9, 3.4323692417604136`*^9}, {
   3.4323694920364137`*^9, 3.4323694935604134`*^9}, {3.4323698839824133`*^9, 
   3.432369896514414*^9}, {3.4323700082064133`*^9, 3.4323700561744137`*^9}, {
   3.4323707612304134`*^9, 3.432370763889414*^9}, 3.4323707945034137`*^9}],

Cell[BoxData[
 RowBox[{"sd", "=", 
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{"fd", ",", 
     RowBox[{"{", 
      RowBox[{"dx", ",", "0", ",", "3"}], "}"}]}], "]"}], "//", 
   "Simplify"}]}]], "Input",
 CellChangeTimes->{{3.4323692299444137`*^9, 3.4323692912204137`*^9}, {
  3.4323694950824137`*^9, 3.4323695218904133`*^9}, {3.432369912514414*^9, 
  3.432369915764414*^9}, {3.432370641276414*^9, 3.4323706413734136`*^9}}],

Cell[CellGroupData[{

Cell["First order", "Subsection",
 CellChangeTimes->{{3.4323705433904133`*^9, 3.4323705447604136`*^9}}],

Cell[BoxData[
 RowBox[{"sol", "=", 
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a0", "+", "a1", "+", "a2", "+", "a3"}], "\[Equal]", "0"}], 
        ",", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"a1", "+", 
            RowBox[{"2", " ", "a2"}], "+", 
            RowBox[{"3", " ", "a3"}]}], ")"}], "dx"}], " ", "\[Equal]", "1"}],
         ",", 
        RowBox[{
         RowBox[{
          RowBox[{"(", 
           RowBox[{"a1", "+", 
            RowBox[{"4", " ", "a2"}], "+", 
            RowBox[{"9", " ", "a3"}]}], ")"}], 
          SuperscriptBox["dx", "2"]}], "\[Equal]", "0"}]}], "}"}], "/.", 
      RowBox[{"a3", "\[Rule]", "0"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"a0", ",", "a1", ",", "a2"}], "}"}]}], "]"}], "//", 
   "First"}]}]], "Input",
 CellChangeTimes->{{3.432369806922414*^9, 3.4323698471334133`*^9}, {
  3.4323699251854134`*^9, 3.4323699669064136`*^9}, {3.4323700305714135`*^9, 
  3.432370032080414*^9}, {3.4323704516544137`*^9, 3.4323704747724133`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"fd", "/.", 
    RowBox[{"a3", "\[Rule]", "0"}]}], "/.", "sol"}], "//", 
  "Simplify"}]], "Input",
 CellChangeTimes->{{3.4323698504754133`*^9, 3.4323698583974133`*^9}, {
  3.4323704827824135`*^9, 3.4323704891054134`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Second order", "Subsection",
 CellChangeTimes->{{3.4323705523104134`*^9, 3.4323705538444138`*^9}}],

Cell[BoxData[
 RowBox[{"sol", "=", 
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"a0", "+", "a1", "+", "a2", "+", "a3"}], "\[Equal]", "0"}], 
       ",", 
       RowBox[{
        RowBox[{"a1", "+", 
         RowBox[{"2", " ", "a2"}], "+", 
         RowBox[{"3", " ", "a3"}]}], " ", "\[Equal]", 
        RowBox[{"0", "/", "dx"}]}], ",", 
       RowBox[{
        RowBox[{"a1", "+", 
         RowBox[{"4", " ", "a2"}], "+", 
         RowBox[{"9", " ", "a3"}]}], "\[Equal]", 
        RowBox[{"1", "/", 
         SuperscriptBox["dx", "2"]}]}], ",", 
       RowBox[{
        RowBox[{"a1", "+", 
         RowBox[{"8", " ", "a2"}], "+", 
         RowBox[{"27", " ", "a3"}]}], "\[Equal]", 
        RowBox[{"0", "/", 
         SuperscriptBox["dx", "3"]}]}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"a0", ",", "a1", ",", "a2", ",", "a3"}], "}"}]}], "]"}], "//", 
   "First"}]}]], "Input",
 CellChangeTimes->{{3.432369806922414*^9, 3.4323698471334133`*^9}, {
  3.4323699251854134`*^9, 3.4323699669064136`*^9}, {3.4323700305714135`*^9, 
  3.432370032080414*^9}, {3.4323704516544137`*^9, 3.4323704747724133`*^9}, {
  3.4323705741484137`*^9, 3.4323705922984133`*^9}, {3.4323706892214136`*^9, 
  3.4323706921034136`*^9}, {3.4323708461924133`*^9, 3.4323708837534137`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fd", "/.", "sol"}], "//", "Simplify"}]], "Input",
 CellChangeTimes->{{3.4323706956884136`*^9, 3.4323707014964137`*^9}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Extrapolations to 2nd order", "Title",
 CellChangeTimes->{{3.432379037434*^9, 3.432379047799*^9}, {
  3.4323791111400003`*^9, 3.4323791317139997`*^9}}],

Cell[BoxData[
 RowBox[{"CleanGlobal", "[", "]"}]], "Input",
 CellChangeTimes->{{3.432379050862*^9, 3.432379055252*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"fd", " ", "=", " ", 
   RowBox[{
    RowBox[{"a1", "*", 
     RowBox[{"f", "[", 
      RowBox[{"x", "+", "dx"}], "]"}]}], "+", 
    RowBox[{"a2", "*", 
     RowBox[{"f", "[", 
      RowBox[{"x", "+", 
       RowBox[{"2", "dx"}]}], "]"}]}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4323691708324137`*^9, 3.4323692417604136`*^9}, {
   3.4323694920364137`*^9, 3.4323694935604134`*^9}, {3.4323698839824133`*^9, 
   3.432369896514414*^9}, {3.4323700082064133`*^9, 3.4323700561744137`*^9}, {
   3.4323707612304134`*^9, 3.432370763889414*^9}, 3.4323707945034137`*^9, {
   3.4323791496949997`*^9, 3.432379160182*^9}, 3.4323792103570004`*^9, {
   3.432379267527*^9, 3.4323792807939997`*^9}, 3.432379393359*^9}],

Cell[BoxData[
 RowBox[{"sd", " ", "=", " ", 
  RowBox[{
   RowBox[{"Series", "[", 
    RowBox[{"fd", ",", 
     RowBox[{"{", 
      RowBox[{"dx", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
   "Simplify"}]}]], "Input",
 CellChangeTimes->{{3.4323792051940002`*^9, 3.432379236724*^9}, {
  3.432379274987*^9, 3.432379320146*^9}, {3.432379381125*^9, 
  3.4323793812469997`*^9}}],

Cell[BoxData[
 RowBox[{"sol", "=", 
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"a1", "+", "a2"}], "\[Equal]", "1"}], ",", 
       RowBox[{
        RowBox[{"a1", "+", 
         RowBox[{"2", " ", "a2"}]}], "\[Equal]", "0"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"a1", ",", "a2"}], "}"}]}], "]"}], "//", "First"}]}]], "Input",
 CellChangeTimes->{{3.432379328835*^9, 3.4323793716219997`*^9}, {
  3.432379402895*^9, 3.432379415054*^9}, {3.432380013557*^9, 
  3.432380031606*^9}}],

Cell[BoxData[
 RowBox[{"fd", "/.", "sol"}]], "Input",
 CellChangeTimes->{{3.432380004953*^9, 3.432380010874*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "x_", "]"}], " ", "=", " ", 
   RowBox[{"g", "[", 
    RowBox[{"x", "-", 
     RowBox[{"dx", "/", "2"}]}], "]"}]}], ";"}]], "Input",
 CellChangeTimes->{{3.432380042273*^9, 3.432380094855*^9}}],

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"3", " ", 
     RowBox[{"f", "[", 
      RowBox[{"x", "+", 
       RowBox[{"2", "dx"}]}], "]"}]}], "-", 
    RowBox[{"2", 
     RowBox[{"f", "[", 
      RowBox[{"x", "+", "dx"}], "]"}]}], "-", 
    RowBox[{"f", "[", 
     RowBox[{"x", "+", 
      RowBox[{"3", "dx"}]}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"dx", ",", "0", ",", "3"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.432379890304*^9, 3.432379991094*^9}, {3.432380121939*^9,
   3.432380128626*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simulation", "Title"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CleanGlobal", "[", "]"}], ";"}]], "Input"],

Cell[CellGroupData[{

Cell["Read simulation data", "Section"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"input", " ", "=", " ", 
   RowBox[{"OpenRead", "[", "\"\<channel.data\>\"", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Read", "[", "input", "]"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{"initial", " ", "data"}], " ", "*)"}], ";"}]}], "Input",
 CellChangeTimes->{{3.4317616253352127`*^9, 3.4317616305252123`*^9}, 
   3.4320082912776117`*^9}],

Cell["Length along flow (x)", "Text"],

Cell[BoxData["a"], "Input"],

Cell["Distance between plates (y)", "Text"],

Cell[BoxData["L"], "Input",
 CellChangeTimes->{{3.432179933412716*^9, 3.4321799340547156`*^9}}],

Cell["Estimated entrance length", "Text",
 CellChangeTimes->{{3.4319646390308247`*^9, 3.4319646433538246`*^9}}],

Cell[BoxData["LE"], "Input",
 CellChangeTimes->{{3.431964652318825*^9, 3.431964652760825*^9}}],

Cell["Number of lattice points along x", "Text"],

Cell[BoxData["NX"], "Input"],

Cell["Number of lattice points along y", "Text"],

Cell[BoxData["NY"], "Input"],

Cell["Kinematic viscosity", "Text"],

Cell[BoxData["nu"], "Input"],

Cell["Reynolds number", "Text"],

Cell[BoxData["Reynolds"], "Input"],

Cell["Entrance plug velocity", "Text"],

Cell[BoxData["u0"], "Input"],

Cell["Convergence parameters", "Text"],

Cell[BoxData["omega"], "Input"],

Cell[BoxData["omegajacobi"], "Input",
 CellChangeTimes->{{3.4325218177876115`*^9, 3.432521819919612*^9}}],

Cell["Obtained accuracy", "Text"],

Cell[BoxData["accuracy"], "Input"],

Cell[BoxData["dx"], "Input",
 CellChangeTimes->{{3.4317691022522125`*^9, 3.4317691103642125`*^9}, 
   3.431788640425*^9, 3.432010681917611*^9}],

Cell[BoxData["dy"], "Input",
 CellChangeTimes->{{3.4317691303112125`*^9, 3.431769137732212*^9}, 
   3.431788644266*^9, 3.4320106849066114`*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualize an iteration", "Section"],

Cell["Read next iteration data", "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ReadList", "[", "input", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Close", "[", "input", "]"}], ";"}]}], "Input",
 CellChangeTimes->{
  3.4320085285736113`*^9, 3.4320091087326117`*^9, {3.4320092823246117`*^9, 
   3.4320092828406115`*^9}, {3.432180636780716*^9, 3.4321806433007154`*^9}, {
   3.4322195743892174`*^9, 3.4322195768352175`*^9}}],

Cell["Number of iterations until this point", "Text"],

Cell[BoxData["iter"], "Input"],

Cell["Precision obtained", "Text"],

Cell[BoxData["precision"], "Input"],

Cell["Value of divergence", "Text"],

Cell[BoxData["divergence"], "Input"],

Cell["\"Real time\"", "Text"],

Cell[BoxData["t"], "Input"],

Cell[BoxData["dt"], "Input"],

Cell["Make interpolations of all the data and plot it", "Text"],

Cell[BoxData[
 RowBox[{"x", "//", "Length"}]], "Input",
 CellChangeTimes->{{3.432543481383*^9, 3.4325435001210003`*^9}}],

Cell[BoxData[
 RowBox[{"vx", "//", "Length"}]], "Input",
 CellChangeTimes->{{3.432543503231*^9, 3.432543509352*^9}}],

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"ListInterpolation", ",", 
   RowBox[{"InterpolationOrder", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4320116704046116`*^9, 3.4320116873876114`*^9}, {
  3.432011805830611*^9, 3.4320118060906115`*^9}, {3.4320185769826117`*^9, 
  3.4320185772216115`*^9}, {3.4320220393786116`*^9, 3.4320220395076113`*^9}, {
  3.4321971672067156`*^9, 3.432197167334716*^9}, {3.4321973204707155`*^9, 
  3.4321973226927156`*^9}, {3.4322187448342175`*^9, 3.4322187450592175`*^9}, {
  3.4322195859272175`*^9, 3.432219586015218*^9}, {3.4323642055674133`*^9, 
  3.432364205677414*^9}, {3.432396108191*^9, 3.432396108277*^9}, {
  3.4324674107282267`*^9, 3.4324674108492265`*^9}, {3.432468447083226*^9, 
  3.4324684471862264`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"vxi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"vx", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"vyi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"vy", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fxi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"fx", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"fyi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"fy", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"pi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"p", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"hi", " ", "=", " ", 
   RowBox[{"ListInterpolation", "[", 
    RowBox[{"h", ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "y"}], "}"}]}], "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.4317692052882123`*^9, 3.4317692172732124`*^9}, {
  3.4317889601730003`*^9, 3.4317889635369997`*^9}, {3.4320189995486116`*^9, 
  3.4320190549666114`*^9}, {3.432025172089611*^9, 3.4320251813616114`*^9}, {
  3.4320252297506113`*^9, 3.4320252326406116`*^9}, {3.432025284837611*^9, 
  3.4320252858076115`*^9}, {3.432532082171574*^9, 3.432532099634574*^9}, {
  3.4325330124065742`*^9, 3.432533015045574*^9}, {3.432543346908*^9, 
  3.4325433761210003`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Plot3D", ",", 
    RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
    RowBox[{"PlotPoints", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"NX", "\[LessEqual]", "50"}], ",", "NX", ",", "50"}], "]"}], 
       ",", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"NY", "\[LessEqual]", "50"}], ",", "NY", ",", "50"}], 
        "]"}]}], "}"}]}]}], "]"}], ";"}]], "Input"],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"vxi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "a"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4317652423852124`*^9, 3.4317652426172123`*^9}, {
   3.4317653401472125`*^9, 3.4317653405092125`*^9}, {3.4317696488652124`*^9, 
   3.4317696489882126`*^9}, {3.4317883572349997`*^9, 3.431788381615*^9}, {
   3.431788636432*^9, 3.4317886375620003`*^9}, {3.4320219631996117`*^9, 
   3.4320219643316116`*^9}, {3.4321799640537157`*^9, 3.4321799685977154`*^9}, 
   3.432479007156*^9, 3.432532147125574*^9, 3.432533853948574*^9, 
   3.432543379145*^9}],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"vyi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "a"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.432179971077716*^9, 3.4321799740697155`*^9}, 
   3.432479011082*^9, 3.432543380217*^9}],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"fxi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "a"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4321799768057156`*^9, 3.4321799811727157`*^9}, 
   3.432479014809*^9, 3.4325433813129997`*^9}],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"fyi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "a"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4321799842297153`*^9, 3.4321799875407157`*^9}, 
   3.432479017139*^9, 3.4325433821689997`*^9}],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"pi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "a"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4321799902297153`*^9, 3.432179996004716*^9}, 
   3.432479021407*^9, 3.432543382993*^9}],

Cell[BoxData[
 RowBox[{"Plot3D", "[", 
  RowBox[{
   RowBox[{"hi", "[", 
    RowBox[{"x1", ",", "y1"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "L"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"y1", ",", "0", ",", "a"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4321799990937157`*^9, 3.4321800020057154`*^9}, 
   3.4324790253900003`*^9, {3.432532103128574*^9, 3.432532106104574*^9}, {
   3.4325329950695744`*^9, 3.432532995429574*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Mid channel flow", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"vxi", "[", 
       RowBox[{"x1", ",", "0"}], "]"}], ",", 
      RowBox[{"1.5", "*", "u0"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x1", ",", "0", ",", "L"}], "}"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<x\>\"", ",", "\"\<\!\(\*SubscriptBox[\(v\), \(x\)]\)\>\""}], 
      "}"}]}], ",", 
    RowBox[{"PlotStyle", "->", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Dashing", "[", 
        RowBox[{"{", "}"}], "]"}], ",", 
       RowBox[{"Dashing", "[", 
        RowBox[{"{", 
         RowBox[{"0.02", ",", "0.02"}], "}"}], "]"}]}], "}"}]}], ",", 
    RowBox[{"ImageSize", "\[Rule]", "200"}]}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", "\"\<cfd-centralxflow\>\"", "]"}]}]], "Input",
 CellChangeTimes->{
  3.3901237090333548`*^9, 3.431754880067212*^9, {3.431755004831212*^9, 
   3.4317550262902126`*^9}, {3.4317550610822124`*^9, 
   3.4317550642072124`*^9}, {3.4317565590562124`*^9, 3.431756563961212*^9}, {
   3.4317903913599997`*^9, 3.4317904053789997`*^9}, {3.4319634014808245`*^9, 
   3.431963425415825*^9}, {3.432180022584716*^9, 3.4321800241517153`*^9}, {
   3.43239625809*^9, 3.4323962597349997`*^9}, {3.432396295644*^9, 
   3.4323962992060003`*^9}}],

Cell["Find 99% point and normalize", "Text",
 CellChangeTimes->{{3.4317545898422127`*^9, 3.4317545902172127`*^9}}],

Cell[BoxData[
 RowBox[{"xE", "=", 
  RowBox[{"x1", "/.", 
   RowBox[{"FindRoot", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"vxi", "[", 
       RowBox[{"x1", ",", "0"}], "]"}], "==", 
      RowBox[{"0.99", 
       RowBox[{"vxi", "[", 
        RowBox[{"L", ",", "0"}], "]"}]}]}], ",", 
     RowBox[{"{", 
      RowBox[{"x1", ",", "0", ",", "0.1"}], "}"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.431790435084*^9, 3.431790504051*^9}, 
   3.4317926336949997`*^9, {3.4319634637138247`*^9, 3.431963484906825*^9}, {
   3.432180029254716*^9, 3.4321800342457156`*^9}, {3.432466448630226*^9, 
   3.4324664750992265`*^9}, {3.4324676565862265`*^9, 3.4324676574522266`*^9}, 
   3.4324679846992264`*^9, {3.432468035090226*^9, 3.4324680393912263`*^9}}],

Cell[BoxData[
 RowBox[{"k", "=", 
  FractionBox["xE", 
   RowBox[{"Reynolds", "*", "2", "a"}]]}]], "Input",
 CellChangeTimes->{{3.432468092262226*^9, 3.4324681108882265`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Entrance flow", "Section"],

Cell[BoxData[
 RowBox[{"poiseuille", " ", "=", " ", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"1.5", "u0", "  ", 
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"y1", "*", 
        RowBox[{"y1", "/", 
         RowBox[{"(", 
          RowBox[{"a", "*", "a"}], ")"}]}]}]}], ")"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"y1", ",", 
      RowBox[{"-", "a"}], ",", "a"}], "}"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"Dashing", "[", 
       RowBox[{"{", 
        RowBox[{"0.01", ",", "0.01"}], "}"}], "]"}], "}"}]}]}], 
   "]"}]}]], "Input",
 CellChangeTimes->{
  3.3901237207833548`*^9, {3.431963515488825*^9, 3.431963580609825*^9}, {
   3.431963809140825*^9, 3.4319638102948246`*^9}, {3.4321800411427155`*^9, 
   3.4321800522627153`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"start", " ", "=", " ", 
   RowBox[{"Graphics", "[", 
    RowBox[{"{", 
     RowBox[{"Dashed", ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "0.5"}], ",", "0"}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "0.5"}], ",", "1."}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0.5", ",", "1."}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0.5", ",", "0"}], "}"}]}], "}"}], "]"}]}], "}"}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.4320057160136113`*^9, 3.4320058231856117`*^9}, 
   3.4320075198766117`*^9}],

Cell[BoxData[
 RowBox[{"flowdata", "=", 
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"Evaluate", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"vxi", "[", 
        RowBox[{"x1", ",", 
         RowBox[{"Abs", "[", "y1", "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"x1", ",", "dx", ",", "LE", ",", 
         FractionBox["LE", "10"]}], "}"}]}], "]"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"y1", ",", 
      RowBox[{"-", "a"}], ",", "a"}], "}"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "a"}], ",", "a"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "All"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4317563179432125`*^9, 3.4317563296342125`*^9}, {
   3.4317887441610003`*^9, 3.431788744833*^9}, {3.4319637564788246`*^9, 
   3.431963792265825*^9}, {3.431963845791825*^9, 3.431963845991825*^9}, {
   3.4319646671948247`*^9, 3.431964674257825*^9}, {3.4321800693517156`*^9, 
   3.432180074919716*^9}, {3.4322196503152175`*^9, 3.432219660895218*^9}, {
   3.4322197036362176`*^9, 3.4322197087342176`*^9}, 3.4322198786272173`*^9, {
   3.4322952777360125`*^9, 3.432295282381013*^9}, {3.43239605526*^9, 
   3.432396063163*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"poiseuille", ",", "flowdata", ",", "start", ",", 
    RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<y\>\"", ",", "\"\<\!\(\*SubscriptBox[\(v\), \(x\)]\)\>\""}], 
      "}"}]}]}], "]"}], "//", "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", "\"\<cfd-flowprofiles\>\"", "]"}]}]], "Input",
 CellChangeTimes->{
  3.3901237307733545`*^9, {3.4317553207502127`*^9, 3.431755327843212*^9}, {
   3.4320058099056115`*^9, 3.4320058109936113`*^9}, {3.4320058479706116`*^9, 
   3.4320058732356114`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fig. 19.6a Pressure at entrance", "Section"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"pi", "[", 
       RowBox[{"x1", ",", "0"}], "]"}], ",", 
      RowBox[{"3", "nu", " ", "u0", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"L", "-", "x1"}], ")"}], "/", 
        RowBox[{"a", "^", "2"}]}]}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x1", ",", "0", ",", "L"}], "}"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<x\>\"", ",", "\"\<p\>\""}], "}"}]}], ",", 
    RowBox[{"ImageSize", "\[Rule]", "200"}]}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", "\"\<cfd-centralpressure\>\"", "]"}]}]], "Input",
 CellChangeTimes->{
  3.390123738095355*^9, {3.4317549012212124`*^9, 3.431754901416212*^9}, {
   3.431755454185212*^9, 3.4317554585362124`*^9}, {3.431755489376212*^9, 
   3.4317554903212123`*^9}, {3.4317562494372125`*^9, 
   3.4317562533922124`*^9}, {3.4319635973128247`*^9, 
   3.4319636287358246`*^9}, {3.4321800827597156`*^9, 
   3.4321800903927155`*^9}, {3.4323962699560003`*^9, 3.432396286736*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Gi", "[", "x1_", "]"}], "=", 
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["\[PartialD]", "x1"], 
     RowBox[{"pi", "[", 
      RowBox[{"x1", ",", "0"}], "]"}]}]}]}], ";"}]], "Input",
 CellChangeTimes->{{3.4320095679636116`*^9, 3.4320096011666117`*^9}, {
   3.432009631700611*^9, 3.4320096417136116`*^9}, 3.4320112520126114`*^9, {
   3.4320249818366117`*^9, 3.4320250088606114`*^9}}],

Cell[BoxData[
 RowBox[{"Gpois", " ", "=", " ", 
  RowBox[{"3", "nu", " ", 
   RowBox[{"u0", "/", 
    RowBox[{"a", "^", "2"}]}]}]}]], "Input",
 CellChangeTimes->{{3.4320111447756114`*^9, 3.4320111904596114`*^9}, {
  3.4321800961967154`*^9, 3.4321800962927155`*^9}}],

Cell[BoxData[
 RowBox[{"FindRoot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Gi", "[", "x1", "]"}], "\[Equal]", 
    RowBox[{"1.01", "  ", 
     RowBox[{"Gi", "[", 
      RowBox[{"L", "-", 
       RowBox[{"2", " ", "dx"}]}], "]"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "5", ",", "5", ",", "7"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4321972743547153`*^9, 3.4321972821817155`*^9}, {
  3.4322147684912176`*^9, 3.432214768631218*^9}, {3.432396090505*^9, 
  3.432396096413*^9}}],

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Gi", "[", "x1", "]"}], ",", "Gpois"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "0", ",", "L"}], "}"}], ",", 
   RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
   RowBox[{"PlotStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", "}"}], ",", "Dashed"}], "}"}]}]}], "]"}]], "Input",
 CellChangeTimes->{{3.4320096447356114`*^9, 3.4320096847116117`*^9}, {
   3.4320103220786114`*^9, 3.4320103225576115`*^9}, {3.4320111825796113`*^9, 
   3.432011212042611*^9}, 3.4320112564086113`*^9, {3.4320225422836113`*^9, 
   3.4320225865416117`*^9}, {3.4321801043897157`*^9, 3.432180105030716*^9}, {
   3.4321972030477157`*^9, 3.4321972166787157`*^9}, {3.4324684289032264`*^9, 
   3.432468429399226*^9}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Entrance length", "Title"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CleanGlobal", "[", "]"}], ";"}]], "Input"],

Cell["\<\
Simulation data on entrance length for different Reynolds numbers\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"lx", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0.001", ",", "0.6307"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.01", ",", "0.6308"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.1", ",", "0.6316"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.3", ",", "0.6310"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0.6412"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "0.6671"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"10", ",", "0.8054"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"30", ",", "1.537"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"100", ",", "4.4892"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"300", ",", "12.5059"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1000", ",", "40.929"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2000", ",", "81.4475"}], "}"}]}], "}"}]}], ";"}]], "Input",
 CellChangeTimes->{
  3.432534800136574*^9, {3.432536458442574*^9, 3.432536466464574*^9}}],

Cell[BoxData[
 RowBox[{"figa", "=", 
  RowBox[{"ListLogLogPlot", "[", 
   RowBox[{"lx", ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"PointSize", "[", "0.02", "]"}]}]}], "]"}]}]], "Input",
 CellChangeTimes->{{3.4325350949065742`*^9, 3.432535106035574*^9}, {
  3.4325353977425737`*^9, 3.432535417618574*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"lxi", "=", 
   RowBox[{"Interpolation", "[", 
    RowBox[{"lx", ",", 
     RowBox[{"InterpolationOrder", "\[Rule]", "1"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.431767457263212*^9, 3.4317674702522125`*^9}, {
  3.432534897264574*^9, 3.432534904935574*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "r_", "]"}], "=", 
   SqrtBox[
    RowBox[{
     SuperscriptBox["a", "2"], "+", 
     RowBox[{
      SuperscriptBox["k", "2"], 
      SuperscriptBox["r", "2"]}]}]]}], ";"}]], "Input",
 CellChangeTimes->{{3.432483029666*^9, 3.432483114977*^9}, {3.432483164277*^9,
   3.4324831793599997`*^9}}],

Cell[BoxData[
 RowBox[{"fit", "=", 
  RowBox[{"FindFit", "[", 
   RowBox[{"lx", ",", 
    RowBox[{"f", "[", "r", "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"a", ",", "k"}], "}"}], ",", "r"}], "]"}]}]], "Input",
 CellChangeTimes->{{3.432482986144*^9, 3.432482999116*^9}, {3.432483184256*^9,
   3.432483202145*^9}, {3.4324832324820004`*^9, 3.4324832336400003`*^9}}],

Cell[BoxData[
 RowBox[{"figb", "=", 
  RowBox[{"LogLogPlot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "x", "]"}], "/.", "fit"}], ",", 
      RowBox[{"0.04", "x"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0.05", ",", "2500"}], "}"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0.05", ",", "2500"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0.1", ",", "200"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "}"}], ",", "Dashed"}], "}"}]}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<Re\>\"", ",", "\"\<\!\(\*SubscriptBox[\(L\), \(e\)]\)\>\""}], 
      "}"}]}], ",", 
    RowBox[{"ImageSize", "\[Rule]", "250"}]}], "]"}]}]], "Input",
 CellChangeTimes->{
  3.3901237965873547`*^9, {3.431754750824212*^9, 3.4317548027622128`*^9}, {
   3.4317559412392125`*^9, 3.4317559456752124`*^9}, {3.4317560745162125`*^9, 
   3.4317560966832123`*^9}, {3.4317561999392123`*^9, 
   3.4317562078972125`*^9}, {3.4317668553422127`*^9, 
   3.4317668579462123`*^9}, {3.431767293987212*^9, 3.4317673228472123`*^9}, {
   3.432483258817*^9, 3.4324832999560003`*^9}, {3.43248333195*^9, 
   3.4324834908640003`*^9}, 3.4324835211610003`*^9, {3.432534926825574*^9, 
   3.432534985653574*^9}, {3.432535029163574*^9, 3.432535061232574*^9}, {
   3.432535124693574*^9, 3.432535125868574*^9}, 3.432535167356574*^9, {
   3.432535203041574*^9, 3.432535260949574*^9}, {3.432535434472574*^9, 
   3.4325354389475737`*^9}, {3.4325354748335743`*^9, 3.432535479545574*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"Show", "[", 
   RowBox[{"figb", ",", "figa"}], "]"}], "//", "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", "\"\<cfd-entrylength\>\"", "]"}]}]], "Input",
 CellChangeTimes->{{3.432535117379574*^9, 3.432535121594574*^9}, {
   3.432535184464574*^9, 3.432535188943574*^9}, 3.432535246292574*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Velocity rise with time", "Title"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CleanGlobal", "[", "]"}], ";"}]], "Input"],

Cell[TextData[{
 "Simulation data on ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "exit", " ", "velocity", " ", "as", " ", "a", " ", "function", " ", "of", 
    " ", "time"}], TraditionalForm]]],
 " for Rey = 100, Ny = 20; Nx = 40;"
}], "Text",
 CellChangeTimes->{{3.432460275445208*^9, 3.432460300114607*^9}, {
  3.432535697802574*^9, 3.432535699681574*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"timeseq", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0.000000", ",", "1.000000"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1.237798", ",", "1.316029"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"2.475595", ",", "1.425386"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3.713393", ",", "1.471314"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4.951191", ",", "1.488469"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"6.188989", ",", "1.494852"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"7.426786", ",", "1.497032"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"8.664584", ",", "1.497673"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"9.902382", ",", "1.497768"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"11.140180", ",", "1.497763"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"12.377977", ",", "1.497816"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"13.615775", ",", "1.497867"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"14.853573", ",", "1.497875"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"16.091371", ",", "1.497860"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"17.329168", ",", "1.497844"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"18.566966", ",", "1.497841"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"19.804764", ",", "1.497845"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"21.042561", ",", "1.497851"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"22.280359", ",", "1.497852"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"23.518157", ",", "1.497850"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"24.755955", ",", "1.497849"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"25.993752", ",", "1.497848"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"27.231550", ",", "1.497849"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"28.469348", ",", "1.497849"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"29.707146", ",", "1.497849"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"30.944943", ",", "1.497849"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"32.182741", ",", "1.497849"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"33.420539", ",", "1.497849"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"34.658337", ",", "1.497849"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"35.896134", ",", "1.497849"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"37.133932", ",", "1.497849"}], "}"}]}], "}"}]}], 
  ";"}]], "Input"],

Cell["Make an interpolation", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"vxi", "=", 
   RowBox[{"Interpolation", "[", 
    RowBox[{"timeseq", ",", 
     RowBox[{"InterpolationOrder", "\[Rule]", "3"}]}], "]"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.4317555532262125`*^9, 3.4317555533302126`*^9}, {
  3.432460104372902*^9, 3.4324601058683505`*^9}, {3.4325356591495743`*^9, 
  3.432535659276574*^9}}],

Cell["Find 99% point", "Text",
 CellChangeTimes->{{3.4317555448442125`*^9, 3.431755545197212*^9}}],

Cell[BoxData[
 RowBox[{"FindRoot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"vxi", "[", "t", "]"}], "==", 
    RowBox[{"0.99", 
     RowBox[{"vxi", "[", "10", "]"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"t", ",", "0.5"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.431754701067212*^9, 3.4317547014552126`*^9}, {
  3.431755594362212*^9, 3.4317555965922127`*^9}}],

Cell["Plot the field", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Plot", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"vxi", "[", "t", "]"}], ",", "1.5"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", "10"}], "}"}], ",", 
    RowBox[{"PlotRange", "\[Rule]", "All"}], ",", 
    RowBox[{"PlotStyle", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Dashing", "[", 
        RowBox[{"{", "}"}], "]"}], ",", 
       RowBox[{"Dashing", "[", 
        RowBox[{"{", 
         RowBox[{"0.02", ",", "0.02"}], "}"}], "]"}]}], "}"}]}], ",", 
    RowBox[{"AxesLabel", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
      "\"\<t\>\"", ",", "\"\<\!\(\*SubscriptBox[\(v\), \(x\)]\)\>\""}], 
      "}"}]}], ",", 
    RowBox[{"ImageSize", "\[Rule]", "200"}]}], "]"}], "//", 
  "\[IndentingNewLine]", 
  RowBox[{"Figure", "[", "\"\<cfd-centraltflow\>\"", "]"}]}]], "Input",
 CellChangeTimes->{
  3.390123768045355*^9, {3.4317556003082123`*^9, 3.4317556175172124`*^9}, {
   3.4317565761682124`*^9, 3.4317565794702125`*^9}, {3.432461399535225*^9, 
   3.4324614032059593`*^9}, 3.432484030118*^9}]
}, Open  ]]
},
AutoGeneratedPackage->None,
WindowSize->{764, 917},
WindowMargins->{{Automatic, 553}, {Automatic, -9}},
ShowSelection->True,
FrontEndVersion->"8.0 for Microsoft Windows (64-bit) (November 7, 2010)",
StyleDefinitions->FrontEnd`FileName[{"Creative"}, "NaturalColor.nb", 
  CharacterEncoding -> "WindowsANSI"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[567, 22, 190, 5, 65, "Title",
 InitializationCell->True],
Cell[760, 29, 382, 8, 62, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[1179, 42, 164, 2, 65, "Title"],
Cell[1346, 46, 129, 2, 41, "Input"],
Cell[CellGroupData[{
Cell[1500, 52, 136, 2, 75, "Section"],
Cell[1639, 56, 782, 20, 41, "Input"],
Cell[2424, 78, 429, 10, 41, "Input"],
Cell[CellGroupData[{
Cell[2878, 92, 103, 1, 36, "Subsection"],
Cell[2984, 95, 1108, 31, 65, "Input"],
Cell[4095, 128, 267, 7, 41, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4399, 140, 104, 1, 36, "Subsection"],
Cell[4506, 143, 1334, 34, 86, "Input"],
Cell[5843, 179, 154, 3, 41, "Input"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[6058, 189, 157, 2, 65, "Title"],
Cell[6218, 193, 119, 2, 41, "Input"],
Cell[6340, 197, 738, 16, 41, "Input"],
Cell[7081, 215, 375, 10, 41, "Input"],
Cell[7459, 227, 558, 16, 41, "Input"],
Cell[8020, 245, 113, 2, 41, "Input"],
Cell[8136, 249, 248, 7, 41, "Input"],
Cell[8387, 258, 544, 17, 41, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[8968, 280, 27, 0, 65, "Title"],
Cell[8998, 282, 77, 2, 41, "Input"],
Cell[CellGroupData[{
Cell[9100, 288, 39, 0, 75, "Section"],
Cell[9142, 290, 397, 10, 62, "Input"],
Cell[9542, 302, 37, 0, 29, "Text"],
Cell[9582, 304, 27, 0, 41, "Input"],
Cell[9612, 306, 43, 0, 29, "Text"],
Cell[9658, 308, 95, 1, 41, "Input"],
Cell[9756, 311, 111, 1, 29, "Text"],
Cell[9870, 314, 94, 1, 41, "Input"],
Cell[9967, 317, 48, 0, 29, "Text"],
Cell[10018, 319, 28, 0, 41, "Input"],
Cell[10049, 321, 48, 0, 29, "Text"],
Cell[10100, 323, 28, 0, 41, "Input"],
Cell[10131, 325, 35, 0, 29, "Text"],
Cell[10169, 327, 28, 0, 41, "Input"],
Cell[10200, 329, 31, 0, 29, "Text"],
Cell[10234, 331, 34, 0, 41, "Input"],
Cell[10271, 333, 38, 0, 29, "Text"],
Cell[10312, 335, 28, 0, 41, "Input"],
Cell[10343, 337, 38, 0, 29, "Text"],
Cell[10384, 339, 31, 0, 41, "Input"],
Cell[10418, 341, 105, 1, 41, "Input"],
Cell[10526, 344, 33, 0, 29, "Text"],
Cell[10562, 346, 34, 0, 41, "Input"],
Cell[10599, 348, 143, 2, 41, "Input"],
Cell[10745, 352, 143, 2, 41, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10925, 359, 41, 0, 75, "Section"],
Cell[10969, 361, 40, 0, 29, "Text"],
Cell[11012, 363, 393, 8, 62, "Input"],
Cell[11408, 373, 53, 0, 29, "Text"],
Cell[11464, 375, 30, 0, 41, "Input"],
Cell[11497, 377, 34, 0, 29, "Text"],
Cell[11534, 379, 35, 0, 41, "Input"],
Cell[11572, 381, 35, 0, 29, "Text"],
Cell[11610, 383, 36, 0, 41, "Input"],
Cell[11649, 385, 29, 0, 29, "Text"],
Cell[11681, 387, 27, 0, 41, "Input"],
Cell[11711, 389, 28, 0, 41, "Input"],
Cell[11742, 391, 63, 0, 29, "Text"],
Cell[11808, 393, 120, 2, 41, "Input"],
Cell[11931, 397, 116, 2, 41, "Input"],
Cell[12050, 401, 773, 12, 41, "Input"],
Cell[12826, 415, 1677, 43, 142, "Input"],
Cell[14506, 460, 507, 15, 41, "Input"],
Cell[15016, 477, 736, 15, 41, "Input"],
Cell[15755, 494, 380, 10, 41, "Input"],
Cell[16138, 506, 387, 10, 41, "Input"],
Cell[16528, 518, 387, 10, 41, "Input"],
Cell[16918, 530, 379, 10, 41, "Input"],
Cell[17300, 542, 465, 11, 41, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[17802, 558, 35, 0, 75, "Section"],
Cell[17840, 560, 1398, 35, 82, "Input"],
Cell[19241, 597, 114, 1, 29, "Text"],
Cell[19358, 600, 747, 17, 41, "Input"],
Cell[20108, 619, 175, 4, 59, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[20320, 628, 32, 0, 75, "Section"],
Cell[20355, 630, 793, 23, 62, "Input"],
Cell[21151, 655, 694, 21, 41, "Input"],
Cell[21848, 678, 1292, 31, 84, "Input"],
Cell[23143, 711, 633, 14, 62, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[23813, 730, 50, 0, 75, "Section"],
Cell[23866, 732, 1145, 28, 82, "Input"],
Cell[25014, 762, 434, 11, 41, "Input"],
Cell[25451, 775, 265, 6, 41, "Input"],
Cell[25719, 783, 498, 13, 41, "Input"],
Cell[26220, 798, 810, 18, 41, "Input"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[27079, 822, 32, 0, 65, "Title"],
Cell[27114, 824, 77, 2, 41, "Input"],
Cell[27194, 828, 89, 2, 29, "Text"],
Cell[27286, 832, 1028, 30, 82, "Input"],
Cell[28317, 864, 318, 7, 41, "Input"],
Cell[28638, 873, 305, 8, 41, "Input"],
Cell[28946, 883, 347, 11, 49, "Input"],
Cell[29296, 896, 364, 8, 41, "Input"],
Cell[29663, 906, 1690, 39, 82, "Input"],
Cell[31356, 947, 327, 6, 62, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[31720, 958, 40, 0, 65, "Title"],
Cell[31763, 960, 77, 2, 41, "Input"],
Cell[31843, 964, 358, 10, 29, "Text"],
Cell[32204, 976, 2514, 67, 242, "Input"],
Cell[34721, 1045, 37, 0, 29, "Text"],
Cell[34761, 1047, 365, 9, 41, "Input"],
Cell[35129, 1058, 98, 1, 29, "Text"],
Cell[35230, 1061, 368, 10, 41, "Input"],
Cell[35601, 1073, 30, 0, 29, "Text"],
Cell[35634, 1075, 1092, 29, 102, "Input"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
